/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.ajax.auth"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.ajax.auth"] = true;
//dojo.require( "ibm.portal.debug" );
dojo.provide("com.ibm.ajax.auth");
com.ibm.ajax.auth = {
   /**
    * This method will wrap ioArgs normally sent to the dojo xhr methods with a special handler function that will check for form 
    * authentication redirection on a request.  This method supports dojo.xhrGet, dojo.xhrPost, dojo.xhrDelete, dojo.xhrPut,
    * dojo.io.iframe.send, and dojo.io.script.send.
    *
    * The testForValidOutput function will be invoked (if specified) with the response and the ioArgs arguments that are normally 
    * passed to the dojo.xhr methods.
    *
    * The onauthenticated method is optional, and will be invoked when the provider 
    * determines the user is successfully authenticated.  
    *
    * The handler method added to the ioArgs would check that the response status code is >= 200 && < 300, and that the mime-type is
    * text/html.
    *
    * @return A version of args suitable to be passed to dojo.xhrGet/xhrPost/xhrDelete/xhrPut/xhrRawPost/io.iframe.send()
    */
   prepareSecure: function( /* object */ args, /* function(response,ioArgs) optional */ isExpectedResponse, /* function() optional */ onauthenticated ) {
      args._handle = args.handle;
      args.handle = dojo.partial(this.testAuthenticationHandler, this, isExpectedResponse, onauthenticated);

      return args;
   },
   
   /**
    * When a request is determined to require authentication, the authenticationRequired function passed to this function will 
    * be invoked with the following arguments:
    *   - the response object from dojo
    *   - the ioArgs object that was passed to dojo
    *   - the onauthenticated function provided to the checkRequest/checkHtmlRequest methods (may be null)
    */
   setAuthenticationHandler: function( /* function(response,ioArgs,onauthenticated) */ authenticationRequired) {
      this.authenticationHandler = authenticationRequired;
   },
   
   /**
    * Callers of the API may choose to override the authentication test mechanism for this class wholesale by calling this
    * this method with an override.  The function will be invoked with the following arguments whenever an xhrGet that was
    * invoked with prepareSecure() returns.  Generally this is provided only as a last resort if a specific product's 
    * authentication test needs are not satisfied, or if the default behavior is not sufficient.
    */
   setTestAuthenticationHandler: function( /* function(auth, isExpected, onauthenticated, response, ioArgs) */handler) {
      this.testAuthenticationHandler = handler;
   },
   
   /**
    * By default there are three authentication tests performed when a request is returned from dojo.xhrGet/Post/Delete/Put.
    *    1) If the caller provides a test themselves (they know that their particular response may be confused for a
    *       login redirect)
    *    2) If the handleAs type is 'xml', 'json', or 'json-comment-optional' and the server returned 'text/html' or
    *       one of the other common HTML mime types, we assume that a redirect was sent from the server and the browser
    *       silently redirected the user (if the status code was >= 200 and < 300).
    *    3) If the status code is 302 in the response, which typically only occurs when multiple redirects are sent by the 
    *       server and the browser decides not to handle all of them, we assume a login redirect was the cause.
    *
    * The three booleans passed to this function will set whether the specific check is enabled.  By default all three
    * are true.
    */
   setDefaultAuthenticationTests: function(checkFromCaller,checkByContentType,checkByStatusCode) {
      this.checkFromCaller = checkFromCaller;
      this.checkByContentType = checkByContentType;
      this.checkByStatusCode = checkByStatusCode;
   },
   
   /**
    * This method allows users to add their own,custom login checks via a handler function.  The function takes three
    * arguments:
    *   - a reference to the auth object
    *   - the response object from dojo
    *   - the ioArgs object that was passed to dojo
    *
    * and returns true if the request requires authentication.
    *
    * See setDefaultAuthenticationTests(3) for more information about the default checks that are performed.
    */
   addAuthenticationCheck: function( /* function(auth, response, ioArgs) */handler ) {
      if (handler)
         this.authenticationChecks.push(handler);
   },
   
   /** 
    * Use this method to check whether a specific request requires authentication after a request has already
    * been received.  Should only be used when the default authentication test and handling behavior is not desired (for 
    * instance, when an application is initializing itself and needs to check whether it is authenticated).
    */
   isAuthenticationRequired: function(response, ioArgs) {
      var handleAs = ioArgs.args.handleAs;
      var unauthenticated = false;
      
      if (!response || dojo.indexOf(["cancel","timeout"],response.dojoType) == -1) {
         if (this.checkByContentType && dojo.indexOf(["xml","json","json-comment-optional","text"],handleAs) != -1 && ioArgs.xhr && /^text\/html/.exec(ioArgs.xhr.getResponseHeader("Content-Type")) && ioArgs.xhr.status >= 200 && ioArgs.xhr.status < 300) {
            ibm.portal.debug.text("auth::isAuthenticationRequired DEBUG content type does not match request, assume logged out");
            return true;
         }
         else if (this.checkByStatusCode && dojo.indexOf(["xml","json","json-comment-optional","text"],handleAs) != -1 && ioArgs.xhr && ioArgs.xhr.status == 302) {
            ibm.portal.debug.text("auth::isAuthenticationRequired DEBUG redirect received, assume login request");
            return true;
         }
         else if (this.checkByStatusCode && ioArgs.xhr && (ioArgs.xhr.status == 401|| ioArgs.xhr.status==0) && ioArgs.xhr.getResponseHeader("WWW-Authenticate") && ioArgs.xhr.getResponseHeader("WWW-Authenticate").indexOf("IBMXHR")!=-1) {
            ibm.portal.debug.text("auth::isAuthenticationRequired DEBUG Portal 401 received, assume login required");
            return true;
         }

      }
      if (!unauthenticated) {
         for (var i=0; i<this.authenticationChecks.length; i++)
            if (this.authenticationChecks[i](this, response, ioArgs))
               return true;
      }
      return false;   
   },
   
   testAuthenticationHandler: function(auth, isExpectedResponse, onauthenticated, response, ioArgs) {
      var args = dojo._toArray(arguments).slice(3);

      var unauthenticated = false;
      if (!response || dojo.indexOf(["cancel","timeout"],response.dojoType) == -1) {
         if (auth.checkFromCaller && typeof isExpectedResponse == "function" && isExpectedResponse(response, ioArgs)) {
            //console.log("auth::_testAuthentication DEBUG caller provided detection logic that indicated form login, assume logged out");
            unauthenticated = true;
         }
         else
            unauthenticated = auth.isAuthenticationRequired(response,ioArgs,isExpectedResponse)
      }
      
      if (unauthenticated) {
         //assume that WAS has set its redirect cookies and clear them
         var path = auth._parseUri(ioArgs.args.url).path;
         dojo.cookie("WASPostParam",null,{expires:-1,path:path});
         dojo.cookie("WASReqURL",null,{expires:-1,path:"/"});

         auth.authenticationHandler(response, ioArgs, onauthenticated);

         args[0] = new Error("xhr unauthenticated");
         args[0].dojoType = "unauthenticated";
      }
      if (ioArgs.args._handle)
         return ioArgs.args._handle.apply(this, args);
      else
      	 return(response);
   },
   
   /** Private methods */

   _parseUri: function(uri) {
      if (!uri)
         return null;
      uri = new dojo._Url(uri);
      var params = this._splitQuery(uri.query);
      uri.queryParameters = params;
      return uri;
   },
   
   _splitQuery: function(query) {
      var params = {};
      if (!query)
         return params;
      if (query.charAt(0) == "?")
         query = query.substring(1);
   
      var args = query.split("&");
      for (var i=0; i<args.length; i++)
         if (args[i].length > 0) {
            var separator = args[i].indexOf("=");
            if (separator == -1) {
               var key = decodeURIComponent(args[i]);
               var existing = params[key];
               if (dojo.isArray(existing))
                  existing.push("");
               else if (existing)
                  params[key] = [existing,""];
               else
                  params[key] = "";
            } else if (separator > 0) {
               var key = decodeURIComponent(args[i].substring(0, separator));
               var value = decodeURIComponent(args[i].substring(separator+1));
               var existing = params[key];
               if (dojo.isArray(existing))
                  existing.push(value);
               else if (existing)
                  params[key] = [existing,value];
               else
                  params[key] = value;
            }
         }
      return params;
   },
   
   checkFromCaller: true,
   checkByContentType: true,
   checkByStatusCode: true,
   authenticationChecks: [],
   authenticationHandler: function() {
      ibm.portal.debug.text("auth::authenticationHandler DEBUG authentication was required");
   }
};

}
