/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.customize.ResultsList"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.customize.ResultsList"] = true;
// This controls the list of results in the customization shelf.  It handles searching datastores and paging.  It usually works with a com.ibm.customize.Container as its parent.

dojo.provide("com.ibm.customize.ResultsList");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.bundles", "Shelf", null, "ROOT,ar,ca,cs,da,de,el,en,es,fi,fr,he,hr,hu,it,ja,ko,nl,no,pl,pt,pt-br,ro,ru,sk,sl,sv,th,tr,uk,zh,zh-tw");

dojo.declare("com.ibm.customize.ResultsList",
	[dijit._Widget, dijit._Templated],
	{
		source: null,						// the source which is currently selected for searching
		parent: null, 						// the parent container for this widget
		searchTerms: [], 					// an array of search terms for fetch 
		currRequest: null,					// the current fetch request
		size: 9,							// the number of results displayed at once
		start: 0,							// the index of the first result displayed in the section
		totalItemCount: 0,					// the total number of items matched by the last search query
		templateString: '<div dojoAttachPoint="focusNode">' +
							'<div class="lotusPaletteWidgetContainer" role="grid" dojoAttachPoint="table"></div>' +
							'<div class="lotusPaging">' +
								'<div class="lotusLeft" dojoAttachPoint="pagingStatus"></div>' +
								'<ul class="lotusRight lotusInlinelist">' +
									'<li class="lotusFirst" dojoAttachPoint="prevLink"></li>' +
									'<li dojoAttachPoint="nextLink"></li>' +
								'</ul>' +
								'<div dojoAttachPoint="jumpToLink"></div>' +
							'<div>' +
						'</div>',
		
		constructor: function() {
			this.shelfBundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
		},
		
		postCreate: function(){ 
			// summary: subscribes to events for when the search terms changes, the source changes, the user wants to add a saved search, or the size of the browser window adjusts
			dojo.subscribe("/epc/customize/search"+this.parent.id, this, this.changeSearchTerms);
			dojo.subscribe("/epc/customize/changeSource"+this.parent.id, this, this.changeSource);
		},
		
		/**********************************/
		/*            SEARCH              */
		/**********************************/
		
		changeSearchTerms: function(/*String[]*/ searchTerms){
			// summary: does a new search when the filters change
			// searchTerms: an array of strings which are filters
			this.searchTerms = searchTerms;
			this.start = 0;
			this._search();
		},
		
		changeSource: function(/*Object*/ source){
			// summary: does a fresh search with no filters when the hub changes
			// source: the new source to search
			if(this.currRequest) this.currRequest.abort();  // make sure any unfinished request is aborted
			this.source = source;
			this.searchTerms = [];
			this.start = 0;
			if(source.hub.noIcons) dojo.addClass(this.domNode.parentNode,"lotusPlain");
			else dojo.removeClass(this.domNode.parentNode,"lotusPlain");
			this._search();
		},
		
		_search: function(){
			// summary: does a fetch operation on the hub and fills the table with the results
			if(this.source.hub == null){
				// alert the user if the hub is null and clear out old info on the paging bar
				this.addMessage("Error",dojo.string.substitute(this.shelfBundle["shelf_badDatastoreError"], [this.source.label]));
				return;
			} else this.setLoading();
			
			// See dojo.data.api.Read.fetch()
			this.currRequest = this.source.hub.fetch({
				queryOptions: { ignoreCase: true },
				query: { keywords: this.searchTerms }, // query is the array of filters that the hub handles in its own way
				count: this.size,
				start: this.start,  
				
				onComplete: dojo.hitch(this, function(/*DatastoreItem[]*/items) {
					// summary: uses the items returned from the search to adjust what is displayed on the paging bars, and fill the table with results
					if(this.parent.filterAdd){
						// filter out any items that should not be displayed
						filterFcn = dojo.hitch(this,function(item){ return this.parent.filterAdd(this.source.hub.mapItem(item)); })
						items = dojo.filter(items,filterFcn);
					}
					if(items.length == 0) this.addMessage("Warning",this.shelfBundle["shelf_noResults"]);
					else this.fill(items);
				}),
				
				onError: dojo.hitch(this, function(/*String*/error, /*Object*/request) {
					// summary: displays an error message in the table and clears out old info on the paging bar
					this.clear();
					this.addMessage("Error", dojo.string.substitute(this.shelfBundle["shelf_feedError"], [this.source.label]));
				}),
				
				onBegin: dojo.hitch(this, function(totalNumber) {
					// summary: records the number of results returns in the current query
					this.totalItemCount = totalNumber;
				})
			});
		},
		
		/***********************************/
		/*              TABLE              */
		/***********************************/
		
		setLoading: function() {
			// summary: adds a loading indicator, for use before search results are appended
			this.clear();
			this.table.appendChild(document.createTextNode(this.shelfBundle["shelf_loading"]));
		},
		
		addMessage: function(/*String*/type,/*String*/msg){
			// summary: clears out the table and then builds it back up enough to enter a single message into the table
			// msg: the message to enter in the table
			this.clear();
			// create the cell
			var message = this.parent.createLotusMsgNode(msg,type);
			this.table.appendChild(message);
			// reset search numbers to 0 and adjust the info on the paging bar
			this.start = 0;
			this.totalItemCount = 0;
		},
		
		clear: function(){
			// summary: clears out the table
			var items = dojo.query(".lotusPaletteWidget",this.table);
			dojo.forEach(items, dojo.hitch(this, function(item){
				dijit.byId(item.id).destroy();
			}));
			this.table.innerHTML = "";
			this.pagingStatus.innerHTML = "";
			this.jumpToLink.innerHTML = "";
			this.prevLink.innerHTML = "";
			this.nextLink.innerHTML = "";
		},
		
		fill: function(/*DatastoreItem[]*/contents){
			// summary: fills the table with rows of catalog item results
			// contents: array holding DatastoreItems to be entered in the table
			this.clear();
			dojo.forEach(contents, dojo.hitch(this, function(item) {
				var newCell = this.parent.newCell(this.source,item)
				this.table.appendChild(newCell);
			}));	
			this.adjustPagingBar();
		},
		
		/************************************/
		/*              PAGING              */
		/************************************/
		
		prev: function(){
			// summary: triggers when the user clicks the Previous link,
			this.start = this.start - this.size;
			this._search();
		},
		
		next: function(){
			// summary: triggers when the user clicks the Next link
			this.start = this.start + this.size;
			this._search();
		},
		
		jumpTo: function(/*Event*/e){
			// summary: triggers when a user jumps to a new page
			// event: the onkeydown event
			e = dojo.fixEvent(e, e.target);
			var key = e.keyCode;
		
			if(key == dojo.keys.ENTER){
				var input = this.jumpToInput.value;
				// if number is not an integer, return
				if(input.toString().search(/^[0-9]+$/) != 0)  return;
				// if number is less than 1, set it to 1
				if(input < 1) input = 1;
				
				// calculate the new starting index for result items and make sure it's within range
				var userInputStart = (input - 1) * this.size;
				var maxStart = (Math.ceil(this.totalItemCount/this.size) - 1) * this.size;
				this.start = (userInputStart < maxStart)?userInputStart:maxStart;
				// search with the new starting index
				this._search();
			}
		},
		
		adjustPagingBar: function(){
			// summary: triggers when a search is performed to make sure the numbers and contents of the bar match up
			var firstItem = (this.totalItemCount == 0)?0:this.start + 1;
			var lastItem = this.start + this.size;
			lastItem = (this.totalItemCount > lastItem)?lastItem:this.totalItemCount;
			
			// displays "Page X of Y" in the paging bar
			var currentPage = Math.ceil(firstItem/this.size);
			var totalPageCount = Math.ceil(this.totalItemCount/this.size);
			this.pagingStatus.innerHTML = dojo.string.substitute(this.shelfBundle["shelf_page"], [currentPage, totalPageCount]);
			
			// create Previous and Next links , and Jump to page ___  if appropriate
			this.createPrevNode(firstItem);
			this.createNextNode(lastItem);
			if(this.totalItemCount > this.size) this.createJumpTo(firstItem);
		},
		
		createPrevNode: function(/*Int*/firstItem){
			// summary: determines if a Previous link should be rendered and creates it if appropriate
			if(firstItem > 1){  // create a link
				var prev = dojo.create("a", {href:"javascript:void(0)",innerHTML:this.shelfBundle["shelf_previous"]});
				prev.onclick = dojo.hitch(this, function(){ this.prev(); return false; /* to prevent onBeforeUnload event from triggering in IE */ });
				this.prevLink.appendChild(prev);
			}
			else  // create unclickable text
				this.prevLink.appendChild(document.createTextNode(this.shelfBundle["shelf_previous"]));
		},
		
		createNextNode: function(/*Int*/lastItem){
			// summary: determines if a Next link should be rendered and creates it if appropriate
			if(this.totalItemCount > lastItem){  // create a link
				var next = dojo.create("a", {href:"javascript:void(0)",innerHTML:this.shelfBundle["shelf_next"]});
				next.onclick = dojo.hitch(this, function(){ this.next(); return false; /* to prevent onBeforeUnload event from triggering in IE */ });
				this.nextLink.appendChild(next);
			}
			else // create unclickable text
				this.nextLink.appendChild(document.createTextNode(this.shelfBundle["shelf_next"]));
		},
		
		createJumpTo: function(firstItem){
			// summary: if there is  more than one page available, this creates a "Jump to page ___" input
			this.jumpToInput = dojo.create("input",{type:"text",value:Math.ceil(firstItem/this.size),className:"lotusText"});
			dojo.connect(this.jumpToInput, "onkeydown", this, this.jumpTo);
			this.jumpToLink.appendChild(document.createTextNode(this.shelfBundle["shelf_jumpTo"]));
			this.jumpToLink.appendChild(this.jumpToInput);
		}
});

}
