/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.data.MashupHubStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.MashupHubStore"] = true;
dojo.provide( "com.ibm.data.MashupHubStore" );

dojo.require( "com.ibm.data.OpenSearchFeedStore");
dojo.require( "com.ibm.data.ProxyHelper" );

dojo.declare( "com.ibm.data.MashupHubStore",
			[com.ibm.data.OpenSearchFeedStore, com.ibm.data.ProxyHelper],
			{
				// namespaces: Map
				//		Map of namespaces used by the MashupHub feeds
				namespaces: dojo.mixin({}, com.ibm.data.OpenSearchFeedStore.prototype.namespaces, {
		            "catalog" : "http://www.ibm.com/xmlns/atom/opensearch/catalog/1.0/"
		        }),
		        
		        // itemClass: String the class used to represent items of this store
				itemClass: "com.ibm.data.MashupHubItem",
				
				constructor: function( /*Map*/ args ) {
					// summary: Creates a new instance of a this data store
					// args: Map
					//		The args map will be used to replace any default values of this store with the
					//		values specified in the args map.
					this._features['dojo.data.api.Identity'] = true;
					//this.url = this.urlThroughProxy(this.url);
				},
				
				handleConstructs: function(/*DOMNode*/ node, /*String*/ prop) {
					switch(prop) {
						case "OpenSearchQueryConstruct": return new com.ibm.data.MashupHubQueryConstruct(node);
						default: return this.inherited(arguments);
					}
				},
			
				fetchItemByIdentity: function(/* object */ keywordArgs){
					//	summary:
					//		Given the identity of an item, this method returns the item that has 
					//		that identity through the onItem callback.  Conforming implementations 
					//		should return null if there is no item with the given identity.  
					//		Implementations of fetchItemByIdentity() may sometimes return an item 
					//		from a local cache and may sometimes fetch an item from a remote server, 
					//
					// 	keywordArgs:
					//		An anonymous object that defines the item to locate and callbacks to invoke when the 
					//		item has been located and load has completed.  The format of the object is as follows:
					//		{
					//			identity: string|object,
					//			onItem: Function,
					//			onError: Function,
					//			scope: object
					//		}
					//	The *identity* parameter.
					//		The identity parameter is the identity of the item you wish to locate and load
					//		This attribute is required.  It should be a string or an object that toString() 
					//		can be called on.
					//		
					//	The *onItem* parameter.
					//		Function(item)
					//		The onItem parameter is the callback to invoke when the item has been loaded.  It takes only one
					//		parameter, the item located, or null if none found.
					//
					//	The *onError* parameter.
					//		Function(error)
					//		The onError parameter is the callback to invoke when the item load encountered an error.  It takes only one
					//		parameter, the error object
					//
					//	The *scope* parameter.
					//		If a scope object is provided, all of the callback functions (onItem, 
					//		onError, etc) will be invoked in the context of the scope object.
					//		In the body of the callback function, the value of the "this"
					//		keyword will be the scope object.   If no scope object is provided,
					//		the callback functions will be called in the context of dojo.global.
					//		For example, onItem.call(scope, item, request) vs. 
					//		onItem.call(dojo.global, item, request)
					var id = keywordArgs.identity;
					
					// our IDs may come in as urn:id:# and all we want is the # part
					if(id.indexOf(":") > -1) id = id.substring(id.lastIndexOf(":") + 1);
					var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
					var args = {
						query: {
							id: id,
							collection: "ui.all"
						},
						onComplete: dojo.hitch(this, 
							function(items) {
								if(items.length == 0 && keywordArgs.onError) {
									keywordArgs.onError.call(scope, "No items returned matching identity " + id);
								}
								else {
									keywordArgs.onItem.call(scope, items[0]);
								}
							}
						)
					};
					if(keywordArgs.onError) args.onError = dojo.hitch(scope, keywordArgs.onError);

					return this.fetch(args);
				}
			}
);

dojo.declare( "com.ibm.data.MashupHubItem",
		com.ibm.data.OpenSearchFeedItem, 
		{
			// namespaces: Map
			//		Map of namespaces used by the MashupHub feeds
			namespaces: dojo.mixin({}, com.ibm.data.MashupHubStore.prototype.namespaces),
			
	        attrMappings: dojo.mixin({}, com.ibm.data.OpenSearchFeedItem.prototype.attrMappings, {
				editLink: {xpath: "./atom:link[@rel='self']", targetName: "href", type: "attr"},
				category: {xpath: "./atom:category", targetName: "term", type: "attr"},
				catalogVersion: {xpath: "./atom:content/catalog:catalog/catalog:version", targetName: "textContent", type: "node"},
				catalogName: {xpath: "./atom:content/catalog:catalog/catalog:name", targetName: "textContent", type: "node"},
				catalogAuthor: {xpath: "./atom:content/catalog:catalog/catalog:author", targetName: "textContent", type: "node"},
				catalogDescription: {xpath: "./atom:content/catalog:catalog/catalog:description", targetName: "textContent", type: "node"},
				catalogRating: {xpath: "./atom:content/catalog:catalog/catalog:rating", targetName: "textContent", type: "node"},
				catalogUseCount: {xpath: "./atom:content/catalog:catalog/catalog:useCount", targetName: "textContent", type: "node"},
				catalogDateModified: {xpath: "./atom:content/catalog:catalog/catalog:dateModified", targetName: "textContent", type: "node"},
				catalogNumComments: {xpath: "./atom:content/catalog:catalog/catalog:numComments", targetName: "textContent", type: "node"},
				catalogTags: {xpath: "./atom:content/catalog:catalog/catalog:tags/catalog:tag", targetName: "textContent", type: "node"},
				catalogCategories: {xpath: "./atom:content/catalog:catalog/catalog:categories", targetName: "textContent", type: "node"},
				catalogDocumentationURL: {xpath: "./atom:content/catalog:catalog/catalog:documentationURL", targetName: "textContent", type: "node"},
				catalogDownloadURL: {xpath: "./atom:content/catalog:catalog/catalog:downloadURL", targetName: "textContent", type: "node"},
				catalogIcon: {xpath: "./atom:content/catalog:catalog/catalog:icon", targetName: "textContent", type: "node"}
			})
		}
);

dojo.declare( "com.ibm.data.MashupHubQueryConstruct",
			com.ibm.data.OpenSearchQueryConstruct,
			{
				namespaces: dojo.mixin({}, com.ibm.data.MashupHubStore.prototype.namespaces),
				
				getSearchType: function() {
					return this._getAttribute(this.root, "catalog:searchType");
				},
				getCollection: function() {
					return this._getAttribute(this.root, "catalog:collection");
				}
			}
);

}
