/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.data.resolver.XmlPocHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.resolver.XmlPocHandler"] = true;
/*
 * @author Ben
 */ 
dojo.provide("com.ibm.data.resolver.XmlPocHandler");

dojo.require("com.ibm.data.resolver.QueryRulePocHandler");
dojo.require("com.ibm.query.XmlQueryRuleEngine");

dojo.declare("com.ibm.data.resolver.XmlPocHandler",
	[com.ibm.data.resolver.QueryRulePocHandler, com.ibm.data.TemplateInstantiator],
	{
		// definition: Object
		//		Defines this POC handler's capabilities and metadata.  Returned by the define API function.
		definition: {
			scheme: "",
			handleAs: "xml",
			features: {}
		},
		
		namespaces: {},
		
		// queryEngineClass: String
		//		Fully-qualified name of the javascript object constructor used
		//		to create an instance of a query rule engine used by this POC
		//		handler.
		queryEngineClass: "com.ibm.query.XmlQueryRuleEngine", 
		
		constructor: function() {
			this.namespaces = dojo.mixin({}, this.namespaces, this.resolver.getNamespaces());
			this.engine.namespaces = this.namespaces;
			this._createTemplates();
		},
		
		_createTemplates: function() {
			// summary: Creates any XML templates that are used by this handler for generating
			//		data for new or seeded items.
		},
		
		buildEngine: function() {
			// summary: Creates and returns an instance of the query rule engine
			//		used by this POC handler for querying data in items and response
			//		data.  Called by the constructor once during instantiation of
			//		this POC handler.  Can be overridden by extensions to provide
			//		additional parameters to the engine.  For convenience, the engine's
			// 		context is set to this POC handler so that delegated functions that
			// 		the engine calls are called in context of this handler.
			// returns: Object instance of a query rule engine.
			var cls = dojo.getObject(this.queryEngineClass);
			return new cls({queryRules: this.queryRules, types: this.types, namespaces: this.namespaces, resolver: this.resolver, handler: this});
		},
		
		syncNamespaces: function(/*DOMNode*/ node) {
			var myNs = this.namespaces; 
			dojo.forEach(node.ownerDocument.documentElement.attributes, function(attrNode) { 
				var idx = attrNode.nodeName.indexOf("xmlns:"); 
				if(idx == 0) {
					var prefix = attrNode.nodeName.substr(6);
					if(myNs[prefix] != attrNode.nodeValue) {
						myNs[prefix] = attrNode.nodeValue;
					}
				} 
			});
		},
		
		setValue: function(	/* item */ item, 
							/* string */ attribute,
							/* almost anything */ value){
			var res = this.inherited(arguments);
			if(res) this._logWrite(item, {attribute: attribute, value: value});
			return res;
		},	
		setValues: function(/* item */ item,
							/* string */ attribute, 
							/* array */ values){
			var res = this.inherited(arguments);
			if(res) this._logWrite(item, {attribute: attribute, values: values});
			return res;
		},	
		unsetAttribute: function(	/* item */ item, 
									/* string */ attribute){
			var res = this.inherited(arguments);
			if(res) this._logWrite(item, {attribute: attribute});
			return res;
		},
		
		newItem: function(/* Object? */ keywordArgs, /*Object?*/ parentInfo){
			this.modifyNewItemArgs(keywordArgs, parentInfo);
			return this.createItem(keywordArgs, parentInfo);
		},
		createItem: function(keywordArgs, parentInfo) {
			var existingItem = null;
			if(keywordArgs.id) {
				existingItem = this.resolver._fromCache(keywordArgs.id);
				if(existingItem && existingItem.data) {
					for(var p in keywordArgs) {
						this[dojo.isArray(keywordArgs[p]) ? "setValues" : "setValue"](existingItem, p, keywordArgs[p]);
					}
					return existingItem;
				}
			}
			var template = this.getTemplate(keywordArgs, parentInfo);
			keywordArgs = dojo.mixin({}, template.templateProps, keywordArgs);
			var instTempl = this.instantiateTemplate(template, keywordArgs, parentInfo);
			var itemStub = existingItem ? existingItem : this.createStub(keywordArgs);
			itemStub.data = instTempl;
			return this.fillInProperties(itemStub, template, keywordArgs, parentInfo);
		},
		fillInProperties: function(newItem, template, keywordArgs, parentInfo) {
			for(var p in keywordArgs) {
				if(!(p in template.templateProps)) {
					this[dojo.isArray(keywordArgs[p]) ? "setValues" : "setValue"](newItem, p, keywordArgs[p]);
				}
			}
			return newItem;
		},
		getTemplate: function(args, parentInfo) {
			// summary: Used to retrieve a template string for creating a new item.  
			// 		Can be overrided by subclasses to return the template string to be used
			//		when instantiating a new item based on the arguments.
			// args: Object arguments that are passed into the newItem function for this store
			// parentInfo: Object parentInfo that is passed into the newItem function for this store
			// returns: Object map of two properties defining a template:
			// 		templateString: String template to instantiate with arguments map
			//		templateProps: Object map of properties used in the template along with default values
			return {
				templateString: "",
				templateProps: {}
			};
		},
		modifyNewItemArgs: function(args, parentInfo) {
			return args;
		},
		instantiateTemplate: function(template, args, parentInfo) {
			return this.getTemplateInstance(template.templateString, args);
		},
		backup: function(item) {
			// summary: Creates a single backup copy of this item on the client for
			//		later restoration if necessary.  Calling this function multiple
			//		times before _restore() or _persist() is called on this item will do nothing
			//		after the initial backup, effectively maintaining only the first
			//		version of the item before any changes were made.
			if(!item._backupData) {
				item._backupData = item.data.cloneNode(true);
				this._makeModifiable(item);
				item._logEnabled = true;
				return true;
			}
			return false;
		},
		restore: function(item) {
			// summary: Restores the single backup copy of this item on the client,
			//		erasing all modifications made since backup() was called.
			if(!item._backupData) {
				delete item.data;
				item.data = item._backupData;
				delete item._backupData;
				item._writeLog.splice(0, item._writeLog.length);
				return true;
			}
			return false;
		},
		persist: function(item) {
			// summary: Persists the changes made to this item since the backup() was
			// 		called.  After this is called, there is no way to restore the backup
			//		as it is erased completely.
			this._writeLogToBackup(item);
			this.inherited(arguments);
		},
		processFetch: function(/*DOMDoc*/ xmlDoc, /*Request*/ request) {
			// summary: Extension point for descendants or consumers of this store to handle
			//		responses from server requests for data.  Most subclasses can leave this
			// 		function as-is, but if special handling is required that cannot be done
			// 		with other extension points, this can be overridden as well.
			// 		Must call onBegin, onItem, and onComplete callbacks correctly as 
			// 		specified by dojo.data.api.Read.  The onError callback is handled elsewhere.
			// xmlDoc: DOMDoc of the response from an IO request
			// request: Request used to fetch the result
			return this.inherited(arguments);
		},
		processSave: function(/*Item*/ item) {
			return this.inherited(arguments);
		},
		_logWrite: function(item, map) {
			// summary: Logs write operations to this item so that they can later
			//		be repeated to the backup during persistence.
			if(item._logEnabled) item._writeLog.push(map);
		},
		_writeLogToBackup: function(item) {
			// summary: Repeates the write log for this item to the original backup
			// 		node.  Used mainly for persisting changes to support a 2-phase
			//		write operation (write-commit).
			if(item._backupData) {
				item.data = item._backupData;
				item._logEnabled = false;
				for(var i = 0; i < item._writeLog.length; i++) {
					var op = item._writeLog[i];
					if(op.values) {
						this.setValues(item, op.attribute, op.values);
					}
					else if(op.value !== undefined){
						this.setValue(item, op.attribute, op.value);
					}
					else this.unsetValue(item, op.attribute);
				}
				item._writeLog.splice(0, item._writeLog.length);
			}
		}
	}
);

}
