/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.data.resolver.tree.PocLMTreeModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.resolver.tree.PocLMTreeModel"] = true;
/*
 * @author Ben
 */ 
dojo.provide("com.ibm.data.resolver.tree.PocLMTreeModel");

dojo.require("com.ibm.data.resolver.tree.PocTreeModel");
/*
 * Instances of this tree model are specific to one page and cannot be shared amongst other pages.
 * This maintains this model as a tree model instead of a forest model with a fake root of all
 * layout models in a system.  A forest model of the system layout models would be useless and
 * would not perform well given the design of the models and their APIs.
 */

dojo.declare("com.ibm.data.resolver.tree.PocLMTreeModel",
	com.ibm.data.resolver.tree.PocTreeModel,
	{
		refItem: null,
		pageId: "",
		
		constructor: function() {
			this._rDfd = null;
			this._allItems = [];
		},
		
		getRoot: function(onItem, onError){
			// summary:
		    //            Calls onItem with the root item for the tree, possibly a fabricated item.
			//            Throws exception on error.
			if(this.root) {
				if(this.resolver.isItemLoaded(this.root)) {
					if(onItem) onItem(this.root);
				}
				else {
					this.resolver.loadItem({
						item: this.root,
						onItem: onItem,
						onError: onError
					});
				}
				return;
			}
			if(this.refItem && !this.getParent(this.refItem)) {
				var compId = this.handler._extractCompositeIds(this.getIdentity(this.refItem));
				if(compId.layoutId == compId.pageId) {
					return this.handler.getValue(this.refItem, "children")
						.addOnLoad(dojo.hitch(this, function(ch){
							this._allItems = ch;
							this.root = ch[0];
							if(onItem) onItem(this.root);
						}))
						.addOnError(onError);
				}
				else {
					this.root = this.refItem;	// in a layout model, the root node is unchangeable
					this._allItems.push(this.root);
					return this.getRoot(onItem, onError);
				}
			}
			if(this._rDfd) {
				this._rDfd.addCallbacks(onItem, onError);
			}
			else {
				this._rDfd = (new dojo.Deferred()).addBoth(dojo.hitch(this, function(){
	        		delete this._rDfd;	// cleanup
	        	})).addCallbacks(onItem, onError);
	        	
	        	if(this.refItem) {
					this.getAncestors(this.refItem, dojo.hitch(this, function(items){
							this._allItems = items;
							this._rDfd.callback(items[items.length - 1]);
						}), dojo.hitch(this, function(error){
							this._rDfd.errback(error);
						})
					);
	        	}
	        	else {
	        		this.resolver.fetch({
	        			query: {
	        				uri: "lm:oid:" + this.pageId
	        			},
        				onComplete: function(items) {
        					this._allItems = items;
        					for(var i = 0; i<items.length; i++) {
        						if(!this.getParent(items[i])) {
        							this.root = items[i];
        							this._rDfd.callback(this.root);
        							return;
        						}
        					}
        					this._rDfd.errback(new Error("lm:oid:" + this.pageId));
        				},
        				onError: function(err){
        					this._rDfd.errback(error);
        				},
        				scope: this
	        		});
	        	}
			}
		},
		getAll: function() {
			return this._allItems;
		},
		
		inModel: function(/*Item|String*/ obj) {
			if(!dojo.isString(obj)) {
				obj = this.getIdentity(obj);
			}
			
			var compId = this.handler._extractCompositeIds(obj);
			return this.pageId == compId.pageId;
		},
		toString: function() {
			return "[PocLMTreeModel of {" + this.handler + "} for page {" + this.pageId + "}]";
		}
	}
);

}
