/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.navigation.controller"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.navigation.controller"] = true;
dojo.provide( "com.ibm.portal.navigation.controller" );

dojo.require( "com.ibm.portal.services.common" );
dojo.require( "com.ibm.portal.state.common" );

dojo.require( "dojox.data.dom" );

com.ibm.portal.navigation.controller.CONSTANTS = new function () {
	// summary: Constants used throughout the controller.
	
	this.serializedStateToken = "_SERIALIZED_STATE_",
	this.portletWindowToken = "_PORTLET_WINDOW_ID_",
	this.selectedNodeToken = "_LAYOUT_NODE_ID_",
	this.portletUrlTemplate = this.serializedStateToken + "?uri=fragment:pm:oid:" + this.portletWindowToken + "@oid:" + this.selectedNodeToken
	this.forceSsaRenderMode = "force_ssa";
	this.forceCsaRenderMode = "force_csa";
}

dojo.declare( "com.ibm.portal.navigation.controller.NavigationalState", 
			  null,
			  {
			  	// summary: Wraps the state document and the selection model and provides read-only access
				//		to the information contained within those two objects.
				// description: Convenience wrapper for the state document and the selection model. Any changes
				// 		made to the state document or selection model should be ignored. The state document
				//		is not updated throughout the life of this object. It should be discarded at the end 
				//		of the rendering process.
				
				constructor: function ( /*dojo.dom.Document*/stateDom, /*NavigationSelectionModel*/navSelModel, /*boolean*/sendQueryParameters ) {
					if (stateDom) {
						this.stateDOM = stateDom;
					} else {
						this.stateDOM = dojox.data.dom.createDocument();
					}
					this.selectionModel = navSelModel;
					this.sendQueryParameters = sendQueryParameters;
					this._serializedState = null;
				},
				createLinkToCurrentState: function ( /*Boolean*/skipPortletState ) {
					// summary: Encapsulates the current navigational state in a URL which can be bookmarked or
					//		distributed.
					// skipPortletState: indicates whether the portlet state (portlet mode, window state, etc.) should be
					//		included in the encapsulated state. Setting this to true will make the URLs shorter, as well
					//		as perform better.
					// returns: the URL with the state encoded as a string
					var stateManager = this._getStateManager();
					var serializationMgr = stateManager.getSerializationManager();
					
					stateToSerialize = stateManager.getState();
					if ( skipPortletState ) {
						var selection = this.getPageSelection();
						var newStateMgr = new com.ibm.portal.state.StateManager( ibmPortalConfig["contentHandlerURI"] );
						newStateMgr.reset( newStateMgr.newState() );
						var selectionAcc = newStateMgr.newSelectionAccessor();
						selectionAcc.setPageSelection( selection );
						stateToSerialize = newStateMgr.getState();
					}
					
					var serializationResult = serializationMgr.serialize( stateToSerialize );
					return location.protocol + "//" + location.host + serializationResult["url"]; // String
				},
				getPageSelection: function () {
					// summary: Gets the currently selected page.
					// returns: the selected page object id without the model feed prefix- String
					var stateManager = this._getStateManager();
					var selectionAcc = stateManager.newSelectionAccessor();
					
					return selectionAcc.getSelection(); // String
				},
				getLocale: function() {
					// summary: Gets the current locale from the nav state
					// returns: the locale or null
					var stateManager = this._getStateManager();
					var localeAcc = stateManager.newLocaleAccessor();
					return localeAcc.getLocale();
				},
				getPortletState: function ( /*String*/portletWindowId ) {
					// summary: Gets the portlet state document for the given portlet window id.
					// returns: the state document for the given portlet id					
					var stateManager = this._getStateManager();
					return stateManager.newPortletAccessor( portletWindowId ).getPortletState();
				},
				getSoloPortlet: function() {
					// summary: Gets the portlet id of the portlet that is in solo mode if one
					// 		exists.  Null is returned if no portlets are in solo mode.
					// returns: the portlet id of the portlet in solo mode, or null				
					var stateManager = this._getStateManager();
					return stateManager.newSoloStateAccessor().getSoloPortlet();
				},
				resolveRelativePortletURL: function ( /*String*/ relativeURL ) {
					ibm.portal.debug.entry( "NavigationalState.resolveRelativePortletURL", [ relativeURL] );
					if ( this._serializedState == null ) {
						// serialize the state into a portal URL
						ibm.portal.debug.text( "Attempting to serialize state!"  )
						var stateManager = this._getStateManager();
						var serializationMgr = stateManager.getSerializationManager();
						var serializationResult = serializationMgr.serialize(stateManager.getState());
						// return the URL
						this._serializedState = serializationResult["returnObject"];
					}
					var resolvedURL = this._serializedState + relativeURL;
					ibm.portal.debug.exit( "NavigationalState.resolveRelativePortletURL", resolvedURL );
					return resolvedURL;
				},
				newPortletRenderURL: function ( /*String*/portletWindowId, /*String?*/portletMode, /*String?*/windowState  ) {

					// summary: Creates a portlet url based on the current state.
					// returns: the url.
					ibm.portal.debug.entry( "NavigationalState.newPortletRenderURL", [ portletWindowId, portletMode, windowState ] );
					
					var stateManager = this._getStateManager();
					
					if ( this._serializedState == null ) {
						ibm.portal.debug.text( "Attempting to serialize state!"  )
						// serialize the state into a portal URL
						var serializationMgr = stateManager.getSerializationManager();
						ibm.portal.debug.text( "State is: " + dojox.data.dom.innerXML( stateManager.getState() ) );
						var serializationResult = serializationMgr.serialize(stateManager.getState());
                        // return the URL if its not null. ie a 401 would cause the serializationResult
                        // to be null
                        if(serializationResult!=null) {
							this._serializedState = serializationResult["returnObject"];
						} else {
                            ibm.portal.debug.exit( "newPortletRenderURL serializationResult is null. return # to short circuit getPorletInfo " );
						    return "#";
                        }

                    }
					
					ibm.portal.debug.text( "Serialized state: " + this._serializedState );
					
					var selectionAcc = stateManager.newSelectionAccessor();
					
					// create the base URL
					var url = com.ibm.portal.navigation.controller.CONSTANTS.portletUrlTemplate; 
					url = url.replace( com.ibm.portal.navigation.controller.CONSTANTS.serializedStateToken, this._serializedState );
					url = url.replace( com.ibm.portal.navigation.controller.CONSTANTS.portletWindowToken, portletWindowId );
					url = url.replace( com.ibm.portal.navigation.controller.CONSTANTS.selectedNodeToken, selectionAcc.getPageSelection() );
					
					// append portlet mode parameter
					if (portletMode != null) {
						url += "&javax.portlet.PortletMode=" + portletMode;
					}
					
					// append window state parameter
					if (windowState != null) {
						url += "&javax.portlet.WindowState=" + windowState;
					}

					// append normalizeState param
					// only needed for portlet URLs generated on the client! (do not move this into PortletFragmentService:flagURL...)
					url += "&ibm.web2.normalizeState=true";

					// append query params
					if ( this.sendQueryParameters ) {
						url = url + "&" + location.search.substring(1);
					}
					
					ibm.portal.debug.exit( "NavigationalState.newPortletRenderURL", url );
					return url;
				},
				newPageURL: function (pageId, portletWindowId, windowState, portletMode, renderParams, newWindow) {
					ibm.portal.debug.entry( "NavigationalState.newPageURL", [ pageId, portletWindowId, windowState, portletMode, renderParams, newWindow ]);
					
					// get the state manager
					var stateManager = this._getStateManager();
					
					// get a copy of the current state
					var stateCopy = stateManager.newState(stateManager.getState());
					
					// get a portlet accessor which operates on the created state copy
					if(portletWindowId != null){
						var portletAcc = stateManager.newPortletAccessor(portletWindowId, stateCopy);
					}
				
					// modify the state
					if (portletMode != null) {
						portletAcc.setPortletMode(portletMode);
					}
                    
					if (windowState != null) {
						if( windowState.toLowerCase() == "solo" ){
							var soloAcc = stateManager.newSoloStateAccessor(stateCopy);
							// set portlet to solo state 
							soloAcc.setSoloPortlet(portletWindowId); 
							// set the return selection 
							var selAcc = stateManager.newSelectionAccessor(stateManager.getState()); 
							soloAcc.setReturnSelection(selAcc.getSelection()); 
						} else {
							portletAcc.setWindowState(windowState);
						}
					}
					
					// add the render params onto the url, note that the method
					// used is for a single value per name
					// renderParams[i] = { name: "paramName", value: "paramValue" }
					if (renderParams != null && renderParams.length > 0) {
						var renderParamsObject = portletAcc.getRenderParameters();
						renderParamsObject.clear();
						for(var i = 0 ; i < renderParams.length ; i++){
							renderParamsObject.setValue( renderParams[i].name, renderParams[i].value);
						}
					}
				    
					//If this url is to point to a new window, we need to set the theme template to "Plain".
					if ( newWindow ) {
						var templateAcc = stateManager.newThemeTemplateAccessor(stateCopy);
						templateAcc.setThemeTemplate( "Plain" );
						var partitionAcc = stateManager.newStatePartitionAccessor(stateCopy);
						partitionAcc.includeStatePartition();
					}	
				
					// encode the page ID using the selection accessor
					var selectionAcc = stateManager.newSelectionAccessor(stateCopy);
					selectionAcc.setPageSelection(pageId);
				
					// serialize the state into a portal URL
					var serializationMgr = stateManager.getSerializationManager();
					var serializationResult = serializationMgr.serialize(stateCopy);
			
					// return the URL
					//ibm.portal.debug.text("page action url: "+serializationResult["returnObject"]);
					if(serializationResult) {
                        return serializationResult["returnObject"];
                    } else {
                        return null;
                    }
					
				},
				_getStateManager: function () {
					// summary: Retrieve an instance of the state manager.
					// description: Creates an instance of the state manager and stores it as an instance
					//		variable. This instance of the state manager should only be used for read access
					//		since it is not reset with the state document every time it is retrieved. 
					// returns: the state manager - StateManager
					if ( !this._stateManager ) {
						this._stateManager = new com.ibm.portal.state.StateManager( ibmPortalConfig["contentHandlerURI"] );
						this._stateManager.reset( this.stateDOM );
					}
					return this._stateManager; // com.ibm.portal.state.StateManager  
				}	
			  }
);

dojo.declare( "com.ibm.portal.navigation.controller.NavigationalStateController",
			  null,
			  {
			  	// summary: The controller for the navigation.
				// description: The controller is responsible for managing the navigational state
				//		and notifying listeners of state change events. It must be initialized with a 
				//		state document via the initialize function before it can be used. The controller
				//		publishes several topics for various situations:
				//			/portal/navigation/controller/init - controller is initialized
				//			/portal/navigation/state		   - a navigational state change has occurred
				//			/portal/navigation/expansion	   - a navigation node has been expanded
				//			/portal/navigation/collapse		   - a navigational node has been collapsed
				//		The state change topic is not fired in case of the back button.
				currentTheme: null,
				currentThemePolicy: null,
				sendQueryParameters: false,
				constructor: function () {
					this._varyMgr = new com.ibm.portal.navigation.controller.StateVaryManager();
					this._mergeTemplate = "<merge><base-state>--basestate--</base-state><new-state>--newstate--</new-state></merge>";
					var xsltURL = dojo.moduleUrl( "com", "ibm/portal/navigation/");
					this._mergeXslt = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer(xsltURL + "StateMerge.xsl");
					this._namespaces = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
					this._endFragmentHandle = com.ibm.portal.EVENT_BROKER.endFragment.register( this, 
																		function ( args ) {
																			var pInfo = args["portletInfo"];
																			if ( !(pInfo instanceof Error) ) {
																				this._updatePortletState( pInfo );
																			}	
																		}
																	);
					this._endPageHandle = com.ibm.portal.EVENT_BROKER.endPage.register( this, function () { this.sendQueryParameters = false; } );	
				
			    },
				setSelection: function ( /*String*/pageId, /*int?*/ navLevel, /*Boolean?*/backButton, /*String*/renderMode ) {
					// summary: Sets the selected page in the navigational state.
					// description: Calculates the selection path if necessary and sets the selection
					//		in the navigational state. 
					// pageId: the ID of the page to select
					// navLevel: OPTIONAL - the navigation level of the page
					// backButton: OPTIONAL - skips the firing of the state change events.
					ibm.portal.debug.entry( "NavigationController.setSelection", [ pageId, navLevel, backButton, renderMode] );
					ibm.portal.debug.text( "Changing selection from: " + this.getState().getPageSelection() +" to: "+pageId );
					com.ibm.portal.EVENT_BROKER.cancelAll.fire();                    
					this._updateInternalSelection( pageId );
					
					//First check if we have a URL node. This will change the render process.
					var selectedNode = this._selectionModel.getSelectedNode();
   
                    if(!selectedNode){
						return;
					}
					

					if ( selectedNode.isExternalURL ) {
						top.location = selectedNode.getContentURL();
						return;
					}
					
					if ( selectedNode.isInternalURL ) {
						com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.setSelection( selectedNode.getContentURL() );
						return;
					}
					
					if (com.ibm.portal.navigation.controller.CONSTANTS.forceSsaRenderMode == renderMode)
					{
					    // on a normal navigation state change:
					    //    NavController fires stateChanged
					    //    Aggregator recieves, renders page and fires PortalPage/complete
					    //    BackButtonHandler receives and updates history and stores state in client
					    // on full refresh, we can't fire the navStateChange, since we don't want the page to re-render
					    // client-side, so currently we will manually update history and client state storage
					    // need to refactor our events to accommodate.
					    // probably add a FullPageRefresh topic.
					    //var state = this.getState();
					    //com.ibm.portal.aggregation.backbutton.BACK_BUTTON_HANDLER.addToHistory( state );
					    //com.ibm.portal.aggregation.backbutton.BACK_BUTTON_HANDLER.saveState( state );

					    this.fullPageRefresh( this._selectionModel.getSelectedNode().URI.substring( 7 ) );
					}
					else if ( this._hasThemeChanged( this._selectionModel ) || this._hasThemePolicyChanged( this._selectionModel ) ) {
						//Check if the theme has changed.
						this.fullPageRefresh( this._selectionModel.getSelectedNode().URI.substring( 7 ) );
					}
					else {
					    this._fireNavigationalStateChanged();
					}
					ibm.portal.debug.exit( "NavigationController.setSelection" );	
				},
				setStateDocument: function ( /*XMLDocument*/newState ) {
					// summary: Sets the internal state DOM to the given state and triggers
					//		any updates based on the delta between the two states.
					// newState: a valid state document
					ibm.portal.debug.entry( "NavigationalStateController.setStateDocument", [ newState ] );
					this._updateInternalState( newState );
					this._fireNavigationalStateChanged();					
					ibm.portal.debug.exit( "NavigationalStateController.setStateDocument" )
				},
				expand: function ( /*String*/pageId, /*int?*/navLevel ) {
					// summary: Expand a navigation node.
					// description: Expands a navigation node. For now we aren't tracking the expand/collapse
					//		state client-side. If we ever do, this would be the place to set this in the state
					//		document.
					// pageId: the id of the node that is to be expanded
					this._fireExpansion( pageId );	
				},
				collapse: function ( /*String*/pageId, /*int?*/navLevel ) {
					// summary: Collapse a navigation node.
					// description: Collapses a navigation node. For now we aren't tracking the expand/collapse
					//		state client-side. If we ever do, this would be the place to set this in the state
					//		document.
					// pageId: the id of the node that is to be collapsed
					this._fireCollapse( pageId );	
				}, 
				getState: function() {
					// summary: Returns the current navigational state.
					// returns: NavigationalState - the current navigational state
					return new com.ibm.portal.navigation.controller.NavigationalState( this._stateDOM, this._selectionModel, this.sendQueryParameters );
				},
				fullPageRefresh: function ( /*String*/pageId ) {
					// summary: Performs a full browser page refresh based on the current state.
					// description: Creates a full portal url to the current state and loads that url in the browser. 
					//		This will reload the full portal page, resulting in re-execution of the theme's JSPs and bootstrap JS.
					// 
					// pageId: (optional) the page to update to be selected prior to the full page refresh.

					ibm.portal.debug.entry( "NavigationalStateController.fullPageRefresh", [ pageId ]);
					// update selection if needed
					var stateManager = this._getStateManager();
					
					if ( pageId ) {
						var selectionAcc = stateManager.newSelectionAccessor();
						selectionAcc.setPageSelection( pageId );
					}
					
					// serialize the state into a portal URL
					var serializationMgr = stateManager.getSerializationManager();
					ibm.portal.debug.text( "Retrieved  current state: " + dojox.data.dom.innerXML( stateManager.getState() ) );
					var serializationResult = serializationMgr.serialize(stateManager.getState());
					ibm.portal.debug.text( "Serialization result is: " );
					//dojo.debugShallow( serializationResult ); 	
					// return the URL
					var fullRefreshUrl = serializationResult["returnObject"];
					
					self.location=fullRefreshUrl;
					ibm.portal.debug.exit( "NavigationalStateController.fullPageRefresh", [ fullRefreshUrl ] );
					return fullRefreshUrl; // String
				},                
				_updateInternalState: function ( /*XMLDocument*/newState ) {
					// summary: Updates the internal state document and selection model if necessary.
					// newState: the new internal state
					ibm.portal.debug.entry( "NavigationalStateController._updateInternalState", [ dojox.data.dom.innerXML( newState ) ] );
					if ( this._hasSelectionChanged( newState ) ) {
						var newSelection = this._getPageSelection( newState );
						//This must be done in this order since the selection model could calculate a 
						//different selection than the one reflected in the newState document.
						this._mergeState(newState);
						this._updateInternalSelection( newSelection );		
					}
					else {
						// merge the new state into our current state
						this._mergeState(newState);
					}
					ibm.portal.debug.exit( "NavigationalStateController._updateInternalState" );
				},
				_mergeState: function(newState) {
					// summary: Merges the new state into the current state
					// description: Embeds both state documents into one "merge document" and applies
					// 		the "StateMerge.xsl" transformation. The result of the transformation
					//		is the state document that represents the new "current" state.
					if (this._stateDOM != null) {
						var serializedState = dojox.data.dom.innerXML(this._stateDOM);
						// get rid of the XML header
						var pos = serializedState.indexOf("?>");
						if (pos > 0) {
							serializedState = serializedState.substring(pos + 2, serializedState.length);
						}
						var serializedNewState = dojox.data.dom.innerXML(newState);
						pos = serializedNewState.indexOf("?>");
						if (pos > 0) {
							serializedNewState = serializedNewState.substring(pos + 2, serializedNewState.length);
						}
						var merge = this._mergeTemplate.replace(/--basestate--/g, serializedState).replace(/--newstate--/g, serializedNewState);
						var mergeDoc = dojox.data.dom.createDocument(merge);
						var results = this._mergeXslt.transformToDocument( mergeDoc, {}, true);
						// update the internal state with the merge result
						this._stateDOM = dojox.data.dom.createDocument(results);
					} else {
						// just assign the new state
						this._stateDOM = newState;
					}
				},
				_updateInternalSelection: function ( /*String*/newSelection ) {
					// summary: Updates the selection in the internal state.	
					if ( newSelection.indexOf("cm:oid:") != 0 )  {
						newSelection = "cm:oid:" + newSelection;
					}				
					this._selectionModel = new com.ibm.portal.services.NavigationSelectionModel( newSelection );
					
					//Set the selection.					
					var stateManager = this._getStateManager();
					var selectionAcc = stateManager.newSelectionAccessor();
                    if(this._selectionModel.getSelectedNode()){
    					selectionAcc.setPageSelection( this._selectionModel.getSelectedNode().URI.substring( 7 ) );	
                    }
				},
				_updatePortletState: function (/*PortletInfo*/portletInfo) {
					// summary: Sets the new portlet state.
					// description: Since we don't know the new portlet state until after we get the response
					//		from the portlet fragment feed, we subscribe to the fragment feed's response topic
					// 		and update the state "posthumously" instead of pre-emptively as we do for selection
					//		changes. It's rare that this function should ever be called directly.
					// portletInfo: the portlet info object passed by the fragment feed response topic
					ibm.portal.debug.entry( "NavigationalStateController.setPortletState", [ portletInfo ] );
					
					var portletWindowId = portletInfo.windowId;
					var stateManager = this._getStateManager();
					
					// register the state vary expressions
					this._updateStateVary( portletInfo.uri, portletInfo.stateVaryExpressions );

					var newState = portletInfo.updatedState;
					if (newState != null) {
						// check if we need to update some of the other portlets due to a navigational state change
						this._determineStateDependencies( portletInfo.otherPortlets, portletInfo.dependentPortlets, portletInfo.updatedState );
						// update the navigational state
						this._updateInternalState( newState );
						//this._updateInternalState( newState );
						ibm.portal.debug.text( "New state (after state merge) is: " + dojox.data.dom.innerXML( this._stateDOM ) );
						this._firePortletStateUpdated();
					}
					ibm.portal.debug.exit( "NavigationalStateController.setPortletState" );
				},
				_updateStateVary: function( /*String*/ id, /*Array*/ expr) {
					// summary: Updates the state vary expressions.
					if (expr != null) {
						this._varyMgr.setExpressions(id, expr);
					}
				},
				_determineStateDependencies: function(/*Array*/ otherPortlets, /*Array*/ depPortlets, /*XMLDocument*/ newState) {
					// summary: Detects state dependencies
					// description: Iterates over all portlets returned by portletInfo.otherPortlets and
					//		checks whether the state of these portlets has changed. State changes are
					//		detected by applying the XPath expressions the specific portlet returned
					//		in its last render response. The XPath expressions are stored by the
					//		StateVaryManager.
					if (otherPortlets != null) {
						for (var i=0; i<otherPortlets.length; i++) {
							var other = otherPortlets[i];
							var id = other.uri;
							var expressions = this._varyMgr.getExpressions(id);
							if (expressions != null) {
								for (var j=0; j<expressions.length; j++) {
									var expr = expressions[j];
									if (this._hasStateChanged(newState, expr)) {
										// add the portlet to the list of dependent portlets
										depPortlets.push({ "link": other.link, "portlet": other.portlet, "uri": other.uri });
										break;
									};
								}
							}
						}
					}
				},
				_hasStateChanged: function(/*dojo.dom.DOMDocument*/newState, /*String*/ xpath) {
					// alert("hasStateChanged: " + xpath);
					// alert(dojo.dom.innerXML(newState));
					// alert(dojo.dom.innerXML(this._stateDOM));

					// assume the state has not changed
					var result = false;

					// apply the xpath expression to both state documents
					var newNodes = com.ibm.portal.xpath.evaluateXPath(xpath, newState, this._namespaces );
					var prevNodes = com.ibm.portal.xpath.evaluateXPath(xpath, this._stateDOM, this._namespaces );
                    
					if (newNodes != null && prevNodes == null) {
						result = true;
					} else if (newNodes != null && prevNodes != null) {
						var nl = newNodes.length;
						if (nl > 0 && nl != prevNodes.length) {
							result = true;
						} else {
							// iterate over nodes
							for (var i=0; i<newNodes.length; i++) {
								var n1 = newNodes[i];
								var n2 = prevNodes[i];
								if (n1.attributes.length != n2.attributes.length) {
									result = true;
									break;
								} else if (n1.childNodes.length != n2.childNodes.length) {
									result = true;
									break;
								} else {
									// serialize
									var ser1 = dojox.data.dom.innerXML(n1);
									var ser2 = dojox.data.dom.innerXML(n2);
									if (ser1 != ser2) {                 
										result = true;
										break;
									}
								}
							}
						}
					}
					return result;
				},
				_hasThemeChanged: function ( /*NavigationSelectionModel*/selectionModel ) {
					var result = false;
					if ( this.currentTheme != null ) {
						result = ( this.currentTheme != selectionModel.getSelectedNodeTheme() );
					}
					else {
						this.currentTheme = selectionModel.getSelectedNodeTheme();
					}
					return result;
				},
				_hasThemePolicyChanged: function ( /*NavigationSelectionModel*/selectionModel ) {
					var result = false;
					if ( this.currentThemePolicy != null ) {
						result = ( this.currentThemePolicy != selectionModel.getSelectedNodeThemePolicy() );
					}
					else {
						this.currentThemePolicy = selectionModel.getSelectedNodeThemePolicy();
					}
					return result;	
				},
				_hasSelectionChanged: function ( /*XMLDocument*/newState ) {
					// summary: Determines if a selection change has occurred.
					// newState: a valid state document
					// returns: true if the selection has changed from the currently stored internal state
					ibm.portal.debug.entry( "NavigationalStateController._hasSelectionChanged", [newState] );
					
					//If there is no internal state, the selection MUST have changed.
					if ( !this._stateDOM ) {
						ibm.portal.debug.exit( "NavigationalStateController._hasSelectionChanged", [true] );
						return true;
					}
					//Get the current page selection.
					var currentSelection = this._getPageSelection();
					ibm.portal.debug.text( "Current selection is: " + currentSelection );
					//Get the page selection in the new state document.
					var newSelection = this._getPageSelection( newState );
					ibm.portal.debug.text( "New selection is: " + newSelection );
					//Determine the return value.
					var retVal = ( currentSelection != newSelection );
									
					ibm.portal.debug.exit( "NavigationalStateController._hasSelectionChanged", [retVal] );
					return retVal;
				},
				_getStateManager: function ( /*XMLDocument?*/ stateDom ) {
					// summary: Returns an instance of the state manager.
					// stateDom: the state document to operate on.
					if ( !this._stateManager ) {
						this._stateManager = new com.ibm.portal.state.StateManager( ibmPortalConfig["contentHandlerURI"] );
					}
					if ( !stateDom ) {
						stateDom = this._stateDOM;
					}	 
					this._stateManager.reset( stateDom );
					return this._stateManager;  
				},
				_getPageSelection: function ( /*XMLDocument?*/ stateDom ) {
					// summary: Returns the current selection.
					// stateDom: the state document to operate upon.					
					//Get the current page selection.
					var stateMgr = this._getStateManager( stateDom );
					var selectionAcc = stateMgr.newSelectionAccessor();
					return selectionAcc.getPageSelection();	
				},
				_fireNavigationalStateChanged: function ( backButton ) {
					// summary: Fires the navigational state changed event. Passes the NavigationalState and the
					//		backbutton indicator to the topic.
					//dojo.publish( "/portal/NavigationalStateController/stateChanged", { state: this.getState(), backButton: backButton } );
					com.ibm.portal.EVENT_BROKER.stateChanged.fire( { state: this.getState(), backButton: backButton } );
				},
				_firePortletStateUpdated: function ( backButton ) {
					// summary: Fires the navigational state changed event. Passes the NavigationalState and the
					//		backbutton indicator to the topic.
					//dojo.publish( "/portal/NavigationalStateController/portletStateUpdated", { state: this.getState(), backButton: backButton } );
					com.ibm.portal.EVENT_BROKER.fragmentUpdated.fire( { state: this.getState(), backButton: backButton } );
				},
				_fireExpansion: function ( /*String*/pageId ) {
					// summary: Fires the node expansion event. Passes the pageId to the topic.
					dojo.publish( "/portal/NavigationalStateController/expansion", [{ pageId: pageId }] );	
				},
				_fireCollapse: function ( /*String*/pageId ) {
					// summary: Fires the node collapse event. Passes the pageId to the topic.
					dojo.publish( "/portal/NavigationalStateController/collapse", [{ pageId: pageId }] );	
				}	
			  }
);

com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER = new com.ibm.portal.navigation.controller.NavigationalStateController();

}
