/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.query.XmlQueryRuleEngine"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.query.XmlQueryRuleEngine"] = true;
/*
 * @author Ben
 */ 
dojo.provide("com.ibm.query.XmlQueryRuleEngine");
// Extension of com.ibm.query.QueryRuleEngine for XML formatted data using native XPath APIs
// for cross-browser, extremely fast queries 

dojo.require("com.ibm.query.QueryRuleEngine");
dojo.require("com.ibm.xml.SimpleXmlAccessor");
dojo.require("com.ibm.domUtilities");
dojo.require("dojox.data.dom");

dojo.declare("com.ibm.query.XmlQueryRuleEngine",
	[com.ibm.query.QueryRuleEngine, com.ibm.xml.SimpleXmlAccessor],
	{
		// queryRules: Object map of key-value pairs.  Values are objects which contain these
		//		properties -
		//		{
		// 			selector: String xpath query to retrieve elements only (not attributes)
		// 			field: String specifies what part of the node to retrieve.  
		// 				- "$value"||"value" gets the "value" property of the node
		// 				- "@value" gets the "value" attribute of the node
		// 				- ""||null gets the node itself
		// 			setter: Function called to set the property on the data matching this
		// 				rule whenever the selector returns no nodes.  
		// 				This handles cases when the xpath query itself returns nothing 
		// 				and nodes need to be created first.  If the selector succeeds to begin 
		// 				with, this setter is not called since the value can be set via the field.
		//			unset: String optional indicating what to set on the node to handle an
		//				unset operation.
		//				- ""||null deletes the value matched in the rule field from the node
		//				- "!" deletes the entire node itself
		//			type: String indicating the type of the value that this query rule
		//				matches.  Supports -
		//				"String" - Default
		//				"Number" - value is explicitly converted to a Number on 
		// 					get/set operations
		//				"Node" - value is a DOMNode
		//				"Date" - value is a Javascript Date object.  If the raw data is a String,
		// 					it is converted to/from ISO8601	format or as W3C specifies at
		// 					http://www.w3.org/TR/NOTE-datetime.
		//			readOnly: Boolean indicates if this attribute is read-only or not.  If a write
		//				operation is attempted on a read-only attribute, it fails.
		//			deferMultiValued: Boolean indicating that multi-valued set operations should
		//				always explicitly call the setter function provided by the rule instead 
		//				of attempting to set the multi-valued value on the nodes retrieved via
		//				the selector.
		//			deferSet: Boolean indicating if all write operations should call the setter
		//				function directly instead of attempting to use the selector and field
		// 				properties via the normal query rule processing.
		//		}
		queryRules: {},
		
		getField: function(/*XmlNodeList*/dataSet, /*String*/field, /*Boolean?*/multiValued) {
			var mapper = this._mapField(field),
				fieldName = field.substr(1),
				value = null;
			if(mapper) {
				value = dataSet[mapper](fieldName);
			}
			else {
				value = dataSet;
			}
			if(!multiValued) {
				value = value[0];
			}
			
			return value;
		},
		setField: function(/*XmlNodeList*/dataSet, /*String*/field, /*anything|Array*/value, /*Boolean?*/multiValued) {
			var mapper = this._mapField(field),
				fieldName = field.substr(1);
			
			if(!mapper) {
				mapper = "replace";
			}
			
			value = dataSet[mapper](fieldName, value);
			
			return true;
		},
		unsetField:  function(/*XmlNodeList*/dataSet, /*String*/field, /*String*/unsetField) {
			var mapper = null,
				fieldName = field.substr(1);
			switch(unsetField.charAt(0)) {
				case "!":
					mapper = "remove";
					break;
				default: 
					switch(fieldName.charAt(0)) {
						case "$":
							mapper = "removeProp";
							break;
						case "@":
							mapper = "removeAttr";
							break;
						default:
							mapper = "remove";
							break;
					}
			}
			
			dataSet[mapper](fieldName);
			return true;
		},
		getByRule: function(/*Object*/ data, /*Object*/ rule, /*Boolean?*/ multiValued) {
			var ret = {
				found: false
			};
			var nodeSet = this.select(rule.selector, data, !multiValued);
			if(nodeSet.length == 0) return ret;	// indicating no selector results
			ret.found = true;
			
			var fieldResult = this.getField(nodeSet, rule.field, multiValued);
			if(multiValued) {
				ret.value = dojo.map(fieldResult, function(fRes){
					return this.toType(fRes, rule.type);
				}, this);
			}
			else {
				ret.value = this.toType(fieldResult, rule.type);
			}
			
			if(ret.value === undefined) ret.found = false;	// the field didn't even exist
			
			return ret;
		},
		setByRule: function(/*Object*/ data, /*Object*/ rule, /*anything|Array*/ value, /*Boolean?*/ multiValued) {
			if(this.isReadOnly(rule)) return false;
			if(multiValued) {
				value = dojo.map(value, function(val){
					return this.fromType(val, rule.type);
				}, this);
			}
			else {
				value = this.fromType(value, rule.type);
			}
			if((rule.deferSet || (multiValued && rule.deferMultiValued)) && rule.setter) {
				return rule.setter.call(this.context, data, rule, value, multiValued);
			}
			var nodeSet = this.select(rule.selector, data, !multiValued);
			if(nodeSet.length == 0) {
				if(rule.setter) {
					return rule.setter.call(this.context, data, rule, value, multiValued);
				}
				return false;	// false, indicating set failed
			}
			
			return this.setField(nodeSet, rule.field, value, multiValued);
		},
		unsetByRule: function(/*Object*/ data, /*Object*/ rule) {
			if(this.isReadOnly(rule)) return false;
			var nodeSet = this.select(rule.selector, data, true);
			if(nodeSet.length == 0) return true;	// undefined, indicating was already unset
			
			return this.unsetField(nodeSet, rule.field, rule.unset);
		},
		_mapField: function(field) {
			switch(field.charAt(0)) {
				case "$":
					return "prop";
				case "@":
					return "attr";
				default: 
					return null;
			}
		}
	}
);

}
