/**
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: hujianh
 * Date: Jul 25, 2007
 */
package com.ibm.alphablox.tooling.querytools;

import java.util.Locale;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.alphablox.blox.AdminBlox;
import com.alphablox.blox.BloxContext;
import com.alphablox.blox.BloxContextFactory;
import com.alphablox.blox.DataBlox;
import com.alphablox.blox.DataBloxCannotConnectException;
import com.alphablox.blox.ServerBloxException;
import com.alphablox.blox.data.mdb.MDBMetaData;
import com.alphablox.blox.repository.DataSource;
import com.alphablox.blox.uimodel.core.IModelDispatcher;
import com.alphablox.blox.uimodel.core.MessageBox;
import com.alphablox.util.BadConnectionException;
import com.alphablox.util.DataException;
import com.alphablox.util.ILocalizedException;
import com.alphablox.util.Message;
import com.ibm.alphablox.tooling.AppContext;

/**
 * Mainly used to get default query
 * @author hujianh@cn.ibm.com
 *
 */
public class QueryDesignerBean {
private static final boolean DEBUG = false;
//  Strings indicating adapter types.
    static public String DS_HYPERION = "Hyperion Essbase";
    static public String DS_HYPERION_EDS = "Essbase Enterprise Server";
    static public String DS_DB2FOROLAP = "IBM DB2 OLAP Server";
    static public String DS_DB2FOROLAP_DS = "IBM DB2 OLAP Server Deployment Services";
    static public String DS_MSAS = "MSOLAP.2";
    static public String DS_SSAS = "MSOLAP.3";
    static public String DS_SAP_BW = "MDrmSap.2";
    static private String DS_ALPHABLOXCUBE = "Alphablox cube database";
    static private String DS_IBMCUBINGSERVICES = "Cubing Services"; // Note: must be consistent with CubingServicesConnection.getDatabaseProductName()
    static private String DS_SQLSERVER = "Microsoft SQL Server";
    static private String DS_ORACLE = "Oracle";
    // DB2 and Sybase return different types depending on the driver.
    static private String DS_DB2_TYPE3 = "DB2/6000";
    static private String DS_DB2_TYPE4 = "DB2/NT";
    static private String DS_INFORMIX = "Informix";
    static private String DS_DERBY = "Apache Derby";
    static private String DS_DB2_AS400 = "AS/400";
    static private String DS_DB2_ZOS = "DB2";

    // Strings indicating catalog table name for relational databases.
    static public String SQLSERVER_CATALOG_TABLE_NAME = "TABLE_NAME";
    static public String DB2_CATALOG_TABLE_NAME = "TABNAME";
    static public String ORACLE_CATALOG_TABLE_NAME = "TNAME";

    private AppContext mAppContext;
    private DataBlox mDataBlox;
    
    private String mCubeList[];

    private AdminBlox mAdminBlox;
    private IModelDispatcher mDispatcher = null;

    public QueryDesignerBean() {
    }
    
    /**
     * initialized
     * @param req
     * @param resp
     * @param appContext
     * @param db
     * @return
     */
    public boolean init(HttpServletRequest req, HttpServletResponse resp, AppContext appContext) {
        try {
            this.mAppContext = appContext;

            BloxContext bloxContext = BloxContextFactory.getBloxContext(req, resp);

            mAdminBlox = new AdminBlox();
            mAdminBlox.init(bloxContext, "queryBuilderAdmin");         

//            changeDataBlox(db);
            return true;
        } catch (Exception e) {
            mAppContext.getString("EXCEPTION_FAILED_TO_INITIALIZE", e.getMessage());
//            setStatusOnClient(false, mAppContext.getString("CLIENT_MESSAGE_NOT_INITIALIZED"));
            return false;
        }

    }

    /**
     * @param db
     */
    public void changeDataBlox(DataBlox db) {
        setDataBlox(db);
        setCubeList();
    }

    public DataBlox getDataBlox() {
        return mDataBlox;
    }   

    public void setDataBlox(DataBlox dataBlox) {
        this.mDataBlox = dataBlox;
    }

// JGD getCubeList() and getCubeListAsString() can go once we incorporate the
// cube formblox.
    public String[] getCubeList() {
        return mCubeList==null?new String[0]:mCubeList;
    }

    public String getCubeListAsString() {
        String cubeListString = "";

        if (mCubeList != null) {
            for (int i = 0; i < mCubeList.length; ++i) {
                if (i > 0) cubeListString += ", ";
                cubeListString += mCubeList[i];
            }
        }
        return cubeListString;
    }

// JGD this method will go once we incorporate cube form blox.
    private void setCubeList() {
        try {
            String dbName = getDatabaseProductName();
            // Right now, only ODBO data sources expose multiple cubes.  For Hyperion, there is
            // only one cube, and it's always named 'Cube1'.
            if (isMultiCubeDS(dbName)) {
                MDBMetaData metaMdb = (MDBMetaData) mDataBlox.getMetaData();
                mCubeList = new String[metaMdb.getCubes().length];
                for (int i = 0; i < metaMdb.getCubes().length; ++i) {
                    mCubeList[i] = metaMdb.getCube(i).getName();
                }
            } else {
                mCubeList = null;
            }
        } catch (Exception e) {
            System.out.println(mAppContext.getString("EXCEPTION_ERROR_BUILDING_CUBE_LIST", e.getMessage()));
        }
    }

    /**
     * @param dbName
     * @return
     */
    private boolean isMultiCubeDS(String dbName) {
        return DS_MSAS.equalsIgnoreCase(dbName) ||
                DS_SSAS.equalsIgnoreCase(dbName) ||
                DS_ALPHABLOXCUBE.equalsIgnoreCase(dbName) ||
                dbName.indexOf(DS_IBMCUBINGSERVICES) >= 0 ||
                DS_SAP_BW.equalsIgnoreCase(dbName);
    }

    public boolean isRelational() throws Exception {
        if (mDataBlox.isConnected()) {
            int dbtype = mAdminBlox.getDataSource(mDataBlox.getDataSourceName()).getAdapterType();
            return (dbtype == DataSource.TYPE_RDB);
        }
        return false;
    }

    public String getDefaultQuery() throws Exception {
        String defaultQuery = null;

        if (!mDataBlox.isConnected()) {
            System.out.println(mAppContext.getString("EXCEPTION_CANNOT_SET_QUERY_NOT_CONNECTED"));
        }

        else {
            String dbName = getDatabaseProductName();

            if (DS_SQLSERVER.equalsIgnoreCase(dbName))
                defaultQuery = mAppContext.getString("SQLSERVER_DEFAULT_QUERY");
            else if (DS_ORACLE.equalsIgnoreCase(dbName))
                defaultQuery = mAppContext.getString("ORACLE_DEFAULT_QUERY");
            else if (dbName.indexOf(DS_INFORMIX) != -1)
                defaultQuery = mAppContext.getString("INFORMIX_DEFAULT_QUERY");
            else if (dbName.indexOf(DS_DERBY) != -1)
                defaultQuery = mAppContext.getString("DERBY_DEFAULT_QUERY");
            else if (dbName.indexOf(DS_DB2_AS400) != -1)
                defaultQuery = mAppContext.getString("DB2_AS400_DEFAULT_QUERY");
            else if (dbName.equals(DS_DB2_ZOS))
                defaultQuery = mAppContext.getString("DB2_ZOS_DEFAULT_QUERY");
            else if ((DS_DB2_TYPE3.equalsIgnoreCase(dbName)) ||
                    (DS_DB2_TYPE4.equalsIgnoreCase(dbName))) {
                defaultQuery = mAppContext.getString("DB2_DEFAULT_QUERY");
            }
            else if (DS_MSAS.equalsIgnoreCase(dbName) || DS_SSAS.equalsIgnoreCase(dbName)) {
                if ((mCubeList != null) && (mCubeList.length > 0)) {
                    defaultQuery = mAppContext.getString("MSAS_DEFAULT_QUERY", mCubeList[0]);
                } else {
                    defaultQuery = "";
                    System.out.println(mAppContext.getString("WARNING_EMPTY_CUBELIST"));
                }
            }
            else if (DS_ALPHABLOXCUBE.equalsIgnoreCase(dbName)) {
                if ((mCubeList != null) && (mCubeList.length > 0)) {
                    defaultQuery = mAppContext.getString("ALPHABLOXCUBE_DEFAULT_QUERY", mCubeList[0]);
                } else {
                    defaultQuery = mAppContext.getString("ALPHABLOXCUBE_DEFAULT_QUERY", "<cubeName>");
                }
            }
            else if (dbName.indexOf(DS_IBMCUBINGSERVICES) >= 0) {
                if ((mCubeList != null) && (mCubeList.length > 0)) {
                    defaultQuery = mAppContext.getString("IBMCUBINGSERVICES_DEFAULT_QUERY", mCubeList[0]);
                } else {
                    defaultQuery = mAppContext.getString("IBMCUBINGSERVICES_DEFAULT_QUERY", "<cubeName>");
                }
            }
            else if (DS_HYPERION.equalsIgnoreCase(dbName) ||
                    DS_DB2FOROLAP.equalsIgnoreCase(dbName) ||
                    dbName.startsWith(DS_DB2FOROLAP) ||
                    dbName.startsWith(DS_DB2FOROLAP_DS)) {
                defaultQuery = mAppContext.getString("HYPERION_DEFAULT_QUERY");
            }
            else if (DS_SAP_BW.equalsIgnoreCase(dbName)) {
                defaultQuery = mAppContext.getString("SAP_BW_DEFAULT_QUERY", mCubeList[0]);
            }
            else {
                defaultQuery = "";
                System.out.println(mAppContext.getString("WARNING_UNRECOGNIZED_DATASOURCE", dbName));
            }
        }
        return defaultQuery;
    }

    public String getDatabaseProductName() throws BadConnectionException, Exception, DataBloxCannotConnectException, ServerBloxException {
        return mDataBlox.getMetaData().getDatabaseProductName();
    }

    public String getDefaultQuery(DataBlox dataBlox, String cubeName)throws Exception {
        f("getDefaultQuery:"+dataBlox+","+cubeName);
        String str = this.getDefaultQuery();
        f("default query="+str);
        if( this.mCubeList.length>0){
            int s = str.indexOf('[');
            int e = str.indexOf(']',s);
            String str2 = str.substring(0, s+1)+cubeName+str.substring(e);
            str = str2;
        }
        f("defaultQuery="+str);
        return str;
    }
    
    public boolean isMDXDataSource(){
        try{
            String dbName = this.getDatabaseProductName();
            f("isMDXDataSource:"+dbName);
            return isMultiCubeDS(dbName);
        }catch(Exception ex){
            ex.printStackTrace();
        }
        return false;
    }
    
    public boolean connect(boolean executeQuery) throws Exception {
        boolean returnStatus;

        try {
            if (mDataBlox.isConnected()) {
                mDataBlox.disconnect(true);
            }

            mDataBlox.connect(false);

            // If the connect works, setup the cube list and execute the default query.
            // JGD setCubeList() call will go once we incorporate the cube form blox.
            setCubeList();
            if(executeQuery){
                executeQuery();
            }
            // setup the return values.
            returnStatus = true;
        } catch (DataBloxCannotConnectException e) {
            String error = mAppContext.getString("EXCEPTION_CANNOT_CONNECT", e.getCause().getMessage());
            System.out.println(error);
            MessageBox.message(mDispatcher , mAppContext.getString("STATUS_DIALOG_TITLE_CONNECTION"), error);
            returnStatus = false;
        } catch (ServerBloxException e) {
            String error = e.getCause().getMessage();
            System.out.println(error);
            MessageBox.message(mDispatcher, mAppContext.getString("STATUS_DIALOG_TITLE_CONNECTION"), error);
            returnStatus = false;
        } catch (Throwable e) {
            String error = e.getMessage();
            System.out.println(error);
            MessageBox.message(mDispatcher, mAppContext.getString("STATUS_DIALOG_TITLE_CONNECTION"), error);
            returnStatus = false;
        }
        return returnStatus;
    }
    
    public boolean executeQuery() {
        boolean returnStatus = true;

        try {
            if (!mDataBlox.isConnected()) {
                System.out.println(mAppContext.getString("EXCEPTION_CANNOT_EXECUTE_QUERY_NOT_CONNECTED"));
                setStatusOnClient(false, mAppContext.getString("CLIENT_MESSAGE_NOT_CONNECTED"));
                mDispatcher.sendClientCommand("alert('" + mAppContext.getString("NOT_CONNECTED_MESSAGE") + "')");
                returnStatus = false;
            }
            else {
                mDataBlox.clearResultSet();
//                mDataBlox.setQuery(mQueryText.getFormValue());
                mDataBlox.updateResultSet();
                setStatusOnClient(true, mAppContext.getString("CLIENT_MESSAGE_QUERY_SUCCEEDED"));
            }
        }

        catch (DataException e) {
            try {
                mDataBlox.clearResultSet();
            }
            catch (ServerBloxException se) {
                String serverError = mAppContext.getString("EXCEPTION_SERVER_EXCEPTION", se.getCause().getMessage());
                System.out.println(serverError);
            }

            sendExceptionToDialog("EXCEPTION_DATA_EXCEPTION", e.getCause());
            returnStatus = false;
        }

        catch (ServerBloxException e) {
            try {
                mDataBlox.clearResultSet();
            } catch (ServerBloxException se) {
                String serverError = mAppContext.getString("EXCEPTION_SERVER_EXCEPTION", se.getCause().getMessage());
                System.out.println(serverError);
            }

            sendExceptionToDialog("EXCEPTION_SERVER_EXCEPTION", e.getCause());
            returnStatus = false;
        }

        catch (Throwable e) {
            try {
                mDataBlox.clearResultSet();
            } catch (ServerBloxException se) {
                String serverError = mAppContext.getString("EXCEPTION_SERVER_EXCEPTION", se.getCause().getMessage());
                System.out.println(serverError);
            }

            sendExceptionToDialog("EXCEPTION_SERVER_EXCEPTION", e);
            returnStatus = false;
        }
        return returnStatus;
    }
    
    private void sendExceptionToDialog(String bundleKey, Throwable t) {
        Throwable cause = t.getCause();
        String msg = null;
        String causedBy = "";
        Locale[] locales = new Locale[] {mAppContext.getPreferredLocale()};

        if (t instanceof ILocalizedException)
            msg = ((ILocalizedException)t).getLocalizedMessage(locales);
        else
            msg = t.getLocalizedMessage();

        if (cause != null) {
            if (cause instanceof ILocalizedException)
                causedBy = ((ILocalizedException)cause).getLocalizedMessage(locales);
            else
                causedBy = cause.getLocalizedMessage();

            if (!causedBy.equals(msg))
                causedBy = mAppContext.getString("EXCEPTION_CAUSED_BY", causedBy);
            else
                causedBy = "";
        }

        String error = mAppContext.getString(bundleKey, msg + " " + causedBy);

        new Message(Message.ERROR, "DHTMLQueryBuilder", t).send();
        setStatusOnClient(false, mAppContext.getString("CLIENT_MESSAGE_QUERY_FAILED"));
        MessageBox.message(mDispatcher, mAppContext.getString("STATUS_DIALOG_TITLE_QUERY_FAILED"), error);
    }
//    private void sendExceptionToDialog(String string, Throwable e) {
//        f("exception:"+e.getMessage());
//        
//    }

    private void setStatusOnClient(boolean b, String string) {
        // TODO Auto-generated method stub
        
    }

    private void f(String s){
        if(DEBUG){
            System.out.println(s);
        }
    }

    /**
     * @return the mDispatcher
     */
    public IModelDispatcher getMDispatcher() {
        return mDispatcher;
    }

    /**
     * @param dispatcher the mDispatcher to set
     */
    public void setMDispatcher(IModelDispatcher dispatcher) {
        mDispatcher = dispatcher;
    }
}
