package com.ibm.alphablox.tooling.relational;

import com.alphablox.blox.DataBlox;
import com.alphablox.blox.ServerBloxException;
import com.alphablox.blox.repository.Log;
import com.alphablox.blox.uimodel.core.ComponentContainer;
import com.alphablox.blox.uimodel.core.Controller;
import com.alphablox.blox.uimodel.core.MessageBox;
import com.alphablox.blox.uimodel.core.ModelException;
import com.alphablox.util.DataException;
import com.ibm.alphablox.tooling.relational.controller.DBItemInfoHandler;
import com.ibm.alphablox.tooling.relational.controller.OperatorController;
import com.ibm.alphablox.tooling.relational.controller.SQLEditorHandler;
import com.ibm.alphablox.tooling.relational.controller.TreeHandler;
import com.ibm.alphablox.tooling.relational.view.GridDialogBuilder;

public class PageController extends Controller{

    private OperatorController operatorController;
    private SQLEditorHandler sqlEditorHandler;
    private TreeHandler treeHandler;
    private ComponentContainer topContainer;
    private DataBlox dataBlox;
//    private QueryToolsBlox qtBlox;
    private DBItemInfoHandler dbItemInfoHandler;
    private GridDialogBuilder gridDlgBuilder;
    private OperationContext operationContext;

    public PageController(DataBlox db, OperationContext context) {
        this.dataBlox = db;
//        this.qtBlox = qtBlox;
        this.operationContext = context;
        operationContext.setPageController(this);
    }

    public void initControllers( ComponentContainer topContainer) {
        this.topContainer = topContainer;
        
        ComponentContainer treeContainer = topContainer.getComponentContainer(RMBConstants.TREE_CONTAINER_ID);
        ComponentContainer sqlContainer = topContainer.getComponentContainer(RMBConstants.SQLCONTAINER_ID);
        ComponentContainer operatorContainer = topContainer.getComponentContainer(RMBConstants.OPERATOR_CONTAINER_ID);
        ComponentContainer itemInfoContainer = topContainer.getComponentContainer(RMBConstants.DBITEM_INFO_CONTAINER_ID);
        //controllers
        sqlEditorHandler = new SQLEditorHandler( sqlContainer,operationContext );
        sqlEditorHandler.setDataBlox(dataBlox);
        //grid dialog builder
        gridDlgBuilder = new GridDialogBuilder(operationContext,sqlEditorHandler);
        
        treeHandler = new TreeHandler(treeContainer);
        treeHandler.setOperationContext(operationContext);
        
        dbItemInfoHandler = new DBItemInfoHandler(itemInfoContainer);
        treeHandler.setItemInfoHandler(dbItemInfoHandler);
        
        operatorController = new OperatorController(operatorContainer, this);
        operatorController.setEditorHandler(sqlEditorHandler);
        operatorController.setTreeHandler(treeHandler);
        
        try{
            operatorContainer.setController(operatorController);  
        }catch(Exception ex){
            ex.printStackTrace();
        }        
        
    }
    
    public void changeOperationContext(OperationContext newOpCtx){
        operationContext = newOpCtx;
        operationContext.setPageController(this);
        treeHandler.setOperationContext(operationContext);
        sqlEditorHandler.setOperationContext(operationContext);
        try{
            newOpCtx.setTarget(
                    topContainer.getComponentContainer(
                            RMBConstants.RADIO_CONTAINER_ID));
        }catch(Exception ex){
            Log.sendException(Log.MESSAGE_LEVEL_ERROR, "", ex);
        }
    }
    /**
     * @return the operatorController
     */
    public OperatorController getOperatorController() {
        return operatorController;
    }

    /**
     * @return the sqlEditorHandler
     */
    public SQLEditorHandler getSqlEditorHandler() {
        return sqlEditorHandler;
    }

    /**
     * @return the treeHandler
     */
    public TreeHandler getTreeHandler() {
        return treeHandler;
    }

    public void showWindow(String sql, String title, boolean isValidate) {
        try{
            String dsName = dataBlox.getDataSourceName();
            gridDlgBuilder.setButtonVisible(true);
            gridDlgBuilder.build(
                    dsName,
                    sql, 
                    isValidate? RMBConstants.VALIDATE:RMBConstants.SAMPLE,
                    title,
                    !operationContext.getSqlFactory().isGeneric()); 
            topContainer.getDispatcher().showDialog(gridDlgBuilder.getDialog());
        }catch(DataException ex){
            showErrorDialog(ex,topContainer);
            Log.sendException(Log.MESSAGE_LEVEL_ERROR, Messages.getString("PageController.MsgDlgTitle"), ex); //$NON-NLS-1$
        }catch(ServerBloxException ex){
            showErrorDialog(ex,topContainer);
            Log.sendException(Log.MESSAGE_LEVEL_ERROR, Messages.getString("PageController.MsgDlgTitle"), ex); //$NON-NLS-1$
        }catch(ModelException ex){
            Log.sendException(Log.MESSAGE_LEVEL_ERROR, Messages.getString("PageController.MsgDlgTitle"), ex); //$NON-NLS-1$
            ex.printStackTrace();
        }
    }

    
    public static void showErrorDialog(Exception ex, ComponentContainer topContainer) {
        try{
            StringBuffer sb = new StringBuffer(256);
            sb.append(ex.getMessage()); 
            Throwable t = ex.getCause();
            if(t!=null){
                sb.append(Messages.getString("PageController.CauseStart")); //$NON-NLS-1$
            }
            while(t!=null && t.getCause()!=null){
                sb.append(Messages.getString("PageController.CauseHeader")); //$NON-NLS-1$
                t = t.getCause();
                sb.append(t.getMessage());
            }
            MessageBox.message(topContainer.getDispatcher(), 
                    Messages.getString("PageController.ErrorDialogTitle"),  //$NON-NLS-1$
                    sb.toString());
        }catch(Exception e){
            e.printStackTrace();
            Log.sendException(Log.MESSAGE_LEVEL_ERROR, "", e); //$NON-NLS-1$
        }
    }

    /**
     * @param dataBlox the dataBlox to set
     */
    public void setDataBlox(DataBlox dataBlox) {
        this.dataBlox = dataBlox;
    }

    /**
     * @return the dbItemInfoHandler
     */
    public DBItemInfoHandler getDbItemInfoHandler() {
        return dbItemInfoHandler;
    }
}
