package com.ibm.alphablox.tooling.relational.controller;

import java.util.HashSet;
import java.util.Set;
import java.text.SimpleDateFormat;

import com.alphablox.blox.DataBlox;
import com.alphablox.blox.RDBDataException;
import com.alphablox.blox.data.ResultSet;
import com.alphablox.blox.data.rdb.RDBResultSet;
import com.alphablox.blox.data.rdb.RDBTypes;
import com.alphablox.blox.repository.Log;
import com.alphablox.blox.uimodel.GridBloxModel;
import com.alphablox.blox.uimodel.GridBrixCellModel;
import com.alphablox.blox.uimodel.core.DialogController;
import com.alphablox.blox.uimodel.core.event.ClosedEvent;
import com.alphablox.blox.uimodel.core.event.ModelEvent;
import com.alphablox.blox.uimodel.core.grid.GridCell;
import com.ibm.alphablox.tooling.relational.view.GridDialogBuilder;

public class GridSelectorDlgController extends DialogController {

    private SQLEditorHandler editorHandler;

    private GridDialogBuilder builder;

    public GridSelectorDlgController(GridDialogBuilder builder,
            SQLEditorHandler editorHandler) {
        super(builder.getDialog());
        this.editorHandler = editorHandler;
        this.builder = builder;
    }

    private Object getRawValue(RDBResultSet rrs, GridCell gridCell) {
        try {
            int nativeRow = ((GridBrixCellModel) gridCell).getNativeRow();
            int nativeColumn = ((GridBrixCellModel) gridCell).getNativeColumn();
            if (nativeColumn >= 0 && nativeRow >= 0) {
                rrs.resetCurrentRow();
                int count = 0;
                Object[] currentRow = rrs.getNextRow(true);
                while (count < nativeRow) {
                    currentRow = rrs.getNextRow(true);
                    count++;
                }
                if (currentRow != null && currentRow.length > nativeColumn) {
                    return currentRow[nativeColumn];
                }
            } else {
                return gridCell.getValue();
            }
        } catch (RDBDataException rde) {
        }
        return gridCell.getValue();
    }

    public void actionSubmit(ModelEvent event) throws Exception {
        try {
            GridBloxModel gridBloxModel = builder.getGridBlox().getGridBloxModel();

            DataBlox db = builder.getGridBlox().getDataBlox();
            GridCell[] selectedCells = gridBloxModel.getGrid().getSelectedCells();
            StringBuffer sb = new StringBuffer(128);
            for (int i = 0; i < selectedCells.length; i++) {
                if (i > 0) {
                    sb.append(",");
                }
                ResultSet rs = db.getResultSet();
                if (rs instanceof RDBResultSet) {
                    RDBResultSet rrs = (RDBResultSet) rs;
                    int type = rrs.getType(selectedCells[i].getColumn().getIndex());
                    if (type == RDBTypes.DATE) {
                        Object value = getRawValue(rrs, selectedCells[i]);
                        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                        if(value instanceof java.util.Date) {
                            sb.append("'"+sdf.format((java.util.Date)value)+"'");
                        }
                        else {
                             sb.append("'"+sdf.format(value)+"'");
                        }
                    } else if (type == RDBTypes.TIME) {
                        Object value = getRawValue(rrs, selectedCells[i]);
                        SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
                        if(value instanceof java.util.Date) {
                            sb.append("'"+sdf.format((java.util.Date)value)+"'");
                        }
                        else {
                             sb.append("'"+sdf.format(value)+"'");
                        }

                    } else if (type == RDBTypes.TIMESTAMP) {
                        Object value = getRawValue(rrs, selectedCells[i]);
                        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                        if(value instanceof java.util.Date) {
                            sb.append("'"+sdf.format((java.util.Date)value)+"'");
                        }
                        else {
                             sb.append("'"+sdf.format(value)+"'");
                        }
                    } else {
                        sb.append(encloseString(db, selectedCells[i]));
                    }
                } else {
                    sb.append(encloseString(db, selectedCells[i]));
                }
            }
            editorHandler.insertQuery(sb.toString());

            builder.getDialog().close();
        } catch (Exception ex) {
//            ex.printStackTrace();
            Log.sendException(Log.MESSAGE_LEVEL_ERROR, "RMB", ex);
        }
    }

    private static Set NonCharTypes = new HashSet(13);
    static{
        NonCharTypes.add(String.valueOf(RDBTypes.NUMERIC));
        NonCharTypes.add(String.valueOf(RDBTypes.DOUBLE));
        NonCharTypes.add(String.valueOf(RDBTypes.FLOAT));
        NonCharTypes.add(String.valueOf(RDBTypes.INTEGER));
        NonCharTypes.add(String.valueOf(RDBTypes.REAL));
        NonCharTypes.add(String.valueOf(RDBTypes.SMALLINT));
        NonCharTypes.add(String.valueOf(RDBTypes.BIGINT));
        NonCharTypes.add(String.valueOf(RDBTypes.TINYINT));
    };
    
    private String encloseString(DataBlox db, GridCell cell) throws Exception{
        ResultSet rs = db.getResultSet();
        if( rs instanceof RDBResultSet){
            RDBResultSet rrs = (RDBResultSet)rs;
            int type = rrs.getType(cell.getColumn().getIndex());
            if(NonCharTypes.contains(String.valueOf(type))){
                return cell.getValue();
            }else{
                return "'"+cell.getValue()+"'";
            }
//            System.out.println("selected cell type is "+type);
        }
        return cell.getValue();
    }
    public void actionCancel(ModelEvent event) throws Exception {
        builder.getDialog().close();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.alphablox.blox.uimodel.core.DialogController#handleClosedEvent(com.alphablox.blox.uimodel.core.event.ClosedEvent)
     */
    public boolean handleClosedEvent(ClosedEvent event) {
        return super.handleClosedEvent(event);
    }
//
//    public void setOperationContext(OperationContext operationContext) {
//        this.operationContext = operationContext;
//    }
}
