package com.ibm.alphablox.tooling.relational.controller;

import java.text.SimpleDateFormat;
import java.util.Date;

import com.alphablox.blox.DataBlox;
import com.alphablox.blox.repository.Log;
import com.alphablox.blox.uimodel.core.ComponentContainer;
import com.alphablox.blox.uimodel.core.Controller;
import com.alphablox.blox.uimodel.core.Edit;
import com.alphablox.blox.uimodel.core.event.ContentsChangedEvent;
import com.alphablox.blox.uimodel.core.event.ModelEvent;
import com.ibm.alphablox.bloxbuilder.tooling.querytools.QueryToolsBlox;
import com.ibm.alphablox.tooling.relational.OperationContext;
import com.ibm.alphablox.tooling.relational.PageController;
import com.ibm.alphablox.tooling.relational.RMBConstants;

public class SQLEditorHandler extends Controller{
    private static final int MAX_ERROR = 5;
    private static final boolean DEBUG = false;
    private ComponentContainer container;
    private final Edit sqlText;  
    private DataBlox dataBlox;
    private OperationContext operationContext;
    
    public SQLEditorHandler(ComponentContainer container, OperationContext opContext){
        this.container = container;
        sqlText = container.getEdit(RMBConstants.SQL_EDITOR_ID);
        this.operationContext = opContext;
        try{
            this.container.setController(this);
            this.addEventHandlerToEnd(this);
        }catch(Exception ex){
            ex.printStackTrace();
        }
    }
    
    public void actionValidate(ModelEvent event) throws Exception{
      String sql = getSqlText().getValue();
      sql = operationContext.getSqlFactory().generateValidateSql(sql);
      Log.sendMessage(Log.MESSAGE_LEVEL_DEBUG, 
              RMBConstants.ErrorLogTitle, 
              "sql to validate:"+sql);
      operationContext.getPageController().showWindow(sql,"Result of the Query",true);
  }
    /**
     * replace the old query with the new query!
     * @param newQuery the new query
     */
    private void setQuery(String newQuery){
        try{
            newQuery = newQuery.replaceAll("\r", "");
            sqlText.setValue(newQuery);
            changeQueryToolBlox(newQuery);
            sqlText.changed();
        }catch(Exception ex){
            Log.sendException(Log.MESSAGE_LEVEL_ERROR, RMBConstants.ErrorLogTitle, ex);
            PageController.showErrorDialog(ex, this.container);
        }
    }
    public void newBlankQuery(){
        setQuery(RMBConstants.DEFAULT_NEW_QUERY);
        try{
            sqlText.setCaretPosition(15);
        }catch(Exception ex){
            
        }
    }
    /**
     * @param newQuery
     */
    private void changeQueryToolBlox(String newQuery) {
        QueryToolsBlox qtBlox = operationContext.getQtBlox();
        qtBlox.setMemberSelected("RDB:"+newQuery);
    }

    /**
     * Insert the sql part into the current caret position!
     * @param sqlParts the sql part to be inserted!
     */
    public void insertQuery(String sqlParts ){
        try{
            if( DEBUG ){
                this.correctCaretPos();
            }
            int idx = sqlText.getCaretPosition();
            if( idx < 0 ){
                idx = 0;
            }
            if(sqlText.getTextSelection()!=null){
                sqlText.insertText("");
            }
            String value = sqlText.getValue();
            char ch1 = idx>0?value.charAt(idx-1):'\0';
            char ch2 = idx<value.length()-1?value.charAt(idx):'\0';
          
            int caretOffset = sqlParts.length();
            StringBuffer sb = new StringBuffer(256);
            sb.append(sqlText.getValue().substring(0, idx));
          
            if(Character.isLetterOrDigit(ch1)){
                sb.append(' ');
                caretOffset++;
            }
            sb.append(sqlParts);
            if(Character.isLetterOrDigit(ch2)){
                sb.append(' ');
                caretOffset++;
            }
            if( idx < value.length()){
                sb.append(sqlText.getValue().substring(idx));
            }
            
            setQuery(sb.toString());
            sqlText.setCaretPosition(idx + caretOffset);            
        }catch(Exception ex){
            Log.sendException(Log.MESSAGE_LEVEL_ERROR, 
                    RMBConstants.ErrorLogTitle, 
                    ex);
            ex.printStackTrace();
        }
    }

    /**
     * @TODO It seems the Edit component count a new line char as two chars.
     * @return the index of the char after the caret
     * @throws NotSupportedException
     */
     private void correctCaretPos() throws Exception {
        int idx = sqlText.getCaretPosition();
        if(idx < 0 || idx >= sqlText.getValue().length()){
            sqlText.setCaretPosition(sqlText.getValue().length());
        }else{
            String value = sqlText.getValue();
            for(int i=0;i<MAX_ERROR;i++){
                char ch = value.charAt(idx); 
                if(!Character.isLetterOrDigit(ch)){
                    break;
                }
                idx--;
            }
            
            sqlText.setCaretPosition(idx);
        }
    }
    
    
    /**
     * @return the dataBlox
     */
    public DataBlox getDataBlox() {
        return dataBlox;
    }

    /**
     * @param dataBlox the dataBlox to set
     */
    public void setDataBlox(DataBlox dataBlox) {
        this.dataBlox = dataBlox;
    }

    /**
     * @return the sqlText
     */
    public Edit getSqlText() {
        return sqlText;
    }

    /**
     * @return the operationContext
     */
    public OperationContext getOperationContext() {
        return operationContext;
    }

    /**
     * @param operationContext the operationContext to set
     */
    public void setOperationContext(OperationContext operationContext) {
        this.operationContext = operationContext;
    }
    
    protected String getCurrentTime(){
        Date date = new Date();
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ");
        return df.format(date);
    }
    
    public boolean handleContentsChangedEvent( ContentsChangedEvent event ) throws Exception
    {
        this.changeQueryToolBlox(event.getContents());
        return false;
    }
}
