package com.ibm.alphablox.tooling.relational.controller;

import com.alphablox.blox.repository.Log;
import com.alphablox.blox.uimodel.core.Button;
import com.alphablox.blox.uimodel.core.Component;
import com.alphablox.blox.uimodel.core.ComponentContainer;
import com.alphablox.blox.uimodel.core.Controller;
import com.alphablox.blox.uimodel.core.event.ClickEvent;
import com.alphablox.blox.uimodel.core.event.ExpandCollapseEvent;
import com.alphablox.blox.uimodel.core.event.SelectionChangedEvent;
import com.alphablox.blox.uimodel.core.tree.Tree;
import com.alphablox.blox.uimodel.core.tree.TreeNode;
import com.ibm.alphablox.tooling.relational.Messages;
import com.ibm.alphablox.tooling.relational.OperationContext;
import com.ibm.alphablox.tooling.relational.PageController;
import com.ibm.alphablox.tooling.relational.RMBConstants;
import com.ibm.alphablox.tooling.relational.view.TreeContainerBuilder;

public class TreeHandler extends Controller{
    private final ComponentContainer treeContainer;
    private final Tree dbFrameTree;
    private DBItemInfoHandler dbItemInfoHandler;
    public OperationContext operationContext;
    private final TreeContainerBuilder builder;
    
    public TreeHandler(ComponentContainer container){
        treeContainer = container;
        dbFrameTree = treeContainer.getTree(RMBConstants.DBFRAME_TREE_ID);
        builder = (TreeContainerBuilder)treeContainer.getUserObject();
        try{
            treeContainer.setController(this);
        }catch(Exception ex){
            ex.printStackTrace();
        }
    }
    /**
     * @return the dbFrameTree
     */
    public Tree getDbFrameTree() {
        return dbFrameTree;
    }
    
    /**
     * Composite the names of the selected nodes into a String, 
     * which will be put into a sql statement.
     * 
     * @return the sql part based on the selected nodes
     */
    public String getSqlPart( ) {
        TreeNode[] nodes = dbFrameTree.getAllSelected();
        StringBuffer sb = new StringBuffer(128);

        if(nodes.length == 1) {
           appendNodeStr(nodes[0], sb);
           return sb.toString();
        }

        for(int i=0; i<nodes.length; i++){
            if( i>0 && i+1 <= nodes.length){
                sb.append(",\n"); //$NON-NLS-1$
                sb.append("  ");
            }
            appendNodeStr(nodes[i], sb);
        }

        sb.append("\n");
        return sb.toString();
    }
    /**
     * Generate the sql based on the insertPolicy and encloseStyle
     * @param tn the tree node being selected
     * @param sb the StringBuffer that contains the generated SQL
     */
    private void appendNodeStr(TreeNode tn, StringBuffer sb) {
        int level = tn.getNodeLevel();
        if( level == RMBConstants.TABLE_NODE_LEVEL ){ //table
            operationContext.getTableNodeAsStr(tn, sb);
        }else if( level == RMBConstants.COLUMN_NODE_LEVEL ){ //column
            operationContext.getColumnNodeAsStr(tn, sb);
        }
    }
    
    public String getSampleSql() throws Exception{
        TreeNode[] tns = this.dbFrameTree.getAllSelected();
        if(tns==null || tns.length==0){
            throw new Exception(Messages.getString("TreeHandler.NoSampleTarget"));//$NON-NLS-1$
        }
        TreeNode tn = tns[tns.length-1];
        
        String schemaName = ""; //$NON-NLS-1$
        String tableName = ""; //$NON-NLS-1$
        String colName = "*"; //$NON-NLS-1$
        
        int level = 0;
        String[] nodeNames = new String[3];
        while(!tn.isRootNode()){
            nodeNames[level++] = tn.getName();
            tn = tn.getParentNode();
        }
        switch(level){
        case RMBConstants.SCHEMA_NODE_LEVEL:
            Log.sendMessage(Log.MESSAGE_LEVEL_ERROR, 
                    Messages.getString("TreeHandler.DialogTitle"), //$NON-NLS-1$ 
                    Messages.getString("TreeHandler.NoSampleMsg")); //$NON-NLS-2$
            throw new Exception(Messages.getString("TreeHandler.NoSampleMsg"));//$NON-NLS-1$
        case RMBConstants.TABLE_NODE_LEVEL:
            schemaName = nodeNames[1];
            tableName = nodeNames[0];
            break;
        case RMBConstants.COLUMN_NODE_LEVEL:
            schemaName = nodeNames[2];
            tableName = nodeNames[1];
            colName = nodeNames[0];
        }
        
        
        String str = this.operationContext.getSqlFactory().getSampleString(colName, schemaName, tableName);
        return str;
    }
    
    public void setItemInfoHandler(DBItemInfoHandler dbItemInfoHandler) {
        this.dbItemInfoHandler = dbItemInfoHandler;
    }
    
    public boolean handleSelectionChangedEvent(SelectionChangedEvent se) {
        TreeNode[] nodes = this.dbFrameTree.getAllSelected();
        StringBuffer sb = new StringBuffer(128);
        for(int i=0;i<nodes.length;i++){
            if(i>0){
                sb.append(";\n");
            }
            sb.append(nodes[i].getUserObject().toString());
        }
        dbItemInfoHandler.updateText(sb.toString());
        return false;
    }
    
    public void setOperationContext(OperationContext operationContext) {
        this.operationContext = operationContext;
    }
    
    public boolean handleClickEvent(ClickEvent ce){
        Component src = ce.getComponent();
        try{
            if( src instanceof Button ){
                String sql = this.getSampleSql();
                operationContext.getPageController().showWindow(sql, "Sample "+getNodeName(),false);
            }
        }catch(Exception ex){
            Log.sendException(Log.MESSAGE_LEVEL_ERROR, "RMB", ex);
            PageController.showErrorDialog(ex, treeContainer);
        }
        return false;
    }
    
    private String getNodeName(){
        String str = "";
        TreeNode[] tnodes = this.dbFrameTree.getAllSelected();
        if(tnodes==null || tnodes.length==0){
            return "";
        }
        TreeNode tn = tnodes[tnodes.length-1];
        str = tn.getName();
        tn = tn.getParentNode();
        while(!tn.isRootNode()){
            str=tn.getName()+"."+str;
            tn = tn.getParentNode();
        }
        return str;
    }

    public boolean handleExpandCollapseEvent(ExpandCollapseEvent e){
        Component cmp = e.getComponent();
        if(!(cmp instanceof TreeNode) ){
            return false;
        }
        TreeNode tn = (TreeNode)cmp;
        if(tn.getNodeLevel()>2 || tn.getNodeCount()>0){
            // only the unexpanded schema, table need to expand 
            return false;
        }
        try{
            if(tn.getNodeLevel() == 0 ){
                builder.buildSchemas( tn );
            }else if(tn.getNodeLevel() == 1 ){
                builder.buildSchemaChild(tn.getName(), tn);
            }else if(tn.getNodeLevel() == 2 ){
                builder.buildTableChild(
                        tn.getParentNode().getName(), tn.getName(), tn);
            }
            tn.changed();
        }catch(Exception ex){
            Log.sendException(Log.MESSAGE_LEVEL_ERROR, "RMB", ex);
        }
        return false;
    }
}
