package com.ibm.alphablox.tooling.relational.view;

import com.alphablox.blox.BloxContext;
import com.alphablox.blox.DataBlox;
import com.alphablox.blox.GridBlox;
import com.alphablox.blox.ServerBloxException;
import com.alphablox.blox.data.ResultSet;
import com.alphablox.blox.data.rdb.RDBResultSet;
import com.alphablox.blox.repository.Log;
import com.alphablox.blox.uimodel.core.Button;
import com.alphablox.blox.uimodel.core.ComponentContainer;
import com.alphablox.blox.uimodel.core.Dialog;
import com.alphablox.blox.uimodel.core.HorizontalLayout;
import com.alphablox.blox.uimodel.core.ModelException;
import com.alphablox.blox.uimodel.core.Spacer;
import com.alphablox.blox.uimodel.core.Static;
import com.alphablox.blox.uimodel.core.VerticalLayout;
import com.alphablox.util.DataException;
import com.ibm.alphablox.tooling.relational.Messages;
import com.ibm.alphablox.tooling.relational.OperationContext;
import com.ibm.alphablox.tooling.relational.RMBConstants;
import com.ibm.alphablox.tooling.relational.util.DB2SQLFactory;
import com.ibm.alphablox.tooling.relational.controller.GridSelectorDlgController;
import com.ibm.alphablox.tooling.relational.controller.SQLEditorHandler;

public class GridDialogBuilder {
    private static final String ROWS_LABEL = "RowsLabel"; //$NON-NLS-1$

    //dialog content
    private DataBlox dataBlox, countResultDataBlox;

    private GridBlox gridBlox;

    private Dialog dialog;

    private ComponentContainer buttonContainer;

    //context
    private BloxContext bloxContext;

    //outerior parameters
    private GridSelectorDlgController dlgController;

    private SQLEditorHandler editorHandler;

    private static final String DataBloxID = "SampleDialogDataBloxID";  //$NON-NLS-1$

    private static final String GridBloxID = "SampleDialogGridBloxID";  //$NON-NLS-1$

    // for sample and validate, you neednot see the buttons
    private boolean buttonVisible = true;

    private OperationContext operationContext;

    public GridDialogBuilder(OperationContext operationContext, SQLEditorHandler editorHandler) {
        dataBlox = new DataBlox();
        countResultDataBlox = new DataBlox();
        gridBlox = new GridBlox();
        
        bloxContext = operationContext.getContext();
        this.editorHandler = editorHandler;

        this.operationContext = operationContext;
        
        dialog = new Dialog();
        buttonContainer = new ComponentContainer();
        try {
            init();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    private void init() throws Exception {
        dataBlox.init(bloxContext, DataBloxID);
        countResultDataBlox.init(bloxContext, DataBloxID+"2"); //$NON-NLS-1$
        gridBlox.init(bloxContext, GridBloxID);
        gridBlox.setDataBlox(dataBlox);

        buttonContainer.setLayout(new HorizontalLayout());
        Button submitButton = new Button(
                "Submit", Messages.getString("GridDialogBuilder.ButtonInsertSelectedValues")); //$NON-NLS-1$ //$NON-NLS-2$
        Button cancelButton = new Button(
                "Cancel", Messages.getString("GridDialogBuilder.ButtonCloseWindow"));  //$NON-NLS-1$ //$NON-NLS-2$
        buttonContainer.add(new Spacer(10,100));
        buttonContainer.add(submitButton);
        buttonContainer.add(new Spacer(10, 50));
        buttonContainer.add(cancelButton);
        buttonContainer.setVisible(this.isButtonVisible());
        
        dialog.setLayout(new VerticalLayout());
        dialog.setHeight(500);
        dialog.setWidth(400);
        gridBlox.setWidth("100%");  //$NON-NLS-1$
        gridBlox.setHeight("170");  //$NON-NLS-1$
        gridBlox.getToolbarBlox().setVisible(false);
        gridBlox.setMenubarVisible(false);
        ComponentContainer gridContainer = new ComponentContainer();
//        gridContainer.add(new Spacer(10,20));
        gridContainer.add(gridBlox.getGridBloxModel());
        
        Static rowsLabel = new Static(ROWS_LABEL,""); //$NON-NLS-1$
        dialog.add(rowsLabel);
        dialog.add(gridContainer);
        dialog.add(buttonContainer);
        
        dlgController = new GridSelectorDlgController(this, editorHandler);
        dialog.setController(dlgController);
    }

    /**
     * Change the dialog based on the new DataSource and new SQL.
     * @throws DataException
     * @throws ServerBloxException
     */
    public void build(String dsName,String sql, int sampleType, 
            String title, boolean addMessage) throws DataException,ServerBloxException {
        try{
            String oldDataSourceName = dataBlox.getDataSourceName();
            dataBlox.setDataSourceName(dsName);
            dataBlox.setQuery(sql);
            System.out.println("build "+this+", sql="+sql);
            if(oldDataSourceName != null && oldDataSourceName.equals(dsName)) {
                dataBlox.updateResultSet();
            }
            else {
                dataBlox.connect(true);
            }
            
            if(addMessage) {
                dialog.getStatic(ROWS_LABEL).setTitle(Messages.getString("GridDialogBuilder.ReturnRows") //$NON-NLS-1$
                        + " "
                        +((sampleType==RMBConstants.VALIDATE)?100:10)
                        + " " 
                        + Messages.getString("GridDialogBuilder.Rows")); //$NON-NLS-1$
                dialog.setTitle(title);
                dialog.changed();
            }
            else {
                dialog.getStatic(ROWS_LABEL).setTitle("");
                dialog.setTitle(title);
                dialog.changed();
            }
        }catch(ModelException ex){
            Log.sendException(Log.MESSAGE_LEVEL_ERROR, Messages.getString("GridDialogBuilder.RelationalMetadataBrowser"), ex); //$NON-NLS-1$
            ex.printStackTrace();
        }
    }

    /**
     * Generate a new SQL based on the original sql, the new sql 
     * count the records contained in the resultset.
     * @param sql the original SQL
     * @return how many records in the resultSet
     */
    public int getTotalLines(String dsName, String sql) {
        int x = 0;
        String newSql = operationContext.getSqlFactory().getResultSetSizeSql(sql);
        try{
            countResultDataBlox.setDataSourceName(dsName);
            countResultDataBlox.setQuery(newSql);
            countResultDataBlox.updateResultSet();
            
            ResultSet rs = countResultDataBlox.getResultSet();
            if(rs instanceof RDBResultSet){
                Object[] objs = ((RDBResultSet)rs).getNextRow(false);
                x = ((Integer)objs[0]).intValue();
            }
        }catch(Exception ex){
            Log.sendException(Log.MESSAGE_LEVEL_ERROR, RMBConstants.ErrorLogTitle, ex);
        }
        return x;
    }

    /**
     * @return the dialog
     */
    public Dialog getDialog() {
        return dialog;
    }

    /**
     * @param dialog
     *            the dialog to set
     */
    public void setDialog(Dialog dialog) {
        this.dialog = dialog;
    }

    /**
     * @return the gridBlox
     */
    public GridBlox getGridBlox() {
        return gridBlox;
    }

    /**
     * @param gridBlox
     *            the gridBlox to set
     */
    public void setGridBlox(GridBlox gridBlox) {
        this.gridBlox = gridBlox;
    }

    /**
     * @return the buttonVisible
     */
    public boolean isButtonVisible() {
        return buttonVisible;
    }

    /**
     * @param buttonVisible
     *            the buttonVisible to set
     */
    public void setButtonVisible(boolean buttonVisible) {
        this.buttonVisible = buttonVisible;
    }

    public OperationContext getOperationContext() {
        return this.operationContext;
    }
}
