/*
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: dmessink
 * Date: Aug 21, 2002 4:38:28 PM
 */
package fastforward;

import org.w3c.dom.Document;
import java.io.File;

import com.alphablox.util.XmlFile;

public class AppConfiguration extends AbstractConfiguration {
    private File configFile;

    public File getConfigFile() {
        return configFile;
    }

    public void setConfigFile(File configFile) {
        this.configFile = configFile;
    }

    public void readConfig() throws XmlParseException {
        try {
            _readConfig();
        } catch (Exception exc) {
            throw new XmlParseException(appContext.getString("ERROR_LOADING_APP_CONFIG",new Object[]{appContext.getConfigFile()}),exc);
        }
    }

    public void saveConfig() throws Exception {
        if (configFile == null)
            throw new IllegalStateException(appContext.getString("ERROR_NO_SAVE_WITHOUT_READ") );

            XMLConfiguration xmlConfig=new XMLConfiguration();
            xmlConfig.init(this,appContext.getResourceBundle());
            Document document = xmlConfig.generateXmlDocument();

            XmlFile xmlFile = new XmlFile();
            xmlFile.setXmlFile(configFile);
            xmlFile.setDocument(document);
            xmlFile.setDTDFile("file:///config.dtd");
            xmlFile.save();

    }

    private void _readConfig() throws XmlParseException {
        File file = configFile;

        if (file == null)
            file = appContext.getConfigFile();
        XmlFile xmlFile = new XmlFile();
        xmlFile.setXmlFile(file);
        xmlFile.setDTDFile(new File(appContext.getRealPath(AppContextImpl.CONFIG_DTD_FILE)));

        XMLErrorHandler errorHandler = new XMLErrorHandler(xmlFile);
        xmlFile.setErrorHandler(errorHandler);

        Document xmlDocument = null;
        try {
            xmlDocument = xmlFile.getDocument();
        } catch (Exception e) {
            throw new XmlParseException(e);
        }

        if (errorHandler.exceptionThrown())
            throw errorHandler.getException();

        readConfig(xmlDocument);
        this.configFile=file; // do we really have to do this???
    }
}
