package fastforward;

import com.alphablox.blox.ContainerBlox;
import com.alphablox.blox.ServerBloxException;
import com.alphablox.blox.uimodel.BloxModel;
import com.alphablox.blox.uimodel.core.*;
import com.alphablox.blox.uimodel.core.event.ClickEvent;
import com.alphablox.blox.uimodel.core.event.IEventHandler;
import com.alphablox.blox.uimodel.core.event.InitialRenderEvent;
import com.alphablox.util.Message;

import java.util.ArrayList;

/**
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: ceubanks
 * Date: Feb 27, 2003 7:47:16 AM
 */
public abstract class FastForwardTabs extends ContainerBlox {
    private ArrayList tabs;
    private int idxFront;
    private String prefix;
    private boolean modelDirty;

    private static String FRONT = "front";
    private static String BACK = "back";
    private static String RIGHT = "right";
    private static String LEFT = "left";
    private static String FTOB = "FtoB";
    private static String BTOF = "BtoF";
    private static String BTOB = "BtoB";
    private static String FRONT_STYLE = "clsFFWDTabFront";
    private static String BACK_STYLE = "clsFFWDTabBack";
    private static String IMAGE_STYLE = "clsFFWDTabImage";
    private static String CNTR_STYLE = "clsFFWDTabs";
    private ArrayList values;
    private FastForwardResourceBundle resource;

    public FastForwardTabs(FastForwardResourceBundle resource) {
        tabs = new ArrayList();
        values = new ArrayList();
        this.resource = resource;
    }

    protected void initializeBloxModel() throws Exception {
        BloxModel bloxModel = getBloxModel();
        bloxModel.setController(new TabController());
        bloxModel.addEventHandler(new IEventHandler() {
            public boolean handleInitialRenderEvent(InitialRenderEvent event) {
                updateModel();
                return false;
            }
        });

        super.initializeBloxModel();
    }


    public void changed() throws ModelException {
        updateModel();
    }

    public void addTab(String text, String value) {
        tabs.add(text);
        values.add(value);
        modelDirty = true;
    }

    public void addTab(int idx, String text, String value) {
        tabs.add(idx, text);
        values.add(idx, value);
        modelDirty = true;
    }


    public void setSelected(int idx) {
        this.idxFront = idx;
        modelDirty = true;
    }

    public void setPrefix(String prefix) {
        this.prefix = prefix;
        modelDirty = true;
    }

    public void updateModel() {
        if (modelDirty == false)
            return;
        BloxModel model = null;
        try {
            model = getBloxModel();
        } catch (ServerBloxException e) {
            new Message(Message.ERROR, "FastForwardTabs", e).send();
            throw new IllegalStateException(resource.getString("ERROR_NO_TABS_MODEL"));
        }
        model.clear();
        model.setLayout(new HorizontalLayout());
        idxFront = (idxFront < 0) || (idxFront > tabs.size() - 1) ? 0 : idxFront;
        try {
            for (int i = 0; i < tabs.size(); i++) {
                String tab = (String) tabs.get(i);

                renderTab(model, tab, i, idxFront == i, i == 0, i == (tabs.size() - 1), (i + 1) == idxFront, i < idxFront);
            }
            model.changed();
        } catch (ModelException e) {
            e.printStackTrace();  //To change body of catch statement use Options | File Templates.
        }
        modelDirty = false;
    }

    /**
     * return true if the tab switch should take place
     * @param tab
     * @return
     */
    abstract public boolean handleTab(String tab);


    private void renderTab(BloxModel model, String tabLabel, int id, boolean inFront, boolean onLeft, boolean onRight,
                           boolean nextInFront, boolean frontDone) throws ModelException {

        // left-most tab is the only one that renders a left edge of the tab
        if (onLeft) {
            String leftPath = prefix + "_" + LEFT + "_" + (inFront ? FRONT : BACK) + ".jpg";
            Image lImg = new Image(leftPath);
            lImg.setClickable(false);
            lImg.setThemeClass(IMAGE_STYLE);
            model.add(lImg);
        }
        Static label = new Static(tabLabel);
        String backStyle = BACK_STYLE + "_" + (frontDone == true ? LEFT : RIGHT);
        label.setThemeClass(inFront ? FRONT_STYLE : backStyle);
        label.setHeight(32);
        label.setWidth(70);
        ComponentContainer ctnr = new ComponentContainer();
        ctnr.add(label);
        ctnr.setClickable(!inFront);
        ctnr.setName(new Integer(id).toString());
        ctnr.setThemeClass(CNTR_STYLE);
        model.add(ctnr);
        String rightPath;
        String sBtoBPart = BTOB + "_" + (frontDone == true ? LEFT : RIGHT);
        if (onRight)
            rightPath = prefix + "_" + RIGHT + "_" + (inFront ? FRONT : BACK) + ".jpg";
        else if (inFront)
            rightPath = prefix + "_" + FTOB + ".jpg";
        else
            rightPath = prefix + "_" + (nextInFront ? BTOF : sBtoBPart) + ".jpg";
        Image rImg = new Image(rightPath);
        rImg.setClickable(false);
        rImg.setThemeClass(IMAGE_STYLE);
        model.add(rImg);
    }

    class TabController extends Controller {
        TabController() {
            addEventHandler(new IEventHandler() {
                public boolean handleClickEvent(ClickEvent event) throws ModelException {
                    ComponentContainer label = (ComponentContainer) event.getComponent();
                    String idx = label.getName();
                    int idxVal = Integer.parseInt(idx);
                    if (handleTab((String) values.get(idxVal)) == true) {
                        setSelected(idxVal);
                        changed();
                    }
                    return false;
                }
            });
        }
    }

}
