/*
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: dmessink
 * Date: Nov 14, 2002 1:24:18 PM
 */
package fastforward;

/**
 * Represents a parameter as defined within a Template. The name, label, and default values are defined in the
 * template.xml file.
 */

public interface Parameter {
    /**
     * Returns the name of the parameter as defined in the template.xml file.
     * @return a String representing the name of the template parameter.
     */
    public String getName();

    /**
     * Returns the label of the parameter as defined in the template.xml file. The label is used in the report
     * admin page to display the parameter edit control.
     * @return a String representing the label of the template parameter.
     */
    public String getLabel();

    /**
     * Returns the default value of the parameter as defined in the template.xml file. If multiple default values are
     * defined this method will return the first one.
     * @return a String representing the default value of the template parameter.
     */
    public String getDefaultValue();

    /**
     * Returns the default values of the parameter as defined in the template.xml file.
     * @return an array of Strings representing the default values of the template parameter.
     */
    public String[] getDefaultValues();
}
