/*
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: dmessink
 * Date: Aug 27, 2002 3:49:29 PM
 */
package fastforward;

/**
 * The common interface implemented by all FastForward objects that support named parameters.
 */

public interface ParameterSupport {
    /**
     * Returns the parameter value of the named parameter or null. Returns null if the named parameter is not defined.
     * @param parameterName the name of the parameter
     * @return the named parameter value or null if the parameter is not defined
     */
    String getParameter(String parameterName);

    /**
     * Returns the parameter value of the named parameter or the defaultValue argument. Returns the given defaultValue
     * if the named parameter is not defined.
     * @param parameterName the name of the parameter
     * @param defaultValue the parameter value to return if no parameter value is defined
     * @return the parameter value if defined or the given default value if the parameter is not defined
     */
    String getParameter(String parameterName, String defaultValue);

    /**
     * Returns the parameter values defined for the named parameter. Returns an empty array if no parameter values are
     * defined.
     * @param parameterName The name of the parameter
     * @return an array of Strings representing the named parameter values
     */
    String[] getParameterValues(String parameterName);

    /**
     * Returns an array containing all of the named parameters defined for this object.
     * @return an array of Strings representing all defined named parameters
     */
    String[] getParameterNames();
}
