/**
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: CEUBANKS
 * Date: Nov 6, 2002
 * Time: 10:28:51 PM
 */
package fastforward;

import fastforward.ParameterSupport;

/**
 * Represents a FastForward report defined by a report template and template parameter values. Public reports are
 * defined by the application administrator and available to all users. Users may in turn define private and group
 * reports based on public reports.
 */

public interface Report extends ParameterSupport {
    public static final String PUBLIC = "public";       // DO NOT LOCALIZE!!!!
    public static final String PRIVATE = "private";     // DO NOT LOCALIZE!!!!
    public static final String GROUP = "group";         // DO NOT LOCALIZE!!!!

    /**
     * Returns the unique id of this report relative to the report visibility public, private or group.
     * @return a String representing the report's unique identifier
     */
    public String getId();

    /**
     * Returns the human readable label of the report.
     * @return a String representing the report's label.
     */
    public String getLabel();

    /**
     * Returns the report's description text. This text is also used as a tooltip in the report navigation trees.
     * @return a String representing the report's description
     */
    public String getDescription();

    /**
     * Returns the Template object that this report is based on. The report Template defines the report's presentation
     * and edit pages.
     * @return the Template object that this report is based on
     */
    public Template getTemplate();
}
