package fastforward;

import com.alphablox.blox.uimodel.core.*;
import com.alphablox.personalization.IUserManager;
import com.alphablox.personalization.alphablox.AlphabloxUserManager;
import com.alphablox.server.Constants;
import com.alphablox.server.user.UserManager;

/**
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: ceubanks
 * Date: Dec 11, 2002 5:24:09 PM
 */
public class SaveAsDialog extends Dialog {
    private AppContextImpl appContext;

    public SaveAsDialog() {
        super();
    }

    public void init(final AppContextImpl appContext) {
        this.appContext = appContext;

        try {
            IUserManager mgr =  UserManager.getExternalUserManager();
            boolean isStandAlone = mgr instanceof AlphabloxUserManager;
            loadFromResource(appContext.getResourcePath("saveas.xml"), appContext.getLocales() );
            setTitle(title);
            final Button okBtn = (Button) searchForComponent("ok");
            okBtn.setSelected(true);
            final Edit editName = (Edit) searchForComponent("editName");
            String newName =  appContext.getString("COPY_OF",new Object[]{appContext.getCurrentReport().getLabel()});
            editName.setValue(newName);
            final Edit editDescription = (Edit) searchForComponent("editDescription");
            editDescription.setValue(appContext.getCurrentReport().getDescription());
            final DropDownList visibility = (DropDownList) searchForComponent("visibility");
            String stPrivate = appContext.getString("PRIVATE_ACCESS");
            visibility.add(stPrivate,"private");
            visibility.setSelected(0);
            User user = appContext.getUser();
            String[] groups = user.getGroups();
            for (int i = 0; i < groups.length; i++) {
                // if standalone server then fix up the "magic" public group name
                String stGroup = isStandAlone && groups[i].equalsIgnoreCase(Constants.PUBLIC_GROUPNAME)
                        ? appContext.getString("PUBLIC_ACCESS") : groups[i];
                visibility.add(stGroup,groups[i]);
            }
            setController(new DialogController(this) {
                public void actionOk() {
                    String name=editName.getValue();
                    String visibilityStr = (String)visibility.getUserObject(visibility.getSelected());

                    try {
                        saveItem(name, editDescription.getValue(), visibilityStr);
                        dialog.getDispatcher().sendClientCommand(appContext.getCallbackFunction("pollUI()"));
                    } catch (Exception exc) {
                        appContext.log(LogFile.ERROR, appContext.getString("ERROR_SAVE_AS_DETAIL", new Object[]{visibilityStr,name}),exc);
                        MessageBox.message(dialog.getDispatcher(),"Save As Error",
                                appContext.getString("ERROR_SAVE_AS", new Object[]{name}));
                    } finally {
                        super.actionOk();
                    }
                }
            });
        } catch (Exception e) {
            appContext.log(LogFile.ERROR, "Unexpected model exception", e);
        }
    }

    void saveItem(String name, String desc, String visibilityStr) throws Exception {
            if (visibilityStr.equals("private"))
                ((SavedStateImpl)appContext.getSavedState()).saveReportAs(name, desc, Report.PRIVATE, null);
            else
                ((SavedStateImpl)appContext.getSavedState()).saveReportAs(name, desc, Report.GROUP, visibilityStr);
    }
}