/*
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: rraghavan
 * Date: Jan 24, 2003 12:54:14 PM
 */
package fastforward;

import com.alphablox.blox.ServerBloxException;
import com.alphablox.blox.Blox;
import com.alphablox.blox.repository.Bookmark;

/**
 * Represents the report information saved as private and group reports. The SavedState data may be used by
 * Templates that need to be restored using non-standard report restore procedures. The standard restore procedure
 * restores all Blox bookmarks and all FormBlox form values.
 */
public interface SavedState {
    /**
     * Returns the Blox object representing the named Blox as defined in the report.
     * @param bloxName the name of the Blox to access
     * @return the Blox object defined in the report page
     * @throws ServerBloxException
     */
    public Blox getBlox(String bloxName) throws ServerBloxException;

    /**
     * An array of all Blox defined by the report.
     * @return an array of Blox objects defined in the report page
     */
    public Blox[] getBlox();

    /**
     * Returns the Bookmark object that was saved when the report was saved. Modifications made to the Bookmark
     * object will be applied to the associated Blox when it is restored.
     * @param bloxName the name of the Blox Bookmark object to access
     * @return the Bookmark object saved for the given Blox name
     * @throws ServerBloxException
     */
    public Bookmark getBloxBookmark(String bloxName) throws ServerBloxException;

    /**
     * Iterates through the Blox object defined by the report restoring their Bookmark or form values.
     * @throws ServerBloxException
     */
    public void restore() throws ServerBloxException;

    /**
     * Restores the named Blox by either restoring the Blox Bookmark or the form values.
     * @param blox the name of the Blox object to restore
     * @throws ServerBloxException
     */
    public void restoreBlox(Blox blox) throws ServerBloxException;

    /**
     * Sets the auto-restore functionality on or off. If autoRestore is true then the FastForward framework will
     * perform the standand report restore behavior. The default value is true.
     * @param autoRestore true or false
     */
    public void setAutoRestore(boolean autoRestore);

    /**
     * Returns true if the FastForward restore behavior is currently enabled. The default value is true.
     * @return true if the FastForward restore behavior is enabled
     */
    public boolean isAutoRestore();
}
