######################################################################
#
#  Licensed Materials - Property of IBM
#  5724-M24
#  Copyright IBM Corporation 2005, 2009. All Rights Reserved.
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
#  
######################################################################

import java

#
# imports for the logger
#
from com.ibm.wbmonitor.install.common.log import LogFactory
from com.ibm.wbmonitor.install.common.log import ILogger
from com.ibm.wbmonitor.install.common.log import ITracer
from com.ibm.wbmonitor.install.common.log import LogConstants

#
# set up the logger
#
trc = LogFactory.createTracer(LogConstants.WBM_INSTALL_GROUP_NAME, LogConstants.PORTDASH)
log = LogFactory.createLogger(LogConstants.WBM_INSTALL_GROUP_NAME, LogConstants.PORTDASH)


#-------------------------------------------------------------------------------
# This jython script installs the referenced EAR
#-------------------------------------------------------------------------------

def wsadminToList(inStr):
        outList=[]
        if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
                tmpList = inStr[1:-1].split(" ")
        else:
                tmpList = inStr.split("\n")  #splits for Windows or Linux
        for item in tmpList:
                item = item.rstrip();        #removes any Windows "\r"
                if (len(item)>0):
                        outList.append(item)
        return outList
#endDef

#######################################
# installEar subroutine
# location - location where the EAR resides
# earName - name of the ear in the location directory
# appName - name of the Application
# nodeName - name of the WAS node
#######################################
def installEarOnDefaultServer ( location, earName, appName, nodeName ):

	className = "installEar_py"
	methodName = "installEarOnServer"
	
	trc.entry(className, methodName)
        
	# TODO - locate default server (or assume "server1") & call installEarOnServer 
	trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "WARN: Method not supported at this time")
	
	trc.exit(className, methodName)

#endDef


#######################################
# installEar subroutine
# location - location where the EAR resides
# earName - name of the ear in the location directory
# appName - name of the Application
# nodeName - name of the WAS node
#######################################
def installEarOnServer ( location, earName, appName, nodeName, serverName ):
        
	className = "installEar_py"
	methodName = "installEarOnServer"
        retVal = 0        
	
	trc.entry(className, methodName)
	trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "INFO: Parameters passed ==> location="+location+", earName="+earName+", appName="+appName+", nodeName="+nodeName+", serverName="+serverName)

        #
	# BOGUS - temporary just to try!!
	#
	#trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Error: Throwing AssertionError exception...")
	#raise AssertionError

	#--------------------------------------------------------------
        # set up globals
        #--------------------------------------------------------------
        global AdminConfig
 
        #--------------------------------------------------------------
        # do some sanity checking 
        #     -- do we have a node by this name? 
        #--------------------------------------------------------------
        #cellname = AdminControl.getCell( )
        cellName = AdminConfig.showAttribute(AdminConfig.list("Cell"), "name")
        node = AdminConfig.getid("/Node:"+nodeName+"/" )
	
	trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO:  checking for existence of node "+nodeName)
        if (len(node) == 0):
		trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Warning: node not found for name "+nodeName)
		log.msg(ILogger.TYPE_WARN, className, methodName, "CWMLP1033W", nodeName)
                retVal = 1
        #endIf 

        #---------------------------------------------------------
        # First, list the existing applications 
        #---------------------------------------------------------
        apps = AdminApp.list( )
	trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO:  Installed applications: ", apps)

        # Check if App is already installed on the target server
        # Comparing the application name.
	trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Determine if " + appName + " is already installed.")
        appExist = "False"
        for tmpAppName in wsadminToList(apps):
       	        if (cmp(tmpAppName, appName) == 0):
               	        appExist = "True"
                       	break 
                #endIf 
	#endFor 
  

        #---------------------------------------------------------
        # Here is the ear file we'll be dealing with... 
        #---------------------------------------------------------
        defapp = location+"/"+earName+".ear"

        #---------------------------------------------------------
        # Application Install sequence
        #---------------------------------------------------------
        if (cmp(appExist, "True") == 0):
		#
		# Uninstall the existing application 
		#
		trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "INFO: "+earName+".ear (ApplicationName: "+appName+") is already installed on server.")
		trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "INFO: Uninstallation of "+appName+" to begin. ")

                AdminApp.uninstall(appName)

                #check for success
                appExist = "False"
                for tmpAppName in wsadminToList(AdminApp.list()):
                        if (cmp(tmpAppName, appName) == 0):
                                appExist = "True"
                                break 
                        #endIf 
                #endFor 
                if (cmp(appExist, "False") == 0):
			trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "INFO:  Existing Application "+appName+" successfully removed. Saving configuration...")
                        AdminConfig.save( )
                else:
			trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "ERROR: Existing Application "+appName+" could not be removed.")
			log.msg(ILogger.TYPE_ERROR, className, methodName, "CWMLP2132E", appName, nodeName)			
			retVal = 1
                #endElse 
        #endIf 

	#
	# Install the application
	#
	trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO:  Installing app: "+appName+" on node: "+nodeName+" & server: "+serverName+"...")
        
	try:
		# AdminApp.install(defapp, ["-appname", appName, "-node", nodeName] )
		AdminApp.install(defapp, ["-appname", appName, "-node", nodeName, "-server", serverName] )
		
		# save the configuration		
		trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO:  Saving configuration changes.")
		AdminConfig.save( )
	
	except:
		# Exception during application install
		# note: must send a list of parameters to ILogger.msg() when more than 2 are required on a message
		log.msg(ILogger.TYPE_ERROR, className, methodName, "CWMLP2133E", [appName, nodeName, serverName])			
		raise
	
	trc.exit(className, methodName)
        
	#return
        return retVal
#endDef

#######################################
# installEar subroutine
# location - location where the EAR resides
# earName - name of the ear in the location directory
# appName - name of the Application
# cluster - name of the WAS cluster
#######################################
def installEarOnCluster ( location, earName, appName, clusterName ):
        
	className = "installEar_py"
	methodName = "installEarOnCluster"
        retVal = 0        
	
	trc.entry(className, methodName)
	trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "INFO: Parameters passed ==> location="+location+", earName="+earName+", appName="+appName+", clusterName="+clusterName)

	#--------------------------------------------------------------
        # set up globals
        #--------------------------------------------------------------
        global AdminConfig
 
        #--------------------------------------------------------------
        # do some sanity checking 
        #     -- do we have a node by this name? 
        #--------------------------------------------------------------
        #cellname = AdminControl.getCell( )
        cellName = AdminConfig.showAttribute(AdminConfig.list("Cell"), "name")
        cluster = AdminConfig.getid("/ServerCluster:"+clusterName+"/" )
	
	trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO:  checking for existence of clusterName "+clusterName)
        if (len(node) == 0):
		trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Warning: cluster not found for clusterName "+clusterName)
		log.msg(ILogger.TYPE_WARN, className, methodName, "CWMLP1033W", clusterName)
                retVal = 1
        #endIf 

        #---------------------------------------------------------
        # First, list the existing applications 
        #---------------------------------------------------------
        apps = AdminApp.list( )
	trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO:  Installed applications: ", apps)

        # Check if App is already installed on the target server
        # Comparing the application name.
	trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Determine if " + appName + " is already installed.")
        appExist = "False"
        for tmpAppName in wsadminToList(apps):
       	        if (cmp(tmpAppName, appName) == 0):
               	        appExist = "True"
                       	break 
                #endIf 
	#endFor 
  

        #---------------------------------------------------------
        # Here is the ear file we'll be dealing with... 
        #---------------------------------------------------------
        defapp = location+"/"+earName+".ear"

        #---------------------------------------------------------
        # Application Install sequence
        #---------------------------------------------------------
        if (cmp(appExist, "True") == 0):
		#
		# Uninstall the existing application 
		#
		trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "INFO: "+earName+".ear (ApplicationName: "+appName+") is already installed on server.")
		trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "INFO: Uninstallation of "+appName+" to begin. ")

                AdminApp.uninstall(appName)

                #check for success
                appExist = "False"
                for tmpAppName in wsadminToList(AdminApp.list()):
                        if (cmp(tmpAppName, appName) == 0):
                                appExist = "True"
                                break 
                        #endIf 
                #endFor 
                if (cmp(appExist, "False") == 0):
			trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "INFO:  Existing Application "+appName+" successfully removed. Saving configuration...")
                        AdminConfig.save( )
                else:
			trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "ERROR: Existing Application "+appName+" could not be removed.")
			log.msg(ILogger.TYPE_ERROR, className, methodName, "CWMLP2132E", appName, clusterName)			
			retVal = 1
                #endElse 
        #endIf 

	#
	# Install the application
	#
	trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO:  Installing app: "+appName+" on cluster: "+clusterName)
        
	try:
		AdminApp.install(defapp, ["-appname", appName, "-cluster", clusterName] )
		
		# save the configuration		
		trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO:  Saving configuration changes.")
		AdminConfig.save( )
	
	except:
		# Exception during application install
		# note: must send a list of parameters to ILogger.msg() when more than 2 are required on a message
		log.msg(ILogger.TYPE_ERROR, className, methodName, "CWMLP2133E", [appName, clusterName])			
		raise
	
	trc.exit(className, methodName)
        
	#return
        return retVal
#endDef 


#-----------------------------------------------------------------
# Main
#-----------------------------------------------------------------
#if ((len(sys.argv) != 5)):
#        print "installEar: this script requires 5 parameters: scriptname, location of ear, ear, application, node"
#        print "e.g.:     installEar.py /opt/WebSphere/VoiceServer/ear wvsapp WVS mynode"
#else:
#        installEar(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4] )
#endif 

