######################################################################
# Licensed Materials - Property of IBM
#  5724-M24
#  Copyright IBM Corporation 2009. All Rights Reserved.
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
######################################################################

import java

from java.io import ByteArrayOutputStream


#
# imports for the logger
#
from com.ibm.wbmonitor.install.common.log import LogFactory
from com.ibm.wbmonitor.install.common.log import ILogger
from com.ibm.wbmonitor.install.common.log import ITracer
from com.ibm.wbmonitor.install.common.log import LogConstants

#
# set up the logger
#
trc = LogFactory.createTracer(LogConstants.WBM_INSTALL_GROUP_NAME, LogConstants.PORTDASH)
log = LogFactory.createLogger(LogConstants.WBM_INSTALL_GROUP_NAME, LogConstants.PORTDASH)
className = "installABXPortalDashboard_py"


#############################################################
# main
#
# Configure the Alphablox Library / Plugin on
# the WAS server.  
#
#############################################################

methodName = "main"

trc.entry(className, methodName)

if ((len(sys.argv) != 1)):
        # script requires 1 parameter: profilePath
	trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: This script requires 1 parameter: profilePath")
	trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: e.g.: <scriptname>.py /opt/WebSphere/Monitor/profiles/MonSrv01")
	raise ValueError	

# get the profile path
profilePath = sys.argv[0]
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "profilePath="+profilePath)

# 
# Load the installation parameters 
#
execfile(profilePath+'/config.wbm/MonitorProfileVals.py')
execfile(moncommonPath+'/MonitorServerUtil.py')

#
# get parameters we need from MonitorProfileVals.py
#
trc.trace(ITracer.TYPE_LEVEL1, className, methodName, "Preset Vales required for Alphablox datasource creation:")
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  datasourceJndi                = "+ datasourceJndi)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  datasourceAuthAliasUsername   =" + datasourceAuthAliasUsername)
trc.traceMask(ITracer.TYPE_LEVEL3, className, methodName, "  datasourceAuthAliasPassword   =:hide:" + datasourceAuthAliasPassword + ":hide:")
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  databaseName                  =" + databaseName)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  datasourceDriverType          =" + datasourceDriverType)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  datasourceServerName          =" + datasourceServerName)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  datasourcePortNumber          =" + datasourcePortNumber)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  dbType                        =" + dbType)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  clusterName                   =" + clusterName)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  cellName                      =" + cellName)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  nodeName                      =" + nodeName)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  serverName                    =" + serverName)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  jdbcProviderName              =" + jdbcProviderName)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  jdbcProviderDesc              =" + jdbcProviderDesc)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  jdbcProviderImplClass         =" + jdbcProviderImplClass)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  jdbcProviderClassPath         =" + jdbcProviderClassPath)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  datasourceAuthAliasName       =" + datasourceAuthAliasName)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  datasourceAuthAliasDesc       =" + datasourceAuthAliasDesc)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  datasourceName                =" + datasourceName)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  datasourceDesc                =" + datasourceDesc)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  datasourceHelperClassname     =" + datasourceHelperClassname)
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "  datasourceMaxConnections      =" + datasourceMaxConnections)

installStep = "Create Alphablox datasource for Portal Dashboard"
success = "true"

log.msg(ILogger.TYPE_INFO, className, methodName, "CWMLP0010I", installStep)


try:
    
    ###############################
    # Create the Monitor datasource
    ###############################
    UtilWsadminHolder.AdminConfig = AdminConfig
    UtilWsadminHolder.AdminTask = AdminTask

    # set the data source properties
    datasourceProps = {"databaseName":databaseName, "driverType":datasourceDriverType}
    datasourceProps["serverName"] = datasourceServerName
    datasourceProps["portNumber"] = datasourcePortNumber
    #Add custom property for webSphereDefaultIsolationLevel:2 (Cursor Stability (CS))
    datasourceProps["webSphereDefaultIsolationLevel"] = "2"
    
    if (dbType == S_COMMON_DB_ORACLE_11g):
        #type 2 = OCI, type 4 = thin.  
        if(datasourceDriverType == "2"):
            datasourceProps["URL"] = "jdbc:oracle:oci:@"+datasourceServerName+":"+datasourcePortNumber+":"+databaseName
        else:
            datasourceProps["URL"] = "jdbc:oracle:thin:@"+datasourceServerName+":"+datasourcePortNumber+":"+databaseName

    scope = Scope(clusterName, cellName, nodeName, serverName)

    jdbcProviderInfo = JdbcProviderInfo(jdbcProviderName, jdbcProviderDesc, jdbcProviderImplClass, jdbcProviderClassPath)

    datasourceAuthAliasInfo = AuthAliasInfo(datasourceAuthAliasName, datasourceAuthAliasDesc, datasourceAuthAliasUsername, datasourceAuthAliasPassword)
    datasourceInfo =  DatasourceInfo(datasourceName, datasourceDesc, datasourceJndi, datasourceHelperClassname, datasourceMaxConnections, datasourceProps, jdbcProviderName, datasourceAuthAliasName)
    
    # Create WAS Environment variable WBM_JDBC_DRIVER_PATH
    trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "Create WAS environment variable WBM_JDBC_DRIVER_PATH using JDBC_DRIVER_PATH: " + JDBC_DRIVER_PATH)
    execfile(moncommonPath+'/setWASVar.py')
    environment(cellName,None,serverName,"WBM_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
    environment(cellName,nodeName,serverName,"WBM_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
    #trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "WBM_JDBC_DRIVER_PATH = " + WBM_JDBC_DRIVER_PATH)
    trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "set  WBM_JDBC_DRIVER_PATH")
    
    # Set WAS Environment variable for DB2UNIVERSAL_JDBC_DRIVER_NATIVEPATH
    # this must get set for WAS 6.0.2 even when the value for DB2_JDBC_DRIVER_NATIVEPATH is ""
    # if(DB2_JDBC_DRIVER_NATIVEPATH != ""):
    trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "Set WAS environment variable DB2UNIVERSAL_JDBC_DRIVER_NATIVEPATH using DB2_JDBC_DRIVER_NATIVEPATH: " + DB2_JDBC_DRIVER_NATIVEPATH)
    environment(cellName,None,serverName,"DB2UNIVERSAL_JDBC_DRIVER_NATIVEPATH",DB2_JDBC_DRIVER_NATIVEPATH,"")
    environment(cellName,nodeName,serverName,"DB2UNIVERSAL_JDBC_DRIVER_NATIVEPATH",DB2_JDBC_DRIVER_NATIVEPATH,"")
    trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "set DB2UNIVERSAL_JDBC_DRIVER_NATIVEPATH")

    # Creates the JDBC Provider.
    trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "Create the JDBC Provider")
    providerConfigId = createJdbcProvider(scope, Scope.CELL_SCOPE, jdbcProviderInfo)
    trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "Return from Create JDBC Provider with id: " + providerConfigId)

    # Check for the Datasource
    # if it does exist use it,  else create it 
    trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "Check if the Datasource exists.")
    if (getDatasourceConfigId(scope, Scope.CELL_SCOPE, datasourceInfo.providerName, datasourceInfo.name) == None): 
        # need to create it
        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "Create authentication alias:" + datasourceAuthAliasInfo.name)
	createAuthenticationAlias(datasourceAuthAliasInfo)
        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "Create datasource:" + datasourceInfo.name)
	createDatasource(scope, Scope.CELL_SCOPE, datasourceInfo)
        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "Create datasource:" + datasourceInfo.name + " completed")

    AdminConfig.save()
    
except:

    # Exception occured... failed to create datasource
    log.msg(ILogger.TYPE_ERROR, className, methodName, "CWMLP2131E", [datasourceJndi, datasourceAuthAliasUsername, profilePath])    
    success = "false"

# end except    
    

if (success == "true"):
    # successful completion
    log.msg(ILogger.TYPE_INFO, className, methodName, "CWMLP0011I", installStep)
else:
    # successful completion
    log.msg(ILogger.TYPE_ERROR, className, methodName, "CWMLP2130E", installStep, profilePath+"/logs/installABXPortalDashboard.log")
    


trc.exit(className, methodName)

