######################################################################
#
#  Licensed Materials - Property of IBM
#  5724-M24
#  Copyright IBM Corporation 2009. All Rights Reserved.
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
#  
######################################################################

import java

#
# imports for the logger
#
from com.ibm.wbmonitor.install.common.log import LogFactory
from com.ibm.wbmonitor.install.common.log import ILogger
from com.ibm.wbmonitor.install.common.log import ITracer
from com.ibm.wbmonitor.install.common.log import LogConstants

#
# set up the logger
#
trc = LogFactory.createTracer(LogConstants.WBM_INSTALL_GROUP_NAME, LogConstants.PORTDASH)
log = LogFactory.createLogger(LogConstants.WBM_INSTALL_GROUP_NAME, LogConstants.PORTDASH) 
className = "installPortalDashboardRestProxy_py"

#############################################################
# configureProxy subroutine
#
# Configure the REST Service Proxy to communicate with  
# REST Services.
#
#############################################################
def configureProxy (clusterName, nodeName, proxyServer):

        methodName = "configureProxy"
       
        trc.entry(className, methodName)
        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Parameters passed ==> clusterName="+clusterName+", nodeName="+nodeName+", proxyServer="+proxyServer)
        
        # Set custom JVM porperty to enable proxy to find the rest services host
        #
        # Application servers > WebSphere_Portal > Process Definition > Custom Properties > 
        #
        
        UtilWsadminHolder.AdminConfig = AdminConfig
        UtilWsadminHolder.AdminTask = AdminTask
        
        cellName = AdminConfig.showAttribute(AdminConfig.list("Cell"), "name")
              
        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Proxy settings ==> DashboardRESTServiceTarget = "+restServicesHost +":"+ restServicesPort)
        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Proxy settings ==> DashboardRESTServiceSecureTarget ="+restServicesHost +":"+ restServicesSecurePort)
        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Proxy settings ==> BusinessSpaceTarget = "+bspaceServicesHost +":"+ bspaceServicesPort)
        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Proxy settings ==> BusinessSpaceSecureTarget = "+bspaceServicesHost +":"+ bspaceServicesSecurePort)
        
        systemProperties = {}
        systemProperties["DashboardRESTServiceTarget"] = restServicesHost +":"+ restServicesPort
        systemProperties["DashboardRESTServiceSecureTarget"] = restServicesHost +":"+ restServicesSecurePort
        systemProperties["BusinessSpaceTarget"] = bspaceServicesHost +":"+ bspaceServicesPort
        systemProperties["BusinessSpaceSecureTarget"] = bspaceServicesHost +":"+ bspaceServicesSecurePort
        
        if (ndType == "managed"):
                trc.exit(className, methodName)
                return
        elif(ndType == "standalone"):
                # Create our system properties for standalone server		
                trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Create System Properties for standalone: ", systemProperties)
                scope = Scope("", cellName, nodeName, serverName)
                setSystemProperties(scope, systemProperties)
                
        # use the cluster 
        elif (ndType == "dmgr"):
                # Create our system properties for node/server              
                cId = AdminConfig.getid("/ServerCluster:"+clusterName+"/" )
                cList = AdminConfig.list("ClusterMember", cId ).split(lineSeparator)
                for sId in cList:
                        cMemName = AdminConfig.showAttribute(sId, "memberName" )
                        cMemNodeName = AdminConfig.showAttribute(sId, "nodeName" ) 
                        memScope =   Scope(clusterName, cellName, cMemNodeName, cMemName)            		
                        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Create System Properties for: "+cMemName+":"+cMemNodeName , systemProperties)
                        setSystemProperties(memScope, systemProperties)
        #endif

        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "INFO: Save the configuration changes")
        AdminConfig.save()
        
        trc.exit(className, methodName)
        return

#endDef 


#############################################################
# main
#
# Install & Configure the REST Service Proxy Application on
# The WAS 6.0.2 server.  
#
#############################################################

methodName = "main"

trc.entry(className, methodName)

if ((len(sys.argv) != 1)):
        # print "This script (installPortalDashboardRestProxy.py) requires 1 parameter: profilePath"
        # print "e.g.: installPortalDashboardRestProxy.py /opt/WebSphere/Monitor/profiles/MonSrv01"
        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "This script (installPortalDashboardRestProxy.py) requires 1 parameter: profilePath")
        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "e.g.: installPortalDashboardRestProxy.py /opt/WebSphere/Monitor/profiles/MonSrv01")
        raise ValueError

# get the profile path
profilePath = sys.argv[0]
trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "profilePath="+profilePath)


# 
# Load the installation parameters 
#
execfile(profilePath+'/config.wbm/MonitorProfileVals.py')


lineSeparator = java.lang.System.getProperty('line.separator')
#
# get parameters we need
#
location         = restproxyPath
ear              = restproxyEarName
app              = restproxyAppName
node             = nodeName
cluster          = clusterName
proxyServerName  = serverName

        
# load the utility script to install the application ear
execfile(moncommonPath+'/installEar.py')
execfile(moncommonPath+'/MonitorServerUtil.py')

installStep = "Install "+ app

try:
        # install REST Services Proxy App
        log.msg(ILogger.TYPE_INFO, className, methodName, "CWMLP0010I", installStep)
        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "Installing REST Services Proxy App...")
        # only deploy the ear if on a dmanager or standalone
        rc = 0 
        if (ndType == "standalone"):
                rc = installEarOnServer(location, ear, app, node, proxyServerName)
                trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "Installation of REST Services Proxy App complete to server.")
        elif (ndType == "dmgr"):
                rc = installEarOnCluster(location, ear, app, cluster)
                trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "Installation of REST Services Proxy App complete to cluster.")
        
        if (rc == 0):
                # configure REST Services Proxy
                trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "Configuring REST Services Proxy.")
                configureProxy(cluster, node, proxyServerName)
                trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "Configuration of REST Services Proxy completed.")
                # Installation of REST Service Proxy was successful
                log.msg(ILogger.TYPE_INFO, className, methodName, "CWMLP0011I", installStep)
        else:
                log.msg(ILogger.TYPE_ERROR, className, methodName, "CWMLP2130E", installStep, profilePath+"/logs/installPortalDashboardRestProxy.log")
                        
except:
        # Error
        log.msg(ILogger.TYPE_ERROR, className, methodName, "CWMLP2130E", installStep, profilePath+"/logs/installPortalDashboardRestProxy.log")
        
        # rethrow the exception
        trc.trace(ITracer.TYPE_LEVEL3, className, methodName, "Rethrowing exception caught during Install/configuration of REST Services Proxy...")
        raise
        #return 1
        
trc.exit(className, methodName)


