/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.data.Resolver"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.Resolver"] = true;
/*
 * @author Ben
 */ 
dojo.provide("com.ibm.data.Resolver");

//dojo.require("dojo.AdapterRegistry");
dojo.require("com.ibm.lang.DeferredQueue");
dojo.require("com.ibm.data.resolver.FaultFactory");
dojo.require("dojo.DeferredList");
dojo.require("com.ibm.data.DependencyManager");
dojo.require("com.ibm.utilities");
dojo.require("dojo.i18n");

(function(){	
	var rzHandlers = {};
	
	com.ibm.data.Resolver = {
		register: function(prefix, cntr) {
			if(!rzHandlers[prefix]) rzHandlers[prefix] = cntr;
		},
		getHandler: function(prefix) {
			return rzHandlers[prefix];
		},
		getStore: function(args) {
			// TODO: Should this factory function return singletons or new objects
			return new com.ibm.data.ResolverStore(args);
		},
		_flagDigestUpdate: function() {
			dojo.cookie( "DigestTracker", (new Date()).getTime(), {path: "/"} );
		},
		TOPICS: {
			UpdateDigest: "/resolver/UpdateDigest"
		}
	}
	
	dojo.subscribe(com.ibm.data.Resolver.TOPICS.UpdateDigest, com.ibm.data.Resolver._flagDigestUpdate);
})();

dojo.requireLocalization("com.ibm.bundles", "Resolver", null, "ROOT,ar,ca,cs,da,de,el,en,es,fi,fr,he,hr,hu,it,ja,ko,nl,no,pl,pt,pt-br,ro,ru,sk,sl,sv,th,tr,uk,zh,zh-tw");

dojo.declare("com.ibm.data.ResolverStore",
	null,
	{
		// tunnel: Boolean flag indicating whether PUT and DELETE requests should
		//		be tunneled through POST requests
		tunnel: false,
		
		
		// states: Object map of possible states that items in this store can have.  These are
		//		not mutually exlusive and do not follow a finite state machine pattern since
		//		items have multiple states simultaneously.
		states: {
			"new": 0,			// new and not yet created on the server
			"dirty": 1,			// update to an existing item that hasn't been saved yet
			"deletedExpl": 2,	// explicitly deleted
			"deletedImpl": 3,	// implicitly deleted by means of dependency relationship
			"deleted": 4,		// deleted
			"loaded": 5,		// has been loaded
			"saving": 6			// has already been processed and queued for saving, yet not completed yet
		},
		
		_features: {
			'dojo.data.api.Read': true,
			'dojo.data.api.Write': true,
			'dojo.data.api.Identity': true,
			'dojo.data.api.Notification': true,
			'com.ibm.data.api.ServerGenerated': true
		},
		
		namespaces: {},
		
		headers: {
			"DIGEST": "X-Request-Digest",
			"STATUS": "X-Status-Code",
			"URI": "Content-ID",
			"CONTENT_TYPE": "Content-Type",
			"METHOD": "X-Method-Override",
			"AUTH_OVERRIDE": "X-IBM-XHR"
		},
		
		_digest: "",
		url: "",
		
		saveWait: 1,
		
		constructor: function(args) {
			dojo.mixin(this, args);
			this._handlers = {};
			this._cachedItems = {};
			this._itemLoaders = {};
			this._cidMap = {};
			this._ioOutQueue = this._getDeferredQueue(new dojo.Deferred()).fire();
			this._laundryQueue = [];
			this._cidCntr = Math.floor(Math.random() * 10000001);
			this._etagMap = {};
			this._notifyOn = true;
			
			this.multiPart = true;
			this.multiPartUrl = this.url + "?uri=mashup:multiplex";
			
			this._auth = dojo.getObject("com.ibm.ajax.auth");
			this._depMgr = new com.ibm.data.DependencyManager(this);
			this._messages = dojo.i18n.getLocalization("com.ibm.bundles", "Resolver");
			this._faultFactory = new com.ibm.data.resolver.FaultFactory();
			// will we need this AdapterRegistry someday???
			// right now, we always know our argument type, but we may not always
			/*
			var reg = this._handlerReg = new dojo.AdapterRegistry(false);
			reg.register("strMatch", 
				dojo.isString, 
				dojo.hitch(this, function(str){
					if(str.indexOf(":") > 0) {
						return this._getHandlerByUri(str);
					}
					else {
						return this._getHandlerByPocType(str);
					}
				})
			);
			reg.register("itemMatch", 
				dojo.hitch(this, this._isItem), 
				dojo.hitch(this, this._getHandlerByItem)
			);
			*/
			
			dojo.addOnWindowUnload(dojo.hitch(this, function(){
				this.destroy();
			}));
		},
		destroy: function() {
			for(var id in this._cachedItems) {
				var item = this._cachedItems[id];
				try {
					if(this._isItem(item)) this._getHandlerByItem(item).uninitialize(item);
					for(var p in item) {
						delete item[p];
					}
				} 
				catch(err) {}
			}
			delete this._cachedItems;
			for(var scheme in this._handlers) {
				var h = this._getHandlerByPocType(scheme);
				h.destroy();
				delete h._$; 
			}
			delete this._handlers;
			delete this._itemLoaders;
			this._depMgr.destroy();
			delete this._depMgr;
			delete this._ioOutQueue;
			delete this._laundryQueue;
			delete this._messages;
		},
		getNamespaces: function() {
			return this.namespaces;
		},
		applyHandlerStrategy: function(/*String*/ scheme, /*Strategy*/ strategy) {
			var h = this._getHandlerByPocType(scheme);
			return com.ibm.strategy.Strategy.applyStrategy(h, strategy);
		},
		unapplyHandlerStrategy: function(/* Advice */ adv) {
			com.ibm.strategy.Strategy.removeStrategy(adv);
		},
		_initStoreMeta: function(/*Item*/ item, /*Object*/ mixin) {
			if(item._$ == null) {
				item._$ = {
					states: [],
					store: this
				};
			}
			item._$ = dojo.mixin(item._$, mixin);
			return item;
		},
		_getDeferredQueue: function(/*Deferred?*/ dfd) {
			var dfdq = new com.ibm.lang.DeferredQueue(dfd);
			dfdq.raiseError = dojo.hitch(this, "raiseError");
			return dfdq;
		},
		_getPocTypeFromUri: function(/*String*/ uri) {
			if(!uri) {
				var err = new Error(this.fmt("errorNoPocHandler", [uri]))
				this.raiseError(err);
				throw err;
			}
			return uri.split(":")[0];
		},
		_getHandlerByPocType: function(/*String*/ prefix) {
			if(!this._handlers[prefix]) {
				var h = com.ibm.data.Resolver.getHandler(prefix);
				if(!h) {
					var err = new Error(this.fmt("errorNoPocHandler", [prefix]));
					this.raiseError(err);
					throw err;
				}
				var inst = new h(this, this._depMgr);
				this._handlers[prefix] = inst;
				inst._$ = {
					stateLists: [[],[],[]]
				};
			}
			return this._handlers[prefix];
		},
		_getHandlerByItem: function(/* item */ item) {
			this._assertItem(item);
			return this._getHandlerByPocType(item._$.pocType);
		},
		_getHandlerByUri: function(/*String*/ uri) {
			return this._getHandlerByPocType(this._getPocTypeFromUri(uri));
		},
		raiseError: function(error) {
			// summary: Raises an error received during some processing related to this store,
			//		by this store, or by one of its subsidiaries.  Calls console.error by default, 
			// 		so connect to it or override it to provide custom implementation.
			// error: Error object to raise.
			console.error(error);
		},
		fmt: function(key, map) {
			return dojo.string.substitute(this._messages[key], map, function(mapValue, mapKey){
				if(mapValue === null) return "null";
				if(mapValue === undefined) return "undefined";
				return mapValue;
			});
		},
		_isItem: function(/*Object*/ something) {
			return something 
				&& something._$
				&& something._$.pocType
				&& something._$.store === this;
		},
		_assertItem: function(/*Object*/ something) {
			// summary: Throws an exception if the argument is not a valid item in this store.
			// something: Object to check to determine if it is an item in this store.
			if(!this._isItem(something)) {
				var err = new Error(this.fmt("errorInvalidItem", [something]));
				this.raiseError(err);
				throw err;
			}
			return true;
		},
		_assertString: function(/*ject*/ something) {
			// summary: Throws an exception if the argument is not a string.
			// something: Object to check to determine if it is a string.
			if(!dojo.isString(something)) {
				var err = new Error(this.fmt("errorInvalidString", [something]));
				this.raiseError(err);
				throw err;
			}
			return true;
		},
		_getDigestFromResponse: function(response, ioArgs) {
			var digest = ioArgs.xhr.getResponseHeader(this.headers.DIGEST);
			if(digest) this._digest = digest;
		},
		_flagDigestUpdate: function() {
			dojo.publish(com.ibm.data.Resolver.TOPICS.UpdateDigest);
		},
		_queryToHeader: function(query) {
			return dojo.objectToQuery(query).replace(/&/mg, "; ");
		},
		_isResponseError: function(partArgs, xhr) {
			var s = xhr.status;
			// 1223 is for MSIE's MSXML "interpretation" of a 204
			if((s >= 200 && s < 300) || s == 302 || s == 1223) {
				return false;
			}
			return true;
		},
		_multiPartContentHandler: function(partArgs, xhr, defaultContentHandler) {
			// summary: Preprocesses multipart part response information, returning an updated response or
			//		the original depending on the required semantics.
			var status = xhr.getResponseHeader(this.headers.STATUS);
			xhr.status  = status ? status - 0 : 200;
			var response = xhr.responseText;
			if(this._isResponseError(partArgs, xhr)) {
				if(xhr.getResponseHeader(this.headers.CONTENT_TYPE) == "application/soap+xml") {
					response = this._faultFactory.getFaultError(response, new Error());
				}
				else response = new Error(response);
			}
			else {
				response = defaultContentHandler(partArgs, xhr);
			}
			return response;
		},
		_io: function(/*Array*/ ioArgsArr, /*Boolean?*/ fetchSiteMap) {
			// summary: Initializes and manages all IO for this store.
			// returns: Object map 
			//		{
			//			start: Function starting the IO this function sets up
			//			deferredList: DeferredList tracking all requests
			//		}
			var multiPartsArr = [],
				dfdBuilder = null,
				deferredFunc = null;
				
			if(this.multiPart){ 
				dfdBuilder = function(ioArgs) {
					var dfd = new dojo.Deferred();
					
					if(!ioArgs.headers) ioArgs.headers = {};
					ioArgs.headers[this.headers.URI] = ioArgs.query.uri;
					ioArgs.headers[this.headers.METHOD] = ioArgs.method.toUpperCase();
					delete ioArgs.query.uri;
					var query = this._queryToHeader(ioArgs.query);
					if(query.length > 0) ioArgs.headers[this.headers.URI] += "; " + query;
					delete ioArgs.query;
					
					var oldError = ioArgs.error;
					ioArgs.error = function(error, args) {
						try {
							if(oldError) oldError.call(ioArgs, error, args);
						} catch(err){}
					};
					
					var oldHandle = ioArgs.handle;
					ioArgs.handle = function(response, args) {
						try {
							if(oldHandle) oldHandle.call(ioArgs, response, args);
						} catch(err){}
						if(response instanceof Error) {
							dfd.errback(response);
						}
						else dfd.callback(response);
					};
					multiPartsArr.push(ioArgs);
					
					return dfd;
				};
				
				deferredFunc = dojo.hitch(this, function() {
					var hdrs = {}; hdrs[this.headers.AUTH_OVERRIDE] = "true";
					var mPartDfd = com.ibm.utilities.multiPartXhr("POST", {
						headers: hdrs,
						url: this.multiPartUrl,
						partContentHandler: dojo.hitch(this, "_multiPartContentHandler"),
						preHandle: dojo.hitch(this, function(response, args){
							if(!this._authHandler(response, args)) return;
							this._getDigestFromResponse(response, args);
						})
					}, multiPartsArr);
				});
			}
			else {
				var ioChain = [];
				dfdBuilder = function(ioArgs) {
					ioArgs.url = this.url + "?" + dojo.objectToQuery(ioArgs.query);
					delete ioArgs.query;
					
					ioArgs.method = ioArgs.method.toLowerCase();
					if(ioArgs.method == "put" || ioArgs.method == "post") {
						ioArgs[ioArgs.method + "Data"] = ioArgs.data;
					}
                    if(!ioArgs.headers) ioArgs.headers = {};
                    ioArgs.headers[this.headers.AUTH_OVERRIDE] = "true";
					if(this.tunnel && ioArgs.method != "post" && ioArgs.method != "get") {
						ioArgs.headers[this.headers.METHOD] = ioArgs.method.toUpperCase();
						ioArgs.method  = "post";
					}
                    var oldHandle = ioArgs.handle;
                    ioArgs.handle = dojo.hitch(this, function(response, args) {
                    	if(!this._authHandler(response, args)) return;
						if(oldHandle) oldHandle.call(args, response, args);
					});
					var oldLoad = ioArgs.load;
					ioArgs.load = dojo.hitch(this, function(response, args) {
						this._getDigestFromResponse(response, args);
						if(oldLoad) oldLoad.call(args, response, args);
					});
					
					var ioDef = new dojo.Deferred();
					ioChain.push(function(){
						return dojo.xhr(ioArgs.method, ioArgs, true).addCallbacks(function(resp){
							return ioDef.callback(resp);
						}, function(err){
							return ioDef.errback(err);
						});
					});
					return ioDef;
				};
				
				deferredFunc = function() {
					dojo.forEach(ioChain, function(ioFunc){
						ioFunc();
					});
				}
			}
		
			var dfds = dojo.map(ioArgsArr, dfdBuilder, this),
				dfdl = new dojo.DeferredList(dfds);
				
			dfdl.addCallback(function(resultArr){
				return dojo.map(resultArr, function(res){
					return res[1];
				});
			});
			
			return {
				start: function(res){
					deferredFunc();
					return dfdl;
				},
				deferredList: dfdl
			};
		},
		_authHandler: function(response, args) {
			if(this._auth) {
                if (this._auth.isAuthenticationRequired(response, args)) {
                    this._auth.authenticationHandler();
                    return false;
                }
            }
            return true;
		},
		_ioOut: function(/*Array*/ ioArgsArr) {
			// summary: Internal function managing IO requests to push data to the server.
			//	Manages all IO transports for this data store, providing multiplexing
			//		of IO requests into complicated chains of potentially multipart requests
			//		for performance enhancements.  Ensures data dependencies between items
			//		are enforced and managed appropriately.  When this is called, it immediately
			//		sends the IO request, returning the Deferred which will be fired upon
			//		IO completion.
			if(!ioArgsArr || ioArgsArr.length == 0) {
				var retDfd = new dojo.Deferred();
				retDfd.callback([]);
				return retDfd;
			}
			var ioObj = this._io(ioArgsArr);
			this._flagDigestUpdate();
			return ioObj.start();				// initialize IO	
		},
		_ioIn: function(/*Array*/ ioArgsArr) {
			// summary: Internal function managing IO requests to get data from the server.
			//		Manages all IO transports for this data store, providing multiplexing
			//		of IO requests into complicated chains of potentially multipart requests
			//		for performance enhancements.
			return (new dojo.DeferredList(dojo.map(ioArgsArr, function(ioArgs){
				/*
				if(this._etagMap[ioArgs.url]) {
					if(!ioArgs.headers) ioArgs.headers = {};
					ioArgs.headers["If-None-Match"] = this._etagMap[ioArgs.url];
				}
				
				ioArgs._oldHandle = ioArgs.handle;
				ioArgs.handle = dojo.hitch(this, function(response, ioArgs) {
					this._etagMap[ioArgs.url] = ioArgs.xhr.getResponseHeader("etag");
					if(ioArgs._oldHandle) return ioArgs._oldHandle(response, ioArgs);
				});
				*/
                var oldHandle = ioArgs.handle;
                ioArgs.handle = dojo.hitch(this, function(response, args) {
					if(!this._authHandler(response, args)) return;
					if(oldHandle) oldHandle.call(args, response, args);
				});
				return dojo.xhrGet(ioArgs);
			}, this))).addCallback(function(resultArr){
				return dojo.map(resultArr, function(res){
					return res[1];
				})
			});
		},
		_fromCache: function(id) {
			return this._cachedItems[id];
		},
		_toCache: function(id, item) {
			this._cachedItems[id] = item;
		},
		
		/********************************/
		/**                            **/
		/**                            **/
		/**     dojo.data.api.Read     **/
		/**                            **/
		/**                            **/
		/********************************/
		getValue: function(	/* item */ item, 
							/* attribute-name-string */ attribute, 
							/* value? */ defaultValue){
			this._assertString(attribute);
			return this._getHandlerByItem(item).getValue(item, attribute, defaultValue);
		},
		getValues: function(/* item */ item,
							/* attribute-name-string */ attribute){
			this._assertString(attribute);
			return this._getHandlerByItem(item).getValues(item, attribute);
		},
		getAttributes: function(/* item */ item){
			return this._getHandlerByItem(item).getAttributes(item);
		},
		hasAttribute: function(	/* item */ item,
								/* attribute-name-string */ attribute){
			this._assertString(attribute);
			return this._getHandlerByItem(item).hasAttribute(item, attribute);
		},
		containsValue: function(/* item */ item,
								/* attribute-name-string */ attribute, 
								/* anything */ value){
			this._assertString(attribute);
			return this._getHandlerByItem(item).containsValue(item, attribute, value);
		},
		isItem: function(/* anything */ something){
			return this._isItem(something)
				&& !this._hasState(something, this.states["deleted"]);
		},
		isItemLoaded: function(/* anything */ something) {
			return this.isItem(something) && this._hasState(something, this.states["loaded"]);
		},
		loadItem: function(/* object */ keywordArgs){
			var it = keywordArgs.item;
			if(this.isItemLoaded(it)) {
				return;
			}
			var ldr = this._getItemLoader(it);
			if(keywordArgs.onItem) ldr.addOnLoad(dojo.hitch(keywordArgs.scope, keywordArgs.onItem));
			if(keywordArgs.onError) ldr.addOnError(dojo.hitch(keywordArgs.scope, keywordArgs.onError));
		},
		fetch: function( keywordArgs ) {
			//    summary:
	        //        Given a query and set of defined options, such as a start and count of items to return,
	        //        this method executes the query and makes the results available as data items.
	        //        The format and expectations of stores is that they operate in a generally asynchronous 
	        //        manner, therefore callbacks are always used to return items located by the fetch parameters.
	        //
	        //    description:
	        //        A Request object will always be returned and is returned immediately.
	        //        The basic request is nothing more than the keyword args passed to fetch and 
	        //        an additional function attached, abort().  The returned request object may then be used 
	        //        to cancel a fetch.  All data items returns are passed through the callbacks defined in the 
	        //        fetch parameters and are not present on the 'request' object.
	        //
	        //        This does not mean that custom stores can not add methods and properties to the request object
	        //        returned, only that the API does not require it.  For more info about the Request API, 
	        //        see dojo.data.api.Request
	        //
	        //    keywordArgs:
	        //        The keywordArgs parameter may either be an instance of 
	        //        conforming to dojo.data.api.Request or may be a simple anonymous object
	        //        that may contain any of the following:
	        //        { 
	        //            query: query-string or query-object,
	        //            queryOptions: object,
	        //            onBegin: Function,
	        //            onItem: Function,
	        //            onComplete: Function,
	        //            onError: Function,
	        //            scope: object,
	        //            start: int
	        //            count: int
	        //            sort: array
	        //        }
	        //        All implementations should accept keywordArgs objects with any of
	        //        the 9 standard properties: query, onBegin, onItem, onComplete, onError 
	        //        scope, sort, start, and count.  Some implementations may accept additional 
	        //        properties in the keywordArgs object as valid parameters, such as 
	        //        {includeOutliers:true}.         
	        //
			//	The *query* parameter.
			//        The query may be optional in some data store implementations.
			//        The dojo.data.api.Read API does not specify the syntax or semantics
			//        of the query itself -- each different data store implementation
			//        may have its own notion of what a query should look like.
			//        However, as of dojo 0.9, 1.0, and 1.1, all the provided datastores in dojo.data
			//        and dojox.data support an object structure query, where the object is a set of 
			//        name/value parameters such as { attrFoo: valueBar, attrFoo1: valueBar1}.  Most of the
			//        dijit widgets, such as ComboBox assume this to be the case when working with a datastore 
			//        when they dynamically update the query.  Therefore, for maximum compatibility with dijit 
			//        widgets the recommended query parameter is a key/value object.  That does not mean that the
			//        the datastore may not take alternative query forms, such as a simple string, a Date, a number, 
			//        or a mix of such.  Ultimately, The dojo.data.api.Read API is agnostic about what the query 
			//        format.  
			//        Further note:  In general for query objects that accept strings as attribute 
			//        value matches, the store should also support basic filtering capability, such as * 
			//        (match any character) and ? (match single character).  An example query that is a query object
			//        would be like: { attrFoo: "value*"}.  Which generally means match all items where they have 
			//        an attribute named attrFoo, with a value that starts with 'value'.
	        //
	        //    The *queryOptions* parameter
	        //        The queryOptions parameter is an optional parameter used to specify optiosn that may modify
	        //        the query in some fashion, such as doing a case insensitive search, or doing a deep search
	        //        where all items in a hierarchical representation of data are scanned instead of just the root 
	        //        items.  It currently defines two options that all datastores should attempt to honor if possible:
	        //        {
	        //            ignoreCase: boolean, //Whether or not the query should match case sensitively or not.  Default behaviour is false.
	        //            deep: boolean     //Whether or not a fetch should do a deep search of items and all child 
	        //                            //items instead of just root-level items in a datastore.  Default is false.
	        //        }
	        //
	        //    The *onBegin* parameter.
	        //        function(size, request);
	        //        If an onBegin callback function is provided, the callback function
	        //        will be called just once, before the first onItem callback is called.
	        //        The onBegin callback function will be passed two arguments, the
	        //        the total number of items identified and the Request object.  If the total number is
	        //        unknown, then size will be -1.  Note that size is not necessarily the size of the 
	        //        collection of items returned from the query, as the request may have specified to return only a 
	        //        subset of the total set of items through the use of the start and count parameters.
	        //
	        //    The *onItem* parameter.
	        //        function(item, request);
	        //        If an onItem callback function is provided, the callback function
	        //        will be called as each item in the result is received. The callback 
	        //        function will be passed two arguments: the item itself, and the
	        //        Request object.
	        //
	        //    The *onComplete* parameter.
	        //        function(items, request);
	        //
	        //        If an onComplete callback function is provided, the callback function
	        //        will be called just once, after the last onItem callback is called.
	        //        Note that if the onItem callback is not present, then onComplete will be passed
	        //        an array containing all items which matched the query and the request object.  
	        //        If the onItem callback is present, then onComplete is called as: 
	        //        onComplete(null, request).
	        //
	        //    The *onError* parameter.
	        //        function(errorData, request); 
	        //        If an onError callback function is provided, the callback function
	        //        will be called if there is any sort of error while attempting to
	        //        execute the query.
	        //        The onError callback function will be passed two arguments:
	        //        an Error object and the Request object.
	        //
	        //    The *scope* parameter.
	        //        If a scope object is provided, all of the callback functions (onItem, 
	        //        onComplete, onError, etc) will be invoked in the context of the scope
	        //        object.  In the body of the callback function, the value of the "this"
	        //        keyword will be the scope object.   If no scope object is provided,
	        //        the callback functions will be called in the context of dojo.global().  
	        //        For example, onItem.call(scope, item, request) vs. 
	        //        onItem.call(dojo.global(), item, request)
	        //
	        //    The *start* parameter.
	        //        If a start parameter is specified, this is a indication to the datastore to 
	        //        only start returning items once the start number of items have been located and
	        //        skipped.  When this parameter is paired withh 'count', the store should be able
	        //        to page across queries with millions of hits by only returning subsets of the 
	        //        hits for each query
	        //
	        //    The *count* parameter.
	        //        If a count parameter is specified, this is a indication to the datastore to 
	        //        only return up to that many items.  This allows a fetch call that may have 
	        //        millions of item matches to be paired down to something reasonable.  
	        //
	        //    The *sort* parameter.
	        //        If a sort parameter is specified, this is a indication to the datastore to 
	        //        sort the items in some manner before returning the items.  The array is an array of 
	        //        javascript objects that must conform to the following format to be applied to the
	        //        fetching of items:
	        //        {
	        //            attribute: attribute || attribute-name-string,
	        //            descending: true|false;   // Optional.  Default is false.
	        //        }
	        //        Note that when comparing attributes, if an item contains no value for the attribute
	        //        (undefined), then it the default ascending sort logic should push it to the bottom 
	        //        of the list.  In the descending order case, it such items should appear at the top of the list.
	        // 
	        //    returns:
	        //        The fetch() method will return a javascript object conforming to the API
	        //        defined in dojo.data.api.Request.  In general, it will be the keywordArgs
	        //        object returned with the required functions in Request.js attached.
	        //        Its general purpose is to provide a convenient way for a caller to abort an
	        //        ongoing fetch.  
	        // 
	        //        The Request object may also have additional properties when it is returned
	        //        such as request.store property, which is a pointer to the datastore object that 
	        //        fetch() is a method of.
	        //
	        //    exceptions:
	        //        Throws an exception if the query is not valid, or if the query
	        //        is required but was not supplied.
			if(!keywordArgs) keywordArgs = {};
			keywordArgs.aborted = false;
			if(!keywordArgs.scope) keywordArgs.scope = dojo.global;
			if(!keywordArgs.queryOptions) keywordArgs.queryOptions = {};
			var preventCache = keywordArgs.queryOptions.preventCache;
			var url = this._buildUrl(keywordArgs, this.url);
			keywordArgs.url = url;
			
			var pocType = this._getPocTypeFromUri(keywordArgs.query.uri);
			var pocHandler = this._getHandlerByPocType(pocType);
			var cType = pocHandler.definition.handleAs;
			
			var loadHandler = dojo.hitch(this, function(response, ioArgs) {
				try {
					if(keywordArgs.aborted) return;
					if(dojo.isIE && cType == "xml") response = com.ibm.domUtilities.docFromString(response);
					this.processResponse(response, keywordArgs, pocType);
				}
				catch(err) {
					this.raiseError(err);
				}
			});
			var errorHandler = function(error, ioArgs) {
				if(keywordArgs.onError) keywordArgs.onError.call(keywordArgs.scope, error, keywordArgs);
			};
			
			var args = {
				preventCache: preventCache,
				content: {},
				url: url,
				handleAs: "text",
				headers: {"X-IBM-XHR":"true"},
				load: loadHandler,
				error: errorHandler
			}
			
			if(cType) {
				if(!dojo.isIE || cType != "xml") {
					// special handling for "xml" in IE
					args.handleAs = cType;
				}
			}
			
			if(keywordArgs.queryOptions.timeout != null) {
				args.timeout = keywordArgs.queryOptions.timeout;
			}
			else if(this.timeout) {
				args.timeout = this.timeout;
			}
			
			var deferred = null;
			keywordArgs.abort = function() {
				keywordArgs.aborted = true;
				deferred.cancel();
			};
			args = this.modifyRequestArgs(args, keywordArgs);
			
			deferred = this._ioIn([args]);
			//keywordArgs.deferred = deferred;

			this._lastRequest = keywordArgs;
			return keywordArgs;
		},
		modifyRequestArgs: function(ioArgs, request) {
			// summary: Allows modification to request arguments before sending a request
			//		to the server.  Subclasses may override this to modify arguments
			//		like setting headers, manipulating the url, overriding a handleAs argument,
			//		or anything else.  Returns the modified request args.
			// ioArgs: dojo.__XhrArgs request args to be sent to the server
			// request: dojo.data.api.Request object
			// returns: dojo.__XhrArgs modified request arguments
			return ioArgs;
		},
		_queryUrlToObject: function(queryStr) {
			if(!queryStr || queryStr.length == 0) return {};
			var start = queryStr.indexOf("?");
			var str = queryStr;
			if(start > -1 && (start < str.length - 1)) { 
				// extract only the query part of the queryStr URL
				str = queryStr.substring(start + 1);
			}
			return dojo.queryToObject(str);
		},
		_prepareQuery: function(keywordArgs) {
			var q = keywordArgs.query;
			if(!q) q = {};
			else if(dojo.isString(q)) {
				q = this._queryUrlToObject(q);
			}
			keywordArgs.query = this._getHandlerByUri(q.uri).modifyFetchQuery(q);
			if(this._digest && this.url.indexOf(this._digest)) keywordArgs.query.digest = this._digest;
			return this.prepareQuery(keywordArgs);
		},
		prepareQuery: function(keywordArgs) {
			// summary: User overridable function to modify the keywordArgs object
			// 		used to fetch results from the back end after the keywordArgs.query
			//		property has been normalized to an object map but before any other
			//		processing has taken place to build the URL.  This can be used to
			//		extract other parts of the keywordArgs map to add properties to the
			//		query object that are dependent on the back end implementation.  One
			//		example includes extracting the keywordArgs.sort parameters and appending
			//		appropriate properties to the keywordArgs.query property that will end
			//		up being built into the generated URL to the server.
			// returns: Object Modified keywordArgs object.
			return keywordArgs;
		},
		_buildUrl: function(keywordArgs, baseUrl) {
			// summary: Builds a url string from this store's base url and the
			// 		query argument.
			// keywordArgs: Object
			//		A request object map as sent to the fetch function
			// returns: String The constructed url.
			var ret = "";
			var keywordArgs = this._prepareQuery(keywordArgs);
			var queryPart = dojo.objectToQuery(keywordArgs.query);
			if(baseUrl.indexOf("?") > -1) {
				ret = baseUrl + "&" + queryPart; // String
			}
			else ret = baseUrl + "?" + queryPart; // String
			
			return (this.modifyUrl(keywordArgs, ret));
		},
		modifyUrl: function(keywordArgs, url) {
			return url;
		},
		processFetchedItems: function(/*Array*/ items, /*Object*/ meta, /*Request*/ request, /*String*/ pocType, /*Object*/ pocHandler) {
			var preventClientCache = (request.queryOptions && request.queryOptions.preventClientCache);
			var replaceCache = (request.queryOptions && request.queryOptions.replaceCache);
			
			// iterate through the items we just fetched and handle them according to the queryOptions cache parameters
			// and what we already have in the cache
			for(var i = 0; i < items.length; i++) {
				this._initStoreMeta(items[i], {pocType: pocType});
				this._addState(items[i], pocHandler, this.states["loaded"]);
				this._toCache(this.getIdentity(items[i]), items[i]);		
				// any pre-existing item detection should be done by POC Handlers themselves
			}
			
			return items;
		},
		processResponse: function(/*Object*/ data, /*Request*/ request, /*String*/ pocType) {
			// summary: Extension point for descendants or consumers of this store to handle
			//		responses from server requests for data.  Most subclasses can leave this
			// 		function as-is, but if special handling is required that cannot be done
			// 		with other extension points, this can be overridden as well.
			// 		Must call onBegin, onItem, and onComplete callbacks correctly as 
			// 		specified by dojo.data.api.Read.  The onError callback is handled elsewhere.
			// data: Object data of the response from an IO request in the format expected by
			//		the handler.
			// request: Request used to fetch the result
			// pocHandler: Object POC handler to use to process this response data
			if(request.aborted) return;
			var pocHandler = this._getHandlerByPocType(pocType);
			var composite = pocHandler.processFetch(data, request);
			items = this.processFetchedItems(composite.items, composite.meta, request, pocType, pocHandler);
			if(request.onBegin) request.onBegin.call(request.scope, composite.meta.resultSize, request);
			if(request.aborted) return;
			if(request.onItem) {
				for(var i = 0; i < items.length; i++) {
					if(request.aborted) return;
					request.onItem.call(request.scope, items[i], request);
				}
			}
			if(request.aborted) return;
			if(request.onComplete) {
				// call onComplete with items array if onItem wasn't specified, otherwise
				// call it with null
				if(!request.onItem) request.onComplete.call(request.scope, items, request);
				else request.onComplete.call(request.scope, null, request);
			}
		},
		
		getFeatures: function(){
			return this._features;
		},
		close: function(/*dojo.data.api.Request || keywordArgs || null */ request){
			if(request && request.abort) request.abort();
		},
		getLabel: function(/* item */ item){
			return this._getHandlerByItem(item).getLabel(item);
		},
		getLabelAttributes: function(/* item */ item){
			return this._getHandlerByItem(item).getLabelAttributes(item);
		},
		
		/********************************/
		/**                            **/
		/**                            **/
		/**   dojo.data.api.Identity   **/
		/**                            **/
		/**                            **/
		/********************************/	
		getIdentity: function(/* item */ item){
			return this._getHandlerByItem(item).getIdentity(item);
		},
		getIdentityAttributes: function(/* item */ item){
			return this._getHandlerByItem(item).getIdentityAttributes(item);
		},
		fetchItemByIdentity: function(/* object */ keywordArgs){
			var item = this.byId(keywordArgs.identity);
			if(this.isItemLoaded(item)) {
				if(keywordArgs.onItem) {
					keywordArgs.onItem.call(scope ? scope : dojo.global, item);
				}
			}
			else {
				this.loadItem({
					item: item,
					onItem: keywordArgs.onItem,
					onError: keywordArgs.onError,
					scope: keywordArgs.scope
				});
			}
		},
		byId: function(/*String*/ id) {
			var item = this._fromCache(id);
			if(!item) {
				item = this.byCId(id);	// might be a cid...
				if(!item) {
					// create stub
					var type = this._getPocTypeFromUri(id),
						h = this._getHandlerByPocType(type);
					item = h.createStub({id: id});
					this._initStoreMeta(item, {pocType: type});
					this._toCache(id, item);
				}
			}
			return item;
		},
		
		/********************************/
		/**                            **/
		/**                            **/
		/**    dojo.data.api.Write     **/
		/**                            **/
		/**                            **/
		/********************************/
		_createItemFromProps: function(keywordArgs, parentInfo, pocHandler) {
			var pocType = keywordArgs.pocType;
			delete keywordArgs.pocType;
			if(!parentInfo) parentInfo = {};
			
			if(parentInfo.parent && !keywordArgs.parentId) {
				// if the parent id wasn't specified in keywordArgs but we got a parent item in the parentInfo
				// object, set the keywordArgs.parentId prop to the id of the parent item
				keywordArgs.parentId = this.getIdentity(parentInfo.parent);
			}
			else if(!parentInfo.parent && keywordArgs.parentId){
				// get the parent item from either cache or as a stub (which is not loaded)
				parentInfo.parent = this.byId(keywordArgs.parentId);
			}
			
			// set the locale since some templates may need it if it's not already supplied
			if(!keywordArgs.locale) keywordArgs.locale = dojo.locale;
			
			var createdItem = this._initStoreMeta(pocHandler.newItem(keywordArgs, parentInfo), {pocType: pocType});
			this._addState(createdItem, pocHandler, this.states["loaded"]);
			return createdItem;
		},
		
		newItem: function(/* Object? */ keywordArgs, /*Object?*/ parentInfo){
			var h = this._getHandlerByPocType(keywordArgs.pocType);
			var parentItem = parentInfo ? parentInfo.parent : null;
			if(!keywordArgs.id) keywordArgs.id = h.definition.scheme + ":" + this.getUniqueCId();
			var id = keywordArgs.id;
			var item = this._createItemFromProps(keywordArgs, parentInfo, h);
			item._$.onCreateDefQueue = this._getDeferredQueue(new dojo.Deferred());
			this.registerCId(h.getIdentity(item), item);	// in case the handler modified the id
			this._toCache(id, item);
			this._changeState(item, h, this.states["new"]);
			if(this._notifyOn) {
				this.onNew(item, {item: parentItem});
			} // notify new
			return item;
		},
		deleteItem: function(/* item */ item){
			var h = this._getHandlerByItem(item);
			this._changeState(item, h, this.states["deletedExpl"]);
			if(this._notifyOn) {} // notify delete
		},
		_getItemLoader: function(item) {
			var id = this.getIdentity(item);
			if(!this._itemLoaders[id]) {
				this._itemLoaders[id] = (new com.ibm.data.ItemsLoader({
					query: {
						uri: id
					},
					store: this,
					filter: function(itemsArray) {
						return itemsArray[0];	// Item
					}
				})).addOnResult(dojo.hitch(this, function(){
					delete this._itemLoaders[id];
				}));
			}
			return this._itemLoaders[id];
		},
		_setValue: function(/* item */ item, 
							/* string */ attribute,
							/* almost anything */ value,
							/* boolean */ multi){
			this._assertString(attribute);
			if(this._notifyOn) var oldVal = this[multi ? "getValues" : "getValue"](item, attribute);
			var h = this._getHandlerByItem(item);
			if(!this.isItemLoaded(item)) {
				var err = new Error(this.fmt("errorWriteNotLoaded"));
				this.raiseError(err);
				throw err;
			}
			h.backup(item);
			var res = h[multi ? "setValues" : "setValue"](item, attribute, value);
			if(res) {
				this._changeState(item, h, this.states["dirty"]);
				if(this._notifyOn) {} // notify set
			}
			return res;
		},
		setValue: function(	/* item */ item, 
							/* string */ attribute,
							/* almost anything */ value){
			return this._setValue(item, attribute, value, false);
		},	
		setValues: function(/* item */ item,
							/* string */ attribute, 
							/* array */ values){
			return this._setValue(item, attribute, values, true);
		},	
		unsetAttribute: function(	/* item */ item, 
									/* string */ attribute){
			this._assertString(attribute);
			if(this._notifyOn) var oldVal = this.getValue(item, attribute);
			var h = this._getHandlerByItem(item);
			h.backup(item);
			var res = h.unsetAttribute(item, attribute);
			if(res) {
				this._changeState(item, h, this.states["dirty"]);
				if(this._notifyOn) {} // notify unset
			}				
			return res;
		},
		onSaveComplete: function(/*SaveList*/ saveList) {
			// summary: Event fired when a save operation has been completed.
			// saveList: SaveList containing information about each item saved during the last save 
			// 		operation, including the errors during save processing for each item, if any
			//		exist.  Connect to this using dojo.connect to receive notifications about save
			//		operations in this store, but take care to note that the saveList is destroyed
			// 		after this function is called.  It will be available to any connected functions
			// 		but will be unusable afterwards. 
		},
		save: function(/* object */ keywordArgs){
			// summary: Saves all dirty items in this store.
			// keywordArgs: Object map specifying callbacks to the save operation.
			//		{
			//			onComplete: function(saveList){} called when the save operation completes successfully
			//			onError: function(saveList, error){} called when some or all parts of the save operation
			//					failed
			//			scope: Object scope in which to call the callbacks
			//		}
			var clist = this._getUnSavedStateList(this.states["new"]),
				ulist = this._getUnSavedStateList(this.states["dirty"]),
				dlist = this._getUnSavedStateList(this.states["deletedExpl"]);
			var dirtyList = clist.concat(ulist);
			if(!keywordArgs) keywordArgs = {};
			keywordArgs._itemsOfInterest = dirtyList.concat(dlist);	// all items that we're picking up to save right now
			this._queueForLaundry(dirtyList, dlist, keywordArgs);			
		},
		saveItems: function(/*Array*/ items, /*Object*/ keywordArgs) {
			// summary: Incremental store save by saving the dirty items in the items argument array.
			//		If any of the items have dependencies that must be resolved before they can be saved,
			//		these dependencies will be tracked down and resolves, even if that means that other items
			//		not included in this list are saved along with this list.
			// items: Array of items to save
			// keywordArgs: Object map specifying callbacks to the save operation.
			//		{
			//			onComplete: function(saveList){} called when the save operation completes successfully
			//			onError: function(saveList, error){} called when some or all parts of the save operation
			//					failed
			//			scope: Object scope in which to call the callbacks
			//		}
			var dirtyList = [], delList = [];
			dojo.forEach(items, function(item){
				if(this._hasState(item, this.states["saving"])) return;
				if(this._hasState(item, this.states["new"]) ||
					this._hasState(item, this.states["dirty"])) {
					dirtyList.push(item);
				}
				else if(this._hasState(item, this.states["deletedExpl"])) {
					delList.push(item);
				}
			}, this);
			if(!keywordArgs) keywordArgs = {};
			keywordArgs._itemsOfInterest = dojo.isArray(items) ? items.slice() : null;	// all items that were specifically requested to be saved
			this._queueForLaundry(dirtyList, delList, keywordArgs);
		},
		_itemToIOArgs: function(item, saveList){
			var handler = this._getHandlerByItem(item);
			var itemSaveArgs = handler.processSave(item);
			var isCreationStep = this._hasState(item, this.states["new"]);
			
			var oldLoad = itemSaveArgs.load;
			itemSaveArgs.load = dojo.hitch(this, function(data, ioArgs){
				this._removeState(item, handler, this.states["new"]);
				this._removeState(item, handler, this.states["dirty"]);
				if(isCreationStep) this._fireCreated(item);
				if(oldLoad) oldLoad.call(itemSaveArgs, data, ioArgs);
				oldLoad = null;
				return data;
			});
			
			var oldError = itemSaveArgs.error;
			itemSaveArgs.error = dojo.hitch(this, function(err, ioArgs){
				saveList.setError(item, err);
				if(oldError) oldError.call(itemSaveArgs, err, ioArgs);
				oldError = null;
				return err;
			});
			
			var oldHandle = itemSaveArgs.handle;
			itemSaveArgs.handle = dojo.hitch(this, function(data, ioArgs){
				this._removeState(item, handler, this.states["saving"]);
				if(oldHandle) oldHandle.call(itemSaveArgs, data, ioArgs);
				oldHandle = null;
				return data;
			});
			return itemSaveArgs;
		},
		_planStepsMapper: function(step, saveList){
			var item = this.byId(step.name);
			var args = this._itemToIOArgs(item, saveList);
			if(this._hasState(item, this.states["new"])) {
				args.method = "POST";
			}
			else {
				args.method = "PUT";
			}
			return args;
		},
		_filterUnmetDependencies: function(steps, type, saveList) {
			return dojo.filter(steps, function(step){
				var item = this.byId(step.name);
				var targets = this._depMgr.getTargetsByType(step.name, type);
				return dojo.every(targets, function(id){
					var targetItem = this.byId(id);
					if(!this.isCreated(targetItem)) {
						saveList.setError(item, saveList.getError(targetItem));
						this._removeState(item, this._getHandlerByItem(item), this.states["saving"]);
						return false;
					}
					return true;
				}, this);
			}, this);
		},
		_queueForLaundry: function(dirtyList, delList, keywordArgs) {
			if(this._laundryQueue.length == 0) {
				setTimeout(dojo.hitch(this, "_launder"), this.saveWait);
			}
			dojo.forEach([dirtyList, delList], function(list){
				dojo.forEach(list, function(item){
					this._addState(item, this._getHandlerByItem(item), this.states["saving"]);
				}, this);
			}, this);
			this._laundryQueue.push({dirtyList: dirtyList, delList: delList, keywordArgs: keywordArgs});
		},
		_callSaveCallbacks: function(keywordArgsList, saveList) {
			dojo.forEach(keywordArgsList, function(kw){
				var errors = saveList.getErrors(kw._itemsOfInterest), method;
				if(errors.length > 0) {
					method = "onError";
				}
				else {
					method = "onComplete";
				}
				if(kw[method]) {
					try { kw[method].call(kw.scope ? kw.scope : dojo.global, saveList); }
					catch(err) { this.raiseError(err); }
				}
			}, this);
		},
		_launder: function() {
			var dirtyList = [], delList = [], argsList = [], pile;
			var saveList = new com.ibm.data.SaveList({store: this});
			
			while((pile = this._laundryQueue.shift()) != null) {
				dirtyList = dirtyList.concat(pile.dirtyList);
				delList = delList.concat(pile.delList);
				if(pile.keywordArgs) argsList.push(pile.keywordArgs);
			}
			
			var onCompleteQ = this._getDeferredQueue(),
				onErrorQ = this._getDeferredQueue();
			// rework to use keywordArgs._itemsOfInterest when firing callbacks
			dojo.forEach(argsList, function(keywordArgs){
				if(keywordArgs.onComplete) {
					onCompleteQ.add((new dojo.Deferred()).addCallback(
						dojo.hitch(keywordArgs.scope ? keywordArgs.scope : dojo.global, keywordArgs.onComplete, saveList)));
				}
				if(keywordArgs.onError) {
					onErrorQ.add((new dojo.Deferred()).addCallback(
						dojo.hitch(keywordArgs.scope ? keywordArgs.scope : dojo.global, keywordArgs.onError, saveList)));
				}
			}, this);
			
			var laundryPlan = this._depMgr.getPlan(dirtyList, com.ibm.data.DependencyManager.types["create"])
					.orderSteps()
					.filter(dojo.hitch(this, function(step){
						var item = this.byId(step.name)
						ret = this.isDirty(item); // && !this._hasState(item, this.states["saving"]);
						if(ret) {
							saveList.addItem(item);
							//this._addState(item, this._getHandlerByItem(item), this.states["saving"]);
						}
						return ret;	
						// plan includes entire dependency tree
						// only clean the dirty laundry that isn't currently in a pile or in the wash
					})
				),
				piles = laundryPlan.getGroupList();
				
			// order each pile using "order" dependencies
			dojo.forEach(piles, function(pile){
				var map = {};
				var pileItems = dojo.map(pile.steps, function(step){
					var item = this.byId(step.name);
					map[step.name] = item;
					return item;
				}, this);
				
				var pilePlan = this._depMgr.getPlan(pileItems, com.ibm.data.DependencyManager.types["order"])
					.orderSteps()
					.filter(dojo.hitch(this, function(step){
						return map[step.name] != null;
					}));
				
				pile.steps = pilePlan.orderedSteps;
			}, this);
			
			delList = dojo.map(delList, function(item) {
				saveList.addItem(item);
				//this._addState(item, this._getHandlerByItem(item), this.states["saving"]);
				var args = {
					query: {
						uri: this.getIdentity(item),
						rep: "empty"
					},
					method: "DELETE",
					load: dojo.hitch(this, function() {
						var h = this._getHandlerByItem(item);
						dojo.forEach(["dirty", "deletedExpl", "deletedImpl", "deleted", "saving"], function(stStr){
							this._removeState(item, h, this.states[stStr]);
						}, this);
						h.uninitialize(item);
						for(var p in item) {
							delete item[p];
						}
					})
				}
				
				var oldError = args.error;
				args.error = dojo.hitch(this, function(err, ioArgs){
					saveList.setError(item, err);
					if(oldError) oldError.call(args, err, ioArgs);
					oldError = null;
					return err;
				});
				
				return args;
			}, this);

			var currentPile = piles.shift(),
				ioReadyDfd = new dojo.Deferred();
			
			ioReadyDfd.addCallback(dojo.hitch(this, function(){
				var planList;
				if(currentPile != null) {
					planList = dojo.map(currentPile.steps, function(step){
						return this._planStepsMapper(step, saveList);
					}, this);
				}
				else planList = [];
				
				var ioArgsArr = planList.concat(delList);
				
				var washNextPile = dojo.hitch(this, function(res) {
					if((currentPile = piles.shift()) != null) {
						try {
							currentPile.steps = this._filterUnmetDependencies(currentPile.steps, com.ibm.data.DependencyManager.types["create"], saveList);
							ioArgsArr = dojo.map(currentPile.steps, function(step){
								return this._planStepsMapper(step, saveList);
							}, this);
							return this._ioOut(ioArgsArr).addCallback(washNextPile);
						}
						catch(err) {
							this.raiseError(err);
						}
					}
					else {
						var errorList = saveList.getErrors();
						if(errorList.length == 0) {
							onCompleteQ.fire(saveList);
						}
						if(errorList.length > 0) {
							onErrorQ.fire(saveList, new Error());
						}
						this.onSaveComplete(saveList);
						saveList.destroy();
					}
					return res;
				});
				
				return this._ioOut(ioArgsArr).addCallback(washNextPile);
			}));
			
			this._ioOutQueue.add(ioReadyDfd);
		},
		revert: function(){
			for(var pre in this._handlers) {
				this.revertHandler(this._handlers[pre]);
			}
		},
		revertHandler: function(/*String*/ handlerUri) {
			var h = this._handlers[handlerUri];
			if(h) {
				var states = ["dirty", "deletedExpl", "new"];
				dojo.forEach(states, function(state){
					var list = this._getHandlerStateList(h, this.states[state]);
					dojo.forEach(list, function(item){
						this._revertHandlerItem(item, h);
					}, this);
				}, this);
			}
		},
		revertItem: function(/*item*/ item) {
			var h = this._getHandlerByItem(item);
			if(h) {
				this._revertHandlerItem(item, h);
			}
		},
		_revertHandlerItem: function(item, handler) {
			if(this._hasState(item, this.states["new"])) {
				this._removeState(item, handler, this.states["new"]);
				handler.uninitialize(item);
			}
			else {
				this._removeState(item, handler, this.states["dirty"]);
				this._removeState(item, handler, this.states["deletedExpl"]);
				handler.restore(item);
			}
		},
		_changeState: function(/* item */ item, /*object*/ handler, /*Number*/ state ) {
			if(this._hasState(item, state)) return;
			switch(state) {
				case this.states["new"]:
					// create dependencies are managed at save time
					if(this._hasState(item, this.states["dirty"]) || this._hasState(item, this.states["deleted"])) return;	// don't change state
					this._addState(item, handler, state);
					return;
				case this.states["dirty"]:
					if(this._hasState(item, this.states["new"]) || this._hasState(item, this.states["deleted"])) return;	// don't change state
					this._addState(item, handler, state);
					break;
				case this.states["deletedExpl"]:
				case this.states["deletedImpl"]:
					// delete dependencies are managed immediately on the client
					this._addState(item, handler, state);
					state = this.states["deleted"];
				case this.states["deleted"]:
					this._removeState(item, handler, this.states["dirty"]);
					this._removeState(item, handler, this.states["new"]);
					this._addState(item, handler, state);
					var targList = this._depMgr.getTargetsByType(this.getIdentity(item), com.ibm.data.DependencyManager.types["delete"]);
					dojo.forEach(targList, function(targetId){
						var targetItem = this.byId(targetId);
						this._changeState(targetItem, this._getHandlerByItem(targetItem), this.states["deletedImpl"]);
					}, this);
					break;
				case this.states["loaded"]:
				default:
					this._addState(item, handler, state);
			}
		},
		_removeState: function(/* item */ item, /*object*/ handler, /*Number*/ state, /*Boolean?*/ skipList ) {
			if(!this._hasState(item, state)) return;
			delete item._$.states[state];
			if(!skipList) {
				var list = this._getHandlerStateList(handler, state);
				if(list) {
					for(var i = 0; i < list.length; i++) {
						if(list[i] == item) list.splice(i, 1);
						break;
					}
				}
			}
			switch(state) {
				case this.states["deleted"]:
					var targList = this._depMgr.getTargetsByType(this.getIdentity(item), com.ibm.data.DependencyManager.types["delete"]);
					dojo.forEach(targList, function(targetItem){
						this._removeState(targetItem, this._getHandlerByItem(targetItem), this.states["deletedImpl"]);
					}, this);
					break;
				case this.states["deletedImpl"]:
					if(!this._hasState(item, this.states["deletedExpl"])) {
						this._removeState(item, handler, "deleted");
					}
					break;
				case this.states["deletedExpl"]:
					if(!this._hasState(item, this.states["deletedImpl"])) {
						this._removeState(item, handler, "deleted");
					}
					break;
				default:					
			}
		},
		_addState: function(/* item */ item, /*object*/ handler, /*Number*/ state) {
			if(this._hasState(item, state)) return;
			item._$.states[state] = true;
			var list = this._getHandlerStateList(handler, state);
			if(list) list.push(item);
		},
		_hasState: function(/* item */ item, /*String*/ state) {
			if(item._$.states[state]) return true;
			return false;
		},
		_getHandlerStateList: function(/*PocHandler*/ handler, /*Number*/ state) {
			switch(state) {
				case this.states["new"]:
				case this.states["dirty"]:
				case this.states["deletedExpl"]:
					return handler._$.stateLists[state];
					break;
				default: return null;
			}
		},
		_getUnSavedStateList: function(/*Number?*/ state) {
			// summary: Gets an array of items in this store which belong to a given state
			//		that have not already been processed for saving.
			var arr = [];
			for(var scheme in this._handlers) {
				var h = this._handlers[scheme];
				if(state != null) {
					arr = arr.concat(this._getHandlerStateList(h, state));
				}
				else {
					dojo.forEach(["new", "dirty", "deletedExpl"], function(st){
						var list = this._getHandlerStateList(h, this.states[st]);
						// for loop cleans up state lists in case something before hasn't
						for(var i = 0; i < list.length; i++) {
							if(!this._hasState(list[i]), this.states[st]) {
								list.splice(i, 1);
							}
						}
						arr = arr.concat(list);
					}, this);
				}
			}
			return dojo.filter(arr, function(item){
				return !this._hasState(item, this.states["saving"]);
			}, this);
		},
		isDirty: function(/* item? */ item){
			if(item && this._assertItem(item)) {
				return this._hasState(item, this.states["dirty"]) || 
						this._hasState(item, this.states["new"]) ||
						this._hasState(item, this.states["deleted"]);
			}
			else {
				for(var scheme in this._handlers) {
					var h = this._handlers[scheme];
					if(this._getHandlerStateList(h, this.states["dirty"]).length > 0 ||
						this._getHandlerStateList(h, this.states["deleted"]).length > 0 ||
						this._getHandlerStateList(h, this.states["new"]).length > 0) {
						return true;
					}
				}
				return false;
			}
		},
		
		/**************************************/
		/**                                  **/
		/**                                  **/
		/** com.ibm.data.api.ServerGenerated **/
		/**                                  **/
		/**                                  **/
		/**************************************/
		
		isCreated: function(/*item*/ item) {
			// summary: Indicates whether or not the item has been created on the server and
			//		the server-generated values were properly applied to the client-side
			//		object.	NOTE: Any client-side modifications	to the values of an item for 
			// 		which isCreated returns true can potentially modify the server-generated 
			// 		values to be out-of-sync with the server.
			// 		Therefore, it's important to note that isCreated merely indicates whether
			//		or not the client-side item has a server-side resource associated with it,
			// 		and any values that the server generated for the resource (such as a unique
			// 		id) were applied to the client-side item object when it was saved to the server.
			this._assertItem(item);
			return !this._hasState(item, this.states["new"]);
		},
		_fireCreated: function(item) {
			if(item._$.onCreateDefQueue) {
				item._$.onCreateDefQueue.fire(item);
				item._$.onCreateDefQueue.destroy();
				delete item._$.onCreateDefQueue;
			}
		},
		addOnCreate: function(/*Item*/ item, /*Function*/ callback) {
			// summary: Register callback to be called when the given item is created on the server.
			//		If the item is already created, this will call the callback immediately.
			this._assertItem(item);
			if(!item._$.onCreateDefQueue) {
				// doesn't have a queue, meaning this was never created by this store but was
				// loaded from the server
				callback(item);
			}
			else {
				item._$.onCreateDefQueue.add((new dojo.Deferred()).addCallback(callback));
			};
		},
		getUniqueCId: function() {
			return "cid:" + this._cidCntr++;
		},
		registerCId: function(/*String*/ cid, /*item*/ item) {
			this._cidMap[cid] = item;
			item._$.cid = cid;
			this.addOnCreate(item, dojo.hitch(this, function(item){
				//console.info("Resetting CID to real ID for %o", item);
				var id = this.getIdentity(item);
				this._toCache(id, item);
				this._depMgr.updateDependencyId(cid, id);
			}));
		},
		getCId: function(item) {
			this._assertItem(item);
			return item._$.cid;
		},
		byCId: function(/*String*/ cid) {
			return this._cidMap[cid];
		},
		seedStore: function(/* Object */ keywordArgs, /*Object?*/ parentInfo) {
			// summary: Seeds the store with a pre-existing item using the same
			// 		signature as the newItem function, but without instructing
			// 		the store to actually create the item on the server since the
			// 		item already exists.
			// description: This function allows the data store to be seeded
			// 		with items that are already known in detail through some
			// 		other mechanism.  It allows the store to make use of those
			// 		items without having to request them, particularly if their
			// 		data is already known, resulting in better performance and
			// 		a method to bootstrap the store with a set of pre-existing 
			// 		items.
			// keywordArgs:  A javascript object defining the initial content 
			// 		of the item as a set of JavaScript 'property name: value' 
			// 		pairs.
			// parentInfo: An optional javascript object defining what item is 
			// 		the parent of this item (in a hierarchical store.  Not all 
			// 		stores do hierarchical items), and what attribute of that parent 
			// 		to assign the new item to.  If this is present, and the attribute 
			// 		specified is a multi-valued attribute, it will append this item 
			// 		into the array of values for that attribute.  The structure	of 
			// 		the object is as follows:
			//		{
			//			parent: someItem,
			//			attribute: "attribute-name-string"
			//		}
			// returns: Item the generated item in this data store
			if(keywordArgs.id && !keywordArgs.pocType) keywordArgs.pocType = this._getPocTypeFromUri(keywordArgs.id);
			var h = this._getHandlerByPocType(keywordArgs.pocType);
			var item = this._createItemFromProps(keywordArgs, parentInfo, h);
			this._toCache(h.getIdentity(item), item);
			return item;
		},
		
		/********************************/
		/**                            **/
		/**                            **/
		/** dojo.data.api.Notification **/
		/**                            **/
		/**                            **/
		/********************************/
		onDelete: function(/* Item */ deletedItem) {
			// summary: See dojo.data.api.Notification.onDelete()
			return deletedItem;
		},

		onNew: function(/* Item */ newItem, /* Object */ parentInfo) {
			// summary: See dojo.data.api.Notification.onNew()
			return {newItem: newItem, parentInfo: parentInfo};
		},

		onSet: function(/* Item */ item, /*String*/ attribute, /*Object | Array*/ oldValue, /*Object | Array*/ newValue) {
			// summary: See dojo.data.api.Notification.onSet()
			return {item: item, attribute: attribute, oldValue: oldValue, newValue: newValue};
		}
	}
);

dojo.declare("com.ibm.data.SaveList",
	null,
	{
		store: null,
		list: null,
		
		constructor: function(args) {
			this.list = [];
			dojo.mixin(this, args);
		},
		addItem: function(item) {
			var id = this.store.getIdentity(item);
			if(!this.list[id]) {
				var inf = [item, null];
				this.list.push(inf);
				this.list[id] = inf;
			}
		},
		removeItem: function(item) {
			var id = this.store.getIdentity(item);
			if(this.list[id]) {
				delete this.list[id];
				for(var i=0, inf; inf=this.list[i], i<this.list.length; i++) {
					if(this.store.getIdentity(inf[0]) == id) {
						this.list.splice(i, 1);
						return;
					}
				}
			}
		},
		_getInf: function(item) {
			// summary: Finds and returns the information associated with the item 
			// 		argument if	it exists.
			if(this.store.isItem(item)) {
				var id = this.store.getIdentity(item);
				if(this.list[id]) return this.list[id];
				else return this.list[this.store.getCId(item)];
			}
			else {
				// for deleted items that are no longer recognized in
				// the store and may have no information whatsoever left on them
				for(var i=0; i<this.list.length; i++) {
					if(item == this.list[i][0]) return this.list[i];
				}
				return null;
			}
		},
		hasItem: function(item){
			// summary: Returns boolean true/false indicating whether or not
			//		the given item argument exists in this SaveList.
			// item: Item
			return this._getInf(item) != null;
		},
		getData: function(item) {
			// summary: Gets the associated data with the item argument, if it
			//		exists.
			// item: Item
			return this._getInfResource(item, 2);
		},
		setData: function(item, data) {
			// summary: Associates data with the item argument in this SaveList.
			// item: Item
			// data: Anything
			this._setInfResource(item, data, 2);
		},
		getDataSet: function(/*Array?*/ items) {
			return this._getInfSet(items, 2);
		},
		getError: function(item) {
			// summary: Gets an error associated with an item in this SaveList,
			//		returning null if none exists.
			// item: Item
			return this._getInfResource(item, 1);
		},
		setError: function(item, err) {
			// summary: Associates an error with an item in this SaveList for 
			//		later retrieval.
			// item: Item
			// err: Error
			this._setInfResource(item, err, 1);
		},
		getErrors: function(/*Array?*/ items) {
			// summary: Gets an array of errors from this SaveList, if any exist.
			//		Takes an optional argument of an array of items to filter by
			//		so that the errors retrieved are only from among the items
			//		specified in the argument array.  If the argument is null,
			//		this returns all errors that exist in this SaveList.
			return this._getInfSet(items, 1);
		},
		_setInfResource: function(item, res, type) {
			// summary: Associates a resource with the item by its type.
			var inf = this._getInf(item);
			if(inf) inf[type] = res;
		},
		_getInfResource: function(item, type) {
			// summary: Gets a resource associated with the item by its type.
			var inf = this._getInf(item);
			return inf ? inf[type] : null;
		},
		_getInfSet: function(/*Array?*/ items, /*Number*/ type) {
			var set = [], method = (type == 1 ? "getError" : "getData");
			if(items) {
				dojo.forEach(items, function(item){
					var res = this[method](item);
					if(res) set.push(err);
				}, this);
			}
			else {
				dojo.forEach(this.list, function(inf) {
					if(inf[type]) {
						set.push(inf[type]);
					}
				}, this);
			}
			return set;
		},
		destroy: function() {
			// summary: Destroys this SaveList, making it unusable by subsequent callers,
			//		freeing up any memory resources it consumed.
			delete this.list;
			delete this.store;
		}
	}
);

}
