/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.data.api.POCHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.api.POCHandler"] = true;
/*
 * @author Ben
 */ 
dojo.provide("com.ibm.data.api.POCHandler");

dojo.require("com.ibm.data.Resolver");

dojo.declare("com.ibm.data.api.POCHandler",
	null,
	{
		// definition: Object
		// 		Defines what this POC handler can do as
		// 		well as the format of the data that this POC Handler parses and
		//		expects as raw data to operate on.  This corresponds directly to the
		//		possible values of the handleAs property of dojo.__XhrArgs objects.
		//		Example -
		//		{
		//			scheme: "scheme",
		//			handleAs: "text",
		//			features: {
		// 				"ForeignReferences": true,
		//				"TreeModeled": true
		// 			}
		//		}
		definition: {
			scheme: "scheme",
			handleAs: "text",
			features: {}
		},
		
		// resolver: ResolverStore object to which this POC handler is registered.
		//		POC handlers should set this in their constructor.
		resolver: null,
		
		// depMgr: DependencyManager which is used by the ResolverStore to manage
		//		dependencies between items in the store (which can be inter- and intra-handler).
		//		POC handlers should set this in their constructor.
		depMgr: null,
		
		// summary: Takes two arguments to initialize the POC handler.
		// resolverStore: ResolverStore should be set to this POC handler's "resolver" property.
		// dependencyManager: DependencyManager should be set to this POC handler's "depMgr" property.
		constructor: function(resolverStore, dependencyManager) {
			this.resolver = resolverStore;
			this.depMgr = dependencyManager;
		},

		/********************************/
		/**                            **/
		/**                            **/
		/**     dojo.data.api.Read     **/
		/**                            **/
		/**                            **/
		/********************************/
		
		/*
		 * All dojo.data.api.Read APIs are required except for the "close", 
		 * "fetch", "load", "isItemLoaded", or "loadItem" API.
		 * The Resolver store provides all the io logic as well as item stubbing
		 * functionality, therefore the semantics have changed for POC handlers
		 * and different functions are to be implemented instead (see below).
		 */
		getValue: function(	/* item */ item, 
							/* attribute-name-string */ attribute, 
							/* value? */ defaultValue){
			
		},
		getValues: function(/* item */ item,
							/* attribute-name-string */ attribute){
			
		},
		getAttributes: function(/* item */ item){
			
		},
		hasAttribute: function(	/* item */ item,
								/* attribute-name-string */ attribute){
			
		},
		containsValue: function(/* item */ item,
								/* attribute-name-string */ attribute, 
								/* anything */ value){
			
		},
		getLabel: function(/* item */ item){
			
		},
		getLabelAttributes: function(/* item */ item){
			
		},
		
		/********************************/
		/**                            **/
		/**                            **/
		/**   dojo.data.api.Identity   **/
		/**                            **/
		/**                            **/
		/********************************/	
		
		/*
		 * All dojo.data.api.Identity APIs are required except for the "fetchByIdentity" API.
		 * Since the Resolver store provides all the io logic, POC handlers 
		 * need not implement any fetch functions.
		 */
		getIdentity: function(/* item */ item){
			
		},
		getIdentityAttributes: function(/* item */ item){
			
		},
		
		/********************************/
		/**                            **/
		/**                            **/
		/**    dojo.data.api.Write     **/
		/**                            **/
		/**                            **/
		/********************************/
		
		/*
		 * All dojo.data.api.Write APIs are required except for the "save" API.
		 * Since the Resolver store provides all the save logic, POC handlers 
		 * need not implement a save function.  Instead the semantics have changed
		 * slightly and been replaced by the "persist" API specified below.
		 */
		
		newItem: function(/* Object? */ keywordArgs, /*Object?*/ parentInfo){
			
		},
		deleteItem: function(/* item */ item){
			
		},
		setValue: function(	/* item */ item, 
							/* string */ attribute,
							/* almost anything */ value){

		},	
		setValues: function(/* item */ item,
							/* string */ attribute, 
							/* array */ values){
			
		},	
		unsetAttribute: function(	/* item */ item, 
									/* string */ attribute){
			
		},		
		
		/*********************************/
		/**                             **/
		/**                             **/
		/** com.ibm.data.api.POCHandler **/
		/**                             **/
		/**                             **/
		/*********************************/
		
		/*
		 * The com.ibm.data.api.POCHandler APIs are all required for POC handlers to implement.
		 * They provide hooks for the Resolver store to interact with a POC handler in order
		 * to deserialize and parse raw data, serialize items to request args in order to save
		 * to the server, client-side persistence of pending changes (in conjunction with the
		 * "revert" API), and a simple "define" function used to describe the POC handler. 
		 */
		backup: function(/*item*/ item) {
			// summary: Creates a single backup of the item that can be restored via the "restore" function
			//		and allows two phase write support (write-commit) for the POC Handler.
			//		If this is called multiple times before a "persist" call, only the first backup is 
			//		maintained.
		},
		restore: function(/*item*/ item) {
			// summary: Restores the backup, if one exists, of the current item so that the item appears in
			//		the same state it did when "backup" was first called on this item.  After "persist" is
			//		called on an item that was backed up, the backup is deleted, making restoration impossible.
		},
		persist: function(/*item*/ item) {
			// summary: Performs a client-side persistence of pending changes to the item
			// 		for this POC Handler.  This is called by the save operation iff after the 
			// 		server indicates a successful change.  It is NOT called when the save operation 
			// 		for these items begins on the client or	if an error from the server is received.
			//		Backups cannot be restored once this is called.
		},
		modifyFetchQuery: function(/*Object*/ query) {
			// summary: Extension point where a POC handler can modify the IO arguments used in fetch
			//		requests against this particular POC handler through the Resolver store.
			//		This allows the POC handler to set any default values or override any values
			//		and then return the modified object to the Resolver for processing.
			//		It should be noted that the Resolver store has final authority on any arguments
			//		that the POC handler may modify, and hence may modify them again.
			// query: Object map of the query to serialize as a query string in the URL
		},
		processFetch: function(/*Object*/ data, /*Request*/ request) {
			// summary: Processes raw unparsed data to extract items and meta data for the
			//		fetch call.
			// data: Object data to parse which is already in the format that this POC handler
			//		expects based on the 
			
		},
		processSave: function(/*Item*/ item) {
			// summary: Accepts an item which is managed by this POC handler.
			//		Returns an io args object used by the resolver store to send a update
			//		request to the server for this particular item.  The io args should 
			// 		include the data to send to the server the represents a serialized version
			// 		of the item including any header information associated	with the item's save 
			// 		operation.  The resolver store will take care of determining the HTTP method 
			// 		to use (either PUT or POST) based on whether or not the item is being created 
			// 		or simply updated.
			// item: Item to save
		},
		createStub: function(/*Object*/ keywordArgs) {
			// summary: Creates a stub item from the keywordArgs where the only guaranteed property
			//		on the argument object is the id of the actual resource on the server.  Any other
			// 		properties on the argument object are extra information which may be known a-priori
			// 		by the caller and provided here ahead of load-time.  Stub items created by this method 
			// 		must be recognized by the POC handler as valid items, but they aren't fully loaded yet.
			// 		This function is used to set up the item object and any properties on the item which 
			// 		are used by the POC	handler as meta or other bookkeeping information.
			// keywordArgs: Object map of properties used to define the item
		},
		uninitialize: function(/*Item*/ item) {
			// summary: Uninitializes an item and frees up any memory or listeners this item is holding.
			//		Called on successful delete of an item or on delete of a pending created item.
		},
		destroy: function() {
			// summary: Destroys this handler and frees up any memory to listeners or references to
			//		items that are unique or created by this handler.  Care should be taken not
			//		to destroy things here that may need to live beyond this handler's life (such as
			//		items created by this handler).
		}
	}
);

// To register a POC handler with the Resolver framework, 
// call the com.ibm.data.Resolver.register() function.
//
// com.ibm.data.Resolver.register("myPOCPrefix", com.ibm.data.api.POCHandler);

}
