/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.data.resolver.QueryRulePocHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.resolver.QueryRulePocHandler"] = true;
/*
 * @author Ben
 */ 
dojo.provide("com.ibm.data.resolver.QueryRulePocHandler");

dojo.require("com.ibm.query.QueryRuleEngine");

/* 
 * An abstract module which implements the POC handler's dojo.data APIs using an implementation of
 * the com.ibm.query.QueryRuleEngine module.
 * 
 * Modules should extend this and provide an instance of a query engine that can be used 
 * to query the data they use in their particular format.  Extensions should also provide
 * implementations for functions in this module that are simply stubs.
 */

dojo.declare("com.ibm.data.resolver.QueryRulePocHandler",
	null,
	{
		// definition: Object
		//		Defines this POC handler's capabilities and metadata.  Returned by the define API function.
		definition: {
			scheme: "",
			handleAs: "text",
			features: {}
		},
		
		// queryRules: Object
		//		Qurey rules which map directly to attributes of items from this POC
		//		handler.  Attributes are keys and values correspond to query rules
		//		in the particular format expected by the implementation of the
		//		QueryRuleEngine module that extensions provide.  All functions inside
		// 		query rules which are called by the engine are called in context
		//		of the engine.
		queryRules: {},
		
		// bodyQueryRules: Object
		//		Query rules used to extract meta data from request bodies but are
		//		not attributes on items from this POC handler
		bodyQueryRules: {},
		
		// customAttributes: Object
		// 		Supports custom attributes that this handler supports
		// 		that are not part of the queryRules framework.  When getting
		//		and setting attributes of items in this handler, the handler
		//		first looks up a rule that matches the attribute in the queryRules
		//		map.  If none is found, it then looks for a rule in this
		//		customAttributes map.  If a rule is found in the customAttributes
		//		map, then the rule is parameterized and the parameters are
		//		passed to the appropriate get/set function in the rule.  The
		//		function is called in context of this POC handler.
		// 		Map of keys, which are attribute names, whose corresponding 
		// 		values are objects with two properties like this:
		//		{
		//			// get supports get operations; multiValued indicates if
		// 			// getValues was called 
		//			get: function(item, multiValued, parameters) {},
		//			// set supports set operations; multiValued indicates if
		// 			// setValues was called
		//			set: function(item, value, multiValued, parameters) {}
		//		}
		//		
		customAttributes: {},
		
		// queryEngineClass: String
		//		Fully-qualified name of the javascript object constructor used
		//		to create an instance of a query rule engine used by this POC
		//		handler.
		queryEngineClass: "com.ibm.query.QueryRuleEngine", 
		
		// types: Object
		//		Map used to instantiate the query engine with in order to pass
		//		custom types for conversion to and from Strings during query
		//		rule processing for get and set operations.
		types: {
			"Item": {
				to: function(value) {
					if(value) return this.resolver.byId(value);
					return null;
				},
				from: function(value) {
					if(this.resolver.isItem(value)) {
						return this.resolver.getIdentity(value);
					}
					else return value;
				}
			},
			"decodedURI": {
				to: function(value) {
					// change from encoded string to decoded string
					return decodeURI(value);
				},
				from: function(value) {
					// change from decoded string to encoded string
					return encodeURI(value);
				}
			}
		},
		
		constructor: function(resolverStore, dependencyManager) {
			this.resolver = resolverStore;
			this.depMgr = dependencyManager;
			this.engine = this.buildEngine();
		},
		
		buildEngine: function() {
			// summary: Creates and returns an instance of the query rule engine
			//		used by this POC handler for querying data in items and response
			//		data.  Called by the constructor once during instantiation of
			//		this POC handler.  Can be overridden by extensions to provide
			//		additional parameters to the engine.  For convenience, the engine's
			// 		context is set to this POC handler so that delegated functions that
			// 		the engine calls are called in context of this handler.
			// returns: Object instance of a query rule engine.
			var cls = dojo.getObject(this.queryEngineClass);
			return new cls({queryRules: this.queryRules, types: this.types, resolver: this.resolver, handler: this});
		},
		
		/********************************/
		/**                            **/
		/**                            **/
		/**     dojo.data.api.Read     **/
		/**                            **/
		/**                            **/
		/********************************/
		_getValue: function(	/* item */ item, 
							/* attribute-name-string */ attribute, 
							/* value? */ defaultValue,
							/* Boolean */ multiValued) {
			if(item._unsetAttrs[attribute]) return defaultValue;
			var rule = this.engine.getQuery(attribute, this.customAttributes, multiValued);
			if(rule) {
				return this.customAttributes[attribute].get.call(this, item, multiValued, rule.parameters);
			}
			else if(item.data) {
				rule = this.engine.getQuery(attribute, this.queryRules, multiValued);
				if(!rule) return defaultValue;
				var res = this.engine.getByRule(item.data, rule, multiValued);
				if(!res.found) {
					if(item._backupData) {
						res = this.engine.getByRule(item._backupData, rule, multiValued);
						if(res.found) return res.value;
					}
					return defaultValue;
				}
				return res.value;
			}
			return defaultValue;
		},
		
		getValue: function(	/* item */ item, 
							/* attribute-name-string */ attribute, 
							/* value? */ defaultValue){
			return this._getValue(item, attribute, defaultValue, false);
		},
		getValues: function(/* item */ item,
							/* attribute-name-string */ attribute){
			return this._getValue(item, attribute, [], true);
		},
		getAttributes: function(/* item */ item){
			var attrs = [];
			for(var x in this.queryRules) {
				attrs.push(x);
			}
			for(var y in this.customAttributes) {
				attrs.push(y);
			}
			return attrs;
		},
		hasAttribute: function(	/* item */ item,
								/* attribute-name-string */ attribute){
			if(item._unsetAttrs[attribute]) return false;
			var rule = this.engine.getQuery(attribute, this.queryRules);
			if(!rule) {
				rule = this.engine.getQuery(attribute, this.customAttributes);
				if(rule) {
					return this.customAttributes[attribute].get.call(this, item, false, rule.parameters) !== undefined;
				}
				return false;
			}
			var res = this.engine.getByRule(item.data, rule);
			if(!res.found) {
				return false;
			}
			return true;
		},
		containsValue: function(/* item */ item,
								/* attribute-name-string */ attribute, 
								/* anything */ value){
			return dojo.some(this.getValues(item, attribute), function(val){
				if(val == value) return true;
				return false;
			}, this);
		},
		getLabel: function(/* item */ item){
			var arr = this.getLabelAttributes(item);
			var str = "";
			for(var i=0; i<arr.length; i++) {
				str += this.getValue(item, arr[i]);
			}
			return str;
		},
		getLabelAttributes: function(/* item */ item){
			
		},
		
		/********************************/
		/**                            **/
		/**                            **/
		/**   dojo.data.api.Identity   **/
		/**                            **/
		/**                            **/
		/********************************/	
		getIdentity: function(/* item */ item){
			var arr = this.getIdentityAttributes(item);
			var str = "";
			for(var i=0; i<arr.length; i++) {
				str += this.getValue(item, arr[i]);
			}
			return str;
		},
		getIdentityAttributes: function(/* item */ item){
			
		},
		
		/********************************/
		/**                            **/
		/**                            **/
		/**    dojo.data.api.Write     **/
		/**                            **/
		/**                            **/
		/********************************/
		newItem: function(/* Object? */ keywordArgs, /*Object?*/ parentInfo){
			
		},
		deleteItem: function(/* item */ item){
			
		},
		_setValue: function(	/* item */ item, 
							/* string */ attribute,
							/* almost anything */ value,
							/* Boolean */ multiValued){
			var res = false,
				rule = this.engine.getQuery(attribute, this.customAttributes, multiValued);
			if(rule) {
				res = this.customAttributes[attribute].set.call(this, item, value, multiValued, rule.parameters);
			}
			else if(item.data) {
				rule = this.engine.getQuery(attribute, this.queryRules, multiValued);
				if(!rule) return false;
				res = this.engine.setByRule(item.data, rule, value, multiValued);
			}
			if(res) {
				delete item._unsetAttrs[attribute];
			}
			return res;
		},
		setValue: function(	/* item */ item, 
							/* string */ attribute,
							/* almost anything */ value){
			return this._setValue(item, attribute, value, false);
		},	
		setValues: function(/* item */ item,
							/* string */ attribute, 
							/* array */ values){
			return this._setValue(item, attribute, values, true);
		},	
		unsetAttribute: function(	/* item */ item, 
									/* string */ attribute){
			var rule = this.engine.getQuery(attribute, this.queryRules);
			if(!rule) {
				rule = this.engine.getQuery(attribute, this.customAttributes);
				return false;
			}
			var res = this.engine.unsetByRule(item.data, rule);
			if(res) {
				item._unsetAttrs[attribute] = true;
			}
			return res;
		},
		backup: function(item) {
			// summary: Creates a single backup copy of this item on the client for
			//		later restoration if necessary.  Calling this function multiple
			//		times before _restore() or _persist() is called on this item will do nothing
			//		after the initial backup, effectively maintaining only the first
			//		version of the item before any changes were made.
			//		Override this to actually backup the data.
			if(!item._backupData) {
				
			}
			return false;
		},
		restore: function(item) {
			// summary: Restores the single backup copy of this item on the client,
			//		erasing all modifications made since backup() was called.
			if(item._backupData) {
				
			}
			return false;
		},
		persist: function(item) {
			// summary: Persists the changes made to this item since the backup() was
			// 		called.  After this is called, there is no way to restore the backup
			//		as it is erased completely.
			delete item._backupData;
		},
		modifyFetchQuery: function(query) {
			return query;
		},
		dataToItem: function(/*Object*/ data) {
			var item = this.createStub({});
			item.data = data;
			return item;
		},
		processFetch: function(/*Object*/ data, /*Request*/ request) {
			// return items and metadata about the data (like stuff in the feed, not merely
			//		the entries)
		},
		processSave: function(/*Item*/ item) {
			this.updateReferences(item);
		},
		updateReferences: function(item) {
			// summary: Called by the processSave function immediately before the data for this item is
			//		actually serialized out for the ioArgs.  This allows the POC handler to update any
			//		data in the item that may have changed due to dependencies being resolved.  One example
			//		is when the item references another item that did not have a real id but was using
			//		a client id instead.  At the point when this is called, the referenced item should have
			//		its real id assigned to it but still be accessible by its client id using the resolver's
			//		byId or byCId functions.
		},
		createStub: function(/*Object*/ keywordArgs) {
			return {
				data: keywordArgs.data,
				id: keywordArgs.id,
				_unsetAttrs: {},
				_writeLog: [],
				_cachedMeta: {}	// for caching complex objects
			}
		},
		uninitialize: function(/*Item*/ item) {
			// summary: Uninitializes an item and frees up any memory or listeners this item is holding.
			//		Called on successful delete of an item or on delete of a pending created item.
			delete item.id;
			delete item._unsetAttrs;
			delete item._cachedMeta;
			delete item._backupData;
			delete item._writeLog;
			delete item.data;
		},
		destroy: function() {
			delete this.resolver;
			delete this.depMgr;
			delete this.engine;
		},
		_makeModifiable: function(item) {
			// summary: Modifies the data of this item to become modifiable
			//		for writing back to the server.  Subclasses can override this
			//		to do things like prune an item to only include necessary information
			//		for updates and nothing more.  This is called by backup once after
			//		backing up the original element first.  After this returns,
			//		this item's element may have been modified.  Any queries against
			//		this item should now include logic to check the backup copy as well
			//		if a query doesn't return anything as it could have been deleted by
			//		this function.  Care should be taken to ensure that the query result
			//		specifically detects if nothing was found or if the actual value was
			//		null or empty.
		},
		toString: function() {
			return "[" + this.declaredClass + " - " + this.definition.scheme + "]";
		}
	}
);

}
