/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.data.resolver.tree.PocTreeModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.resolver.tree.PocTreeModel"] = true;
/*
 * @author Ben
 */ 
dojo.provide("com.ibm.data.resolver.tree.PocTreeModel");

dojo.declare("com.ibm.data.resolver.tree.PocTreeModel",
	null,
	{
		root: null,
		resolver: null,
		handler: null,
		deferWrites: true,
		listening: true,
		
		constructor: function(args) {
			dojo.mixin(this, args);
			this._dirtyItems = {};
			this._allItems = {};
			this._ancLoaders = {};
			if(this.root) {
				this._allItems[this.getIdentity(this.root)] = this.root;
			}
			this._connections= [].concat([
				dojo.connect(this.resolver, "onNew", this, "_onNew"),
				dojo.connect(this.resolver, "onSet", this, "_onSet"),
				dojo.connect(this.resolver, "onDelete", this, "_onDelete")
			]);
			
			if(dojo.isIE) {
				dojo.addOnWindowUnload(dojo.hitch(this, function(){
					this.destroy();
				}));
			}
		},
		assertRelevance: function(item) {
			if(!this.inModel(item)) {
				throw new Error(this.resolver.fmt("errorNotInModel"));
			}
		},
		
		destroy: function(){
			// summary: destroys this object, releasing connections to the store
			 dojo.forEach(this._connections, dojo.disconnect);
			 delete this._connections;
			 delete this._allItems;
			 delete this._dirtyItems;
			 delete this._ancLoaders;
			 delete this.root;
		},
				
		// =======================================================================
		// Methods for traversing hierarchy
		
		getRoot: function(onItem, onError){
			// summary:
		    //            Calls onItem with the root item for the tree, possibly a fabricated item.
			//            Throws exception on error.
			if(this.store.isItemLoaded(this.root)) {
				onItem(this.root);
			}
			else {
				this.store.loadItem({
					item: this.root, 
					onItem: onItem,
					onError: onError
				});
			}
		},
		
		mayHaveChildren: function(/*dojo.data.Item*/ item){
	        // summary
	        //            Tells if an item has or may have children.  Implementing logic here
	        //            avoids showing +/- expando icon for nodes that we know don't have children.
	        //            (For efficiency reasons we may not want to check if an element actually
	        //            has children until user clicks the expando node)
	        return this.handler.getValue(item, "mayHaveChildren");
		},
		getChildren: function(/*dojo.data.Item*/ parentItem, /*Function(items)*/onComplete, /*Function(error)*/onError){
	        // summary
	        //           Calls onComplete() with array of child items of given parent item, all loaded.
	        //            Throws exception on error.
	        this.handler.getValue(parentItem, "children")
	        	.addOnLoad(dojo.hitch(this, function(items){
	        		dojo.forEach(items, function(item) {
	        			this._allItems[this.getIdentity(item)] = item;
	        		}, this);
		        	if(onComplete) {
		        		onComplete(this.filterChildren(parentItem, items.slice()));
		        	}
		        }))
		        .addOnError(dojo.hitch(this, function(error){
		        	if(onError) onError(error);
		        }));
		},
		
		filterChildren: function(parentItem, childItems) {
			return childItems;
		},
		
		getIdentity: function(/* item */ item){
	        // summary: returns identity for an item
	        return this.handler.getIdentity(item);
		},
		getLabel: function(/*dojo.data.Item*/ item){
	        // summary: get the label for an item
	        return this.handler.getLabel(item);
		},
		getAncestors: function(/*dojo.data.Item*/item, /*Function(items)*/onComplete, /*Function(error)*/onError) {
			// summary: Gets an array of ancestors of the current item which includes the item itself as the first
			//		element then walks up the tree to the root, which is the last element.
			this.assertRelevance(item);
			var id = this.getIdentity(item);
			if(this._ancLoaders[id]) {
				this._ancLoaders[id].addOnLoad(onComplete).addOnError(onError);
			}
			else {
				var parent = null, loadFirst = true, chain = [];
				while(this.resolver.isItemLoaded(item)) {
					this._allItems[this.getIdentity(item)] = item;
					chain.push(item);
					parent = this.getParent(item);
					if(!parent) break;
					item = parent;
					loadFirst = false;
				}
				if(!parent && onComplete && !loadFirst) {	// full chain
					onComplete(chain);
				}
				else {
					this._ancLoaders[id] = (new com.ibm.data.ItemsLoader({
						store: this.resolver,
						query: this.getAncestorQuery(item),
						filter: function(resultSet) {
							return chain.concat(resultSet);
						}
					})).addOnResult(dojo.hitch(this, function(items){
						delete this._ancLoaders[id];	// cleanup
						dojo.forEach(items, function(item) {
		        			this._allItems[this.getIdentity(item)] = item;
		        		}, this);
		        	})).addOnLoad(onComplete).addOnError(onError);
				}
			}
		},
		getAncestorQuery: function(/*dojo.data.Item*/item) {
			// summary: Returns a query object which can be passed to the resolver store fetch function to load
			//		the ancestors of a particular item in the tree model.
			return {
				uri: this.getIdentity(item),
				levels: "-all"
			};
		},
		getParent: function(/*dojo.data.Item*/ item) {
			// summary: returns the parent item of this item argument
			return this.handler.getValue(item, "parent");
		},
		
		// =======================================================================
		// Write interface
		newItem: function(/* Object? */ args, /*Item?*/ parent){
	        // summary
	        //            Creates a new item.   See dojo.data.api.Write for details on args.
	        var parentInfo = {parent: parent, attribute: "parent"};
	        var item = this.store.newItem(args, parentInfo);
	        this.store.save({onComplete: args.onComplete, onError: args.onError, scope: args.scope});
	        return item;
		},
		isCreated: function(/*item*/ item) {
			// summary: Indicates whether or not the item has been created on the server and
			//		the server-generated values were properly applied to the client-side
			//		object.	NOTE: Any client-side modifications	to the values of an item for 
			// 		which isCreated returns true can potentially modify the server-generated 
			// 		values to be out-of-sync with the server.
			// 		Therefore, it's important to note that isCreated merely indicates whether
			//		or not the client-side item has a server-side resource associated with it,
			// 		and any values that the server generated for the resource (such as a unique
			return this.resolver.isCreated(item);	
		},
		addOnCreate: function(/*item*/ item, /*Function*/ callback) {
			// summary: Register callback to be called when the given item is created on the server.
			//		If the item is already created, this will call the callback immediately.
			return this.resolver.addOnCreate(item, callback);			
		},
		pasteItem: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Boolean*/ bCopy, /*int|Item?*/ pos){
	        // summary:
	        //            Move an item from one parent item to another or to another position under the same parent.
	        //            Used in drag & drop.
	        // childItem: Item to move.
	        // oldParentItem: Previous parent of item being moved.
	        // newParentItem: Item that the item being moved was dropped on.
	        // bCopy: Boolean indicating if the item should be copied or not.
	        // pos: Number|Item indicating the position at which this item gets inserted into the
	        //		model.  If it is a number, it indicates the index in the list of children where
	        //		to insert the child.  If it's an item in the model, it indicates the next sibling
	        //		of the item that will be inserted.  If null, the item is added to the end of the
	        //		list of children.
	        
	        // turning off notifications while doing multiple updates... that way
	        // we can send one onChildrenChange event to all listeners with the proper
	        // list of children
	        var chId = this.getIdentity(childItem),
	        	oldPId = this.getIdentity(oldParentItem),
	        	newPId = this.getIdentity(newParentItem),
	        	listeningState = this.listening;
	        if(chId == newPId) {
	        	// dropped on ourself... do nothing... as if this is possible somehow
	        	return;
	        }
	        else {
	        	this._dirtyItems[chId] = childItem;
	        	if(oldPId != newPId) {
	        		this.listening = false;
	        		this.resolver.setValue(childItem, "parentId", newPId);
	        		this.listening = listeningState;
	        	}
	        	
	        	this.getChildren(newParentItem, dojo.hitch(this, function(items){
	        		// to manage the children on the client, we need to load them first...
	        		// usually this will already have been done before operations like pasteItem
	        		// are called on some parent items
	        		this.listening = false;
    				var nextSibling = null;
    				if(pos != null) {
    					// normalize pos to a number, and nextSibling to an item
		        		if(typeof(pos) == "number") {
		        			if(items.length < pos) {
		        				nextSibling = items[pos + 1];
		        			}
		        		}
		        		else if(this.resolver.isItem(pos)) {
		        			nextSibling = pos;
		        			var nextId = this.getIdentity(pos);
		        			pos = dojo.indexOf(items, function(item) {
		        				return this.getIdentity(item) == nextId;
		        			}, this);
		        			if(pos < 0) pos = items.length;
		        		}
		        		else pos = Infinity;	// land softly if pos is invalid
						if(nextSibling) {
							this.resolver.setValue(childItem, "next", nextSibling);
						}
		        	}
		        	else {
		        		this.resolver.unsetAttribute(childItem, "next");
		        	}
		        	
		        	if(!this.deferWrites) this.save();
    				
        			// reparenting involves removing from the old and inserting into the new
        			// moving within the same parent works the same way
	        		this._removeChildItem(oldParentItem, childItem);
	        		this._insertChildItem(newParentItem, childItem, pos);

					this.listening = listeningState;
		        	this.onTreeChange(childItem, oldParentItem, newParentItem, nextSibling);
		        	if(newParentItem) this.getChildren(newParentItem, dojo.hitch(this, "onChildrenChange", newParentItem));
		        	if(oldParentItem) this.getChildren(oldParentItem, dojo.hitch(this, "onChildrenChange", oldParentItem));
    			}));
	        }
		},
		
		save: function(args) {
			var arr = [];
			for(var id in this._dirtyItems) {
				arr.push(this._dirtyItems[id]);
				delete this._dirtyItems[id];
			}
			this.resolver.saveItems(arr, args);
		},
		
		// explicitly uses item._cachedMeta.children array in managing children of an item in a tree model
		// on the client
		_insertChildItem: function(/*Item*/parentItem, /*Item*/ item, /*Number*/ pos) {
			var items = parentItem._cachedMeta.children;
			if(!this.mayHaveChildren(parentItem)) {
				// we just inserted an item under another item that had no children before
				if(!items) {
					items = parentItem._cachedMeta.children = [];
				}
				items.loaded = true;;
			}
			if(!items || !items.loaded) return false;	// children haven't loaded yet, so we can't manage them on the client
			if(pos == null || pos >= items.length) {
				items.push(item);
			}
			else {
				items.splice(pos, 0, item);
			}
			this.handler.modifyChildren(items);
			return true;
		},
		_removeChildItem: function(/*Item*/parentItem, /*Item*/ item) {
			var items = parentItem._cachedMeta.children;
			if(!items || !items.loaded) {
				return false;	// children haven't loaded yet, so we can't manage them on the client
			}

			var id = this.getIdentity(item);
			for(var i = 0; i < items.length; i++) {
				if(this.getIdentity(items[i]) == id) {
					items.splice(i,1);	// remove the item from the array
					break;	// break out of the loop since we already found ourselves
				}
			}
			this.handler.modifyChildren(items);
			return true;
		},
		
		inModel: function(/*Item|String*/ obj) {
			return true;
		},
		
		loadedInModel: function(/*Item|String*/ obj) {
			if(!dojo.isString(obj)) {
				obj = this.getIdentity(obj);
			}
			return !!this._allItems[obj];
		},
		
		_onNew: function(/* Item */ newItem, /* Object */ parentInfo) {
			// summary: Callback handler for the store attached to this tree model to call
			//		when an item is created 
			if(!this.listening || !parentInfo || !parentInfo.item) {
				return;
			}
			if(this.handler != this.resolver._getHandlerByItem(newItem)) {
				return;
			}
			var parentItem = parentInfo.item;
			if(this.loadedInModel(parentItem)) {
				this._dirtyItems[this.handler.getIdentity(newItem)] = newItem;
				var nextSibling = this.handler.getValue(newItem, "next");
				this._insertChildItem(parentItem, newItem, nextSibling);
				this.onTreeChange(newItem, null, parentItem, nextSibling);
				this.getChildren(parentItem, dojo.hitch(this, "onChildrenChange", parentItem));
			}
		},
		
		_onDelete: function(/* Item */ deletedItem) {
			// summary: Callback handler for the store attached to this tree model to call
			//		when an item is deleted
			if(!this.listening) return;
			if(this.loadedInModel(deletedItem)) {
				this._dirtyItems[this.handler.getIdentity(deletedItem)] = deletedItem;
				var parentItem = this.getParent(deletedItem);
				this._removeChildItem(parentItem, deletedItem);
				this.onTreeChange(deletedItem, parentItem);
				//this.getChildren(parentItem, dojo.hitch(this, "onChildrenChange", parentItem));
				this.onDelete(deletedItem);
			}
		},
		
		_onSet: function(/* Item */ item, /*String*/ attribute, /*Object | Array*/ oldValue, /*Object | Array*/ newValue) {
			// summary: Callback handler for the store attached to this tree model to call
			//		when an item is modified
			if(!this.listening) return;
			if(this.loadedInModel(item)) {
				switch(attribute) {
					case "parentId":
						newValue = this.resolver.byId(newValue);
					case "parent":
						this.pasteItem(item, this.handler.getValue("parent"), newValue, false, this.handler.getValue("next"));
						break;
					case "nextId":
						newValue = this.resolver.byId(newValue);
					case "next":
						var oldParent = this.handler.getValue("parent")
						this.pasteItem(item, oldParent, oldParent, false, newValue);
						break;
					default:
						this.onChange(item);
				}
			}
		},
		
		onDelete: function(/*dojo.data.Item*/ item) {
			// summary:
			//		Callback whenever an item has been deleted from this model.
		},
		
		// =======================================================================
		// Callbacks
		
		onChange: function(/*dojo.data.Item*/ item){
			// summary:
			//		Callback whenever an item has changed, so that Tree
			//		can update the label, icon, etc.   Note that changes
			//		to an item's children or parent(s) will trigger an
			//		onChildrenChange() so you can ignore those changes here.
			// tags:
			//		callback
		},
		
		onChildrenChange: function(/*dojo.data.Item*/ parent, /*dojo.data.Item[]*/ newChildrenList){
	        // summary: Callback to do notifications about new or updated items under a parent.  Deletes
	        //		are handled by the onDelete notification.
		},
		
		onTreeChange: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Item*/ nextSiblingItem) {
			// summary: Called when the tree model is changed through one of three scenarios.
			// 		1) An item is moved in the tree model.  This can be called 
			//			when an item is reparented or reordered within the same parent.  In both
			//			cases the nextSiblingItem refers to the next sibling of childItem, sharing
			//			its same newParentItem.
			//		2) A new item is inserted into the tree at a given parent.  In this case,
			//			the oldParentItem is null.
			//		3) An item is removed from the tree at a given parent.  In this case, the
			//			newParentItem and the nextSiblingItem is null.
			// childItem: Item to move.
			// oldParentItem: Item old parent of the childItem.
			// newParentItem: Item new parent of the childItem.  If this is the same as the
			//		oldParentItem, it indicates that the childItem is being reordered in
			//		the list of children under the parent.
			// nextSiblingItem: Item that is becoming the next sibling of the moved child item.  
			// 		Null indicates moving to the end of the list of children. 
		},				
		toString: function() {
			return "[PocTreeModel of {" + this.handler + "}]";
		}
	}
);

}
