/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.aggregation.event.markup"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.event.markup"] = true;
dojo.provide( "com.ibm.portal.aggregation.event.markup" );

dojo.require( "dojo.dnd.common");

dojo.declare( "com.ibm.portal.aggregation.event.markup.PortletMarkupRewriter",
			  null,
			  {
			  	needsRewrite: function ( /*HTMLElement*/elem ) {
					// summary: Indicates whether or not this rewriter needs to handle this node.
					// elem: the current markup element
					// returns: true if the Rewriter can handle the element, false if it cannot
					dojo.unimplemented( "com.ibm.portal.aggregation.event.PortletMarkupRewriter.needsRewrite" );
				},
				rewrite: function ( /*HTMLElement*/elem ) {
					// summary: Rewrites the HTMLElement.
					// elem: the current markup element.
					dojo.unimplemented( "com.ibm.portal.aggregation.event.PortletMarkupRewriter.rewrite" );
				},
				selectNodes: function ( /*HTMLElement*/rootElement ) {
					// summary: Selects the nodes that need to be rewritten.
					// rootElement: the root node to search from
					// returns: an array of the nodes that need to be rewritten
					dojo.unimplemented( "com.ibm.portal.aggregation.event.PortletMarkupRewriter.selectNodes" );
				}
			  }
);

dojo.declare( "com.ibm.portal.aggregation.event.markup.PortletMarkupEventHandler",
			  null,
			  {
			  	// summary: Traverses any new portlet markup and gives each registered rewriter a
				//		chance to alter each node in the new markup individually.
				// description: Starts at the base portlet element and traverses each node below the
				//		root element, passing each node into the rewriters registered with this handler.
				
				constructor: function () {
				  	this._rewriters = new Array ();
					this._portletMarkupUpdatedHandle = dojo.subscribe( "/portal/PortletMarkup/updated",
											 	this,
												function ( /*Object*/args ) {
													var element = args[["newMarkup"]];
													this.process( element );
												} 
					);
			    },
				addRewriter: function( /*PortletMarkupRewriter*/rewriter) {
					// summary: Register a new rewriter to this handler.
					// rewriter: the rewriter to register.
					this._rewriters.push( rewriter );
				},
				process: function ( rootElement ) {
					// summary: Processes the updated portlet markup.
					// rootElement: the root markup element of the portlet markup.
					ibm.portal.debug.entry( "PortletMarkupEventHandler.process", [ rootElement ] );
					
					ibm.portal.debug.text( "Root element's html: " + rootElement.innerHTML );
					
					dojo.forEach( this._rewriters, 
  							      function ( rewriter ) {
										var elems = rewriter.selectNodes( rootElement );
									   	for ( var i = 0; i < elems.length; i++ ) {
										   	if ( rewriter.needsRewrite( elems[i] ) ) {
												rewriter.rewrite( elems[i] );
											}
										}		
								  }  
					);
						
					ibm.portal.debug.exit( "PortletMarkupEventHandler.process");	
				}
			  }  
);

dojo.declare( "com.ibm.portal.aggregation.event.markup.OnclickRewriter",
			  com.ibm.portal.aggregation.event.markup.PortletMarkupRewriter,
			  {
			  	// summary: Rewrites onclick events to call our PortletEventHandler first.
				// description: Overwrites all onclick events to call our PortletEventHandler first
				//		so that we get a chance to intercept those clicks in IE. The IE event model
				//		only allows for the bubbling phase of the W3C event model. So, in order to
				//		"intercept" clicks inside a portlet, we have to take over the onclick event
				//		at the lowest level possible in the DOM.
				
				needsRewrite: function ( node ) {
					// summary: Indicates whether or not this rewriter needs to handle this node.
					// elem: the current markup element
					// returns: true if the Rewriter can handle the element, false if it cannot
					ibm.portal.debug.entry( "OnclickRewriter.needsRewrite", [ node ] );
                    var retVal = (( node.nodeName == "a" ) || ( node.onclick != null ) ); 
					ibm.portal.debug.exit( "OnclickRewriter.needsRewrite", [ retVal ] );
					return retVal;
				},
				rewrite: function ( element ) {
					// summary: Rewrites the onclick event to point to the PortletEventHandler.
					// element: the current markup element
					ibm.portal.debug.entry( "OnclickRewriter.rewrite", [ element ] );	
				
					com.ibm.portal.utilities.overwriteProperty( element, "onclick", function () { 
							var restoreValue = null;                            
                            //Clear out the overwritten value for the onclick property so that the click 
                            //handler correctly handles any onclick handlers. We need to put it back in 
                            //case this handler is called multiple times.
                            if ( com.ibm.portal.utilities.getOverwrittenProperty(this, "onclick" ) != null ) { 
								restoreValue = com.ibm.portal.utilities.restoreProperty( this, "onclick" );
							} else { 
								restoreValue = this.onclick;
                                this.onclick = null; 
							}
                             
                            com.ibm.portal.aggregation.event.PORTLET_EVENT_HANDLER.handle();
                            
                            if ( restoreValue != null ) {
                                com.ibm.portal.utilities.overwriteProperty( this, "onclick", restoreValue );
                            }
					}, true );
					
					ibm.portal.debug.exit( "OnclickRewriter.rewrite" );
				},
				selectNodes: function ( /*HTMLElement*/rootElement ) {
					// summary: Selects the nodes that need to be rewritten.
					// rootElement: the root node to search from
					// returns: an array of the nodes that need to be rewritten
					return dojo.query( "[onclick]", rootElement );
				}		
			  } 
);

dojo.declare( "com.ibm.portal.aggregation.event.markup.OnloadRewriter",
			  com.ibm.portal.aggregation.event.markup.PortletMarkupRewriter,
			  {
			  	// summary: Rewrites onload event handlers to run when script handling is finished.
				// description: Overwrites all onload handlers in the DOM so that we can defer
				// 		running them until the portlet markup is updated and scripts have been run.
				
				constructor: function() {
					this._onloadHandlers = new Array();
				},
				needsRewrite: function ( node ) {
					// summary: Indicates whether or not this rewriter needs to handle this node.
					// elem: the current markup element
					// returns: true if the Rewriter can handle the element, false if it cannot
                    ibm.portal.debug.entry( "OnloadRewriter.needsRewrite", [ node ] );
                    var retVal = false;
                    try {
                    	retVal = ( node.nodeName.toUpperCase() != "OBJECT" && node.onload && node.onload != "" );
                    }
                    catch (e) {
                    	ibm.portal.debug.text( "Unable to check for onload! Assuming none used for this tag.");
                    }	 
					ibm.portal.debug.exit( "OnloadRewriter.needsRewrite", retVal );
					return retVal;
				},
				rewrite: function ( element ) {
					// summary: Rewrites the onload event handler to run when the portlet is ready.
					// element: the current markup element
					ibm.portal.debug.entry( "OnloadRewriter.rewrite", [ element ] );
					
					var nodeId = element.id;
					if(!nodeId || nodeId.length == 0) {
						nodeId = dojo.dnd.getUniqueId();
						element.setAttribute("id", nodeId);
					}
					
					this._onloadHandlers.push( {handler: element.onload, id: nodeId } );
					
					element._oldOnload = element.onload;
					element.setAttribute("onload", null);
					element.onload = null;

					ibm.portal.debug.exit( "OnloadRewriter.rewrite" );
				},
				process: function ( rootElement ) {
					// summary: Processes the updated portlet markup.
					// rootElement: the root markup element of the portlet markup.
					ibm.portal.debug.entry( "OnloadRewriter.process", [ rootElement ]);
					
					ibm.portal.debug.text( "Root element is: " + rootElement.nodeName );
					
					//var elems = com.ibm.portal.utilities.html.getElementsByTagNames(rootElement, "body", "frame", "frameset", "img", "link", "script");
					var elems = dojo.query( "body[onload]", rootElement);
					elems = elems.concat( dojo.query( "frame[onload]", rootElement ) );
					elems = elems.concat( dojo.query( "frameset[onload]", rootElement ) );
					elems = elems.concat( dojo.query( "img[onload]", rootElement ) );
					elems = elems.concat( dojo.query( "link[onload]", rootElement ) );
					elems = elems.concat( dojo.query( "script[onload]", rootElement ) );
					for( var i = 0; i < elems.length; i++ ) {
						if(!elems[i]) continue;
						//if(this.needsRewrite(elems[i])) {
							this.rewrite( elems[i] );
						//}
					}
					// replaced the below commented code since we don't need to walk the entire DOM, just certain elements
					/*
					// PK70387 for ( child in rootElement.childNodes ) {
					for ( var child = 0; child < rootElement.childNodes.length; child++ ) {
						var elem = rootElement.childNodes[child];
						
						ibm.portal.debug.text( "Examining chlid element: " + elem.nodeName );
						
						if ( !elem ) {
							continue;	
						}
						
					   	if ( this.needsRewrite( elem ) ) {
					   		this.rewrite( elem );
					   	}	
						
						
						if ( elem.childNodes && elem.childNodes.length > 0) {
							this.process( elem );
						}
					}
					*/	
					
					ibm.portal.debug.exit( "OnloadRewriter.process" );
				},
				runHandlers: function () {
					ibm.portal.debug.entry( "OnloadRewriter.runHandlers");
					for(var idx = 0; idx < this._onloadHandlers.length; idx++) {
						var node = dojo.byId(this._onloadHandlers[idx].id);
						if(node) {
							try {
								ibm.portal.debug.text( "OnloadRewriter.runHandlers " + this._onloadHandlers[idx].handler);
								if ( this._onloadHandlers[idx].handler.apply ) {
									this._onloadHandlers[idx].handler.apply( node );
									node.onload = this._onloadHandlers[idx].handler;
									node.setAttribute("onload", this._onloadHandlers[idx].handler);
								}	
							}
							catch(err) {
								// handle gracefully as if this were in its own thread
								ibm.portal.debug.text( "An error occurred during this execution: " + err.message );
							}
						}
					}
					ibm.portal.debug.exit( "OnloadRewriter.runHandlers");
				}
			  } 
);

com.ibm.portal.aggregation.event.markup.PORTLET_MARKUP_EVENT_HANDLER = new com.ibm.portal.aggregation.event.markup.PortletMarkupEventHandler();

if ( dojo.isIE ) {
	com.ibm.portal.aggregation.event.markup.PORTLET_MARKUP_EVENT_HANDLER.addRewriter( new com.ibm.portal.aggregation.event.markup.OnclickRewriter() );
}	

}
