/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.aggregation.forms"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.forms"] = true;
dojo.provide( "com.ibm.portal.aggregation.forms" );

dojo.require( "dojo.dnd.common");
dojo.require( "com.ibm.portal.aggregation.event" );
dojo.require( "com.ibm.portal.aggregation.event.common" );

dojo.declare( "com.ibm.portal.aggregation.forms.actions.FormRewriteAction",
			  null,
			  {
			  	// summary:	Base interface for an action. Each implementation must implement
 				//		the apply function.
				// description: Passes the given form element through to give the implementation of
				//		this action to modify the form element. 
			  	
				apply: function ( /*HTMLElement*/formElem ) {
					// summary: Modify the given form element
					// description: Allows this action to the form. 
					// formElem: the form element 
					dojo.unimplemented( "com.ibm.portal.aggregation.FormRewriteAction.apply" );
				}	
			  }	
);

dojo.declare( "com.ibm.portal.aggregation.forms.actions.SubmitInputOverrideAction",
			  com.ibm.portal.aggregation.forms.actions.FormRewriteAction,
			  {
			  	// summary: Base interface for rewriting any input of type "submit" (or button) in Firefox.
			  	
			  	newSubmitOnclickHandler: function ( /*HTMLElement*/ submitInput ) {
					// summary: Creates a new onclick handler for a "submit" type input.
					// description: The onclick handler must handle any existing handlers, as well
					//		as overriding the form submission by calling our portlet form handler instead
					//		of letting the browser handle the form submission.
					// submitInput: the input element of type submit (or button in firefox) 
					// returns: an onclick Function object
					ibm.portal.debug.entry( "FormRewriteAction.newSubmitOnclickHandler", [ submitInput ] );
					
					var name = submitInput.getAttribute( "name" );
					var formElem = submitInput.form;
					var onSubmitFn = formElem.onsubmit; 
					var onclickFn = function(evt) { 
						ibm.portal.debug.entry( name + ".onclick", [evt] );
						
						var retVal = (formElem.submitted != true );
						
						//If there is an onsubmit function, we need to execute it in the correct context since
						//it's return value can affect whether or not we actually allow the form to be submitted.
						if ( onSubmitFn && retVal ) {
							ibm.portal.debug.text( "Found onsubmit function: " + onSubmitFn );
							try { 
								retVal = onSubmitFn.apply( evt.target.form );
							}
							catch ( err ) {
								//In the browser, if a javascript error occurs in the onsubmit handler, the browser
								//appears to just ignore the onsubmit function altogether. So we will mirror that behavior
								//here, for lack of a better solution.								
								ibm.portal.debug.text( "Error encountered executing onsubmit handler... pretending we never saw it since this seems to match the browser behavior");
								retVal = true;
							}
							
							ibm.portal.debug.text( "Return value from onsubmit function: " + retVal );		
						}
						
						if ( retVal != false ) {
							var target = formElem.getAttribute("target");
							ibm.portal.debug.text( "Found form target: " + target );
							if(target && target != "" && target != "_self" ) {
								ibm.portal.debug.text( "Executing old submit function: " + formElem._oldsubmit ); 
								if ( formElem._oldsubmit ) {
									formElem._oldsubmit();
								} else {
									com.ibm.portal.utilities.callOverwrittenFunction( formElem, "submit" );
								}
								return false;
							}
							
							//Set the active submit trigger so that we can encode it into the form. This is part of the form submission
							//"magic" that the browser usually handles.
							com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER.setActiveSubmitTrigger( formElem.getAttribute( "id" ), { name: evt.target.getAttribute( "name" ), value: evt.target.getAttribute( "value" ) } );
                            //Have to use getAttributeNode here, otherwise IE returns the INPUT object instead of the form action when there's an INPUT named 'action'
							com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER.handle( formElem.getAttributeNode( 'id' ).value, formElem.getAttributeNode( 'action' ).value );
						}
							
						ibm.portal.debug.exit( name + ".onclick", [ false ] );
						//evt.preventDefault();
						return false;
					};
				
					ibm.portal.debug.exit( "FormRewriteAction.newSubmitOnclickHandler", [ onclickFn ] );
					return onclickFn; // Function
				}
			  }
);

dojo.declare( "com.ibm.portal.aggregation.forms.actions.SubmitOverrideAction",
			  com.ibm.portal.aggregation.forms.actions.FormRewriteAction,
			  {
			  	// summary: Overrides the submit function associated with a form to call our portlet form handler. 
				// description: A form's action cannot execute normally since it will return an 
				//		atom feed as a response so we change the submit function to pass the action url to
				//		the form handler object which is responsible for actually submitting the form. This handles
				//		the case where the form is submitted via javascript.
				
				apply: function ( /*HTMLElement*/formElem ) {
					// summary: Modify the given form element
					// description: Allows this action to the form. 
					// formElem: the form element 
					ibm.portal.debug.entry( "SubmitOverrideAction.apply", [ formElem ] );

					//Give the form an ID if it doesn't already have one.
					var formId = formElem.getAttribute("id");
					if ( !formId || formId.length == 0 ) {
						formId = dojo.dnd.getUniqueId();
						formElem.setAttribute("id", formId );
					}
					
					//Override the submit function to handle the case where the action is set via JS and then
					//submitted via JS.
					//formElem._oldsubmit = formElem.submit;
					try{
						//Have to use getAttributeNode here, otherwise IE returns the INPUT object instead of the form action when there's an INPUT named 'action'
						//formElem.submit = function () { com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER.handle( this.getAttributeNode( 'id').value, this.getAttributeNode( 'action' ).value ); }
						com.ibm.portal.utilities.overwriteProperty(formElem, 
																   "submit", 
																   function () { com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER.handle( this.getAttributeNode( 'id').value, this.getAttributeNode( 'action' ).value ); },
																   true
																   );
					}
					catch(err) {	// IE error if form.submit is actually a child element of the form
						var attr = formElem.getAttributeNode( 'submit' );
						if(attr != null) {
							formElem._oldsubmit = attr.value;
							var newAttr = document.createAttribute( 'submit' );
							newAttr.value = "function () { com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER.handle( this.getAttributeNode( 'id').value, this.getAttributeNode( 'action' ).value ); }";
							formElem.setAttributeNode(newAttr);
						}
						else {
							var tempForm = document.createElement("form");
							com.ibm.portal.utilities.setOverwrittenProperty( formElem, "submit", tempForm.submit );
						}
					}
					ibm.portal.debug.exit( "SubmitOverrideAction.apply" );	
				}	
			  }	  
);

dojo.declare( "com.ibm.portal.aggregation.forms.actions.AddContentTypeInput",
			  com.ibm.portal.aggregation.forms.actions.FormRewriteAction,
			  {
			  	// summary: Required only for IE. Ensures that the content type of
				//		the feed response is text/xml since that is the only content-type
				//		for which IE sets the responseXML property on the XHR.	
			  
			  	apply: function ( formElem ) {
					// summary: Modify the given form element
					// description: Allows this action to the form. 
					// formElem: the form element
					ibm.portal.debug.entry( "AddContentTypeInput.apply", [ formElem ] );
					ibm.portal.debug.text( "Adding content type input to form " + formElem.name );
						
					var input = document.createElement( "<input name='ibm.web2.contentType' value='text/xml' type='hidden' />" );
					formElem.appendChild( input );
					
					ibm.portal.debug.exit( "AddContentTypeInput.apply" );
				}
			  }  
);

dojo.declare( "com.ibm.portal.aggregation.forms.actions.TagSubmitInputs",
			  com.ibm.portal.aggregation.forms.actions.SubmitInputOverrideAction,
			  {
			  	// summary: Tag all form submit elements with an onclick handler so that we can
				//		identify the element that triggered the submit when a form has multiple
				//		submit inputs.
				
				apply: function ( formElem ) {
					// summary: Modify the given form element
					// description: Allows this action to the form. 
					// formElem: the form element
					ibm.portal.debug.entry( "Portlet._tagSubmitInputs", [ formElem ] );
					
                    var inputs = com.ibm.portal.utilities.html.getElementsByTagNames( formElem, "input", "button" );
                    					
					ibm.portal.debug.text( "found " + inputs.length + " inputs associated with this form." );
					
					for ( input in inputs ) {
						
						ibm.portal.debug.text( "Processing input: " + inputs[input] );
						
						if ( !inputs[input].type ) { continue; }

						if ( inputs[input].type.toLowerCase() == "submit" || inputs[input].type.toLowerCase() == "image" ) {
								ibm.portal.debug.text( "Setting onclick handler for submit or image: " + inputs[input].name );
								var onclickFn = this.newSubmitOnclickHandler( inputs[input] );
								//dojo.event.connect( inputs[input], "onclick", onclickFn );
								com.ibm.portal.aggregation.event.DEFAULT_ACTION_MANAGER.setDefaultAction( inputs[input], "onclick", onclickFn );
						}
					}
					
					ibm.portal.debug.exit( "Portlet._tagSubmitInputs" );
				}
			  } 
);

dojo.declare( "com.ibm.portal.aggregation.forms.actions.IFrameOnloadOverrideAction",
			  null,
			  {
			  	// summary: Action for rewriting any iframe onload handler.
			  	
			  	apply: function ( iframeElem ) {
					// summary: Modify the given iframe element
					// description: Non-destructively changes the onload handler, or adds one 
					// 		if non-existent to publish the portlet markup updated on the iframe 
					// 		load event
					// iframeElem: the iframe element
					ibm.portal.debug.entry( "IFrameOnloadOverrideAction.apply", [ iframeElem ] );
					ibm.portal.debug.text( "Overriding iframe onload handler " + iframeElem.name );
					
					// only do this once per iframe
					if(!iframeElem._oldonload) {
						if(!iframeElem.onload || iframeElem.onload == "" || (dojo.isIE && iframeElem.onload === document.body.onload)) {
							iframeElem._oldonload = function() {};
						}
						else {
							iframeElem._oldonload = iframeElem.onload;
						}
						iframeElem.onload = function() {
							// change the iframe's onload handler to publish a markup change,
							// but make sure we call the existing onload handler also
							this._oldonload.apply( this );
							try{
								var portlet = dojo.byId( com.ibm.portal.utilities.findPortletIdByElement(this) );
								if(portlet != null) {
									try{
										// Firefox will see this iframe as empty later unless we delete the iframe reference
										delete window[iframeElem.name];	
									}
									catch(err) {}
									dojo.publish( "/portal/PortletMarkup/updated" , 
										[{newMarkup: portlet}] );
								}
							}
							catch(err) {}
						};
					}
					
					ibm.portal.debug.exit( "IFrameOnloadOverrideAction.apply" );
				}
			  }
);

//dojo.declare( "com.ibm.portal.aggregation.forms.actions.TagButtonInputs",
//			  com.ibm.portal.aggregation.forms.actions.SubmitInputOverrideAction,
//			  null,
//			  {
//			  	// summary: Tag all form button elements with an onclick handler so that we can
//				//		identify the element that triggered the submit when a form has multiple
//				//		submit inputs.
//				
//				apply: function ( formElem ) {
//					// summary: Modify the given form element
//					// description: Allows this action to the form. 
//					// formElem: the form element
//					ibm.portal.debug.entry( "Portlet._tagButtonInputs", [ formElem ] );
//					
//					var buttons = formElem.getElementsByTagName( "button" );
//										
//					ibm.portal.debug.text( "found " + buttons.length + " buttons associated with this form." );
//					
//					for ( var i = 0; i < buttons.length; i++ ) {
//						
//						ibm.portal.debug.text( "Processing button: " + buttons[i] );
//						
//						ibm.portal.debug.text( "Setting onclick handler for button: " + buttons[i].name );
//						var onclickFn = this.newSubmitOnclickHandler( buttons[i] );
//						//dojo.event.connect( buttons[i], "onclick", onclickFn );
//						com.ibm.portal.aggregation.event.DEFAULT_ACTION_MANAGER.setDefaultAction( buttons[i], "onclick", onclickFn );
//					}
//					
//					ibm.portal.debug.exit( "Portlet._tagButtonInputs" );
//				}
//			  }
//);

dojo.declare( "com.ibm.portal.aggregation.forms.PortletFormRewriter",
			  com.ibm.portal.aggregation.event.PortletMarkupRewriter,
			  {
			  	// summary: Controls the registration and execution of the actions associated
				//		with this rewriter.
				
				constructor: function () {
			  		this._actions = new Array();
			 	},
				needsRewrite: function ( /*HTMLElement*/elem ) {
					// summary: Indicates whether or not this rewriter needs to handle this node.
					// elem: the current markup element
					//return (dojo.html.isTag( elem, "form" ) != ""); // Boolean
					return (elem && elem.tagName && elem.tagName.toLowerCase() == "form" ); // Boolean
				}, 
				addAction: function ( action ) {
					// summary: Register a filter at the end of the filter chain.
					// filter: the filter to append
					if ( !this._actions ) { this._actions = new Array(); }
					this._actions.push( action );
				},
				rewrite: function ( formElem ) {
					// summary: Execute the filter chain.
					// description: Calls doFilter on every filter registered in this filter chain
					//		until all filters have been called or one of the filters returns true.
					ibm.portal.debug.entry( "PortletFormRewriter.rewrite", [ formElem ] );
					
					var i = 0;
					while ( i < this._actions.length )	{
						this._actions[i].apply( formElem );
						i = i + 1;		
					}
					
					ibm.portal.debug.exit( "PortletFormRewriter.rewrite" );
				},
				selectNodes: function ( /*HTMLElement*/rootElement ) {
					return dojo.query( "form", rootElement );
				}	
			  } 
);

dojo.declare( "com.ibm.portal.aggregation.forms.PortletIFrameRewriter",
			  com.ibm.portal.aggregation.event.PortletMarkupRewriter,
			  {
			  	// summary: Controls the registration and execution of the actions associated
				//		with this rewriter.
				
				constructor: function () {
			  		this._actions = new Array();
			 	},
				needsRewrite: function ( /*HTMLElement*/elem ) {
					// summary: Indicates whether or not this rewriter needs to handle this node.
					// elem: the current markup element
					//return (dojo.html.isTag( elem, "iframe" ) != ""); // Boolean
					return (elem && elem.tagName && elem.tagName.toLowerCase() == "iframe");
				}, 
				addAction: function ( action ) {
					// summary: Register a filter at the end of the filter chain.
					// filter: the filter to append
					if ( !this._actions ) { this._actions = new Array(); }
					this._actions.push( action );
				},
				rewrite: function ( iframeElem ) {
					// summary: Execute the filter chain.
					// description: Calls doFilter on every filter registered in this filter chain
					//		until all filters have been called or one of the filters returns true.
					ibm.portal.debug.entry( "PortletIFrameRewriter.rewrite", [ iframeElem ] );
					
					var i = 0;
					while ( i < this._actions.length )	{
						this._actions[i].apply( iframeElem );
						i = i + 1;		
					}
					
					ibm.portal.debug.exit( "PortletIFrameRewriter.rewrite" );
				},
				selectNodes: function (/*HTMLElement*/rootElement) {
					return dojo.query( "iframe", rootElement );
				}	
			  } 
);

com.ibm.portal.aggregation.forms.PORTLET_FORM_REWRITER = new com.ibm.portal.aggregation.forms.PortletFormRewriter();
com.ibm.portal.aggregation.forms.PORTLET_FORM_REWRITER.addAction( new com.ibm.portal.aggregation.forms.actions.SubmitOverrideAction() );
com.ibm.portal.aggregation.forms.PORTLET_FORM_REWRITER.addAction( new com.ibm.portal.aggregation.forms.actions.TagSubmitInputs() );

com.ibm.portal.aggregation.forms.PORTLET_IFRAME_REWRITER = new com.ibm.portal.aggregation.forms.PortletIFrameRewriter();
com.ibm.portal.aggregation.forms.PORTLET_IFRAME_REWRITER.addAction( new com.ibm.portal.aggregation.forms.actions.IFrameOnloadOverrideAction() );

if ( dojo.isIE ) {
	com.ibm.portal.aggregation.forms.PORTLET_FORM_REWRITER.addAction( new com.ibm.portal.aggregation.forms.actions.AddContentTypeInput() );
}

//Add the rewriter to the markup event handler
com.ibm.portal.aggregation.event.markup.PORTLET_MARKUP_EVENT_HANDLER.addRewriter( com.ibm.portal.aggregation.forms.PORTLET_FORM_REWRITER );
com.ibm.portal.aggregation.event.markup.PORTLET_MARKUP_EVENT_HANDLER.addRewriter( com.ibm.portal.aggregation.forms.PORTLET_IFRAME_REWRITER );

dojo.declare( "com.ibm.portal.aggregation.forms.PortletForm",
			  com.ibm.portal.services.PortalRestServiceForm,
			  {
			  	// summary: This object represents a form in the state of not being submitted to the 
			  	//		server yet, but after the user has performed the action which should result
			  	//		in the form being submitted.
			  	
			  	constructor: function ( /*HTMLElement*/formElem, /*{name:, value:}*/submitInputNameAndValue ) {
			  		this.submitTrigger = submitInputNameAndValue;
			  	},
			  	submit: function () {
			  		// summary: Submits the form. This is necessary because the native submit function
			  		//		may have been overridden to ensure the form submission was intercepted.
			  		ibm.portal.debug.entry( "PortletForm.submit" );
			  		var utils = com.ibm.portal.utilities;
			  		var formElem = this.getDOMElement();
			  		ibm.portal.debug.text( "Submitting: " + formElem + "; old submit=" + utils.getOverwrittenProperty( this.getDOMElement(), "submit" ) );
			  		try {
			  			//com.ibm.portal.utilities.callOverwrittenFunction( this.getDOMElement(), "submit" );
			  			//IE native functions don't support apply to change the scope, so we must restore the property
			  			//temporarily to get the correct scope.
			  			var restoreValue = utils.restoreProperty( formElem, "submit" );
			  			formElem.submit();
			  			utils.overwriteProperty( formElem, "submit", restoreValue );
			  		}
			  		catch ( e ) {
			  			ibm.portal.debug.text( "Error calling overwritten function: " + e.message );
			  		}	
			  		ibm.portal.debug.exit( "PortletForm.submit" );
			  	},
			  	toQuery: function () {
			  		// summary: Translates the form into a query string. Only elements which make sense
			  		//		for a query string are translated (i.e. NOT inputs of type "file").
			  		// returns: a string of name/value pairs suitable for being included as part of a url's
			  		//		query string.
			  		ibm.portal.debug.entry( "PortletForm.toQuery" );
			  		var queryStr =  com.ibm.portal.utilities.html.convertFormToQuery( this.getDOMElement(), this.submitTrigger );
			  		ibm.portal.debug.exit( "PortletForm.toQuery", queryStr );
			  		return queryStr;
			  	}
			  }
);

dojo.declare( "com.ibm.portal.aggregation.forms.PortletFormHandler",
			  null,
			  {
			  	// summary: Examines all clicks that originate inside a portlet body
				//		and handles if appropriate.
				// description: Intercepts all clicks that originate from inside a 
				//		portlet and passes the event through the filter chain. If none of 
				//		filters can handle the event, it is left alone. Otherwise, it is
				//		cancelled and handled.
				
				constructor: function () {
			  		this._activeFormSubmitTriggers = {};
			  		this._callbackfns = {};
			  	},
				handle: function ( formId, actionURL ) {
					ibm.portal.debug.entry( "PortletFormHandler.handle", [ formId, actionURL ] );
					
					var portletForm = new com.ibm.portal.aggregation.forms.PortletForm( dojo.byId( formId ), this._activeFormSubmitTriggers[ formId ] ); 
					ibm.portal.debug.text( "Found form node: " + portletForm.getDOMElement() );
					ibm.portal.debug.text( "form node's submit fn: " + portletForm.getDOMElement().submit );
					
					//It's possible this form could be submitting to a servlet packaged with a portlet, in which case we want to let
					//the natural form submission process occur.
					if ( !com.ibm.portal.utilities.isPortalUrl( actionURL ) ) {
						ibm.portal.debug.exit( "PortletFormHandler.handle (form points to non-portal servlet)" );
						portletForm.submit();
						return;
					}
					//It's possible the form has a target pointing to a new window. In this case, we also are going to let the natural
					//form submission process occur.
					var target = portletForm.getDOMElement().getAttributeNode( "target" );
					ibm.portal.debug.text( "Target node is : " + target );
					if ( target != null && target.value && target.value != "" && !( target.value == "_self" || target.value == "_top" ) ) {
    					ibm.portal.debug.text( "Target node value is : " + target.value );
						ibm.portal.debug.exit( "PortletFormHandler.handle (form has a target attribute)" );
						portletForm.submit();
						return;
					}
					
					if ( portletForm.getDOMElement().submitted == true ) { 
						ibm.portal.debug.text( "Form has already been submitted!!" );
					}
					else {
	                    if ( portletForm.isMultipart && this._activeFormSubmitTriggers[ formId ] ) {
                            //Create the url and add the necessary parameters.
					        var urlObj = new com.ibm.portal.utilities.HttpUrl( actionURL );
                            var activeSubmitInputName = this._activeFormSubmitTriggers[formId].name;
					        urlObj.addParameter( activeSubmitInputName, this._activeFormSubmitTriggers[formId].value );
                            actionURL = urlObj.toString();
                            ibm.portal.debug.text( "New action url is: " + actionURL );        
                        }				
                        else {
						    ibm.portal.debug.text( "Encoded form should be: " + portletForm.toQuery() );
                        }    
						
						var portletId = com.ibm.portal.utilities.findPortletIdByElement( portletForm.getDOMElement() );
						
						var serviceReq = new com.ibm.portal.services.PortletFragmentService();
						serviceReq.getPortletInfo( portletId, actionURL, null, portletForm );
						//formNode might have been overwritten.
						portletForm.getDOMElement().submitted = true;
					}	
					
					ibm.portal.debug.exit( "PortletFormHandler.handle (normal)" );
				},
				handleMultiPartResult: function ( iframeId ) { 
					// summary: Global function for IE to use as a callback for dynamically inserted IFRAMEs.
					// description: IE seems to not recognize onload handlers set in the traditional sense on dynamically created
					//		IFRAMEs. We have to set the onload handler in text written into the DOM which prevents us from sending
					//		along a Function object. The callback function is stored as an instance variable on this class, sorted
					//		by IFRAME id. This callback function is called when this function is notified of an IFRAME load event.
					//		The only case where we use IFRAMEs is for multi-part forms so hopefully this will be a rare occurrence.
					// iframeId: the iframe ID 
					ibm.portal.debug.entry( "PortletFormHandler.handleMultiPartResult", [ iframeId ] );
					
					var callbackfn = this._callbackfns[iframeId].fn;
					var callbackargs = this._callbackfns[iframeId].args;
					this._callbackfns[iframeId] = null;
					var origXmlText = this.extractEncodedXml(self.frames[iframeId].document);
                    
					ibm.portal.debug.text("XML from the multi-part form: " + origXmlText);
					
                    // unescape the xml text sent by multi-part form
					var decodedXmlText = com.ibm.portal.utilities.decodeXML(origXmlText);
					var xmlDoc = dojox.data.dom.createDocument(decodedXmlText);
					callbackfn( "load", xmlDoc, null, callbackargs );
                    
					ibm.portal.debug.exit( "PortletFormHandler.handleMultiPartResult" );
				}, 
				extractEncodedXml: function ( xmlDoc ) {
					ibm.portal.debug.entry( "PortletFormHandler.extractEncodedXml", [ xmlDoc ] );
					
					var xmlNodeArray = xmlDoc.getElementsByTagName('PRE');
					
					return xmlNodeArray[0].innerHTML;
				}, 
				setActiveSubmitTrigger: function ( /*String*/formId, /*{name: "", value:""}*/inputName ) {
                    this._activeFormSubmitTriggers[ formId ] = inputName;	
				},
                isMultipartForm: function ( /*String or HTMLElement*/ form ) {
                    return (dojo.byId( form ).getAttribute( "enctype" ) == "multipart/form-data");    
                }
                
			  }  
);

com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER = new com.ibm.portal.aggregation.forms.PortletFormHandler();

}
