/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.aggregation.javascript.parser"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.javascript.parser"] = true;
dojo.provide( "com.ibm.portal.aggregation.javascript.parser" );

dojo.declare( "com.ibm.portal.aggregation.javascript.parser.ASParserResult",
			  null,
			  {
			  	// summary: Summarizes the result of an AssignmentStatementParser parsing operation.
				// description: Contains the relevant information about a parsing result. Keeps the start and end
				//		indices, the variable name and the variable value. The value will include any quotation
				//		marks surrounding it (or if a variable name, it will be just the variable name string).
				// parsingSuccessful: indicates whether the parsing operation was successful or not
				// originalString: the string passed into be parsed
				// startIndex: the starting index of the originalString for parsing
				// endIndex: the ending index of the parsed string 
				// variableName: the parsed portion on the left side of the assignment
				// variableValue: the parsed portion on the right side of the assignment
				constructor: function ( /*Boolean*/successful, /*String*/javascript, /*int*/sIndex, /*int*/eIndex, /*String*/varName, /*String*/varValue ) {
				  	this.parsingSuccessful = successful; 
				  	this.originalString = javascript;
					this.startIndex = sIndex;
					this.endIndex = eIndex;
					this.variableName = varName;
					this.variableValue = varValue;
			  	}
			  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.parser.AssignmentStatementParser",
			  null,
			  {
				// summary: Parses simple javascript assignment statements given a javascript string and a 
				//		starting index. Please note that the parser must be reset between parsing calls.
				// description: This parser is intended for parsing very simple javascript assignment statements. 
				//		For example, "window.location.href='http://www.xyz.com';". It's primary purpose is for
				//		extracting the value of a window.location.href assignment so that this assignment can
				//		be re-written for client-side aggregation.
				
				//Parsing States
				
				//parsing the variable name				
				_PARSING_NAME: 0,
				//parsing the assignment operator and any whitespace surrounding it
				_PARSING_ASSIGNMENT: 1,
				//started parsing the value of the assignment
				_PARSING_VALUE: 2,
				
				//We need to know which type of quote (if any) is used to delimit the value string since the single
				//quote could appear in the URL value.
				
				//the value starts with a single quote, another single quote closes the value parsing
				_PARSING_VALUE_OPEN_SINGLE_QUOTE: 3,
				//the value starts with a double quote, another double quote closes the value parsing
				_PARSING_VALUE_OPEN_DOUBLE_QUOTE: 4,
				//the value is most likely a variable name (integer would be another option, either should be parsed correctly)
				_PARSING_VALUE_VARIABLE: 5,
				//the opening quotation mark is closed
				_PARSING_VALUE_CLOSE_EITHER_QUOTE: 6,
				//the parsing is complete
				_PARSING_DONE: 7,
				//the statement could not be parsed
				_PARSING_ERROR: 8,
				
				constructor: function() {
				  	this.name = "";
					this.value = "";
					this._javascriptString = "";	
			  	},				
				parse: function ( /*String*/javascript, /*int*/startIndex ) {
					// summary: Parses the next assignment statement from the given javascript string starting from 
					//		the given index
					// javascript: the javascript string
					// startIndex: the index to start parsing from
					// returns: a parsing result 
					ibm.portal.debug.entry( "AssignmentStatementParser.parse", [ javascript, startIndex ]);
					
					var state = this._PARSING_NAME;
					var i = startIndex;
					this._javascriptString = javascript;
					
					//Basically using a state machine here. Our state can change based on
					//the character read in.
					while ( ( state != this._PARSING_DONE && state != this._PARSING_ERROR )	 && i < javascript.length ) {
						state = this._processCharacter( this._javascriptString.charAt(i), i, state );		
						i = i+1;
					}
					
					var returnObj = new com.ibm.portal.aggregation.javascript.parser.ASParserResult( (state == this._PARSING_DONE), javascript, startIndex, i, this.name, this.value );

					ibm.portal.debug.exit( "AssignmentStatementParser.parse" , [ returnObj ] ); 
					return returnObj;	// com.ibm.portal.aggregation.javascript.parser.ASParserResult		 
				},
				reset: function () {
					// summary: Resets the parser's internal variables so the parser object may be reused.
					this.name = "";
					this.value = "";
					this._javascriptString = "";
				},
				_processCharacter: function( /*String*/character, /*int*/index, /*int*/state ) {
					// summary: Processes the given character and state to produce the next state for the parser.
					// character: the character to process
					// state: the current state of the parser
					// returns: the next state of the parser
					
					//TODO: comment handling!
					ibm.portal.debug.entry( "AssignmentStatementParser._processCharacter", [ character, state ] );
					
					var addToName = false;
					var addToValue = false;
					
					//Default to the current state.
					var nextState = state;
					
					//The initial state is PARSING_NAME. We remain in this state as long as the character given
					//is not an illegal character for a variable name. Basically, whenever we encounter a space
					//or a non A-Z, 0-9 character.
					if ( state == this._PARSING_NAME ) {
						var add = character.search( /[A-Za-z_\.][A-Za-z_0-9\.]*/ );
						if ( add >= 0 ) {
							addToName = true;
						}
						else if ( character == "=") {
							//There's no whitespace before the assignment operator, so we can skip right to 
							//parsing the value.
							if ( (this._javascriptString.length > ( index + 1 )) && this._javascriptString.charAt( index+1 ) != '=' ) {
								nextState = this._PARSING_VALUE;
							}	
							else {
								nextState = this._PARSING_ERROR;
							}
						}
						else if ( character == "!" || character == ")" || character == ";" ) {
							//Not an assignment statement.
							nextState = this._PARSING_ERROR;
						}
						else {
							//Have to check for whitespace before the assignment operator.
							nextState = this._PARSING_ASSIGNMENT;
						}
					}
					//Throw away any whitespace before the assignment operator.
					else if ( state == this._PARSING_ASSIGNMENT ) {
						if ( character == "=" ) {
							//Found the assignment operator, move on to the value.
							if ( (this._javascriptString.length > ( index + 1 )) && this._javascriptString.charAt( index+1 ) != '=' ) {
								nextState = this._PARSING_VALUE;
							}	
							else {
								nextState = this._PARSING_ERROR;
							}
						}
						else if ( character == "!" || character == ")" || character == ";" ) {
							nextState = this._PARSING_ERROR;
						}
					}
					//If we have a string value, we need to track the open quotation mark so we can match
					//it later on and close the string. Otherwise, we assume we have a variable or an 
					//integer as the value. We skip any leading spaces.
					else if ( state == this._PARSING_VALUE ) {
						if ( character == "'" ) {
							addToValue = true;
							nextState = this._PARSING_VALUE_OPEN_SINGLE_QUOTE;
						}
						else if ( character == '"' ) {
							addToValue = true;
							nextState = this._PARSING_VALUE_OPEN_DOUBLE_QUOTE;
						}
						else if ( character == ' ' ) {
							nextState = this._PARSING_VALUE;
						}
						else {
							var add = character.search( /[A-Za-z_\.][A-Za-z_0-9\.]*/ );
							if ( add >= 0 ) {
								addToValue = true;
								nextState = this._PARSING_VALUE_VARIABLE;
							}
						}		
					}
					
					//Keep going until we find a matching closing quotation mark.
					else if ( state == this._PARSING_VALUE_OPEN_SINGLE_QUOTE ) {
						addToValue = true;
						if ( character == "'" ) {
							nextState = this._PARSING_VALUE_CLOSE_EITHER_QUOTE;
						}
					}
					else if ( state == this._PARSING_VALUE_OPEN_DOUBLE_QUOTE ) {
						addToValue = true;
						if ( character == '"' ) {
							nextState = this._PARSING_VALUE_CLOSE_EITHER_QUOTE;
						} 
					}
					
					//Keep going until we find an invalid character in an identifier or a semi-colon.
					else if ( state == this._PARSING_VALUE_VARIABLE ) {
						var add = character.search( /[A-Za-z_\.\[\]][A-Za-z_0-9\.\[\]]*/ );
						if ( add >= 0 ) {
							addToValue = true;
						}
						else if ( character == ';' ) {
							nextState = this._PARSING_DONE;
						}
						else {
							nextState = this._PARSING_VALUE_CLOSE_EITHER_QUOTE;
						}
					}
					else if ( state == this._PARSING_VALUE_CLOSE_EITHER_QUOTE ) {
						//TODO: what about the case where there is no semi-colon?
						if ( character == ';' ) {
							nextState = this._PARSING_DONE;
						}
					}
					
					//Add the character to the appropriate instance variable, if necessary.
					if ( addToName ) {
						this.name += character;
					}
					if ( addToValue ) {
						this.value += character;
					}
					
					ibm.portal.debug.exit( "AssignmentStatementParser._processCharacter", [ nextState ] );
					return nextState; // int
				}			  	
			  }
); 

}
