/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.aggregation.storage"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.storage"] = true;
dojo.provide( "com.ibm.portal.aggregation.storage" );

dojo.require( "dojo.cookie" );
dojo.require( "com.ibm.portal.aggregation.error" );

dojo.declare( "com.ibm.portal.aggregation.storage.StorageUnit",
			  null,
			  {
			  	// summary: Generic client-side storage interface. 
			  	hasRoom: function ( /*String*/name, /*String*/ value ) {		
			  		// summary: Indicates whether the storage unit has room for
			  		//		the given value. 
			  		// returns: true if there is room, false if it cannot be stored
			  	},
			  	store: function ( /*String*/name, /*String*/ value ) {
			  		// summary: Stores the given value under the given name.
			  		// description: If there is not room for the value, the value will not be stored. The
			  		// 		hasRoom function should be called prior to storing a value to allow for proper
			  		// 		error handling. This function will return true if the storage was successful,
			  		// 		false if it was not.
			  		// returns: true if the value was stored, false if it was not
			  	},
			  	retrieve: function ( /*String*/name ) {
			  		// summary: Retrieves a value given the name it was stored under.
			  		// returns: the value or NULL if the value can't be found
			  	},
			  	remove: function ( /*String*/ name ) {
			  		// summary: Removes a value from the storage unit.
			  		// returns: the value or NULL if the value doesn't exist
			  	},
			  	getContents: function () {
			  		// summary: Get the contents of the storage unit.
			  	}
			  }
);

dojo.declare( "com.ibm.portal.aggregation.storage.CookieStorageUnit",
			  com.ibm.portal.aggregation.storage.StorageUnit,
			  {
			  	maxStringSize: 7000,
			  	constructor: function () {
			  		/*if ( !dojo.cookie.isSupported() ) {
			  			this._cookiesEnabled = false;
			  			com.ibm.portal.aggregation.error.ERROR_HANDLER.reportWarning( "Cookies are not supported!", "Some features may not work correctly in client-side aggregation. Please enable cookies on your browser to fix this issue." );
			  		}*/
			  		this._cookiesEnabled = true;
			  	},
			  	hasRoom: function ( /*String*/name, /*String*/ value ) {		
			  		// summary: Indicates whether the storage unit has room for
			  		//		the given value. 
			  		// returns: true if there is room, false if it cannot be stored
			  		//return ( (name.length + value.length) <= this.maxStringSize );
			  		ibm.portal.debug.entry( "CookieStorageUnit.hasRoom", [ name, value ] );
			  		var hasRoom = true;
			  		
			  		//Based on the theory that browsers silently fail when setting a cookie that exceeds the 
			  		//maximum size and simply don't change the value of the cookie to the attempted value. Attempt to set the cookie
			  		//value, retrieve the value of the cookie after attempting to set it, and compare that value
			  		//to the value that we attempted to set the cookie to. If the two values don't match, then we know
			  		//the cookie is too big (or failed for another reason, but either way it's not going to work). This way,
			  		//if a browser allows a user configurable cookie size or if the standard maximum cookie size increases, we can
			  		//automagically take advantage of the bigger cookie size. Leave the maximum string size in place to provide some upper
			  		//limit on the cookie size; simply make it larger than we normally would.
			  		var cv = dojo.cookie( name );
			  		if ( cv != null ) {
			  			dojo.cookie( name, value, {path: "/"} );
			  			var sizePermitted = false;
			  			if ( dojo.isIE ) {
				  			var length = 0;
				  			if(cv && cv.length > 0) length = name.length + cv.length;
				  			sizePermitted = (document.cookie.length - length + name.length + value.length <= this.maxStringSize);
			  			}
			  			else {
			  				sizePermitted = ( (name.length + value.length) <= this.maxStringSize );
			  			}	
			  			hasRoom = ( ( value == dojo.cookie( name ) ) && sizePermitted );
			  			//Make sure to replace the original value in case the caller decides not to go ahead
			  			//with the store() operation.
			  			dojo.cookie( name, cv, {path: "/"} );
			  		}	
			  		
			  		ibm.portal.debug.exit( "CookieStorageUnit.hasRoom", hasRoom ); 
			  		return hasRoom;
			  	},
			  	store: function ( /*String*/name, /*String*/ value ) {
			  		// summary: Stores the given value under the given name.
			  		// description: If there is not room for the value, the value will not be stored. The
			  		// 		hasRoom function should be called prior to storing a value to allow for proper
			  		// 		error handling. This function will return true if the storage was successful,
			  		// 		false if it was not.
			  		// returns: true if the value was stored, false if it was not
			  		ibm.portal.debug.entry( "CookieStorageUnit.store", [ name, value ] );
			  		var success = false;
			  		
			  		if ( this._cookiesEnabled ) {
			  			dojo.cookie( name, value, {path: "/"} );
			  			success = ( dojo.cookie( name ) == value );
			  		}
			  		
			  		ibm.portal.debug.exit( "CookieStorageUnit.store", [ success ] );
			  		return success;
			  	},
			  	retrieve: function ( /*String*/name ) {
			  		// summary: Retrieves a value given the name it was stored under.
			  		// returns: the value or NULL if the value can't be found
			  		var retVal = null;
			  		if ( this._cookiesEnabled ) {
			  			retVal = dojo.cookie( name );
			  		}
			  		
			  		return retVal;
			  	},
			  	remove: function ( /*String*/ name ) {
			  		// summary: Removes a value from the storage unit.
			  		// returns: the value or NULL if the value doesn't exist
			  		var retVal = this.retrieve( name );
			  		if ( this._cookiesEnabled ) {
			  			dojo.cookie(name, null, {expires: -1});
			  		}	
			  		return retVal;
			  	}
			  }		
			  
);
			  

}
