/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.data.ConnectionsDogearStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.data.ConnectionsDogearStore"] = true;
dojo.provide( "com.ibm.portal.data.ConnectionsDogearStore" );

dojo.require( "com.ibm.data.CatalogMixin" );
dojo.require( "com.ibm.data.OpenSearchFeedStore" );
dojo.require( "com.ibm.data.ProxyHelper" );
dojo.require( "com.ibm.utilities" );

dojo.declare( "com.ibm.portal.data.ConnectionsDogearStore",
			[com.ibm.data.OpenSearchFeedStore, com.ibm.data.ProxyHelper,com.ibm.data.CatalogMixin],
			{
				namespaces: dojo.mixin({}, com.ibm.data.OpenSearchFeedStore.prototype.namespaces, {
					"snx": "http://www.ibm.com/xmlns/prod/sn"
				}),
				
				// itemClass: String the class used to represent items of this store
				itemClass: "com.ibm.portal.data.ConnectionsDogearItem",
				
				constructor: function( /*Map*/ args ) {
					// summary: Creates a new instance of a com.ibm.portal.data._PortalModelStore
					// args: Map
					//		The args map will be used to replace any default values of this store with the
					//		values specified in the args map.
					this._features['dojo.data.api.Identity'] = true;
					this.url = this.urlThroughProxy(this.url);
				},
				
				/********************************/
				/**                            **/
				/**                            **/
				/**  catalog extension point   **/
				/**                            **/
				/**                            **/
				/********************************/
				
				mapItem: function(/*item*/item){
					var map = {};
					map["label"] = this.getValue(item, "title", "untitled");
					map["tags"] = this.getValues(item, "category", "");
					map["url"] = this.getValue(item, "url", "");
					return map;
				},

				prepareQuery: function(keywordArgs) {
					if(keywordArgs.query.keywords && keywordArgs.query.keywords.length > 0) {
						keywordArgs.query.search = keywordArgs.query.keywords.join(" ");
						delete keywordArgs.query.keywords;
					}
					if(keywordArgs.query.tags) {
						keywordArgs.query.tag = keywordArgs.query.tags.join(",");
						delete keywordArgs.query.tags;
					}
					dojo.mixin(keywordArgs.query, 
						{
							searchOperator: 'and',	// always use 'and' search since it's a filter
							showFavIcon: true
						}
					);
					
					
					if(keywordArgs.sort&& keywordArgs.sort.length > 0) {
						switch(keywordArgs.sort[0].attribute) {
							case "date": 
								keywordArgs.query.sortField = "date";
								break;
							case "title":
								keywordArgs.query.sortField = "popularity";
								break;
							default:
								keywordArgs.query.sortField = keywordArgs.sort[0].attribute;
						}
						keywordArgs.query.sortOrder = "asc";
						if(keywordArgs.sort[0].descending == true) {
							keywordArgs.query.sortOrder = "desc";
						}
					}
					if(keywordArgs.count) {
						keywordArgs.query.ps = keywordArgs.count;
						if(keywordArgs.start == null) {
							keywordArgs.start = 0;
						}
						keywordArgs.query.page = (keywordArgs.start / keywordArgs.count) + 1;
					}
					
					return keywordArgs;
				},
				
				
				/********************************/
				/**                            **/
				/**                            **/
				/**   dojo.data.api.Identity   **/
				/**                            **/
				/**                            **/
				/********************************/				
				getIdentity: function(/* com.ibm.portal.data._PortalModelItem */ item) {
					// summary: See dojo.data.api.Identity.getIdentity()
					// returns: The Portal Model Store entry's unique identifier
					return this.getValue(item, "id");
				},
				
				getIdentityAttributes: function(/* item */ item){
					//	summary:
					//		Returns an array of attribute names that are used to generate the identity. 
					//		For most stores, this is a single attribute, but for some complex stores
					//		such as RDB backed stores that use compound (multi-attribute) identifiers
					//		it can be more than one.  If the identity is not composed of attributes
					//		on the item, it will return null.  This function is intended to identify
					//		the attributes that comprise the identity so that so that during a render
					//		of all attributes, the UI can hide the the identity information if it 
					//		chooses.
					//	item:
					//		The item from the store from which to obtain the array of public attributes that 
					//		compose the identifier, if any.
					return ['id']; // Array
				},
			
				fetchItemByIdentity: function(/* object */ keywordArgs){
					//	summary:
					//		Given the identity of an item, this method returns the item that has 
					//		that identity through the onItem callback.  Conforming implementations 
					//		should return null if there is no item with the given identity.  
					//		Implementations of fetchItemByIdentity() may sometimes return an item 
					//		from a local cache and may sometimes fetch an item from a remote server, 
					//
					// 	keywordArgs:
					//		An anonymous object that defines the item to locate and callbacks to invoke when the 
					//		item has been located and load has completed.  The format of the object is as follows:
					//		{
					//			identity: string|object,
					//			onItem: Function,
					//			onError: Function,
					//			scope: object
					//		}
					//	The *identity* parameter.
					//		The identity parameter is the identity of the item you wish to locate and load
					//		This attribute is required.  It should be a string or an object that toString() 
					//		can be called on.
					//		
					//	The *onItem* parameter.
					//		Function(item)
					//		The onItem parameter is the callback to invoke when the item has been loaded.  It takes only one
					//		parameter, the item located, or null if none found.
					//
					//	The *onError* parameter.
					//		Function(error)
					//		The onError parameter is the callback to invoke when the item load encountered an error.  It takes only one
					//		parameter, the error object
					//
					//	The *scope* parameter.
					//		If a scope object is provided, all of the callback functions (onItem, 
					//		onError, etc) will be invoked in the context of the scope object.
					//		In the body of the callback function, the value of the "this"
					//		keyword will be the scope object.   If no scope object is provided,
					//		the callback functions will be called in the context of dojo.global.
					//		For example, onItem.call(scope, item, request) vs. 
					//		onItem.call(dojo.global, item, request)
					var id = keywordArgs.identity;
					var ret = null;
					return ret;
				}
			}
);

dojo.declare( "com.ibm.portal.data.ConnectionsDogearItem",
			com.ibm.data.AtomEntryItem, 
			{
				namespaces: dojo.mixin({}, com.ibm.portal.data.ConnectionsDogearStore.prototype.namespaces),
				
				attrMappings: dojo.mixin({}, com.ibm.data.AtomEntryItem.prototype.attrMappings, {		
					url:{xpath: "./atom:link[@rel='http://www.ibm.com/xmlns/prod/sn/same']", targetName: "href", type: "attr"}
				})
			}
);

}
