/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.data._PortalModelStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.data._PortalModelStore"] = true;
dojo.provide( "com.ibm.portal.data._PortalModelStore" );

dojo.require( "com.ibm.data.AtomFeedStore" );
dojo.require( "com.ibm.data.TemplateInstantiator" );
dojo.require( "com.ibm.data._NotificationHelper" );
dojo.require("dojo.i18n");

dojo.declare( "com.ibm.portal.data._PortalModelStore",
			[com.ibm.data.AtomFeedStore, com.ibm.data.TemplateInstantiator, com.ibm.data._NotificationHelper],
			{
				// namespaces: Map
				//		Map of namespaces used by the Portal Model SPI feeds
				namespaces: dojo.mixin({}, com.ibm.data.AtomFeedStore.prototype.namespaces, {
		            "xsl" : "http://www.w3.org/1999/XSL/Transform",
		            "thr" : "http://purl.org/syndication/thread/1.0",
		            "xhtml" : "http://www.w3.org/1999/xhtml",
		            "model" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0.1/portal-model-elements",
		            "base" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0/ibm-portal-composite-base",
		            "portal" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0.1/portal-model",
		            "creation-context" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1.0/portal-creation-context",
		            "xsi" : "http://www.w3.org/2001/XMLSchema-instance"
		        }),
		        
		        // _notifyOn: Boolean indicating whether or not notifications should be
		        //		made with the dojo.data.Notification API.  Can be turned on/off after
		        //		store creation to disable notification during some operations if desired
		        _notifyOn: true,
		        
		        // itemClass: String the class used to represent items of this store
				itemClass: "com.ibm.portal.data._PortalModelItem",
				
				constructor: function( /*Map*/ args ) {
					// summary: Creates a new instance of a com.ibm.portal.data._PortalModelStore
					// args: Map
					//		The args map will be used to replace any default values of this store with the
					//		values specified in the args map.
					this._cachedItems = new Object();
				},
				
				/********************************/
				/**                            **/
				/**                            **/
				/**   dojo.data.api.Identity   **/
				/**                            **/
				/**                            **/
				/********************************/
				
				// inheriting getIdentity and getIdentityAttributes function from AtomFeedStore
							
				fetchItemByIdentity: function(/* object */ keywordArgs){
					//	summary:
					//		Given the identity of an item, this method returns the item that has 
					//		that identity through the onItem callback.  Conforming implementations 
					//		should return null if there is no item with the given identity.  
					//		Implementations of fetchItemByIdentity() may sometimes return an item 
					//		from a local cache and may sometimes fetch an item from a remote server, 
					//
					// 	keywordArgs:
					//		An anonymous object that defines the item to locate and callbacks to invoke when the 
					//		item has been located and load has completed.  The format of the object is as follows:
					//		{
					//			identity: string|object,
					//			onItem: Function,
					//			onError: Function,
					//			scope: object
					//		}
					//	The *identity* parameter.
					//		The identity parameter is the identity of the item you wish to locate and load
					//		This attribute is required.  It should be a string or an object that toString() 
					//		can be called on.
					//		
					//	The *onItem* parameter.
					//		Function(item)
					//		The onItem parameter is the callback to invoke when the item has been loaded.  It takes only one
					//		parameter, the item located, or null if none found.
					//
					//	The *onError* parameter.
					//		Function(error)
					//		The onError parameter is the callback to invoke when the item load encountered an error.  It takes only one
					//		parameter, the error object
					//
					//	The *scope* parameter.
					//		If a scope object is provided, all of the callback functions (onItem, 
					//		onError, etc) will be invoked in the context of the scope object.
					//		In the body of the callback function, the value of the "this"
					//		keyword will be the scope object.   If no scope object is provided,
					//		the callback functions will be called in the context of dojo.global.
					//		For example, onItem.call(scope, item, request) vs. 
					//		onItem.call(dojo.global, item, request)
					// returns: Request|null the request object used to fetch the item from
					//		the datastore if a remote call is required.  Returns null if the item
					//		is found in the cache, in which case the onItem function is called
					//		immediately.
					var id = keywordArgs.identity;
					var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
					if(this._cachedItems[id] && (!keywordArgs.queryOptions || !keywordArgs.queryOptions.preventCache)) {
						if(keywordArgs.onItem) keywordArgs.onItem.call(scope, this._cachedItems[id]);
						return keywordArgs;
					} 
					var args = {
						query: {
							uri: id,
							levels: 1,
							rep: "full",
							mode: "download"
						},
						queryOptions: keywordArgs.queryOptions,
						onComplete: dojo.hitch(this, 
							function(items) {
								if(items.length == 0 && keywordArgs.onError) {
									keywordArgs.onError.call(scope, "No items returned matching identity " + id);
								}
								else if(keywordArgs.onItem) {
									keywordArgs.onItem.call(scope, items[0]);
								}
							}
						)
					};
					if(keywordArgs.onError) args.onError = dojo.hitch(scope, keywordArgs.onError);

					return this.fetch(args);
				},
				
				_getItem: function(args) {
					// summary: See com.ibm.data.AtomFeedStore#_getItem
					var item = this.inherited(arguments);
					item.isFull = (args.query.rep && args.query.rep.toLowerCase() == "full");
					item.created = true;
					return item; 
				},
				
				invalidateCache: function(/*Item?*/ item) {
					// summary: Invalidates the client cache of items in this store.  Can invalidate
					//		a single item by passing the item as an argument, or the entire cache
					//		by passing no arguments.
					// item: Item optional data item to invalidate.
					if(item) delete this._cachedItems[this.getIdentity(item)];
					else this._cachedItems = new Object();
				},
				
				_replaceItemInCache: function(item, /*Boolean?*/ full) {
					// summary: Puts the item in the cache, replacing an older version of the same
					// 		item if it's already present, otherwise just adding it to the cache
					// item: Item to replace
					// full: Boolean flag to indicate if the entire item object should be replaced
					//		or just the element contents.  Default is to simply replace the element
					//		contents (in order to keep any properties on the item from being lost).
					var id = this.getIdentity(item);
					if(!full && this._cachedItems[id]) {
						this._cachedItems[id].element = item.element;
					}
					else this._cachedItems[id] = item;
				},
				
				_fromCache: function(id) {
					return this._cachedItems[id];
				},
				
				processFetchedItems: function(/*Array*/ items, /*Request*/request) {
					// summary: Extension point for descendants or consumers of this store to preprocess
					// 		the list of items that the store returns from a fetch call.  Allows a store
					// 		to filter the list or implement caching techniques on the client or anything
					// 		else.
					// items: Array items of this data store found in the response of a fetch call
					// returns: Array items of this data store after processing
					var preventClientCache = (request.queryOptions && request.queryOptions.preventClientCache);
					var replaceCache = (request.queryOptions && request.queryOptions.replaceCache);
					
					// iterate through the items we just fetched and handle them according to the queryOptions cache parameters
					// and what we already have in the cache
					for(var i = 0; i < items.length; i++) {
						var id = this.getIdentity(items[i]);
						
						if(this._cachedItems[id]) {
							// this item is already in our client cache
							if(replaceCache && !this.isDirty(this._cachedItems[id])) {
								// we want to replace the cached item with the retrieved item 
								// as long as our cached copy is not dirty
								this._replaceItemInCache(items[i]);
							}
							if(!preventClientCache && this._cachedItems[id].isFull == items[i].isFull) {
								// we want to replace what the server returns with what we have in the cache
								// and send back our cached version instead of what we got from the server
								// as long as we asked for the same representation as what's in our cache (full vs compact?)
								items.splice(i, 1, this._cachedItems[id]);
							}
						}
						else {
							// the item is not in our cache, so lets add it regardless of the queryOptions
							this._replaceItemInCache(items[i]);
						}
					}
					
					return items;
				},
				
				/********************************/
				/**                            **/
				/**                            **/
				/**    dojo.data.api.Write     **/
				/**                            **/
				/**                            **/
				/********************************/
				setValue: function(	/* Object */ item, 
						/* String */ attribute,
						/* almost anything */ value){
					var ret = this.inherited(arguments);
				
					if(ret.success) {
						if(!this._settingComplexAttr) {
							this._settingComplexAttr = true;
							var notify = this._notifyOn;
							this._notifyOn = false;
							var par = null;
							switch(attribute) {
								case "next":
									this.setValue(item, "nextURI", value);
									par = this.getValue(item, "parent");
									break;
								case "nextURI":
									this.setValue(item, "next", value);
									par = this.getValue(item, "parent");
									break;
								case "parent":
									this.setValue(item, "parentURI", value);
									break;
								case "parentURI":
									this.setValue(item, "parent", value);
									break;
							}
							if(par) {
								this.setValue(item, "parent", par);
								this.setValue(item, "parentURI", par);
							}
							this._notifyOn = notify;
							this._settingComplexAttr = false;
						}
						
						if(this._notifyOn) this.onSet(item, attribute, ret.oldValue, ret.newValue);
					}
					if((attribute == "next" || attribute == "nextURI") && !value) {
						var rule = this.getMapRule(attribute, item.attrMappings);
						var resultSet = this._getNodeByXpath(item.element, rule.xpath);
						if(resultSet.found) {
							resultSet.value.parentNode.removeChild(resultSet.value);
							delete resultSet.value;
						}
					}

					return ret;
				},
				deleteItem: function(/* Object */ item){
					var ret = this.inherited(arguments);
					if(ret && this._notifyOn) this.onDelete(item);
					return ret;
				},
				newItem: function(/* Object? */ keywordArgs, /*Object?*/ parentInfo){
					// summary: Creates a new Portal Content Model node in the Portal which can be either a
					//		label, url (internal or external), or page.
					// 		Unfortunately, this implementation of the newItem function must
					//		deviate slightly from the API details as specified by dojo.data.api.Write.  
					//		Generating a new item on the client can create an entire DOM entry that can 
					//		represent this model node and return that DOM node directly.  However, the model
					// 		feeds for Portal only provide the actual ID of the new item in the response from
					//		the POST operation to create the new item on the server.  Therefore, this function
					//		immediately returns the new item, but the item's ID will not be the real ID that
					//		the server has generated for it and therefore callers should take careful heed
					//		not to attempt to write back any additional changes to this returned item until
					//		the server has returned back and updated the item's ID correctly.  In order to support
					//		that, onComplete and onError handlers can be specified in the keywordArgs map
					//		that are handled in a special way to be called on response from the server in order
					//		to pass control to the caller after the real ID has been updated in the item.  The
					//		the onComplete handler can assume that the item passed into it has the correct ID
					//		as specified by the server and can therefore operate on that item without any side-effects.
					// 		The onError handler will be used to notify the caller of problems encountered when saving
					// 		the new item to the server.
					// keywordArgs:  A javascript object defining the initial content of the item as a set of 
					// 		JavaScript 'property name: value' pairs.
			        //
			        // returns: Item An item stub that is incomplete in that its ID is not the ID of the item
			        //		on the server.  The server has no knowledge of this item until it has been
			        //		sent to the server for creation.  At that time, the server will respond with
			        //		an XML response that has the correct ID for the item in the DOM.  In order
			        //		to initiate this send operation, the caller must call this store's save function,
			        //		at which time all pending write operations (edits, creates, deletes) will be sent
			        //		to the server.  On response from the create operations, the item's information will
			        // 		be completed as specified by the server, and the appropriate onComplete
			        //		and onError handlers for each create will be called as they were specified in the
			        //		keywordArgs for this particular item.
			        if(!parentInfo) parentInfo = {};
					var item = this._itemFromProps(keywordArgs, parentInfo);
					delete item.created;
					this._newItems.push(item);
					
					parentInfo.item = parentInfo.parent;
					item.loaded = false;
					item.created = false;
					if(this._notifyOn) this.onNew(item, parentInfo);
					return item;
				},
				getStub: function(/* Object? */ keywordArgs, /*Object?*/ parentInfo) {
					if(!parentInfo) parentInfo = {};
					var item = this._itemFromProps(keywordArgs, parentInfo);
					item.loaded = false;
					
					if(keywordArgs.id) {
						// use setValueByMapping to get around notification APIs and adding this stub
						// to the dirty list
						this.setValueByMapping(item, this.getMapRule("id", item.attrMappings), keywordArgs.id);
					}
					
					delete item._createUri;
					return item;
				},
				getItem: function(/*String*/ id) {
					// summary: Gets an item by its identity directly from the cache if it's 
					// 		present; if it's not present, return a stub which is not loaded but 
					// 		can be used to load the item from the store.
					//		Note - if no item exists with the given id, a stub will still be 
					//		returned but it will be impossible to load from the server.
					var item = this._fromCache(id);
					if(!item) {
						item = this.getStub({id: id});
					}
					return item;
				},
				_itemFromProps: function(/* Object? */ keywordArgs, /*Object?*/ parentInfo) {
					var map = this._prepareNewItemArgs(keywordArgs, parentInfo);
					var createUri = this._getCreateUri(map, parentInfo);
					var xmlDoc = this.getTemplateInstance(this._getTemplate(map, parentInfo), dojo.mixin({parent: ""}, map));
					if(xmlDoc) {
						var result = this._getNodeByXpath(xmlDoc, "//atom:feed");
						if(result.found) {
							result = this._getNodeByXpath(result.value, "./atom:entry");
							if(result.found) xmlDoc = result.value;
						}
						else {
							result = this._getNodeByXpath(xmlDoc, "//atom:entry");
							if(result.found) xmlDoc = result.value;
						}
						
						if(!map.parent) {
							// we don't want the thr:in-reply-to element in the DOM data to send, so remove it
							var inReplyTo = this._getNodeByXpath(xmlDoc, "./thr:in-reply-to").value;
							if(inReplyTo) {
								inReplyTo.parentNode.removeChild(inReplyTo);
								delete inReplyTo;
							}
						}
					}
					// the query.rep stuff is for _getItem to use in order to know the representation
					// of this item
					var item = this._getItem({element: xmlDoc, query: {rep: 'compact'}});
					item._createUri = createUri;
					return item;
				},
				
				_prepareNewItemArgs: function(args, parentInfo) {
					// summary: Used to modify arguments used to create a new item.  Can be overrided by subclasses
					//		to provide custom modifications and preparations for the object map which is used
					//		to instantiate the template.
					// args: Object arguments that are passed into the newItem function for this store
					// parentInfo: Object parentInfo that is passed into the newItem function for this store
					// returns: Object modified arguments map used to instantiate the template
					
					if(parentInfo.parent) {
						// if the parent id wasn't specified in keywordArgs but we got a parent item in the parentInfo
						// object, set the keywordArgs.parent prop to the id of the parent item
						if(!args.parent) args.parent = parentInfo.parent.store.getIdentity(parentInfo.parent);
					}
					else if(args.parent){
						if(dojo.isString(args.parent)) {
							// get the parent item from either cache or as a stub (which is not loaded)
							parentInfo.parent = this.getItem(args.parent);
						}
						else {
							// set the parentInfo.parent to the item, and the args.parent to the ID
							parentInfo.parent = args.parent;
							args.parent = args.parent.store.getIdentity(args.parent);
						}
					}
					// copy the args map
					// mixin the locale since some templates may need it... if it's already supplied
					// then it's just an exact copy of the original
					return dojo.mixin({locale: dojo.locale}, args);
				},
				_getTemplate: function(args, parentInfo) {
					// summary: Used to modify arguments used to create a new item.  Can be overrided by subclasses
					//		to provide custom modifications and preparations for the object map which is used
					//		to instantiate the template.
					// args: Object arguments that are passed into the newItem function for this store
					// parentInfo: Object parentInfo that is passed into the newItem function for this store
					// returns: String template to instantiate with arguments map
					return this.template;
				},
				_getCreateUri: function(args, parentInfo) {
					// summary: Used to modify arguments used to create a new item.  Can be overrided by subclasses
					//		to provide custom modifications and preparations for the object map which is used
					//		to instantiate the template.
					// args: Object arguments that are passed into the newItem function for this store
					// parentInfo: Object parentInfo that is passed into the newItem function for this store
					// returns: String value to use as the uri parameter when creating a resource.  If null
					//		is returned, it is not used at all.  When creating multiple resources, only the value 
					// 		of create uri of the first item in the list of new items will be used when sending
					// 		the single request to create all items. 
					return null;
				},
				_fixModelId: function(id, prefix) {
					if(id.indexOf(prefix + ":oid:") < 0) {
						id = prefix + ":oid:" + id;
					}
					return id;
				},
				
				onSaveComplete: function(/*Array*/ errors) {
					// summary: Callers can attach listeners to this function to get notified when a save operation
					//		on this store has received its last response from the server for all write operations.
					//		This is called before callback handlers to the save function itself are called but after
					// 		all the responses for the requests to the server have returned.
					// errors: Array of any errors that occurred during the save operation.  There can be multiple
					//		errors from a save operation since it is not atomic and may consist of many requests
					//		sent to the server.  Each error response from the server is recorded and kept for handling
					//		in this callback.
				},
				
				save: function(/* Object */ keywordArgs){
					var me = this;
					var numLeft = 0;
					if(this._newItems.length > 0) numLeft++; 
					if(this._dirtyItems.length > 0) numLeft++; 
					numLeft += this._deletedItems.length;
					var errors = new Array();
					
					// if there are any updates to be made, this is called once for the entire batch of updates
					// if there are any creates to be made, this is called once for the entire batch of creates
					// if there are any deletes to be made, this is called once per each delete
					var callbackfn = function(obj, isNew, data, ioArgs) {
						if(data) {
							if(data instanceof Error) {
								errors.push(data);
							}
							else if(isNew) {
								if(dojo.isIE) data = com.ibm.domUtilities.docFromString(data);
								var entries = me._getNodesByXpath(data, "./atom:feed/atom:entry").value;
								for(var i = 0; i < entries.length; i++) {
									obj[i].element = entries[i];
									me._replaceItemInCache(obj[i]);
									obj[i].created = true;
									obj[i]._onCreate();
								}
							}
						}
						
						if(--numLeft == 0) {
							// notify listeners that we've received all our responses
							me.onSaveComplete(errors);
							// only execute the onComplete or onError handler once at the end
							// we don't want to execute the onError handler on the first error
							// since we won't get a roll-up of all the errors afterwards and they will be lost
							if(errors.length > 0) {
								if(keywordArgs && keywordArgs.onError) { 
									keywordArgs.onError.call(keywordArgs.scope ? keywordArgs.scope : dojo.global, errors);
								} else { 
									dojo.forEach(errors, function(error){
										var message = error.message;
										var details = error.responseText;
										if(message.match(/\D401/)){ // Portal session not authenticated
											bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
											details = message;
											message = bundle["shelf_session_timeout"];
										}
										var statusMessage = new com.ibm.widgets.StatusMessage( "error", message, details );
										dojo.publish("/portal/status",[{message: statusMessage, uid: 'ibmPortalStatusBox'}]);
									});
								}
							}
							else {
								if(keywordArgs && keywordArgs.onComplete) {
									keywordArgs.onComplete.call(keywordArgs.scope ? keywordArgs.scope : dojo.global);
								}
							}
						}
					};
					
					// nothing to do, call onComplete immediately
					if(numLeft == 0) {
						setTimeout(function(){
							if(keywordArgs && keywordArgs.onComplete) { 
								keywordArgs.onComplete.call(keywordArgs.scope ? keywordArgs.scope : dojo.global);
							}
							me.onSaveComplete(errors);
						},1);
					}
					
					/////////////////////////////
					/////// Send Creates ////////
					/////////////////////////////
					if(this._newItems.length > 0) {
						var url = this.url;
						if(url.indexOf("?") < 0) url += "?";
						var createUri = this.getValue(this._newItems[0], "parent");
						if(!createUri) createUri = this._newItems[0]._createUri;
						url += "uri=" + createUri;
						var updateDoc = this.extractMultipleEntryFeed(this._newItems);
						var args = {
							postData: com.ibm.domUtilities.stringFromDoc(updateDoc), 
							url: url,
							content: {},
							headers: {'Content-type': 'application/atom+xml'},
							// when creating multiple Portal Model feed items in a single POST request,
							// they come back in reverse order in which they appear in the POST 
							// body of the request, so reverse the order we expect
							handle: dojo.partial(callbackfn, this._newItems.slice().reverse(), true),
							handleAs: "xml"
						};
						if(dojo.isIE) {
							args.handleAs = "text";
							args.content["ibm.web2.contentType"] = "text/xml";
						}
						
						dojo.rawXhrPost(args);
					}
					/////////////////////////////
					
					/////////////////////////////
					/////// Send Updates ////////
					/////////////////////////////
					if(this._dirtyItems.length > 0) {
						var url = this.url;
						if(url.indexOf("?") < 0) url += "?";
						url += "uri=" + this.getIdentity(this._dirtyItems[0]) + "&mode=merge&rep=empty";
						var updateDoc = this.extractMultipleEntryFeed(this._dirtyItems);
						var args = {
							putData: com.ibm.domUtilities.stringFromDoc(updateDoc), 
							url: url,
							content: {},
							headers: {'Content-type': 'application/atom+xml'},
							handle: dojo.partial(callbackfn, this._dirtyItems.slice(), false),
							handleAs: "xml"
						};
						if(dojo.isIE) {
							args.handleAs = "text";
							args.content["ibm.web2.contentType"] = "text/xml";
						}
						
						dojo.rawXhrPut(args);
					}
					/////////////////////////////
					
					/////////////////////////////
					/////// Send Deletes ////////
					/////////////////////////////
					dojo.forEach(this._deletedItems, function(item) {
						me._removeItem(item, dojo.partial(callbackfn, item, false));
					});
					/////////////////////////////
					
					// disable notification while persist operations are taking place, then turn
					// it back on
					this._notifyOn = false;
					this.inherited(arguments);
					this._notifyOn = true;
				},
				
				_removeItem: function(/*Item*/ item, /*Function?*/callbackfn) {
					var url = this.url;
					if(url.indexOf("?") < 0) url += "?";
					url += "uri=" + item.id + "&rep=empty";
					var args = {
						url: url,
						content: {},
						handle: callbackfn,
						handleAs: "xml"
					};
					if(dojo.isIE) {
						args.handleAs = "text";
						args.content["ibm.web2.contentType"] = "text/xml";
					}
					
					dojo.xhrDelete(args);
				}
			}
);

(function(){
	var namespaces = com.ibm.portal.data._PortalModelStore.prototype.namespaces;
	var nsDecl = new Array();
	
	for(var prefix in namespaces) {
		nsDecl.push('xmlns:', prefix, '="', namespaces[prefix], '" ');
	}
	dojo.extend(com.ibm.portal.data._PortalModelStore, {
		_feed:	 	com.ibm.domUtilities.docFromString(['<?xml version="1.0" encoding="UTF-8"?>',
					'<atom:feed ',
						nsDecl.join(''),
						'>',
						'<atom:author>',
							'<atom:name>IBM WebSphere Portal/6.1</atom:name>',
						'</atom:author>',
						'<atom:title>IBM WebSphere Portal Model Feed</atom:title>',
						'<atom:link href="." rel="self" type="application/atom+xml"/>',
					'</atom:feed>'].join(''))
	}
);
})();


dojo.declare( "com.ibm.portal.data._PortalModelItem",
			com.ibm.data.AtomEntryItem, 
			{
				// namespaces: Map
				//		Map of namespaces used by the Portal Model SPI feeds
				namespaces: dojo.mixin({}, com.ibm.portal.data._PortalModelStore.prototype.namespaces),
				
		        attrMappings: dojo.mixin({},com.ibm.data.AtomEntryItem.prototype.attrMappings, {
					uniqueName: {xpath: "./atom:id[@portal:uniquename]", targetName: "portal:uniquename", type: "attr", setter: 
						function(item, rule, value){
							var elem = this._createNSElement(item.element.ownerDocument, "atom:id");
							this._setAttribute(elem, rule.targetName, value);
							item.element.appendChild(elem);
							return true;
						}
					},
					parent: {xpath: "./thr:in-reply-to", targetName: "ref", type: "attr", setter: 
						function(item, rule, value){
							var elem = this._createNSElement(item.element.ownerDocument, "thr:in-reply-to");
							this._setAttribute(elem, rule.targetName, value);
							item.element.appendChild(elem);
							return true;
						}
					},
					parentURI: {xpath: "./thr:in-reply-to", targetName: "href", type: "attr", setter: 
						function(item, rule, value){
							var elem = this._createNSElement(item.element.ownerDocument, "thr:in-reply-to");
							this._setAttribute(elem, rule.targetName, value);
							item.element.appendChild(elem);
							return true;
						}
					},
					
					// next and nextURI are used to reorder nodes in a list of children of a node in a tree model
					// not all portal models are tree models and thus not all use these (ex. Portlet Model)
					next: {xpath: "./atom:link[@portal:rel='next']", targetName: "portal:uri", type: "attr", setter: 
						function(item, rule, value){
							var attrs = {
								"portal:rel": "next", 
								"rel": "related",
								"type": "application/atom+xml"
							};
							attrs[rule.targetName] = value;
							var elem = this._createNSElement(item.element.ownerDocument, "atom:link", attrs);
							item.element.appendChild(elem);
							return true;
						}
					},
					nextURI: {xpath: "./atom:link[@portal:rel='next']", targetName: "href", type: "attr", setter: 
						function(item, rule, value){
							var attrs = {
								"portal:rel": "next", 
								"rel": "related",
								"type": "application/atom+xml"
							};
							attrs[rule.targetName] = value;
							var elem = this._createNSElement(item.element.ownerDocument, "atom:link", attrs);
							item.element.appendChild(elem);
							return true;
						}
					},
					childrenURI: {xpath: "./atom:link[@rel='replies']", targetName: "href", type: "attr", setter: 
						function(item, rule, value){
							var elem = this._createNSElement(item.element.ownerDocument, "atom:link");
							this._setAttribute(elem, "rel", "replies");
							this._setAttribute(elem, rule.targetName, value);
							item.element.appendChild(elem);
							return true;
						}
					},
					
					// localizedTitle and localizedDescription are parameterized by language code; not all Portal models use them
					localizedTitle$: {xpath: "./atom:content/*[model:title]/model:title/base:nls-string[" + (dojo.isIE ? "@xml:lang = '${0}'": "lang('${0}')") + "]", targetName: "textContent", type: "node", locale: "${0}", setter: 
						function(item, rule, value){
							var result = this._getNodeByXpath(item.element, "./atom:content/*");
							var modelNode = result.value;
							var node = this._getNodeByXpath(modelNode, "./model:title").value;
							if(!node) {
								node = this._createNSElement(item.element.ownerDocument, "model:title");
								modelNode.appendChild(node);
							}
							// here we're using the locale specified by the parameterized rule that was passed by the caller
							var elem = this._createNSElement(item.element.ownerDocument, "base:nls-string", {"xml:lang": rule.locale});
							this._setNodeProp(elem, rule.targetName, value);
							node.appendChild(elem);
							return true;
						}
					},
					localizedDescription$: {xpath: "./atom:content/*[model:description]/model:description/base:nls-string[" + (dojo.isIE ? "@xml:lang = '${0}'": "lang('${0}')") + "]", targetName: "textContent", type: "node", locale: "${0}", setter: 
						function(item, rule, value){
							var result = this._getNodeByXpath(item.element, "./atom:content/*");
							var modelNode = result.value;
							var node = this._getNodeByXpath(modelNode, "./model:description").value;
							if(!node) {
								node = this._createNSElement(item.element.ownerDocument, "model:description");
								modelNode.appendChild(node);
							}
							// here we're using the locale specified by the parameterized rule that was passed by the caller
							var elem = this._createNSElement(item.element.ownerDocument, "base:nls-string", {"xml:lang": rule.locale});
							this._setNodeProp(elem, rule.targetName, value);
							node.appendChild(elem);
							return true;
						}
					}
				}),
		        _makeModifiable: function() {
					// summary: Modifies the element of this item to become modifiable
					//		for writing back to the server.  Subclasses can override this
					//		to do things like prune an item to only include necessary information
					//		for updates and nothing more.  This is called by backup once after
					//		backing up the original element first.  After this returns,
					//		this item's element may have been modified.  Any queries against
					//		this item should now include logic to check the backup copy as well
					//		if a query doesn't return anything as it could have been deleted by
					//		this function.  Care should be taken to ensure that the query result
					//		specifically detects if nothing was found or if the actual value was
					//		null or empty.
					
					var toRemove = ["./thr:in-reply-to"];
					dojo.forEach(toRemove, function(expr) {
						var nodes = this._getNodesByXpath(this.element, expr).value;
						dojo.forEach(nodes, function(node) {
							node.parentNode.removeChild(node);
						}, this);
					}, this);
					
					this.inherited(arguments);
				}
			}
);

				

}
