/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.services.PortalRestServiceRequest"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.services.PortalRestServiceRequest"] = true;
dojo.provide( "com.ibm.portal.services.PortalRestServiceRequest" );

dojo.require( "com.ibm.ajax.auth" );
dojo.require( "com.ibm.portal.EventBroker" );
dojo.require( "dojo.dnd.common");
dojo.require( "com.ibm.portal.services.PortalRestServiceRequestQueue" );

dojo.declare( "com.ibm.portal.services.ContentHandlerURL",
			  null,
			  {
		  		// summary: Provides convenience methods to construct URLs to the 
				// 		content handler servlet which serves most of the Portal 
				//		REST services.
				
				constructor: function ( /*String*/uri, /*String or int*/levels, /*String*/verb, /*String?*/params ) {
				  	ibm.portal.debug.entry( "ContentHandlerURL.constructor", [ uri, levels, verb, params ]);
				  	//If no URI is passed in, just return null.
					if ( uri == null ) {
						return null;
					}	
					
					if ( !levels ) { 
						levels = 2;
					}

					// check if we need to enforce a specific locale
					var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
					var locale = state.getLocale();
					if (locale) {
						if (params) {
							params += "&locale=" + locale;
						} else {
							params = "&locale=" + locale;
						}
					}
				  
				  	this.url = "";
					if ( uri.charAt(0) == "?" ) {
						this.url = this._fromQueryString( uri, params );	
					}
					else {
						this.url = this._fromURI( uri, levels, "download", params );
					}
					ibm.portal.debug.exit( "ContentHandlerURL.constructor" );
			    },
				_fromQueryString: function ( /*String*/queryString, /*String?*/params  ) {
					// summary: Constructs an url from a given query string.
					// description: Constructs the URL to the content handler servlet using
					//		the given query string. Replaces any '&amp;' characters with '&' 		
					//		since Firefox thinks the 'amp;' is part of the parameter name.
					// queryString: the query string to construct the URL from
					// returns: String - the constructed URL
					ibm.portal.debug.entry( "fromQueryString", [ queryString ] );
				
					var str = ibmPortalConfig["contentHandlerURI"] + queryString;
					str = str.replace(/&amp;/g,"&" );
					if ( params ) {
						str = str + params;
					}
					
					if ( str.indexOf( "rep=compact" ) < 0 && str.indexOf( "rep=full" ) < 0 ) {
						str = str + "&rep=compact";
					}	
				
					ibm.portal.debug.exit( "fromQueryString", [ str ] );
					
					return str; //String	
				},
				_fromURI: function ( /*String*/uri, /*int?*/levels, /*String?*/verb, /*String?*/params ) {
					// summary: Constructs an URL using the given parameters.
					// description: Constructs the URL to the content handler servlet for
					//		the given URI, number of levels to traverse, and the action verb
					//		for the REST service to execute. It is not generally recommended
					//		to request more than 2 levels at a time since this will cause an
					//		expensive tree traversal server-side. The verb is generally expected
					//		to be set to "download".
					// uri: the uri for the resource to request
					// levels: the number of levels to include in the response
					// verb: the action to take (e.g. "view" or "download")
					// returns: String - the constructed URL
					ibm.portal.debug.entry( "ContentHandlerURL._fromURI", [ uri, levels, verb, params ] );
					
					uri = com.ibm.portal.utilities.encodeURI( uri );
					
					var qStr = "?uri=" + uri;
					
					if ( levels ) {
						qStr = qStr + "&levels=" + encodeURIComponent( levels );
					}
					
					if ( verb ) {
						qStr = qStr + "&mode=" + encodeURIComponent( verb );
					}
					if ( params ) {
						qStr = qStr + params;
					}
					
					if ( qStr.indexOf( "rep=compact" ) < 0 && qStr.indexOf( "rep=full" ) < 0 ) {
						qStr = qStr + "&rep=compact";
					}	
					
					return this._fromQueryString( qStr ); //String
				},
				getURI: function() {
					ibm.portal.debug.entry( "ContentHandlerURL.getURI" );
					return com.ibm.portal.utilities.decodeURI( this._extractParamValue( "uri" ) ); //String
				},
				getLevels: function () {
					return this._extractParamValue( "levels" );
				},
				getVerb: function () {
					return this._extractParamValue( "verb" );
				},
				_extractParamValue: function ( /*String*/ paramName ) {
					ibm.portal.debug.entry( "ContentHandlerURL._extractParamValue", [ paramName ] );
					
					var startIndex = this.url.indexOf( paramName );
					var endIndex = this.url.indexOf( "&", startIndex );
					var uriParamStr = this.url.slice( startIndex + paramName.length + 1, endIndex );
					
					ibm.portal.debug.exit( "ContentHandlerURL._extractParamValue", [ uriParamStr ]);
					
					return uriParamStr;
				}
			  }
);

dojo.require( "com.ibm.portal.utilities.html" );

dojo.declare( "com.ibm.portal.services.PortalRestServiceForm",
			  null,
			  {
			  	// summary: Object representation of a form that will be submitted to one of the Portal
			  	//		REST services. Encapsulates all the information about the form necessary to get
			  	//		the form info to the server.
			  	// method: the METHOD of the form (usually GET or POST)
			  	// isMultipart: indicates whether this is a multipart encoded form or not
			  	// encoding: the encoding used to submit the form
			  	// DomId: the ID of the DOM element corresponding to the form
			  	method: "GET",
			  	isMultipart: false,
			  	encoding: "application/x-www-form-urlencoded",
			  	DomId: null,
			  	constructor: function ( /*HTMLElement*/formElem ) {
			  		if ( formElem.getAttributeNode("method" ) ) {
			  			this.method = formElem.getAttributeNode( "method" ).value;	
			  		}
			  		if ( formElem.getAttributeNode( "encType" ) ) {
			  			this.encoding = formElem.getAttributeNode( "encType" ).value;
			  		}
			  		if ( formElem.getAttributeNode( "id" ) ) {	
			  			this.DomId = formElem.getAttributeNode( "id" ).value;
			  		}
			  		else {
			  			DomId = formElem;
			  		}	
			  		this.isMultipart = ( this.encoding == "multipart/form-data");
			  	},
			  	getDOMElement: function () {
			  		// summary: Returns the DOM element for this form.
			  		// returns: the DOM element corresponding to this form
			  		return dojo.byId( this.DomId );
			  	},
			  	submit: function () {
			  		// summary: Submits the form. This is necessary because the native submit function
			  		//		may have been overridden to ensure the form submission was intercepted.
			  		this.getDOMElement().submit();
			  	},
			  	toQuery: function () {
			  		// summary: Translates the form into a query string. Only elements which make sense
			  		//		for a query string are translated (i.e. NOT inputs of type "file").
			  		// returns: a string of name/value pairs suitable for being included as part of a url's
			  		//		query string.
			  		return com.ibm.portal.utilities.html.convertFormToQuery( this.getDOMElement() );
			  	}			  	
			  }
);

com.ibm.portal.services.REQUEST_QUEUE = new com.ibm.portal.services.PortalRestServiceRequestQueue();

dojo.declare( "com.ibm.portal.services.PortalRestServiceRequest",
			  null,
			  {
			  	// summary: Sends a request to one of the Portal REST Services and handles the response.
				// description: Create a new instance of this object for every REST service request that is
				//		sent. The URL is passed in to the constructor. If a form node and form filter is 
				// 		passed into the constructor, the REST service request is handled as a form submit.
				// 		The request can also be executed synchronously if TRUE is passed into the constructor.
				// 		In general, only the model implementations should send these requests.
				
				constructor: function( /*ContentHandlerURL*/feedlocation, /*PortalRestServiceForm?*/form, /*boolean?*/textOnly, /*boolean?*/sync ) {
				  	ibm.portal.debug.entry( "PortalRestServiceRequest.constructor", [ feedlocation, form, textOnly, sync ] );
					this._feedURI = feedlocation.url;
					this._textOnly = textOnly;
					this._sync = sync;
					this._form = form;
					this._customResponseValidator = null;
					this._onauthenticated = null;
					
					//Make sure this isn't undefined.
					if ( !this._sync ) {
							this._sync = false;
					}
					ibm.portal.debug.exit( "PortalRestServiceRequest.constructor" );
			    },
			    setAuthenticationValidator: function ( /*Function*/checkFn ) {
			    	// summary: Sets a function that will be used to check whether the response is valid (i.e. does not require  authentication).
			    	//		The function will take two parameters: one will be the response object (xml, json, string, etc);
			    	//		the second will be the dojo._ioArgs as provided to the dojo.xhr method. The function should return true
			    	//		if the response is valid and false otherwise.
			    	this._customResponseValidator = checkFn;
			    },
			    setOnAuthenticatedHandler: function ( /*Function*/onauth ) {
			    	// summary: Sets a function that CAN be called once authentication has been achieved. IMPORTANT: This handler will NOT
			    	//		be called in the default authentication handler for Portal because it redirects the browser to the login page.
			    	this._onauthenticated = onauth;
			    },
				create: function ( /*String*/data, /*Function*/callbackfn, callbackargs ) {
					// summary: Creates an entry in the backing model.
					// description: Sends a POST request to create an entry in the addressed model.
					// data: the body of the POST request
			    	// callbackfn: the function to call when the operation is complete
			    	// callbackargs: additional arguments to pass to callbackfn
					
			    	this._doXmlHttpRequest( "POST", data, callbackfn, callbackargs );
				},
				read: function ( /*Function?*/callbackfn, /*Object?*/addtlCallbackFnArguments ) {
					// summary: Read the ATOM feed provided by the REST service. 
					// description: If textOnly is set to true, the ATOM feed is returned as text and 
					//		 passed in as a single argument to the specified callback function. 
					//		 Otherwise, the ATOM feed is parsed into an {@link ibm.atom.Feed} object 
					//		 and passed into the callback function along with the original XML Document 
					//		 Object Model object.
					// callbackfn: the function to call when the operation is complete (only valid if
					//		 sync is false)
					// addtlCallbackFnArguments: any arguments to pass through to the callback function
					ibm.portal.debug.entry( "PortalRestServiceRequest.read", [ callbackfn, addtlCallbackFnArguments ] );
					
					if ( !this._sync ) {
						ibm.portal.debug.text( "Queueing request!" );
						var q = com.ibm.portal.services.REQUEST_QUEUE;
						var me = this;
						q.add( 
							   { 
							   	execute: function ( doneCallback ) { 
											com.ibm.portal.EVENT_BROKER.startRequest.fire( { uri: me._feedURI } );
											
											var wrappedCallbackFn = function ( arg1, arg2, arg3, arg4 ) { 
												callbackfn( arg1, arg2, arg3, arg4 );
												if ( doneCallback ) {
													doneCallback();
												}
											}
											
											if ( me._textOnly ) {
												me._retrieveRawFeed( wrappedCallbackFn, addtlCallbackFnArguments );
											}	
											else {
												me._retrieve( wrappedCallbackFn, addtlCallbackFnArguments );
											}
								          }
							   }	          
							);
					}
					else {
						com.ibm.portal.EVENT_BROKER.startRequest.fire( { uri: this._feedURI } );	
						
						if ( this._textOnly ) {
							this._retrieveRawFeed( callbackfn, addtlCallbackFnArguments );
						}	
						else {
							this._retrieve( callbackfn, addtlCallbackFnArguments );
						}
					}	
							
					ibm.portal.debug.exit( "PortalRestServiceRequest.read");
				},
				update: function ( /*String*/data, /*Function*/callbackfn, callbackargs ) {
					// summary: Updates an entry in the backing model. 
					// description: Sends a PUT request to update the addressed entry using the provided data.
					// data: the payload of the request
					// callbackfn: the function to call when the operation is complete
					// callbackargs: any additional arguments to pass to the callbackfn
					
					this._doXmlHttpRequest( "Put", data, callbackfn, callbackargs );
				},
				remove: function ( /*Function*/callbackfn, callbackargs ) {
					// summary: Removes the addressed entry from the backing model. 
					// description: Sends a DELETE request to remove the addressed entry.
					// callbackfn: the function to call when the operation is complete
					// callbackargs: any arguments to be passed to the callbackfn
					
					this._doXmlHttpRequest( "Delete", null, callbackfn, callbackargs );
				},
				_retrieveRawFeed: function ( callbackfn, callbackargs ) {
					ibm.portal.debug.entry( "_retrieveRawFeed", [callbackfn, callbackargs] );
					var me = this;
					dojo.xhrGet({
						url: this._feedURI,
						load: function( type, data, evt ) {
							callbackfn( data, callbackargs );
							com.ibm.portal.EVENT_BROKER.endRequest.fire( { uri: me._feedURI } ); 
						},
						sync: this._sync
					});
					
					ibm.portal.debug.exit( "_retrieveRawFeed" );
				},
				_retrieve: function ( callbackfn, callbackargs, formNode, formFilter ) {
					ibm.portal.debug.entry( "_retrieve", [ callbackfn ] );
					
					if ( this._form && this._form.isMultipart ) {
						this._doIframeRequest( callbackfn, callbackargs );
					}
					else {
						this._doXmlHttpRequest( "Get", null, callbackfn, callbackargs );
					}
					
					ibm.portal.debug.exit( "PortalRestServiceRequest._retrieve" );  
				},
				_doIframeRequest: function ( callbackfn, callbackargs ) {
					// summary: Handles a multi-part form submission.
					// description: Multi-part form submissions are not currently feasible using XMLHttpRequests, so
					// 		we need to submit these forms using IFRAMEs. 
					
					ibm.portal.debug.entry( "PortalRestServiceRequest._doIframeRequest", [ callbackfn ] );
					
					var iframe = null;
					var iframeName = dojo.dnd.getUniqueId(); 
					
					if ( dojo.isIE ) { 
						//Any other method of assigning the onload handler in IE seems to fail miserably...
						iframe = document.createElement( "<iframe name='" + iframeName + "' id='"+iframeName + "' src='about:blank' onload='com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER.handleMultiPartResult(this.id);'></iframe>" );
						//This is necessary so that we actually have the Function object to call when the iframe loads (since we seem to
						//only be able to assign an onload handler using text insertion methods).
						com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER._callbackfns[iframeName]= { fn: callbackfn, args: callbackargs };
						//IE applies an XSLT to an XML result in an IFRAME so we need text only.
						var url = new com.ibm.portal.utilities.HttpUrl( this._feedURI );
						url.addParameter( "ibm.web2.contentType", "text/plain" );
						this._form.getDOMElement().setAttribute( "action", url.toString() );
					}
					else {
						ibm.portal.debug.text( "Creating the iframe... name is: " + iframeName + "; url is: " + this._feedURI );
						iframe = document.createElement( "IFRAME" );
						iframe.setAttribute( "name", iframeName );
						iframe.setAttribute( "id", iframeName );
						var me = this;
						iframe.onload = function() { var xml = window.frames[iframeName].document; callbackfn( "load", xml, null, callbackargs ); com.ibm.portal.EVENT_BROKER.endRequest.fire( { uri: me._feedURI } ); };
						this._form.getDOMElement().setAttribute( "action", this._feedURI );	
					} 
					//Can't set the iframe's display to 'none' or problems recognizing the iframe will result...
					iframe.style.visibility = "hidden";
					iframe.style.height = "1px";
					iframe.style.width = "1px";
					document.body.appendChild( iframe );
					
					//IE Workaround again... the name isn't always correctly assigned from the above createElement call. 
					if(window.frames[iframeName].name != iframeName) { window.frames[iframeName].name = iframeName; }
					
					ibm.portal.debug.text( "Setting the iframe target attribute to: " + iframeName );
					this._form.getDOMElement().setAttribute( "target", iframeName );
					this._form.submit();
					
					ibm.portal.debug.exit( "PortalRestServiceRequest._doIframeRequest" );
				},
				_doXmlHttpRequest: function ( method, body, callbackfn, callbackargs ) {
					// summary: Performs a standard request using an XMLHttpRequest object.
					// description: If an error response code is returned, the Error object is returned instead of the
					// 		xml payload. This means that a check should be done to see if the response is instanceof Error.
					ibm.portal.debug.entry( "PortalRestServiceRequest._doXmlHttpRequest", [ method, body, callbackfn, callbackargs ] );					
					ibm.portal.debug.text( "Attempting to retrieve: " + this._feedURI + " using method: " + method + "; synchronously? " + this._sync );
					var me = this;
					var args = {
						url: this._feedURI,
						content: {},
                        headers: { "X-IBM-XHR":"true" },
						//headers: { "If-Modified-Since": "Thu, 1 Jan 1970 00:00:00 GMT"}, //temporary to force the browser to ignore cached requests
						handle: function( response, ioArgs ) {
							ibm.portal.debug.entry( "PortalRestServiceRequest.handle", [ response, ioArgs ] );
							//Get the location of the resource actually returned by the response.
							//All re-directs have been followed at this point.
							var xhr = ioArgs.xhr;
							
							ibm.portal.debug.text( "XHR object: " + xhr ); 
							var config = com.ibm.portal.services.PortalRestServiceConfig;
							var digest = xhr.getResponseHeader( "X-Request-Digest" );
							
							if ( digest ) {
								config.digest = digest;	
							}
							
							if ( xhr.status == 200 ) {
								var data = response;
								var loc = xhr.getResponseHeader( "IBM-Web2-Location");
								
								if ( loc ) {
									//Check to see if we are going from a public resource to a protected resource.
									//This means a successful login has occurred and we need to force a full page 
									//refresh.
									if ( loc.indexOf( ibmPortalConfig["portalProtectedURI"]) >= 0 && me._feedURI.indexOf( ibmPortalConfig["portalPublicURI"] ) >= 0 ) {
										top.location.href = loc;
										return;
									}
								}
								
								var contentType = xhr.getResponseHeader( "Content-Type" );
								ibm.portal.debug.text( "content-type is: " + contentType );

                                // XHR request returns a content-type of text/html
                                // on cross page wire events, if the web-2 location is present and
                                // points to a non-contenthandler, skip auth check

                                if (/^text\/html/.exec (contentType) && loc &&  (loc.indexOf( ibmPortalConfig["portalProtectedURI"]) >-1 || loc.indexOf( ibmPortalConfig["portalPublicURI"]) >-1) ){
                                    ibm.portal.debug.text( "content-type is text .. follow IBM-Web2-Location" );
									top.location.href = loc;
									return;
								}								


								
								var auth = com.ibm.ajax.auth;
								var authRequired = false;
								
								if ( me._customResponseValidator ) {
									authRequired = me._customResponseValidator( response, ioArgs );
								}
								if ( !authRequired ) {
									authRequired = auth.isAuthenticationRequired( response, ioArgs );
								}
								
								//If the HTML content is returned, this is probably another re-direction so we need to
								//force a full-page refresh.
								if ( authRequired ) {
									auth.authenticationHandler( response, ioArgs, me._onauthenticated );
									return;
								}	
								
								ibm.portal.debug.text( "Read feed: " + me._feedURI );
								
								if ( dojo.isIE ) {
									var doc = dojox.data.dom.createDocument( data );
									callbackfn( "load", doc, xhr, callbackargs );
								}
								else {
									callbackfn( "load", data, xhr, callbackargs );
								}
                            } else if  (xhr.status==401 || xhr.status==0) {
                                ibm.portal.debug.text( "Basic auth 401 found, trigger reload");
                                com.ibm.ajax.auth.authenticationHandler();
                                return;
                            } else {
                                callbackfn( "error", response, xhr, callbackargs );
							}	
							
							com.ibm.portal.EVENT_BROKER.endRequest.fire( { uri: me._feedURI } );
							ibm.portal.debug.exit( "PortalRestServiceRequest.handle" );
						},
						sync: this._sync,
						handleAs: "xml"	
					}
					
					
					if ( this._form ) {
						//DOJO expects the content parameter to be of the format created by
						//dojo.queryToObject(). The contents of the content parameter are sent either
						//as part of the query string (method=GET) or as the body of the request (method=POST).
						args.content = dojo.queryToObject( this._form.toQuery() );
						method = this._form.method;
					}
					
					//Convert to upper case to make comparisons easier. We always convert to "proper" case right before
					//calling the dojo xhrYyy function.
					method = method.toUpperCase();
					
					if ( method != "GET" && method != "POST" ) {
						if ( ibmPortalConfig && ibmPortalConfig.xMethodOverride ) {
							args.headers["X-Method-Override"] = method.toUpperCase();
							method = "Post";	
						}
					}
					
					if ( method == "PUT" && body ) {
						args.putData = body;
					} else if ( method == "POST" && body ) {
						args.postData = body;
					}	
					
					if ( dojo.isIE ) {
						args.content["ibm.web2.contentType"] = "text/xml";
						args.handleAs = "text"; //temp
					}
					var config = com.ibm.portal.services.PortalRestServiceConfig;
					if ( config.timeout ) {
						args.timeout = config.timeout;
					}
					if ( config.digest ) {
						args.content["digest"] = config.digest;
					}
					
					//Match dojo's convention of xhrGet/xhrPost/xhrPut/etc.
					method = com.ibm.portal.utilities.string.properCase( method );
					
					var xhrMethod = dojo["xhr" + method];
					if ( xhrMethod ) {
						xhrMethod( args );
					} else {
						throw new Error( "Invalid request method attempted: " + method );
					}	
						
					ibm.portal.debug.exit( "PortalRestServiceRequest._doXmlHttpRequest" );
				},
				toString: function () {
					return this._feedURI;
				}
			  }
);
com.ibm.portal.services.PortalRestServiceConfig = {
	timeout: null,
	digest: null
}

com.ibm.ajax.auth.setAuthenticationHandler( function () { 
	if ( typeof( document.isCSA ) == "undefined" ) {
		top.location.reload();
	}
	else {
		ibm.portal.debug.entry( "DefaultAuthenticationHandler" );
		ibm.portal.debug.text( "Illegal response content-type detected!" );
		ibm.portal.debug.text( "Parameterized redirect URL is: " + ibmPortalConfig["contentModelBlankURL"] );
		var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
		var fullPageRefreshURL = ibmPortalConfig["contentModelBlankURL"].replace( "-----oid-----", state.getPageSelection() );
		ibm.portal.debug.text( "fullPageRefreshURL is currently: " + fullPageRefreshURL );
		/*if ( loc ) { 
			fullPageRefreshURL = loc;
			ibm.portal.debug.text( "fullPageRefreshURL is now: " + fullPageRefreshURL );
		}*/
	
		if (dojo.cookie("WASReqURL") != null) {
			// create URL to current navigational state
			// we cannot use fullPageRefreshURL because it does not encode the entire nav state
			var fullStateUrl = state.createLinkToCurrentState();
			var cookieStr = "WASReqURL=" + fullStateUrl + "; path=/";
			// does not work with Dojo cookie APIs
			document.cookie=cookieStr;
		}
	
		ibm.portal.debug.text( "Redirecting to: " + fullPageRefreshURL );
		
		//Fire the redirect event so any necessary clean up can occur.
		com.ibm.portal.EVENT_BROKER.redirect.fire( { url: fullPageRefreshURL } ); 
		top.location.href = fullPageRefreshURL;
		ibm.portal.debug.exit( "DefaultAuthenticationHandler" );
	}
});

}
