/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.portal.utilities"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.utilities"] = true;
dojo.provide( "com.ibm.portal.utilities" );

com.ibm.portal.utilities = {
	findPortletIdByElement: function ( /*DOMElement*/ element ) {
		// summary: Finds the portlet ID which the given element belongs to
		// description: Finds the portlet ID to which the give DOM element belongs
		//		by walking up the parents of the given element until the ancestor is 
		//		found with one of the defined class names.
		// element: the DOM element to find the portlet ID of
		
		ibm.portal.debug.entry( "findPortletID", [ element ] );
		var id = "";
		var current = element.parentNode;
		while ( current && id.length == 0 ) {
			ibm.portal.debug.text( "examining element " + current.tagName + "; class=" + current.className,"findPortletID" );
		
			if ( current.className && (current.className.match(/\bwpsPortletBody\b/) || current.className.match(/\bwpsPortletBodyInlineMode\b/)) ) {
				id = current.id;
				var indexOf = id.indexOf( "_mode" );
				if ( indexOf >= 0 )  {
					id = id.substring( 0, indexOf );
				}
			}
			current = current.parentNode;	
		}
		
		if ( id.indexOf( "portletActions_" ) >= 0 ) {
			id = id.substring( "portletActions_".length );
		}
		
		ibm.portal.debug.exit( "findPortletID", [ id ] );
		
		return id;
	},
	findFormByElement: function ( /*DOMElement*/ element) {
		var current = element;
		while(current) {
			if(current.tagName && current.tagName.toLowerCase() == "form") break;
			current = current.parentNode;
		}
		return current;
	},
	encodeURI: function ( /*String*/uri ) {
		// summary: Encodes a model uri for use in an URL.
		// uri: the unencoded model uri to encode
		// returns: the encoded uri ready for use in an URL
		ibm.portal.debug.entry( "encodeURI", [ uri ] );
		
		var encodedURI = uri;
		var index = uri.lastIndexOf( ":" );
		
		while ( index >= 0 ) {
			var scheme = encodedURI.substring(0, index );
			var part = encodedURI.substring( index + 1  );
			
			encodedURI = scheme + ":" + encodeURIComponent( part );
			index = scheme.lastIndexOf( ":" );
		}
		
		encodedURI = encodeURIComponent( encodedURI );
		
		ibm.portal.debug.exit( "encodeURI", [ encodedURI ] );
		return encodedURI; // String
	},
	decodeURI: function ( /*String*/uri ) { 
		// summary: Decodes a model uri.
		// uri: the encode model uri
		// returns: the decoded model uri
		ibm.portal.debug.entry( "decodeURI", [ uri ] );
		
		var decodedURI = decodeURIComponent( uri );
		var index = decodedURI.indexOf( ":" );
		
		while ( index >= 0 ) {
			var scheme = decodedURI.substring(0, index );
			var part = decodedURI.substring( index + 1 );
			
			decodedURI = scheme + ":" + decodeURIComponent( part );
			index = decodedURI.indexOf( ":", index+1 );
		}
		
		ibm.portal.debug.exit( "decodeURI", [ decodedURI ] );
		return decodedURI;	//String
	},
	getSelectionNodeId: function ( /*String*/fullPortletId ) {
		// summary: Extracts the selection node object id from the full portlet id returned
		//		by the layout model feed.
		ibm.portal.debug.entry( "getSelectionNodeId", [ fullPortletId ] );
		
		// this split will break the full portlet id into the control with prefixes at index [0]
		// and the selected node will be at index [1]
		var selNodeId = fullPortletId.split('@oid:');
				
		ibm.portal.debug.exit( "getSelectionNodeId", [ selNodeId[1] ] );
		return selNodeId[1];
	},
	getControlId: function ( /*String*/fullPortletId ) {
		// summary: Extracts the control object id from the full portlet id returned
		//		by the feed.
		ibm.portal.debug.entry( "_getControlId", [ fullPortletId ] );
		
		// this first split will get rid of the page oid
		var controlParts = fullPortletId.split('@oid:');
		
		// this second split will leave the feed prefix (i.e. lm) at index [0]
		// and index [1] will have the control oid
		var controlId = controlParts[0].split("oid:");
		
		ibm.portal.debug.exit( "getControlId", [ controlId[1] ] );
		return controlId[1];
	},
	overwriteProperty: function ( /*Object*/obj, /*String*/propertyName, /*Object*/newValue, /*Boolean?*/onlyOnce ) {
		// summary: Overwrites the given property.
		ibm.portal.debug.entry( "overwriteProperty", [ obj, propertyName, newValue, onlyOnce ] );
		
		if ( !obj["_overwritten_"] ) {
			obj["_overwritten_"] = new Object();
		}
		
		//In case onlyOnce is undefined.
		if ( !onlyOnce ) { onlyOnce = false; }
		
		var skipOverwrite = (onlyOnce &&  (obj["_overwritten_"][propertyName] != null) );
		if ( !skipOverwrite ) {
			if ( obj["_overwritten_"][propertyName] == null ) {
                obj["_overwritten_"][propertyName] = obj[propertyName];
            }
            else {
                obj["_overwritten_"][propertyName] = null;
            }    
			obj[propertyName] = newValue;
			ibm.portal.debug.text( "Property overwrite successful!" );
		}
		
		ibm.portal.debug.exit( "overwriteProperty");	
	},
	restoreProperty: function ( /*Object*/obj, /*String*/propertyName ) {
		// summary: Restores an overwritten property.
        // returns: the overwritten value for the property
		ibm.portal.debug.entry( "utilities.restoreProperty", [ obj, propertyName ] );
		var retVal = obj[propertyName];

        if ( obj["_overwritten_"] != null ) {
            ibm.portal.debug.text( "overwritten property value: " + obj["_overwritten_"] ); 
            obj[propertyName] = obj["_overwritten_"][propertyName];
            obj["_overwritten_"][propertyName] = null;
        }
        else {
            obj[propertyName] = null;
        }
        
        ibm.portal.debug.exit( "utilities.restoreProperty", retVal );    
        return retVal;
	},
	getOverwrittenProperty: function ( /*Object*/obj, /*String*/propertyName ) {
		// summary: Access an overwritten property.
		if ( obj["_overwritten_"] ) {
			return obj["_overwritten_"][propertyName];
		}
		else {
			return null;
		}	
	},
	setOverwrittenProperty: function ( /*Object*/obj, /*String*/propertyName, /*Object*/value ) {
		// summary: Sets the overwritten property directly; bypassing the overwriting process.
		ibm.portal.debug.entry( "utilities.setOverwrittenProperty", [ obj, propertyName, value ] ); 
		
		if ( !obj["_overwritten_"] ) {
			obj["_overwritten_"] = new Object();
		}
		
		obj["_overwritten_"][propertyName] = value;
		
		ibm.portal.debug.exit( "utilities.setOverwrittenProperty" );
	},
	callOverwrittenFunction: function ( /*Object*/thisObj, /*String*/propertyName, /*Object[]?*/args ) {
		ibm.portal.debug.entry( "utilities.callOverwrittenFunction", [ thisObj, propertyName, args ]);
		//IE doesn't like it if we pass in an undefined args object to apply.
		var retVal = null;
		var oldProp = this.getOverwrittenProperty( thisObj, propertyName );
		ibm.portal.debug.text( "Overwritten property: " + oldProp );
		ibm.portal.debug.text( "old property's apply function: " + oldProp.apply );
		
		if ( args ) {
			retVal = oldProp.apply( thisObj, args );
		}
		else {
			retVal = oldProp.apply( thisObj );
		}
		
		ibm.portal.debug.exit( "utilities.callOverwrittenFunction", retVal );
		return retVal;
	},
	isExternalUrl: function ( /*String*/urlString ) {
		// summary: Determines if the given url points to an external server.
		// urlString: the url in string form
		// returns: true if the url points to an external server
		ibm.portal.debug.entry( "isExternalUrl", [ urlString ] );
		
		var host = window.location.host;
		var protocol = window.location.protocol;
		var urlStringNoQuery = urlString.split( "?" )[0];
		var retVal = !(urlStringNoQuery.indexOf("://") < 0 || (urlStringNoQuery.indexOf(protocol) == 0 && urlStringNoQuery.indexOf(host) == protocol.length+2) );
		
		ibm.portal.debug.text( "urlStringNoQuery.indexOf(\"://\") = " + urlStringNoQuery.indexOf("://") );
		ibm.portal.debug.text( "urlStringNoQuery.indexOf(protocol) = " + urlStringNoQuery.indexOf(protocol) );
		
		ibm.portal.debug.exit( "isExternalUrl", retVal );
		return retVal;
	},
	isJavascriptUrl: function ( /*String*/urlString ) {
		// summary: Determines if the given url is a javascript url.
		// description: Checks to see if the url begins with the javascript: protocol
		// urlString: the url in string form
		// returns: true if the url is actually a javascript call.
		ibm.portal.debug.entry( "isJavascriptUrl", [ urlString ] );
		var url = com.ibm.portal.utilities.string.trim( urlString.toLowerCase() );
		var retVal = ( url.indexOf( "javascript:" ) == 0 );
		ibm.portal.debug.exit( "isJavascriptUrl", retVal );
		return retVal;
	},
	isPortalUrl: function ( /*String*/urlString ) {
		// summary: Determines if a given url is a url to the portal servlet.
		// description: Checks to see if the url begins with the path to the portal servlet.
		// urlString: the url to check in string form
		// returns: true if the url is a portal url
		ibm.portal.debug.entry( "utilities.isPortalUrl", [ urlString ] );
		
		var retVal = ( urlString.indexOf( ibmPortalConfig["portalURI"] ) >= 0 );
		
		ibm.portal.debug.exit( "utilities.isPortalUrl", retVal );
		return retVal;
	},	

	addExternalNode: function ( /*DOMDocument*/ doc, /*Element*/ node ) {
		var newNode = null;
		if(doc.importNode) newNode = doc.importNode(node, true);
		else newNode = node;
		doc.appendChild( newNode );
	},
    decodeXML: function ( /*String*/xmlText ) {
		// summary: decodes xml sent by multi-part form 
		ibm.portal.debug.entry( "decodeXML", [ xmlText ] );

		var newXmlText1 = xmlText.replace(/&amp;/g, "&");
		var newXmlText2 = newXmlText1.replace(/&amp;/g, "&");
		newXmlText1 = newXmlText2.replace(/&#039;/g, "\'");
		newXmlText2 = newXmlText1.replace(/&#034;/g, '\"');
		
		newXmlText2 = newXmlText2.replace( /&lt;/g, "<" );
		newXmlText2 = newXmlText2.replace( /&gt;/g, ">" );

		ibm.portal.debug.exit( "decodeXML", [ newXmlText2 ] );
		return newXmlText2;
	},
	eventHandlerToString: function ( eventHandler ) {
		var eventHandlerStr = eventHandler.toString();
		var startIndex = eventHandlerStr.indexOf("{");
		var endIndex = eventHandlerStr.lastIndexOf("}");
		onclickStr = eventHandlerStr.substring(startIndex+1, endIndex);
		
		return onclickStr;
	},
	_waitingForScript: false,
	_isWaitingForScript: function() { return com.ibm.portal.utilities._waitingForScript; },
	stopWaitingForScript: function() { com.ibm.portal.utilities._waitingForScript = false; },

	waitFor: function(testFunction, pollInterval, callback, args) {
		var interval = setInterval(function() {
			if (testFunction())
			{
				clearInterval(interval);
				
				if ( !args ) {
					callback();
				}
				else {
					callback(args);
				}
			}
		},
		pollInterval);
	},

	waitForScript: function( callback, args ) {
		com.ibm.portal.utilities._waitingForScript = true;
		com.ibm.portal.utilities.waitFor(function()
		{
			return (!com.ibm.portal.utilities._isWaitingForScript());
		},
		500,
		callback,
		args);
	}
}

com.ibm.portal.utilities.string = {
	findNext: function ( /*String*/ string, /*Array*/searchStrings, /*int?*/from ) {
		// summary: Finds the next index of any of the given search strings.
		// description: Looks for each search string in the given string and returns the index 
		//		of the first one encountered. If none of the search strings are discovered, 
		//		-1 is returned. 
		// string: the string to search
		// searchStrings: the array of strings to look for in the string
		// from: the index to start looking from - the length of the search string is always added 
		//		from index before searching for the next instance
		ibm.portal.debug.entry( "string.findNext", [ string, searchStrings ] );
		
		var lowValueNotZero = -1;
		
		for ( var i = 0; i < searchStrings.length; i++ ) {
			var fromIndex = null;
			if ( from ) {
				fromIndex = from + searchStrings[i].length;
			}
			var index = string.indexOf( searchStrings[i], fromIndex );
				
			if ( index > -1 && ( index < lowValueNotZero || lowValueNotZero == -1 ) ) {
				lowValueNotZero = index;
			}
		}
		
		ibm.portal.debug.exit( "string.findNext", [ lowValueNotZero ] );
		return lowValueNotZero;		
	},
	contains: function ( /*String*/ stringToSearch, /*String*/stringToLookFor ) {
		// summary: Determines if the stringToSearch contains the stringToLookFor
		ibm.portal.debug.entry( "string.contains", [ stringToSearch, stringToLookFor ] );
		
		var retVal = false;
		
		if ( stringToSearch != null && stringToLookFor != null ) {
			retVal = ( stringToSearch.indexOf( stringToLookFor ) != -1 );		
		}
		
		ibm.portal.debug.exit( "string.contains", [ retVal ] );
		return retVal;
	},
	strip: function ( /*String*/string, /*String*/stringToStrip ) {
		// summary: Strips out the given string from the original string if it contains
		//		the stringToStrip.
		// string: the original string
		// stringToStrip: the string to remove from the original string
		// returns: the original string without any instances of the stringToStrip
		ibm.portal.debug.entry( "string.strip", [ string, stringToStrip ] );
		
		var retVal = string.replace( new RegExp( stringToStrip, "g" ), "" );
				
		ibm.portal.debug.exit ( "string.strip", [ retVal ]);
		return retVal;
	},
	properCase: function ( /*String*/string ) {
		// summary: Changes the first character of the string to Upper Case and all remaining characters
		//		to Lower Case.  Note that this does not split the string into words and capitalize
		//		each word, but only acts on the entire string as a whole.
		// string: The original string to change
		// returns: The original string with the first character in Upper Case and all other characters
		//		in Lower Case.
		if(string == null || string.length < 1) return "";
		ibm.portal.debug.entry("string.properCase", [string] );
		var retVal = string.charAt(0).toUpperCase();
		if(string.length > 1) {
			retVal += string.substring(1).toLowerCase();
		}
		ibm.portal.debug.exit("string.properCase", [retVal] );
		return retVal;
	},
	trim: function ( /*String*/ string ) {
		// summary: Removes leading and trailing spaces from a string.
		// string: the string to trim
		// returns: the original string minus any leading or trailing spaces
		ibm.portal.debug.entry( "string.trim", [ string ] );
		var retVal = string;
		
		retVal = retVal.replace( /^\s+/, '' );
		retVal = retVal.replace( /\s+$/, '' );
		
		ibm.portal.debug.exit( "string.trim", retVal );
		return retVal;
	}
}

dojo.declare( "com.ibm.portal.utilities.HttpUrl", 
			  null,
			  {
			  	constructor: function ( /*String*/urlString ) {
					this.scheme = "http://";
				  	this.server = this._extractServer( urlString );
					this.port = this._extractPort( urlString );
					this.path = this._extractPath( urlString );
					this.query = this._extractQuery( urlString );
					this.anchor = "";
				},
			  	addParameter: function ( /*String*/name, /*String*/value ) {
					this.query += "&" + name + "=" + value;
				},
				toString: function () {
					var str = "";
					if ( this.server != "" ) {
						str += this.scheme + this.server;
					} 
					if ( this.port != "" ) {
						str += ":" + this.port;
					}
					
					str += "/" + this.path;
					
					if ( this.query != "" ) {
						str += "?" + this.query;
					}
					if ( this.anchor != "" ) {
						str += "#" + this.anchor;
					}
					
					return str;
				},
				_extractServer: function (/*String*/urlString ) {
					var indexOfScheme = urlString.indexOf( this.scheme );
					var retVal = "";
					if ( indexOfScheme == 0 ) {
						var indexOfSlash = urlString.indexOf( "/", indexOfScheme + this.scheme.length );
						var serverAndPort = urlString.substring( indexOfScheme+ this.scheme.length, indexOfSlash );
						retVal = serverAndPort.split( ":" )[0];
					}	
					
					return retVal; 
				},
				_extractPort: function (/*String*/urlString ) {
					var indexOfServer = urlString.indexOf( this.server );
					var retVal = "";
					if ( indexOfServer >= 0 ) {
						var indexOfSlash = urlString.indexOf( "/", indexOfServer );
						var serverAndPort = urlString.substring( indexOfServer, indexOfSlash );
						var serverAndPortParts = serverAndPort.split( ":" );
						if ( serverAndPortParts.length > 1 ) {
							retVal = serverAndPortParts[1];
						}
					}	
					
					return retVal; 
				},
				_extractPath: function ( /*String*/urlString ) {
					var indexOfServer = urlString.indexOf( this.server );
					var retVal = "";
					if ( indexOfServer >= 0 ) {
						var indexOfSlash = urlString.indexOf( "/", indexOfServer );
						var indexOfQuery = urlString.indexOf( "?" );
						var indexOfHash = urlString.lastIndexOf( "#" );
						if ( indexOfQuery >= 0 ) {
							retVal = urlString.substring( indexOfSlash + 1, indexOfQuery );
						}
						else {
							if ( indexOfHash >= 0 ) {
								retVal = urlString.substring( indexOfSlash + 1, indexOfHash );
							}
							else {
								retVal = urlString.substring( indexOfSlash + 1 );
							}		
						}
					}	
					
					return retVal;
				},
				_extractQuery: function ( /*String*/urlString ) {
					var retVal = "";
					var urlParts = urlString.split( "?" );
					
					if ( urlParts.length > 1 ) {
						retVal = urlParts[1].split( "#" )[0];
					}	
					
					return retVal;
				},
				_extractAnchor: function ( /*String*/urlString ) {
					var retVal = "";
					var urlParts = urlString.split( "#" );
					
					if ( urlParts.length > 1 ) {
						retVal = urlParts[urlParts.length - 1];
					}	
					
					return retVal;
				}
					
			  }
);

}
