/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.strategy.Strategy"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.strategy.Strategy"] = true;
/*
 * @author Ben
 */ 
dojo.provide("com.ibm.strategy.Strategy");

dojo.require("dojox.lang.aspect");

dojo.declare("com.ibm.strategy.Strategy",
	null,
	{
		// extensions: Array
		//		Array of strings indicating which functions in this strategy should be used
		//		to wrap functions of the same name in the object this strategy is applied to.
		//		Extensions cannot be named "strategyInit", "around", or "proceed", as these
		//		reserved words for strategy management.  If an extension is defined here
		//		but no matching mapped function is found in the extensionsMap object AND no
		//		function with the same name as the function to extend is found in this strategy,
		//		then the strategy will simply call the original function without special
		//		handling.
		extensions: [],
		
		// extensionsMap: Object
		//		Map of key-value pairs indicating what functions in the strategy are called
		// 		when a function on the wrapped object is called.  Keys in this map correspond
		//		to function names on the wrapped object, and should also appear in this strategy's
		//		extensions array.  Values associated with the keys correspond to the function
		//		in this strategy that are called when the wrapped function on the original object
		//		is called.  Note that these mappings are optional, and if a mapping does not
		//		exist in this map for an extension found in the extensions array, then the function
		//		called in this strategy is the same name as the function it wrapped.  Because of
		//		this, this extensionsMap object itself is optional and can be null to provide
		// 		default wrapping where the extension function names are the same in the original
		// 		object and the strategy wrapping them.
		extensionsMap: {},
		
		strategyInit: function(obj) {
			// summary: Function called when a strategy instance is applied to an object.
			// obj: The object that this strategy is applied to.			
		},
		
		getStrategyContext: function() {
			return dojox.lang.aspect.getContext();
		},
		
		around: function() {
			// summary: Helper function used to manage the around advice that this strategy
			//		incorporates using its extension functions.
			var fName = this.getStrategyContext().joinPoint.targetName;
			if(this.extensionsMap[fName]) {
				return this[this.extensionsMap[fName]].apply(this, arguments);
			}
			else if(this[fName]) {
				return this[fName].apply(this, arguments);
			}
			else {
				return this.proceed(arguments);
			}
		},
		
		proceed: function(args) {
			// summary: Helper function to proceed to the next function in the advice
			//		wrapping chain.  Strategy extensions can call this.proceed(arguments)
			//		to call the next function in the chain of their current context.
			return dojox.lang.aspect.proceed.apply(null, args);
		}
	}
);

com.ibm.strategy.Strategy.applyStrategy = function(obj, strategy) {
	strategy.strategyInit(obj);
	var adv = dojox.lang.aspect.advise(obj, strategy.extensions, strategy);
	return adv;
}

com.ibm.strategy.Strategy.removeStrategy = function(adv) {
	dojox.lang.aspect.unadvise(adv);
}

}
