/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.widgets.SelectionModeled"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.widgets.SelectionModeled"] = true;
dojo.provide("com.ibm.widgets.SelectionModeled");

dojo.require("com.ibm.widgets.TreeModeled");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.bundles", "Theme", null, "ROOT,ar,ca,cs,da,de,el,en,es,fi,fr,he,hr,hu,it,ja,ko,nl,no,pl,pt,pt-br,ro,ru,sk,sl,sv,th,tr,uk,zh,zh-tw");

dojo.declare( "com.ibm.widgets.SelectionModeled",
			com.ibm.widgets.TreeModeled,
			{
				// summary:
				//		Mixin for building widgets from a selection model, which is a list structure
				//		that represents a current selection in a model starting at the root and walking
				//		down a tree to the currently selected node.
				
				// level: Number the level of the current modeled widget in a heirarchy of modeled
				//		widgets.  This represents the level in the tree from the root where this
				//		widget is rooted.  From the selection model's point of view, it is the index
				//		into the reverse selection path (since that goes up the tree instead).
				level: 0,
				
				// selectionModel: An instance of the com.ibm.data.SelectionModel class representing
				//		a selection model.
				selectionModel: null,
				
				// selectedClass: String name of the CSS class to apply to a node that is in the
				//		selection path.
				selectedClass: "",
				
				// selectedChild: Child widget which is currently marked as selected, indicating that
				//		it is part of the selection model.
				selectedChild: null,
				
				// tiedToSelection: Boolean flag indicating if this widget is tied to the selection
				//		model at this level.  If this is true, then if the selection path changes
				//		at this level, this widget will reload itself completely to root itself
				//		at the new item in the selection path at this level.
				tiedToSelection: false,
				
				constructor: function(){
					this.bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Theme");
					this.currentlySelected = this.bundle["currently_selected"];
				},
				postMixInProperties: function() {
					this.inherited(arguments);
					if(this.tiedToSelection) this.item = this.selectionModel.atLevel(this.level, true);
				},
				initializeModelWidget: function() {
					this.inherited(arguments);
					this.connect(this.selectionModel, "onSelectionPathChange", "_updateSelection");
				},
				_markSel: function(childWidget, value) {
					if(value) {
						if(this.selectedChild) this.markSelected(this.selectedChild, false);
						this.selectedChild = childWidget;
					}
					this.markSelected(childWidget, value);
				},
				markSelected: function(childWidget, value) { 
					// summary: Marks the childWidget as selected or unselected depending on the value.
					//		By default, adds/removes the selectedClass to the childWidget's domNode.
					dojo[value ? "addClass" : "removeClass"](childWidget.domNode, this.selectedClass);
					if(childWidget.labelNode){
						if(value){  
							// add a strong tag to a selected child widget
							var label = childWidget.labelNode.innerHTML,
								selectedLabel = "";
							dojo.empty(childWidget.labelNode);
							
							if(this.selectionModel.selectionPath[0] == childWidget.item){

								var hiddenText = dojo.create("span", {innerHTML: this.currentlySelected, className: "ibmPortalHiddenText"});
								var hiddenSpan = dojo.create("span");
								hiddenSpan.appendChild(hiddenText);
								selectedLabel = dojo.create("strong",{innerHTML:label+hiddenSpan.innerHTML});
							}else{
								selectedLabel = dojo.create("strong",{innerHTML:label});
							}
							
							childWidget.labelNode.appendChild(selectedLabel);
						} else { 
							// remove a strong tag from a unselected child widget
							var label = dojo.query("strong",childWidget.labelNode); 
							if(label[0]) childWidget.labelNode.innerHTML = label[0].innerHTML; 
						}
					}
				},
				_updateSelection: function() {
					var curLevelItem = this.selectionModel.atLevel(this.level, true),
						lastLevelItem = this.selectionModel.atLevel(this.level + 1, true);
					if(curLevelItem != null){
						if(this.tiedToSelection && this.model.getIdentity(this.item) != this.model.getIdentity(curLevelItem) ) {
							this.item = curLevelItem;
							this.reload();
						}
						else {
							var curLevelId = curLevelItem ? this.model.getIdentity(curLevelItem) : null,
								levelId = lastLevelItem ? this.model.getIdentity(lastLevelItem) : null;
							
							if(curLevelItem && curLevelId != levelId && this.items[curLevelId]) {
								this._markSel(this.items[curLevelId], false);
							}
							if(levelId && this.items[levelId]) {
								this._markSel(this.items[levelId], true);
							}
						}
					}
				},
				_setupNewChild: function(item) {
					var childWidget = this.inherited(arguments);
					if(this.selectionModel.atLevel(this.level + 1, true) == item) {
						this._markSel(childWidget, true);
					}
					return childWidget;
				},
				getInheritableParameters: function(item) {
					return dojo.mixin(this.inherited(arguments), {
						selectionModel: this.selectionModel,
						selectedClass: this.selectedClass
					});
				},
				onModelItemClick: function(widget, item) {
					// summary: Extension point for getting notifications when users click
					//		a widget representing an item in this model.  By default, this
					//		is propogated to all cascading modeled widgets created by the
					//		model so that all descendants fire the same function.
					//		Implementations should ensure that any onclick event on a child 
					// 		widget representing	modeled data calls results in a call to this 
					// 		function with the appropriate arguments. 
					//		For SelectionModeled widgets, this changes the current selection
					//		to the widget's item that was clicked.
					if(widget.disabled) return false;
					var id = this.model.getIdentity(item);
						thisSel = this.selectionModel.atLevel(this.level, true),
						thisSelId = thisSel ? this.model.getIdentity(thisSel) : null;
					if(thisSelId != id) {
						// didn't click the current selection
						this.selectionModel.changeSelection(item);
						return true;
					}
					return false;
				}
			}
);

}
