/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/
dojo.provide("com.ibmCSA");
if(!dojo._hasResource["dojo.back"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.back"] = true;
dojo.provide("dojo.back");

/*=====
dojo.back = {
	// summary: Browser history management resources
}
=====*/


(function(){ 
	var back = dojo.back;

	// everyone deals with encoding the hash slightly differently

	function getHash(){ 
		var h = window.location.hash;
		if(h.charAt(0) == "#"){ h = h.substring(1); }
		return dojo.isMozilla ? h : decodeURIComponent(h); 
	}
	
	function setHash(h){
		if(!h){ h = ""; }
		window.location.hash = encodeURIComponent(h);
		historyCounter = history.length;
	}
	
	// if we're in the test for these methods, expose them on dojo.back. ok'd with alex.
	if(dojo.exists("tests.back-hash")){
		back.getHash = getHash;
		back.setHash = setHash;		
	}
	
	var initialHref = (typeof(window) !== "undefined") ? window.location.href : "";
	var initialHash = (typeof(window) !== "undefined") ? getHash() : "";
	var initialState = null;

	var locationTimer = null;
	var bookmarkAnchor = null;
	var historyIframe = null;
	var forwardStack = [];
	var historyStack = [];
	var moveForward = false;
	var changingUrl = false;
	var historyCounter;

	function handleBackButton(){
		//summary: private method. Do not call this directly.

		//The "current" page is always at the top of the history stack.
		var current = historyStack.pop();
		if(!current){ return; }
		var last = historyStack[historyStack.length-1];
		if(!last && historyStack.length == 0){
			last = initialState;
		}
		if(last){
			if(last.kwArgs["back"]){
				last.kwArgs["back"]();
			}else if(last.kwArgs["backButton"]){
				last.kwArgs["backButton"]();
			}else if(last.kwArgs["handle"]){
				last.kwArgs.handle("back");
			}
		}
		forwardStack.push(current);
	}

	back.goBack = handleBackButton;

	function handleForwardButton(){
		//summary: private method. Do not call this directly.
		var last = forwardStack.pop();
		if(!last){ return; }
		if(last.kwArgs["forward"]){
			last.kwArgs.forward();
		}else if(last.kwArgs["forwardButton"]){
			last.kwArgs.forwardButton();
		}else if(last.kwArgs["handle"]){
			last.kwArgs.handle("forward");
		}
		historyStack.push(last);
	}

	back.goForward = handleForwardButton;

	function createState(url, args, hash){
		//summary: private method. Do not call this directly.
		return {"url": url, "kwArgs": args, "urlHash": hash};	//Object
	}

	function getUrlQuery(url){
		//summary: private method. Do not call this directly.
		var segments = url.split("?");
		if(segments.length < 2){
			return null; //null
		}
		else{
			return segments[1]; //String
		}
	}
	
	function loadIframeHistory(){
		//summary: private method. Do not call this directly.
		var url = (dojo.config["dojoIframeHistoryUrl"] || dojo.moduleUrl("dojo", "resources/iframe_history.html")) + "?" + (new Date()).getTime();
		moveForward = true;
        if(historyIframe){
		    dojo.isWebKit ? historyIframe.location = url : window.frames[historyIframe.name].location = url;
        }else{
            //console.warn("dojo.back: Not initialised. You need to call dojo.back.init() from a <script> block that lives inside the <body> tag.");
        }
		return url; //String
	}

	function checkLocation(){
		if(!changingUrl){
			var hsl = historyStack.length;
			
			var hash = getHash();

			if((hash === initialHash||window.location.href == initialHref)&&(hsl == 1)){
				// FIXME: could this ever be a forward button?
				// we can't clear it because we still need to check for forwards. Ugg.
				// clearInterval(this.locationTimer);
				handleBackButton();
				return;
			}
			
			// first check to see if we could have gone forward. We always halt on
			// a no-hash item.
			if(forwardStack.length > 0){
				if(forwardStack[forwardStack.length-1].urlHash === hash){
					handleForwardButton();
					return;
				}
			}
	
			// ok, that didn't work, try someplace back in the history stack
			if((hsl >= 2)&&(historyStack[hsl-2])){
				if(historyStack[hsl-2].urlHash === hash){
					handleBackButton();
					return;
				}
			}
			
			if(dojo.isSafari && dojo.isSafari < 3){
				var hisLen = history.length;
				if(hisLen > historyCounter) handleForwardButton();
				else if(hisLen < historyCounter) handleBackButton();
			  historyCounter = hisLen;
			}
		}
	};
	
	back.init = function(){
		//summary: Initializes the undo stack. This must be called from a <script> 
		//         block that lives inside the <body> tag to prevent bugs on IE.
		if(dojo.byId("dj_history")){ return; } // prevent reinit
		var src = dojo.config["dojoIframeHistoryUrl"] || dojo.moduleUrl("dojo", "resources/iframe_history.html");
		document.write('<iframe style="border:0;width:1px;height:1px;position:absolute;visibility:hidden;bottom:0;right:0;" name="dj_history" id="dj_history" src="' + src + '"></iframe>');
	};

	back.setInitialState = function(/*Object*/args){
		//summary: 
		//		Sets the state object and back callback for the very first page
		//		that is loaded.
		//description:
		//		It is recommended that you call this method as part of an event
		//		listener that is registered via dojo.addOnLoad().
		//args: Object
		//		See the addToHistory() function for the list of valid args properties.
		initialState = createState(initialHref, args, initialHash);
	};

	//FIXME: Make these doc comments not be awful. At least they're not wrong.
	//FIXME: Would like to support arbitrary back/forward jumps. Have to rework iframeLoaded among other things.
	//FIXME: is there a slight race condition in moz using change URL with the timer check and when
	//       the hash gets set? I think I have seen a back/forward call in quick succession, but not consistent.

	
	/*=====
	dojo.__backArgs = function(kwArgs){
		// back: Function?
		//		A function to be called when this state is reached via the user
		//		clicking the back button.
		//	forward: Function?
		//		Upon return to this state from the "back, forward" combination
		//		of navigation steps, this function will be called. Somewhat
		//		analgous to the semantic of an "onRedo" event handler.
		//	changeUrl: Boolean?|String?
		//		Boolean indicating whether or not to create a unique hash for
		//		this state. If a string is passed instead, it is used as the
		//		hash.
	}
	=====*/

	back.addToHistory = function(/*dojo.__backArgs*/ args){
		//	summary: 
		//		adds a state object (args) to the history list. 
		//	description:
		//		To support getting back button notifications, the object
		//		argument should implement a function called either "back",
		//		"backButton", or "handle". The string "back" will be passed as
		//		the first and only argument to this callback.
		//	
		//		To support getting forward button notifications, the object
		//		argument should implement a function called either "forward",
		//		"forwardButton", or "handle". The string "forward" will be
		//		passed as the first and only argument to this callback.
		//
		//		If you want the browser location string to change, define "changeUrl" on the object. If the
		//		value of "changeUrl" is true, then a unique number will be appended to the URL as a fragment
		//		identifier (http://some.domain.com/path#uniquenumber). If it is any other value that does
		//		not evaluate to false, that value will be used as the fragment identifier. For example,
		//		if changeUrl: 'page1', then the URL will look like: http://some.domain.com/path#page1
		//
	 	//	example:
		//		|	dojo.back.addToHistory({
		//		|		back: function(){ console.log('back pressed'); },
		//		|		forward: function(){ console.log('forward pressed'); },
		//		|		changeUrl: true
		//		|	});

		//	BROWSER NOTES:
		//  Safari 1.2: 
		//	back button "works" fine, however it's not possible to actually
		//	DETECT that you've moved backwards by inspecting window.location.
		//	Unless there is some other means of locating.
		//	FIXME: perhaps we can poll on history.length?
		//	Safari 2.0.3+ (and probably 1.3.2+):
		//	works fine, except when changeUrl is used. When changeUrl is used,
		//	Safari jumps all the way back to whatever page was shown before
		//	the page that uses dojo.undo.browser support.
		//	IE 5.5 SP2:
		//	back button behavior is macro. It does not move back to the
		//	previous hash value, but to the last full page load. This suggests
		//	that the iframe is the correct way to capture the back button in
		//	these cases.
		//	Don't test this page using local disk for MSIE. MSIE will not create 
		//	a history list for iframe_history.html if served from a file: URL. 
		//	The XML served back from the XHR tests will also not be properly 
		//	created if served from local disk. Serve the test pages from a web 
		//	server to test in that browser.
		//	IE 6.0:
		//	same behavior as IE 5.5 SP2
		//	Firefox 1.0+:
		//	the back button will return us to the previous hash on the same
		//	page, thereby not requiring an iframe hack, although we do then
		//	need to run a timer to detect inter-page movement.

		//If addToHistory is called, then that means we prune the
		//forward stack -- the user went back, then wanted to
		//start a new forward path.
		forwardStack = []; 

		var hash = null;
		var url = null;
		if(!historyIframe){
			if(dojo.config["useXDomain"] && !dojo.config["dojoIframeHistoryUrl"]){
				console.warn("dojo.back: When using cross-domain Dojo builds,"
					+ " please save iframe_history.html to your domain and set djConfig.dojoIframeHistoryUrl"
					+ " to the path on your domain to iframe_history.html");
			}
			historyIframe = window.frames["dj_history"];
		}
		if(!bookmarkAnchor){
			bookmarkAnchor = dojo.create("a", {style: {display: "none"}}, dojo.body());
		}
		if(args["changeUrl"]){
			hash = ""+ ((args["changeUrl"]!==true) ? args["changeUrl"] : (new Date()).getTime());
			
			//If the current hash matches the new one, just replace the history object with
			//this new one. It doesn't make sense to track different state objects for the same
			//logical URL. This matches the browser behavior of only putting in one history
			//item no matter how many times you click on the same #hash link, at least in Firefox
			//and Safari, and there is no reliable way in those browsers to know if a #hash link
			//has been clicked on multiple times. So making this the standard behavior in all browsers
			//so that dojo.back's behavior is the same in all browsers.
			if(historyStack.length == 0 && initialState.urlHash == hash){
				initialState = createState(url, args, hash);
				return;
			}else if(historyStack.length > 0 && historyStack[historyStack.length - 1].urlHash == hash){
				historyStack[historyStack.length - 1] = createState(url, args, hash);
				return;
			}

			changingUrl = true;
			setTimeout(function() { 
					setHash(hash); 
					changingUrl = false; 					
				}, 1);
			bookmarkAnchor.href = hash;
			
			if(dojo.isIE){
				url = loadIframeHistory();

				var oldCB = args["back"]||args["backButton"]||args["handle"];

				//The function takes handleName as a parameter, in case the
				//callback we are overriding was "handle". In that case,
				//we will need to pass the handle name to handle.
				var tcb = function(handleName){
					if(getHash() != ""){
						setTimeout(function() { setHash(hash); }, 1);
					}
					//Use apply to set "this" to args, and to try to avoid memory leaks.
					oldCB.apply(this, [handleName]);
				};
		
				//Set interceptor function in the right place.
				if(args["back"]){
					args.back = tcb;
				}else if(args["backButton"]){
					args.backButton = tcb;
				}else if(args["handle"]){
					args.handle = tcb;
				}
		
				var oldFW = args["forward"]||args["forwardButton"]||args["handle"];
		
				//The function takes handleName as a parameter, in case the
				//callback we are overriding was "handle". In that case,
				//we will need to pass the handle name to handle.
				var tfw = function(handleName){
					if(getHash() != ""){
						setHash(hash);
					}
					if(oldFW){ // we might not actually have one
						//Use apply to set "this" to args, and to try to avoid memory leaks.
						oldFW.apply(this, [handleName]);
					}
				};

				//Set interceptor function in the right place.
				if(args["forward"]){
					args.forward = tfw;
				}else if(args["forwardButton"]){
					args.forwardButton = tfw;
				}else if(args["handle"]){
					args.handle = tfw;
				}

			}else if(!dojo.isIE){
				// start the timer
				if(!locationTimer){
					locationTimer = setInterval(checkLocation, 200);
				}
				
			}
		}else{
			url = loadIframeHistory();
		}

		historyStack.push(createState(url, args, hash));
	};

	back._iframeLoaded = function(evt, ifrLoc){
		//summary: 
		//		private method. Do not call this directly.
		var query = getUrlQuery(ifrLoc.href);
		if(query == null){ 
			// alert("iframeLoaded");
			// we hit the end of the history, so we should go back
			if(historyStack.length == 1){
				handleBackButton();
			}
			return;
		}
		if(moveForward){
			// we were expecting it, so it's not either a forward or backward movement
			moveForward = false;
			return;
		}
	
		//Check the back stack first, since it is more likely.
		//Note that only one step back or forward is supported.
		if(historyStack.length >= 2 && query == getUrlQuery(historyStack[historyStack.length-2].url)){
			handleBackButton();
		}else if(forwardStack.length > 0 && query == getUrlQuery(forwardStack[forwardStack.length-1].url)){
			handleForwardButton();
		}
	};
 })();

}

if(!dojo._hasResource["dojo.dnd.common"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.common"] = true;
dojo.provide("dojo.dnd.common");

dojo.dnd._isMac = navigator.appVersion.indexOf("Macintosh") >= 0;
dojo.dnd._copyKey = dojo.dnd._isMac ? "metaKey" : "ctrlKey";

dojo.dnd.getCopyKeyState = function(e) {
	// summary: abstracts away the difference between selection on Mac and PC,
	//	and returns the state of the "copy" key to be pressed.
	// e: Event: mouse event
	return e[dojo.dnd._copyKey];	// Boolean
};

dojo.dnd._uniqueId = 0;
dojo.dnd.getUniqueId = function(){
	// summary: returns a unique string for use with any DOM element
	var id;
	do{
		id = dojo._scopeName + "Unique" + (++dojo.dnd._uniqueId);
	}while(dojo.byId(id));
	return id;
};

dojo.dnd._empty = {};

dojo.dnd.isFormElement = function(/*Event*/ e){
	// summary: returns true, if user clicked on a form element
	var t = e.target;
	if(t.nodeType == 3 /*TEXT_NODE*/){
		t = t.parentNode;
	}
	return " button textarea input select option ".indexOf(" " + t.tagName.toLowerCase() + " ") >= 0;	// Boolean
};

// doesn't take into account when multiple buttons are pressed
dojo.dnd._lmb = dojo.isIE ? 1 : 0;	// left mouse button

dojo.dnd._isLmbPressed = dojo.isIE ?
	function(e){ return e.button & 1; } : // intentional bit-and
	function(e){ return e.button === 0; };

}

if(!dojo._hasResource["com.ibm.ajax.auth"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.ajax.auth"] = true;
//dojo.require( "ibm.portal.debug" );
dojo.provide("com.ibm.ajax.auth");
com.ibm.ajax.auth = {
   /**
    * This method will wrap ioArgs normally sent to the dojo xhr methods with a special handler function that will check for form 
    * authentication redirection on a request.  This method supports dojo.xhrGet, dojo.xhrPost, dojo.xhrDelete, dojo.xhrPut,
    * dojo.io.iframe.send, and dojo.io.script.send.
    *
    * The testForValidOutput function will be invoked (if specified) with the response and the ioArgs arguments that are normally 
    * passed to the dojo.xhr methods.
    *
    * The onauthenticated method is optional, and will be invoked when the provider 
    * determines the user is successfully authenticated.  
    *
    * The handler method added to the ioArgs would check that the response status code is >= 200 && < 300, and that the mime-type is
    * text/html.
    *
    * @return A version of args suitable to be passed to dojo.xhrGet/xhrPost/xhrDelete/xhrPut/xhrRawPost/io.iframe.send()
    */
   prepareSecure: function( /* object */ args, /* function(response,ioArgs) optional */ isExpectedResponse, /* function() optional */ onauthenticated ) {
      args._handle = args.handle;
      args.handle = dojo.partial(this.testAuthenticationHandler, this, isExpectedResponse, onauthenticated);

      return args;
   },
   
   /**
    * When a request is determined to require authentication, the authenticationRequired function passed to this function will 
    * be invoked with the following arguments:
    *   - the response object from dojo
    *   - the ioArgs object that was passed to dojo
    *   - the onauthenticated function provided to the checkRequest/checkHtmlRequest methods (may be null)
    */
   setAuthenticationHandler: function( /* function(response,ioArgs,onauthenticated) */ authenticationRequired) {
      this.authenticationHandler = authenticationRequired;
   },
   
   /**
    * Callers of the API may choose to override the authentication test mechanism for this class wholesale by calling this
    * this method with an override.  The function will be invoked with the following arguments whenever an xhrGet that was
    * invoked with prepareSecure() returns.  Generally this is provided only as a last resort if a specific product's 
    * authentication test needs are not satisfied, or if the default behavior is not sufficient.
    */
   setTestAuthenticationHandler: function( /* function(auth, isExpected, onauthenticated, response, ioArgs) */handler) {
      this.testAuthenticationHandler = handler;
   },
   
   /**
    * By default there are three authentication tests performed when a request is returned from dojo.xhrGet/Post/Delete/Put.
    *    1) If the caller provides a test themselves (they know that their particular response may be confused for a
    *       login redirect)
    *    2) If the handleAs type is 'xml', 'json', or 'json-comment-optional' and the server returned 'text/html' or
    *       one of the other common HTML mime types, we assume that a redirect was sent from the server and the browser
    *       silently redirected the user (if the status code was >= 200 and < 300).
    *    3) If the status code is 302 in the response, which typically only occurs when multiple redirects are sent by the 
    *       server and the browser decides not to handle all of them, we assume a login redirect was the cause.
    *
    * The three booleans passed to this function will set whether the specific check is enabled.  By default all three
    * are true.
    */
   setDefaultAuthenticationTests: function(checkFromCaller,checkByContentType,checkByStatusCode) {
      this.checkFromCaller = checkFromCaller;
      this.checkByContentType = checkByContentType;
      this.checkByStatusCode = checkByStatusCode;
   },
   
   /**
    * This method allows users to add their own,custom login checks via a handler function.  The function takes three
    * arguments:
    *   - a reference to the auth object
    *   - the response object from dojo
    *   - the ioArgs object that was passed to dojo
    *
    * and returns true if the request requires authentication.
    *
    * See setDefaultAuthenticationTests(3) for more information about the default checks that are performed.
    */
   addAuthenticationCheck: function( /* function(auth, response, ioArgs) */handler ) {
      if (handler)
         this.authenticationChecks.push(handler);
   },
   
   /** 
    * Use this method to check whether a specific request requires authentication after a request has already
    * been received.  Should only be used when the default authentication test and handling behavior is not desired (for 
    * instance, when an application is initializing itself and needs to check whether it is authenticated).
    */
   isAuthenticationRequired: function(response, ioArgs) {
      var handleAs = ioArgs.args.handleAs;
      var unauthenticated = false;
      
      if (!response || dojo.indexOf(["cancel","timeout"],response.dojoType) == -1) {
         if (this.checkByContentType && dojo.indexOf(["xml","json","json-comment-optional","text"],handleAs) != -1 && ioArgs.xhr && /^text\/html/.exec(ioArgs.xhr.getResponseHeader("Content-Type")) && ioArgs.xhr.status >= 200 && ioArgs.xhr.status < 300) {
            ibm.portal.debug.text("auth::isAuthenticationRequired DEBUG content type does not match request, assume logged out");
            return true;
         }
         else if (this.checkByStatusCode && dojo.indexOf(["xml","json","json-comment-optional","text"],handleAs) != -1 && ioArgs.xhr && ioArgs.xhr.status == 302) {
            ibm.portal.debug.text("auth::isAuthenticationRequired DEBUG redirect received, assume login request");
            return true;
         }
         else if (this.checkByStatusCode && ioArgs.xhr && (ioArgs.xhr.status == 401|| ioArgs.xhr.status==0) && ioArgs.xhr.getResponseHeader("WWW-Authenticate") && ioArgs.xhr.getResponseHeader("WWW-Authenticate").indexOf("IBMXHR")!=-1) {
            ibm.portal.debug.text("auth::isAuthenticationRequired DEBUG Portal 401 received, assume login required");
            return true;
         }

      }
      if (!unauthenticated) {
         for (var i=0; i<this.authenticationChecks.length; i++)
            if (this.authenticationChecks[i](this, response, ioArgs))
               return true;
      }
      return false;   
   },
   
   testAuthenticationHandler: function(auth, isExpectedResponse, onauthenticated, response, ioArgs) {
      var args = dojo._toArray(arguments).slice(3);

      var unauthenticated = false;
      if (!response || dojo.indexOf(["cancel","timeout"],response.dojoType) == -1) {
         if (auth.checkFromCaller && typeof isExpectedResponse == "function" && isExpectedResponse(response, ioArgs)) {
            //console.log("auth::_testAuthentication DEBUG caller provided detection logic that indicated form login, assume logged out");
            unauthenticated = true;
         }
         else
            unauthenticated = auth.isAuthenticationRequired(response,ioArgs,isExpectedResponse)
      }
      
      if (unauthenticated) {
         //assume that WAS has set its redirect cookies and clear them
         var path = auth._parseUri(ioArgs.args.url).path;
         dojo.cookie("WASPostParam",null,{expires:-1,path:path});
         dojo.cookie("WASReqURL",null,{expires:-1,path:"/"});

         auth.authenticationHandler(response, ioArgs, onauthenticated);

         args[0] = new Error("xhr unauthenticated");
         args[0].dojoType = "unauthenticated";
      }
      if (ioArgs.args._handle)
         return ioArgs.args._handle.apply(this, args);
      else
      	 return(response);
   },
   
   /** Private methods */

   _parseUri: function(uri) {
      if (!uri)
         return null;
      uri = new dojo._Url(uri);
      var params = this._splitQuery(uri.query);
      uri.queryParameters = params;
      return uri;
   },
   
   _splitQuery: function(query) {
      var params = {};
      if (!query)
         return params;
      if (query.charAt(0) == "?")
         query = query.substring(1);
   
      var args = query.split("&");
      for (var i=0; i<args.length; i++)
         if (args[i].length > 0) {
            var separator = args[i].indexOf("=");
            if (separator == -1) {
               var key = decodeURIComponent(args[i]);
               var existing = params[key];
               if (dojo.isArray(existing))
                  existing.push("");
               else if (existing)
                  params[key] = [existing,""];
               else
                  params[key] = "";
            } else if (separator > 0) {
               var key = decodeURIComponent(args[i].substring(0, separator));
               var value = decodeURIComponent(args[i].substring(separator+1));
               var existing = params[key];
               if (dojo.isArray(existing))
                  existing.push(value);
               else if (existing)
                  params[key] = [existing,value];
               else
                  params[key] = value;
            }
         }
      return params;
   },
   
   checkFromCaller: true,
   checkByContentType: true,
   checkByStatusCode: true,
   authenticationChecks: [],
   authenticationHandler: function() {
      ibm.portal.debug.text("auth::authenticationHandler DEBUG authentication was required");
   }
};

}

if(!dojo._hasResource["com.ibm.portal.debug"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.debug"] = true;
dojo.provide( "com.ibm.portal.debug" );

//To satisfy dojo.require
//TODO: convert all ibm.portal.debug statements to com.ibm.portal.debug stmts
dojo.provide( "ibm.portal.debug" );

ibm.portal.debug.setTrace = function ( /*String*/functionName ) {
	ibm.portal.debug._traceString = functionName;
}

ibm.portal.debug._isDebugEnabled = function () {
	var retVal = false;
	
	if ( typeof ( ibmPortalConfig ) != "undefined" ) {
		if ( ibmPortalConfig && ibmPortalConfig.isDebug ) {
			retVal = true;
		}
	}
	
	return retVal;	
}

ibm.portal.debug.text = function( str, functionName ) {
	//First check that ibmPortalConfig is defined. Next check to see if the
	//isDebug flag is enabled. We don't want to actually print the statement
	//if we can avoid it since it can be expensive performance-wise. If a trace
	//string is defined, we only want to print debug statements that match that trace string.
	//otherwise, we print everything. This way the debug output can be filtered down. The 
	//one caveat is that not all functions specify a function name.
	if ( typeof ( ibmPortalConfig ) != "undefined" ) {
		if ( ibmPortalConfig && ibmPortalConfig.isDebug ) {
			var traceStr = ibm.portal.debug._traceString;
			if ( traceStr ) {
				if ( functionName ) {
					if ( functionName.indexOf( traceStr ) >= 0 ) {
						window.console.log(str);
					}
				}
			}
			else {
				window.console.log(str);
			}		
		}
	}	
} 

ibm.portal.debug.entry = function ( functionName, args ) {
	if ( ibm.portal.debug._isDebugEnabled() ) {
	
		var debugStr = functionName + " --> entry; { ";
		
		if ( args && args.length > 0 )
		{
			for ( arg in args ) {
				debugStr = debugStr + args[arg] + " ";
			}
		} 
		
		debugStr = debugStr + " } ";
		
		ibm.portal.debug.text( debugStr, functionName );
	}	
}

ibm.portal.debug.exit = function( functionName, retValue ) {
	if ( ibm.portal.debug._isDebugEnabled() ) {
		var debugStr = functionName + " --> exit;"
		if ( typeof( retValue ) != "undefined" )
		{
			debugStr = debugStr +" { " + retValue + " } ";
		}		
		ibm.portal.debug.text( debugStr, functionName );
	}			
}

ibm.portal.debug.escapeXmlForHTMLDisplay = function( string ) {
	string = string.replace( /</g, "&lt;" );
	string = string.replace( />/g, "&gt;" );
	
	return string;
}

}

if(!dojo._hasResource["com.ibm.portal.EventBroker"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.EventBroker"] = true;
dojo.provide( "com.ibm.portal.EventBroker" );

dojo.require( "com.ibm.portal.debug" );

dojo.declare( "com.ibm.portal.Event",
			  null,
			  {
			  	constructor: function ( /*String*/uniqueEventId ) {
				  	this.eventName = uniqueEventId;
				  	this._listeners = new Array();
				},
			  	fire: function ( /*Object?*/ parameters ) {
					// summary: Fires the event to the listeners.
					// parameters: any parameters that are to be passed to the listeners
					ibm.portal.debug.text( "Firing event: " + this.eventName + " with parameters: " );
					////dojo.debugShallow( parameters );
					//dojo.event.topic.publish( this.eventName, parameters );
					dojo.publish( this.eventName, [parameters] );
				},
				register: function ( /*Function or Object*/listener, /*Function or String*/listenerFunc ) {
					// summary: Registers a listener with an event.
					// listener: the listener function to be called when an event is fired.
					
					if ( !listenerFunc ) {
						//dojo.event.topic.subscribe( this.eventName, listener );
						return dojo.subscribe( this.eventName, null, listener ); // Handle
					}
					else {
						//dojo.event.topic.subscribe( this.eventName, listener, listenerFunc );
						return dojo.subscribe( this.eventName, listener, listenerFunc ); // Handle
					}	
				},
				unregister: function( /*Handle*/handle ) { 
					// summary: Unregisters a listener from an event. The parameters must match
					//		exactly.
					//dojo.event.topic.unsubscribe( this.eventName, listener, listenerFunc );
					dojo.unsubscribe( handle );
				},
				cancel: function ( /*Object?*/ parameters ) { 
					// summary: Cancels an event.
					// parameters: any parameters to be passed to the listeners
					dojo.publish( this.id + "/cancel" );	
				} 
			  }
);

dojo.declare( "com.ibm.portal.EventBroker",
			  null,
			  {
			  	// summary: All events should be instance variables of an Event.	
				
				//required events
				startPage: new com.ibm.portal.Event( "portal/StartPage" ),
				endPage: new com.ibm.portal.Event( "portal/EndPage" ),
				startFragment: new com.ibm.portal.Event( "portal/StartFragment" ),
				endFragment: new com.ibm.portal.Event( "portal/EndFragment" ),
				fragmentUpdated: new com.ibm.portal.Event( "portal/FragmentUpdated" ),
				startRequest: new com.ibm.portal.Event( "portal/StartRequest" ),
				endRequest: new com.ibm.portal.Event( "portal/EndRequest" ),
				cancelAll: new com.ibm.portal.Event( "portal/CancelAll" ),
				stateChanged: new com.ibm.portal.Event( "portal/StateChanged" ),
				startScriptHandling: new com.ibm.portal.Event( "portal/StartScriptHandling" ),
				endScriptHandling: new com.ibm.portal.Event( "portal/EndScriptHandling" ),
				startScriptExecution: new com.ibm.portal.Event( "portal/StartScriptExecution" ),
				endScriptExecution: new com.ibm.portal.Event( "portal/EndScriptExecution" ),
                javascriptCleanup: new com.ibm.portal.Event( "portal/JavascriptCleanup" ),
                //The events below this comment need to be removed. They are deprecated. We may have to leave them in for compatibility.
                //startScriptExecution and endScriptExecution should be used instead. There will be no replacement for restorePointUpdated
                //and clearRestorePoint as these should never have been introduced.
                beforeSnapShot: new com.ibm.portal.Event( "portal/BeforeSnapShot" ),
                afterSnapShot: new com.ibm.portal.Event( "portal/AfterSnapShot" ),
                restorePointUpdated: new com.ibm.portal.Event( "portal/RestorePointUpdated" ),
                clearRestorePoint: new com.ibm.portal.Event( "portal/ClearRestorePoint" ),
				//The events above this comment need to be removed. They are deprecated.
                stopEvent: new com.ibm.portal.Event( "portal/StopEvent" ),
				redirect: new com.ibm.portal.Event( "portal/Redirect" )
			  }
);

com.ibm.portal.EVENT_BROKER = new com.ibm.portal.EventBroker();

}

if(!dojo._hasResource["com.ibm.portal.services.PortalRestServiceRequestQueue"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.services.PortalRestServiceRequestQueue"] = true;
dojo.provide( "com.ibm.portal.services.PortalRestServiceRequestQueue" );

dojo.declare( "com.ibm.portal.services.PortalRestServiceRequestQueue",
			  null,
			  {
			  	maxNumberOfActiveRequests: 4,
			  	constructor: function () {
			  		// summary: Constructs a new queue to limit the number of active requests at any given time.
			  		var _method = "PortalRestServiceRequestQueue.constructor"
			  		ibm.portal.debug.entry( _method );
			  		
			  		this._activeRequests = 0;
			  		this._requestQueue = [];
			  		
			  		ibm.portal.debug.exit( _method );
			  	},
			  	add: function ( /*Request*/req ) {
			  		// summary: Adds a new request to the queue. 
			  		// description: The request object should have a single method called "execute" that takes a function object
			  		//		as a parameter. The function should be called when the request is complete to let the queue know it can
			  		//		move on to the next request. 
			  		var _method = "PortalRestServiceRequestQueue.add";
			  		ibm.portal.debug.entry( _method, [ req ] );
			  		
			  		this._requestQueue.push( req );
			  		var me = this;
			  		setTimeout( function () { me._executeNextRequest(); }, 5 );
			  		
			  		ibm.portal.debug.exit( _method );
			  	},
			  	_executeNextRequest: function () {
			  		// summary: Execute the next request in the queue, if necessary.
			  		var _method = "PortalRestServiceRequestQueue._executeNextRequest";
			  		ibm.portal.debug.entry( _method );
			  		
			  		ibm.portal.debug.text( this._requestQueue.length + " request(s) in the queue. " + this._activeRequests + " active request(s) currently.", _method );
			  		if ( this._requestQueue.length > 0 && this._activeRequests < this.maxNumberOfActiveRequests ) {
				  		var reqToExec = this._requestQueue.shift();
			  			
			  			ibm.portal.debug.text( "Executing request: " + reqToExec, _method );
			  			
			  			var me = this;
			  			setTimeout( function () { reqToExec.execute( function () { me._notifyComplete(); } ); me.activeRequests = me._activeRequests+1; }, 1 );
			  		}
			  		else {
			  			ibm.portal.debug.text( "No request(s) pending or maximum number of requests already currently active.", _method );
			  		}
			  		
			  		ibm.portal.debug.exit( _method );	
			  	},
			  	_notifyComplete: function () {
			  		// summary: Called to notify the queue that an active request has completed and the next waiting request
			  		//		can now be executed.
			  		var _method = "PortalRestServiceRequestQueue._notifyComplete";
			  		ibm.portal.debug.entry( _method );
			  		
			  		this._activeRequests = this._activeRequests-1;
			  		if ( this._activeRequests < 0 ) {
			  			this._activeRequests = 0;
			  		}
			  		var me = this;
			  		setTimeout( function () { me._executeNextRequest(); }, 5 );
			  		
			  		ibm.portal.debug.exit( _method );
			  	}
			  }
);

}

if(!dojo._hasResource["com.ibm.portal.utilities"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.utilities"] = true;
dojo.provide( "com.ibm.portal.utilities" );

com.ibm.portal.utilities = {
	findPortletIdByElement: function ( /*DOMElement*/ element ) {
		// summary: Finds the portlet ID which the given element belongs to
		// description: Finds the portlet ID to which the give DOM element belongs
		//		by walking up the parents of the given element until the ancestor is 
		//		found with one of the defined class names.
		// element: the DOM element to find the portlet ID of
		
		ibm.portal.debug.entry( "findPortletID", [ element ] );
		var id = "";
		var current = element.parentNode;
		while ( current && id.length == 0 ) {
			ibm.portal.debug.text( "examining element " + current.tagName + "; class=" + current.className,"findPortletID" );
		
			if ( current.className && (current.className.match(/\bwpsPortletBody\b/) || current.className.match(/\bwpsPortletBodyInlineMode\b/)) ) {
				id = current.id;
				var indexOf = id.indexOf( "_mode" );
				if ( indexOf >= 0 )  {
					id = id.substring( 0, indexOf );
				}
			}
			current = current.parentNode;	
		}
		
		if ( id.indexOf( "portletActions_" ) >= 0 ) {
			id = id.substring( "portletActions_".length );
		}
		
		ibm.portal.debug.exit( "findPortletID", [ id ] );
		
		return id;
	},
	findFormByElement: function ( /*DOMElement*/ element) {
		var current = element;
		while(current) {
			if(current.tagName && current.tagName.toLowerCase() == "form") break;
			current = current.parentNode;
		}
		return current;
	},
	encodeURI: function ( /*String*/uri ) {
		// summary: Encodes a model uri for use in an URL.
		// uri: the unencoded model uri to encode
		// returns: the encoded uri ready for use in an URL
		ibm.portal.debug.entry( "encodeURI", [ uri ] );
		
		var encodedURI = uri;
		var index = uri.lastIndexOf( ":" );
		
		while ( index >= 0 ) {
			var scheme = encodedURI.substring(0, index );
			var part = encodedURI.substring( index + 1  );
			
			encodedURI = scheme + ":" + encodeURIComponent( part );
			index = scheme.lastIndexOf( ":" );
		}
		
		encodedURI = encodeURIComponent( encodedURI );
		
		ibm.portal.debug.exit( "encodeURI", [ encodedURI ] );
		return encodedURI; // String
	},
	decodeURI: function ( /*String*/uri ) { 
		// summary: Decodes a model uri.
		// uri: the encode model uri
		// returns: the decoded model uri
		ibm.portal.debug.entry( "decodeURI", [ uri ] );
		
		var decodedURI = decodeURIComponent( uri );
		var index = decodedURI.indexOf( ":" );
		
		while ( index >= 0 ) {
			var scheme = decodedURI.substring(0, index );
			var part = decodedURI.substring( index + 1 );
			
			decodedURI = scheme + ":" + decodeURIComponent( part );
			index = decodedURI.indexOf( ":", index+1 );
		}
		
		ibm.portal.debug.exit( "decodeURI", [ decodedURI ] );
		return decodedURI;	//String
	},
	getSelectionNodeId: function ( /*String*/fullPortletId ) {
		// summary: Extracts the selection node object id from the full portlet id returned
		//		by the layout model feed.
		ibm.portal.debug.entry( "getSelectionNodeId", [ fullPortletId ] );
		
		// this split will break the full portlet id into the control with prefixes at index [0]
		// and the selected node will be at index [1]
		var selNodeId = fullPortletId.split('@oid:');
				
		ibm.portal.debug.exit( "getSelectionNodeId", [ selNodeId[1] ] );
		return selNodeId[1];
	},
	getControlId: function ( /*String*/fullPortletId ) {
		// summary: Extracts the control object id from the full portlet id returned
		//		by the feed.
		ibm.portal.debug.entry( "_getControlId", [ fullPortletId ] );
		
		// this first split will get rid of the page oid
		var controlParts = fullPortletId.split('@oid:');
		
		// this second split will leave the feed prefix (i.e. lm) at index [0]
		// and index [1] will have the control oid
		var controlId = controlParts[0].split("oid:");
		
		ibm.portal.debug.exit( "getControlId", [ controlId[1] ] );
		return controlId[1];
	},
	overwriteProperty: function ( /*Object*/obj, /*String*/propertyName, /*Object*/newValue, /*Boolean?*/onlyOnce ) {
		// summary: Overwrites the given property.
		ibm.portal.debug.entry( "overwriteProperty", [ obj, propertyName, newValue, onlyOnce ] );
		
		if ( !obj["_overwritten_"] ) {
			obj["_overwritten_"] = new Object();
		}
		
		//In case onlyOnce is undefined.
		if ( !onlyOnce ) { onlyOnce = false; }
		
		var skipOverwrite = (onlyOnce &&  (obj["_overwritten_"][propertyName] != null) );
		if ( !skipOverwrite ) {
			if ( obj["_overwritten_"][propertyName] == null ) {
                obj["_overwritten_"][propertyName] = obj[propertyName];
            }
            else {
                obj["_overwritten_"][propertyName] = null;
            }    
			obj[propertyName] = newValue;
			ibm.portal.debug.text( "Property overwrite successful!" );
		}
		
		ibm.portal.debug.exit( "overwriteProperty");	
	},
	restoreProperty: function ( /*Object*/obj, /*String*/propertyName ) {
		// summary: Restores an overwritten property.
        // returns: the overwritten value for the property
		ibm.portal.debug.entry( "utilities.restoreProperty", [ obj, propertyName ] );
		var retVal = obj[propertyName];

        if ( obj["_overwritten_"] != null ) {
            ibm.portal.debug.text( "overwritten property value: " + obj["_overwritten_"] ); 
            obj[propertyName] = obj["_overwritten_"][propertyName];
            obj["_overwritten_"][propertyName] = null;
        }
        else {
            obj[propertyName] = null;
        }
        
        ibm.portal.debug.exit( "utilities.restoreProperty", retVal );    
        return retVal;
	},
	getOverwrittenProperty: function ( /*Object*/obj, /*String*/propertyName ) {
		// summary: Access an overwritten property.
		if ( obj["_overwritten_"] ) {
			return obj["_overwritten_"][propertyName];
		}
		else {
			return null;
		}	
	},
	setOverwrittenProperty: function ( /*Object*/obj, /*String*/propertyName, /*Object*/value ) {
		// summary: Sets the overwritten property directly; bypassing the overwriting process.
		ibm.portal.debug.entry( "utilities.setOverwrittenProperty", [ obj, propertyName, value ] ); 
		
		if ( !obj["_overwritten_"] ) {
			obj["_overwritten_"] = new Object();
		}
		
		obj["_overwritten_"][propertyName] = value;
		
		ibm.portal.debug.exit( "utilities.setOverwrittenProperty" );
	},
	callOverwrittenFunction: function ( /*Object*/thisObj, /*String*/propertyName, /*Object[]?*/args ) {
		ibm.portal.debug.entry( "utilities.callOverwrittenFunction", [ thisObj, propertyName, args ]);
		//IE doesn't like it if we pass in an undefined args object to apply.
		var retVal = null;
		var oldProp = this.getOverwrittenProperty( thisObj, propertyName );
		ibm.portal.debug.text( "Overwritten property: " + oldProp );
		ibm.portal.debug.text( "old property's apply function: " + oldProp.apply );
		
		if ( args ) {
			retVal = oldProp.apply( thisObj, args );
		}
		else {
			retVal = oldProp.apply( thisObj );
		}
		
		ibm.portal.debug.exit( "utilities.callOverwrittenFunction", retVal );
		return retVal;
	},
	isExternalUrl: function ( /*String*/urlString ) {
		// summary: Determines if the given url points to an external server.
		// urlString: the url in string form
		// returns: true if the url points to an external server
		ibm.portal.debug.entry( "isExternalUrl", [ urlString ] );
		
		var host = window.location.host;
		var protocol = window.location.protocol;
		var urlStringNoQuery = urlString.split( "?" )[0];
		var retVal = !(urlStringNoQuery.indexOf("://") < 0 || (urlStringNoQuery.indexOf(protocol) == 0 && urlStringNoQuery.indexOf(host) == protocol.length+2) );
		
		ibm.portal.debug.text( "urlStringNoQuery.indexOf(\"://\") = " + urlStringNoQuery.indexOf("://") );
		ibm.portal.debug.text( "urlStringNoQuery.indexOf(protocol) = " + urlStringNoQuery.indexOf(protocol) );
		
		ibm.portal.debug.exit( "isExternalUrl", retVal );
		return retVal;
	},
	isJavascriptUrl: function ( /*String*/urlString ) {
		// summary: Determines if the given url is a javascript url.
		// description: Checks to see if the url begins with the javascript: protocol
		// urlString: the url in string form
		// returns: true if the url is actually a javascript call.
		ibm.portal.debug.entry( "isJavascriptUrl", [ urlString ] );
		var url = com.ibm.portal.utilities.string.trim( urlString.toLowerCase() );
		var retVal = ( url.indexOf( "javascript:" ) == 0 );
		ibm.portal.debug.exit( "isJavascriptUrl", retVal );
		return retVal;
	},
	isPortalUrl: function ( /*String*/urlString ) {
		// summary: Determines if a given url is a url to the portal servlet.
		// description: Checks to see if the url begins with the path to the portal servlet.
		// urlString: the url to check in string form
		// returns: true if the url is a portal url
		ibm.portal.debug.entry( "utilities.isPortalUrl", [ urlString ] );
		
		var retVal = ( urlString.indexOf( ibmPortalConfig["portalURI"] ) >= 0 );
		
		ibm.portal.debug.exit( "utilities.isPortalUrl", retVal );
		return retVal;
	},	

	addExternalNode: function ( /*DOMDocument*/ doc, /*Element*/ node ) {
		var newNode = null;
		if(doc.importNode) newNode = doc.importNode(node, true);
		else newNode = node;
		doc.appendChild( newNode );
	},
    decodeXML: function ( /*String*/xmlText ) {
		// summary: decodes xml sent by multi-part form 
		ibm.portal.debug.entry( "decodeXML", [ xmlText ] );

		var newXmlText1 = xmlText.replace(/&amp;/g, "&");
		var newXmlText2 = newXmlText1.replace(/&amp;/g, "&");
		newXmlText1 = newXmlText2.replace(/&#039;/g, "\'");
		newXmlText2 = newXmlText1.replace(/&#034;/g, '\"');
		
		newXmlText2 = newXmlText2.replace( /&lt;/g, "<" );
		newXmlText2 = newXmlText2.replace( /&gt;/g, ">" );

		ibm.portal.debug.exit( "decodeXML", [ newXmlText2 ] );
		return newXmlText2;
	},
	eventHandlerToString: function ( eventHandler ) {
		var eventHandlerStr = eventHandler.toString();
		var startIndex = eventHandlerStr.indexOf("{");
		var endIndex = eventHandlerStr.lastIndexOf("}");
		onclickStr = eventHandlerStr.substring(startIndex+1, endIndex);
		
		return onclickStr;
	},
	_waitingForScript: false,
	_isWaitingForScript: function() { return com.ibm.portal.utilities._waitingForScript; },
	stopWaitingForScript: function() { com.ibm.portal.utilities._waitingForScript = false; },

	waitFor: function(testFunction, pollInterval, callback, args) {
		var interval = setInterval(function() {
			if (testFunction())
			{
				clearInterval(interval);
				
				if ( !args ) {
					callback();
				}
				else {
					callback(args);
				}
			}
		},
		pollInterval);
	},

	waitForScript: function( callback, args ) {
		com.ibm.portal.utilities._waitingForScript = true;
		com.ibm.portal.utilities.waitFor(function()
		{
			return (!com.ibm.portal.utilities._isWaitingForScript());
		},
		500,
		callback,
		args);
	}
}

com.ibm.portal.utilities.string = {
	findNext: function ( /*String*/ string, /*Array*/searchStrings, /*int?*/from ) {
		// summary: Finds the next index of any of the given search strings.
		// description: Looks for each search string in the given string and returns the index 
		//		of the first one encountered. If none of the search strings are discovered, 
		//		-1 is returned. 
		// string: the string to search
		// searchStrings: the array of strings to look for in the string
		// from: the index to start looking from - the length of the search string is always added 
		//		from index before searching for the next instance
		ibm.portal.debug.entry( "string.findNext", [ string, searchStrings ] );
		
		var lowValueNotZero = -1;
		
		for ( var i = 0; i < searchStrings.length; i++ ) {
			var fromIndex = null;
			if ( from ) {
				fromIndex = from + searchStrings[i].length;
			}
			var index = string.indexOf( searchStrings[i], fromIndex );
				
			if ( index > -1 && ( index < lowValueNotZero || lowValueNotZero == -1 ) ) {
				lowValueNotZero = index;
			}
		}
		
		ibm.portal.debug.exit( "string.findNext", [ lowValueNotZero ] );
		return lowValueNotZero;		
	},
	contains: function ( /*String*/ stringToSearch, /*String*/stringToLookFor ) {
		// summary: Determines if the stringToSearch contains the stringToLookFor
		ibm.portal.debug.entry( "string.contains", [ stringToSearch, stringToLookFor ] );
		
		var retVal = false;
		
		if ( stringToSearch != null && stringToLookFor != null ) {
			retVal = ( stringToSearch.indexOf( stringToLookFor ) != -1 );		
		}
		
		ibm.portal.debug.exit( "string.contains", [ retVal ] );
		return retVal;
	},
	strip: function ( /*String*/string, /*String*/stringToStrip ) {
		// summary: Strips out the given string from the original string if it contains
		//		the stringToStrip.
		// string: the original string
		// stringToStrip: the string to remove from the original string
		// returns: the original string without any instances of the stringToStrip
		ibm.portal.debug.entry( "string.strip", [ string, stringToStrip ] );
		
		var retVal = string.replace( new RegExp( stringToStrip, "g" ), "" );
				
		ibm.portal.debug.exit ( "string.strip", [ retVal ]);
		return retVal;
	},
	properCase: function ( /*String*/string ) {
		// summary: Changes the first character of the string to Upper Case and all remaining characters
		//		to Lower Case.  Note that this does not split the string into words and capitalize
		//		each word, but only acts on the entire string as a whole.
		// string: The original string to change
		// returns: The original string with the first character in Upper Case and all other characters
		//		in Lower Case.
		if(string == null || string.length < 1) return "";
		ibm.portal.debug.entry("string.properCase", [string] );
		var retVal = string.charAt(0).toUpperCase();
		if(string.length > 1) {
			retVal += string.substring(1).toLowerCase();
		}
		ibm.portal.debug.exit("string.properCase", [retVal] );
		return retVal;
	},
	trim: function ( /*String*/ string ) {
		// summary: Removes leading and trailing spaces from a string.
		// string: the string to trim
		// returns: the original string minus any leading or trailing spaces
		ibm.portal.debug.entry( "string.trim", [ string ] );
		var retVal = string;
		
		retVal = retVal.replace( /^\s+/, '' );
		retVal = retVal.replace( /\s+$/, '' );
		
		ibm.portal.debug.exit( "string.trim", retVal );
		return retVal;
	}
}

dojo.declare( "com.ibm.portal.utilities.HttpUrl", 
			  null,
			  {
			  	constructor: function ( /*String*/urlString ) {
					this.scheme = "http://";
				  	this.server = this._extractServer( urlString );
					this.port = this._extractPort( urlString );
					this.path = this._extractPath( urlString );
					this.query = this._extractQuery( urlString );
					this.anchor = "";
				},
			  	addParameter: function ( /*String*/name, /*String*/value ) {
					this.query += "&" + name + "=" + value;
				},
				toString: function () {
					var str = "";
					if ( this.server != "" ) {
						str += this.scheme + this.server;
					} 
					if ( this.port != "" ) {
						str += ":" + this.port;
					}
					
					str += "/" + this.path;
					
					if ( this.query != "" ) {
						str += "?" + this.query;
					}
					if ( this.anchor != "" ) {
						str += "#" + this.anchor;
					}
					
					return str;
				},
				_extractServer: function (/*String*/urlString ) {
					var indexOfScheme = urlString.indexOf( this.scheme );
					var retVal = "";
					if ( indexOfScheme == 0 ) {
						var indexOfSlash = urlString.indexOf( "/", indexOfScheme + this.scheme.length );
						var serverAndPort = urlString.substring( indexOfScheme+ this.scheme.length, indexOfSlash );
						retVal = serverAndPort.split( ":" )[0];
					}	
					
					return retVal; 
				},
				_extractPort: function (/*String*/urlString ) {
					var indexOfServer = urlString.indexOf( this.server );
					var retVal = "";
					if ( indexOfServer >= 0 ) {
						var indexOfSlash = urlString.indexOf( "/", indexOfServer );
						var serverAndPort = urlString.substring( indexOfServer, indexOfSlash );
						var serverAndPortParts = serverAndPort.split( ":" );
						if ( serverAndPortParts.length > 1 ) {
							retVal = serverAndPortParts[1];
						}
					}	
					
					return retVal; 
				},
				_extractPath: function ( /*String*/urlString ) {
					var indexOfServer = urlString.indexOf( this.server );
					var retVal = "";
					if ( indexOfServer >= 0 ) {
						var indexOfSlash = urlString.indexOf( "/", indexOfServer );
						var indexOfQuery = urlString.indexOf( "?" );
						var indexOfHash = urlString.lastIndexOf( "#" );
						if ( indexOfQuery >= 0 ) {
							retVal = urlString.substring( indexOfSlash + 1, indexOfQuery );
						}
						else {
							if ( indexOfHash >= 0 ) {
								retVal = urlString.substring( indexOfSlash + 1, indexOfHash );
							}
							else {
								retVal = urlString.substring( indexOfSlash + 1 );
							}		
						}
					}	
					
					return retVal;
				},
				_extractQuery: function ( /*String*/urlString ) {
					var retVal = "";
					var urlParts = urlString.split( "?" );
					
					if ( urlParts.length > 1 ) {
						retVal = urlParts[1].split( "#" )[0];
					}	
					
					return retVal;
				},
				_extractAnchor: function ( /*String*/urlString ) {
					var retVal = "";
					var urlParts = urlString.split( "#" );
					
					if ( urlParts.length > 1 ) {
						retVal = urlParts[urlParts.length - 1];
					}	
					
					return retVal;
				}
					
			  }
);

}

if(!dojo._hasResource["dojo.fx.Toggler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.fx.Toggler"] = true;
dojo.provide("dojo.fx.Toggler");

dojo.declare("dojo.fx.Toggler", null, {
	// summary:
	//		class constructor for an animation toggler. It accepts a packed
	//		set of arguments about what type of animation to use in each
	//		direction, duration, etc.
	//
	// example:
	//	|	var t = new dojo.fx.Toggler({
	//	|		node: "nodeId",
	//	|		showDuration: 500,
	//	|		// hideDuration will default to "200"
	//	|		showFunc: dojo.wipeIn, 
	//	|		// hideFunc will default to "fadeOut"
	//	|	});
	//	|	t.show(100); // delay showing for 100ms
	//	|	// ...time passes...
	//	|	t.hide();

	// FIXME: need a policy for where the toggler should "be" the next
	// time show/hide are called if we're stopped somewhere in the
	// middle.

	constructor: function(args){
		var _t = this;

		dojo.mixin(_t, args);
		_t.node = args.node;
		_t._showArgs = dojo.mixin({}, args);
		_t._showArgs.node = _t.node;
		_t._showArgs.duration = _t.showDuration;
		_t.showAnim = _t.showFunc(_t._showArgs);

		_t._hideArgs = dojo.mixin({}, args);
		_t._hideArgs.node = _t.node;
		_t._hideArgs.duration = _t.hideDuration;
		_t.hideAnim = _t.hideFunc(_t._hideArgs);

		dojo.connect(_t.showAnim, "beforeBegin", dojo.hitch(_t.hideAnim, "stop", true));
		dojo.connect(_t.hideAnim, "beforeBegin", dojo.hitch(_t.showAnim, "stop", true));
	},

	// node: DomNode
	//	the node to toggle
	node: null,

	// showFunc: Function
	//	The function that returns the dojo._Animation to show the node
	showFunc: dojo.fadeIn,

	// hideFunc: Function	
	//	The function that returns the dojo._Animation to hide the node
	hideFunc: dojo.fadeOut,

	// showDuration:
	//	Time in milliseconds to run the show Animation
	showDuration: 200,

	// hideDuration:
	//	Time in milliseconds to run the hide Animation
	hideDuration: 200,

	/*=====
	_showArgs: null,
	_showAnim: null,

	_hideArgs: null,
	_hideAnim: null,

	_isShowing: false,
	_isHiding: false,
	=====*/

	show: function(delay){
		// summary: Toggle the node to showing
		return this.showAnim.play(delay || 0);
	},

	hide: function(delay){
		// summary: Toggle the node to hidden
		return this.hideAnim.play(delay || 0);
	}
});

}

if(!dojo._hasResource["dojo.fx"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.fx"] = true;
dojo.provide("dojo.fx");

/*=====
dojo.fx = {
	// summary: Effects library on top of Base animations
};
=====*/
(function(){
	
	var d = dojo, 
		_baseObj = {
			_fire: function(evt, args){
				if(this[evt]){
					this[evt].apply(this, args||[]);
				}
				return this;
			}
		};

	var _chain = function(animations){
		this._index = -1;
		this._animations = animations||[];
		this._current = this._onAnimateCtx = this._onEndCtx = null;

		this.duration = 0;
		d.forEach(this._animations, function(a){
			this.duration += a.duration;
			if(a.delay){ this.duration += a.delay; }
		}, this);
	};
	d.extend(_chain, {
		_onAnimate: function(){
			this._fire("onAnimate", arguments);
		},
		_onEnd: function(){
			d.disconnect(this._onAnimateCtx);
			d.disconnect(this._onEndCtx);
			this._onAnimateCtx = this._onEndCtx = null;
			if(this._index + 1 == this._animations.length){
				this._fire("onEnd");
			}else{
				// switch animations
				this._current = this._animations[++this._index];
				this._onAnimateCtx = d.connect(this._current, "onAnimate", this, "_onAnimate");
				this._onEndCtx = d.connect(this._current, "onEnd", this, "_onEnd");
				this._current.play(0, true);
			}
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			if(!this._current){ this._current = this._animations[this._index = 0]; }
			if(!gotoStart && this._current.status() == "playing"){ return this; }
			var beforeBegin = d.connect(this._current, "beforeBegin", this, function(){
					this._fire("beforeBegin");
				}),
				onBegin = d.connect(this._current, "onBegin", this, function(arg){
					this._fire("onBegin", arguments);
				}),
				onPlay = d.connect(this._current, "onPlay", this, function(arg){
					this._fire("onPlay", arguments);
					d.disconnect(beforeBegin);
					d.disconnect(onBegin);
					d.disconnect(onPlay);
				});
			if(this._onAnimateCtx){
				d.disconnect(this._onAnimateCtx);
			}
			this._onAnimateCtx = d.connect(this._current, "onAnimate", this, "_onAnimate");
			if(this._onEndCtx){
				d.disconnect(this._onEndCtx);
			}
			this._onEndCtx = d.connect(this._current, "onEnd", this, "_onEnd");
			this._current.play.apply(this._current, arguments);
			return this;
		},
		pause: function(){
			if(this._current){
				var e = d.connect(this._current, "onPause", this, function(arg){
						this._fire("onPause", arguments);
						d.disconnect(e);
					});
				this._current.pause();
			}
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			this.pause();
			var offset = this.duration * percent;
			this._current = null;
			d.some(this._animations, function(a){
				if(a.duration <= offset){
					this._current = a;
					return true;
				}
				offset -= a.duration;
				return false;
			});
			if(this._current){
				this._current.gotoPercent(offset / this._current.duration, andPlay);
			}
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			if(this._current){
				if(gotoEnd){
					for(; this._index + 1 < this._animations.length; ++this._index){
						this._animations[this._index].stop(true);
					}
					this._current = this._animations[this._index];
				}
				var e = d.connect(this._current, "onStop", this, function(arg){
						this._fire("onStop", arguments);
						d.disconnect(e);
					});
				this._current.stop();
			}
			return this;
		},
		status: function(){
			return this._current ? this._current.status() : "stopped";
		},
		destroy: function(){
			if(this._onAnimateCtx){ d.disconnect(this._onAnimateCtx); }
			if(this._onEndCtx){ d.disconnect(this._onEndCtx); }
		}
	});
	d.extend(_chain, _baseObj);

	dojo.fx.chain = function(/*dojo._Animation[]*/ animations){
		// summary: Chain a list of dojo._Animation s to run in sequence
		// example:
		//	|	dojo.fx.chain([
		//	|		dojo.fadeIn({ node:node }),
		//	|		dojo.fadeOut({ node:otherNode })
		//	|	]).play();
		//
		return new _chain(animations) // dojo._Animation
	};

	var _combine = function(animations){
		this._animations = animations||[];
		this._connects = [];
		this._finished = 0;

		this.duration = 0;
		d.forEach(animations, function(a){
			var duration = a.duration;
			if(a.delay){ duration += a.delay; }
			if(this.duration < duration){ this.duration = duration; }
			this._connects.push(d.connect(a, "onEnd", this, "_onEnd"));
		}, this);
		
		this._pseudoAnimation = new d._Animation({curve: [0, 1], duration: this.duration});
		var self = this;
		d.forEach(["beforeBegin", "onBegin", "onPlay", "onAnimate", "onPause", "onStop"], 
			function(evt){
				self._connects.push(d.connect(self._pseudoAnimation, evt,
					function(){ self._fire(evt, arguments); }
				));
			}
		);
	};
	d.extend(_combine, {
		_doAction: function(action, args){
			d.forEach(this._animations, function(a){
				a[action].apply(a, args);
			});
			return this;
		},
		_onEnd: function(){
			if(++this._finished == this._animations.length){
				this._fire("onEnd");
			}
		},
		_call: function(action, args){
			var t = this._pseudoAnimation;
			t[action].apply(t, args);
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			this._finished = 0;
			this._doAction("play", arguments);
			this._call("play", arguments);
			return this;
		},
		pause: function(){
			this._doAction("pause", arguments);
			this._call("pause", arguments);
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			var ms = this.duration * percent;
			d.forEach(this._animations, function(a){
				a.gotoPercent(a.duration < ms ? 1 : (ms / a.duration), andPlay);
			});
			this._call("gotoPercent", arguments);
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			this._doAction("stop", arguments);
			this._call("stop", arguments);
			return this;
		},
		status: function(){
			return this._pseudoAnimation.status();
		},
		destroy: function(){
			d.forEach(this._connects, dojo.disconnect);
		}
	});
	d.extend(_combine, _baseObj);

	dojo.fx.combine = function(/*dojo._Animation[]*/ animations){
		// summary: Combine an array of `dojo._Animation`s to run in parallel
		//
		// description:
		//		Combine an array of `dojo._Animation`s to run in parallel, 
		//		providing a new `dojo._Animation` instance encompasing each
		//		animation, firing standard animation events.
		//
		// example:
		//	|	dojo.fx.combine([
		//	|		dojo.fadeIn({ node:node }),
		//	|		dojo.fadeOut({ node:otherNode })
		//	|	]).play();
		//
		// example:
		//	When the longest animation ends, execute a function:
		//	| 	var anim = dojo.fx.combine([
		//	|		dojo.fadeIn({ node: n, duration:700 }),
		//	|		dojo.fadeOut({ node: otherNode, duration: 300 })
		//	|	]);
		//	|	dojo.connect(anim, "onEnd", function(){
		//	|		// overall animation is done.
		//	|	});
		//	|	anim.play(); // play the animation
		//
		return new _combine(animations); // dojo._Animation
	};

	dojo.fx.wipeIn = function(/*Object*/ args){
		// summary:
		//		Returns an animation that will expand the
		//		node defined in 'args' object from it's current height to
		//		it's natural height (with no scrollbar).
		//		Node must have no margin/border/padding.
		args.node = d.byId(args.node);
		var node = args.node, s = node.style, o;

		var anim = d.animateProperty(d.mixin({
			properties: {
				height: {
					// wrapped in functions so we wait till the last second to query (in case value has changed)
					start: function(){
						// start at current [computed] height, but use 1px rather than 0
						// because 0 causes IE to display the whole panel
						o = s.overflow;
						s.overflow="hidden";
						if(s.visibility=="hidden"||s.display=="none"){
							s.height="1px";
							s.display="";
							s.visibility="";
							return 1;
						}else{
							var height = d.style(node, "height");
							return Math.max(height, 1);
						}
					},
					end: function(){
						return node.scrollHeight;
					}
				}
			}
		}, args));

		d.connect(anim, "onEnd", function(){ 
			s.height = "auto";
			s.overflow = o;
		});

		return anim; // dojo._Animation
	}

	dojo.fx.wipeOut = function(/*Object*/ args){
		// summary:
		//		Returns an animation that will shrink node defined in "args"
		//		from it's current height to 1px, and then hide it.
		var node = args.node = d.byId(args.node), s = node.style, o;
		
		var anim = d.animateProperty(d.mixin({
			properties: {
				height: {
					end: 1 // 0 causes IE to display the whole panel
				}
			}
		}, args));

		d.connect(anim, "beforeBegin", function(){
			o = s.overflow;
			s.overflow = "hidden";
			s.display = "";
		});
		d.connect(anim, "onEnd", function(){
			s.overflow = o;
			s.height = "auto";
			s.display = "none";
		});

		return anim; // dojo._Animation
	}

	dojo.fx.slideTo = function(/*Object?*/ args){
		// summary:
		//		Returns an animation that will slide "node" 
		//		defined in args Object from its current position to
		//		the position defined by (args.left, args.top).
		// example:
		//	|	dojo.fx.slideTo({ node: node, left:"40", top:"50", unit:"px" }).play()

		var node = args.node = d.byId(args.node), 
			top = null, left = null;

		var init = (function(n){
			return function(){
				var cs = d.getComputedStyle(n);
				var pos = cs.position;
				top = (pos == 'absolute' ? n.offsetTop : parseInt(cs.top) || 0);
				left = (pos == 'absolute' ? n.offsetLeft : parseInt(cs.left) || 0);
				if(pos != 'absolute' && pos != 'relative'){
					var ret = d.coords(n, true);
					top = ret.y;
					left = ret.x;
					n.style.position="absolute";
					n.style.top=top+"px";
					n.style.left=left+"px";
				}
			};
		})(node);
		init();

		var anim = d.animateProperty(d.mixin({
			properties: {
				top: args.top || 0,
				left: args.left || 0
			}
		}, args));
		d.connect(anim, "beforeBegin", anim, init);

		return anim; // dojo._Animation
	}

})();

}

if(!dojo._hasResource["com.ibm.portal.utilities.html"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.utilities.html"] = true;
dojo.provide( "com.ibm.portal.utilities.html" );

dojo.require( "com.ibm.portal.utilities" );
dojo.require( "dojo.fx" );

com.ibm.portal.utilities.html = {
	createAnchor: function ( /*HTMLElement*/parent, /*String*/href, 
									     /*String*/id, /*String*/displayText, 
										 /*String*/className ) {
					
		// summary: Creates an anchor element using the given attributes and appends it to the parent.
		// parent: the parent element that will contain the created anchor
		// href: the value of the href attribute
		// id: the id of the anchor element
		// displayText: the inner text of the anchor
		// className: the CSS style name to apply
		// returns: the created anchor object
		ibm.portal.debug.entry( "SkinRenderer.createAnchor", [ parent, href, id, displayText, className ] );
		
		//Create and set the attributes for the anchor.
		var anchor = document.createElement( "A" );
		anchor.href = href;
		if ( id ) { anchor.id = id; }
		if ( className ) { anchor.className = className; }
		if ( displayText ) { anchor.appendChild( document.createTextNode( displayText ) ); }
												 	
		//Append the anchor to the parent.
		parent.appendChild( anchor );
		
		ibm.portal.debug.exit( "SkinRenderer.createAnchor", [ anchor ]);
		return anchor; //HTMLElement																
	},
	createButton: function ( /*HTMLElement*/parent, /*String*/href, 
						     /*String*/id, /*String?*/displayText, 
							 /*String*/className ) {
		
		// summary: Creates a button element using the given attributes and appends it to the parent.
		// parent: the parent element that will contain the created anchor
		// href: the value of the href attribute
		// id: the id of the anchor element
		// displayText: the inner text of the anchor
		// className: the CSS style name to apply
		// returns: the created anchor object
		ibm.portal.debug.entry( "SkinRenderer.createButton", [ parent, href, id, displayText, className ] );
		
		//Create and set the attributes for the anchor.
		var button = document.createElement( "BUTTON" );
		if ( href ) { button.href = href; }
		if ( id ) { button.id = id; }
		if ( className ) { button.className = className; }
		if ( displayText ) {
			button.appendChild( document.createTextNode( displayText ) );
		}	
												 	
		//Append the anchor to the parent.
		parent.appendChild( button );
		
		ibm.portal.debug.exit( "SkinRenderer.createButton", [ button ]);
		return button; //HTMLElement																
	},
	createImage: function ( /*HTMLElement*/parent, /*String*/src,
							/*String*/id, /*String*/altText,
							/*String*/className ) {
								
		// summary: Creates an img element using the given attributes and appends it to the parent.
		// parent: the parent element that will contain the created image
		// src: the url to the image
		// id: the id of the created image element
		// altText: the alternate text for the image
		// className: the CSS style name to apply.
		ibm.portal.debug.entry( "SkinRenderer.createImage", [ parent, src, id, altText, className ] );
		
		//Create and set attributes for our image.
		var img = document.createElement( "IMG" );
		img.src = src;
		if ( id ) { img.id = id; }
		if ( altText ) { 
			img.alt = altText; img.setAttribute("title", altText); 
			//set the title for the button ("tooltip")
			if( parent.nodeName == "BUTTON") {
				parent.setAttribute("title", altText);
			}
		}
		if ( className ) { img.className = className; }
		
		//Append the image to the parent.
		parent.appendChild( img );
		
		ibm.portal.debug.exit( "SkinRenderer.createImage", [ img ] );
		return img; //HTMLElement	
	},
	createImageAnchor: function ( /*HTMLElement*/parent, /*String*/src,
							/*String*/id, /*String*/altText,
							/*String*/className ) {
								
		// summary: Creates an img element using the given attributes and appends it to the parent.
		// parent: the parent element that will contain the created image
		// src: the url to the image
		// id: the id of the created image element
		// altText: the alternate text for the image
		// className: the CSS style name to apply.
		ibm.portal.debug.entry( "SkinRenderer.createImageAnchor", [ parent, src, id, altText, className ] );
		
		var anchor = com.ibm.portal.utilities.html.createAnchor( parent, "javascript:void(0);" );
		
		//Create and set attributes for our image.
		var img = document.createElement( "IMG" );
		img.src = src;
		if ( id ) { img.id = id; }
		if ( altText ) { img.alt = altText; img.title = altText; }
		if ( className ) { img.className = className; }
		
		//Append the image to the parent.
		anchor.appendChild( img );
		
		ibm.portal.debug.exit( "SkinRenderer.createImageAnchor", [ img ] );
		return anchor; //HTMLElement	
	},
	createTemporaryMarkupDiv: function ( /*String*/ markupStr ) {
		// summary: Creates a div for putting our markup into.  Returns a map with these properties:
		//		{ 	
		// 			node: the new div node,
		//			objects: a map of the extracted OBJECTS keyed by the unique id of the placeholder node in the div node
		//		}
		// description: Necessary because IE doesn't like it when you set innerHTML on a div and the first
		//		element is a script element. For some reason, any script elements up until the first actual
		//		markup element will not be inserted into the DOM. Our workaround is to create a hidden <p>
		//		element which will always be the first element in the markup string.  In IE, this will also
		//		extract all OBJECT elements and put them into the objects property map on the return object
		//		to later be put back into the DOM once the div in the node property of the return object
		//		has been placed into the DOM.  The works around an issue where using innerHTML or outerHTML
		//		of OBJECT elements doesn't retrieve their children in IE.
		// markupStr: the markup string.
		ibm.portal.debug.entry( "html.createTemporaryMarkupDiv" );
		
		var markupMap = {markup: markupStr, objects: {}};
		// fix for inserting OBJECT elements with contents in IE
		if(dojo.isIE) {
			markupMap = com.ibm.portal.utilities.html.extractObjectElementsFromString(markupStr);
		}
		
		var div = document.createElement( "DIV" );
		div.innerHTML = "<p style='display: none;'>&nbsp;</p>" + markupMap.markup;
		
		ibm.portal.debug.exit( "html.createTemporaryMarkupDiv", [ div ] );
		return {node: div, objects: markupMap.objects};
	},
	extractObjectElementsFromString: function ( /*String*/ markupStr ) {
		// summary: Extracts all OBJECT elements from the markupStr and returns an object with the following
		//		properties -
		//		{
		//			markup: the modified markup with uniquely identifiable placeholder nodes in place of the removed
		//				OBJECT elements,
		//			objects: a map of the extracted OBJECTS keyed by the unique id of the placeholder node in the new
		//				markup string
		//		}
		// markupStr: the markup string
		var objElems = {};
		var openRegExp = /<object/gi;
		var closeRegExp = /<\/object>/gi;
		var markup = markupStr;
		var match = null;
		try {
			match = openRegExp.exec(markup);
			if(match && match.index > -1) {
				var start = match.index;
				var buf;
				var end;
				var middle;
				var id;
				while(start > -1) {
					buf = markup.substring(0, start);
					end = markup.indexOf(">", start);
					if(markup.charAt(end - 1) == "/") {
						// we encountered an <OBJECT .... /> tag, so just leave it since it has no contents
						openRegExp.lastIndex = end;
						match = openRegExp.exec(markup);
						if(match) {
							start = match.index;
							continue;
						}
						else break;	// we didn't find anything after the last match
					}
					closeRegExp.lastIndex = start;
					match = closeRegExp.exec(markup);
					if(match) end = match.index;
					else break; // bad markup, OBJECT not closed, just let it go
					
					middle = markup.substring(start, end + 9);
					id = dojo.dnd.getUniqueId();
					markup = buf + "<div id='" + id + "'></div>" + markup.substring(end + 9);
					objElems[id] = middle;
					openRegExp.lastIndex = 0;	// start at the beginning since the markup string has changed
					match = openRegExp.exec(markup);
					
					if(match) start = match.index;
					else break;	// no more matches
				}
			}
			markupStr = markup;
		}
		catch(e) {
			// if we get a failure during the processing of the markup, then the markupStr will still be the same
			// that means we should clear out the objElems and just let the markup pass through as-is
			objElems = {};
		}
		return {markup: markupStr, objects: objElems};
	},
	replaceObjectElementsInMarkup: function ( /*Map*/ objects ) {
		// summary: For every key in the objects argument map, this function looks up a node in the DOM whose id
		// 		equals the key and replaces the outerHTML of that node with the string contents of the property
		//		value of the objects map with that key.  It's useful for putting markup from various nodes back 
		// 		into the DOM that was extracted earlier for some purpose.
		for(var id in objects) {
    		var objContainer = dojo.byId(id);
    		if(objContainer) {
    			objContainer.outerHTML = objects[id];
    		}
    	}
	},
	removeNodesOnCondition: function ( /*DOMElement*/node, /*Function*/condition ) {
		// summary: Walks down this node's descendants and removes all nodes that cause the given
		// condition function to return true, effectively satisfying the condition for removal.
		// Useful for delinting XSL transformations that have created unwanted nodes in the markup.
		if(!condition) condition = function(){return false;};
		if(node && node.childNodes) {
			for(var i = 0; i < node.childNodes.length; i++) {
				if(condition(node.childNodes[i])) {
					var nonNode = node.childNodes[i];
					node.removeChild(nonNode);
					delete nonNode;
					i--; // going back because the childNodes array has been updated
				}
				else {
					this.removeNodesOnCondition(node.childNodes[i], condition);
				}
			}
		}
	},
    getElementsByTagNames: function ( /*HTMLElement*/baseElem /*String 1...N*/ ) {
        // summary: Retrieves elements by tag name for 1 to N tag names.
        // description: Builds a single array with all elements of any tag name specified. A base element
        //        must be specified event if it's just the document object. Any number of tag names can be
        //        searched for, just simply keep appending parameters to the function call.
        // baseElem: the base element to search from
        // tagName1: a tag name
        // tagName2: a tag name
        // ...
        // tagNameN: a tag name
        ibm.portal.debug.entry( "html.getElementsByTagNames", [ baseElem ] );
        
        var retVal = new Array();
        
        for ( var i=1; i < arguments.length; i++ ) {
            var elements = baseElem.getElementsByTagName( arguments[i] );
            ibm.portal.debug.text( 'found ' + elements.length + ' ' + arguments[i] + ' tags.');
            for ( var j=0; j < elements.length; j++ ) {
                retVal.push( elements[j] );
            }    
        }
        
        ibm.portal.debug.exit( "html.getElementsByTagNames", [ retVal ] );
        return retVal;
    },
    getX: function ( /*HTMLElement*/elem ) {
    	// summary: Calculates the X coordinate of the top left corner of the given element. This
    	//		function was stolen from WCL and included here so that we have no dependency on 
    	//		WCL.
    	// elem: the html element to calculate the position of.
    	// returns: an integer with the X position.
    	ibm.portal.debug.entry( "html.getX", [ elem ] );
    	
    	var size = 0;
	    
	    if ( elem != null ) {
	       if ( elem.offsetParent != null ) {
	          size += com.ibm.portal.utilities.html.getX( elem.offsetParent );
	       }
	       if ( elem != null ) {
	          size += elem.offsetLeft;
	       }
	
	    }
	    
	    ibm.portal.debug.exit( "html.getX", [ size ] );
	    return size; // int
    },
    getY: function ( /*HTMLElement*/elem ) {
    	// summary: Calculates the Y coordinate of the top left corner of the given element. This
    	//		function was stolen from WCL and included here so that we have no dependency on 
    	//		WCL.
    	// elem: the html element to calculate the position of.
    	// returns: an integer with the Y position
    	ibm.portal.debug.entry( "html.getY" [ elem ] );
    	
    	var size = 0;

	    if ( elem != null ) {

	       if ( elem.offsetParent != null ) {
	          size += com.ibm.portal.utilities.html.getY( elem.offsetParent );
	       }
	       if ( elem != null ) {
	          size += elem.offsetTop;
	       }
	
 	    }
 	    
 	    ibm.portal.debug.exit( "html.getY", [ size ] );
 	    return size; // int
    },
    convertFormToQuery: function ( /*HTMLElement*/formObj, /*{name:, value:}*/submitTrigger ) {
    	// summary: Converts a given form object to a query string.
    	// description: Iterates through the given form object's "elements" and attempts to
    	//		a name/value pair to the query string. All parameters are encoded using the native
    	//		javascript function "encodeURIComponent". Since a form may contain multiple submit
    	//		inputs, the submit input name which triggered the current submission may be passed 
    	// 		to this function so that the submit input which triggered the submission may be properly
    	//		encoded into the query string. 
    	// formObj: the form object to decode
    	// submitTrigger: the name and value of the input which triggered the submission.
    	// returns: the query string with all name/value pairs that were successfully decoded
		ibm.portal.debug.entry( "html.convertFormToQuery", [ formObj, submitTrigger ] );
		
		//For some reason, formObj.elements didn't seem to always return ALL input elements.
		var elements = this.getElementsByTagNames( formObj, "input", "select", "textarea", "button" );
    	var queryStr = "";
    	var querySep = "&";
    	var queryNameValueSep = "=";
		var processed = 0;
    	for ( var i = 0; i < elements.length; i++ ) {
    		var inputNV = this.convertInputToNameValuePairs( elements[i], submitTrigger );
    		for ( var k = 0; k < inputNV.length; k++ ) {
				var pair = inputNV[k];
				if ( pair.name != "" ) {
					if ( processed != 0 ) {
						queryStr += querySep;
					}	
					queryStr += encodeURIComponent( pair.name );
		    		
					for ( var j = 0; j < pair.values.length; j++ ) {
		    			if ( j == 0 ) {
		    				queryStr += ( queryNameValueSep + encodeURIComponent( pair.values[j] ) ); 
		    			}
		    			else {
		    				queryStr += ( querySep + encodeURIComponent(pair.name) + queryNameValueSep + encodeURIComponent( pair.values[j] ) );
		    			}	
		    		}
					
					processed = processed+1;
				}
			}	
   		
    	}
		
    	ibm.portal.debug.exit( "html.convertFormToQuery", queryStr );
    	return queryStr;
    },
    convertInputToNameValuePairs: function ( /*HTMLElement*/input, /*{name:,value:}*/submitTrigger ) {
    	// summary: Converts a given form element to a name value pair.
    	// description: Given an HTML INPUT element, produces an object representation of the following format:
    	//		{ name: <input name>, values: [ <input value1>, <input value2, ..., <input valueN> ] } where values is likely
    	//		an array with one value. The array is used to cover the case where a single input has multiple values. Submit
    	//		inputs are only included if no submitTrigger is specified OR the submitTrigger variable value matches the 
    	//		submit input's "name" attribute.
    	// input: the HTML INPUT element to convert
    	// submitTrigger: the input which triggered the form submission
    	// returns: a name/value pair { name: "", values: [] }, guaranteed to be not NULL
    	ibm.portal.debug.entry( "html.convertInputToNameValuePairs", [ input, submitTrigger ]);
    	
    	var type = input.type;
    	ibm.portal.debug.text( "Input type is: " + type );
		ibm.portal.debug.text( "Input name is: " + input.name );
    	var name = "";
    	var values = [];
		var retPairs = [];
    	
    	if ( !input.disabled ) {
    		    		
			switch ( type.toLowerCase() ) {
				case "text":
				case "password":
				case "hidden":
					name = input.name; values.push( input.value );
					retPairs.push( { name: name, values: values } );
					break;
				case "reset":
				case "button": 
					if ( !submitTrigger || ( input.name == submitTrigger.name && input.value == submitTrigger.value ) ) {
						name = input.name; values.push( input.value );
						retPairs.push( { name: name, values: values } );				
					}
					break;		
				case "radio":
				case "checkbox": if ( input.checked ) {
						name = input.name;
						values.push( input.value );
					}
					retPairs.push( { name: name, values: values } );
					break;
				case "image": if ( !submitTrigger || input.name == submitTrigger ) {
						name = input.name;
						if ( input.value ) {
							values.push( input.value );
							retPairs.push( { name: name, values: values } );
						}
						
						retPairs.push( { name: name + ".x", values: [ this.getX( input ) ] } );
						retPairs.push( { name: name + ".y", values: [ this.getY( input ) ] } );
					}
					break;
				case "submit": if ( !submitTrigger || ( input.name == submitTrigger.name && input.value == submitTrigger.value ) ) {
						name = input.name;
						if ( input.value ) {
							values.push( input.value );
						}
						retPairs.push( { name: name, values: values } );
					}
					break;
				case "select-one": 
				case "select-multiple": name = input.name;
					for ( var i = 0; i < input.options.length; i++ ) {
						if ( input.options[i].selected ) {
							var value = input.options[i].value ? input.options[i].value : input.options[i].text;
							values.push( value );
						}
					}
					if ( values.length != 0 ) { 
						retPairs.push( { name: name, values: values } );
					}	
					
					break;
				case "file": break;
				default: name = input.name; values.push( input.value ); retPairs.push( { name: name, values: values } );
			}
    	}	
		
		ibm.portal.debug.exit( "html.convertInputToNameValuePairs", retPairs );
		return retPairs;	
    },
		isHidden: function(node) {
    	// summary: determines whether the specified node is hidden or shown.
    	// description: this is a lightweight replacement for the obsolete dojo.html.isHidden function. It checks the element
    	//		only, not its parents, for display:none being set.
    	// input: the dom node
    	// returns: true if the node has display:none style set, false otherwise	
			return dojo.style(node, 'display') == 'none';
		},
		hide: function(node) {
    	// summary:  hides the specified node from the rendered page.
    	// description: this is a replacement for the obsolete dojo.html.hide() function. 
    	// input: the dom node to hide
			dojo.fx.wipeOut({node: node, duration: 5}).play();
		},
		show: function(node) {
    	// summary:  shows the specified node from the rendered page.
    	// description: this is a replacement for the obsolete dojo.html.show) function. 
    	// input: the dom node to show
			dojo.fx.wipeIn({node: node, duration: 5}).play();
		},
		isDescendantOf: function(node, ref) {
			//summary: Checks to see if the given node descends from the reference node.
			//node: node to check
			//ref: the reference node
			// returns: true if the node is a descendant of the given reference node
			var node = node.parentNode;
			var found = false;
			
			while ( node && !found ) {
				if ( node == ref ) {
					found = true;
				}
				node = node.parentNode;
			}
			
			return found;
		}
	
};

}

if(!dojo._hasResource["com.ibm.portal.services.PortalRestServiceRequest"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.services.PortalRestServiceRequest"] = true;
dojo.provide( "com.ibm.portal.services.PortalRestServiceRequest" );

dojo.require( "com.ibm.ajax.auth" );
dojo.require( "com.ibm.portal.EventBroker" );

dojo.require( "com.ibm.portal.services.PortalRestServiceRequestQueue" );

dojo.declare( "com.ibm.portal.services.ContentHandlerURL",
			  null,
			  {
		  		// summary: Provides convenience methods to construct URLs to the 
				// 		content handler servlet which serves most of the Portal 
				//		REST services.
				
				constructor: function ( /*String*/uri, /*String or int*/levels, /*String*/verb, /*String?*/params ) {
				  	ibm.portal.debug.entry( "ContentHandlerURL.constructor", [ uri, levels, verb, params ]);
				  	//If no URI is passed in, just return null.
					if ( uri == null ) {
						return null;
					}	
					
					if ( !levels ) { 
						levels = 2;
					}

					// check if we need to enforce a specific locale
					var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
					var locale = state.getLocale();
					if (locale) {
						if (params) {
							params += "&locale=" + locale;
						} else {
							params = "&locale=" + locale;
						}
					}
				  
				  	this.url = "";
					if ( uri.charAt(0) == "?" ) {
						this.url = this._fromQueryString( uri, params );	
					}
					else {
						this.url = this._fromURI( uri, levels, "download", params );
					}
					ibm.portal.debug.exit( "ContentHandlerURL.constructor" );
			    },
				_fromQueryString: function ( /*String*/queryString, /*String?*/params  ) {
					// summary: Constructs an url from a given query string.
					// description: Constructs the URL to the content handler servlet using
					//		the given query string. Replaces any '&amp;' characters with '&' 		
					//		since Firefox thinks the 'amp;' is part of the parameter name.
					// queryString: the query string to construct the URL from
					// returns: String - the constructed URL
					ibm.portal.debug.entry( "fromQueryString", [ queryString ] );
				
					var str = ibmPortalConfig["contentHandlerURI"] + queryString;
					str = str.replace(/&amp;/g,"&" );
					if ( params ) {
						str = str + params;
					}
					
					if ( str.indexOf( "rep=compact" ) < 0 && str.indexOf( "rep=full" ) < 0 ) {
						str = str + "&rep=compact";
					}	
				
					ibm.portal.debug.exit( "fromQueryString", [ str ] );
					
					return str; //String	
				},
				_fromURI: function ( /*String*/uri, /*int?*/levels, /*String?*/verb, /*String?*/params ) {
					// summary: Constructs an URL using the given parameters.
					// description: Constructs the URL to the content handler servlet for
					//		the given URI, number of levels to traverse, and the action verb
					//		for the REST service to execute. It is not generally recommended
					//		to request more than 2 levels at a time since this will cause an
					//		expensive tree traversal server-side. The verb is generally expected
					//		to be set to "download".
					// uri: the uri for the resource to request
					// levels: the number of levels to include in the response
					// verb: the action to take (e.g. "view" or "download")
					// returns: String - the constructed URL
					ibm.portal.debug.entry( "ContentHandlerURL._fromURI", [ uri, levels, verb, params ] );
					
					uri = com.ibm.portal.utilities.encodeURI( uri );
					
					var qStr = "?uri=" + uri;
					
					if ( levels ) {
						qStr = qStr + "&levels=" + encodeURIComponent( levels );
					}
					
					if ( verb ) {
						qStr = qStr + "&mode=" + encodeURIComponent( verb );
					}
					if ( params ) {
						qStr = qStr + params;
					}
					
					if ( qStr.indexOf( "rep=compact" ) < 0 && qStr.indexOf( "rep=full" ) < 0 ) {
						qStr = qStr + "&rep=compact";
					}	
					
					return this._fromQueryString( qStr ); //String
				},
				getURI: function() {
					ibm.portal.debug.entry( "ContentHandlerURL.getURI" );
					return com.ibm.portal.utilities.decodeURI( this._extractParamValue( "uri" ) ); //String
				},
				getLevels: function () {
					return this._extractParamValue( "levels" );
				},
				getVerb: function () {
					return this._extractParamValue( "verb" );
				},
				_extractParamValue: function ( /*String*/ paramName ) {
					ibm.portal.debug.entry( "ContentHandlerURL._extractParamValue", [ paramName ] );
					
					var startIndex = this.url.indexOf( paramName );
					var endIndex = this.url.indexOf( "&", startIndex );
					var uriParamStr = this.url.slice( startIndex + paramName.length + 1, endIndex );
					
					ibm.portal.debug.exit( "ContentHandlerURL._extractParamValue", [ uriParamStr ]);
					
					return uriParamStr;
				}
			  }
);

dojo.require( "com.ibm.portal.utilities.html" );

dojo.declare( "com.ibm.portal.services.PortalRestServiceForm",
			  null,
			  {
			  	// summary: Object representation of a form that will be submitted to one of the Portal
			  	//		REST services. Encapsulates all the information about the form necessary to get
			  	//		the form info to the server.
			  	// method: the METHOD of the form (usually GET or POST)
			  	// isMultipart: indicates whether this is a multipart encoded form or not
			  	// encoding: the encoding used to submit the form
			  	// DomId: the ID of the DOM element corresponding to the form
			  	method: "GET",
			  	isMultipart: false,
			  	encoding: "application/x-www-form-urlencoded",
			  	DomId: null,
			  	constructor: function ( /*HTMLElement*/formElem ) {
			  		if ( formElem.getAttributeNode("method" ) ) {
			  			this.method = formElem.getAttributeNode( "method" ).value;	
			  		}
			  		if ( formElem.getAttributeNode( "encType" ) ) {
			  			this.encoding = formElem.getAttributeNode( "encType" ).value;
			  		}
			  		if ( formElem.getAttributeNode( "id" ) ) {	
			  			this.DomId = formElem.getAttributeNode( "id" ).value;
			  		}
			  		else {
			  			DomId = formElem;
			  		}	
			  		this.isMultipart = ( this.encoding == "multipart/form-data");
			  	},
			  	getDOMElement: function () {
			  		// summary: Returns the DOM element for this form.
			  		// returns: the DOM element corresponding to this form
			  		return dojo.byId( this.DomId );
			  	},
			  	submit: function () {
			  		// summary: Submits the form. This is necessary because the native submit function
			  		//		may have been overridden to ensure the form submission was intercepted.
			  		this.getDOMElement().submit();
			  	},
			  	toQuery: function () {
			  		// summary: Translates the form into a query string. Only elements which make sense
			  		//		for a query string are translated (i.e. NOT inputs of type "file").
			  		// returns: a string of name/value pairs suitable for being included as part of a url's
			  		//		query string.
			  		return com.ibm.portal.utilities.html.convertFormToQuery( this.getDOMElement() );
			  	}			  	
			  }
);

com.ibm.portal.services.REQUEST_QUEUE = new com.ibm.portal.services.PortalRestServiceRequestQueue();

dojo.declare( "com.ibm.portal.services.PortalRestServiceRequest",
			  null,
			  {
			  	// summary: Sends a request to one of the Portal REST Services and handles the response.
				// description: Create a new instance of this object for every REST service request that is
				//		sent. The URL is passed in to the constructor. If a form node and form filter is 
				// 		passed into the constructor, the REST service request is handled as a form submit.
				// 		The request can also be executed synchronously if TRUE is passed into the constructor.
				// 		In general, only the model implementations should send these requests.
				
				constructor: function( /*ContentHandlerURL*/feedlocation, /*PortalRestServiceForm?*/form, /*boolean?*/textOnly, /*boolean?*/sync ) {
				  	ibm.portal.debug.entry( "PortalRestServiceRequest.constructor", [ feedlocation, form, textOnly, sync ] );
					this._feedURI = feedlocation.url;
					this._textOnly = textOnly;
					this._sync = sync;
					this._form = form;
					this._customResponseValidator = null;
					this._onauthenticated = null;
					
					//Make sure this isn't undefined.
					if ( !this._sync ) {
							this._sync = false;
					}
					ibm.portal.debug.exit( "PortalRestServiceRequest.constructor" );
			    },
			    setAuthenticationValidator: function ( /*Function*/checkFn ) {
			    	// summary: Sets a function that will be used to check whether the response is valid (i.e. does not require  authentication).
			    	//		The function will take two parameters: one will be the response object (xml, json, string, etc);
			    	//		the second will be the dojo._ioArgs as provided to the dojo.xhr method. The function should return true
			    	//		if the response is valid and false otherwise.
			    	this._customResponseValidator = checkFn;
			    },
			    setOnAuthenticatedHandler: function ( /*Function*/onauth ) {
			    	// summary: Sets a function that CAN be called once authentication has been achieved. IMPORTANT: This handler will NOT
			    	//		be called in the default authentication handler for Portal because it redirects the browser to the login page.
			    	this._onauthenticated = onauth;
			    },
				create: function ( /*String*/data, /*Function*/callbackfn, callbackargs ) {
					// summary: Creates an entry in the backing model.
					// description: Sends a POST request to create an entry in the addressed model.
					// data: the body of the POST request
			    	// callbackfn: the function to call when the operation is complete
			    	// callbackargs: additional arguments to pass to callbackfn
					
			    	this._doXmlHttpRequest( "POST", data, callbackfn, callbackargs );
				},
				read: function ( /*Function?*/callbackfn, /*Object?*/addtlCallbackFnArguments ) {
					// summary: Read the ATOM feed provided by the REST service. 
					// description: If textOnly is set to true, the ATOM feed is returned as text and 
					//		 passed in as a single argument to the specified callback function. 
					//		 Otherwise, the ATOM feed is parsed into an {@link ibm.atom.Feed} object 
					//		 and passed into the callback function along with the original XML Document 
					//		 Object Model object.
					// callbackfn: the function to call when the operation is complete (only valid if
					//		 sync is false)
					// addtlCallbackFnArguments: any arguments to pass through to the callback function
					ibm.portal.debug.entry( "PortalRestServiceRequest.read", [ callbackfn, addtlCallbackFnArguments ] );
					
					if ( !this._sync ) {
						ibm.portal.debug.text( "Queueing request!" );
						var q = com.ibm.portal.services.REQUEST_QUEUE;
						var me = this;
						q.add( 
							   { 
							   	execute: function ( doneCallback ) { 
											com.ibm.portal.EVENT_BROKER.startRequest.fire( { uri: me._feedURI } );
											
											var wrappedCallbackFn = function ( arg1, arg2, arg3, arg4 ) { 
												callbackfn( arg1, arg2, arg3, arg4 );
												if ( doneCallback ) {
													doneCallback();
												}
											}
											
											if ( me._textOnly ) {
												me._retrieveRawFeed( wrappedCallbackFn, addtlCallbackFnArguments );
											}	
											else {
												me._retrieve( wrappedCallbackFn, addtlCallbackFnArguments );
											}
								          }
							   }	          
							);
					}
					else {
						com.ibm.portal.EVENT_BROKER.startRequest.fire( { uri: this._feedURI } );	
						
						if ( this._textOnly ) {
							this._retrieveRawFeed( callbackfn, addtlCallbackFnArguments );
						}	
						else {
							this._retrieve( callbackfn, addtlCallbackFnArguments );
						}
					}	
							
					ibm.portal.debug.exit( "PortalRestServiceRequest.read");
				},
				update: function ( /*String*/data, /*Function*/callbackfn, callbackargs ) {
					// summary: Updates an entry in the backing model. 
					// description: Sends a PUT request to update the addressed entry using the provided data.
					// data: the payload of the request
					// callbackfn: the function to call when the operation is complete
					// callbackargs: any additional arguments to pass to the callbackfn
					
					this._doXmlHttpRequest( "Put", data, callbackfn, callbackargs );
				},
				remove: function ( /*Function*/callbackfn, callbackargs ) {
					// summary: Removes the addressed entry from the backing model. 
					// description: Sends a DELETE request to remove the addressed entry.
					// callbackfn: the function to call when the operation is complete
					// callbackargs: any arguments to be passed to the callbackfn
					
					this._doXmlHttpRequest( "Delete", null, callbackfn, callbackargs );
				},
				_retrieveRawFeed: function ( callbackfn, callbackargs ) {
					ibm.portal.debug.entry( "_retrieveRawFeed", [callbackfn, callbackargs] );
					var me = this;
					dojo.xhrGet({
						url: this._feedURI,
						load: function( type, data, evt ) {
							callbackfn( data, callbackargs );
							com.ibm.portal.EVENT_BROKER.endRequest.fire( { uri: me._feedURI } ); 
						},
						sync: this._sync
					});
					
					ibm.portal.debug.exit( "_retrieveRawFeed" );
				},
				_retrieve: function ( callbackfn, callbackargs, formNode, formFilter ) {
					ibm.portal.debug.entry( "_retrieve", [ callbackfn ] );
					
					if ( this._form && this._form.isMultipart ) {
						this._doIframeRequest( callbackfn, callbackargs );
					}
					else {
						this._doXmlHttpRequest( "Get", null, callbackfn, callbackargs );
					}
					
					ibm.portal.debug.exit( "PortalRestServiceRequest._retrieve" );  
				},
				_doIframeRequest: function ( callbackfn, callbackargs ) {
					// summary: Handles a multi-part form submission.
					// description: Multi-part form submissions are not currently feasible using XMLHttpRequests, so
					// 		we need to submit these forms using IFRAMEs. 
					
					ibm.portal.debug.entry( "PortalRestServiceRequest._doIframeRequest", [ callbackfn ] );
					
					var iframe = null;
					var iframeName = dojo.dnd.getUniqueId(); 
					
					if ( dojo.isIE ) { 
						//Any other method of assigning the onload handler in IE seems to fail miserably...
						iframe = document.createElement( "<iframe name='" + iframeName + "' id='"+iframeName + "' src='about:blank' onload='com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER.handleMultiPartResult(this.id);'></iframe>" );
						//This is necessary so that we actually have the Function object to call when the iframe loads (since we seem to
						//only be able to assign an onload handler using text insertion methods).
						com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER._callbackfns[iframeName]= { fn: callbackfn, args: callbackargs };
						//IE applies an XSLT to an XML result in an IFRAME so we need text only.
						var url = new com.ibm.portal.utilities.HttpUrl( this._feedURI );
						url.addParameter( "ibm.web2.contentType", "text/plain" );
						this._form.getDOMElement().setAttribute( "action", url.toString() );
					}
					else {
						ibm.portal.debug.text( "Creating the iframe... name is: " + iframeName + "; url is: " + this._feedURI );
						iframe = document.createElement( "IFRAME" );
						iframe.setAttribute( "name", iframeName );
						iframe.setAttribute( "id", iframeName );
						var me = this;
						iframe.onload = function() { var xml = window.frames[iframeName].document; callbackfn( "load", xml, null, callbackargs ); com.ibm.portal.EVENT_BROKER.endRequest.fire( { uri: me._feedURI } ); };
						this._form.getDOMElement().setAttribute( "action", this._feedURI );	
					} 
					//Can't set the iframe's display to 'none' or problems recognizing the iframe will result...
					iframe.style.visibility = "hidden";
					iframe.style.height = "1px";
					iframe.style.width = "1px";
					document.body.appendChild( iframe );
					
					//IE Workaround again... the name isn't always correctly assigned from the above createElement call. 
					if(window.frames[iframeName].name != iframeName) { window.frames[iframeName].name = iframeName; }
					
					ibm.portal.debug.text( "Setting the iframe target attribute to: " + iframeName );
					this._form.getDOMElement().setAttribute( "target", iframeName );
					this._form.submit();
					
					ibm.portal.debug.exit( "PortalRestServiceRequest._doIframeRequest" );
				},
				_doXmlHttpRequest: function ( method, body, callbackfn, callbackargs ) {
					// summary: Performs a standard request using an XMLHttpRequest object.
					// description: If an error response code is returned, the Error object is returned instead of the
					// 		xml payload. This means that a check should be done to see if the response is instanceof Error.
					ibm.portal.debug.entry( "PortalRestServiceRequest._doXmlHttpRequest", [ method, body, callbackfn, callbackargs ] );					
					ibm.portal.debug.text( "Attempting to retrieve: " + this._feedURI + " using method: " + method + "; synchronously? " + this._sync );
					var me = this;
					var args = {
						url: this._feedURI,
						content: {},
                        headers: { "X-IBM-XHR":"true" },
						//headers: { "If-Modified-Since": "Thu, 1 Jan 1970 00:00:00 GMT"}, //temporary to force the browser to ignore cached requests
						handle: function( response, ioArgs ) {
							ibm.portal.debug.entry( "PortalRestServiceRequest.handle", [ response, ioArgs ] );
							//Get the location of the resource actually returned by the response.
							//All re-directs have been followed at this point.
							var xhr = ioArgs.xhr;
							
							ibm.portal.debug.text( "XHR object: " + xhr ); 
							var config = com.ibm.portal.services.PortalRestServiceConfig;
							var digest = xhr.getResponseHeader( "X-Request-Digest" );
							
							if ( digest ) {
								config.digest = digest;	
							}
							
							if ( xhr.status == 200 ) {
								var data = response;
								var loc = xhr.getResponseHeader( "IBM-Web2-Location");
								
								if ( loc ) {
									//Check to see if we are going from a public resource to a protected resource.
									//This means a successful login has occurred and we need to force a full page 
									//refresh.
									if ( loc.indexOf( ibmPortalConfig["portalProtectedURI"]) >= 0 && me._feedURI.indexOf( ibmPortalConfig["portalPublicURI"] ) >= 0 ) {
										top.location.href = loc;
										return;
									}
								}
								
								var contentType = xhr.getResponseHeader( "Content-Type" );
								ibm.portal.debug.text( "content-type is: " + contentType );

                                // XHR request returns a content-type of text/html
                                // on cross page wire events, if the web-2 location is present and
                                // points to a non-contenthandler, skip auth check

                                if (/^text\/html/.exec (contentType) && loc &&  (loc.indexOf( ibmPortalConfig["portalProtectedURI"]) >-1 || loc.indexOf( ibmPortalConfig["portalPublicURI"]) >-1) ){
                                    ibm.portal.debug.text( "content-type is text .. follow IBM-Web2-Location" );
									top.location.href = loc;
									return;
								}								


								
								var auth = com.ibm.ajax.auth;
								var authRequired = false;
								
								if ( me._customResponseValidator ) {
									authRequired = me._customResponseValidator( response, ioArgs );
								}
								if ( !authRequired ) {
									authRequired = auth.isAuthenticationRequired( response, ioArgs );
								}
								
								//If the HTML content is returned, this is probably another re-direction so we need to
								//force a full-page refresh.
								if ( authRequired ) {
									auth.authenticationHandler( response, ioArgs, me._onauthenticated );
									return;
								}	
								
								ibm.portal.debug.text( "Read feed: " + me._feedURI );
								
								if ( dojo.isIE ) {
									var doc = dojox.data.dom.createDocument( data );
									callbackfn( "load", doc, xhr, callbackargs );
								}
								else {
									callbackfn( "load", data, xhr, callbackargs );
								}
                            } else if  (xhr.status==401 || xhr.status==0) {
                                ibm.portal.debug.text( "Basic auth 401 found, trigger reload");
                                com.ibm.ajax.auth.authenticationHandler();
                                return;
                            } else {
                                callbackfn( "error", response, xhr, callbackargs );
							}	
							
							com.ibm.portal.EVENT_BROKER.endRequest.fire( { uri: me._feedURI } );
							ibm.portal.debug.exit( "PortalRestServiceRequest.handle" );
						},
						sync: this._sync,
						handleAs: "xml"	
					}
					
					
					if ( this._form ) {
						//DOJO expects the content parameter to be of the format created by
						//dojo.queryToObject(). The contents of the content parameter are sent either
						//as part of the query string (method=GET) or as the body of the request (method=POST).
						args.content = dojo.queryToObject( this._form.toQuery() );
						method = this._form.method;
					}
					
					//Convert to upper case to make comparisons easier. We always convert to "proper" case right before
					//calling the dojo xhrYyy function.
					method = method.toUpperCase();
					
					if ( method != "GET" && method != "POST" ) {
						if ( ibmPortalConfig && ibmPortalConfig.xMethodOverride ) {
							args.headers["X-Method-Override"] = method.toUpperCase();
							method = "Post";	
						}
					}
					
					if ( method == "PUT" && body ) {
						args.putData = body;
					} else if ( method == "POST" && body ) {
						args.postData = body;
					}	
					
					if ( dojo.isIE ) {
						args.content["ibm.web2.contentType"] = "text/xml";
						args.handleAs = "text"; //temp
					}
					var config = com.ibm.portal.services.PortalRestServiceConfig;
					if ( config.timeout ) {
						args.timeout = config.timeout;
					}
					if ( config.digest ) {
						args.content["digest"] = config.digest;
					}
					
					//Match dojo's convention of xhrGet/xhrPost/xhrPut/etc.
					method = com.ibm.portal.utilities.string.properCase( method );
					
					var xhrMethod = dojo["xhr" + method];
					if ( xhrMethod ) {
						xhrMethod( args );
					} else {
						throw new Error( "Invalid request method attempted: " + method );
					}	
						
					ibm.portal.debug.exit( "PortalRestServiceRequest._doXmlHttpRequest" );
				},
				toString: function () {
					return this._feedURI;
				}
			  }
);
com.ibm.portal.services.PortalRestServiceConfig = {
	timeout: null,
	digest: null
}

com.ibm.ajax.auth.setAuthenticationHandler( function () { 
	if ( typeof( document.isCSA ) == "undefined" ) {
		top.location.reload();
	}
	else {
		ibm.portal.debug.entry( "DefaultAuthenticationHandler" );
		ibm.portal.debug.text( "Illegal response content-type detected!" );
		ibm.portal.debug.text( "Parameterized redirect URL is: " + ibmPortalConfig["contentModelBlankURL"] );
		var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
		var fullPageRefreshURL = ibmPortalConfig["contentModelBlankURL"].replace( "-----oid-----", state.getPageSelection() );
		ibm.portal.debug.text( "fullPageRefreshURL is currently: " + fullPageRefreshURL );
		/*if ( loc ) { 
			fullPageRefreshURL = loc;
			ibm.portal.debug.text( "fullPageRefreshURL is now: " + fullPageRefreshURL );
		}*/
	
		if (dojo.cookie("WASReqURL") != null) {
			// create URL to current navigational state
			// we cannot use fullPageRefreshURL because it does not encode the entire nav state
			var fullStateUrl = state.createLinkToCurrentState();
			var cookieStr = "WASReqURL=" + fullStateUrl + "; path=/";
			// does not work with Dojo cookie APIs
			document.cookie=cookieStr;
		}
	
		ibm.portal.debug.text( "Redirecting to: " + fullPageRefreshURL );
		
		//Fire the redirect event so any necessary clean up can occur.
		com.ibm.portal.EVENT_BROKER.redirect.fire( { url: fullPageRefreshURL } ); 
		top.location.href = fullPageRefreshURL;
		ibm.portal.debug.exit( "DefaultAuthenticationHandler" );
	}
});

}

if(!dojo._hasResource["dojox.xml.parser"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.xml.parser"] = true;
dojo.provide("dojox.xml.parser");

//DOM type to int value for reference.
//Ints make for more compact code than full constant names.
//ELEMENT_NODE                  = 1;
//ATTRIBUTE_NODE                = 2;
//TEXT_NODE                     = 3;
//CDATA_SECTION_NODE            = 4;
//ENTITY_REFERENCE_NODE         = 5;
//ENTITY_NODE                   = 6;
//PROCESSING_INSTRUCTION_NODE   = 7;
//COMMENT_NODE                  = 8;
//DOCUMENT_NODE                 = 9;
//DOCUMENT_TYPE_NODE            = 10;
//DOCUMENT_FRAGMENT_NODE        = 11;
//NOTATION_NODE                 = 12;

dojox.xml.parser.parse = function(/*String?*/ str, /*String?*/ mimetype){
	//	summary:
	//		cross-browser implementation of creating an XML document object from null, empty string, and XML text..
	//
	//	str:
	//		Optional text to create the document from.  If not provided, an empty XML document will be created.  
	//		If str is empty string "", then a new empty document will be created.
	//	mimetype:
	//		Optional mimetype of the text.  Typically, this is text/xml.  Will be defaulted to text/xml if not provided.
	var _document = dojo.doc;
	var doc;

	mimetype = mimetype || "text/xml";
	if(str && dojo.trim(str) && "DOMParser" in dojo.global){
		//Handle parsing the text on Mozilla based browsers etc..
		var parser = new DOMParser();
		doc = parser.parseFromString(str, mimetype);
		var de = doc.documentElement;
		var errorNS = "http://www.mozilla.org/newlayout/xml/parsererror.xml";
		if(de.nodeName == "parsererror" && de.namespaceURI == errorNS){
			var sourceText = de.getElementsByTagNameNS(errorNS, 'sourcetext')[0];
			if(!sourceText){
				sourceText = sourceText.firstChild.data;
			}
        	throw new Error("Error parsing text " + nativeDoc.documentElement.firstChild.data + " \n" + sourceText);
		}
		return doc;

	}else if("ActiveXObject" in dojo.global){
		//Handle IE.
		var ms = function(n){ return "MSXML" + n + ".DOMDocument"; };
		var dp = ["Microsoft.XMLDOM", ms(6), ms(4), ms(3), ms(2)];
		dojo.some(dp, function(p){
			try{
				doc = new ActiveXObject(p);
			}catch(e){ return false; }
			return true;
		});
		if(str && doc){
			doc.async = false;
			doc.loadXML(str);
			var pe = doc.parseError;
			if(pe.errorCode !== 0){
				throw new Error("Line: " + pe.line + "\n" +
					"Col: " + pe.linepos + "\n" +
					"Reason: " + pe.reason + "\n" + 
					"Error Code: " + pe.errorCode + "\n" +
					"Source: " + pe.srcText);
			}
		}
		if(doc){
			return doc; //DOMDocument
		}
	}else if(_document.implementation && _document.implementation.createDocument){
		if(str && dojo.trim(str) && _document.createElement){
			//Everyone else that we couldn't get to work.  Fallback case.
			// FIXME: this may change all tags to uppercase!
			var tmp = _document.createElement("xml");
			tmp.innerHTML = str;
			var xmlDoc = _document.implementation.createDocument("foo", "", null);
			dojo.forEach(tmp.childNodes, function(child){
				xmlDoc.importNode(child, true);
			});
			return xmlDoc;	//	DOMDocument
		}else{
			return _document.implementation.createDocument("", "", null); // DOMDocument
		}
	}
	return null;	//	null
}

dojox.xml.parser.textContent = function(/*Node*/node, /*String?*/text){
	//	summary:
	//		Implementation of the DOM Level 3 attribute; scan node for text
	//	description:
	//		Implementation of the DOM Level 3 attribute; scan node for text
	//		This function can also update the text of a node by replacing all child 
	//		content of the node.
	//	node:
	//		The node to get the text off of or set the text on.
	//	text:
	//		Optional argument of the text to apply to the node.
	if(arguments.length>1){
		var _document = node.ownerDocument || dojo.doc;  //Preference is to get the node owning doc first or it may fail
		dojox.xml.parser.replaceChildren(node, _document.createTextNode(text));
		return text;	//	String
	}else{
		if(node.textContent !== undefined){ //FF 1.5 -- remove?
			return node.textContent;	//	String
		}
		var _result = "";
		if(node){
			dojo.forEach(node.childNodes, function(child){
				switch(child.nodeType){
					case 1: // ELEMENT_NODE
					case 5: // ENTITY_REFERENCE_NODE
						_result += dojox.xml.parser.textContent(child);
						break;
					case 3: // TEXT_NODE
					case 2: // ATTRIBUTE_NODE
					case 4: // CDATA_SECTION_NODE
						_result += child.nodeValue;
				}
			});
		}
		return _result;	//	String
	}
}

dojox.xml.parser.replaceChildren = function(/*Element*/node, /*Node || Array*/ newChildren){
	//	summary:
	//		Removes all children of node and appends newChild. All the existing
	//		children will be destroyed.
	//	description:
	//		Removes all children of node and appends newChild. All the existing
	//		children will be destroyed.
	// 	node:
	//		The node to modify the children on
	//	newChildren:
	//		The children to add to the node.  It can either be a single Node or an
	//		array of Nodes.
	var nodes = [];

	if(dojo.isIE){
		dojo.forEach(node.childNodes, function(child){
			nodes.push(child);
		});
	}

	dojox.xml.parser.removeChildren(node);
	dojo.forEach(nodes, dojo.destroy);

	if(!dojo.isArray(newChildren)){
		node.appendChild(newChildren);
	}else{
		dojo.forEach(newChildren, function(child){
			node.appendChild(child);
		});
	}
}

dojox.xml.parser.removeChildren = function(/*Element*/node){
	//	summary:
	//		removes all children from node and returns the count of children removed.
	//		The children nodes are not destroyed. Be sure to call dojo.destroy on them
	//		after they are not used anymore.
	//	node:
	//		The node to remove all the children from.
	var count = node.childNodes.length;
	while(node.hasChildNodes()){
		node.removeChild(node.firstChild);
	}
	return count; // int
}


dojox.xml.parser.innerXML = function(/*Node*/node){
	//	summary:
	//		Implementation of MS's innerXML function.
	//	node:
	//		The node from which to generate the XML text representation.
	if(node.innerXML){
		return node.innerXML;	//	String
	}else if(node.xml){
		return node.xml;		//	String
	}else if(typeof XMLSerializer != "undefined"){
		return (new XMLSerializer()).serializeToString(node);	//	String
	}
	return null;
}

}

if(!dojo._hasResource["dojox.data.dom"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.data.dom"] = true;
dojo.provide("dojox.data.dom");


//DOM type to int value for reference.
//Ints make for more compact code than full constant names.
//ELEMENT_NODE                  = 1;
//ATTRIBUTE_NODE                = 2;
//TEXT_NODE                     = 3;
//CDATA_SECTION_NODE            = 4;
//ENTITY_REFERENCE_NODE         = 5;
//ENTITY_NODE                   = 6;
//PROCESSING_INSTRUCTION_NODE   = 7;
//COMMENT_NODE                  = 8;
//DOCUMENT_NODE                 = 9;
//DOCUMENT_TYPE_NODE            = 10;
//DOCUMENT_FRAGMENT_NODE        = 11;
//NOTATION_NODE                 = 12;

//This file contains internal/helper APIs as holders for people who used them.  They have been migrated to
//a better project, dojox.xml and experimental has been removed there.  Please update usage to the new package.
dojo.deprecated("dojox.data.dom", "Use dojox.xml.parser instead.", "2.0");

dojox.data.dom.createDocument = function(/*string?*/ str, /*string?*/ mimetype){
	//	summary:
	//		cross-browser implementation of creating an XML document object.
	//
	//	str:
	//		Optional text to create the document from.  If not provided, an empty XML document will be created.  
	//		If str is empty string "", then a new empty document will be created.
	//	mimetype:
	//		Optional mimetype of the text.  Typically, this is text/xml.  Will be defaulted to text/xml if not provided.
	dojo.deprecated("dojox.data.dom.createDocument()", "Use dojox.xml.parser.parse() instead.", "2.0");
	try{
		return dojox.xml.parser.parse(str,mimetype); //DOMDocument.
	}catch(e){
		/*Squeltch errors like the old parser did.*/
		return null;
	}
};

dojox.data.dom.textContent = function(/*Node*/node, /*string?*/text){
	//	summary:
	//		Implementation of the DOM Level 3 attribute; scan node for text
	//	description:
	//		Implementation of the DOM Level 3 attribute; scan node for text
	//		This function can also update the text of a node by replacing all child 
	//		content of the node.
	//	node:
	//		The node to get the text off of or set the text on.
	//	text:
	//		Optional argument of the text to apply to the node.
	dojo.deprecated("dojox.data.dom.textContent()", "Use dojox.xml.parser.textContent() instead.", "2.0");
	if(arguments.length> 1){
		return dojox.xml.parser.textContent(node, text); //string
	}else{
		return dojox.xml.parser.textContent(node); //string
	}
};

dojox.data.dom.replaceChildren = function(/*Element*/node, /*Node || array*/ newChildren){
	//	summary:
	//		Removes all children of node and appends newChild. All the existing
	//		children will be destroyed.
	//	description:
	//		Removes all children of node and appends newChild. All the existing
	//		children will be destroyed.
	// 	node:
	//		The node to modify the children on
	//	newChildren:
	//		The children to add to the node.  It can either be a single Node or an
	//		array of Nodes.
	dojo.deprecated("dojox.data.dom.replaceChildren()", "Use dojox.xml.parser.replaceChildren() instead.", "2.0");
	dojox.xml.parser.replaceChildren(node, newChildren);
};

dojox.data.dom.removeChildren = function(/*Element*/node){
	//	summary:
	//		removes all children from node and returns the count of children removed.
	//		The children nodes are not destroyed. Be sure to call dojo._destroyElement on them
	//		after they are not used anymore.
	//	node:
	//		The node to remove all the children from.
	dojo.deprecated("dojox.data.dom.removeChildren()", "Use dojox.xml.parser.removeChildren() instead.", "2.0");
	return dojox.xml.parser.removeChildren(node); //int
};

dojox.data.dom.innerXML = function(/*Node*/node){
	//	summary:
	//		Implementation of MS's innerXML function.
	//	node:
	//		The node from which to generate the XML text representation.
	dojo.deprecated("dojox.data.dom.innerXML()", "Use dojox.xml.parser.innerXML() instead.", "2.0");
	return dojox.xml.parser.innerXML(node); //string||null
};


}

if(!dojo._hasResource["com.ibm.portal.xslt"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.xslt"] = true;
/* ***************************************************************** */
/*                                                                                                                                    */
/* IBM Confidential                                                                                                         */
/*                                                                                                                                    */
/* OCO Source Materials                                                                                                */
/*                                                                                                                                    */
/* (C) Copyright IBM Corp. 2006,2007                                                                          */
/*                                                                                                                                    */
/* The source code for this program is not published or otherwise                               */
/* divested of its trade secrets, irrespective of what has been                                    */
/* deposited with the U.S. Copyright Office.                                                                 */
/*                                                                                                                                    */
/* ***************************************************************** */
dojo.provide( "com.ibm.portal.xslt" );

// TODO: Not needed anymore here, test before removing
dojo.require( "dojox.data.dom" );

dojo.declare( "com.ibm.portal.xslt.TransformerFactory",
    null,
    {
        constructor: function() {
            // create our internal XSL cache
            this._xsltMap = new Array();
        },
        /**
         * summary:
         *		Creates a new com.ibm.portal.xslt.Transformer object.
         */
        newTransformer: function(/*String*/ xsltUrl) {
             ibm.portal.debug.entry( "newTransformer", [ xsltUrl ] );
             var trf = this._getCached(xsltUrl);
             if(trf == null) {
             	trf = new com.ibm.portal.xslt.Transformer(xsltUrl);
             	this._xsltMap.push({url: xsltUrl, transformer: trf});
             }
             return trf;
        },
        _getCached: function(xsltUrl) {
        	var xslt = null;
             // try to get the XSL object from our internal cache
             for (i=0; i<this._xsltMap.length; i++) {
                 var entry = this._xsltMap[i];
                 if (xsltUrl == entry.url) {
                     // alert("cache hit");
                     xslt = entry.transformer;
                     break;
                 }
             }
             return xslt;
        }
    }
);

dojo.declare( "com.ibm.portal.xslt.Transformer",
    null,
    {
        /**
         * summary:
         *		Creates a new transformer for cross-browser XSLT operations
         */
        constructor: function(/* String */ xsltUrl) {
            this._xslt = com.ibm.portal.xslt.loadXsl(xsltUrl);
        },
        /**
         * summary:
         *		Transforms the given xmlDoc and puts the result into the passed
         *		DOM node
         *
         * xmlDoc:
         * 		XML Document to transform
         * xsltParams:
         *		Set of configuration parameters to pass to the transformation
         *		engine.  Object map in this format: {param1Name: "param1Value", param2Name: "param2Value"}
         * node:
         * 		DOM node to put the transform results in
         * doc:
         * 		HTML document to update
         */ 
        transformToRegion: function(xmlDoc, xsltParams, node, doc) {
			if (dojo.isIE) {
				var resultStr = com.ibm.portal.xslt.transform(xmlDoc, this._xslt, null, xsltParams, true);
				node.innerHTML = dojo.string.trim(resultStr);
			} else {
				/*
				var resultsDoc = this.transformToDocument( xmlDoc, xsltParams);
				dojox.data.dom.replaceChildren(node, resultsDoc.documentElement);
				*/
				var frag = com.ibm.portal.xslt.gecko._transformToFragment(xmlDoc, this._xslt, null, xsltParams, doc);
				node.innerHTML = "";
				node.appendChild(frag);
			}
        },
        /** 
         * summary:
         *		Transforms the given xmlDoc and returns a new XML document or String containing
         *		the result
         * xmlDoc:
         * 		XML Document to transform
         * xsltParams:
         *		Set of configuration parameters to pass to the transformation
         *		engine.  Object map in this format: {param1Name: "param1Value", param2Name: "param2Value"}
         * bReturnAsString:
         * 		Boolean indicating whether a XML document or String should be returned.
         */
        transformToDocument: function(xmlDoc, xsltParams, bReturnAsString) {
             var results = com.ibm.portal.xslt.transform(xmlDoc, this._xslt, null, xsltParams, bReturnAsString);
             return results;
        }
    }
);

// global transformer factory
com.ibm.portal.xslt.TRANSFORMER_FACTORY = new com.ibm.portal.xslt.TransformerFactory();



//---------------------------------------------------------------------- xml related utility methods
/*
 * loadXml(sUrl) - returns oDomDoc. parses xml from the url into DOM document object.
 * loadXmlString() - returns oDomDoc. parses xml from the contents of the string into DOM document object.
 * loadXsl()
 * transform()
 */
com.ibm.portal.xslt.ie = {};
com.ibm.portal.xslt.gecko = {};

com.ibm.portal.xslt.getXmlHttpRequest = function() {
	var oXml = null;
	if (typeof ActiveXObject != "undefined") {
		oXml = new ActiveXObject("Microsoft.XMLHTTP");
	} else {
		oXml = new XMLHttpRequest();
	}
  return oXml;
}

com.ibm.portal.xslt.loadXml = function(sUrl) {
	if (typeof ActiveXObject != "undefined") 
		return com.ibm.portal.xslt.ie.loadXml(sUrl);
	else 
		return com.ibm.portal.xslt.gecko.loadXml(sUrl);
}

com.ibm.portal.xslt.loadXmlString = function(sXml) {
	if (typeof ActiveXObject != "undefined") 
		return com.ibm.portal.xslt.ie.loadXmlString(sXml);
	else 
		return com.ibm.portal.xslt.gecko.loadXmlString(sXml);
}

com.ibm.portal.xslt.loadXsl = function(sUrl) {
	if (typeof ActiveXObject != "undefined") 
		return com.ibm.portal.xslt.ie.loadXsl(sUrl);
	else 
		return com.ibm.portal.xslt.gecko.loadXsl(sUrl);
}

com.ibm.portal.xslt.transform = function (xml, xsl, sXslMode, aXslParams, bReturnString)
{
   ibm.portal.debug.entry( "transform", [xml,xsl,sXslMode,aXslParams,bReturnString] );  
   if (typeof ActiveXObject != "undefined") {
    return com.ibm.portal.xslt.ie.transform(xml, xsl, sXslMode, aXslParams,bReturnString);
  }
  else 
    return com.ibm.portal.xslt.gecko.transform(xml, xsl, sXslMode, aXslParams,bReturnString);
}

com.ibm.portal.xslt.transformAndUpdate = function (/*HTMLElement*/nodeToUpdate, /*XMLDocument*/xml, /*XMLDocument*/xsl, /*String?*/sXslMode, /*Map*/aXslParams) {
	ibm.portal.debug.entry( "transformAndUpdate", [ nodeToUpdate, xml, xsl, sXslMode, aXslParams ]);
	
	if ( typeof ActiveXObject != "undefined" ) {
		var resultStr = com.ibm.portal.xslt.transform(xml, xsl, sXslMode, aXslParams, true);
		nodeToUpdate.innerHTML = dojo.string.trim(resultStr);
	} else {
		var doc = nodeToUpdate.ownerDocument ? nodeToUpdate.ownerDocument : document;
		var frag = com.ibm.portal.xslt.gecko._transformToFragment(xml, xsl, sXslMode, aXslParams, doc);
		nodeToUpdate.innerHTML = "";
		nodeToUpdate.appendChild(frag);
	}
	
	ibm.portal.debug.exit( "transformAndUpdate" );
}

//---------------------------------------------------------------------- IE xml related utility methods

com.ibm.portal.xslt.ie.XSLT_PROG_IDS = [
		"Msxml2.XSLTemplate.6.0",
		"Msxml2.XSLTemplate.4.0",
		"MSXML2.XSLTemplate.3.0",
		"MSXML2.XSLTemplate"
	];
	
com.ibm.portal.xslt.ie.DOM_PROG_IDS = [
		"Msxml2.DOMDocument.6.0",
		"Msxml2.DOMDocument.4.0",
		"MSXML2.DOMDocument",
		"MSXML.DOMDocument",
		"Microsoft.XMLDOM"
	];
	
com.ibm.portal.xslt.ie.FTDOM_PROG_IDS = [
		"Msxml2.FreeThreadedDOMDocument.6.0",
		"Msxml2.FreeThreadedDOMDocument.4.0",
		"MSXML2.FreeThreadedDOMDocument",
		"MSXML.FreeThreadedDOMDocument",
		"Microsoft.FreeThreadedXMLDOM"
	];
	
com.ibm.portal.xslt.ie._getMSXMLImpl = function(array) {
	// This iterates through the passed in array of Strings which represent ProgIDs for
	// MSXML objects, and attempts to find the first available one and returns an instance
	// to the caller
	while(array.length > 0) {
		try {
			var impl = new ActiveXObject(array[0]);
			if(impl) {
				return impl;
			}
		}
		catch(err){}
		array.splice(0,1);
	}
	throw new Error("No MSXML implementation exists");
}

com.ibm.portal.xslt.ie.loadXml = function(sUrl) {
	//var oXmlDoc = new ActiveXObject("MSXML2.DOMDocument");
	var oXmlDoc = this._getMSXMLImpl(this.DOM_PROG_IDS);
	oXmlDoc.async=0;
	oXmlDoc.resolveExternals = 0;
  	if(!oXmlDoc.load(sUrl))
  	{
  		//Callers should catch this and can substitute their own error message
  		throw new Error("Error loading xml file " + sUrl);
  	}
	return oXmlDoc;

}

com.ibm.portal.xslt.ie.loadXmlString = function(sXml) {
	var oXmlDoc = this._getMSXMLImpl(this.DOM_PROG_IDS);
	oXmlDoc.async=0;
	oXmlDoc.resolveExternals = 0;
  	if(!oXmlDoc.loadXML(sXml))
  	{
  	    //Callers should catch this and can substitute their own error message
  		throw new Error("Error loading xml string " + sXml); 
  	}
	return oXmlDoc;
}

com.ibm.portal.xslt.ie.loadXsl = function(sUrl) {
	//we need to use MSXML2.FreeThreadedDOMDocument interface in order to support 
	//mode and parameters in XSL transformation.
	var oXslDoc = this._getMSXMLImpl(this.FTDOM_PROG_IDS);
	oXslDoc.async=0;
	oXslDoc.resolveExternals = 0;
  	if(!oXslDoc.load(sUrl))
  	{
  		//Callers should catch this and can substitute their own error message
  		throw new Error("Error loading xsl file " + sUrl);
  	}	
	return oXslDoc;
}

com.ibm.portal.xslt.ie.transform = function(xmlDoc, xsl, sXslMode, aXslParams,bReturnString) {
	var oXml = xmlDoc;
	var oXsl = xsl;

     try {		
		if(!oXsl.documentElement) oXsl = this.loadXsl(xsl);
	 }
	 catch(e) {
		var sMsg = e.message;
		throw new Error(""+sMsg, ""+sMsg);
	 }
	//create the xsl processor and apply the transformation
	var oXslt = this._getMSXMLImpl(this.XSLT_PROG_IDS);
	oXslt.stylesheet = oXsl;
	var oXslProc = oXslt.createProcessor();
	oXslProc.input = oXml;
	
	//set paramaters if any
	if(aXslParams) {
		for(var p in aXslParams) {
			oXslProc.addParameter(p, aXslParams[p]);
		}
	}
	if (sXslMode) oXslProc.addParameter("mode", sXslMode);
	
	if (bReturnString) {
		if(!oXslProc.transform()) {
			//Callers should catch this and can substitute their own error message
  			throw new Error("Error transforming xml doc " + oXml); 
  		}
  		return oXslProc.output;
	} else {
		var oHtmlDoc = this._getMSXMLImpl(this.DOM_PROG_IDS);
		oHtmlDoc.async = false;
		oHtmlDoc.validateOnParse = false;
		oXml.transformNodeToObject(oXsl,oHtmlDoc);
		return oHtmlDoc;	
	}
}

//---------------------------------------------------------------------- GECKO xml related utility methods

com.ibm.portal.xslt.gecko.loadXml = function(sUrl) {
    //var oXmlResponse = NG.ServerRequest.postRequest(sUrl);
//    if (oXmlResponse) return xmlLoadString(oXmlResponse.responseText);
//    else return null;
	var oDomDoc = null;
	if(dojo.isSafari) {
		var xhr = new XMLHttpRequest();
		xhr.open( "GET", sUrl, false );
		xhr.send( null );
		  
		if ( xhr.status == 200 ) {
			oDomDoc = xhr.responseXML;
		}
	}
	else {
		oDomDoc = document.implementation.createDocument('','',null);
		oDomDoc.async = 0; // this is the important part
		oDomDoc.load(sUrl);
	} 
	return oDomDoc;
}

com.ibm.portal.xslt.gecko.loadXmlString = function(sXml) {
    var parser = new DOMParser();
    try { oXmlDoc = parser.parseFromString(sXml, "text/xml"); }
    catch (exc) {
	    //Callers should catch this and can substitute their own error message
  		throw new Error("Error loading xml string " + sXml); 
    }
	return oXmlDoc;
}

com.ibm.portal.xslt.gecko.loadXsl = function(sUrl) {
	//This is done through createDocument because of anchor(#) we have in portal url. 
	//Do not change the code without testing the case.
	var oDomDoc = null;
	if(dojo.isSafari) {
		var xhr = new XMLHttpRequest();
		xhr.open( "GET", sUrl, false );
		xhr.send( null );
		  
		if ( xhr.status == 200 ) {
			oDomDoc = xhr.responseXML;
		}
	}
	else {
		oDomDoc = document.implementation.createDocument('','',null); 
		oDomDoc.async = 0; // this is the important part
		oDomDoc.load(sUrl);
	}
	
	return oDomDoc;
}

com.ibm.portal.xslt.gecko._getXSLTProc = function(xmlDoc, xsl, sXslMode, aXslParams) {
	var xslDoc = xsl;
	if(!xslDoc.documentElement) 
	{
		xslDoc = this.loadXsl(xsl);
	}        
    var proc = new XSLTProcessor();
    proc.importStylesheet(xslDoc); 

	
	//set parameters if any
	if(aXslParams) {
		for(var p in aXslParams)  {
			proc.setParameter(null, p, aXslParams[p]);
		}
	}
	
	if (sXslMode) proc.setParameter(null, "mode", sXslMode);
	
	return proc;
}

com.ibm.portal.xslt.gecko._transformToFragment = function(xmlDoc, xsl, sXslMode, aXslParams, doc) {
	var proc = com.ibm.portal.xslt.gecko._getXSLTProc(xmlDoc, xsl, sXslMode, aXslParams);
	
	var resultDoc = null;
	resultDoc = proc.transformToFragment(xmlDoc, doc);
	proc.clearParameters();
	return resultDoc;
}

com.ibm.portal.xslt.gecko.transform = function(xmlDoc, xsl, sXslMode, aXslParams,bReturnString) {
	try {
		var resultDoc = null;
		if (!bReturnString) {
			var proc = com.ibm.portal.xslt.gecko._getXSLTProc(xmlDoc, xsl, sXslMode, aXslParams);
			resultDoc = proc.transformToDocument(xmlDoc);
			return resultDoc;
		}
		else {
			resultDoc = com.ibm.portal.xslt.gecko._transformToFragment(xmlDoc, xsl, sXslMode, aXslParams, document);
		}

		var serializer = new XMLSerializer();
		// need to trim newlines and spaces from beginning and end so that the root node always has the same
		// number of child nodes regardless of the browser implementation
		// this should be lossless of real expected content
		var resultStr = dojo.string.trim(serializer.serializeToString(resultDoc));
		if(dojo.isOpera && resultDoc.firstChild && resultDoc.firstChild.nodeName == 'result') {
			var start = resultStr.indexOf("<result>") + 8;
			var end = resultStr.lastIndexOf("</result>");
			resultStr = dojo.string.trim(resultStr.substring(start, end));
		}
		return resultStr;    
    }
    catch (exc)
    {
        //alert("error transforming document: "+exc)
		//Callers should catch this and can substitute their own error message
  		throw new Error("Error transforming xml doc " + exc);
	}
}

/* This method sets the content of a layer within the HTML page
 * to the result of transforming the xml parameter by the xsl
 * parameter. The xml and xsl parameters may be of any form
 * supported by the transformXml() method. The layer parameter
 * may be either a DOM object or the name of a DOM object that
 * can be found using the findObject() method.
 */

com.ibm.portal.xslt.setLayerContentByXml = function (layer, xml, xsl, xslparam,bReturnString) {
    var result = com.ibm.portal.xslt.transform(xml,xsl,null,xslparam,bReturnString);
    if (layer.innerHTML) layer.innerHTML = result;
    else {
        var obj = document.getElementById(layer);
        obj.innerHTML=result;
    }
}

}

if(!dojo._hasResource["com.ibm.portal.services.NavigationModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.services.NavigationModel"] = true;
dojo.provide( "com.ibm.portal.services.NavigationModel" );

dojo.require( "dojo.dnd.common" );

dojo.require( "com.ibm.portal.services.PortalRestServiceRequest" );
dojo.require( "com.ibm.portal.xslt" );


dojo.declare( "com.ibm.portal.services.NavigationNode",
			  null,
			  {
			  	constructor: function ( /*HTMLDivElement*/ div, /*String*/nmPrefix ) {
					if ( !div ) { return null; }
					
					this._nmPrefix = nmPrefix;
					this.URI = div.getAttribute( "id" ).substring( nmPrefix.length );
					this.hasChildren = ( div.getAttribute( 'hasChildren' ) == "true" );
					this.isLabel = ( div.getAttribute( 'contentNodeType' ) == 'LABEL' );
					this.isExternalURL = ( div.getAttribute( 'contentNodeType' ) == 'EXTERNAL-URL' );
					this.isInternalURL = ( div.getAttribute( 'contentNodeType' ) == 'INTERNAL-URL' );
					this.isStaticPage = ( div.getAttribute( 'contentNodeType' ) == 'STATIC-PAGE' );
					this.title = div.getAttribute( 'localizedTitle' );
								
					if ( div.getAttribute( "lmURI" ) ) {
						this.layoutModel = div.getAttribute( "lmURI" );
					}
					
					if ( div.getAttribute( "cmURI" ) ) {
						this.contentModel = div.getAttribute( "cmURI" );
					}
					
					if ( div.getAttribute( "childrenURI" ) ) {
						this.childrenURI = div.getAttribute( 'childrenURI' );
					}
					
					if ( div.getAttribute( "navLevel" ) ) {
						this.navigationLevel = div.getAttribute( "navLevel" );
					}
					
					if ( div.getAttribute( "childOf" ) ) {
						this.childOf = div.getAttribute( "childOf" );
					}
					
					if ( div.getAttribute( "theme" ) ) {
						this.theme = div.getAttribute( "theme" );
					}
					
					if ( div.getAttribute( "themePolicy" ) ) {
						this.themePolicy = div.getAttribute( "themePolicy" ); 
					}
					
					if ( div.getAttribute( "applicationGuid" ) ) {
						this.applicationGuid = div.getAttribute( "applicationGuid" );
					}
					
					this._children = null;
			    },
			  	_deserialize: function () {
					var div = document.createElement( "DIV" );
					div.setAttribute( "id" , this._nmPrefix + this.URI );
					div.setAttribute( "hasChildren", this.hasChildren.toString() );
					div.setAttribute( "localizedTitle", this.title );
					
					if ( this.isLabel ) { div.setAttribute( "contentNodeType", "LABEL" ); }
					if ( this.isExternalURL ) { div.setAttribute( "contentNodeType", "EXTERNAL-URL" ); }
					if ( this.isInternalURL ) { div.setAttribute( "contentNodeType", "INTERNAL-URL" ); }
					if ( this.isStaticPage ) { div.setAttribute( "contentNodeType", "STATIC-PAGE" ); }
					if ( !this.isLabel && !this.isExternalURL && !this.isInternalURL && !this.isStaticPage ) { div.setAttribute( "contentNodeType", "COMPOSITION" ); }
					
					if ( this.layoutModel ) { div.setAttribute( "lmURI", this.layoutModel ); }
					if ( this.contentModel ) { div.setAttribute( "cmURI", this.contentModel ); }
					if ( this.childrenURI ) { div.setAttribute( "childrenURI", this.childrenURI ); }
					if ( this.navigationLevel ) { div.setAttribute( "navLevel", this.navigationLevel ); }
					if ( this.childOf ) { div.setAttribute( "childOf", this.childOf ); }
					if ( this.theme ) { div.setAttribute( "theme", this.theme ); }
					if ( this.themePolicy ) { div.setAttribute( "themePolicy", this.themePolicy ); }
					if ( this.applicationGuid ) { div.setAttribute( "applicationGuid", this.applicationGuid ); }
					
					return div;
				},
				getLayoutModel: function ( /*Function*/callbackFn, /*Array?*/fnArguments ) {
					// summary: Retrieve the layout model feed for this navigation node.
					// description: Retrieves the layout model feed for this navigation node
					//		asynchronously. The provided callback function will be called
					//		with the 3 standard dojo.io.bind arguments as well as any of the
					//		fnArguments passed in to this function.
					// callbackFn: the function to call when the layout model is loaded
					// fnArguments: any arguments to pass through to the callback function
					ibm.portal.debug.entry( "NavigationNode.getLayoutModel", [ callbackFn, fnArguments ] );
					var url = new com.ibm.portal.services.ContentHandlerURL( this.layoutModel, null, null, "&mdname=WIDTH" );
					var serviceReq = new com.ibm.portal.services.PortalRestServiceRequest( url );
					serviceReq.read( callbackFn, fnArguments );
					ibm.portal.debug.exit( "NavigationNode.getLayoutModel" );
				},
				getChildren: function ( /*Function?*/callbackFn ) {
					// summary: Retrieve the children of this node.
					// callbackFn: the function to call when the retrieval process is finished. if
					//		null, the retrieval is done synchronously
					// returns: Array of the children nodes or NULL if callbackFn not specified
					ibm.portal.debug.entry( "NavigationNode.getChildren", [ callbackFn ]);
					if ( this._children != null ) {
						return this._children;
					}
					ibm.portal.debug.text( "Children have not been previously requested." );
					this._children = new Array();
					var div = dojo.byId( this._nmPrefix + this.URI );
					
					ibm.portal.debug.text( "Found associated div element: " + div );
					
					//Unless the _children array is populated, we can't guarantee all the child nodes have been
					//populated. One or more nodes may have been loaded by the NavigationSelectionModel creation as
					//part of a "-all" request. Checking if the div has children nodes is not sufficient before 
					//sending this request. We may end up with some duplication as a result, but this is, unfortunately,
					//the only way to guarantee all of the children have been loaded.
					ibm.portal.debug.text( "Fetching children nodes: " + this.childrenURI );
					com.ibm.portal.services.NAVIGATION_MODEL.getNode( this.childrenURI );
					
					div = dojo.byId( this._nmPrefix + this.URI );
					ibm.portal.debug.text( "Div now has " + div.childNodes.length + " elements. " );
					
					for ( var i = 0; i < div.childNodes.length; i++ ) {
						ibm.portal.debug.text( "Adding child node: " + div.childNodes[i] );
						this._children.push( new com.ibm.portal.services.NavigationNode( div.childNodes[i], this._nmPrefix ) )	
					}
					
					ibm.portal.debug.exit( "NavigationNode.getChildren", [ this._children ] );
					
					return this._children;	
				},
				removeChild: function ( /*String*/ childId ) {
					// summary: Removes a child of this navigation node.
					// childId: The id of the child to remove
					ibm.portal.debug.entry( "NavigationNode.removeChild", [ childId ]);
					if( this._children ) {
						for(var i = 0; i < this._children.length; i++) {
							if(childId == this._children[i].URI) {
								this._children.splice(i, 1);
								break;
							}
						}
					}
					var thisNodeElem = dojo.byId(this._nmPrefix + this.URI);
					if(thisNodeElem) {
						var childNodeElem = dojo.byId(this._nmPrefix + childId);
						if(childNodeElem && childNodeElem.parentNode == thisNodeElem) {
							thisNodeElem.removeChild(childNodeElem);
							delete childNodeElem;
						}
					}
					
					ibm.portal.debug.exit( "NavigationNode.removeChild", [ childId ]);
				},
				getContentURL: function() {
					// summary: Retrieve the external URL associated with this NavigationNode if it's node type is EXTERNAL-URL.
					// description: Retrieves the external URL associated with this NavigationNode. If the node type is not EXTERNAL-URL,
					//		then return null. If a callback is provided, then the function is called with the url string as it's lone parameter. If
					//		no callback is provided, the function is executed synchronously, otherwise it is asynchronous.
					// callbackfn: *OPTIONAL* callback function when the url is retrieved, takes the url string as the only parameter
					ibm.portal.debug.entry( "NavigationNode.getExternalURL" );
					ibm.portal.debug.exit( "NavigationNode.getExternalURL", [ this.contentModel ] );
					return this.contentModel;
				}
			  }
);

dojo.declare( "com.ibm.portal.services.NavigationModel",
			  null,
			  {
			  	constructor: function () {
					this._containerCreated = false;
					this._nodeContainer = "";  
					this._xsltURL = dojo.moduleUrl( "com", "ibm/portal/services/");
					this._uniqueNamesMap = {};
				},
			  	_createRootDiv: function () {
					//create the container div for all of model information		  
				  	var div = document.createElement( "DIV" );
					div.style.display = "none";
					div.setAttribute( "id", dojo.dnd.getUniqueId() ); 
					dojo.body().appendChild( div );
					this._nodeContainer = div.getAttribute( "id" );
					this._containerCreated = true;	
				},
				_newContentHandlerURL: function ( /*String*/uri, /*int or String*/levels ) {
					return new com.ibm.portal.services.ContentHandlerURL( uri, levels, "download", "&mdname=RenderMode&mdname=com.ibm.portal.ThemePolicy&mdname=com.ibm.portal.applicationGuid" );
				},
				namespaces: {
		            "xsl" : "http://www.w3.org/1999/XSL/Transform",
		            "thr" : "http://purl.org/syndication/thread/1.0",
		            "atom" : "http://www.w3.org/2005/Atom",
		            "xhtml" : "http://www.w3.org/1999/xhtml",
		            "model" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0.1/portal-model-elements",
		            "base" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0/ibm-portal-composite-base",
		            "portal" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0.1/portal-model",
		            "xsi" : "http://www.w3.org/2001/XMLSchema-instance"
		        },
		        getCompleteAncestorPath: function ( /*String*/uri ) {
		        	// summary: Retrieve all ancestor nodes including the given uri.
		        	// description: Retrieves the given node and all the parent nodes
		        	//		associated with that node.
		        	// uri: the unique identifier of the node
					ibm.portal.debug.entry( "NavigationModel.getCompleteAncestorPath", [ uri ] );
					
					if ( !this._containerCreated ) {
						this._createRootDiv();
					}
					
					var node = this._getCachedNode( uri );
		        	var returnArray = new Array(), done = false;
					
					if ( node == null ) {
						//Retrieve the parent node up to the content root and merge it into our local cache
		        				var url = this._newContentHandlerURL( uri, "-all" );
					        	var serviceReq = new com.ibm.portal.services.PortalRestServiceRequest( url, null, false, true );
								var me = this;
								
								serviceReq.read( 
									function ( type, data, xhr, args ) {
										
										var results = me._doSelModelTransform( data, uri );
										var tempNode = document.createElement('div');
										tempNode.innerHTML = results;
										var reversePath = new Array();
										var currentNode = tempNode.firstChild.firstChild;
										var containerDiv = dojo.byId( me._nodeContainer );
										while ( currentNode ) {
											//reversePath.push( new com.ibm.portal.services.NavigationNode( currentNode, me._nodeContainer ) );
											var newDiv = new com.ibm.portal.services.NavigationNode( currentNode, me._nodeContainer )._deserialize();
											containerDiv.appendChild( newDiv );
											containerDiv = newDiv;
											currentNode = currentNode.childNodes[0];
										}
									}
								);
					}
					
					node = this._getCachedNode( uri );
		        	
		        	while ( node != null && !done ) {
		        		returnArray.push( node );
		        		if ( node.childOf ) {
		        			//Check to see if we have retrieved the parent node.
		        			var nextNode = this._getCachedNode( node.childOf );
		        			if ( nextNode != null ) {
		        				node = nextNode;
		        			}
		        			else {
		        				//Retrieve the parent node up to the content root and merge it into our local cache
		        				var url = this._newContentHandlerURL( node.childOf, "-all" );
					        	var serviceReq = new com.ibm.portal.services.PortalRestServiceRequest( url, null, false, true );
								var me = this;
								
								serviceReq.read( 
									function ( type, data, xhr, args ) {
										var results = me._doSelModelTransform( data, node.childOf );
										var reversePath = new Array();
										var tempNode = document.createElement('div');
										tempNode.innerHTML = results;
										var currentNode = tempNode.firstChild.firstChild;
										while ( currentNode ) {
											reversePath.push( new com.ibm.portal.services.NavigationNode( currentNode, me._nodeContainer ) );
											currentNode = currentNode.childNodes[0];
										}
										
										var prevNode = node.childOf;
										for ( var i = (reversePath.length - 1); i >= 0; i-- ) {
											var containerDiv = dojo.byId( me._nodeContainer );
											var newDiv = reversePath[ i ];
											
											var placeHolderID = "placeholder_" + me._nodeContainer + prevNode;
											var placeHolderDiv = dojo.byId( placeHolderID );
											
											if ( placeHolderDiv ) {
												containerDiv.appendChild( newDiv._deserialize() );
												var div1 = dojo.byId( me._nodeContainer + prevNode );
												div1.innerHTML = placeHolderDiv.innerHTML;										
												//dojo.dom.removeNode( placeHolderDiv );
												placeHolderDiv.parentNode.removeChild(placeHolderDiv);
											}
											else {
												var prevNodeID = me._nodeContainer + prevNode;
												var prevNodeDiv = dojo.byId( prevNodeID );
												if ( prevNodeDiv ) {
													var div = newDiv._deserialize();
													div.innerHTML = prevNodeDiv.parentNode.innerHTML;
													containerDiv.appendChild( div );
													
													//dojo.dom.removeNode( prevNodeDiv );
													prevNodeDiv.parentNode.removeChild(prevNodeDiv);
												}
											}	
										
										}
									}
								);
								
								node = this._getCachedNode( node.childOf );;
		        			}
		        		}else {
		        			done = true;
		        		}
		        				
		        	}
		        	
					returnArray = returnArray.reverse();
		        	
		        	ibm.portal.debug.exit( "NavigationModel.getCompleteAncestorPath",[returnArray] );
		        	return returnArray;
		        },
			  _doSelModelTransform: function ( /*XMLDocument*/xml, /*String*/rootNode ) {
					ibm.portal.debug.entry( "NavigationModel._doSelModelTransform");
                        var trf = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer( this._xsltURL + "NavigationSelectionModel.xsl" );
                        if (trf == null) {
                            ibm.portal.debug.text("XSL transformer is null");
                        }
					var results = trf.transformToDocument( xml, {"nmPrefix":this._nodeContainer,"rootNode":rootNode}, true );
		        	this._doUniqueNamesMapping( xml );
					ibm.portal.debug.exit( "NavigationModel._doSelModelTransform", results );
		        	return results;
		        },
		        _doUniqueNamesMapping: function( /*XMLDocument*/xml ) {
                        var trf = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer( this._xsltURL + "NMUniqueNamesMapping.xsl" );
                        if (trf == null) {
                            ibm.portal.debug.text("XSL transformer is null");
                        }
                        var results = trf.transformToDocument( xml, {}, true );
                        var array = eval( results );
                        for (var i = 0; i < array.length; i++ ) {
                            this._uniqueNamesMap[array[i].uniqueName] = array[i].objectID;
                        }	
		        },
		        _getCachedNode: function ( /*String*/ uri ) {
		        	// summary: Retrieves the requested node from the "cached" location, if it exists there.
		        	// uri: the node to lookup
		        	// returns: the requested node or NULL if not found in the "cache"
		        	ibm.portal.debug.entry( "NavigationModel._getCachedNode", [ uri ] );
		        	
		        	var retVal = null;
		        	retVal = dojo.byId( this._nodeContainer + uri ); 
		        	
		        	//try by unique name
		        	if ( retVal == null ) {
		        		var uniqueName = uri;
		        		if ( uri.indexOf("cm:oid:") == 0 ) {
		        			uniqueName = uri.substring(7);
		        			uniqueName = uniqueName.replace( /%252520/g, ' ' );
		        		} 
		        		retVal = dojo.byId( this._nodeContainer + this._uniqueNamesMap[ uniqueName ] );
		        	}
		        	
		        	if ( retVal != null ) {
		        		retVal = new com.ibm.portal.services.NavigationNode( retVal, this._nodeContainer );
		        	}
		        	
		        	ibm.portal.debug.exit( "NavigationModel._getCachedNode", [ retVal ] );
		        	return retVal; //com.ibm.portal.services.NavigationNode
		        },
		        _getObjectIDFromUniqueName: function ( /*XMLDocument*/ xmlDoc, /*String*/requestedURI ) {
		        	ibm.portal.debug.entry( "_getObjectIDFromUniqueName", [ xmlDoc, requestedURI ] );
		        	
		        	requestedURI = requestedURI.substring(7);
		        	//requestedURI = requestedURI.replace( /%252520/g, ' ' );
		        	var atomIdXPath = "/atom:feed/atom:entry/atom:id[@portal:uniquename='"+requestedURI+"']";
					var nodes = com.ibm.portal.xpath.evaluateXPath( atomIdXPath, xmlDoc, this.namespaces );
					var result = null;
					
					if ( nodes != null && nodes.length > 0 ) {
						result = dojox.data.dom.textContent( nodes[0] );
					}
					
					ibm.portal.debug.exit( "_getObjectIDFromUniqueName", [ result ] );
					
					return result;
		        },
				getNode: function ( /*String*/ uri, /*Boolean?*/async, /*Function?*/ callbackfn ) {
					// summary: Retrieves the NavigationNode associated with
					//		the given uri.
					// description: Looks in memory for the requested NavigationNode. If
					//		it has not been loaded yet, uses a synchronous request to load
					//		the requested uri and it's children (if they exist). Returns
					//		null if the requested node cannot be found.
					// uri: the unique identifier of the node
					// async: indicates whether retrieving the node should be a blocking operation
					//		the default value is false
					// callbackfn: function to call when node is loaded - only valid if "async" is 
					//		true 
					// returns: NavigationNode or null if not found
					ibm.portal.debug.entry( "NavigationModel.getNode", [ uri, async, callbackfn ] );
					
					if ( !this._containerCreated ) {
						this._createRootDiv();
					}
					
					var retVal = this._getCachedNode( uri );
					
					ibm.portal.debug.text( "result of dom lookup: " + retVal );
										
					if ( !retVal ) {
						if ( !( uri.indexOf("cm:oid:") == 0 ) && !( uri.indexOf( "?uri=" ) == 0  )) {
							uri = "cm:oid:" + uri;
						}
					
						//not loaded yet, load from the REST service
						var url = this._newContentHandlerURL( uri, 2 );
						var serviceReq = new com.ibm.portal.services.PortalRestServiceRequest( url, null, false, true );
						var me = this;
						serviceReq.read( 
							function ( type, data, xhr, args ) {
								ibm.portal.debug.entry( "NavigationModel.onNodeLoad", [type, data, xhr ] );
								
								var url = new com.ibm.portal.services.ContentHandlerURL( args[ "requestedURI" ] );
								var requestedURI = me._getObjectIDFromUniqueName( data, url.getURI() );
								var isUniqueName = true;
								if ( requestedURI == null ) { requestedURI = url.getURI(); isUniqueName = false; } else { uri = requestedURI }
								var results = me._doXSLTransformation( data, requestedURI, isUniqueName );
								ibm.portal.debug.text( "xslt result: " + dojox.data.dom.innerXML( results ) );
								
								//var requestedNodeDiv = dojo.byId( me._nodeContainer + url.getURI(), results );
								var requestedNodeDiv = me._getElementToAdd( me._nodeContainer + requestedURI, results );
								var requestedNode = new com.ibm.portal.services.NavigationNode( requestedNodeDiv, me._nodeContainer );
								
								ibm.portal.debug.text( "found requested node in results: " + requestedNode.URI );
								
								var nodeToUpdate = me._getNodeToUpdate( requestedNode );
								
								//If the requested node already exists in our node container, we want to replace it with our new retrieval.
								var requestedNodeInContainer = dojo.byId( me._nodeContainer + url.getURI() );
								ibm.portal.debug.text( "The requested node is already in the container? " + requestedNodeInContainer );
								
								if ( requestedNodeInContainer ) {
									//dojox.data.dom.replaceNode( requestedNodeInContainer, requestedNodeDiv );
									requestedNodeInContainer.parentNode.replaceChild(requestedNodeDiv,requestedNodeInContainer);
								} 
								else {
									ibm.portal.debug.text( "Appending the new div to our container." );
									//nodeToUpdate.appendChild( requestedNodeDiv );
									nodeToUpdate.appendChild( requestedNodeDiv );	
								}
								//If a placeholder node exists for the requested node, we must copy the placeholder children
								//over to our new requested node
								var placeholder = dojo.byId( "placeholder_" + me._nodeContainer + requestedNode.URI );
								
								ibm.portal.debug.text( "Found placeholder div: " + placeholder );
								
								if ( placeholder ) {
									//Need to replace all existing nodes with the matching nodes copied over
									//from the placeholder node. This is because if a placeholder exists, we
									//have already retrieved some portion of the navigation tree.
									var array = placeholder.childNodes;
									for ( var i = 0; i < array.length; i++ ) {
										var node = array[i].parentNode.removeChild(array[i]);
										var existingNode = dojo.byId( node.getAttribute( "id" ) );
										//dojo.dom.replaceNode( existingNode, node ); 
										existingNode.parentNode.replaceChild(node, existingNode);
									}
									//dojo.dom.removeNode( placeholder );	
									placeholder.parentNode.removeChild(placeholder);
								}
																
								if ( callbackfn ) {
									callbackfn();
								}
								
								ibm.portal.debug.exit( "NavigationModel.onNodeLoad" );
							},
							{ "requestedURI": uri }
						);
					}
					
					//If this is an asynchronous call, don't wait around for the return value.
					if ( async ) {
						ibm.portal.debug.exit( "NavigationModel.getNode" );
						return;
					}
					
					//Try again, now that we've attempted to retrieve the node.
					retVal = this._getCachedNode( uri );
					
					ibm.portal.debug.exit( "NavigationModel.getNode", [ retVal ]);
					
					return retVal; //com.ibm.portal.services.NavigationNode
				},
				_doXSLTransformation: function ( /*XMLDocument*/xmlDoc, /*String*/requestedURI, /*boolean*/isUniqueName ) {
					ibm.portal.debug.entry( "NavigationModel._doXSLTransformation", [ xmlDoc, requestedURI, isUniqueName ] );
					var trf = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer( this._xsltURL + "NavigationModel.xsl" );
					if (trf == null) {
						ibm.portal.debug.text("XSL transformer is null");
					}
					
					var tempNode = document.createElement('div');
					var xsltParams = {"nmPrefix":this._nodeContainer,"rootNode":requestedURI};
					// determine if we retrieved by unique name or by object id.
					if( isUniqueName ) {
						xsltParams["byUniqueName"] = "true";
					}
					
					trf.transformToRegion( xmlDoc, xsltParams, tempNode, document );
					
					var results = this._convertTransformedDoc( tempNode.firstChild );
					this._doUniqueNamesMapping( xmlDoc );
					ibm.portal.debug.exit( "NavigationModel._doXSLTransformation", [ results ] );
					return results;
				},
				_convertTransformedDoc: function ( /*HTMLElement*/root ) {
					var rootDiv = document.createElement( "div" );
					ibm.portal.debug.text("created root div");
					this._populateNewDoc( root, rootDiv );
					ibm.portal.debug.text( dojox.data.dom.innerXML( rootDiv ) );					
					return rootDiv;	
				},
				_populateNewDoc: function ( /*Element*/root, /*Node*/parentNode ) {
					for ( var i = 0; i < root.childNodes.length; i++ ) {
						var div = document.createElement( "div" );
						ibm.portal.debug.text( "creating div for " + root.childNodes[i].getAttribute("id") );
						div.setAttribute( "id", root.childNodes[i].getAttribute("id") );
						ibm.portal.debug.text( "set id attribute: " + div.getAttribute("id") );
						div.setAttribute( "hasChildren", root.childNodes[i].getAttribute("hasChildren"));
						ibm.portal.debug.text( "set hasChildren attribute: " + div.getAttribute("hasChildren") );
						div.setAttribute( "contentNodeType", root.childNodes[i].getAttribute("contentNodeType"));
						ibm.portal.debug.text( "set nodeType attribute: " + div.getAttribute("contentNodeType") );
						div.setAttribute( "localizedTitle", root.childNodes[i].getAttribute( "localizedTitle" ) );
						if ( root.childNodes[i].getAttribute( "lmURI" ) ) {
							div.setAttribute( "lmURI", root.childNodes[i].getAttribute("lmURI") );
						}
						
						if ( root.childNodes[i].getAttribute( "cmURI" ) ) {
							div.setAttribute( "cmURI", root.childNodes[i].getAttribute( "cmURI" ) );
						}
						
						if ( root.childNodes[i].getAttribute( "childrenURI" ) ) {
							div.setAttribute( "childrenURI", root.childNodes[i].getAttribute("childrenURI") );
						}
						
						if ( root.childNodes[i].getAttribute( "navLevel" ) ) {
							div.setAttribute( "navLevel", root.childNodes[i].getAttribute("navLevel") );
						}
						
						if ( root.childNodes[i].getAttribute( "childOf" ) ) {
							div.setAttribute( "childOf", root.childNodes[i].getAttribute("childOf") );
						}
						if ( root.childNodes[i].getAttribute( "theme" ) ) {
							div.setAttribute( "theme", root.childNodes[i].getAttribute("theme") );
						}
						if ( root.childNodes[i].getAttribute( "themePolicy" ) ) {
							div.setAttribute( "themePolicy", root.childNodes[i].getAttribute("themePolicy") );
						}
						if ( root.childNodes[i].getAttribute( "applicationGuid" ) ) {
							div.setAttribute( "applicationGuid", root.childNodes[i].getAttribute("applicationGuid") );
						}
						parentNode.appendChild( div );
						this._populateNewDoc( root.childNodes[i], div );
					}	
				},
				_getNodeToUpdate: function ( /*NavigationNode*/requestedNode ) {
					ibm.portal.debug.entry( "NavigationModel._getNodeToUpdate", [requestedNode] );
					var nodeToUpdate = dojo.byId( this._nodeContainer );
					
					var parentNode = dojo.byId( this._nodeContainer + requestedNode.childOf );
								
					//If so, we update the parent node.
					if ( parentNode ) {
						ibm.portal.debug.text( "found parent node: " + parentNode.id );
						nodeToUpdate = parentNode;
					}
					else {
						//If a placeholder exists for the parent node, we copy the children there.
						var placeHolderID = "placeholder_" + this._nodeContainer + requestedNode.childOf;
						var parentPlaceholder = dojo.byId( placeHolderID );
						if ( parentPlaceholder ) {
							ibm.portal.debug.text( "parent placeholder found: " + parentPlaceholder );
							nodeToUpdate = parentPlaceholder;
						}
						else if ( requestedNode.childOf ){
							var div = document.createElement( "DIV" );
							div.setAttribute( "id", placeHolderID);
							div.style.display = "none";
							dojo.byId( this._nodeContainer ).appendChild( div );
							nodeToUpdate = div;
						}
					}
					ibm.portal.debug.exit( "NavigationModel._getNodeToUpdate", [nodeToUpdate.getAttribute("id")] );
					return nodeToUpdate;
				},
				_getElementToAdd: function ( /*String*/id, /*XMLDocument*/doc ) {
					ibm.portal.debug.entry( "NavigationModel._getElementToAdd", [id, doc] );
					var retVal = null;
					
					//Note: Since moving to the dojo xslt impl, the dojo.byId query doesn't seem to work anymore
					//if ( dojo.isIE ) {
						var divs = doc.getElementsByTagName( "div" );
						var found = false, i = 0;
						
						while ( i < divs.length && !found ) {
							ibm.portal.debug.text( "Examining div: " + divs[i] + " with id: " + divs[i].getAttribute("id") );
							found = ( id == divs[i].getAttribute( "id" ) );
							i++;	
						}
						
						if ( found ) {
							retVal = divs[i - 1]; //i was incremented once after we found our div
						}		
					/*}
					else {
						retVal = dojo.byId( id, doc );
					}*/
					
					ibm.portal.debug.exit( "NavigationModel._getElementToAdd", [ retVal ] );
					return retVal;
				}	
			  }  
);

//Static helper variable
com.ibm.portal.services.NAVIGATION_MODEL = new com.ibm.portal.services.NavigationModel();

}

if(!dojo._hasResource["com.ibm.portal.services.LayoutModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.services.LayoutModel"] = true;
dojo.provide( "com.ibm.portal.services.LayoutModel" );

dojo.declare( "com.ibm.portal.services.LayoutModel", 
			  null,
			  {
			  	// summary: Convenience object to contain the namespaces for this feed.
				// description: Currently only holds the namespaces for the layout model feed.
				//		The layout model should be retrieved via the NavigationNode object 
				//		obtained from the NavigationModel.
				
				namespaces: {
		            "xsl" : "http://www.w3.org/1999/XSL/Transform",
		            "thr" : "http://purl.org/syndication/thread/1.0",
		            "atom" : "http://www.w3.org/2005/Atom",
		            "xhtml" : "http://www.w3.org/1999/xhtml",
		            "model" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0.1/portal-model-elements",
		            "base" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0/ibm-portal-composite-base",
		            "portal" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0.1/portal-model",
		            "xsi" : "http://www.w3.org/2001/XMLSchema-instance"
		        }
			  }
);

com.ibm.portal.services.LAYOUT_MODEL = new com.ibm.portal.services.LayoutModel();

}

if(!dojo._hasResource["com.ibm.portal.services.NavigationSelectionModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.services.NavigationSelectionModel"] = true;
dojo.provide("com.ibm.portal.services.NavigationSelectionModel");

dojo.require( "com.ibm.portal.services.NavigationModel" );

dojo.declare( "com.ibm.portal.services.NavigationSelectionModel",
			  null,
			  {
			  	constructor: function ( /*String*/pageID ) {
				  	ibm.portal.debug.entry( "NavigationSelectionModel.constructor", [ pageID ] );
				  
				  	this.selectionPath = new Array();
					this._populateParentNodes( pageID );

                    if(this.selectionPath.length<1) {
					    ibm.portal.debug.text( "ctr:this.selectionPath < 1: Wait for page refresh"); 	
                        return this.selectionPath;
                    }                    

					var navNode = this.selectionPath[ this.selectionPath.length - 1 ];
					//var navNode = com.ibm.portal.services.NAVIGATION_MODEL.getNode( pageID );
					var doValidation = navNode.isLabel;
					//Populate any parent nodes if they exist.
					
					//Append this node.
					//this.selectionPath.push( navNode );
					//If the given pageID points to a label, we have
					//to find the actual selected page.
					if ( navNode.isLabel && navNode.hasChildren ) {
						var calcSel = this._calculateSelectedPage( navNode );
						var selected = calcSel.found ? calcSel.found : calcSel.maybe;
						if(selected != null) {
							var selectedNode = com.ibm.portal.services.NAVIGATION_MODEL.getNode( selected );
							var addToPath = new Array ();
							
							if ( selected ) {
								var lastNodeURI = this.selectionPath[ this.selectionPath.length - 1 ].URI;
								ibm.portal.debug.text( "Last entry in selection path: " + lastNodeURI );
								
								var parentURI = selectedNode.childOf;
								var done = ( lastNodeURI == parentURI );
								//We know what the selected node is now. Now we have to walk back up from the
								//selected node and add any nodes that are in between it and the last of the
								//parent nodes.
								while ( !done ) {
									ibm.portal.debug.text( "parentURI = " + parentURI );
									var node = com.ibm.portal.services.NAVIGATION_MODEL.getNode( parentURI );
									addToPath.push( node );
									parentURI = node.childOf;
									ibm.portal.debug.text( "lastNodeURI is " + lastNodeURI + "; parentURI is " + parentURI + "; equal? " + ( lastNodeURI == parentURI ) );
									done = ( lastNodeURI == parentURI );	
								}
								
								var toAdd = addToPath.pop();
							
								while ( toAdd ) {
									this.selectionPath.push( toAdd );
									toAdd = addToPath.pop();
								}
								
								//Only perform validation if the node originally passed into this function is
								//a label.
								if ( doValidation ) {
									var validatedSelection = this._validateSelection( selected );
									
									if ( validatedSelection != selected ) {
										ibm.portal.debug.text( "Selection did not validate! Replacing with the validated selection id." );
										//selected = validatedSelection;
										selectedNode = com.ibm.portal.services.NAVIGATION_MODEL.getNode( validatedSelection );
									}
								}	 
								
								//Finally add the calculated selection.
								this.selectionPath.push( selectedNode );	
							}
						}
					}
					
					ibm.portal.debug.exit( "NavigationSelectionModel.constructor", [this.selectionPath] );				
			    },
			  	getSelectedNode: function () {
					// summary: Get the selected navigation node.
					// returns: the selected NavigationNode
					return this.selectionPath[ this.selectionPath.length - 1 ]; //NavigationNode
				},
				getInheritedProperty: function ( /*String*/property, /*String*/defaultValue ) {
					// summary: Certain properties are inherited down the content/navigation model. This function
					//		walks up the selection path looking for the most recent definition for the given
					//		property. A default value may be specified to return if no explicit value is discovered
					//		during the search.
					// property: the property to search for.
					// defaultValue: the value to return if the property is not found
					ibm.portal.debug.entry( "NavigationSelectionModel.getInheritedProperty", [ property, defaultValue ] );
					
					var retVal = defaultValue;
					var i = this.selectionPath.length - 1;
					var done = false;
					while ( !done && i >= 0 ) {
						var node = this.selectionPath[i];
						if ( node[property] != null ) {
							done = true;
							retVal = node[property];
							ibm.portal.debug.text( "Property \"" + retVal + "\" found for page: " + node.title );
						}
						i--;
					}
					
					ibm.portal.debug.exit( "NavigationSelectionModel.getInheritedProperty", retVal );
					return retVal;	
				},
				getSelectedNodeTheme: function () { 
					// summary: Get the selected node's theme. If no theme is specified on the selection path, the default
					//		theme is assumed.
					// returns: the selected node's theme
					ibm.portal.debug.entry( "NavigationSelectionModel.getSelectedNodeTheme" );
					
					var theme = this.getInheritedProperty( "theme", "default" );

					ibm.portal.debug.exit( "NavigationSelectionModel.getSelectedNodeTheme", [ theme ]);
					return theme;
				},
				getSelectedNodeThemePolicy: function () { 
					// summary: Get the selected node's theme policy. If no theme policy is specified on the selection path, the default
					//		theme policy is assumed.
					// returns: the selected node's theme policy
					ibm.portal.debug.entry( "NavigationSelectionModel.getSelectedNodeThemePolicy" );
					var themePolicy = this.getInheritedProperty( "themePolicy", "default" );
					ibm.portal.debug.exit( "NavigationSelectionModel.getSelectedNodeThemePolicy", [ themePolicy ]);
					return themePolicy;
				},
				getSelectedNodeApplicationGuid: function () {
					// summary: Get the selected node's application guid if it has one. If no application guid is specified on the selection path, NULL
					//		is returned
					// returns: the selected node's theme
					ibm.portal.debug.entry( "NavigationSelectionModel.getSelectedNodeApplicationGuid" );
					var appGuid = this.getInheritedProperty( "applicationGuid", null );
					ibm.portal.debug.exit( "NavigationSelectionModel.getSelectedNodeApplicationGuid", [ appGuid ]);
					return appGuid;	
				},
				isNodeInSelectionPath: function ( /*String*/oid ) {
					ibm.portal.debug.entry( "NavigationSelectionModel.isNodeInSelectionPath", [ oid ] );
					
					var i = 0;
					var found = false;
					
					while ( !found && i < this.selectionPath.length ) {
						
						//console.log( "i = " + i );
						found = (this.selectionPath[i].URI == oid);
						i = i+1;
					}
					
					ibm.portal.debug.exit( "NavigationSelectionModel.isNodeInSelectionPath", found );
					return found;
				},
				_populateParentNodes: function ( /*String*/ uri ) {
					// summary: Populates the parent nodes of the given node.
					// description: Populates the selection path with the parent
					//		nodes of the given node all the way to the root node.
					// navNode: the selected node.
					ibm.portal.debug.entry( "NavigationSelectionModel._populateParentNodes", [ uri ] );
					
					/*var parentURI = navNode.childOf;
					var parentChain = new Array();
					
					while ( parentURI ) {
						//Add the parent node to the end of the chain.
						parentChain.push( parentURI );
						//Retrieve the information about the parent node.
						var parentNode = com.ibm.portal.services.NAVIGATION_MODEL.getNode( parentURI );
						parentURI = parentNode.childOf;
					}*/
					var parentChain = com.ibm.portal.services.NAVIGATION_MODEL.getCompleteAncestorPath( uri );
					
					//We have to reverse the order in which we added the nodes.
					//We can't add to the front of the array using unshift() because
					//it apparently doesn't work properly in IE so we reverse the order
					//in which the parent nodes were added this way.
					//var toAdd = parentChain.pop();
					
					//while ( toAdd ) {
					//	this.selectionPath.push( toAdd );
					//	toAdd = parentChain.pop();
					//}
					this.selectionPath = parentChain;
					
					
					
					ibm.portal.debug.exit( "NavigationSelectionModel._populateParentNodes" );
				},
				
				_calculateSelectedPageDefault: function( /*NavigationNode*/ navNode ) {
					ibm.portal.debug.entry( "NavigationSelectionModel._calculateSelectedPageDefault", [ navNode ] );
					var children = navNode.getChildren();
					var found = null, maybe = null;
					var i = 0;
					
					while ( i < children.length && !found ) {
						if ( !children[i].isLabel ) {
							maybe = children[i].URI;
							if(!children[i].isExternalURL && !children[i].isInternalURL) {
								found = maybe;
							}
						}
						i++;
					}
					
					var j = 0;
					var ret = null;
					while ( j < children.length && !found ) {
						if(children[j].hasChildren) {
							ret = this._calculateSelectedPage( children[j] );
							found = ret.found;
							if(!maybe) maybe = ret.maybe;
						}
						j++;
					}
					
					ibm.portal.debug.exit( "NavigationSelectionModel._calculateSelectedPageDefault", [ found ] );					
					return {found: found, maybe: maybe};	
				},
				
				_calculateSelectedPageSSABehavior: function(/*NavigationNode*/ navNode){	
				
					ibm.portal.debug.entry( "NavigationSelectionModel._calculateSelectedPageSSABehavior", [ navNode ] );
					var children = navNode.getChildren();
					var found = null, maybe = null;
					var i = 0;
					var ret = null;
					
					while ( i < children.length && !found ) {
						if ( !children[i].isLabel ) {
							maybe = children[i].URI;
							if(!children[i].isExternalURL && !children[i].isInternalURL) {
								found = maybe;
							}
						} else if(children[i].isLabel) {
						
						        ret = this._calculateSelectedPageSSABehavior( children[i] );
						   	found = ret.found;
							if(!maybe) maybe = ret.maybe;
						
						}
						
						i++;
					}
					

					
					ibm.portal.debug.exit( "NavigationSelectionModel._calculateSelectedPageSSABehavior", [ found ] );						
					
					return {found: found, maybe: maybe};				
				
				
				
				},
				
				_calculateSelectedPage: function( /*NavigationNode*/ navNode ) {
				
				       var ssaBehavior = ibmPortalConfig.selectedPageSSAbehavior;
				       if(ssaBehavior != null && ssaBehavior){
					return this._calculateSelectedPageSSABehavior(navNode)
				      }
				          return this._calculateSelectedPageDefault(navNode);
				},

				_validateSelection: function( /*String*/ selectedNodeID) {
				    ibm.portal.debug.entry( "NavigationSelectionModel._validateSelection", [ selectedNodeID ] );
					
					//Strip out the "cm:oid:"
					selectedNodeID = selectedNodeID.substring( 7 );
					// create a new state
				    var stateManager = new com.ibm.portal.state.StateManager( ibmPortalConfig["contentHandlerURI"] );
					var state = stateManager.newState();
				
				    // write the ID of the selected node into the state
				    var selectionAcc = stateManager.newSelectionAccessor(state);
				    selectionAcc.setSelection(selectedNodeID);
				
				    // validate the state using the serialization manager
				    var serializationManager = stateManager.getSerializationManager();
				    var result = serializationManager.serialize(state, true);
				    
				    // retrieve the validated state from the serialization result
				    var validatedState = result["state"];
				    
				    // read the validated selection and return it
				    selectionAcc = stateManager.newSelectionAccessor(validatedState);
				    
					ibm.portal.debug.exit( "NavigationSelectionModel._validateSelection", [ "cm:oid:" + selectionAcc.getSelection() ] );
					return "cm:oid:" + selectionAcc.getSelection();
				}
	
				
			  }
);

}

if(!dojo._hasResource["com.ibm.portal.services.PortletFragmentService"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.services.PortletFragmentService"] = true;
dojo.provide( "com.ibm.portal.services.PortletFragmentService" );

dojo.require( "dojox.data.dom" );

dojo.require( "com.ibm.portal.services.PortalRestServiceRequest" );
dojo.require( "com.ibm.portal.utilities" );
dojo.require( "com.ibm.portal.debug" );
dojo.require( "com.ibm.portal.EventBroker" );

dojo.declare( "com.ibm.portal.services.PortletFragmentURL",
			  null,
			  {
			  	constructor: function ( /*String*/uri ) {
				  	if ( uri.indexOf( "?uri=" ) == 0 ) {
						this.url = ibmPortalConfig["portalURI"] + uri;
						this.url = this.url.replace(/&amp;/g,"&" );
						this.url = this.url.replace( /lm:/, "pm:" );
					}
					else if ( uri.indexOf( "lm:" ) == 0 ) {
						this.url = ibmPortalConfig["portalURI"] + "?uri=fragment:" + uri;
						this.url = this.url.replace( /lm:/, "pm:" );
					}
					else {
						this.url = uri;
					}
				  }
			  }
);

dojo.declare( "com.ibm.portal.services.PortletInfo",
			  null,
			  {
				constructor: function( /*String*/wId, /*String*/pId, /*String*/pMarkup, /*Array*/pModes, /*Array*/wStates, /*Array*/depPortlets, /*Array*/otherPortlets, /*XMLDocument*/stateDOM, /*Array*/ stateVaryExpr, /*PortletMode*/ portletMode, /* WindowState */ windowState ) {
					ibm.portal.debug.entry( "PortletInfo.constructor", [ wId, pId, pMarkup, pModes, wStates, depPortlets, stateDOM ]);
					
					this.windowId = wId;
					this.portletId = pId;
					this.uri = "fragment:pm:oid:" + wId + "@oid:" + pId;
					this.markup = pMarkup;
					this.portletModes = pModes;
					this.windowStates = wStates;
					this.dependentPortlets = depPortlets;
					this.otherPortlets = otherPortlets;
					this.stateVaryExpressions = stateVaryExpr;
					this.updatedState = stateDOM;
					this.currentMode = portletMode;
					this.currentWindowState = windowState;
					
					ibm.portal.debug.exit( "PortletInfo.constructor" );
			    }
			  }
);

dojo.declare( "com.ibm.portal.services.PortletFragmentService",
			  null,
			  {
			  	namespaces: {
		            "xsl" : "http://www.w3.org/1999/XSL/Transform",
		            "thr" : "http://purl.org/syndication/thread/1.0",
		            "atom" : "http://www.w3.org/2005/Atom",
		            "xhtml" : "http://www.w3.org/1999/xhtml",
		            "model" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0.1/portal-model-elements",
		            "base" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0/ibm-portal-composite-base",
		            "portal" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0.1/portal-model",
		            "xsi" : "http://www.w3.org/2001/XMLSchema-instance",
				"state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state",
				"state-vary" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state-vary"
		        },
				_flagPortletUrl: function ( /*String*/ url, /*String*/ portletId ) {
					ibm.portal.debug.entry( "PortletFragmentService._flagPortletUrl", [ url ] );
					
					// Check to see if the uri parameter is already included in url
					var uriIncluded = url.indexOf("uri=fragment:pm:oid:");

					//Create the url and add the necessary parameters.
					var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
					urlObj.addParameter( "ibm.web2.keepRenderMode", "false" );
					
					// If uriIncluded is -1, then add the uri as a parameter
					if(uriIncluded < 0 ){
						//Convert the portlet id to a fragment id.
						portletId = portletId.replace( /lm:/g, "fragment:pm:" );
						urlObj.addParameter( "uri", portletId );
					}
					
					ibm.portal.debug.exit( "PortletFragmentService._flagPortletUrl", [ urlObj.toString() ]);
					return urlObj.toString();
				},
				getPortletInfo: function ( /*String*/portletId, /*String*/ portletUrl, /*Function?*/ callbackFn, /*PortalRestServiceForm?*/form, /*Boolean?*/stateless ) {
					// summary: Retrieves the PortletInfo for the given portlet url.
					// description: Sends a request for the given portlet information, including markup for the current navigational state. If the request does not
					//		complete successfully, a null object will be returned or passed into the callback function. 
					// portletUrl: a portlet URL, should come directly from the Portlet Fragment Feed or be created using the
					//		NavigationalState.newPortletRenderURL() function.
					// callbackFn: OPTIONAL - a callback function to be called when the portlet info is retrieved. the first parameter is the portlet info object
					//		or NULL if an error occurred and the second parameter is the XMLHttpRequest object used to make the request.
					// formNode: OPTIONAL - the form to submit as part of the request OR the query string produced by a "serialized" form
					// formFilter: OPTIONAL - the form filter for elements to encode (see dojo documentation)
					// stateless: OPTIONAL - indicates that this request should not affect the state of the portlet. Setting this to
					//		true cancels the topic publishing for this request only. 
					ibm.portal.debug.entry( "PortletFragmentService.getPortletInfo", [portletId, portletUrl, callbackFn, form, stateless ] );
					
					//Two cases where we don't want this request to continue:
					//1. The url is a hash mark (frequenty used for simply firing onclick events).
					//2. The url is a javascript: url. Usually this would have been discovered by now, but in the off chance
					//		that it isn't, we just want to eval it and return. 
					if ( portletUrl == "#" || portletUrl == window.location.href + "#" ) {
						ibm.portal.debug.text( "Illegal portlet url provided: " + portletUrl );
						ibm.portal.debug.text( "Aborting request." );
						return false;
					}
					
					if ( com.ibm.portal.utilities.isJavascriptUrl( portletUrl )) {
						return eval( portletUrl );
					}
					
					// if the URL is relative (e.g. "?fragment:pm:oid:..."), we need to resolve it
					var tempUrl = portletUrl;
					if(tempUrl.indexOf(top.location.href) == 0) {
						// here we're checking to see if the URL is actually the same as our current
						// url... we then trim that down and see what it really points to, like if it's
						// a relative URL in disguise 
						tempUrl = tempUrl.substring(top.location.href.length);
						while(tempUrl.length > 0 && tempUrl.charAt(0) == "/") {
							tempUrl = tempUrl.substring(1);
						}
					}
					if (tempUrl.indexOf('?') == 0) {
						var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
						portletUrl = state.resolveRelativePortletURL(tempUrl);
					}

					if ( com.ibm.portal.utilities.isExternalUrl( portletUrl ) ) {
						self.location.href = portletUrl;	
					}
					else{
						var url = { url: this._flagPortletUrl( portletUrl, portletId ) }
						var serviceReq = new com.ibm.portal.services.PortalRestServiceRequest( url, form );
						if ( !stateless ) {
							com.ibm.portal.EVENT_BROKER.startFragment.fire( { id: portletId } );
						}
						var me = this;
						serviceReq.read(
							function ( /*String*/type, /*XMLDocument*/xmlData, /*XMLHttpRequest*/xhr ) {
								var pInfo = null;
								if ( type == "load" ) {
									pInfo = me.createPortletInfo( xmlData );
								}
								
								if ( xmlData instanceof Error ) {
									pInfo = xmlData;
								}
								
								if ( !stateless ) {
									me._fireEvents( pInfo, portletId, xhr );
								}
								
								if ( callbackFn ) {
									callbackFn( pInfo, xhr );
								}
							}	
						);
					}

					ibm.portal.debug.exit( "PortletFragmentService.getPortletInfo" );	
				},
				readWindowID: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "PortletFragmentService.readWindowID", [xmlData] );
					
					var windowIdXPath = "/atom:feed/atom:entry/atom:id";
					var nodes = com.ibm.portal.xpath.evaluateXPath( windowIdXPath, xmlData, this.namespaces );
					var windowId = dojox.data.dom.textContent( nodes[0] );
					
					ibm.portal.debug.exit( "PortletFragmentService.readWindowID", [windowId.substring(4)]);		
					return windowId.substring(4);	
				},
				readPortletID: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "PortletFragmentService.readPortletID", [xmlData] );
					
					var portletIdXPath = "/atom:feed/atom:id";
					var nodes = com.ibm.portal.xpath.evaluateXPath( portletIdXPath, xmlData, this.namespaces );
					var portletId = dojox.data.dom.textContent( nodes[0] );
					
					ibm.portal.debug.exit( "PortletFragmentService.readPortletID", [portletId.substring(4)]);
					return portletId.substring(4);
				},
				readMarkup: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "PortletFragmentService.readMarkup", [xmlData] );
					var contentsXPath = "/atom:feed/atom:entry/atom:content";
					var nodes = com.ibm.portal.xpath.evaluateXPath(contentsXPath, xmlData, this.namespaces);
					var markup = "";
					
					if (nodes != null && nodes.length > 0) {
			            markup = dojox.data.dom.textContent( nodes[0] );
			        }
					
					ibm.portal.debug.exit( "PortletFragmentService.readMarkup", [markup]);
					return markup;
				},
				readPortletModes: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "PortletFragmentService.readPortletModes", [xmlData] );
					var modesXPath = "/atom:feed/atom:entry/atom:link[@portal:rel='portlet-mode']";
					var nodes = com.ibm.portal.xpath.evaluateXPath(modesXPath, xmlData, this.namespaces);
					var modes = new Array();
					
					if (nodes != null && nodes.length > 0) {
			        	var nodeCount = nodes.length;
			            for (var i = 0; i < nodeCount; i++) {
							modes.push( { "link": nodes[i].getAttribute("href"), "mode": nodes[i].getAttribute("title") } );
			            }
			        }
					ibm.portal.debug.exit( "PortletFragmentService.readPortletModes", [modes]);
					return modes;
				},
				readWindowStates: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "PortletFragmentService.readWindowStates", [xmlData] );
					var wStatesXPath = "/atom:feed/atom:entry/atom:link[@portal:rel='window-state']";
					var nodes = com.ibm.portal.xpath.evaluateXPath(wStatesXPath, xmlData, this.namespaces);
					var wStates = new Array();
					
					if (nodes != null && nodes.length > 0) {
			        	var nodeCount = nodes.length;
			            for (var i = 0; i < nodeCount; i++) {
							wStates.push( { "link": nodes[i].getAttribute("href"), "mode": nodes[i].getAttribute("title") } );
			            }
			        }
					ibm.portal.debug.exit( "PortletFragmentService.readWindowStates", [wStates]);
					return wStates;
				},
				readDependentPortlets: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "PortletFragmentService.readDependentPortlets", [xmlData] );
					var depPortletsXPath = "/atom:feed/atom:link[@portal:rel='dependent']";
					var nodes = com.ibm.portal.xpath.evaluateXPath(depPortletsXPath, xmlData, this.namespaces );
					var depPortlets = new Array();
					
					if (nodes != null && nodes.length > 0) {
			        	var nodeCount = nodes.length;
			            for (var i = 0; i < nodeCount; i++) {
							depPortlets.push( { "link": nodes[i].getAttribute("href"), "portlet": nodes[i].getAttribute("title"), "uri": nodes[i].getAttribute("portal:uri") ? nodes[i].getAttribute("portal:uri") :  nodes[i].getAttribute("uri") } );
			            }
			        }
					
					ibm.portal.debug.exit( "PortletFragmentService.readDependentPortlets", [depPortlets]);
					return depPortlets;
				},
				readOtherPortlets: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "PortletFragmentService.readOtherPortlets", [xmlData] );
					var otherPortletsXPath = "/atom:feed/atom:link[@portal:rel='other']";
					var nodes = com.ibm.portal.xpath.evaluateXPath(otherPortletsXPath, xmlData, this.namespaces );
					var otherPortlets = new Array();
					if (nodes != null && nodes.length > 0) {
			        	var nodeCount = nodes.length;
			            for (var i = 0; i < nodeCount; i++) {
							otherPortlets.push( { "link": nodes[i].getAttribute("href"), "portlet": nodes[i].getAttribute("title"), "uri": nodes[i].getAttribute("portal:uri") } );
			            }
			        }
					ibm.portal.debug.exit( "PortletFragmentService.readOtherPortlets", [otherPortlets]);
					return otherPortlets;
				},
				readStateVaryExpressions: function (/*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "PortletFragmentService.readStateVaryExpressions", [xmlData] );
					var stateVaryXPath = "/atom:feed/atom:entry/state-vary:state-vary/state-vary:expr";
					var nodes = com.ibm.portal.xpath.evaluateXPath(stateVaryXPath, xmlData, this.namespaces );
					var varyExpr = new Array();
					if (nodes != null && nodes.length > 0) {
						var nodeCount = nodes.length;
						for (var i = 0; i < nodeCount; i++) {
							var txtNode = nodes[i].firstChild;
							if (txtNode != null) {
								varyExpr.push( txtNode.nodeValue );
							}
						}
					}
					ibm.portal.debug.exit( "PortletFragmentService.readStateVaryExpressions", [varyExpr]);
					return varyExpr;
				},
				readPortletState: function ( /*XMLDocument*/xmlData ) {
					return this._readPortletState(xmlData);
				},
				_readPortletState: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "PortletFragmentService.readPortletState", [xmlData] );
					var stateXPath = "/atom:feed/atom:entry/state:root";
					var nodes = com.ibm.portal.xpath.evaluateXPath(stateXPath, xmlData, this.namespaces);
					var result = null;
					if ( nodes != null && nodes.length > 0 ) {
						var doc = dojox.data.dom.createDocument();
						com.ibm.portal.utilities.addExternalNode(doc, nodes[0]);
						result = doc;
					} else {
						stateXPath = "/atom:feed/state:root";
						nodes = com.ibm.portal.xpath.evaluateXPath(stateXPath,xmlData,this.namespaces);
						if ( nodes != null && nodes.length > 0 ) {
							var doc = dojox.data.dom.createDocument();
							com.ibm.portal.utilities.addExternalNode(doc, nodes[0]);
							result = doc;
						}
					}
					ibm.portal.debug.exit( "PortletFragmentService.readPortletState", [ result ] );
					return result;
				},
				_fireEvents: function ( /*PortletInfo*/pInfo, /*String*/portletId, /*XMLHttpRequest*/xhr ) {
					this._fireGlobalPortletStateChange( pInfo, portletId, xhr );
					//this._fireIndividualPortletStateChange( pInfo );
				},
				_fireGlobalPortletStateChange: function ( /*PortletInfo*/pInfo, /*String*/portletId, /*XMLHttpRequest*/xhr ) {
					//dojo.event.topic.publish( "/portal/PortletFragmentService/portletInfoRetrieved", { portletInfo: pInfo } );
					com.ibm.portal.EVENT_BROKER.endFragment.fire( { portletInfo: pInfo, id: portletId, xhr: xhr } );		
				},
				_fireIndividualPortletStateChange: function ( /*PortletInfo*/pInfo ) {
					//ibm.portal.debug.text( "Firing individual portlet event: /portal/navigation/portlet/" + pInfo.portletId + "@" + pInfo.windowId  );
					//dojo.event.topic.publish( "/portal/PortletFragmentService/portletInfoRetrieved/oid:" + pInfo.windowId + "@oid:" + pInfo.portletId , { portletInfo: pInfo } );	
				},
				createPortletInfo: function( /*XMLDocument*/xmlData ) {
					var windowId = this.readWindowID( xmlData );
					var portletId = this.readPortletID( xmlData ); 
					var markup = this.readMarkup( xmlData );
					var modes = this.readPortletModes( xmlData );
					var wStates = this.readWindowStates( xmlData );
					var depPortlets = this.readDependentPortlets( xmlData );
					var otherPortlets = this.readOtherPortlets( xmlData );
					var newState = this.readPortletState( xmlData );
					var stateVaryExpr = this.readStateVaryExpressions( xmlData );
					// determine the current portlet mode and window state
					var currentState = newState;
					if (currentState == null) {
						currentState = this._readPortletState(xmlData);
					}
					var stateMgr = new com.ibm.portal.state.StateManager();
					var portletAcc = stateMgr.newPortletAccessor( windowId, currentState );
					var mode = portletAcc.getPortletMode();
					var winState = portletAcc.getWindowState();
					return new com.ibm.portal.services.PortletInfo( windowId, portletId, markup, modes, wStates, depPortlets, otherPortlets, newState, stateVaryExpr, mode, winState );
				} 
			  }
);

dojo.declare( "com.ibm.portal.services.IndependentPortletFragmentService",
			  com.ibm.portal.services.PortletFragmentService,
			  {
			  	// summary: Ignores any dependent portlets.
			  	
			  	readDependentPortlets: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "DependentPortletFragmentService.readDependentPortlets", [xmlData] );
					var depPortlets = new Array();
				
					ibm.portal.debug.exit( "DependentPortletFragmentService.readDependentPortlets", [depPortlets]);
					return depPortlets;
				},	
				readOtherPortlets: function ( /*XMLDocument*/xmlData ) {
					ibm.portal.debug.entry( "DependentPortletFragmentService.readOtherPortlets", [xmlData] );
					var otherPortlets = new Array();
				
					ibm.portal.debug.exit( "DependentPortletFragmentService.readOtherPortlets", [otherPortlets]);
					return otherPortlets;
				},
				readPortletState: function ( /*XMLDocument*/xmlData ) {
					return null;
				}
			}
);

}

if(!dojo._hasResource["com.ibm.portal.services.common"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.services.common"] = true;
dojo.provide("com.ibm.portal.services.common");

dojo.require( "com.ibm.portal.services.NavigationModel" );
dojo.require( "com.ibm.portal.services.LayoutModel" );
dojo.require( "com.ibm.portal.services.NavigationSelectionModel" );
dojo.require( "com.ibm.portal.services.PortalRestServiceRequest" );
dojo.require( "com.ibm.portal.services.PortletFragmentService" );

}

if(!dojo._hasResource["com.ibm.aggregation"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.aggregation"] = true;
dojo.provide( "com.ibm.aggregation" );

dojo.declare( "com.ibm.aggregation.Aggregator", 
			  null, 
			  {
			  	// summary: Aggregates and render the appropriate information based
				//		on the navigational state. 
				// description: The Aggregator is responsible for rendering the page. An 
				//		aggregated page consists of two parts: the Navigation and the Page.
				//		The rendering is based on the navigational state provided by the 
				//		Navgiation object.
				// _navigation: com.ibm.aggregation.Navigation - the navgiation implementation
				// _pages: com.ibm.aggregation.Page - the page that this Aggregator object will render
				// _stateInfo: Object - information about the state 			  
			  	navigation: null,
				page: null,
				stateInfo: null,			  
			  	start: function () {
					// summary: Starts the aggregation rendering process.
					// description: Pulls the aggregation information and starts
					//		the rendering process. The information is based on the
					//		current navigational state provided by the Navigation 
					//		object.
					dojo.unimplemented( "com.ibm.aggregation.Aggregator.start" );
				} 	
			  }
);

dojo.declare( "com.ibm.aggregation.Page", 
			  null,
			  {
			  	// summary: A page that contains one or more Fragments.
				// description: Describes a page which contains one or more Fragments. The
				//		Fragments are arranged according to a layout provided by this page.
				// _fragments: Array of com.ibm.aggregation.Fragment - the fragments on this page.
				_fragments: new Array(),
				render: function ( /*dojo.dom.DOMDocument*/ state ) {
					// summary: Renders the page.
					// description: Renders the page and all its fragments in its specified layout.
					// state: com.ibm.aggregation.NavigationState - the current navigation state
					dojo.unimplemented( "com.ibm.aggregation.Page.render" );
				},
				getFragment: function ( /*String*/ fragId ) {
					// summary: Retrieves the Fragment on this page that matches the given id.
					// description: Retrieves the Fragment with the given id and returns the
					//		Fragment object. Returns null if no fragment with the given id can
					//		be found.
					// fragId: String - the fragment id
					// returns: com.ibm.aggregation.Fragment or null
					dojo.unimplemented( "com.ibm.aggregation.Page.getFrament" ); 
				}
					
			  } 
);

dojo.declare( "com.ibm.aggregation.Fragment",
			  null,
			  {
			  	// summary: An HTML fragment.
				// id: the unique identifier for this fragment
				id: null,
				_markup: null,
				render: function ( /*NavigationalState*/ state) {
					// summary: Renders this HTML fragment.
					// description: Renders this HTML fragment. Processes any included javascript using 
					// 		the specified javascript filter. 
					// state: the current navigational state
					dojo.unimplemented( "com.ibm.aggregation.Fragment.render" );
				}	
			  }
);

dojo.declare( "com.ibm.aggregation.FragmentMarkup",
			  null,
			  {
			  	update: function ( /*HTMLElement*/elementToUpdate, /*String*/markup ) {
					// summary: Update the specified element with the given markup.
					// elementToUpdate: HTMLElement - the element to update
					// markup: the markup string.
					dojo.unimplemented( "com.ibm.aggregation.FragmentMarkup.update" );	
				}	
			  }
);

dojo.declare( "com.ibm.aggregation.Navigation",
			  null,
			  {
			  	// summary: Contains all the information about the navigation for this aggregation.
				// description: Contains the information about navigating from page to page in the
				//		aggregation. 
				render: function ( /*dojo.dom.DOMDocument*/ state ) {
					// summary: Renders the navigation based on the navigational state.
					// state: the current navigational state
					dojo.unimplemented( "com.ibm.aggregation.Navigation.render" );
				}
			  }
);

}

if(!dojo._hasResource["com.ibm.portal.aggregation.theme"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.theme"] = true;
dojo.provide( "com.ibm.portal.aggregation.theme" );

dojo.require( "com.ibm.portal.services.common" );
dojo.require( "com.ibm.portal.utilities.html" );
dojo.require( "com.ibm.aggregation" );


dojo.declare( "com.ibm.portal.aggregation.theme.PortalNavigation",
    com.ibm.aggregation.Navigation,
    {
        // summary: Renders all registered NavigationElements using the current state.
        constructor: function () {
            this._elements = new Array();
        },
        render: function ( /*NavigationalState*/ state ) {
            // summary: Renders the navigation based on the navigational state.
            // state: the current navigational state
            ibm.portal.debug.entry( "PortalNavigation.render", [state]);
            ibm.portal.debug.text( "Navigation has " + this._elements.length + " elements." );
            for ( var i = 0; i < this._elements.length; i++ ) {
                if(this._elements[i].isEnabled()){
                    this._elements[i].render( state );
                }
            }
            ibm.portal.debug.exit( "PortalNavigation.render" );
        },
        addNavigationElement: function (/*NavigationElement*/ elem) {
            this._elements.push( elem );
        }
    } 
);

dojo.declare( "com.ibm.portal.aggregation.theme.NavigationElement",
              null,
              {
                // summary: Describes a generic navigation element.
                // description: Provides a generic abstract navigation element with various helper 
                //      and convenience functions. Determines if the state has changed, if it's 
                //      changed, the url from _getURL is used to send a PortalRestServiceRequest
                //      the result of which is passed into applyXslt. The specified xslt is applied
                //      and the html element is updated with the results. The render function can be
                //      overridden to implement any custom behavior, ignoring the helper functions. 
                
                _xslt: null,
                _htmlElementID: null,
                _resourceBaseURL: dojo.baseUrl + "../com/ibm/portal/aggregation/",        
                render: function ( /*NavigationalState*/ state ) {
                    // summary: Renders the navigation element.
                    // state: the current navigational state.
                    ibm.portal.debug.entry( "NavigationElement.render", [state]);
                    if ( this.hasChanged( state ) ) {
                        this.setLoading();
                        var url = this._getURL( state );
                        if ( url == null ) {
                            this.clearHTMLElement();
                            return; 
                        }
                        var serviceReq = new com.ibm.portal.services.PortalRestServiceRequest( url );
                        var me = this;
                        var xsltParams = this.getXSLTParams( state );
                        serviceReq.read( 
                            function ( type, xmlData, xhr, args ) { 
                                me.applyXslt( type, xmlData, xhr, args, xsltParams );
                                me.applySelectionPath( state.selectionModel );
                            }
                        );
                    } 
                    else {
                        //The selected node still could have changed.
                        this.applySelectionPath( state.selectionModel );
                    }
                        
                    ibm.portal.debug.exit( "NavigationElement.render" );                    
                },
                setLoading: function () {
                    this.getHTMLElement().innerHTML = "<img src='" + this._resourceBaseURL + "/progress-anim.gif' />&nbsp;Loading...";
                    var str = "<img src='" + this._resourceBaseURL + "progress-anim.gif' />&nbsp;Loading...";
                },
                applyXslt: function ( /*String*/type, /*dojo.dom.Document*/xmlData, /*XMLHttpRequest*/xhr, /*Object?*/args, /*Object*/xsltParams ) {
                    // summary: Applies the associated XSL transformation to the retrieved
                    //      XML data.
                    // description: Clears the html element and updates it with the result of the XSLT.
                    // xmlData: the retrieved XML Data
                    // args: any arguments passed through from the original caller
                    ibm.portal.debug.entry( "NavigationElement.applyXslt", [xmlData, xhr, args, xsltParams ] );
                    ibm.portal.debug.text( "Loading xslt: " + this._xslt );
                    var xslt = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer( this._xslt );
                    if (xslt == null) {
                        ibm.portal.debug.text("XSL transformer is null");
                    }
                    this.clearHTMLElement();
                    xslt.transformToRegion( xmlData, xsltParams, this.getHTMLElement(), document );
                    com.ibm.portal.utilities.html.removeNodesOnCondition(this.getHTMLElement(), function(node) {
						if(node && node.nodeType == 3 && node.nodeValue == "\n") {
							return true;
						}
						else return false;
					});
                        
                    ibm.portal.debug.exit( "NavigationElement.applyXslt" );
                },
                isEnabled: function () {
                    // summary: Determines if the element is enabled to be rendered.
                    // description: Indicates if the element is currently active and available to be rendered.
                    //      By default, this just verifies the HTML element is defined on the page, but could
                    //      be extended by navigational elements to control whet

                    return this.getHTMLElement() != null;
                },
                clearHTMLElement: function () {
                    // summary: Clears the markup associated with the navigation element.
                    if ( this.getHTMLElement() ) {
                    	this.getHTMLElement().innerHTML = "";
                    }	
                },
                getHTMLElement: function () {
                    // summary: Returns the markup elements associated with the navigation element.
                    // returns: the HTML element 
                    return dojo.byId( this._htmlElementID ); //HTMLElement
                },
                _getURL: function ( /*NavigationalState*/ state ) {
                    // summary: Returns the url for this service request.
                    // description: Constructs the url to the appropriate content for this element. If this URL is null,
                    //      the HTML element is cleared and it is assumed that this element does not apply to the current
                    //      navigational state.
                    // returns: a URL or NULL if this element should be cleared and doesn't apply to the current state
                    dojo.unimplemented( "com.ibm.portal.aggregation.theme.NavigationElement._getURL" ); 
                },
                applySelectionPath: function ( /*NavigationSelectionModel*/selModel ) {
                    // summary: Sets the selected element for this NavigationElement.
                    // selectedURI: the unique identifier of the selected element.
                    dojo.unimplemented( "com.ibm.portal.aggregation.theme.NavigationElement.applySelectionPath" );  
                },
                hasChanged: function ( /*NavigationalState*/ newState ) {
                    // summary: Determines if the element has changed.
                    // description: Indicates if the element has changed since the last time
                    //      it was rendered.
                    // newState: the state that is being rendered
                    dojo.unimplemented( "com.ibm.portal.aggregation.theme.NavigationElement.hasChanged" );
                },
                getXSLTParams: function ( /*NavigationalState*/ state ) {
                    // summary: Returns any XSLT params necessary for performing this transformation.
                    // state: the current navigational state
                    // returns: A two-dimensional array keeping the XSL parameters, e.g. [["a", "b"], ["x", "y"]]
                }
              } 
);

dojo.declare( "com.ibm.portal.aggregation.theme.ExpandableNavigationElement",
              com.ibm.portal.aggregation.theme.NavigationElement,
              {
                // summary: A navigation element that is expandable.
                
                expand: function ( /*NavigationalState*/state, /*Function?*/callbackFn ) {
                    // summary: Expand the element.
                    // state: the current navigational state
                    // callbackFn: the function to call when the expansion is complete
                    dojo.unimplemented( "com.ibm.portal.aggregation.ExpandableNavigationElement.expand" );
                },
                collapse: function ( /*NavigationalState*/state, /*Function?*/callbackFn ) {
                    // summary: Collapse the element.
                    // state: the current navigational state
                    // callbackFn: the function to call when the collapse is complete
                    dojo.unimplemented( "com.ibm.portal.aggregation.ExpandableNavigationElement.collapse" );
                }

              }  
);

dojo.declare( "com.ibm.portal.aggregation.theme.MainMenu",
              com.ibm.portal.aggregation.theme.NavigationElement,
                // summary: The main menu of the theme.
                // description: The main menu of the theme is constant and always contains the top two
                //      levels of navigation.
                // elemId: the target elem id where the context menu will display
                // rootNode: the rootNode's unique name or object id -- must point to a valid node in the Content Model. If not
                //      provided, defaults to "wps.content.root".
              {     
                constructor: function( /*String*/ elemId, /*String?*/rootNode ) {
                    ibm.portal.debug.entry( "MainMenu.constructor", [ elemId ] );
                
                    this._htmlElementID = elemId;
                    this._menuImgElementId = "wp_main_menu_img";
                    
                    //Set default fallback for the root node.
                    if ( !rootNode ) {
                        rootNode = "wps.content.root";
                    }
                    this._rootNode = rootNode;
                    
                    //create the unique id to reference context menu
                    this._menuId = elemId+"_menuId";
                },
                connected: false,
                render: function(){
                    ibm.portal.debug.entry( "MainMenu.render" );

                    //The menu only has to be connected once.
                    if ( this.connected ) {
                        return;
                    }
                    
                    dojo.connect( this.getHTMLElement(), "onclick", dojo.hitch(this, "displayMainMenu") );
                    this.connected = true;
                    
                    ibm.portal.debug.exit( "MainMenu.render" );
                },
                displayMainMenu: function (){
                    ibm.portal.debug.entry( "MainMenu.displayMainMenu" );
                    
                    var menu = getContextMenu(this._menuId);
                    var menuImgElement = dojo.byId( this._menuImgElementId );
                    
                    //check to see if menu is already created
                    if(menu == null){
                        if(!menuImgElement.oldSrc || menuImgElement.oldSrc == "") {
                            menuImgElement.oldSrc = menuImgElement.getAttribute( "src" );
                        }
                        menuImgElement.setAttribute( "src", this._resourceBaseURL + "/loading.gif" );
                        var url = new com.ibm.portal.services.ContentHandlerURL( "cm:oid:" + this._rootNode, 2, "download", "&mdname=com.ibm.portal.Hidden&mdname=com.ibm.portal.PageIcon&mdname=RenderMode" );
                        var serviceReq = new com.ibm.portal.services.PortalRestServiceRequest( url );
                        serviceReq.read( dojo.hitch( this, this.loadMenuContents ) );
                    }else{
                        return showContextMenu(this._menuId, this.getHTMLElement());
                    }
                    
                    ibm.portal.debug.exit( "MainMenu.displayMainMenu" );
                },
                
                loadMenuContents: function ( type, xmlData, args ) {
                    //summary: callback function to load the main menu navigation elements and actions, and then display the menu to the user.
                    ibm.portal.debug.entry( "MainMenu.loadMenuContents", [ type, xmlData ] );
                                        
                    //load navigation elements from the xmlData. The XSLT generates javascript to create the menu items, so we eval the results
                    // to execute the javascript
                    var createMenuItemsJs = this.applyXslt( type, xmlData, args );
                    //update the dom with the results
                    eval (createMenuItemsJs);

                    //add global actions to the menu
                    var engineProperties = dojo.i18n.getLocalization("com.ibm.portal", "engine", csaUserLocale);
                    var menu = getContextMenu(this._menuId);
                    // actions differ when logged in
                    if(ibmPortalConfig.isUserLoggedIn){
                        menu.add(new UilMenuItem( engineProperties["link.selfcare"], true, '', "asynchDoFormSubmit('"+this.getSelfcareActionURL()+"');" , null, ibm.portal.ThemeImages.getImagePath('icons/blank.gif'), null, "main-menu-item", "main-menu-item-selected" ));
                        menu.add( new UilMenuItem( engineProperties["link.logout"], true, '', "asynchDoFormSubmit('"+ibmPortalConfig.logoutURL+"')", null, ibm.portal.ThemeImages.getImagePath('icons/blank.gif'), null, "main-menu-item", "main-menu-item-selected" ) );                       
                    }
                    else {
                        menu.add(new UilMenuItem( engineProperties["link.enrollment"], true, '', "asynchDoFormSubmit('"+this.getSelfcareActionURL()+"');" , null, ibm.portal.ThemeImages.getImagePath('icons/blank.gif'), null, "main-menu-item", "main-menu-item-selected" ));
                        menu.add( new UilMenuItem( engineProperties["link.login"], true, '', 'javascript:goToLogin();', null, ibm.portal.ThemeImages.getImagePath('icons/blank.gif'), null, "main-menu-item", "main-menu-item-selected" ) );
                    }
                    
                    var menuImgElement = dojo.byId( this._menuImgElementId );
                    if(menuImgElement.oldSrc && menuImgElement.oldSrc != "") {
                        menuImgElement.setAttribute( "src", menuImgElement.oldSrc);
                    }
                    
                    ibm.portal.debug.exit( "MainMenu.loadMenuContents" );
                    //return the menu to display
                    return showContextMenu(this._menuId, this.getHTMLElement());
                },
                
                applyXslt: function ( type, xmlData, args ) {
                    ibm.portal.debug.entry( "MainMenu.applyXslt", [ type, xmlData ] );
                                        
                    //load the xslt
                    var xslDoc = com.ibm.portal.xslt.loadXsl(ibmPortalConfig["mainMenuXslt"]);
            
                    //apply the xslt
                    if (xslDoc.documentElement === null) {ibm.portal.debug.text("xslDoc is null");}
                    
                    var xsltParams = { menuId: this._menuId };
                    var results = com.ibm.portal.xslt.transform(xmlData, xslDoc, null, xsltParams, true);
                    ibm.portal.debug.exit( "MainMenu.applyXslt", results );
                    return results;
                },
                getSelfcareActionURL: function(){
                    // summary: create and return a url to the selfcare portlet.
                    ibm.portal.debug.entry( "MainMenu.getSelfcareActionURL" );
                    var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                    var currentSelectedNodeOid = state.selectionModel.getSelectedNode().URI;
                    var url = state.newPageURL( "wps.Selfcare", ibmPageActionWindowIDs["wps.Selfcare"] /*portletWindowId*/, "normal" /*portletWindowState*/);
                    var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                    urlObj.addParameter( "ao", "thm" );
                    urlObj.addParameter( "OCN", currentSelectedNodeOid);
                    ibm.portal.debug.exit( "MainMenu.getSelfcareActionURL" );
                    return urlObj.toString();                    
                }   
              }
);

dojo.declare( "com.ibm.portal.aggregation.theme.CrumbTrail",
              com.ibm.portal.aggregation.theme.NavigationElement,
              {
                // summary: The breadcrumb trail reflecting the current selection path.
                constructor: function ( /*int?*/startLevel, /*int?*/stopLevel ) {
                    // summary: Creates a breadcrumb trail which displays the selection path from the
                    //      start navigation level to the stop navigation level. If no stop level is 
                    //      specified, displays all the way to the selected page.
                    // startLevel: OPTIONAL -- defines the navigation level to start the crumbtrail from
                    // stopLevel: OPTIONAL -- defines the navigation level to stop the crumbtrail at. Omitting this parameter
                    //      or passing in -1 indicates that the crumbtrail should go all the way to the current selection.
                    if ( typeof( startLevel ) == "undefined" ) {
                        startLevel = 1;
                    }
                    this._startLevel = startLevel;
                    if ( typeof( stopLevel ) == "undefined" ) {
                        //stop level of -1 indicates no stop level provided.
                        stopLevel = -1;
                    }
                    this._stopLevel = stopLevel;
                },          
                _htmlElementID: "breadcrumbtrail",
                _renderedPath: null,
                render: function( /*NavigationalState*/ state ) {
                    ibm.portal.debug.entry( "CrumbTrail.render", [ state ] );
                    
                    var string = "";
                    var selectionPath = state.selectionModel.selectionPath;
                    
                    if ( selectionPath == this._renderedPath ) {
                        return;
                    }
                    
                    var loopStartIndex = this._startLevel;
                    var loopStopIndex = this._stopLevel;
                    
                    if ( loopStopIndex < 0 || loopStopIndex < loopStartIndex || loopStopIndex > selectionPath.length ) {
                        loopStopIndex = selectionPath.length;
                    } 
                    
                    for ( var i = loopStartIndex; i < loopStopIndex; i++ ) {
                        if ( i > loopStartIndex ) {
                            string += "<span class='wptheme-breadcrumbSeparator'>/</span>";
                        }
                        
                        ibm.portal.debug.text( "Selection Path [" + i + "] = " + selectionPath[i].URI );
                        var node = selectionPath[i];
                        ibm.portal.debug.text( "examining node: " + node.title + "; isLabel? " + node.isLabel );
                        if ( !node.isLabel ) {
                            var href = "javascript:com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.setSelection('" + node.URI + "'); ";
                            string += '<a href="' + href + '" >';
                        }
                        string += node.title;
                        if ( !node.isLabel ) { 
                            string += '</a>';
                        }
                    }

                    var htmlElem = this.getHTMLElement();
                    // might be null, e.g. in case of login link
                    if (htmlElem != null) {
                        htmlElem.innerHTML = string;
                        this._renderedPath = selectionPath;
                    }
                }   
              }
);

dojo.declare( "com.ibm.portal.aggregation.theme.TopLevelLinks",
              com.ibm.portal.aggregation.theme.NavigationElement,
              {
                constructor: function ( /*String*/elementID, /*int*/numToDisplay ) {
                    // summary: Displays the nodes directly underneath content root.
                    // elementID: the html element's id which contains the top level links. The markup will be inserted
                    //      into this element.
                    // numToDisplay: The number of links to display before dumping the rest into the "More..." menu. 
                    this._htmlElementID = elementID;
                    this._numToDisplay = numToDisplay;
                    this.rendered = false;
                },
                _xslt: ibmPortalConfig[ "topLevelLinksXslt" ], 
                _getURL: function ( /*NavigationalState*/ state ) {
                    // summary: Returns the url for this service request.
                    ibm.portal.debug.entry( "TopLevelLinks._getURL", [state] );
                    var url = new com.ibm.portal.services.ContentHandlerURL( "cm:oid:wps.content.root", 2, "download", "&mdname=com.ibm.portal.Hidden&mdname=com.ibm.portal.PageIcon&mdname=RenderMode" );
                    ibm.portal.debug.exit( "TopLevelLinks._getURL", url );
                    return url;
                },
                applySelectionPath: function ( selModel ) {
                    if ( !this.rendered ) {
                        this._buildMoreMenuIfNecessary();
                    }
                    this.rendered = true;
                    
                    var lis = this.getHTMLElement().childNodes[0].childNodes;
                    
                    for ( var i = 0; i < lis.length; i++) {
                        if ( selModel.isNodeInSelectionPath( lis[i].getAttribute( "oid" ) ) ) {
                            var imgs = lis[i].getElementsByTagName( "IMG" );
                            if ( imgs.length > 0 ) {
                                imgs[0].style.display='inline';
                            }   
                            //lis[i].className += " selectedNode";
                            dojo.addClass( lis[i], "wptheme-selectedTopLevelLink" );
                        }
                        else {
                            var imgs = lis[i].getElementsByTagName( "IMG" );
                            if ( imgs.length > 0 ) {
                                imgs[0].style.display='none';
                            }
                            dojo.removeClass( lis[i], "wptheme-selectedTopLevelLink" );
                        }
                    }
                    
                },
                hasChanged: function () {
                    return !this.rendered;
                },
                getXSLTParams: function ( /*NavigationalState*/ state ) {
                    // summary: Returns any XSLT params necessary for performing this transformation.
                    // state: the current navigational state
                    // returns: A two-dimensional array keeping the XSLT params.
                    var v3themeProperties = dojo.i18n.getLocalization("com.ibm.portal", "v3theme", csaUserLocale);
                    var isRTL = bidiSupport.isRTL;
                    var isFirefoxArabic = isRTL && !dojo.isIE && csaUserLocale == "ar";
                    ibm.portal.debug.text( "numToDisplay: " + this._numToDisplay + "; moreText: " + v3themeProperties["link.more"] + "; themeRoot: " + ibmPortalConfig["themeRootURI"] + "; isRTL: " + isRTL + "; isFirefoxArabic: " + isFirefoxArabic);
                    return {"numToDisplay": this._numToDisplay, "moreText": v3themeProperties["link.more"], "themeRoot": ibmPortalConfig["themeRootURI"], "isRTL": isRTL, "isFirefoxArabic": isFirefoxArabic};
                },
                _buildMoreMenuIfNecessary: function () {
                    var anchor = document.getElementById( "wptheme_more_menu" );
                    if ( anchor ) {
                        var menuID = 'wptheme_more_menu';
                        var isLTR = !bidiSupport.isRTL;
                        var menuBorderStyle="wptheme-more-menu-border";
                        var menuTableStyle="wptheme-more-menu";
                        var emptyMenuText="No actions currently available.";
                        var menuItemStyle = "wptheme-more-menu-item";
                        var menuItemSelectedStyle = "wptheme-more-menu-item-selected";
                        var icon = null;
                        var menu = createContextMenu( menuID, isLTR, null, menuBorderStyle, menuTableStyle, emptyMenuText, null, true );
                        
                        var lis = this.getHTMLElement().childNodes[0].childNodes; 
                        for ( var i = this._numToDisplay + 1; i < lis.length; i++ ) {
                            console.log( "processing: " + lis[i]);
                            var title = lis[i].getElementsByTagName("A")[0].innerHTML;
                            var href = lis[i].getElementsByTagName("A")[0].href;
                            var imgs = lis[i].getElementsByTagName( "IMG" );
                            var icon = null;
                            if ( imgs.length > 0 ) {
                                icon = imgs[0].src;
                            }    
                            
                            menu.add( new UilMenuItem( title, true, '', href, null, icon, null, menuItemStyle, menuItemSelectedStyle ) );
                        }
                        
                        dojo.connect( anchor, "onclick", anchor, function () { showContextMenu( menuID, this ); return false; } );
                    }
                }
          }
);

dojo.declare( "com.ibm.portal.aggregation.theme.ApplicationName",
              com.ibm.portal.aggregation.theme.NavigationElement,
              {
                // summary: The breadcrumb trail reflecting the current selection path.
                constructor: function ( elementID, navLevelOfNodeToDisplay ) {
                    // summary: Displays the current site area's name to the left of the top navigation tabs.
                    // elementID: the html element's ID. the markup will be inserted as a child of this node. 
                    // navLevelOfNodeToDisplay: the navigation level of the node to display the name of
                    this._htmlElementID = elementID;
                    this._navLevel = navLevelOfNodeToDisplay;
                },
                _renderedNode: null,            
                render: function( /*NavigationalState*/ state ) {
                    ibm.portal.debug.entry( "ApplicationName.render", [ state ] );
                    
                    var string = "";
                    var selectionPath = state.selectionModel.selectionPath;
                    
                    var node = selectionPath[ this._navLevel ];
                    
                    if ( node && node.URI == this._renderedNode ) {
                        return;
                    }
                    
                    var htmlElem = this.getHTMLElement();
                    if (htmlElem != null) {
                        htmlElem.innerHTML = node.title;
                        this._renderedPath = node.URI;
                    }
                }   
              }
);

dojo.declare( "com.ibm.portal.aggregation.theme.TopNavigation",
              com.ibm.portal.aggregation.theme.NavigationElement,
              {
                // summary: The top navigation of the theme.
                constructor: function ( /*String*/htmlElementID, /*int?*/startLevel, /*boolean?*/notFirstRow ) {
                    // summary: Creates the controller object for displaying the tabs in the top navigation.
                    // htmlElementID: the id of the html element which will contain the markup.
                    // startLevel: OPTIONAL -- indicates the navigation level in the selection path to pull the 
                    //      page titles from... currently only gathers 1 level of navigation.
                    if ( ibmPortalCurrentThemePolicy && ibmPortalCurrentThemePolicy[ "renderContextMenus" ] ) { 
                    	this._pageMenu = new com.ibm.portal.aggregation.theme.PageMenu();
                    }	
                    this._htmlElementID = htmlElementID;
                    if ( typeof( startLevel ) == "undefined" || startLevel < 1 ) {
                        startLevel = 2;
                    }
                    this._startLevel = startLevel;
                    if ( typeof( notFirstRow ) == "undefined" ) {
                        notFirstRow = false;
                    }
                    this._firstRow = !notFirstRow;
                },
                
                _xslt: ibmPortalConfig["topNavXslt"],
                _previousState: null,
                _getURL: function ( /*NavigationalState*/ state ) {
                    // summary: Returns the url for this service request.
                    ibm.portal.debug.entry( "TopNavigation._getURL", [state] );
                    
                    var node = state.selectionModel.selectionPath[this._startLevel - 1] ;
                    
                    ibm.portal.debug.exit( "TopNavigation._getURL", node.URI );
                    return new com.ibm.portal.services.ContentHandlerURL( node.URI, null, null, "&mdname=RenderMode");  
                },
                applySelectionPath: function ( /*NavigationSelectionModel*/selModel ) {
                    // summary: Sets the selected element for this NavigationElement.
                    // selectedURI: the unique identifier of the selected element.
                    ibm.portal.debug.entry( "TopNavigation.applySelectionPath", [ selModel ] );
                    
                    if ( selModel.selectionPath[this._startLevel] == null ) { return false; } 
                    
                    var selNode = selModel.selectionPath[this._startLevel].URI;
                    var topnavdiv = this.getHTMLElement();
                    var ul = topnavdiv.childNodes[0];
                    
                    dojo.forEach( ul.childNodes, function ( node ) { node.className = "wpsUnSelectedPage"; } );
                    dojo.byId( selNode ).className = "wpsSelectedPage";
                    //get current page displayed
                    var selectedPage = selModel.getSelectedNode();  
                    
                    // check to display the page menu
                    if( (this._pageMenu && selectedPage.URI == selNode) /*&& (dojo.byId("contextMenu_Page") != null)*/ ){
                        var elem = dojo.byId(selNode);
                        this._pageMenu.renderPageMenu(elem, selNode );
                    }
                    
                    ibm.portal.debug.exit( "TopNavigation.applySelectionPath" );    
                },
                hasChanged: function ( /*NavigationalState*/ newState ) {
                    // summary: Determines if the element has changed.
                    // description: Indicates if the element has changed since the last time
                    //      it was rendered.
                    // newState: the state that is being rendered
                    ibm.portal.debug.entry( "TopNavigation.hasChanged", [ newState ] );
                    
                    var retVal = true;
                    var parentLevel = this._startLevel - 1;
                    if ( newState.selectionModel.selectionPath[parentLevel] == null ) {
                        this._previousState = null;
                    }
                    else {
                        ibm.portal.debug.text( "Previous state: " + this._previousState + "; newState = " + newState.selectionModel.selectionPath[parentLevel].URI );
                        
                        if ( this._previousState != null ) {
                            retVal = !( this._previousState == newState.selectionModel.selectionPath[parentLevel].URI );    
                        }
                        
                        if ( retVal ) {
                            this._previousState = newState.selectionModel.selectionPath[parentLevel].URI;
                        }
                    }
                                        
                    ibm.portal.debug.exit ( "TopNavigation.hasChanged", [ retVal ]);
                    return retVal;
                },
                getXSLTParams: function ( /*NavigationalState*/ state, /*Boolean*/addingChildren, /*int*/ navLevel ) {
                    // summary: Returns any XSLT params necessary for performing this transformation.
                    // state: the current navigational state
                    // returns: A two-dimensional array keeping the XSL parameters
                    var params = null;
                    if ( this._firstRow ) {
                        params = {"topNavFirstRow": "true"};
                    }
                    return params;
                }
              } 
);

dojo.declare( "com.ibm.portal.aggregation.theme.SideNavigation",
              com.ibm.portal.aggregation.theme.ExpandableNavigationElement,
              {
                // summary: The side navigation of the theme.
                // description: The side navigation receives expand and collapse notifications
                //      by subscribing to the events published by the navigation controller.
                
                constructor: function ( /*int?*/startLevel ) {
                    // summary: Creates the controller object for the side navigation. 
                    // startLevel: OPTIONAL -- indicates the navigation level in the selection path to pull the 
                    //      page titles from
                    
                    this._expansionHandler = dojo.subscribe( "/portal/NavigationalStateController/expansion", 
                                                this, 
                                                function ( /*Object*/args ) {
                                                    var page = args["pageId"];
                                                    this.expand( page );
                                                }  
                    );
                    this._collapseHandler = dojo.subscribe( "/portal/NavigationalStateController/collapse", 
                                                this, 
                                                function ( /*Object*/args ) {
                                                    ibm.portal.debug.entry( "collapse handler", [args]);
                    								var page = args["pageId"];
                                                    this.collapse( page );
                                                    ibm.portal.debug.exit( "collapse handler" );
                                                }  
                    );
                    this._lastSelected = null;
                    if ( ibmPortalCurrentThemePolicy && ibmPortalCurrentThemePolicy[ "renderContextMenus" ] ) {
                    	this._pageMenu = new com.ibm.portal.aggregation.theme.PageMenu();
                    }	
                    
                    if ( typeof( startLevel ) == "undefined" || startLevel < 1 ) {
                        startLevel = 3;
                    }
                    this._startLevel = startLevel;
                },
                
                _xslt: ibmPortalConfig["sideNavXslt"],
                _htmlElementID: "sideNav",
                _previousState: null,
                _getURL: function ( /*NavigationalState*/ state ) {
                    // summary: Returns the url for this service request.
                    ibm.portal.debug.entry( "SideNavigation._getURL", [ state ]);
                    
                    if ( state.selectionModel.selectionPath.length < this._startLevel ) {
                        ibm.portal.debug.exit( "SideNavigation._getURL", [ null ] );
                        return null;
                    }
                    var topNavLevel = this._startLevel - 1;
                    var topNavNode = state.selectionModel.selectionPath[ topNavLevel ];
                    ibm.portal.debug.text( "top nav node has children? " + topNavNode.hasChildren );
                    if ( !topNavNode.hasChildren ) {
                        ibm.portal.debug.exit( "SideNavigation._getURL", [ null ] );
                        return null;
                    }
                    var node = state.selectionModel.selectionPath[topNavLevel];
                    ibm.portal.debug.exit( "SideNavigation._getURL", node.URI );
                    return new com.ibm.portal.services.ContentHandlerURL( node.URI, null, null, "&mdname=RenderMode" );     
                },
                applySelectionPath: function ( /*NavigationSelectionModel*/selModel, /*int?*/currentLevel ) {
                    // summary: Applies the selection path to this navigation element.
                    // description: Extracts any relevant information from the selection path and uses 
                    //      to set styles, etc in this navigation element.
                    // selModel: the NavigationSelectionModel
                    // currentLevel: when this function is called asynchronously after loading an expanded portion of the side 
                    //      navigation, this parameter indicates the place in the selection path where processing left off.
                    ibm.portal.debug.entry( "SideNavigation.applySelectionPath", [ selModel, currentLevel ] );
                    
                    var startIndex = this._startLevel, selPath = selModel.selectionPath, selIndex;
                    
                    if ( currentLevel ) {
                        selIndex = currentLevel;
                    }                   
                    else {
                        selIndex = startIndex;
                    }
                    
                    //In the case where we have gone from a side nav page to a top nav page, we need to clear out
                    //the old selected side nav item
                    if ( startIndex >= selPath.length && this._lastSelected ) {
                        var lastSelectedElement = this._getSideNavPageElement( this._lastSelected );
                        if ( lastSelectedElement ) {
                        	dojo.removeClass( lastSelectedElement, "wptheme-sideNavSelected" );
                        }	
                        this._lastSelected = null;
                    }
                    
                    for ( ; selIndex < selPath.length; selIndex++ ) {
                        ibm.portal.debug.text( "Examining element " + selIndex + " in the selection path: " + selPath[selIndex].URI );
                        var pageElement = this._getSideNavPageElement( selPath[selIndex].URI );
                        
                        if ( !pageElement ) {
                            // we may only expand nodes being part of the displayed subtree
                            if ( selPath.length-1 > this._startLevel ) {
                                var me = this;
                                this.expand( selPath[selIndex-1].URI, function () { me.applySelectionPath( selModel, selIndex ); } );
                            }
                            break;
                        } else {
                            if ( selIndex == ( selPath.length - 1) ) {
                                ibm.portal.debug.text( "Adding selected class to " + pageElement );
                                dojo.addClass( pageElement, "wptheme-sideNavSelected" );
                                if ( this._lastSelected && this._lastSelected != selPath[selIndex].URI ) {
                                    //Remove the selected class from the previous node.
                                    var lastSelectedElement = this._getSideNavPageElement( this._lastSelected );
                                    if ( lastSelectedElement ) {
                                    	ibm.portal.debug.text( "Removing selected from: " + dojo.byId( this._lastSelected ));
                                    	dojo.removeClass( lastSelectedElement, "wptheme-sideNavSelected" );
                                    }
                                    else {
                                    	this._checkForToggleIcon( selPath[selIndex].URI );
                                    }
                                }   
                                //Store this element as the last selected now.
                                ibm.portal.debug.text( "Setting last selected to: " + selPath[selIndex].URI );
                                this._lastSelected = selPath[selIndex].URI;
                                //get current page displayed                            
                                var selectedPage = selModel.getSelectedNode();  
                                                    
                                // check to display the page menu
                                if( this._pageMenu && (selectedPage.URI == this._lastSelected) ){
                                    var elem = dojo.byId(this._lastSelected).childNodes[0];
                                    this._pageMenu.renderPageMenu(elem, this._lastSelected);
                                }
                            }
                        }
                    } 
                    ibm.portal.debug.exit( "SideNavigation.applySelectionPath" );
                },
                _getSideNavPageElement: function ( uri ) {
                    ibm.portal.debug.entry( "SideNavigation._getSideNavPageElement" , [ uri ]);
                    var retVal = dojo.byId( uri );
                    
                    if ( retVal ) {
                        retVal = retVal.childNodes[0];
                    }
                    
                    ibm.portal.debug.exit( "SideNavigation._getSideNavPageElement", [ retVal ] );
                    return retVal;
                },
                hasChanged: function ( /*NavigationalState*/ newState ) {
                    // summary: Determines if the element has changed.
                    // description: Indicates if the element has changed since the last time
                    //      it was rendered.
                    // newState: the state that is being rendered
                    ibm.portal.debug.entry( "SideNavigation.hasChanged", [ newState ] );
                    
                    var retVal = true;
                    var topNavLevel = this._startLevel - 1;
                    if ( newState.selectionModel.selectionPath[topNavLevel] == null ) {
                        this._previousState = null;
                        this._lastSelected = null;
                    }
                    else {
                        if ( this._previousState != null ) {
                            retVal = !( this._previousState == newState.selectionModel.selectionPath[topNavLevel].URI );    
                        }
                        
                        if ( retVal ) {
                            this._changeOldTopNavLink( this._lastSelected ); // change previous top nav link to reference last selection, order matters
                            this._restoreNewTopNavLink( newState.selectionModel );  // restores the new top nav link if it was changed
                            this._previousState = newState.selectionModel.selectionPath[topNavLevel].URI;
                            this._lastSelected = null;
                        }
                    }   
                    
                    ibm.portal.debug.exit ( "SideNavigation.hasChanged", [ retVal ]);
                    return retVal;
                },
                expand: function ( /*String*/uri, /*Function?*/callbackFn ) {
                    // summary: Expand the element.
                    // uri: the uri of the element to expand
                    // callbackFn: the function to call when the expansion is complete
                    ibm.portal.debug.entry( "SideNavigation.expand", [ uri, callbackFn ] );
                    
                    var node = dojo.byId( uri );
                    if ( node.getElementsByTagName( "UL" ).length > 0 ) {
                        //PK84164: In IE, using dojo.fx.wipeIn/Out leaves the collapsed node in a bad state. Instead,
                    	//simply hide and show using the style.display property.
                    	//com.ibm.portal.utilities.html.show( node.getElementsByTagName("UL")[0] );
                    	node.getElementsByTagName("UL")[0].style.display = "block";
                        this._toggleExpandCollapseIcon( uri );
                        
                        ibm.portal.debug.exit( "SideNavigation.expand" );
                        return;
                    }
                    
                    var url = new com.ibm.portal.services.ContentHandlerURL( uri, 2, "download", "&mdname=RenderMode" );
                    var serviceReq = new com.ibm.portal.services.PortalRestServiceRequest( url );
                    var me = this;
                    serviceReq.read(
                        function ( type, xmlData, xhr ) {
                            me._addChildren( xmlData, uri );
                            me._toggleExpandCollapseIcon( uri );
                            if ( callbackFn ) {
                                callbackFn();
                            }   
                        }  
                    );
                    
                    ibm.portal.debug.exit( "SideNavigation.expand" );
                },
                _checkForToggleIcon: function ( /*String*/uri ) {
                	ibm.portal.debug.entry( "SideNavigation._checkForToggleIcon", [ uri ] );
                	var liNode = dojo.byId(uri);
                	var aNode = liNode.childNodes[0].childNodes[0];
                                        
                    if ( aNode ) {
                    	var hasChildren = ( liNode.childNodes.length >= 2 && liNode.childNodes[1].tagName && liNode.childNodes[1].tagName.toUpperCase() == "UL" );
                    	ibm.portal.debug.text( "List Item has children? " + hasChildren );
                    	if ( !hasChildren ) {
                    		//Delete the old anchor element.
                    		var node = aNode.parentNode.removeChild( aNode );
                    		delete node;
                    		//Create the spacer gif that exists when a node has no toggle arrow
                    		var dotGifPath = ibm.portal.ThemeImages.dotGifPath;
                    		var imgNode = document.createElement( "IMG" );
                    		imgNode.setAttribute("src", dotGifPath);
                    		imgNode.setAttribute("alt", "");
                    		imgNode.setAttribute("title", "" );
                    		dojo.addClass( imgNode, "wptheme-sideNavIcon" );
                    		dojo.place( imgNode, liNode.childNodes[0], "first" );
                    	}
                    }
                    ibm.portal.debug.entry( "SideNavigation._checkForToggleIcon" );	
                },
                _toggleExpandCollapseIcon: function ( /*String*/uri ) {
                    ibm.portal.debug.entry( "SideNavigation._toggleExpandCollapseIcon", [ uri ] );
                    var v3themeProperties = dojo.i18n.getLocalization("com.ibm.portal", "engine", csaUserLocale);
                    
                    var liNode = dojo.byId(uri);
                    var aNode = liNode.childNodes[0].childNodes[0];
                    var imgNode = aNode.childNodes[0];
                    // change href to collapse
                    //aNode.setAttribute("href","javascript:ibm.portal.Navigation.SideNav.collapse('"+uri+"', 2)");
                    //toggle image
                    if ( imgNode ) {
	                    if ( imgNode.getAttribute("src").indexOf( ibm.portal.ThemeImages.minusGifPath ) >= 0 ) {
	                        imgNode.setAttribute("src", bidiSupport.isRTL ? ibm.portal.ThemeImages.plusGifPathRTL : ibm.portal.ThemeImages.plusGifPath);
	                        imgNode.setAttribute("title", v3themeProperties["link.expand"] );
	                        imgNode.setAttribute("alt", v3themeProperties["link.expand"]);
	                        aNode.setAttribute("href", "javascript:com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.expand('" + uri +"' );" );
	                    }
	                    else {
	                        imgNode.setAttribute("src", ibm.portal.ThemeImages.minusGifPath);
	                        imgNode.setAttribute("title", v3themeProperties["link.collapse"] );
	                        imgNode.setAttribute("alt", v3themeProperties["link.collapse"] );
	                        aNode.setAttribute("href", "javascript:com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.collapse('" + uri +"' );" );
	                    }
                    }
                    else {
                    	ibm.portal.debug.text( "Creating the expand/collapse icon!" );
                    	var anchor = document.createElement( "A" );
                    	imgNode = document.createEleent( "IMG" );
                    	imgNode.setAttribute("src", ibm.portal.ThemeImages.minusGifPath);
	                    imgNode.setAttribute("title", v3themeProperties["link.collapse"] );
	                    imgNode.setAttribute("alt", v3themeProperties["link.collapse"] );
	                    anchor.setAttribute("href", "javascript:com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.collapse('" + uri +"' );" );
	                    dojo.addClass( anchor, "wptheme-sideNavIcon" );
	                    anchor.appendChild( imgNode );
	                    dojo.place( anchor, liNode.childNodes[0], "first" );
                    }  
                    
                    ibm.portal.debug.exit( "SideNavigation._toggleExpandCollapseIcon" );     
                },
                collapse: function ( /*String*/uri, /*Function?*/callbackFn ) {
                    // summary: Collapse the element.
                    // uri: the uri of the element to collapse
                    // callbackFn: the function to call when the collapse is complete
                    ibm.portal.debug.entry( "SideNavigation.collapse", [ uri ] );
                	
                	var liNode = dojo.byId(uri);
                    var selNode = dojo.byId(this._lastSelected);
                    var isDescendant = false;
                    
                    if ( selNode ) {
	                    isDescendant = com.ibm.portal.utilities.html.isDescendantOf( selNode, liNode );
                    }
                    
                    if(liNode.hasChildNodes() && !isDescendant){ 
                        ibm.portal.debug.text("hiding childNode: "+liNode.lastChild);
                        //PK84164: In IE, using dojo.fx.wipeIn/Out leaves the collapsed node in a bad state in this case. Instead,
                    	//simply hide and show using the elem.style.display property.
                    	//com.ibm.portal.utilities.html.hide( liNode.getElementsByTagName("UL")[0] );
                        liNode.getElementsByTagName("UL")[0].style.display = "none";
	                }
	                if(!isDescendant) {
	                    this._toggleExpandCollapseIcon( uri );
	                }
                    
                    ibm.portal.debug.exit( "SideNavigation.collapse" );
                },
                getXSLTParams: function ( /*NavigationalState*/ state, /*Boolean*/addingChildren, /*int*/ navLevel ) {
                    // summary: Returns any XSLT params necessary for performing this transformation.
                    // state: the current navigational state
                    // returns: A two-dimensional array keeping the XSL parameters
                    
                    var plusGifPath = ibm.portal.ThemeImages.plusGifPath;
                    if(bidiSupport.isRTL)
                        plusGifPath = ibm.portal.ThemeImages.plusGifPathRTL; 
                        
                    var minusGifPath = ibm.portal.ThemeImages.minusGifPath;
                    var dotGifPath = ibm.portal.ThemeImages.dotGifPath;
                    var v3themeProperties = dojo.i18n.getLocalization("com.ibm.portal", "engine", csaUserLocale);
                    
                    var level = 1;
                    var rootSideNav = null;
                    if ( addingChildren ) {
                        rootSideNav = "false";
                        level = navLevel;
                    }
                    ibm.portal.debug.text("plusGifPath: " + plusGifPath + "; plusGifAltText: "+v3themeProperties["link.expand"]+"; minusGifPath: " + minusGifPath + "; dotGifPath" + dotGifPath + "; navLevel: " + level + "; rootSideNav: " + rootSideNav );
                    var params = null;
                    if (rootSideNav != null) {
                        params = {"plusGifPath": plusGifPath, "plusGifAltText": v3themeProperties["link.expand"], "minusGifPath": minusGifPath, "dotGifPath": dotGifPath, "navLevel": level, "rootSideNav": rootSideNav};
                    } else {
                        params = {"plusGifPath": plusGifPath, "plusGifAltText": v3themeProperties["link.expand"], "minusGifPath": minusGifPath, "dotGifPath": dotGifPath, "navLevel": level};
                    }
                    return params;
                },
                _addChildren: function ( /*dojo.dom.Document*/xmlData, /*String*/uriToUpdate ) {
                    ibm.portal.debug.entry( "SideNavigation._addChildren", [ xmlData, uriToUpdate ] );

                    ibm.portal.debug.text( "Loading xslt: " + this._xslt );
                    ibm.portal.debug.text( "Applying to: " + dojox.data.dom.innerXML( xmlData ) );
                    var xslt = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer( this._xslt );
                    if (xslt == null) {
                        ibm.portal.debug.text("XSL transformer is null");
                    }
                    var node = dojo.byId( uriToUpdate );
                    var navLevel = node.getAttribute("navLevel");
                    navLevel++;
                    var content = xslt.transformToDocument(xmlData, this.getXSLTParams( null, true, navLevel ), true);
                    node.innerHTML += content;
                    ibm.portal.debug.exit( "SideNavigation._addChildren" );
                },
                _changeOldTopNavLink: function ( /*String*/uri ) {
                    var topSelElem = dojo.byId( this._topSelNode );
                    if(topSelElem && topSelElem.childNodes.length > 0 && uri && uri != "") {
                        var topAnchor = topSelElem.childNodes[0];
                        if(!topAnchor.topNodeHref || topAnchor.topNodeHref == "") {
                            topAnchor.topNodeHref = topAnchor.getAttribute("href"); // only save link to the top nav node
                        }
                        topAnchor.setAttribute("href", "javascript:com.ibm.portal." +
                            "navigation.controller.NAVIGATION_CONTROLLER.setSelection('" + uri + "', null, null, '');");
                    }
                },
                _restoreNewTopNavLink: function ( /*NavigationSelectionModel*/selModel ) {
                    this._topSelNode = selModel.selectionPath[2].URI;   // brittle... need agnostic way of getting selected top nav node
                    var topSelElem = dojo.byId( this._topSelNode );
                    if(topSelElem && topSelElem.childNodes.length > 0) {
                        var topAnchor = topSelElem.childNodes[0];
                        if(topAnchor.topNodeHref && topAnchor.topNodeHref != "") {
                            topAnchor.setAttribute("href", topAnchor.topNodeHref); // restore the old link
                        }
                    }
                }
              } 
);

dojo.declare( "com.ibm.portal.aggregation.theme.PageMenu",
              null,
              {
                constructor: function ( ) {
                
                    this._setPageActionWindowIds();
                    this._xslt = ibmPortalConfig["contextMenuXslt"];
                    this._pageMenuArray = new Array();
                    this._id = null;                
                    this._originalPageMenuImgSrc = null;                
                    this._engineProperties = dojo.i18n.getLocalization("com.ibm.portal", "engine", csaUserLocale);
                
                    this._deletePageHandler = dojo.subscribe("/portal/PageMenu/DeletePage/",
                                                this, 
                                                "_doDeletePage");
                                                
                    ibm.portal.debug.exit( "PageMenu.constructor" );
                },
                renderPageMenu: function ( /*HTMLElement*/elem, /*String*/ id, /*String*/ color ) {
                    
                    ibm.portal.debug.entry( "PageMenu.renderPageMenu", [ elem , id, color] );
                        
                    //Clear the menu.
                    this.clearMenu();
                    
                    this._id = id;
                    var menuButtonId = "menu_pageContextMenuButton";
                    var menuImageId = "menu_pageContextMenuButton_img";
                    
                    if(!color){
                        pageMenuIconHoverPath = ibm.portal.ThemeImages.pageMenuIconHoverPath["def"];
                    }else{
                        pageMenuIconHoverPath = ibm.portal.ThemeImages.pageMenuIconHoverPath[color];                    
                    }
                    //Create the button element. We use a button so that the context menu will appear
                    //directly below the image (limitation of the WCL context menu code).
                    var pageMenuButton = com.ibm.portal.utilities.html.createButton( elem,  
                                                                                  "javascript:void(0);",
                                                                                  menuButtonId,
                                                                                  null,
                                                                                  "wptheme-contextMenuIcon" );
                    
                    //Create the image to go on the button.
                    var pageMenuIcon = com.ibm.portal.utilities.html.createImage(pageMenuButton, 
                                                                               ibm.portal.ThemeImages.pageMenuIconPath, 
                                                                               menuImageId, 
                                                                               this._engineProperties["link.page.actions"], 
                                                                               "wptheme-contextMenuIcon" );
                    
                    //Connect all events.
                    var me = this;
                    dojo.connect( pageMenuButton, "onclick", function(){ me["displayPageMenu"](id); });
                    dojo.connect( pageMenuButton, "onmouseover", function () { document.getElementById(menuImageId).src=pageMenuIconHoverPath; });
                    dojo.connect( pageMenuButton, "onmouseout", function () { document.getElementById(menuImageId).src=ibm.portal.ThemeImages.pageMenuIconPath; });             
                    dojo.connect( pageMenuButton, "onfocus", function () { document.getElementById(menuImageId).src=pageMenuIconHoverPath; });
                    
                    ibm.portal.debug.exit( "PageMenu.renderPageMenu" );
                    
                }, 
                displayPageMenu: function (id){
                    
                    var menuId = "contextMenu_Page_" + id;
                    var menu = getContextMenu(menuId);
                    
                    if(menu == null){
                        var domSelNode = dojo.byId(id);
                        var menuUri = domSelNode.getAttribute("menuLink");
                        this.loadPageMenu(menuId, menuUri);
                    }else{
                        var targetImg = document.getElementById( "menu_pageContextMenuButton_img" );
                        return showContextMenu( menuId , targetImg ); 
                    }

                },
                loadPageMenu: function(menuId, menuUri){
                    
                    ibm.portal.debug.entry( "PageMenu.loadPageMenu" );
                    
                    this._originalPageMenuImgSrc = this._getPageMenuImgSrc();
                    this._setPageMenuImgSrc(ibm.portal.ThemeImages.contextLoadingPath);
    
                    var url = new com.ibm.portal.services.ContentHandlerURL( /*String*/menuUri, /*int*/null, /*String*/null, /*String?*/null );
                    var serviceReq = new com.ibm.portal.services.PortalRestServiceRequest( url );
                    var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                    serviceReq.read( dojo.hitch( this, "applyContextMenuXslt" ), { state: state, me: this, menuId: menuId } );
                    
                },  
                applyContextMenuXslt: function ( /*String*/type, /*dojo.dom.Document*/xmlData, /*XMLHttpRequest*/xhr, /*Array*/callbackargs ) {
                    // summary: Apply the XSLT to get the page action items from menu feed
                    // type: the type of response
                    // xmlData: the xml of the Feed
                    // xhr: the XMLHttpRequest used to retrieve the feed
                    // callbackargs: any arguments passed through to this function
                    
                    ibm.portal.debug.entry( "PageMenu.applyContextMenuXslt", [xmlData,xhr,callbackargs] );

                    var trf = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer( ibmPortalConfig["contextMenuXslt"] );
                    if (trf == null) {
                        ibm.portal.debug.text("XSL transformer is null");
                    }
                    
                    var results = trf.transformToDocument(xmlData, {}, true); 
                    eval( results );
                        
                    var me = callbackargs["me"];
                    me._pageMenuArray = menuItems;
                        
                    for(var i = 0; i< me._pageMenuArray.length; i++)                            
                        ibm.portal.debug.text("title: ["+me._pageMenuArray[i].title+"] label: ["+me._pageMenuArray[i].label+"] oncommand: []"+me._pageMenuArray[i].oncommand+"]");
                        
                    var menuId = callbackargs["menuId"];
                    this.createContextMenu(menuId, me);
                    
                },
                createContextMenu: function (menuId, me){
                    
                    ibm.portal.debug.entry( "PageMenu.createContextMenu" );

                    me.createPageMenuURLs(me);
                
                    menu = createContextMenu(menuId, !bidiSupport.isRTL, null, ""/*menuBorderStyle*/, "wptheme-pageContextMenuTable"/*menuTableStyle*/, this._engineProperties["info.emptymenu"]);
                    
                    var contents = me._pageMenuArray;
                    //Cycle through the menu item array.
                    for ( i=0; i < contents.length; i++ ) 
                    {
                       if ( contents[i] && contents[i].length != 0 )
                        {
                            if(contents[i].label != "DeletePage")
                                menu.add( new UilMenuItem( contents[i].title, true, '', "asynchDoFormSubmit('"+contents[i].url+"');", null, /*no icon*/ null, null, 'wptheme-pageContextMenuItem'/*menuItemStyle*/, 'wptheme-pageContextMenuItemHover'/*menuItemSelectedStyle*/) );
                            else
                                menu.add( new UilMenuItem( contents[i].title, true, '', 'dojo.publish("/portal/PageMenu/DeletePage/",[{pageId: "'+me._id+'"}] );', null, /*no icon*/ null, null, 'wptheme-pageContextMenuItem'/*menuItemStyle*/, 'wptheme-pageContextMenuItemHover'/*menuItemSelectedStyle*/) );
                        }
                        
                    } 
                    
                    var targetImg = dojo.byId( "menu_pageContextMenuButton_img");
                    this._setPageMenuImgSrc(this._originalPageMenuImgSrc);
                    
                    return showContextMenu( menuId , targetImg );
                    
                },
                createPageMenuURLs: function(me){   
                    ibm.portal.debug.entry( "PageMenu.createPageMenuURLs" );
                    var nodeOid = me._getSelectionNodeOid(me._id);
                    var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                    var cdoGuid = state.selectionModel.getSelectedNodeApplicationGuid();
                    ibm.portal.debug.text( "Application guid is " + cdoGuid ); 
                    for(var i = 0; i< me._pageMenuArray.length; i++){

                        switch (me._pageMenuArray[i].label){
                                
                            case "CreatePage":
                                var url = state.newPageURL( ibmPageActionIDs.createPage, ibmPageActionWindowIDs[ibmPageActionIDs.createPage] /*portletWindowId*/, "Solo" /*portletWindowState*/);

                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "nId", nodeOid );
                                urlObj.addParameter( "cId", nodeOid );
                                urlObj.addParameter( "init", "init" );
                                urlObj.addParameter( "OCN", nodeOid);
                                urlObj.addParameter( "WCN", nodeOid );
                                urlObj.addParameter( "PAT", "PTQ" );
                                urlObj.addParameter( "PNT", "PNTC" );
                                urlObj.addParameter( "psh", "push" );
                                
                                ibm.portal.debug.text("PageMenu.createPageMenuURLs \"Create Page\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                            case "EditLayout":
                                var url = state.newPageURL( ibmPageActionIDs.editLayout, ibmPageActionWindowIDs[ibmPageActionIDs.editLayout]/*portletWindowId*/, "normal" /*portletWindowState*/ );
                                
                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "nId", nodeOid );
                                urlObj.addParameter( "cId", nodeOid );
                                urlObj.addParameter( "init", "init" );
                                urlObj.addParameter( "WCN", nodeOid );
                                urlObj.addParameter( "OCN", nodeOid);
                                urlObj.addParameter( "WNN", nodeOid );
                                urlObj.addParameter( "psh", "push" );
                                
                                ibm.portal.debug.text("PageMenu.createPageMenuURLs\"Edit Page Layout\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                            case "EditPageProperties":
                                var selModel = state.selectionModel;
                                
                                var url = state.newPageURL( ibmPageActionIDs.editPageProperties , ibmPageActionWindowIDs[ibmPageActionIDs.editPageProperties] /*portletWindowId*/, "Solo" /*portletWindowState*/);
                                
                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "nId", nodeOid );
                                urlObj.addParameter( "cId", nodeOid );
                                urlObj.addParameter( "init", "init" );
                                urlObj.addParameter( "OCN", nodeOid);
                                urlObj.addParameter( "WCN", nodeOid );
                                urlObj.addParameter( "PAT", "PTQE" );
                                urlObj.addParameter( "PAT", "PTE" );
                                if (selModel.getSelectedNode().isLabel) {
                                    urlObj.addParameter( "PNT", "PNTL" );
                                }else{
                                    urlObj.addParameter( "PNT", "PNTC" );
                                }
                                urlObj.addParameter( "psh", "push" );
                                
                                ibm.portal.debug.text("PageMenu.createPageMenuURLs \"Page Properties\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                            case "AssignRoles":
                                var url = state.newPageURL( ibmPageActionIDs.assignRoles, ibmPageActionWindowIDs[ibmPageActionIDs.assignRoles]  /*portletWindowId*/, "Solo" /*portletWindowState*/ );
                                
                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "nId", nodeOid );
                                urlObj.addParameter( "cId", nodeOid );
                                urlObj.addParameter( "init", "init" );
                                urlObj.addParameter( "WCN", nodeOid );
                                urlObj.addParameter( "OCN", nodeOid);
                                urlObj.addParameter( "psh", "push" );
                                
                                ibm.portal.debug.text("PageMenu.createPageMenuURLs \"Resource Permissions\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                            case "EditApplicationProperties":
                                var url = state.newPageURL( ibmPageActionIDs.editAppProperties, ibmPageActionWindowIDs[ibmPageActionIDs.editAppProperties], "Solo" );
                                
                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "OCN", nodeOid );
                                urlObj.addParameter( "cdoguid", cdoGuid );
                                
                                ibm.portal.debug.text( "PageMenu.createPageMenuURLs \"Edit App Properties\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                            case "EditApplicationLayout":
                                var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                                
                                var url = state.newPageURL( ibmPageActionIDs.editAppLayout, ibmPageActionWindowIDs[ibmPageActionIDs.editAppLayout], "Solo" );
                                
                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "OCN", nodeOid );
                                urlObj.addParameter( "cdoguid", cdoGuid );
                                
                                ibm.portal.debug.text( "PageMenu.createPageMenuURLs \"Edit App Layout\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                            case "AssignApplicationMember":
                                var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                                
                                var url = state.newPageURL( ibmPageActionIDs.assignAppMembers, ibmPageActionWindowIDs[ibmPageActionIDs.assignAppMembers], "Solo" );
                                
                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "OCN", nodeOid );
                                urlObj.addParameter( "cdoguid", cdoGuid );
                                
                                ibm.portal.debug.text( "PageMenu.createPageMenuURLs \"Assign App Members\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                            case "ManageApplicationRoles":
                                var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                                
                                var url = state.newPageURL( ibmPageActionIDs.assignAppRoles, ibmPageActionWindowIDs[ibmPageActionIDs.assignAppRoles], "Solo" );
                                
                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "OCN", nodeOid );
                                urlObj.addParameter( "cdoguid", cdoGuid );
                                
                                ibm.portal.debug.text( "PageMenu.createPageMenuURLs \"Assign App Roles\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                            case "ShowApplicationStatusPolicy":
                                var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                                
                                var url = state.newPageURL( ibmPageActionIDs.showAppPolicyStatus, ibmPageActionWindowIDs[ibmPageActionIDs.showAppPolicyStatus], "Solo" );
                                
                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "OCN", nodeOid );
                                urlObj.addParameter( "cdoguid", cdoGuid );
                                
                                ibm.portal.debug.text( "PageMenu.createPageMenuURLs \"Show App Policy Status\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                            case "SaveAsTemplate":
                                var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                                
                                var url = state.newPageURL( ibmPageActionIDs.editAppProperties, ibmPageActionWindowIDs[ibmPageActionIDs.editAppProperties], "Solo" );
                                
                                var urlObj = new com.ibm.portal.utilities.HttpUrl( url );
                                urlObj.addParameter( "OCN", nodeOid );
                                urlObj.addParameter( "cdoguid", cdoGuid );
                                urlObj.addParameter( "act", "exp" );
                                
                                ibm.portal.debug.text( "PageMenu.createPageMenuURLs \"Save App As Template\" url: " + url );
                                me._pageMenuArray[i].url = urlObj.toString();
                            break;
                        }
                    }
                    return;                 
                },
                clearMenu: function(){
                                
                    ibm.portal.debug.entry( "PageMenu.clearMenu" ); 
                        
                    var menuButton = dojo.byId("menu_pageContextMenuButton");
                    if(menuButton != null){
                        var parent = menuButton.parentNode;
                        parent.removeChild(menuButton);
                    }
                    
                    var menuDiv = dojo.byId("contextMenu_Page");
                    if(menuDiv != null){
                        var parent = menuDiv.parentNode;
                        parent.removeChild(menuDiv);
                    }
                    
                    this._pageMenuArray = new Array();

                },
                _doDeletePage: function ( /*Object*/ args ) {
                    ibm.portal.debug.entry( "PageMenu._doDeletePage", [ args ] );
                    
                    var pageId = null;
                    
                    // validate input params
                    if(args != null){
                        pageId = args["pageId"];
                    }else{
                        ibm.portal.debug.text("PageMenu_doDeletePage: a required parameter is missing or null.");
                        return false;
                    }
                    
                    // check if the event is for us
                    if (this._id != pageId) {
                        return;
                    }
                    
                    var check = confirm(this._engineProperties["link.deletepage.confirm"]);
                    
                    if(check){
                        var deleteURL = ibmPortalConfig["contentHandlerURI"]+ '?uri='+pageId;
                        ibm.portal.debug.text("makeDeleteRequest: Delete URL is " + deleteURL);
                        
                        //get resource bundle for status messages
                        var v3themeProperties = dojo.i18n.getLocalization("com.ibm.portal", "v3theme", csaUserLocale);
                        //get parent container to remove deleted node
                        var parentURI = com.ibm.portal.services.NAVIGATION_MODEL.getNode( pageId ).childOf;
                        var parentNavNode = com.ibm.portal.services.NAVIGATION_MODEL.getNode( parentURI );
                        var me = this;
                        
                        var request = new com.ibm.portal.services.PortalRestServiceRequest( { url: deleteURL } );
                        request.remove( function(type, data, xhr, callbackargs) {
                        	var resTxt = data;
                            if (type == "load") {
                                me.removePageDOMItem(pageId);
                                parentNavNode.removeChild(pageId);
                                com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.setSelection(parentURI);
                                
                                ibm.portal.debug.text("PageMenu._doDeletePage: success: "+resTxt);
                                var statusMessage = new com.ibm.portal.aggregation.StatusMessage('info', v3themeProperties["pageDeleteOk"], "");
                                dojo.publish("/portal/status",[{message: statusMessage}]);
                                
                            } else {
                                ibm.portal.debug.text("PageMenu._doDeletePage: failure: "+resTxt);
                                var statusMessage = new com.ibm.portal.aggregation.StatusMessage('error', v3themeProperties["pageDeleteFailed"], resTxt);
                                dojo.publish("/portal/status",[{message: statusMessage}]);
                                dojo.publish("/portal/PortalPage/reload");
                            }
                        });
                    }
                },
                removePageDOMItem: function (pageId){
                    var domItem = dojo.byId(pageId);
                    var domItemParent = domItem.parentNode;
                    
                    if(domItemParent.childNodes.length <= 1){
                        domItemParent.parentNode.removeChild(domItemParent);    
                    }else{
                        domItemParent.removeChild(domItem);
                    }
                },
                applyExtractPortletWindowIdsXslt: function( /*String*/type, /*dojo.dom.Document*/xmlData, /*XMLHttpRequest*/xhr, /*Array*/callbackargs ) {
                    // summary: Apply the XSLT to extract the portlet window id's from the page specified
                    ibm.portal.debug.entry( "PageMenu.applyExtractPortletWindowIdsXslt", [xmlData,xhr,callbackargs] );
                    var pageUnid = callbackargs["page"];
                    var trf = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer( ibmPortalConfig["extractPortletWindowIdsXslt"] );
                    if (trf == null) {
                        ibm.portal.debug.text("XSL transformer is null");
                    }
                    var results = trf.transformToDocument(xmlData, {}, true);
                    eval( results );
                        
                    if(controls[0].id != null){
                        ibmPageActionWindowIDs[pageUnid] = com.ibm.portal.utilities.getControlId(controls[0].id);   
                    } else {
                        window.console.log( "Page action control id is null!" );
                    }   
                },
                _setPageActionWindowIds: function (){
                    if(ibmPageActionIDs != null && ibmPageActionWindowIDs == null ){
                        for (var unid in ibmPageActionIDs){
                            var pageNode = com.ibm.portal.services.NAVIGATION_MODEL.getNode(ibmPageActionIDs[unid]);
                            if(pageNode != null){
                                pageNode.getLayoutModel( dojo.hitch( this, "applyExtractPortletWindowIdsXslt" ), {page: ibmPageActionIDs[unid]} );
                            }
                        }
                    }
                },
                _getSelectionNodeOid: function (fullSelectionNodeId){
                    var selectionNodeOid = fullSelectionNodeId.substring(7);
                    return selectionNodeOid;
                },
                _getPageMenuImgSrc: function (){
                    return dojo.byId("menu_pageContextMenuButton_img").src;
                },
                _setPageMenuImgSrc: function (newSrc){
                    dojo.byId("menu_pageContextMenuButton_img").src = newSrc;
                }
                
              }
);

dojo.declare( "com.ibm.portal.aggregation.theme.PaletteURLFactory",
              null,
              {
                _lastSelection: null,
                _lastState: null,
                _pageName: null,
                _portletName: null,
                constructor: function () {
                    com.ibm.portal.EVENT_BROKER.stateChanged.register( this, this.storeLastState );
                },
                storeLastState: function ( /*Object*/args ) { 
                    // summary: Renders the navigation element.
                    // state: the current navigational state.
                    ibm.portal.debug.entry( "PortletPalette.render", [args]);
                    
                    var selectionChanged = false;
                    var currentState = args["state"];
                    if ( this._lastSelection != null ) {
                        if ( this._lastSelection != currentState.getPageSelection() ) {
                            selectionChanged = true;
                        }
                    }
                    
                    this._lastState = currentState;
                    this._lastSelection = currentState.getPageSelection();
                    
                    if ( selectionChanged ) {
                        var palettes = wptheme_InlinePalettes;
                        var contexts = palettes.paletteContextArray;
                        for ( var i = 0; i < contexts.length; i++ ) {
                            if ( contexts[i].selectionDependent && i == palettes.currentIndex && palettes.paletteStatus == 1 ) {
                                palettes.show( i );
                            }       
                        }
                    }       
                    
                    ibm.portal.debug.exit( "PortletPalette.render" );
                },
                createURL: function ( /*PaletteContext*/context ) {
                    // summary: Creates the url to render the portlet palette.
                    // returns: the serialized url pointing to the portlet palette
                    var renderParams = null;
                    if ( context.selectionDependent ) {
                        renderParams = [ {name: "WCN", value: this._lastState.getPageSelection() } ];
                    }
                    
                    var page = null, portlet = null, windowstate = null, portletmode = null, newwindow = null;
                    
                    if ( context.page ) { page = context.page; }
                    if ( context.portlet ) { portlet = context.portlet; }
                    if ( context.portletWindowState ) { windowstate = context.portletWindowState }
                    if ( context.portletMode ) { portletmode = context.portletMode; }
                    if ( context.newWindow ) { newwindow = context.newWindow; } 
                    
                    return this._lastState.newPageURL( page,
                                                       portlet, 
                                                       windowstate, 
                                                       portletmode, 
                                                       renderParams, 
                                                       newwindow); // String
                }   
              } 
);

// summary: Responsible for making sure the mouse cursor reflects the correct state of the
//      page.
com.ibm.portal.aggregation.theme.CURSOR_CONTROLLER = new function () {
    this._portalPageStartHandler = dojo.subscribe( "/portal/PortalPage/start", this, function () { document.body.style.cursor = 'progress'; } );
    this._portalPageCompleteHandler = dojo.subscribe( "/portal/PortalPage/complete", this, function () { document.body.style.cursor = ''; } );
}

}

if(!dojo._hasResource["com.ibm.portal.xpath"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.xpath"] = true;
/* Confidential                                                                                                         */
/*                                                                                                                                    */
/* OCO Source Materials                                                                                                */
/*                                                                                                                                    */
/* (C) Copyright IBM Corp. 2006,2007                                                                          */
/*                                                                                                                                    */
/* The source code for this program is not published or otherwise                               */
/* divested of its trade secrets, irrespective of what has been                                    */
/* deposited with the U.S. Copyright Office.                                                                 */
/*                                                                                                                                    */
/* ***************************************************************** */

dojo.provide( "com.ibm.portal.xpath" );

// generic impl

com.ibm.portal.xpath.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {
    if (typeof ActiveXObject != "undefined") {
        return com.ibm.portal.xpath.ie.evaluateXPath(xpathExpr, doc, namespaces);
    }
    else {
        return com.ibm.portal.xpath.gecko.evaluateXPath(xpathExpr, doc, namespaces);
    }
}

// IE Implementation

dojo.provide( "com.ibm.portal.xpath.ie" );

com.ibm.portal.xpath.ie.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {
        if (namespaces) {        
            var ns = "";
            for (var prop in namespaces) {
                ns += "xmlns:"+prop+"='"+namespaces[prop]+"' ";
            }
            if (doc.ownerDocument) {
                doc.ownerDocument.setProperty("SelectionNamespaces", ns);
            } else {
                doc.setProperty("SelectionNamespaces", ns);
            }
        }
        var result = doc.selectNodes(xpathExpr);

        var thisResult;
        var resultSet = new Array();
        var len = 0;
        for (var i=0; i<result.length; i++) {
            thisResult = result[i];
            if (thisResult) {
                resultSet[len] = thisResult;
                len++;
            }
        }
        return resultSet;
}


// GECKO Implementation

dojo.provide( "com.ibm.portal.xpath.gecko" );

com.ibm.portal.xpath.gecko.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {
        var result;
        try {
        	var rootDoc = doc;
        	if(!rootDoc.evaluate) rootDoc = doc.ownerDocument;
            result = rootDoc.evaluate( xpathExpr, doc, function(prefix) { return namespaces[prefix] || null;}, XPathResult.ANY_TYPE, null );
        } 
        catch (exc)
        {
            //Callers should catch this and can substitute their own error message
            throw new Error("Error with xpath expression" + exc);
        }
    
        //alert(result.resultType);
        var thisResult;
        var resultSet = new Array();
        var len = 0;
        do {
            thisResult = result.iterateNext();
            if (thisResult) {
                resultSet[len] = thisResult;
                len++;
            }
        } while ( thisResult );
        return resultSet;
}


}

if(!dojo._hasResource["ibm.portal.xml.xpath"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.portal.xml.xpath"] = true;
/* Confidential                                                                                                         */
/*                                                                                                                                    */
/* OCO Source Materials                                                                                                */
/*                                                                                                                                    */
/* (C) Copyright IBM Corp. 2006,2007                                                                          */
/*                                                                                                                                    */
/* The source code for this program is not published or otherwise                               */
/* divested of its trade secrets, irrespective of what has been                                    */
/* deposited with the U.S. Copyright Office.                                                                 */
/*                                                                                                                                    */
/* ***************************************************************** */

dojo.provide( "ibm.portal.xml.xpath" );
dojo.require( "com.ibm.portal.xpath" );

// generic impl

ibm.portal.xml.xpath.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {
    return com.ibm.portal.xpath.evaluateXPath(xpathExpr, doc, namespaces);
}

// IE Implementation

dojo.provide( "ibm.portal.xml.xpath.ie" );

ibm.portal.xml.xpath.ie.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {
        return com.ibm.portal.xpath.ie.evaluateXPath(xpathExpr, doc, namespaces);
}


// GECKO Implementation

dojo.provide( "ibm.portal.xml.xpath.gecko" );

ibm.portal.xml.xpath.gecko.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {
        return com.ibm.portal.xpath.gecko.evaluateXPath(xpathExpr, doc, namespaces);
}


}

if(!dojo._hasResource["ibm.portal.xml.xslt"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.portal.xml.xslt"] = true;
/* ***************************************************************** */
/*                                                                                                                                    */
/* IBM Confidential                                                                                                         */
/*                                                                                                                                    */
/* OCO Source Materials                                                                                                */
/*                                                                                                                                    */
/* (C) Copyright IBM Corp. 2006,2007                                                                          */
/*                                                                                                                                    */
/* The source code for this program is not published or otherwise                               */
/* divested of its trade secrets, irrespective of what has been                                    */
/* deposited with the U.S. Copyright Office.                                                                 */
/*                                                                                                                                    */
/* ***************************************************************** */
dojo.provide( "ibm.portal.xml.xslt" );
dojo.require( "com.ibm.portal.xslt" );

//---------------------------------------------------------------------- xml related utility methods
/*
 * loadXml(sUrl) - returns oDomDoc. parses xml from the url into DOM document object.
 * loadXmlString() - returns oDomDoc. parses xml from the contents of the string into DOM document object.
 * loadXsl()
 * transform()
 */
ibm.portal.xml.xslt.ie = {};
ibm.portal.xml.xslt.gecko = {};

ibm.portal.xml.xslt.getXmlHttpRequest = function() {
	return com.ibm.portal.xslt.getXmlHttpRequest();
}

ibm.portal.xml.xslt.loadXml = function(sUrl) {
	return com.ibm.portal.xslt.loadXml(sUrl);
}

ibm.portal.xml.xslt.loadXmlString = function(sXml) {
	return com.ibm.portal.xslt.loadXmlString(sXml);
}

ibm.portal.xml.xslt.loadXsl = function(sUrl) {
	return com.ibm.portal.xslt.loadXsl(sUrl);
}

ibm.portal.xml.xslt.transform = function (xml, xsl, sXslMode, aXslParams, bReturnString)
{
   ibm.portal.debug.entry( "transform", [xml,xsl,sXslMode,aXslParams,bReturnString] );  
   return com.ibm.portal.xslt.transform(xml, xsl, sXslMode, aXslParams, bReturnString );
}

ibm.portal.xml.xslt.transformAndUpdate = function (/*HTMLElement*/nodeToUpdate, /*XMLDocument*/xml, /*XMLDocument*/xsl, /*String?*/sXslMode, /*Map*/aXslParams) {
	ibm.portal.debug.entry( "transformAndUpdate", [ nodeToUpdate, xml, xsl, sXslMode, aXslParams ]);
	com.ibm.portal.xslt.transformAndUpdate(nodeToUpdate, xml, xsl, sXslMode, aXslParams);	
	ibm.portal.debug.exit( "transformAndUpdate" );
}

//---------------------------------------------------------------------- IE xml related utility methods

ibm.portal.xml.xslt.ie.loadXml = function(sUrl) {
	return com.ibm.portal.xslt.ie.loadXml(sUrl);
}

ibm.portal.xml.xslt.ie.loadXmlString = function(sXml) {
	return com.ibm.portal.xslt.ie.loadXmlString(sXml);
}

ibm.portal.xml.xslt.ie.loadXsl = function(sUrl) {
	return com.ibm.portal.xslt.ie.loadXsl(sUrl);
}

ibm.portal.xml.xslt.ie.transform = function(xmlDoc, xsl, sXslMode, aXslParams,bReturnString) {
	return com.ibm.portal.xslt.ie.transform(xmlDoc, xsl, sXslMode, aXslParams,bReturnString);
}

//---------------------------------------------------------------------- GECKO xml related utility methods

ibm.portal.xml.xslt.gecko.loadXml = function(sUrl) {
    return com.ibm.portal.xslt.gecko.loadXml(sUrl);
}

ibm.portal.xml.xslt.gecko.loadXmlString = function(sXml) {
    return com.ibm.portal.xslt.gecko.loadXmlString(sXml);
}

ibm.portal.xml.xslt.gecko.loadXsl = function(sUrl) {
	return com.ibm.portal.xslt.gecko.loadXsl(sUrl);
}

ibm.portal.xml.xslt.gecko.transform = function(xmlDoc, xsl, sXslMode, aXslParams,bReturnString) {
	return com.ibm.portal.xslt.gecko.transform(xmlDoc, xsl, sXslMode, aXslParams,bReturnString);
}

/* This method sets the content of a layer within the HTML page
 * to the result of transforming the xml parameter by the xsl
 * parameter. The xml and xsl parameters may be of any form
 * supported by the transformXml() method. The layer parameter
 * may be either a DOM object or the name of a DOM object that
 * can be found using the findObject() method.
 */

ibm.portal.xml.xslt.setLayerContentByXml = function (layer, xml, xsl, xslparam,bReturnString) {
    com.ibm.portal.xslt.setLayerContentByXml(layer, xml, xsl, xslparam,bReturnString);
}

}

if(!dojo._hasResource["ibm.portal.portlet.portlet"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.portal.portlet.portlet"] = true;
dojo.provide("ibm.portal.portlet.portlet")








ibm.portal.portlet._SafeToExecute = false;

if ( window.addEventListener ) {
    window.addEventListener( "load", function () { ibm.portal.portlet._SafeToExecute = true; }, false );
}
else if ( window.attachEvent ) {
    window.attachEvent( "onload", function () { ibm.portal.portlet._SafeToExecute = true; } );
}

dojo.declare("ibm.portal.portlet.PortletWindow", null, {

    STATUS_UNDEFINED: 0,
    STATUS_OK: 1,
    STATUS_ERROR: 2,
    
    constructor: function (/*String*/ windowid) {
        /**
         * summary:
        *     Constructor
        *
        * windowid: The window id
        */

        if (windowid==null) return; // special constructor for static initializers

        this.windowID = windowid;
        var element = document.getElementById("com.ibm.wps.web2.portlet.preferences."+this.windowID);
        this.preferenceEditID = element.getAttribute("editid");
        this.preferenceConfigID = element.getAttribute("configid");
        this.preferenceEditDefaultsID = element.getAttribute("editdefaultsid");
        this.pageID = element.getAttribute("pageid");

        this.attributes = new Array();
        this._queuedFuncs = new Array();
        this.portletState = new ibm.portal.portlet.PortletState(windowid);
        
        this.isCSA = false; 
        try {
            this.isCSA = (typeof(document.isCSA) != "undefined" );
        }
        catch (e) {}
        
        //Certain functions aren't safe to be executed until the page has loaded. This will execute any functions
        //which have been queued at the page load event.
        var me = this;
        function executeQueued() {
            for ( var i = 0; i < me._queuedFuncs.length; i++ ) {
                me._queuedFuncs[i]();
            }   
        }
        
        if ( window.addEventListener ) {
            window.addEventListener( "load", function () { if ( !ibm.portal.portlet._SafeToExecute) { ibm.portal.portlet._SafeToExecute = true; } executeQueued(); }, false );
        }
        else if ( window.attachEvent ) {
            window.attachEvent( "onload", function () { if ( !ibm.portal.portlet._SafeToExecute) { ibm.portal.portlet._SafeToExecute = true; } executeQueued(); } );
        }
    },
    
    reportError: function(/*ibm.portal.portlet.Error*/ error) {
    /**
     * summary:
     *     Reports an error to the framework for display purposes
     */
        var code;
        if (error.getErrorCode()==ibm.portal.portlet.Error.ERROR) code = "error";
        else if (error.getErrorCode()==ibm.portal.portlet.Error.INFO) code = "info";
        else if (error.getErrorCode()==ibm.portal.portlet.Error.WARN) code = "warning";
        var statusMessage = { "_type" : code,
                              "_message" : error.getMessage(), 
                              "_details" : error.getDescription() };
        if ( this.isCSA )
        {
            dojo.publish("/portal/status",[{message: statusMessage}]);
        }
        else {
            if ( typeof( console ) != "undefined") {
                if (error.getErrorCode()==ibm.portal.portlet.Error.ERROR) 
                    console.error(statusMessage._message+"\n"+statusMessage._details);
                else if (error.getErrorCode()==ibm.portal.portlet.Error.INFO) 
                    console.info(statusMessage._message+"\n"+statusMessage._details);
                else if (error.getErrorCode()==ibm.portal.portlet.Error.WARN) 
                    console.warn(statusMessage._message+"\n"+statusMessage._details);
            }
            else {
                alert(statusMessage._type.toUpperCase()+"\nMessage: "+statusMessage._message+"\nDetails: "+statusMessage._details);
            }
        }
    },
    getAttribute: function(/*String*/ name) {
    /**
     * summary:
     *     Gets the attribute for the given name scoped by the portlet window
     */
        return this.attributes[name]; // Object, maybe null
    },

    setAttribute: function(/*String*/ name, /*Object*/ value) {
    /**
     * summary:
     *     Sets the attribute for the given name scoped by the portlet window
     */
         var ret = this.attributes[name];
         this.attributes[name] = value;
         return ret; // Object, maybe null
    },

    removeAttribute: function(/*String*/ name) {
    /**
     * summary:
     *     Removes the attribute for the given name scoped by the portlet window
     *     
     */
        this.attributes[name] = null;
    },

    clearAttributes: function() {
    /**
     * summary:
     *     Removes all attributes scoped by the portlet window
     *     
     */
         this.attributes = new Array();
    },

    getPortletState: function(/*Function*/ callbackfn) {
    /**
     * summary:
     *     Gets the portlet state bean scoped by the window id
     *     
     * callbackfn: Function - defines the callback function
     *     that is called when method finishes loading the portlet state.
     *     In case NULL is passed in for callbackfn, the method executes synchronously
     */
        var returnObject = this.portletState;
        var pointer2this = this;
        var result = null;
        if (callbackfn != null) {
            // invoke the callback function
            callbackfn(pointer2this, ibm.portal.portlet.PortletWindow.STATUS_OK, returnObject);
        } else {
            result = { "portletWindow" : pointer2this, 
                       "status" : ibm.portal.portlet.PortletWindow.STATUS_OK,
                       "returnObject" : returnObject };
        }
        return result;
    },

    setPortletState: function(/*PortletState*/state, /*Function*/ callbackfn) {
    /**
     * summary:
     *     Sets the portlet state bean (holding the navigational state) scoped
     *     by the window id. The behavior of this function varies depending on
     *     whether or not the surrounding theme is capable of dealing with
     *     client-side state changes. If not, this function will trigger a full
     *     page refresh. Otherwise the provided callback function decides how to
     *     react on the state change.
     *     
     * callbackfn: Function - defines the callback function
     *     that is called when method finishes storing the portlet state.
     *     The callback function defines how to update the portlet's markup
     *     to reflect the new navigational state. If no callback function is
     *     provided (or null is passed in) the method executes synchronously,
     *     and in addition refreshes the portlet (default behavior of reacting
     *     to a navigational state change.
     */
        this.portletState = state;
        
        if ( this.isCSA ) {
            // if no callback function is provided, execute the default behavior:
            // refresh the portlet by sending a render request to the server
            if (callbackfn == null) {
                var navState = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                var url = navState.newPortletRenderURL( this.windowID );
                var service = new com.ibm.portal.services.PortletFragmentService();
                service.getPortletInfo( "lm:oid:" + this.windowID + "@oid:"+ this.pageID, url );
            }
        } else {
            var stateManager = new com.ibm.portal.state.StateManager(ibmPortalConfig["contentHandlerURI"]);
            stateManager.reset( state.portletAccessor.stateDOM );
            // serialize the state into a portal URL
            var serializationMgr = stateManager.getSerializationManager();
            var serializationResult = serializationMgr.serialize(stateManager.getState());
            
            // return the URL
            var serializedState = serializationResult["returnObject"];
            
            var url = serializedState;
            window.location.href = url;
        }
        // dispatch
        return this.getPortletState(callbackfn);
    },
    
    _queueUp: function ( /*Function*/funcToQ ) {
        // summary: Certain functions are not safe to execute before the page loads. Any function
        //      passed to this function will be queued up and executed only after the page loads.
        // funcToQ: the function to queue
        this._queuedFuncs.push( funcToQ );
    },
    
    _throwInappropriateRequestError: function ( /*String*/funcName ) {
        throw new Error( "Cannot execute a synchronous call before the page loads! Please use an onload handler to execute this call to \"" + funcName + "\"." );
        return null;
    },
    
    getPortletPreferences: function(/*Function*/ callbackfn) {
    /**
     * summary:
     *     Gets the portlet preferences bean scoped by the window id
     *     
     * callbackfn: Function - defines the callback function
     *     that is called when method finishes loading the portlet preferences.
     *     In case NULL is passed in for callbackfn, the method executes synchronously
     */
        if ( !ibm.portal.portlet._SafeToExecute ) {
            if ( callbackfn ) {
                var me = this;
                this._queueUp( function () { me.getPortletPreferences( callbackfn ) } );
                return false; 
            }
            else {
                return this._throwInappropriateRequestError( "getPortletPreferences" );
            }
        }
        
        var portletMode = this.getPortletState().returnObject.getPortletMode();
         this.status = ibm.portal.portlet.PortletWindow.STATUS_UNDEFINED;
         var tempurl = document.getElementById("com.ibm.wps.web2.portlet.root."+this.windowID).innerHTML;
         var idx = tempurl.indexOf("--portletwindowid--");
         var _url = tempurl.replace(/--portletwindowid--/g, this.windowID);
         
         if ( _url.indexOf( "?" ) < 0 ) {
            _url = _url + "?"; 
         } 
         
         _url += "&verb=download&levels=-all&rep=compact&preferences=aggregated";

         this.requestedPreferenceID = "pm:oid:"+this.preferenceEditID; //+"@oid:"+this.pageID;
         if (portletMode==ibm.portal.portlet.PortletMode.CONFIG) 
             this.requestedPreferenceID = "pm:oid:"+this.preferenceConfigID;
         else if (portletMode==ibm.portal.portlet.PortletMode.EDIT_DEFAULTS) 
             this.requestedPreferenceID = "pm:oid:"+this.preferenceEditDefaultsID;
         
         var pointer2this = this;
         var pointer2return = null;
         
         dojo.xhrGet({
             url: _url,
             handleAs: "xml",
             headers: { "X-IBM-XHR":"true", "If-Modified-Since": "Thu, 1 Jan 1970 00:00:00 GMT"}, //temporary to force the browser to ignore cached requests
             sync: (callbackfn)?false:true,
             handle: function(response, ioArgs){
                 if ( pointer2this.isAuthenticationRequired( ioArgs.xhr, ioArgs.args.handleAs ) ) {
                	 pointer2this.doAuthentication();
                 } else {
            	 
	            	 var type = ( response instanceof Error ) ? "error" : "load";
	                  if(type == "load") {
	                      var responseXML = response;
	                      
	                      //IE doesn't populate the responseXML property here because of the content-type of the response. So we need to create the XML
	                      //document from text to be able to execute XPath queries.
	                      if ( !responseXML || ( typeof( dojox.data.dom.innerXML( response )) == "undefined" ) ) {
	                        responseXML = dojox.data.dom.createDocument( ioArgs.xhr.responseText );
	                      }
	                      
	                      var returnObject = 
	                          new ibm.portal.portlet.PortletPreferences(pointer2this.windowID,
	                                                                        pointer2this.requestedPreferenceID,
	                                                                        responseXML );
	                      if (callbackfn) {
	                          callbackfn(pointer2this,ibm.portal.portlet.PortletWindow.STATUS_OK,returnObject);
	                      }
	                      else {
	                          pointer2return = {"portletWindow" : pointer2this, 
	                              "status" : ibm.portal.portlet.PortletWindow.STATUS_OK,
	                              "returnObject" : returnObject };
	                      }
	                  }else if(type == "error"){
	                      if (callbackfn) {
	                          callbackfn(pointer2this, ibm.portal.portlet.PortletWindow.STATUS_ERROR, null);
	                      }
	                      else {
	                          pointer2return = {"portletWindow" : pointer2this, 
	                              "status" : ibm.portal.portlet.PortletWindow.STATUS_ERROR,
	                              "returnObject" : null };
	                      }
	                  }
                 } 
             },
             transport: "XMLHTTPTransport"
         });
         
         return pointer2return; // Object - synchronously: { portletWindow, status, returnObject } / asynchronously: NULL
    },

    setPortletPreferences: function(/*PortletPreferences*/preferences, /*Function*/ callbackfn) {
    /**
     * summary:
     *     Sets the portlet preferences bean scoped by the window id
     *     
     * callbackfn: Function - defines the callback function
     *     that is called when method finishes storing the portlet preferences.
     *     In case NULL is passed in for callbackfn, the method executes synchronously
     */
         if ( !ibm.portal.portlet._SafeToExecute ) {
            if ( callbackfn ) {
                var me = this;
                this._queueUp( function () { me.setPortletPreferences( preferences, callbackfn ) } );
                return false; 
            }
            else {
                return this._throwInappropriateRequestError( "setPortletPreferences" );
            }
        }
         
         this.status = ibm.portal.portlet.PortletWindow.STATUS_UNDEFINED;
         var tempurl = document.getElementById("com.ibm.wps.web2.portlet.root."+this.windowID).innerHTML;
         var idx = tempurl.indexOf("--portletwindowid--");
         var _url = tempurl.replace(/--portletwindowid--/g, this.windowID);
         if ( _url.indexOf( "?" ) < 0 ) {
             _url += "?verb=download";
         }
         else {
             _url += "&verb=download";
         }

         var requestedPreferenceID = preferences.requestedPreferenceID;
         // strip down to necessary data
         var expr = "/atom:feed/atom:entry[atom:id='"+requestedPreferenceID+"']";
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, preferences.xmlData, preferences.ns);
         var entry;
         if (nodes && nodes.length>0) {
             entry = nodes[0];
         }
         else {
             return null; // TODO
         }
         var parentNode = entry.parentNode;
         expr = "/atom:feed/atom:entry";
         nodes = ibm.portal.xml.xpath.evaluateXPath(expr, preferences.xmlData, preferences.ns);
         for (var i=0; i<nodes.length; i++) {
             var node = nodes[i];
             if (node!=entry) {
                 parentNode.removeChild(node);
             }
         }
         
         //alert (dojox.data.dom.innerXML(preferences.xmlData));

         var pointer2this = this;
         var pointer2return = null;


         dojo.rawXhrPut({
             url: _url,
             sync: (callbackfn)?false:true,
             putData: dojox.data.dom.innerXML(preferences.xmlData),
             contentType: "application/xml",
             headers: { "X-IBM-XHR":"true"},
             handleAs: "xml",
             handle: function(response, ioArgs){
	        	 if ( pointer2this.isAuthenticationRequired( ioArgs.xhr, ioArgs.args.handleAs ) ) {
	        		 pointer2this.doAuthentication();
	             } else {  
	        	 	var type = ( response instanceof Error ) ? "error" : "load";
	                  if(type == "load") {
	                      if (callbackfn) {
	                          callbackfn(pointer2this,ibm.portal.portlet.PortletWindow.STATUS_OK,preferences);
	                      }
	                      else {
	                          pointer2return = {"portletWindow" : pointer2this, 
	                              "status" : ibm.portal.portlet.PortletWindow.STATUS_OK,
	                              "returnObject" : preferences };
	                      }
	                  }else if(type == "error") {
	                      if (callbackfn) {
	                          callbackfn(pointer2this, ibm.portal.portlet.PortletWindow.STATUS_ERROR, null);
	                      }
	                      else {
	                          pointer2return = {"portletWindow" : pointer2this, 
	                              "status" : ibm.portal.portlet.PortletWindow.STATUS_ERROR,
	                              "returnObject" : null };
	                      }
	                  }
	             }
             },
             transport: "XMLHTTPTransport"
         });
         return pointer2return; // Object - synchronously: { portletWindow, status, returnObject } / asynchronously: NULL
    },

    getUserProfile: function(/*Function*/ callbackfn) {
    /**
     * summary:
     *     Gets the current user profile
     *     
     * callbackfn: Function - defines the callback function
     *     that is called when method finishes loading the portlet preferences.
     *     In case NULL is passed in for callbackfn, the method executes synchronously
     */
         if ( !ibm.portal.portlet._SafeToExecute ) {
            if ( callbackfn ) {
                var me = this;
                this._queueUp( function () { me.getUserProfile( callbackfn ) } );
                return false; 
            }
            else {
                return this._throwInappropriateRequestError( "getUserProfile" );
            }
        }
         
         this.status = ibm.portal.portlet.PortletWindow.STATUS_UNDEFINED;
         var _url = document.getElementById("com.ibm.wps.web2.portlet.user."+this.windowID).innerHTML;

         var pointer2this = this;
         var pointer2return = null;
         dojo.xhrGet({
             url: _url,
             headers: { "X-IBM-XHR":"true","If-Modified-Since": "Thu, 1 Jan 1970 00:00:00 GMT"}, //temporary to force the browser to ignore cached requests
             sync: (callbackfn)?false:true,
             handleAs: "xml",
             handle: function(response, ioArgs){
            	 if ( pointer2this.isAuthenticationRequired( ioArgs.xhr, ioArgs.args.handleAs ) ) {
            		 pointer2this.doAuthentication();
                 } else { 
	            	 var type = ( response instanceof Error ) ? "error" : "load";
	                  if (type == "load") {
	                      var responseXML = response;
	                      
	                      // IE doesn't populate the responseXML property here because of the content-type of the response.
	                      // We need to create the XML document from text to be able to execute XPath queries.
	                      if ( !responseXML || ( typeof( dojox.data.dom.innerXML( response )) == "undefined" ) ) {
	                          responseXML = dojox.data.dom.createDocument( ioArgs.xhr.responseText );
	                      }
	
	                      var returnObject = 
	                          new ibm.portal.portlet.UserProfile(pointer2this.windowID, responseXML );
	
	                      if (callbackfn) {
	                          callbackfn(pointer2this,ibm.portal.portlet.PortletWindow.STATUS_OK,returnObject);
	                      }
	                      else {
	                          pointer2return = {"portletWindow" : pointer2this, 
	                              "status" : ibm.portal.portlet.PortletWindow.STATUS_OK,
	                              "returnObject" : returnObject };
	                      }
	                  } else if(type == "error"){
	                      if (callbackfn) {
	                          callbackfn(pointer2this, ibm.portal.portlet.PortletWindow.STATUS_ERROR, null);
	                      }
	                      else {
	                          pointer2return = {"portletWindow" : pointer2this, 
	                              "status" : ibm.portal.portlet.PortletWindow.STATUS_ERROR,
	                              "returnObject" : null };
	                      }
	                  }
                 } 
             },
             transport: "XMLHTTPTransport"
         });
         return pointer2return; // Object - synchronously: { portletWindow, status, returnObject } / asynchronously: NULL
    },

    setUserProfile: function(/*userProfile*/userProfile, /*Function*/ callbackfn) {
    /**
     * summary:
     *     Sets the current user profile
     *     
     * callbackfn: Function - defines the callback function
     *     that is called when method finishes loading the portlet preferences.
     *     In case NULL is passed in for callbackfn, the method executes synchronously
     */
         //var serializer = new XMLSerializer();
         //resultStr = serializer.serializeToString(userProfile.xmlData);
         //alert(resultStr);
        if ( !ibm.portal.portlet._SafeToExecute ) {
            if ( callbackfn ) {
                var me = this;
                this._queueUp( function () { me.setUserProfile( userProfile, callbackfn ) } );
                return false; 
            }
            else {
                return this._throwInappropriateRequestError( "setUserProfile" );
            }
        }

         this.status = ibm.portal.portlet.PortletWindow.STATUS_UNDEFINED;
         var _url = document.getElementById("com.ibm.wps.web2.portlet.user."+this.windowID).innerHTML;

         var pointer2this = this;
         var pointer2return = null;
         dojo.rawXhrPost({
             url: _url,
             sync: (callbackfn)?false:true,
             postData: dojox.data.dom.innerXML(userProfile.xmlData),
             contentType: "application/xml",
             headers: { "X-IBM-XHR":"true"},
             handleAs: "xml",
             handle: function(response, ioArgs){
	        	 if ( pointer2this.isAuthenticationRequired( ioArgs.xhr, ioArgs.args.handleAs ) ) {
	        		 pointer2this.doAuthentication();
	             } else {  
	        	 	var type = ( response instanceof Error ) ? "error" : "load";
	                  if(type == "load") {
	                      if (callbackfn) {
	                          callbackfn(pointer2this,ibm.portal.portlet.PortletWindow.STATUS_OK,userProfile);
	                      }
	                      else {
	                          pointer2return = {"portletWindow" : pointer2this, 
	                              "status" : ibm.portal.portlet.PortletWindow.STATUS_OK,
	                              "returnObject" : userProfile };
	                      }
	                  } else if(type == "error"){
	                      if (callbackfn) {
	                          callbackfn(pointer2this, ibm.portal.portlet.PortletWindow.STATUS_ERROR, null);
	                      }
	                      else {
	                          pointer2return = {"portletWindow" : pointer2this, 
	                              "status" : ibm.portal.portlet.PortletWindow.STATUS_ERROR,
	                              "returnObject" : null };
	                      }
	                  }
	             } 
             },
             transport: "XMLHTTPTransport"
         });
         return pointer2return; // Object - synchronously: { portletWindow, status, returnObject } / asynchronously: NULL
    },
    newXMLPortletRequest: function() {
        // summary: Creates a new XMLPortletRequest object with the current page ID and portlet window ID.
        // returns: a new XMLPortletRequest
        return new ibm.portal.portlet.XMLPortletRequest( this ); // ibm.portal.portlet.XMLPortletRequest
    }, 
    isAuthenticationRequired: function ( /*XMLHttpRequest*/xhrResponse, /*String*/expectedContentType ) {
    	// summary: Given an HTTP RESPONSE (completed XMLHttpRequest object), determine if the response indicates that authentication
    	//		is required.
    	// xhrResponse: the XMLHttpRequest object whose readyState property is 4
    	// expectedContentType: one of "xml", "json", or "json-comment-optional".
    	// returns: true if authentication is required; false otherwise
    	if ( xhrResponse.readyState != 4 ) {
    		throw new Error( "isAuthenticationRequired should only be called with a COMPLETED XMLHttpRequest! The readyState on the given XMLHttpRequest is not 4 (COMPLETE)!" );
    	}
    	//Have to wrap the pertinent information in dojo-like objects since the Auth API was designed only for Dojo use-cases.
    	var fakeResponseObj = { dojoType: "valid" };
    	var fakeArgsObj = { 
    			xhr: xhrResponse,
    			args: {
    				handleAs: expectedContentType
    			}
    	};
    	
    	return com.ibm.ajax.auth.isAuthenticationRequired( fakeResponseObj, fakeArgsObj );
    },
    setAuthenticationHandler: function ( /*Function*/authenticationFn ) {
		// summary: Set a PortletWindow instance-specific authentication function.
		// authenticationFn: the function to call
		this._authenticationFn = authenticationFn;
	},
	doAuthentication: function () {
		// summary: Call the configured authentication handler. NOTE: This function call will most likely
		//		result in a REDIRECT.
		if ( this._authenticationFn ) {
			this._authenticationFn();
		} else {
			com.ibm.ajax.auth.authenticationHandler();
		}	
	}
});


/**
 * summary:
 * The <CODE>PortletPreferences</CODE> interface allows the portlet to store
 * configuration data. It is not the
 * purpose of this interface to replace general purpose databases.
 * 
 * description:
 * There are two different types of preferences:
 * <ul>
 * <li>modifiable preferences - these preferences can be changed by the
 *     portlet in any standard portlet mode (<code>EDIT, HELP, VIEW</code>).
 *     Per default every preference is modifiable.
 * <li>read-only preferences - these preferences cannot be changed by the
 *     portlet in any standard portlet mode, but may be changed by administrative modes.
 *     Preferences are read-only, if the are defined in the
 *     deployment descriptor with <code>read-only</code> set to <code>true</code>,
 *     or if the portlet container restricts write access.
 * </ul>
 * <p>
 * Changes are persisted when the <code>store</code> method is called.
 */

dojo.declare("ibm.portal.portlet.PortletPreferences", null, {
    constructor: function (/*String*/ windowid, 
                           /*String*/ requestedpreferenceid,
                           /*XMLDocument */ data) {
        /** 
        * summary:
        *     Objects of this type should never be called directly but always created
        *     through the PortletManagers method getPortletPreferences.
        *
        * windowid: The window id
        */

        this.windowID = windowid;
        this.requestedPreferenceID = requestedpreferenceid;
        this.xmlData = data;
        this.xsltURL = dojo.moduleUrl( "ibm", "portal/portlet/");
        
        this.ns = {
            "xsl" : "http://www.w3.org/1999/XSL/Transform",
            "thr" : "http://purl.org/syndication/thread/1.0",
            "atom" : "http://www.w3.org/2005/Atom",
            "xhtml" : "http://www.w3.org/1999/xhtml",
            "model" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0.1/portal-model-elements",
            "base" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0/ibm-portal-composite-base",
            "portal" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0.1/portal-model",
            "xsi" : "http://www.w3.org/2001/XMLSchema-instance"
        };

        // performance optimization
        this.internal_reset();
    },

    getMap: function (){
    /**
     * summary: 
     *     Returns a <code>Map</code> of the preferences.
     *     If no preferences exist this method returns an empty <code>Map</code>.
     */
        //ibm.portal.debug.entry( "PortletPreferences.internal_applyXSLT" , [ feed.toString(), dom, args ] );

        if (this.result_getMap) {
            return this.result_getMap;
        }

        //load the xslt
        var xslDoc = ibm.portal.xml.xslt.loadXsl(this.xsltURL + "PortletPreferencesMap.xsl");

        //apply the xslt
        if (xslDoc.documentElement == null) {alert("xslDoc is null");}
        var results = ibm.portal.xml.xslt.transform(this.xmlData, xslDoc, null, {"selectionid":this.requestedPreferenceID}, true);
        if (results == null) {
            this.result_getNames = null;
            return null;
        }

        var prefs = eval( results );
        // "portletPreferences = { \"preferences\": [ { \"name\":\"parallel\" , \"values\": [ \"false\" ] , \"read-only\": \"false\" }, { \"name\":\"test\" , \"values\": [ \"newValue\" ] , \"read-only\": \"false\" } ] };"

        if (prefs)            
            prefs = prefs.preferences;

        this.result_getMap = prefs;

        return this.result_getMap; /* object[] {name: string, values: string[], readonly: boolean}
     *             a <code>Map</code> containing preference names as 
     *             keys and preference values as map values, or an empty <code>Map</code>
     *             if no preference exist.
     */
    },

    getNames: function (){
    /**
     * summary:
     *     Returns all of the keys that have an associated value,
     *     or an empty <code>Enumeration</code> if no keys are available.
     */
         //ibm.portal.debug.entry( "PortletPreferences.internal_applyXSLT" , [ feed.toString(), dom, args ] );
         
         if (this.result_getNames) {
             return this.result_getNames;
         }
         
         //load the xslt
         var xslDoc = ibm.portal.xml.xslt.loadXsl(this.xsltURL + "PortletPreferencesNames.xsl");

         //apply the xslt
         if (xslDoc.documentElement == null) {alert("xslDoc is null");}
         var results = ibm.portal.xml.xslt.transform(this.xmlData, xslDoc, null, {"selectionid":this.requestedPreferenceID}, true);
         if (results == null) {
             this.result_getNames = null;
             return null;
         }

         var prefsNames = eval( results );
         // "portletPreferences = { \"preferences\": [ { \"name\":\"parallel\" , \"values\": [ \"false\" ] , \"read-only\": \"false\" }, { \"name\":\"test\" , \"values\": [ \"newValue\" ] , \"read-only\": \"false\" } ] };"

         if (prefsNames)
             prefsNames = prefsNames.names;

         this.result_getNames = prefsNames;

         return this.result_getNames; /* String[] - an Enumeration of the keys that have an associated value,
         *         or an empty <code>Enumeration</code> if no keys are
        *         available.
        */
    },

    getValue: function( /*String*/ key, /*String*/ def ){
    /**
     * summary:
     *     Returns the first String value associated with the specified key of this preference.
     * description:
     *     If there is one or more preference values associated with the given key 
     *     it returns the first associated value.
     *     If there are no preference values associated with the given key, or the 
     *     backing preference database is unavailable, it returns the given 
     *     default value.
     *
     * key: String - key for which the associated value is to be returned
     * def: String - the value to be returned in the event that there is no 
     *            value available associated with this <code>key</code>.
     */
         var expr = "/atom:feed/atom:entry[atom:id='"+this.requestedPreferenceID+"']/atom:content/*/model:portletpreferences[@name='"+key+"']/base:value";
         // /atom:feed/atom:entry[atom:id='pm:oid:5_J12SV3D00O6JD02ECR217F00E0@oid:6_J12SV3D00O6JD02ECR217F00M6']/atom:content/model:portlet-entity/model:portletpreferences[@name='parallel']
         
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);         
         var result;
         if (nodes && nodes.length>0) {
             result = nodes[0].getAttribute("value");
         }
         else {
             result = def;
         }

         return result; /* String - the value associated with <code>key</code>, or <code>def</code>
         *         if no value is associated with <code>key</code>, or the backing
         *         store is inaccessible.
         */
    },

    getValues: function( /*String*/ key, /*String[]*/ def ){
    /**
     * summary:
     *     Returns the String array value associated with the specified key in this preference.
     * description:
     *     Returns the specified default if there is no value
     *     associated with the key, or if the backing store is inaccessible.
     *
     *     If the implementation supports <i>stored defaults</i> and such a
     *     default exists and is accessible, it is used in favor of the
     *     specified default.
     *
     * key: String - key for which associated value is to be returned.
     * def: String[] - the value to be returned in the event that this
     *        preference node has no value associated with <code>key</code>
     *        or the associated value cannot be interpreted as a String array,
     *        or the backing store is inaccessible.
     */

         var expr = "/atom:feed/atom:entry[atom:id='"+this.requestedPreferenceID+"']/atom:content/*/model:portletpreferences[@name='"+key+"']/base:value";
         // /atom:feed/atom:entry[atom:id='pm:oid:5_J12SV3D00O6JD02ECR217F00E0@oid:6_J12SV3D00O6JD02ECR217F00M6']/atom:content/model:portlet-entity/model:portletpreferences[@name='parallel']
         
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);
         var result;
         if (nodes && nodes.length>0) {
             result = new Array();
             for (var i=0; i<nodes.length; i++) {
                 result[i] = nodes[i].getAttribute("value");
             }
         }
         else {
             result = def;
         }
        
         return result; /* String[] - the String array value associated with
         *         <code>key</code>, or <code>def</code> if the
         *         associated value does not exist.
         */
    },

    isReadOnly: function( /*String*/ key ){
    /**
     * summary:
     *     Returns true, if the value of this key cannot be modified by the user.
     * description:
     *     Modifiable preferences can be changed by the
     *     portlet in any standard portlet mode (<code>EDIT, HELP, VIEW</code>). 
     *     Per default every preference is modifiable.
     * 
     *     Read-only preferences cannot be changed by the
     *     portlet in any standard portlet mode, but inside of custom modes
     *     it may be allowed changing them.
     *     Preferences are read-only, if they are defined in the 
     *     deployment descriptor with <code>read-only</code> set to <code>true</code>,
     *     or if the portlet container restricts write access.
     *
     * key: String - key for which the associated value is to be returned
     */

         var id = this.requestedPreferenceID;
         var expr = "/atom:feed/atom:entry[atom:id='"+id+"']/atom:content/*/model:portletpreferences[@name='"+key+"']";
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);         
         
         var result = false;
         if (nodes && nodes.length>0) {
             var temp = nodes[0].getAttribute("read-only");
             if (temp != null) {
                 // convert to a boolean
                 if ( temp == "true" ) {
                    result = true;
                 }
             }
         }
         return result; // boolean - false, if the value of this key can be changed, or if the key is not known
    },

    reset: function ( /*String*/ key ){
    /**
     * summary:
     *     Resets or removes the value associated with the specified key.
     * description:
     *     If this implementation supports stored defaults, and there is such
     *     a default for the specified preference, the given key will be 
     *     reset to the stored default.
     *
     *     If there is no default available the key will be removed.
     *
     * key: String - key to reset
     */
         this.internal_reset();

         var expr = "/atom:feed/atom:entry[atom:id='"+this.requestedPreferenceID+"']/atom:content/*/model:portletpreferences[@name='"+key+"']";
         
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);
         if (nodes && nodes.length>0) {
             var parent = nodes[0].parentNode;
             parent.removeChild(nodes[0]);
         }

    },

    setValue: function ( /*String*/ key, /*String*/ value ){
    /**
     * summary:
     *     Associates the specified String value with the specified key in this preference.
     * description:
     *     The key cannot be <code>null</code>, but <code>null</code> values
     *     for the value parameter are allowed.
     *
     * key: String - key with which the specified value is to be associated.
     * value: String - value to be associated with the specified key.
     */
         var values = new Array();
         values[0] = value;
         this.setValues(key,values);
    },

    setValues: function( /*String*/ key, /*String[]*/ values ){
    /**
     * summary:
     *     Associates the specified String array value with the specified key in this preference.
     * description:
     *     The key cannot be <code>null</code>, but <code>null</code> values
     *     in the values parameter are allowed.
     *
     * key: String - key with which the  value is to be associated
     * values: String[] - values to be associated with key
     */
         this.internal_reset();

         var expr = "/atom:feed/atom:entry[atom:id='"+this.requestedPreferenceID+"']/atom:content/*/model:portletpreferences[@name='"+key+"']";
         
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);

         var prefNode = null;
         if (nodes && nodes.length>0) {
             prefNode = nodes[0];
             for (var i=prefNode.childNodes.length-1; i>=0; i--) {
                 prefNode.removeChild(prefNode.childNodes[i]);
             }
         }
         else {
             var expr2 = "/atom:feed/atom:entry[atom:id='"+this.requestedPreferenceID+"']/atom:content/*";
             var nodes2 = ibm.portal.xml.xpath.evaluateXPath(expr2, this.xmlData, this.ns);
             if (dojo.isIE) {
                 prefNode = this.xmlData.createNode(1, "model:portletpreferences", this.ns.model); // 1 = NODE_ELEMENT
             }
             else {
                 prefNode = this.xmlData.createElementNS(this.ns.model,"model:portletpreferences");
             }
             prefNode.setAttribute("name",key);
             prefNode.setAttribute("read-only","false");
             nodes2[0].appendChild(prefNode);
         }
         for (var i=0; i<values.length; i++) {
             var newNode;
             if (dojo.isIE) {
                 newNode = this.xmlData.createNode(1, "base:value", this.ns.base); // 1 = NODE_ELEMENT
                 var attribute = this.xmlData.createNode(2, "xsi:type",this.ns.xsi ); // 1 = NODE_ATTRIBUTE
                 attribute.nodeValue = "String";
                 newNode.setAttributeNode(attribute);
             }
             else {
                 newNode = this.xmlData.createElementNS(this.ns.base,"base:value");
                 newNode.setAttributeNS(this.ns.xsi,"xsi:type","String");
             }
             newNode.setAttribute("value",values[i]);
             prefNode.appendChild(newNode);
         }
    },

    internal_reset: function() {
         this.result_getMap = null;
         this.result_getNames = null;
    },

    clone:function()
    {
        var resultStr = dojox.data.dom.innerXML(this.xmlData);
        var xmlDataCopy = dojox.data.dom.createDocument( resultStr ); 

        return new ibm.portal.portlet.PortletPreferences(this.windowID, 
                                                             this.requestedPreferenceID,
                                                             xmlDataCopy);
    }


});















dojo.declare("ibm.portal.portlet.PortletMode", null, {
    VIEW: "view",
    EDIT: "edit",
    EDIT_DEFAULTS: "edit_defaults",
    HELP: "help",
    CONFIG: "config"    
});

dojo.declare("ibm.portal.portlet.WindowState", null, {
    NORMAL: "normal",
    MINIMIZED: "minimized",
    MAXIMIZED: "maximized"
});

dojo.declare("ibm.portal.portlet.PortletState", null, {
    constructor: function (/*String or XMLDocument*/ windowid, /*String?*/portletWindowId ) {
        /**
         * summary:
         *     Objects of this type should never be called directly but always created
         *     through the PortletManagers method getPortletState.
         *
         * windowid: The window id
         */
        var stateManager = new com.ibm.portal.state.StateManager( ibmPortalConfig["contentHandlerURI"] );
        if ( dojo.isString( windowid ) ) {
            var stateDOM = this._getExistingState( windowid, stateManager.getSerializationManager() );
            stateManager.reset( stateDOM );
        }
        else {
            stateManager.reset( windowid );
            windowid = portletWindowId; 
        }
        
        // get a state accessor that allows for reading and writing portlet state
        this.portletAccessor = stateManager.newPortletAccessor(windowid);
        // also get the Parameters object for performance reasons
        this.renderParameters = this.portletAccessor.getRenderParameters();
    },
    _isCSA: function () {
        var retVal = false;
        try {
            retVal = (typeof(document.isCSA )!= "undefined" );
        }
        catch (e) {
            //do nothing here... something was not defined so we aren't in CSA
        }
        
        return retVal;
    },
    _getExistingState: function ( /*String*/windowid, serializationMgr ) {
        var retVal = null;
        if ( this._isCSA() ) {
            // CSA has access to the current state
            retVal = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState().stateDOM;
        } else if (serializationMgr != null) {
            // get the current state from the server
            var result = serializationMgr.deserialize( location.href );
            retVal = result.returnObject;
        } else {
            // fall back to an "empty" state
            retVal = dojox.data.dom.createDocument();
        }
        return retVal;
    },
    getPortletMode:function() {
    /**
     * summary:
     *     Returns the portlet mode.
     */
        return this.portletAccessor.getPortletMode(); /*ibm.portal.portlet.PortletMode*/
    },

    setPortletMode:function(/*ibm.portal.portlet.PortletMode*/ portletMode) {
    /**
     * summary:
     *     Sets the portlet mode.
     */
        this.portletAccessor.setPortletMode(portletMode);
        return portletMode;
    },

    getWindowState:function() {
    /**
     * summary:
     *     Returns the window state.
     */
        return this.portletAccessor.getWindowState(); /*ibm.portal.portlet.WindowState*/
    },
    
    setWindowState:function(/*ibm.portal.portlet.WindowState*/windowState) {
    /**
     * summary:
     *     Sets the window state.
     */
        this.portletAccessor.setWindowState(windowState);
        return windowState;
    },

    getParameterNames: function() {
    /**
     * summary:
     *     Returns a String[] containing the names of all parameters that have an
     *     associated value. Returns an empty array if no parameters are available.
     */
        return this.renderParameters.getNames(); // String[]
    },

    getParameterValue: function(/*String*/ name) {
    /**
     * summary:
     *     Returns the String value associated with the render parameter with the given name.
     *     
     * description:
     *     If there are one or more render parameter values associated with the given name 
     *     it returns the first associated value.
     *
     * name: String - Name of the render parameter for which the associated value is to
     *     be returned. Must not be NULL.
     */
        return this.renderParameters.getValue(name); // String
    },

    getParameterValues: function(/*String*/ name) {
    /**
     * summary:
     *     Returns the String[] representing the values that are associated with
     *     the render parameter with the given name.
     *
     * name: String - Name of the render parameter for which the associated values
     *     are to be returned. Must not be NULL.
     */
        return this.renderParameters.getValues(name); // String[]
    },

    getParameterMap: function() {
    /**
     * summary:
     *     Returns an unmodfiable map of render parameters.
     */
        return this.renderParameters.getMap(); /* object[] {name: string, values: string[]} */
    },

    setParameterValue: function(/*String*/ name, /*String*/ value) {
    /**
     * summary:
     *     Sets a simple String value for the render parameter with the given name. Overrides
     *     the value of the render parameter if it already exists.
     *
     * name: String - The name of the render parameter. Must not be NULL.
     * value: String - The value of the render parameter that should be set. Must not be NULL. 
     */
        this.renderParameters.setValue(name, value);
        return value; // String
    },
    
    setParameterValues: function(/*String*/ name, /*String[]*/ values) {
    /**
     * summary:
     *     Sets a String[] representing the value of the render parameter with the given name.
     *     Overrides the value of the render parameter if it already exists.
     *
     * name: String - The name of the render parameter. Must not be NULL.
     * value: String[] - The value of the render parameter that should be set. Must not be NULL. 
     */
        this.renderParameters.setValues(name, values);
        return values; // String[]
    },
    
    setParameterMap:function(/*object[] {name: string, values: string[]}*/map, /*boolean*/replace) {
    /**
     * summary:
     *     Adds all render parameters that are contained in the given map. Overrides the values
     *     of parameters that already exist. Returns the resulting map.
     *
     * map: Object[] {name: String, values: String[]} - Two dimensional object array
     *     representing the map of parameters that should be added. Must not be NULL.
     * replace: boolean - Optional argument that indicates whether the given map should
     *     completely replace the existing parameter map.
     */
        // check if we need to delete the existing parameters
        if (replace == true) {
            this.renderParameters.clear();
        }
        this.renderParameters.putAll(map);
        return this.renderParameters.getMap(); /* object[] {name: string, values: string[]} */
    },
    
    removeParameter:function(/*String*/name) {
    /**
     * summary:
     *     Removes the render parameter with the given name.
     *
     * name: String - Name of the render parameter that should be removed. Must not be NULL.
     */
        this.renderParameters.remove(name);
    }
    
});







dojo.require( "com.ibm.portal.services.PortletFragmentService" );

dojo.declare("ibm.portal.portlet.XMLPortletRequest", null, {
     // summary: Provides XMLHttpRequest functionality for addressing portlets.
     // description: Wrappers and exposes all standard properties on an XMLHttpRequest, as well
     //         as, exposing an additional property called onportletstateready. This new property
     //         can be assigned a function that will be called when the portlet state is ready. If
     //         a handler is set for onportletstateready, the portlet state will not be updated 
     //         globally. The XMLPortletRequest will only work for URLs which point to the portlet
     //         fragment feed.
     // 
     // onportletstateready: Function - handler for intercepting global portlet state updates, the 
     //         PortletState object is the lone parameter passed into this function when called.
     // onreadystatechange: Function - handler for the readyState property changing
     // readyState: int - indicates the current state of the request
     // responseText: String - the text of the response body (only available when readyState == 4)
     // responseXML: XMLDocument - the xml document in the response body
     // status: int - the HTTP status code received in the response
     // statusText: String - the String corresponding to the HTTP status code
     // 
    onreadystatechange: null,     /*Function*/
    readyState: 0,            /* int */
    responseText: null,           /* String */
    responseXML: null,            /* Document */
    status: null,                 /* int */
    statusText: null,             /* String */
    onportletstateready: null,     /* Function(ibm.portal.portlet.PortletState) */
    _location: null,              /* String - stores the location of the request */
    _async: null,				  /* boolean - stores whether or not the request is asynchronous */
    
    constructor: function ( /*PortletWindow*/pWindow ) {
        // summary: Constructor for the XMLPortletRequest object. 
        // description: A correctly initialized instance of this object can be obtained from the portlet window object.  The only
        //      case where this constructor should be called directly is when sending an XPR to a different portlet than the
        //      one represented by the portletWindow object. 
        // page: the page's ObjectID
        // window: the porltet window's ObjectID
		var _method = this.declaredClass + ".constructor";
		ibm.portal.debug.entry( _method, [ pWindow ] )
		this.pageID = pWindow.pageID;
        this.windowID = pWindow.windowID;
        this.windowObj = pWindow;
        ibm.portal.debug.exit( _method );
    },
    _getXHR: function () {
         // summary: Creates the XMLHttpRequest object if necessary and returns it
         // returns: the XMLHttpRequest instance associated with this XMLPortletRequest
        var _method = this.declaredClass + "._getXHR";
        ibm.portal.debug.entry( _method );
    	if ( !this._xhr ) { 
            this._xhr = this._createXHR();
        }
        retVal.setRequestHeader("X-IBM-XHR","true");
    	ibm.portal.debug.exit( _method, this._xhr );
        return this._xhr; //XMLHttpRequest
    },
    _createXHR: function () {
        // summary: Cross-browser creation of XMLHttpRequest or it's equivalent.
        // returns: a new instance of an XMLHttpRequest or it's equivalent
    	var _method = this.declaredClass + "._createXHR";
        ibm.portal.debug.entry( _method );
    	var retVal = null;
        
        if ( typeof( XMLHttpRequest ) != "undefined" ) {
            retVal = new XMLHttpRequest();  
        }
        else {
            retVal = new ActiveXObject( "Microsoft.XMLHTTP" );
        }
        
        ibm.portal.debug.exit( _method, retVal );
        return retVal; //XMLHttpRequest
    },
    _onreadystatechangehandler: function () {
        // summary: Wrappers any user-set onreadystatechange handlers and processes the 
        //      response from the Portlet Fragment feed.
        // description: Intercepts the underlying XMLHttpRequest readyState changes and
        //      extracts the necessary information from the Portlet Fragment feed to 
        //      make the XMLPortletRequest object ready for consumption by the developer.
    	var _method = this.declaredClass + "._onreadystatechangehandler";
        ibm.portal.debug.entry( _method );
        
        if ( !this.handled ) {
        
	        var xhr = this._getXHR();
	        
	        //Copy over common properties that may have changed.
	        this.readyState = xhr.readyState;
	        
	        ibm.portal.debug.text( "ready state is " + xhr.readyState );
	        
	        //Extract the response portlet state and any dependent portlets. We check for the handled
	        //flag to ensure that we only process the response once. This is in case the behavior in 
	        //Mozilla changes for synchronous requests (In 2.x and 3.0.x (and 3.5?) the onreadystatechange handler is
	        //never fired for synchronous requests - so we have to call it manually - if the manual step
	        //is made unnecessary by the browser correcting the behavior, we don't want to process the result
	        //twice).
	        if ( this.readyState == 4 ) {
	        	var authRequired = this.windowObj.isAuthenticationRequired( xhr, "xml" );
	        	
	        	ibm.portal.debug.text( "is auth required: " + authRequired );
	        	
	        	if ( authRequired ) {
	        		this.windowObj.doAuthentication( xhr );
	        		return; 
	        	} else {
		            //Can't copy these common properties over until readyState is 4 because of IE
		            this.responseText = xhr.responseText;
		            this.responseXML = xhr.responseXML;
		            this.status = xhr.status;
		            this.statusText = xhr.statusText;
		            //Use our service object for convenience here. We're just using to parse the
		            //ATOM Feed response, not for actually retrieving any data because we need
		            //direct access to the XMLHttpRequest object in this use case.
		            var service = new com.ibm.portal.services.PortletFragmentService();
		            var portletInfo = service.createPortletInfo( xhr.responseXML );
		            this.responseText = portletInfo.markup;
		            //Will need to set this based on the atom:content type attribute, but since we
		            //aren't supporting any other markup types but HTML out of the box, this should
		            //never be set.
		            this.responseXML = null;
		            
		            var doGlobalStateUpdate = true;
		            var xmlState = portletInfo.updatedState;
		            
		            if ( this.onportletstateready != null ) {
		            	var windowId = portletInfo.windowId;
		                var portletState = new ibm.portal.portlet.PortletState( xmlState, windowId );
		                
		                //allow developer to handle portlet state
		                doGlobalStateUpdate = this.onportletstateready( portletState );
		            }
		            
		            if ( doGlobalStateUpdate && this._isCSA() )
		            {
		                //update global state
		                service._fireGlobalPortletStateChange( portletInfo );
		            } 
		            
		            this._handleDependentPortlets( service.readDependentPortlets( xhr.responseXML ), xmlState );
		            this.handled = true;
	        	}     
	            
	        }
	        
	        //Finally call the user's onreadystate change handler.
	        if ( this.onreadystatechange != null ) {
	            this.onreadystatechange();
	        }
        }
        
        ibm.portal.debug.exit( _method );
    },
    _handleDependentPortlets: function ( /*Array*/portlets, xmlState ) {
        // summary: Trigger reloads for all dependent portlets.
        // description: Reloads any portlets whose content has changed as a result of
        //      this XMLPortletRequest. This generally affects C2A portlets. If we are 
        //      not in Client-Side Aggregation, this will trigger a full page refresh.
    	var _method = this.declaredClass + "._handleDependentPortlets";
        ibm.portal.debug.entry( _method, [ portlets, xmlState ] );
    	
        if ( !this._isCSA() ) {
            if ( portlets.length > 0 ) {
                //Until we have Differential Page Rendering on the server side, we have no 
                //choice but to do a full page refresh, but we must point to the page
                //so the action isn't submitted twice and all portlets are rendered with fresh
                //parameters 
                
                window.location.href = this._newPageURL( xmlState );
            }   
        }
    	
    	ibm.portal.debug.exit( _method );
    },
    _isCSA: function () {
    	var _method = this.declaredClass + "._isCSA";
        ibm.portal.debug.entry( _method );
        
    	var retVal = false;
        try {
            retVal = (typeof(document.isCSA )!= "undefined" );
        }
        catch (e) {
            //do nothing here... something was not defined so we aren't in CSA
        }
        
        ibm.portal.debug.exit( _method, retVal );
        return retVal;
    },
    _flag: function ( /*String*/ uriToFlag ) {
        // summary: Flags a URL with the appropriate url parameters to retrieve a portlet fragment.
    	var _method = this.declaredClass + "._flag";
        ibm.portal.debug.entry( _method, [ uriToFlag ] );
        
    	var id = "lm:oid:" + this.windowID + "@oid:" + this.pageID;
        var service = new com.ibm.portal.services.PortletFragmentService();
        var url = service._flagPortletUrl( uriToFlag, id );
        
        ibm.portal.debug.exit( _method, url );
        return url;
    },
    _newPageURL: function ( xmlState ) {
        // summary: Creates a new page url based on the portlet window id and page id.
        // description: Creates a new page url to the page containing the portlet that 
        // originated this XMLPortletRequest object.
    	var _method = this.declaredClass + "._newPageURL";
        ibm.portal.debug.entry( _method, [ xmlState ] );
    	
        ibm.portal.debug.text( dojox.data.dom.innerXML( xmlState ) );
        
        // get a copy of the state which contains the state of our portlet only    
        var stateManager = new com.ibm.portal.state.StateManager(ibmPortalConfig["contentHandlerURI"]);
        var stateDOM = xmlState;
        if ( !xmlState ) {
        	stateDOM = dojox.data.dom.createDocument();
        }
        stateManager.reset( stateDOM );
            
        //var portletState = stateManager.newPortletAccessor( this.windowID ).getPortletState();
        
        //var selectionAcc = stateManager.newSelectionAccessor(portletState);
        //selectionAcc.setPageSelection(this.pageID);
        
        // serialize the state into a portal URL
        var serializationMgr = stateManager.getSerializationManager();
        var serializationResult = serializationMgr.serialize(stateDOM);
        
        // return the URL
        var serializedState = serializationResult["returnObject"];
        
        var url = serializedState; 
        
        ibm.portal.debug.exit( _method, url );
        return url; // String
    },
    open: function( /*String*/ method, /*String*/ uri ) {
        // @see XMLHttpRequest
    	var _method = this.declaredClass + ".open";
        ibm.portal.debug.entry( _method, [ method, uri ] );
    	
    	this.open( method, uri, false ); 
    	
    	ibm.portal.debug.exit( _method );
    },
    open: function( /*String*/ method, /*String*/ uri,  /*boolean*/ async ) {
        // @see XMLHttpRequest
    	var _method = this.declaredClass + ".open";
        ibm.portal.debug.entry( _method, [ method, uri, async ] );
    	
    	var xhr = this._getXHR();
        var me = this;
        this._location = uri;
        if ( async == undefined ) {
        	async = false;
        }
        this._async = async;
        xhr.onreadystatechange = function () { me._onreadystatechangehandler(); };
        xhr.open( method, this._flag(uri), async );
        
        ibm.portal.debug.exit( _method );
    },
    setRequestHeader: function( /*String*/ header, /*String*/ value ) {
        // @see XMLHttpRequest
    	var _method = this.declaredClass + ".setRequestHeader";
        ibm.portal.debug.entry( _method, [ header, value ] );
    	
    	this._getXHR().setRequestHeader( header, value );
    	
    	ibm.portal.debug.exit( _method );
    },
    send: function( /*String or Document*/ data ) {
        // @see XMLHttpRequest
    	var _method = this.declaredClass + ".send";
        ibm.portal.debug.entry( _method, [ data ] );
    	
    	this._getXHR().send( data );   
    	
    	//Mozilla handles synchronous XHRs differently - the readystatechange event is never dispatched. Call it manually,
    	//but check in the handler if this has already been called. The reason for using this solution is that once Mozilla
    	//fixes the behavior, we won't have to make any code changes to take advantage.
    	if ( !this._async ) {
    		this._onreadystatechangehandler();
    	}
    	
    	ibm.portal.debug.exit( _method );
    },
    abort: function( ) {
        // @see XMLHttpRequest
    	var _method = this.declaredClass + ".abort";
        ibm.portal.debug.entry( _method );
    	
    	this._getXHR().abort();
    	
    	ibm.portal.debug.exit( _method );
    },
    getAllResponseHeaders: function( ) {
        // @see XMLHttpRequest
        return this._getXHR().getAllResponseHeaders();
    },
    getResponseHeader: function( /*String*/ header ) {
        // @see XMLHttpRequest
        return this._getXHR().getResponseHeader( header );
    }

});

















dojo.declare("ibm.portal.portlet.UserProfile", null, {

    constructor: function (/*String*/ windowid,
                           /*XMLDocument */ data) {
    /**
     * summary:
     *     Objects of this type should never be called directly but always created
     *     through the PortletWindows method getUser.
     *
     * windowid: The window id
     */
        this.windowID = windowid;
        this.xmlData = data;

        this.ns = {
            "xsl" : "http://www.w3.org/1999/XSL/Transform",
            "atom" : "http://www.w3.org/2005/Atom",
            "xhtml" : "http://www.w3.org/1999/xhtml",
            "xsi" : "http://www.w3.org/2001/XMLSchema-instance",
            "um" : "http://www.ibm.com/xmlns/prod/websphere/um.xsd"
        };
    },

    getAttribute:function(/*String*/ name) {
    /**
     * summary:
     *     Returns the value of the given user attribute name as defined by the P3P Spec.
     * description:
     *     The portlet must define the user information attribute it is interested in
     *     inside the user-attribute section of the deployment descriptor. 
     *     If an attribute is not supported by the current runtime system or if it is supported
     *     but not defined for a particular user, then this method will return null
     * 
     * name: String - the name of the P3P attribute to return
     */
         var expr = "/atom:entry/atom:content/um:profile[@type='user']/um:attribute[@name='"+name+"']/um:attributeValue";
         //var expr = "/um:profile[@type='user']/um:attribute[@name='"+name+"']";
         
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);         

         var result = null;
         if (nodes && nodes.length>0) {
             if (nodes[0].textContent) {
                 result = nodes[0].textContent;
             }
             else {
                 result = nodes[0].text;
             }
         }

         return result; /* String */

    },

    setAttribute:function(/*String*/ name, /*String*/ value) {
         var expr = "/atom:entry/atom:content/um:profile[@type='user']/um:attribute[@name='"+name+"']/um:attributeValue";
         //var expr = "/um:profile[@type='user']/um:attribute[@name='"+name+"']";
         
         var nodes = ibm.portal.xml.xpath.evaluateXPath(expr, this.xmlData, this.ns);         
         var previousText = null;
         if (nodes && nodes.length>0) {
             if (nodes[0].textContent) {
                 previousText = nodes[0].textContent;
                 nodes[0].textContent = value;
             }
             else {
                 previousText = nodes[0].text;
                 nodes[0].text = value;
             }
         }
         else {
             // TODO             
             var expr1 = "/atom:entry/atom:content/um:profile[@type='user']/um:attribute[@name='"+name+"']";
             var nodes1 = ibm.portal.xml.xpath.evaluateXPath(expr1, this.xmlData, this.ns);
             var attrNode = null;
             if (nodes1 && nodes1.length>0) {
                 attrNode = nodes1[0];
             }
             else {
                 var expr2 = "/atom:entry/atom:content/um:profile[@type='user']";
                 var nodes2 = ibm.portal.xml.xpath.evaluateXPath(expr2, this.xmlData, this.ns);
                 
                 if (dojo.isIE) {
                     attrNode = this.xmlData.createNode(1, "um:attribute", this.ns.um); // 1 = NODE_ELEMENT
                 }
                 else {
                     attrNode = this.xmlData.createElementNS(this.ns.um, "um:attribute");
                 }
                 attrNode.setAttribute("type","xs:string");
                 attrNode.setAttribute("multiValued","false");
                 attrNode.setAttribute("name",name);
                 nodes2[0].appendChild(attrNode);
             }
             var newNode;
             if (dojo.isIE) {
                 newNode = this.xmlData.createNode(1, "um:attributeValue", this.ns.um); // 1 = NODE_ELEMENT
                 newNode.text = value;
             }
             else {
                 newNode = this.xmlData.createElementNS(this.ns.um, "um:attributeValue");
                 newNode.textContent = value;
             }
             attrNode.appendChild(newNode);
         }

         return previousText;
    },

    clone:function()
    {
        var resultStr = dojox.data.dom.innerXML(this.xmlData);
        var xmlDataCopy = dojox.data.dom.createDocument( resultStr ); 

        return new ibm.portal.portlet.UserProfile(this.windowID,
                                                      xmlDataCopy);
    }
    
});







dojo.declare("ibm.portal.portlet.Error", null, {

    INFO: 0,
    WARN: 1,
    ERROR: 2,
    
    constructor: function (/*String*/ errorCode, /*String*/ message, /*String*/ description) {
        this.errorCode = errorCode;
        this.message = message;
        this.description = description;
    },
    
    getErrorCode: function() {
    /**
     * summary:
     *     Returns the error code
     */
        return this.errorCode;
    },
    
    getMessage: function() {
    /**
     * summary:
     *     Returns the message
     */
        return this.message;
    },
    
    getDescription: function() {
    /**
     * summary:
     *     Returns the description
     */
        return this.description;
    }

});


{ // create static initializers
var com_ibm_portal_portlet_portletwindow = new ibm.portal.portlet.PortletWindow();
ibm.portal.portlet.PortletWindow.STATUS_UNDEFINED = com_ibm_portal_portlet_portletwindow.STATUS_UNDEFINED;
ibm.portal.portlet.PortletWindow.STATUS_OK = com_ibm_portal_portlet_portletwindow.STATUS_OK;
ibm.portal.portlet.PortletWindow.STATUS_ERROR = com_ibm_portal_portlet_portletwindow.STATUS_ERROR;
com_ibm_portal_portlet_portletwindow = null;
var com_ibm_portal_portlet_portletmode = new ibm.portal.portlet.PortletMode();
ibm.portal.portlet.PortletMode.VIEW = com_ibm_portal_portlet_portletmode.VIEW;
ibm.portal.portlet.PortletMode.EDIT = com_ibm_portal_portlet_portletmode.EDIT;
ibm.portal.portlet.PortletMode.EDIT_DEFAULTS = com_ibm_portal_portlet_portletmode.EDIT_DEFAULTS;
ibm.portal.portlet.PortletMode.HELP = com_ibm_portal_portlet_portletmode.HELP;
ibm.portal.portlet.PortletMode.CONFIG = com_ibm_portal_portlet_portletmode.CONFIG;
com_ibm_portal_portlet_portletmode = null;
var com_ibm_portal_portlet_windowstate = new ibm.portal.portlet.WindowState();
ibm.portal.portlet.WindowState.NORMAL = com_ibm_portal_portlet_windowstate.NORMAL;
ibm.portal.portlet.WindowState.MINIMIZED = com_ibm_portal_portlet_windowstate.MINIMIZED;
ibm.portal.portlet.WindowState.MAXIMIZED = com_ibm_portal_portlet_windowstate.MAXIMIZED;
com_ibm_portal_portlet_windowstate = null;
var com_ibm_portal_portlet_error = new ibm.portal.portlet.Error();
ibm.portal.portlet.Error.INFO = com_ibm_portal_portlet_error.INFO;
ibm.portal.portlet.Error.WARN = com_ibm_portal_portlet_error.WARN;
ibm.portal.portlet.Error.ERROR = com_ibm_portal_portlet_error.ERROR;
com_ibm_portal_portlet_error = null;
}


}

if(!dojo._hasResource["com.ibm.portal.state"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.state"] = true;
dojo.provide("com.ibm.portal.state");








dojo.declare("com.ibm.portal.state.StateManager", null, {

    /**
     * summary:
     *     Constructor which initializes our internal properties.
     */
    constructor: function(/*String*/ contentHandlerURL) {
        this.stateDOM = null;
        this.stateNode = null;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
        this.serializationManager = new com.ibm.portal.state.SerializationManager(contentHandlerURL);
    },

    /**
     * summary:
     *     Gets the current navigational state that can be read or modified
     *     using the offered state accessors.
     */
    getState: function() {
        return this.stateDOM; // dojo.dom.DOMDocument - DOM representing the navigational state
    },
	
    /**
     * summary:
     *     Creates a new state DOM. If a state is provided, this method will create a
     *     deep copy of the given state. Otherwise a new empty state DOM is created.
     *     
     * state: dojo.dom.DOMDocument - State that should be cloned. May be NULL.
     * xslDoc: dojo.dom.DOMDocument - XSL transformation which should be applied to the state copy. May be NULL.
     * xslParams: Map - Map of XSL parameters. May be NULL.
     */
    newState: function(/*dojo.dom.DOMDocument*/ state, /*dojo.dom.DOMDocument*/ xslDoc, /*Map*/ xslParams) {
        var resultState = null;
        // just create a new document if state is null
        if (state == null) {
            resultState = dojox.data.dom.createDocument();
        } else {
            // just clone the given state if no XSL transformation has been passed in
            if (xslDoc == null) {
                resultState = dojox.data.dom.createDocument(dojox.data.dom.innerXML(state));
            } else {
                // apply transformation; pass in XSL parameters
                var xslt = com.ibm.portal.xslt;
                var results = xslt.transform(state, xslDoc, null, xslParams, true);
                resultState = dojox.data.dom.createDocument(results);
            }
        }
        return resultState; /*dojo.dom.DOMDocument*/
    },
    
    /**
     * summary:
     *     Sets the current navigational state that can be read or modified
     *     using the offered state accessors.
     *     
     * state: dojo.dom.DOMDocument - the navigational state represented as a DOM.
     */
    reset: function(/*dojo.dom.DOMDocument*/ stateDOM) {        
        // remember the state
        this.stateDOM = stateDOM;
        // make sure that at least the "<state type='navigational'>" element exists
        this.stateNode = this._getStateNode(stateDOM);         
    },
	
    /**
     * summary:
     *     Returns a SerializationManager which cam be used to serialize /deserialize
     *     state. 
     */
    getSerializationManager: function() {
        return this.serializationManager;
    },
    
    /**
     * summary:
     *     Gets a state accessor that allows for reading and writing navigational
     *     state of a portlet. This includes portlet mode, window state, and render
     *     parameters.
     *     
     * portletWindowID: String - Unique identifier of the portlet window. Must not
     *     be NULL. 
     * state: dojo.dom.DOMDocument - the navigational state the requested portlet
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used.  
     */
    newPortletAccessor: function(/*String*/ portletWindowID, /*dojo.dom.DOMDocument*/ state) {
        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }
        
        // get the portlet node
        var expr = "state:portlet[@id='" + portletWindowID + "']";
        var portletNode = this._getSpecificStateNode("portlet", expr, stNode, stDOM);
        portletNode.setAttribute("id", portletWindowID);
        
        return new com.ibm.portal.state.PortletAccessor(portletNode, stDOM);
        // com.ibm.portal.state.PortletAccessor - Accessor to read and write portlet-specific state.
    },
	
    /**
     * summary:
     *     Gets a state accessor that allows for reading the set of portlet IDs that are
     *     part of the state DOM.
     *     
     * state: dojo.dom.DOMDocument - the navigational state the requested portlet
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used.  
     */
    newPortletListAccessor: function(/*dojo.dom.DOMDocument*/ state) {
        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }

        return new com.ibm.portal.state.PortletListAccessor(stNode, stDOM);
        // com.ibm.portal.state.PortletListAccessor - Accessor to read portlet IDs.
    },    
    
    /**
     * summary:
     *     Gets a state accessor that allows for reading and writing page selection
     *     state information. This includes the currently selected page as well as label
     *     mappings.
     *     
     * state: dojo.dom.DOMDocument - the navigational state the requested selection
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used. 
     */
    newSelectionAccessor: function(/*dojo.dom.DOMDocument*/ state) {

        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }

        // get the selection node
        var selectionNode = this._getSpecificStateNode("selection", "state:selection", stNode, stDOM);

        return new com.ibm.portal.state.SelectionAccessor(selectionNode, stDOM);
        // com.ibm.portal.state.SelectionAccessor - Accessor to read and write page selection state.
    },
    
    /**
     * summary:
     *     Gets a state accessor that allows to check whether or not a portlet window
     *     is in solo state or to set a portlet window to solo state.
     *     
     * state: dojo.dom.DOMDocument - the navigational state the requested solo state
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used. 
     */
    newSoloStateAccessor: function(/*dojo.dom.DOMDocument*/ state) {

        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }

        // get the node keeping the solo state information
        var soloNode = this._getSpecificStateNode("solo", "state:solo", stNode, stDOM);

        return new com.ibm.portal.state.SoloStateAccessor(soloNode, stDOM);
        // com.ibm.portal.state.SoloStateAccessor - Accessor to read and write solo state.
    },
    /**
     * summary:
     *     Gets a state accessor that allows to check whether or not a theme-template
     * 	   has been specified and, if not, set one.
     *     
     * state: dojo.dom.DOMDocument - the navigational state the requested theme-template
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used. 
     */
    newThemeTemplateAccessor: function(/*dojo.dom.DOMDocument*/ state) {

        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }

        // get the node keeping the theme-template information
        var templateNode = this._getSpecificStateNode("theme-template", "state:theme-template", stNode, stDOM);

        return new com.ibm.portal.state.ThemeTemplateAccessor(templateNode, stDOM);
        // com.ibm.portal.state.ThemeTemplateAccessor - Accessor to read and write theme-template state.
    },
    
    /**
     * summary:
     *     Gets a state accessor that allows to check whether or not a locale
     * 	   has been specified and, if not, set one.
     *     
     * state: dojo.dom.DOMDocument - the navigational state the requested locale
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used. 
     */
    newLocaleAccessor: function(/*dojo.dom.DOMDocument*/ state) {

        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }

        // get the node keeping the locale information
        var localeNode = this._getSpecificStateNode("locale", "state:locale", stNode, stDOM);

        return new com.ibm.portal.state.LocaleAccessor(localeNode, stDOM);
        // com.ibm.portal.state.LocaleAccessor - Accessor to read and write locale information.
    },

    /**
     * summary:
     *     Returns a state accessor that allows to include a state partition id into the state.
     *     
     * state: dojo.dom.DOMDocument - the navigational state the requested state partition
     *     accessor should operate on. Pass in NULL to indicate that the internal state
     *     of the state manager should be used. 
     */
    newStatePartitionAccessor: function(/*dojo.dom.DOMDocument*/ state) {

        var stNode;
        var stDOM;
        if (state == null || this.stateDOM == state) {
            stNode = this.stateNode;
            stDOM = this.stateDOM;
        } else {
            stNode = this._getStateNode(state);
            stDOM = state;
        }

        // get the node keeping the state partition information
        var partitionNode = this._getSpecificStateNode("statepartition", "state:statepartition", stNode, stDOM);

        return new com.ibm.portal.state.StatePartitionAccessor(partitionNode, stDOM);
        // com.ibm.portal.state.StatePartitionAccessor - Accessor to set a state partition.
    },    

    /**
     * summary:
     *     Internal method that resolves the <state type='navigational' /> node which is
     *     the root node of navigational state information. If not already present, this
     *     method will create the state node.
     *     
     * aStateDOM: dojo.dom.DOMDocument - the navigational state document that should be inspected.
     *    Must not be NULL. 
     */
    _getStateNode: function(aStateDOM) {
        var expr = "state:root/state:state[@type='navigational']";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, aStateDOM, this.ns);
        var stateNode = null;         
        if (nodes == null || nodes.length <= 0) {
            var rootNode = aStateDOM.firstChild;
            
            //make sure we really have the root node and not any processing instructions
            while ( rootNode && rootNode.nodeType == 7 ) {
            	rootNode = rootNode.nextSibling;
            }
            
			if (rootNode == null) {
                // create the root element
                rootNode = this._createElement(aStateDOM, "root");
                this._prependChild(rootNode, aStateDOM);
			} 
            stateNode = rootNode.firstChild;
			if (stateNode == null) {
                // create the state element
                stateNode = this._createElement(aStateDOM, "state");
                this._prependChild(stateNode, rootNode);
            }
            // add the 'type' attribute
            stateNode.setAttribute("type", "navigational");
        } else {
            // in most cases we will end up here
            stateNode = nodes[0];
        }
        return stateNode; 
    },
   
    /**
     * summary:
     *     Internal method that resolves a specific state node with the given node name
     *     which is located as a child of the given <state type='navigational'> node.
     *     If the node does not exist, this method will create a new node. 
     *     
     * aNodeName: String - the name of the node that should be resolved. Must not be NULL.
     * aPathExpression: String - XPath expression that addresses the node to be resolved
     *     relative to the given state node.
     * aStateNode: Element - DOM element representing the <state type='navigational'> node.
     * aStateDOM: dojo.dom.DOMDocument - the navigational state document that should be
     *     inspected. Must not be NULL. 
     */
    _getSpecificStateNode: function(aNodeName, aPathExpression, aStateNode, aStateDOM) {
        // evaluate the expression starting from the state node
        var nodes = com.ibm.portal.xpath.evaluateXPath(aPathExpression, aStateNode, this.ns);
        var node;
        if (nodes == null || nodes.length <= 0) {
            // create the state element
            node = this._createElement(aStateDOM, aNodeName);
            this._prependChild(node, aStateNode);
        } else {
            node = nodes[0];
        }
        return node;
    },
	_prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	},	    
    _createElement: function(dom, name) {
        var newElem;
        if (dojo.isIE) {
            newElem = dom.createNode(1, name, this.ns.state); // 1 = NODE_ELEMENT
        } else {
            newElem = dom.createElementNS(this.ns.state, name);
        }
        return newElem;
    }
    
});


dojo.declare("com.ibm.portal.state.PortletAccessor", null, {
    
    /**
     * summary:
     *     Constructor which initializes this acccessor with the
     *     state DOM and the DOM node that keeps the state information
     *     of the respective portlet window in its subtree.
     */
    constructor: function (/*Element*/ portletNode, 
                           /*dojo.dom.DOMDocument*/ stateDOM) {

        this.portletNode = portletNode;
        this.stateDOM = stateDOM;
        this.parameters = new com.ibm.portal.state.Parameters(portletNode, stateDOM);
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };

        this.xsltURL = dojo.moduleUrl( "com", "ibm/portal/state/");
    },

    /**
     * summary:
     *     Gets the current portlet mode of the portlet window this accessor
     *     operates on.
     */
    getPortletMode: function() {
        
        var expr = "state:portlet-mode";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.portletNode, this.ns);
        var portletMode = ibm.portal.portlet.PortletMode.VIEW; 
        if (nodes != null && nodes.length > 0) {
            var txtNode = nodes[0].firstChild;
            if (txtNode != null) {
                portletMode = txtNode.nodeValue;
            }
        }
        return portletMode; // ibm.portal.portlet.PortletMode - The current portlet mode.
    },
	
    /**
     * summary:
     *     Gets the current window state of the portlet window this accessor
     *     operates on.
     */
    getWindowState: function() {
        var expr = "state:window-state";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.portletNode, this.ns);
        var windowState = ibm.portal.portlet.WindowState.NORMAL; 
        if (nodes != null && nodes.length > 0) {
            var txtNode = nodes[0].firstChild;
            if (txtNode != null) {
                windowState = txtNode.nodeValue;
            }
        }
        return windowState; // ibm.portal.portlet.WindowState - The current window state.
    },

    /**
     * summary:
     *     Gets the current render parameters of the portlet window this accessor
     *     operates on. The returned Parameters object allows for querying, adding,
     *     modifying and removing parameters.
     */
    getRenderParameters: function() {
        return this.parameters; // com.ibm.portal.state.Parameters - The current render parameters. 
    },

    /**
     * summary:
     *     Sets the portlet mode of the portlet window this accessor operates on
     *     by changing the navigational state DOM accordingly.
     *     
     * portletMode: ibm.portal.portlet.PortletMode - the portlet mode to be set.
     *     Must not be NULL. 
     */
    setPortletMode: function(/*ibm.portal.portlet.PortletMode*/ portletMode) {
        var expr = "state:portlet-mode";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.portletNode, this.ns);
        if (nodes == null || nodes.length <= 0) {
            var portletModeNode = this._createElement(this.stateDOM, "portlet-mode");
            this._prependChild(portletModeNode, this.portletNode);
			var txtNode = this.stateDOM.createTextNode(portletMode);
            this._prependChild(txtNode, portletModeNode);
        } else {
            nodes[0].firstChild.nodeValue = portletMode;
        }
    },

    /**
     * summary:
     *     Sets the window state of the portlet window this accessor operates on
     *     by changing the navigational state DOM accordingly.
     *     
     * windowState: ibm.portal.portlet.WindowState - the window state to be set.
     *     Must not be NULL. 
     */
    setWindowState: function(/*ibm.portal.portlet.WindowState*/ windowState) {
        var expr = "state:window-state";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.portletNode, this.ns);
        if (nodes == null || nodes.length <= 0) {
            var windowStateNode = this._createElement(this.stateDOM, "window-state");
            this._prependChild(windowStateNode, this.portletNode);
            var txtNode = this.stateDOM.createTextNode(windowState)
            this._prependChild(txtNode, windowStateNode);
        } else {
            nodes[0].firstChild.nodeValue = windowState;
        }
    },
    
    /**
     * summary:
     *     Gets a dojo.dom.DOMDocument which contains only the state of the portlet
     *     this accessor operates on. The returned DOMDocument still complies with the
     *     state.DTD i.e. it contains the complete path to the portlet node that
     *     contains the portlet's state information. 
     */
    getPortletState: function() {
        // TODO implement this function using XSLT
        
        /*var xslt = com.ibm.portal.xslt;
        // load the xsl file
        var uri = this.xsltURL + "SelectPortletState.xsl";
        var xslDoc = xslt.loadXsl(uri);
	    // apply the transformation
	    if (xslDoc.documentElement == null) { alert("xslDoc is null"); }
    	var results = xslt.transform(this.stateDOM, xslDoc, null, {"id" : this.portletNode.getAttribute("id")}, true);
        return dojo.dom.createDocumentFromText(results);*/
        
        // get a new portlet accessor operating on a new empty state
        var newDOM = dojox.data.dom.createDocument();
        var portletAcc = com.ibm.portal.state.STATE_MANAGER.newPortletAccessor(this.portletNode.getAttribute("id"), newDOM);
        portletAcc.setPortletMode(this.getPortletMode());
        portletAcc.setWindowState(this.getWindowState());
        var paramMap = this.getRenderParameters().getMap();
        if (paramMap.length > 0) {
            portletAcc.getRenderParameters().putAll(paramMap);
        }
        return newDOM; /* dojo.dom.DOMDocument */
    },

    /**
     * summary:
     *     Sets the complete state of the portlet this accessor operates on.
     *     Ignores all state information in the given state that is not
     *     related to the portlet this accessor operates on. 
     *     
     *  state: dojo.dom.DOMDocument - The state DOM that contains state for the
     *      portlet this accessor operates on. Must not be NULL.
     *  merge: boolean - Flag that indicates whether the given state of the portlet
     *      should be merged into the state of the accessor or if it should be replaced.
     *      May be NULL which defaults to false, i.e. the current state of the portlet
     *      is replaced with the state retrieved from the given DOM. 
     */
    setPortletState: function(/* dojo.dom.DOMDocument */ state, /* boolean */ merge) {
        // TODO implement this function using XSLT
        
        // get a new portlet accessor operating on a new empty state
        var portletAcc = com.ibm.portal.state.STATE_MANAGER.newPortletAccessor(this.portletNode.getAttribute("id"), state);
        this.setPortletMode(portletAcc.getPortletMode());
        this.setWindowState(portletAcc.getWindowState());
        var paramMap = portletAcc.getRenderParameters().getMap();
        // clear parameters only if we should replace them
        if (merge == null || merge == false) {
            this.getRenderParameters().clear();
        }
        if (paramMap.length > 0) {
            this.getRenderParameters().putAll(paramMap);
        }
    },
	_prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	},	
    _createElement: function(dom, name) {
        var newElem;
        if (dojo.isIE) {
            newElem = dom.createNode(1, name, this.ns.state); // 1 = NODE_ELEMENT
        } else {
            newElem = dom.createElementNS(this.ns.state, name);
        }
        return newElem;
    }
    
});	



dojo.declare("com.ibm.portal.state.Parameters", null, {

    /**
     * summary:
     *     Constructor which initializes this Parameters object with the
     *     state DOM and the DOM node that keeps the parameters information.
     */
    constructor: function(/*Element*/ baseNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.baseNode = baseNode;
        this.stateDOM = stateDOM;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },

    /**
     * summary: 
     *     Returns a read-only parameter map represented as a Object[].
     *     Returns an empty map if no parameters are available.
     */
    getMap: function() {
        var names = this.getNames();
        var map = new Array(names.length);
        for (var i = 0; i < names.length; i++) {
            var name = names[i];
            map[i] = {name: name, values : this.getValues(name)};
        }
        return map; // Object[] {name: String, values: String[]}
    },

    /**
     * summary:
     *     Returns a String[] containing the names of all parameters that have an
     *     associated value. Returns an empty array if no parameters are available.
     */
    getNames: function() {
        var expr = "state:parameters/state:param";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.baseNode, this.ns);
        var names = new Array();
        if (nodes != null && nodes.length > 0) {
            var nodeCount = nodes.length;
            for (var i = 0; i < nodeCount; i++) {
                names[i] = nodes[i].getAttribute("name");
            }
        }
        return names;
    },

    /**
     * summary:
     *     Returns the String value associated with the parameter with the given name.
     *     
     * description:
     *     If there are one or more parameter values associated with the given name 
     * 	   it returns the first associated value.
     *
     * name: String - Name of the parameter for which the associated value is to
     *     be returned. Must not be NULL.
     */
    getValue: function(/*String*/ name) {
        var values = this.getValues(name);
        var firstValue = null;
        if (values != null && values.length > 0) {
            firstValue = values[0];
        }
		return firstValue;
    },

    /**
     * summary:
     *     Returns the String[] representing the values that are associated with
     *     the parameter with the given name.
     *
     * name: String - Name of the parameter for which the associated values are to
     *     be returned. Must not be NULL.
     */
    getValues: function(/*String*/ name) {
        var expr = "state:parameters/state:param[@name='" + name + "']/state:value";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.baseNode, this.ns);
        var values = null;
        if (nodes != null && nodes.length > 0) {
            values = new Array(nodes.length);
            var nodeCount = nodes.length;
            for (var i = 0; i < nodeCount; i++) {
                // get the text value
                var txtNode = nodes[i].firstChild;
                if (txtNode != null) {
                    values[i] = txtNode.nodeValue;
                }
            }
        }
        return values;
    },

    /**
     * summary:
     *     Removes the parameter with the given name.
     *
     * name: String - Name of the parameter that should be removed. Must not be NULL.
     */
    remove: function (/*String*/ name) {
        var expr = "state:parameters/state:param[@name='" + name + "']";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.baseNode, this.ns);
        if (nodes != null) {
			var toRemove = nodes[0];
			if(toRemove && toRemove.parentNode){
				toRemove.parentNode.removeChild(toRemove);
			}
		} 
    },

    /**
     * summary:
     *     Adds all parameters that are contained in the given map. Overrides the values
     *     of parameters that already exist.
     *
     * map: Object[] {name: String, values: String[]} - Two dimensional object array
     *     representing the map of parameters that should be added. Must not be NULL.
     */
    putAll: function(/*Object[] {name: String, values: String[]}*/ map) {
        if (map != null && map.length > 0) {
            for (var i = map.length - 1; i >= 0; i--) {
                var paramName = map[i].name;
                var paramValues = map[i].values;
                this.setValues(paramName, paramValues);
            }
        }
    },

    /**
     * summary:
     *     Sets a simple String value for the parameter with the given name. Overrides
     *     the value of the parameter if it already exists.
     *
     * name: String - The name of the parameter. Must not be NULL.
     * value: String - The value of the parameter that should be set. Must not be NULL. 
     */
    setValue: function (/*String*/ name, /*String*/ value) {
        this.setValues(name, new Array(value));
    },

    /**
     * summary:
     *     Sets a String[] representing the value of the parameter with the given name.
     *     Overrides the value of the parameter if it already exists.
     *
     * name: String - The name of the parameter. Must not be NULL.
     * value: String[] - The value of the parameter that should be set. Must not be NULL. 
     */
    setValues: function(/*String*/ name, /*String[]*/ values) {
        var expr = "state:parameters/state:param[@name='" + name + "']";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.baseNode, this.ns);
        var paramNode;
        if (nodes == null || nodes.length == 0) {
            // create the parameters node
            var paramsNode = null;// this._getFirstChildWithTag(this.baseNode, "state:parameters"); // do we really need to use the NS prefix here? bug??
            if (paramsNode == null) {
                paramsNode = this._createElement(this.stateDOM, "parameters");
                this._prependChild(paramsNode, this.baseNode);
            }
            // create the param node
            paramNode = this._createElement(this.stateDOM, "param");
            paramNode.setAttribute("name", name);
            this._prependChild(paramNode, paramsNode);
        } else {
            paramNode = nodes[0];
			// remove all children of paramNode
			dojox.data.dom.removeChildren(paramNode);
        }
        // putAll sometimes passes in null for values
        if (values != null) {
            for (var i = values.length - 1; i >= 0; i--) {
                var valueNode = this._createElement(this.stateDOM, "value");
                this._prependChild(valueNode, paramNode);
                var value = values[i];
                // make sure that we do not encode something like "undefined" into a value node 
                if (value != null) {
                    var txtNode = this.stateDOM.createTextNode(value);
                    this._prependChild(txtNode, valueNode);
                }
            }
        }
    },

    /**
     * summary:
     *     Removes all existing parameters.
     */
    clear: function() {
        var expr = "state:parameters";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.baseNode, this.ns);
        if (nodes != null) {
			var toRemove = nodes[0];
			if(toRemove && toRemove.parentNode){
				toRemove.parentNode.removeChild(toRemove);
			}
        } 
    },
	_getFirstChildWithTag: function(parentNode, tagName){
		// validate arguments
		if(!parentNode || !tagName) return null;
		var node = parentNode.firstChild;
		// iterate over all children
		while(node){
			// if an element node has a tag equal to tagname, return it
			if( node.nodeType == 1 && node.tagName && node.tagName.toLowerCase() == tagName.toLowerCase()){
				return node;
			}
			// otherwise, move on to the next sibling
			node = node.nextSibling;
		}
		// if we got here, we didn't find a match
		return null;
	},	
	_prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	},	    
    _createElement: function(dom, name) {
        var newElem;
        if (dojo.isIE) {
            newElem = dom.createNode(1, name, this.ns.state); // 1 = NODE_ELEMENT
        } else {
            newElem = dom.createElementNS(this.ns.state, name);
        }
        return newElem;
    }
	
});	


dojo.declare("com.ibm.portal.state.PortletListAccessor", null, {
   
    /**
     * summary:
     *     Constructor which initializes this accessor with the state DOM
     *     and the DOM node representing the root of navigational state
     *     information in the state DOM.
     */
    constructor: function(/*Element*/stateNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.stateNode = stateNode;
        this.stateDOM = stateDOM;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },

    /**
     * summary:
     *     Returns an array of Strings representing the IDs of the portlet windows
     *     that are part of the state DOM this accessor operates on. Returns null
     *     the state DOM does not contain any portlet-related state information.
     */
    getPortlets: function() {
        // get all portlet nodes
        var expr = "state:portlet";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.stateNode, this.ns);
        var portletIDs = null;
        if (nodes != null && nodes.length > 0) {
            portletIDs = new Array(nodes.length);
            for (var i=0; i < nodes.length; i++) {
               var node = nodes[i];
               portletIDs[i] = node.getAttribute("id");  
            }
        }
        return portletIDs; /*String[]*/
    }
    
});


dojo.declare("com.ibm.portal.state.SelectionAccessor", null, {
    
    /**
     * summary:
     *     Constructor which initializes this accessor with the state DOM
     *     and the DOM node that keeps the selection information.
     */
    constructor: function(/*Element*/selectionNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.selectionNode = selectionNode;
        this.stateDOM = stateDOM;
        this.parameters = new com.ibm.portal.state.Parameters(this.selectionNode, stateDOM);
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },

    /**
     * summary:
     *     Returns a String representing the unique identifier of the
     *     navigation node that represents the currently selected page.
     */
    getPageSelection: function() {
        return this.selectionNode.getAttribute("selection-node"); // String
    },
	
    /**
     * summary:
     *     Returns a String representing the unique identifier of the
     *     selected fragment. Typically the fragment is a layout node
     *     that is part of the layout model of the currently selected page.
     */
    getFragmentSelection: function() {
        var params = this.getParameters();
        var fragmentParams = params.getValues("frg");
        var fragmentID = null;
        if (fragmentParams != null && fragmentParams.length > 0) {
            fragmentID = fragmentParams[0];
            if (fragmentParams.length > 1) {
                if (fragmentID == "pw") {
                    fragmentID = fragmentParams[1];
                }
            }    
        }
        return fragmentID; // String
    },
    
    /**
     * summary:
     *     Returns a String representing the unique ID of the navigation node
     *     that represents the page that should be selected instead of the
     *     label with the given identifier. Returns null if there is no
     *     mapping available for the given label ID.
     *     
     * srcID:
     *     String representing the ID of the label that should be resolved.
     *     Must not be NULL.
     */
    getMapping: function(/*String*/ srcID) {
        var expr = "state:mapping[@src='" + srcID + "']";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.selectionNode, this.ns);
        var targetID = null;
        if (nodes != null && nodes.length > 0) {
            var mappingNode = nodes[0];
            targetID = mappingNode.getAttribute("dst");
        }
        return targetID; // String or null
    },

    /**
     * summary:
     *     Returns a com.ibm.portal.state.Parameters object which contains
     *     selection parameters. Does not return NULL.
     * 
     */
    getParameters: function() {
        return this.parameters; // com.ibm.portal.state.Parameters
    },
    
    /**
     * summary:
     *     Changes the selection according to the given navigation node ID.
     *     
     * navigationNodeID:
     *     String representing the identifier of the page or label that should
     *     be selected. Must not be NULL.
     */
    setPageSelection: function(/*String*/ navigationNodeID) {
        this.selectionNode.setAttribute("selection-node", navigationNodeID);
    },

    /**
     * summary:
     *     Changes the fragment selection according to the given layout node ID.
     *     
     * layoutNodeID:
     *     String representing the identifier of the layout node that represents
     *     the oage fragment that should be selected. Must not be NULL.
     * isPortletWindow:
     *     Flag which indicates whether or not the layout node represents a
     *     portlet window. May be NULL to indicate that the layout node is a
     *     portlet window.
     */
    setFragmentSelection: function(/*String*/ layoutNodeID, /*boolean*/ isPortletWindow) {
        var params = this.getParameters();
        if (isPortletWindow == null || isPortletWindow == true) {
            var values = new Array(2);
            values[0] = layoutNodeID;
            values[1] = "pw";
            params.setValues("frg", values);
        } else {
            params.setValue("frg", layoutNodeID);
        }
    },

    /**
     * summary:
     *     Sets a label mapping that specifies the page that should be
     *     selected instead of the label. The label is identified via the
     *     given source ID, the page via the target ID. If the given targetID
     *     is null, the method will completely remove the mapping from the state.
     *     
     * srcID:
     *     ID of the label that should be mapped to a page. Must not be NULL.
     * targetID:
     *     ID of the page the label should be mapped to. May be NULL..
     */
    setMapping: function(/*String*/ srcID, /*String*/ targetID ) {
        if (targetID != null) {
            var expr = "state:mapping[@src='" + srcID + "']";
            var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.selectionNode, this.ns);
            var mappingNode;
            if (nodes != null && nodes.length > 0) {
                mappingNode = nodes[0];
            } else {
                mappingNode = this._createElement(this.stateDOM, "mapping");
                this._prependChild(mappingNode, this.selectionNode);
                mappingNode.setAttribute("src", srcID);
            }
            mappingNode.setAttribute("dst", targetID);
        } else {
            // this is equivalent with removing the mapping
            this.removeMapping(srcID);
        }
    },

    /**
     * summary:
     *     Removes the label mapping with the given source ID.
     *     
     * srcID:
     *     ID of the label that should be removed. Must not be NULL.
     */
    removeMapping: function(/*String*/ srcID) {
        var expr = "state:mapping[@src='" + srcID + "']";
        var nodes = com.ibm.portal.xpath.evaluateXPath(expr, this.selectionNode, this.ns);
        var removed = false;
        if (nodes != null && nodes.length > 0) {
            // actually there should be only one such node
            // but clean up if we ran into an inconsistent state 
            for (var i=0; i < nodes.length; i++) {
	   			var toRemove = nodes[i];
				if(toRemove && toRemove.parentNode){
					toRemove.parentNode.removeChild(toRemove);
				}

            }
            removed = true;
        }
        return removed;
    },
	_prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	},	    
    _createElement: function(dom, name) {
        var newElem;
        if (dojo.isIE) {
            newElem = dom.createNode(1, name, this.ns.state); // 1 = NODE_ELEMENT
        } else {
            newElem = dom.createElementNS(this.ns.state, name);
        }
        return newElem;
    },
    
    /**
     * @deprecated: use getPageSelection instead
     */
    getSelection: function() {
        return this.getPageSelection(); // String
    },

    /**
     * @deprecated: use setPageSelection instead
     */
    setSelection: function(/*String*/ navigationNodeID) {
        this.setPageSelection(navigationNodeID);
    }
    
});	


dojo.declare("com.ibm.portal.state.SoloStateAccessor", null, {
    
    /**
     * summary:
     *     Constructor which initializes this accessor with the state DOM
     *     and the DOM node that keeps the solo state information.
     */
    constructor: function(/*Element*/soloNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.soloNode = soloNode;
        this.stateDOM = stateDOM;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },

    /**
     * summary:
     *     Sets the portlet window with the given ID into solo state.
     *     
     * portletWindowID:
     *     String representing the identifier of the portlet window which should
     *     be set to solo state. To entirely delete the solo state information from
     *     the state DOM, pass in NULL.
     */
    setSoloPortlet: function(/*String*/ portletWindowID) {
         // first remove child nodes
         dojox.data.dom.removeChildren(this.soloNode);
         if (portletWindowID != null) {
             // create a new text node and append it
             var txtNode = this.stateDOM.createTextNode(portletWindowID)
             this._prependChild(txtNode, this.soloNode);
         }
    },

    /**
     * summary:
     *     Returns the ID of the portlet window which is currently in solo state.
     *     Returns NULL if no such portlet window exists.
     */
    getSoloPortlet: function() {
        var txtNode = this.soloNode.firstChild;
        if (txtNode != null) {
            return txtNode.nodeValue;
        } else {
            return null;
        }
    },

    /**
     * summary:
     *     Sets the page the portal should navigate to after leaving
     *     Solo state.
     * 
     * pageID:
     *     ID of the return page. Must not be NULL.
     */ 
    setReturnSelection: function(/*String*/ pageID) {
        this.soloNode.setAttribute("return-selection", pageID);
    },

    /**
     * summary:
     *     Returns the ID of the page the portal navigates to
     *     after leaving Solo state.
     */
    getReturnSelection: function() {
        return this.soloNode.getAttribute("return-selection");
    },
	_prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	}   


});

dojo.declare("com.ibm.portal.state.ThemeTemplateAccessor", null, {
    
    /**
     * summary:
     *     Constructor which initializes this accessor with the state DOM
     *     and the DOM node that keeps the theme-template state information.
     */
    constructor: function(/*Element*/themeTemplateNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.themeTemplateNode = themeTemplateNode;
        this.stateDOM = stateDOM;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },
    
    /**
     * summary:
     *     Sets the theme-template to the given value.
     *     
     * themeTemplate:
     *     String representing the theme-template which should be set. To entirely delete the theme-template
     *     information from the state DOM, pass in NULL.
     */
    setThemeTemplate: function(/*String*/ themeTemplate) {
         // first remove child nodes
         dojox.data.dom.removeChildren(this.themeTemplateNode);
         if (themeTemplate != null) {
             // create a new text node and append it
             var txtNode = this.stateDOM.createTextNode(themeTemplate)
             this._prependChild(txtNode, this.themeTemplateNode);
         }
    },

    /**
     * summary:
     *     Returns the name of the current theme-template.
     *     Returns NULL if no such value exists.
     */
    getThemeTemplate: function() {
        var txtNode = this.themeTemplateNode.firstChild;
        if (txtNode != null) {
            return txtNode.nodeValue;
        } else {
            return null;
        }
    },
	_prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	}
	
});

dojo.declare("com.ibm.portal.state.LocaleAccessor", null, {
    
    /**
     * summary:
     *     Constructor which initializes this accessor with the state DOM
     *     and the DOM node that keeps the locale information.
     */
    constructor: function(/*Element*/localeNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.localeNode = localeNode;
        this.stateDOM = stateDOM;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },
    
    /**
     * summary:
     *     Sets the locale to the given value.
     *     
     * themeTemplate:
     *     String representing the locale which should be set. To entirely delete the locale
     *     information from the state DOM, pass in NULL.
     */
    setLocale: function(/*String*/ locale) {
         // first remove child nodes
         dojox.data.dom.removeChildren(this.localeNode);
         if (locale != null) {
             // create a new text node and append it
             var txtNode = this.stateDOM.createTextNode(locale)
             this._prependChild(txtNode, this.localeNode);
         }
    },

    /**
     * summary:
     *     Returns the name of the current locale in the state.
     *     Returns NULL if no such value exists.
     */
    getLocale: function() {
        var txtNode = this.localeNode.firstChild;
        if (txtNode != null) {
            return txtNode.nodeValue;
        } else {
            return null;
        }
    },

    _prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
        parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
    }
	
});


dojo.declare("com.ibm.portal.state.StatePartitionAccessor", null, {
    
    /**
     * summary:
     *     Constructor which initializes this accessor with the state DOM
     *     and the DOM node that keeps the state partition information.
     */
    constructor: function(/*Element*/statePartitionNode, /*dojo.dom.DOMDocument*/ stateDOM) {
        this.statePartitionNode = statePartitionNode;
        this.stateDOM = stateDOM;
        this.ns = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
    },
    
    /**
     * summary:
     *     Includes a state partition into the state.
     */
    includeStatePartition: function() {
         // first remove the exisiting partition ID
         dojox.data.dom.removeChildren(this.statePartitionNode);
         var txtNode = this.stateDOM.createTextNode(this._generateID());
         this._prependChild(txtNode, this.statePartitionNode);
    },
	
    _prependChild: function(/*DOMElement*/node, /*DOMElement*/parent){
		parent.firstChild ?	parent.insertBefore(node, parent.firstChild): parent.appendChild(node);
	},

    _generateID: function() {
        return Math.floor(Math.random()*100);
    }
	
});


/**
 * The serialization manager is able to serialize /deserialize state documents. 
 */
dojo.declare("com.ibm.portal.state.SerializationManager", null, {
    
    /**
     * Scheme of the uri that addresses the state REST service.
     */
    STATE_URI_SCHEME: "state",
    STATE_URI_POST: "state:encode",
    
    /**
     * The download mode
     */
    DOWNLOAD_MODE: "download",
    
    STATUS_UNDEFINED: 0,
    STATUS_OK: 1,
    STATUS_ERROR: 2,
    
    STATE_NS_URI: "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state",

    /**
     * Maximum length the serialized state must not exceed.
     * The actual URL length limit is 2048 but we need to take into account the base URL incl. query params.
     */
    STATE_THRESHOLD: 1024,
    
    /**
     * summary:
     *     Constructor which initializes the serialization manager with a URL that
     *     can be used to access the state REST service of portal.
     */
    constructor: function(/*String*/ stateServiceURL) {
        this.serviceURL = stateServiceURL;
    },
    
    /**
     * summary:
     *     Serializes the given state DOM into a form that can be included into URLs.
     *     If a callback function is provided, the method will execute asynchronously.
     *     Otherwise it will execute synchronously and return a
     *     {status: STATUS_OK /STATUS_ERROR /STATUS_UNDEFINED, state: dojo.dom.DOMDocument, url: String} object. 
     *     
     * state: dojo.dom.DOMDocument - The state DOM that should be serialized. Must not
     *     be NULL.
     * validate: boolean - Optional flag specifying whether the server should validate the state or not.
     *     Defaults to false. 
     * callbackfn: function - The callback function which should be called in case of
     *     internal status changes. May be NULL to indcate that the method should execute
     *     synchronously.
     */
    serialize: function(/*dojo.dom.DOMDocument*/ state, /*boolean*/ validate, /*Function*/ callbackfn) {
         ibm.portal.debug.entry( "SerializationManager.serialize", [ dojox.data.dom.innerXML( state ), validate, callbackfn ] );
         
         // get the serialized state
         var stateXML = dojox.data.dom.innerXML(state).replace(/[\r\n]/mg, "");
         // apply ASCII encoding to state XML
         // because we want to use it as a scheme-specific part in a URI or as part of a POST body
		 //RES: this used to call dojo.string.encodeAscii(), which also included special handling for unicode characters
		 // if escape() is not sufficient, see the _encodeAscii function defined in this object (though it doesn't work yet :)
         var encodedStateXML = escape(stateXML);
         //var encodedStateXML = dojo.string.encodeAscii(stateXML);
         //var encodedStateXML = this._encodeAscii(stateXML);
         var mType = this._getMimeType();
         var result = null;
         var me = this;
         ibm.portal.debug.text( "Mime type for response: " + mType );
         ibm.portal.debug.text( "Length of encoded state XML is: " + encodedStateXML.length );
         ibm.portal.debug.text( "Encoded state XML is: " + encodedStateXML );
         var digest = com.ibm.portal.services.PortalRestServiceConfig.digest;
         ibm.portal.debug.text( "Digest: " + digest );
         // check if we need to send a GET or a POST based on the length of the serialized state
         if (encodedStateXML.length <= this.STATE_THRESHOLD) {
             // GET
             // create the uri
             var payload = this.STATE_URI_SCHEME + ":" + encodedStateXML;
             var params;
             validate = (validate != null && validate == true);
             if (validate == true) {
                 if (digest != null) {
                     params = { "uri" : payload, "mode" : this.DOWNLOAD_MODE, "xmlns" : this.STATE_NS_URI, "sessionDependencyAllowed": "true", "preprocessors": "true", "digest": digest};
                 } else {
                     params = { "uri" : payload, "mode" : this.DOWNLOAD_MODE, "xmlns" : this.STATE_NS_URI, "sessionDependencyAllowed": "true", "preprocessors": "true"};
                 }
             } else {
                 if (digest != null) {
                     params = { "uri" : payload, "mode" : this.DOWNLOAD_MODE, "xmlns" : this.STATE_NS_URI, "sessionDependencyAllowed": "true", "digest": digest};
                 } else {
                     params = { "uri" : payload, "mode" : this.DOWNLOAD_MODE, "xmlns" : this.STATE_NS_URI, "sessionDependencyAllowed": "true"};
                 }
             }
             
             ibm.portal.debug.text( "Doing a GET request: { url: \"" + this.serviceURL + "\", sync: " + ((callbackfn)?false:true) + ", content: " + params + ", handleAs: " + mType + ", transport: XMLHTTPRequest" );
             ibm.portal.debug.text( "Parameters: uri=\"" + params.uri + "\" mode=\"" + params.mode + "\" xmlns=\"" + params.xmlns + "\"" );
              
             dojo.xhrGet({
                 url: this.serviceURL,
                 sync: (callbackfn)?false:true,
                 content: params,
                 headers: {"X-IBM-XHR":"true"},
                 handleAs: mType,
                 handle: function(response, ioArgs) {
                      ibm.portal.debug.text( "Response: " + response );
                      if(com.ibm.ajax.auth && (ioArgs.xhr.status==401 || ioArgs.xhr.status==0 )) {
                         ibm.portal.debug.text( " 401 found, re-auth");
                         com.ibm.ajax.auth.authenticationHandler();
                         return;
                      } else {
                          result = me._handleSerializationResponse.call(me, response, callbackfn, state, validate);
                          return response;
                      }
                 },
                 transport: "XMLHTTPTransport"
             });
         } else {
             ibm.portal.debug.text( "Doing a POST request." );
             // tentative fix: content handler framework cannot deal with UTF-16 at the moment
             if (dojo.isIE) {
                 var idx = stateXML.indexOf("UTF-16");
                 if (idx >= 0) {
                     stateXML = stateXML.replace(/UTF-16/, "UTF-8");
                 }
             }
             var url = this.serviceURL + "?uri=" + this.STATE_URI_POST + "&xmlns=" + this.STATE_NS_URI + "&sessionDependencyAllowed=true";
             if (digest != null) {
                 url += "&digest=" + digest;
             }
             // POST
             dojo.rawXhrPost({
                 url: url,
                 sync: (callbackfn)?false:true,
                 postData: stateXML,
                 handleAs: mType,
                 headers: {"Content-Type":"text/xml", "X-IBM-XHR":"true"},
                 handle: function(response, ioArgs) {
                      if(com.ibm.ajax.auth && (ioArgs.xhr.status==401 || ioArgs.xhr.status==0 )) {
                         ibm.portal.debug.text( "401 found, re-auth");
                         com.ibm.ajax.auth.authenticationHandler();
                         return;
                      } else {
                         result = me._handleSerializationResponse.call(me, response, callbackfn, state, validate);
                         return response;
                      }
                 },
                 transport: "XMLHTTPTransport"
             });
         }
         
         ibm.portal.debug.exit( "SerializationManager.serialize", result );
         return result;
    },
    
    /**
     * summary:
     *     Deserializes the state from the given URL. If a callback function is provided,
     *     the method will execute asynchronously. Otherwise it will execute synchronously
     *     and return a
     *     {status: STATUS_OK /STATUS_ERROR /STATE_UNDEFINED, url: String, state: dojo.dom.DOMDocument}
     *     object. 
     *     
     * url: String - String representing the URL the state DOM should be retrieved from.
     *     Must not be NULL.
     * callbackfn: function - The callback function which should be called in case of
     *     internal status changes. May be NULL to indcate that the method should execute
     *     synchronously.
     */
    deserialize: function(/*String*/ url, /*Function*/ callbackfn) {
        var payload = this.STATE_URI_SCHEME + ":" + url;
        var result = null;
        var mType = this._getMimeType();
        var me = this;
        var digest = com.ibm.portal.services.PortalRestServiceConfig.digest;
        ibm.portal.debug.text( "Digest: " + digest );
        var params;
        if (digest != null) {
            params = { "uri" : payload, "mode" : this.DOWNLOAD_MODE, "xmlns" : this.STATE_NS_URI, "digest" : digest};
        } else {
            params = { "uri" : payload, "mode" : this.DOWNLOAD_MODE, "xmlns" : this.STATE_NS_URI};
        }
        dojo.xhrGet({
             url: this.serviceURL,
             sync: (callbackfn)?false:true,
             content: params,
             headers: {"X-IBM-XHR":"true"},
             handleAs: mType,
             handle: function(response, ioArgs){
                  var type = ( response instanceof Error ) ? "error" : "load";
                  if (type == "load") {
                      var stateDOM = me._getResponseXML(response);
                      //If we get a parser error, then valid XML was not returned by 
                      //the serialization service and we want to return an empty xml
                      //document in this case.
                      if ( stateDOM.documentElement.nodeName == "parsererror" ) {
                          stateDOM = dojox.data.dom.createDocument();
                      }
                      if (callbackfn) {
                          // TODO use this.STATUS_OK here
                          callbackfn(1, url, stateDOM);
                      } else {
                          // TODO use this.STATUS_OK here
                          result = {"status" : 1, "input" : me.serviceURL, "url" : me.serviceURL, "returnObject" : stateDOM, "state" : stateDOM};
                      }
                  } else if (type == "error") {
                      if(com.ibm.ajax.auth && (ioArgs.xhr.status==401 || ioArgs.xhr.status==0 )) {
                          ibm.portal.debug.text( " 401 found, reauth");
                          com.ibm.ajax.auth.authenticationHandler();
                          return;
                      }


                      if (callbackfn) {
                          // TODO use this.STATUS_ERROR here
                          callbackfn(2, url, null);
                      } else {
                          // TODO use this.STATUS_ERROR here
                          result = {"status" : 2, "input" : me.serviceURL, "url" : me.serviceURL, "returnObject" : null, "state" : null};
                      }
                  }
             },
             transport: "XMLHTTPTransport"
         });
         return result;
    },
    

    _handleSerializationResponse: function(response, callbackfn, state, validate) {
        var result = null;
        var type = ( response instanceof Error ) ? "error" : "load";
        if (type == "load") {
            var responseXML = this._getResponseXML(response);
            // get the serialized state from response XML
            var linkExpr = "atom:entry/atom:link";
            var ns = {"atom" : "http://www.w3.org/2005/Atom", "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state"};
            var serializedState = null;
            var linkNodes = com.ibm.portal.xpath.evaluateXPath(linkExpr, responseXML, ns);
            if (linkNodes != null && linkNodes.length > 0) {
                // take the fist node found and read its href attribute
                serializedState = linkNodes[0].getAttribute("href");
            }
            var resultState = state;
            // check if the state potentially changed
            if (validate == true) {
                // get the new state from the response XML
                var stateExpr = "atom:entry/atom:content/state:root";
                var stateNodes = com.ibm.portal.xpath.evaluateXPath(stateExpr, responseXML, ns);
                if (stateNodes != null && stateNodes.length > 0) {
                    var newState = dojox.data.dom.innerXML(stateNodes[0]); 
                    resultState = dojox.data.dom.createDocument(newState);
                }
            }
            if (callbackfn) {
                // TODO use this.STATUS_OK here
                callbackfn(1, resultState, serializedState);
            } else {
                // TODO use this.STATUS_OK here
                result = {"status" : 1, "input" : resultState, "state" : resultState, "returnObject" : serializedState, "url" : serializedState};
            }
        } else if (type == "error") {                      
            if (callbackfn) {
                // TODO use this.STATUS_ERROR here
                callbackfn(this.STATUS_ERROR, state, null);
            } else {
                // TODO use this.STATUS_ERROR here
                result = {"status" : this.STATUS_ERROR, "input" : state, "state" : state, "returnObject" : null, "url" : null};
            }
        }
        return result;
    },


    _getMimeType: function() {
        var mtype = "xml";
        // IE does not work with "text/xml"
        if (dojo.isIE) {
             mtype = "text";
        }
        return mtype;
    },
    
    _getResponseXML: function(data) {
        var responseDOM = data;
        if (dojo.isIE) {
            responseDOM = dojox.data.dom.createDocument(data);
        }
        return responseDOM;
    },
	_encodeAscii: function(str) {
		var ret = str;
		if(dojo.isString(ret)) {
			var escapedStr = escape(ret);
			// \u#### patterns will be replaced with %u#### patterns by the escape() function
			var unicodeRegx = /%u([A-F0-9][A-F0-9][A-F0-9][A-F0-9])/i;
			var unicodeVal = null;
			// now find all %u#### patterns and replace them with an 
			while((unicodeVal = escapedStr.match(unicodeRegx))) {
				ret += escapedStr.substring(0, unicodeVal.index) + escape(Number("0x"+unicodeVal[1]));
				escapedStr = escapedStr.substring(unicodeVal.index + unicodeVal[0].length);
			}
			ret += escapedStr;
			ret = ret.replace(/\+/g, "%2B");
		}
		return ret;
	}	
    
    
});

/**
 * Component that manages state vary expressions on an ID basis.
 */
dojo.declare("com.ibm.portal.navigation.controller.StateVaryManager", null, {
    
				/**
				 * summary:
				 *     Constructor which initializes the state vary manager with a URL
				 */
				constructor: function() {
					this._expr = new Array();
				},

				setExpressions: function (/*String*/ id, /*Array*/ expressions) {
					var bucket = this._findBucket(id);
					if (bucket == null) {
						// create a new empty bucket
						bucket = { "id": id, "expr": null };
						// update the cache
						this._expr.push(bucket);
					}
					// add /replace the expressions
					bucket.expr = expressions;
				},

				getExpressions: function(/*String*/ id) {
					var result = null;
					var bucket = this._findBucket(id);
					if (bucket != null) {
						result = bucket.expr;
					}
					return result;
				},

				_findBucket: function(/*String*/ id) {
					var bucket = null;
					for (i=0; i<this._expr.length; i++) {
						var temp = this._expr[i];
						if (temp.id == id) {
							bucket = temp;
							break;
						}
					}
			      	return bucket;
				}
    
});

com.ibm.portal.state.STATE_MANAGER = new com.ibm.portal.state.StateManager(); 	
com.ibm.portal.state.STATE_MANAGER.reset(dojox.data.dom.createDocument()); // workaround


}

if(!dojo._hasResource["com.ibm.portal.state.common"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.state.common"] = true;
dojo.provide("com.ibm.portal.state.common");
dojo.require( "com.ibm.portal.state" );

}

if(!dojo._hasResource["com.ibm.portal.navigation.controller"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.navigation.controller"] = true;
dojo.provide( "com.ibm.portal.navigation.controller" );

dojo.require( "com.ibm.portal.services.common" );
dojo.require( "com.ibm.portal.state.common" );

dojo.require( "dojox.data.dom" );

com.ibm.portal.navigation.controller.CONSTANTS = new function () {
	// summary: Constants used throughout the controller.
	
	this.serializedStateToken = "_SERIALIZED_STATE_",
	this.portletWindowToken = "_PORTLET_WINDOW_ID_",
	this.selectedNodeToken = "_LAYOUT_NODE_ID_",
	this.portletUrlTemplate = this.serializedStateToken + "?uri=fragment:pm:oid:" + this.portletWindowToken + "@oid:" + this.selectedNodeToken
	this.forceSsaRenderMode = "force_ssa";
	this.forceCsaRenderMode = "force_csa";
}

dojo.declare( "com.ibm.portal.navigation.controller.NavigationalState", 
			  null,
			  {
			  	// summary: Wraps the state document and the selection model and provides read-only access
				//		to the information contained within those two objects.
				// description: Convenience wrapper for the state document and the selection model. Any changes
				// 		made to the state document or selection model should be ignored. The state document
				//		is not updated throughout the life of this object. It should be discarded at the end 
				//		of the rendering process.
				
				constructor: function ( /*dojo.dom.Document*/stateDom, /*NavigationSelectionModel*/navSelModel, /*boolean*/sendQueryParameters ) {
					if (stateDom) {
						this.stateDOM = stateDom;
					} else {
						this.stateDOM = dojox.data.dom.createDocument();
					}
					this.selectionModel = navSelModel;
					this.sendQueryParameters = sendQueryParameters;
					this._serializedState = null;
				},
				createLinkToCurrentState: function ( /*Boolean*/skipPortletState ) {
					// summary: Encapsulates the current navigational state in a URL which can be bookmarked or
					//		distributed.
					// skipPortletState: indicates whether the portlet state (portlet mode, window state, etc.) should be
					//		included in the encapsulated state. Setting this to true will make the URLs shorter, as well
					//		as perform better.
					// returns: the URL with the state encoded as a string
					var stateManager = this._getStateManager();
					var serializationMgr = stateManager.getSerializationManager();
					
					stateToSerialize = stateManager.getState();
					if ( skipPortletState ) {
						var selection = this.getPageSelection();
						var newStateMgr = new com.ibm.portal.state.StateManager( ibmPortalConfig["contentHandlerURI"] );
						newStateMgr.reset( newStateMgr.newState() );
						var selectionAcc = newStateMgr.newSelectionAccessor();
						selectionAcc.setPageSelection( selection );
						stateToSerialize = newStateMgr.getState();
					}
					
					var serializationResult = serializationMgr.serialize( stateToSerialize );
					return location.protocol + "//" + location.host + serializationResult["url"]; // String
				},
				getPageSelection: function () {
					// summary: Gets the currently selected page.
					// returns: the selected page object id without the model feed prefix- String
					var stateManager = this._getStateManager();
					var selectionAcc = stateManager.newSelectionAccessor();
					
					return selectionAcc.getSelection(); // String
				},
				getLocale: function() {
					// summary: Gets the current locale from the nav state
					// returns: the locale or null
					var stateManager = this._getStateManager();
					var localeAcc = stateManager.newLocaleAccessor();
					return localeAcc.getLocale();
				},
				getPortletState: function ( /*String*/portletWindowId ) {
					// summary: Gets the portlet state document for the given portlet window id.
					// returns: the state document for the given portlet id					
					var stateManager = this._getStateManager();
					return stateManager.newPortletAccessor( portletWindowId ).getPortletState();
				},
				getSoloPortlet: function() {
					// summary: Gets the portlet id of the portlet that is in solo mode if one
					// 		exists.  Null is returned if no portlets are in solo mode.
					// returns: the portlet id of the portlet in solo mode, or null				
					var stateManager = this._getStateManager();
					return stateManager.newSoloStateAccessor().getSoloPortlet();
				},
				resolveRelativePortletURL: function ( /*String*/ relativeURL ) {
					ibm.portal.debug.entry( "NavigationalState.resolveRelativePortletURL", [ relativeURL] );
					if ( this._serializedState == null ) {
						// serialize the state into a portal URL
						ibm.portal.debug.text( "Attempting to serialize state!"  )
						var stateManager = this._getStateManager();
						var serializationMgr = stateManager.getSerializationManager();
						var serializationResult = serializationMgr.serialize(stateManager.getState());
						// return the URL
						this._serializedState = serializationResult["returnObject"];
					}
					var resolvedURL = this._serializedState + relativeURL;
					ibm.portal.debug.exit( "NavigationalState.resolveRelativePortletURL", resolvedURL );
					return resolvedURL;
				},
				newPortletRenderURL: function ( /*String*/portletWindowId, /*String?*/portletMode, /*String?*/windowState  ) {

					// summary: Creates a portlet url based on the current state.
					// returns: the url.
					ibm.portal.debug.entry( "NavigationalState.newPortletRenderURL", [ portletWindowId, portletMode, windowState ] );
					
					var stateManager = this._getStateManager();
					
					if ( this._serializedState == null ) {
						ibm.portal.debug.text( "Attempting to serialize state!"  )
						// serialize the state into a portal URL
						var serializationMgr = stateManager.getSerializationManager();
						ibm.portal.debug.text( "State is: " + dojox.data.dom.innerXML( stateManager.getState() ) );
						var serializationResult = serializationMgr.serialize(stateManager.getState());
                        // return the URL if its not null. ie a 401 would cause the serializationResult
                        // to be null
                        if(serializationResult!=null) {
							this._serializedState = serializationResult["returnObject"];
						} else {
                            ibm.portal.debug.exit( "newPortletRenderURL serializationResult is null. return # to short circuit getPorletInfo " );
						    return "#";
                        }

                    }
					
					ibm.portal.debug.text( "Serialized state: " + this._serializedState );
					
					var selectionAcc = stateManager.newSelectionAccessor();
					
					// create the base URL
					var url = com.ibm.portal.navigation.controller.CONSTANTS.portletUrlTemplate; 
					url = url.replace( com.ibm.portal.navigation.controller.CONSTANTS.serializedStateToken, this._serializedState );
					url = url.replace( com.ibm.portal.navigation.controller.CONSTANTS.portletWindowToken, portletWindowId );
					url = url.replace( com.ibm.portal.navigation.controller.CONSTANTS.selectedNodeToken, selectionAcc.getPageSelection() );
					
					// append portlet mode parameter
					if (portletMode != null) {
						url += "&javax.portlet.PortletMode=" + portletMode;
					}
					
					// append window state parameter
					if (windowState != null) {
						url += "&javax.portlet.WindowState=" + windowState;
					}

					// append normalizeState param
					// only needed for portlet URLs generated on the client! (do not move this into PortletFragmentService:flagURL...)
					url += "&ibm.web2.normalizeState=true";

					// append query params
					if ( this.sendQueryParameters ) {
						url = url + "&" + location.search.substring(1);
					}
					
					ibm.portal.debug.exit( "NavigationalState.newPortletRenderURL", url );
					return url;
				},
				newPageURL: function (pageId, portletWindowId, windowState, portletMode, renderParams, newWindow) {
					ibm.portal.debug.entry( "NavigationalState.newPageURL", [ pageId, portletWindowId, windowState, portletMode, renderParams, newWindow ]);
					
					// get the state manager
					var stateManager = this._getStateManager();
					
					// get a copy of the current state
					var stateCopy = stateManager.newState(stateManager.getState());
					
					// get a portlet accessor which operates on the created state copy
					if(portletWindowId != null){
						var portletAcc = stateManager.newPortletAccessor(portletWindowId, stateCopy);
					}
				
					// modify the state
					if (portletMode != null) {
						portletAcc.setPortletMode(portletMode);
					}
                    
					if (windowState != null) {
						if( windowState.toLowerCase() == "solo" ){
							var soloAcc = stateManager.newSoloStateAccessor(stateCopy);
							// set portlet to solo state 
							soloAcc.setSoloPortlet(portletWindowId); 
							// set the return selection 
							var selAcc = stateManager.newSelectionAccessor(stateManager.getState()); 
							soloAcc.setReturnSelection(selAcc.getSelection()); 
						} else {
							portletAcc.setWindowState(windowState);
						}
					}
					
					// add the render params onto the url, note that the method
					// used is for a single value per name
					// renderParams[i] = { name: "paramName", value: "paramValue" }
					if (renderParams != null && renderParams.length > 0) {
						var renderParamsObject = portletAcc.getRenderParameters();
						renderParamsObject.clear();
						for(var i = 0 ; i < renderParams.length ; i++){
							renderParamsObject.setValue( renderParams[i].name, renderParams[i].value);
						}
					}
				    
					//If this url is to point to a new window, we need to set the theme template to "Plain".
					if ( newWindow ) {
						var templateAcc = stateManager.newThemeTemplateAccessor(stateCopy);
						templateAcc.setThemeTemplate( "Plain" );
						var partitionAcc = stateManager.newStatePartitionAccessor(stateCopy);
						partitionAcc.includeStatePartition();
					}	
				
					// encode the page ID using the selection accessor
					var selectionAcc = stateManager.newSelectionAccessor(stateCopy);
					selectionAcc.setPageSelection(pageId);
				
					// serialize the state into a portal URL
					var serializationMgr = stateManager.getSerializationManager();
					var serializationResult = serializationMgr.serialize(stateCopy);
			
					// return the URL
					//ibm.portal.debug.text("page action url: "+serializationResult["returnObject"]);
					if(serializationResult) {
                        return serializationResult["returnObject"];
                    } else {
                        return null;
                    }
					
				},
				_getStateManager: function () {
					// summary: Retrieve an instance of the state manager.
					// description: Creates an instance of the state manager and stores it as an instance
					//		variable. This instance of the state manager should only be used for read access
					//		since it is not reset with the state document every time it is retrieved. 
					// returns: the state manager - StateManager
					if ( !this._stateManager ) {
						this._stateManager = new com.ibm.portal.state.StateManager( ibmPortalConfig["contentHandlerURI"] );
						this._stateManager.reset( this.stateDOM );
					}
					return this._stateManager; // com.ibm.portal.state.StateManager  
				}	
			  }
);

dojo.declare( "com.ibm.portal.navigation.controller.NavigationalStateController",
			  null,
			  {
			  	// summary: The controller for the navigation.
				// description: The controller is responsible for managing the navigational state
				//		and notifying listeners of state change events. It must be initialized with a 
				//		state document via the initialize function before it can be used. The controller
				//		publishes several topics for various situations:
				//			/portal/navigation/controller/init - controller is initialized
				//			/portal/navigation/state		   - a navigational state change has occurred
				//			/portal/navigation/expansion	   - a navigation node has been expanded
				//			/portal/navigation/collapse		   - a navigational node has been collapsed
				//		The state change topic is not fired in case of the back button.
				currentTheme: null,
				currentThemePolicy: null,
				sendQueryParameters: false,
				constructor: function () {
					this._varyMgr = new com.ibm.portal.navigation.controller.StateVaryManager();
					this._mergeTemplate = "<merge><base-state>--basestate--</base-state><new-state>--newstate--</new-state></merge>";
					var xsltURL = dojo.moduleUrl( "com", "ibm/portal/navigation/");
					this._mergeXslt = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer(xsltURL + "StateMerge.xsl");
					this._namespaces = { "state" : "http://www.ibm.com/xmlns/prod/websphere/portal/v6.1/portal-state" };
					this._endFragmentHandle = com.ibm.portal.EVENT_BROKER.endFragment.register( this, 
																		function ( args ) {
																			var pInfo = args["portletInfo"];
																			if ( !(pInfo instanceof Error) ) {
																				this._updatePortletState( pInfo );
																			}	
																		}
																	);
					this._endPageHandle = com.ibm.portal.EVENT_BROKER.endPage.register( this, function () { this.sendQueryParameters = false; } );	
				
			    },
				setSelection: function ( /*String*/pageId, /*int?*/ navLevel, /*Boolean?*/backButton, /*String*/renderMode ) {
					// summary: Sets the selected page in the navigational state.
					// description: Calculates the selection path if necessary and sets the selection
					//		in the navigational state. 
					// pageId: the ID of the page to select
					// navLevel: OPTIONAL - the navigation level of the page
					// backButton: OPTIONAL - skips the firing of the state change events.
					ibm.portal.debug.entry( "NavigationController.setSelection", [ pageId, navLevel, backButton, renderMode] );
					ibm.portal.debug.text( "Changing selection from: " + this.getState().getPageSelection() +" to: "+pageId );
					com.ibm.portal.EVENT_BROKER.cancelAll.fire();                    
					this._updateInternalSelection( pageId );
					
					//First check if we have a URL node. This will change the render process.
					var selectedNode = this._selectionModel.getSelectedNode();
   
                    if(!selectedNode){
						return;
					}
					

					if ( selectedNode.isExternalURL ) {
						top.location = selectedNode.getContentURL();
						return;
					}
					
					if ( selectedNode.isInternalURL ) {
						com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.setSelection( selectedNode.getContentURL() );
						return;
					}
					
					if (com.ibm.portal.navigation.controller.CONSTANTS.forceSsaRenderMode == renderMode)
					{
					    // on a normal navigation state change:
					    //    NavController fires stateChanged
					    //    Aggregator recieves, renders page and fires PortalPage/complete
					    //    BackButtonHandler receives and updates history and stores state in client
					    // on full refresh, we can't fire the navStateChange, since we don't want the page to re-render
					    // client-side, so currently we will manually update history and client state storage
					    // need to refactor our events to accommodate.
					    // probably add a FullPageRefresh topic.
					    //var state = this.getState();
					    //com.ibm.portal.aggregation.backbutton.BACK_BUTTON_HANDLER.addToHistory( state );
					    //com.ibm.portal.aggregation.backbutton.BACK_BUTTON_HANDLER.saveState( state );

					    this.fullPageRefresh( this._selectionModel.getSelectedNode().URI.substring( 7 ) );
					}
					else if ( this._hasThemeChanged( this._selectionModel ) || this._hasThemePolicyChanged( this._selectionModel ) ) {
						//Check if the theme has changed.
						this.fullPageRefresh( this._selectionModel.getSelectedNode().URI.substring( 7 ) );
					}
					else {
					    this._fireNavigationalStateChanged();
					}
					ibm.portal.debug.exit( "NavigationController.setSelection" );	
				},
				setStateDocument: function ( /*XMLDocument*/newState ) {
					// summary: Sets the internal state DOM to the given state and triggers
					//		any updates based on the delta between the two states.
					// newState: a valid state document
					ibm.portal.debug.entry( "NavigationalStateController.setStateDocument", [ newState ] );
					this._updateInternalState( newState );
					this._fireNavigationalStateChanged();					
					ibm.portal.debug.exit( "NavigationalStateController.setStateDocument" )
				},
				expand: function ( /*String*/pageId, /*int?*/navLevel ) {
					// summary: Expand a navigation node.
					// description: Expands a navigation node. For now we aren't tracking the expand/collapse
					//		state client-side. If we ever do, this would be the place to set this in the state
					//		document.
					// pageId: the id of the node that is to be expanded
					this._fireExpansion( pageId );	
				},
				collapse: function ( /*String*/pageId, /*int?*/navLevel ) {
					// summary: Collapse a navigation node.
					// description: Collapses a navigation node. For now we aren't tracking the expand/collapse
					//		state client-side. If we ever do, this would be the place to set this in the state
					//		document.
					// pageId: the id of the node that is to be collapsed
					this._fireCollapse( pageId );	
				}, 
				getState: function() {
					// summary: Returns the current navigational state.
					// returns: NavigationalState - the current navigational state
					return new com.ibm.portal.navigation.controller.NavigationalState( this._stateDOM, this._selectionModel, this.sendQueryParameters );
				},
				fullPageRefresh: function ( /*String*/pageId ) {
					// summary: Performs a full browser page refresh based on the current state.
					// description: Creates a full portal url to the current state and loads that url in the browser. 
					//		This will reload the full portal page, resulting in re-execution of the theme's JSPs and bootstrap JS.
					// 
					// pageId: (optional) the page to update to be selected prior to the full page refresh.

					ibm.portal.debug.entry( "NavigationalStateController.fullPageRefresh", [ pageId ]);
					// update selection if needed
					var stateManager = this._getStateManager();
					
					if ( pageId ) {
						var selectionAcc = stateManager.newSelectionAccessor();
						selectionAcc.setPageSelection( pageId );
					}
					
					// serialize the state into a portal URL
					var serializationMgr = stateManager.getSerializationManager();
					ibm.portal.debug.text( "Retrieved  current state: " + dojox.data.dom.innerXML( stateManager.getState() ) );
					var serializationResult = serializationMgr.serialize(stateManager.getState());
					ibm.portal.debug.text( "Serialization result is: " );
					//dojo.debugShallow( serializationResult ); 	
					// return the URL
					var fullRefreshUrl = serializationResult["returnObject"];
					
					self.location=fullRefreshUrl;
					ibm.portal.debug.exit( "NavigationalStateController.fullPageRefresh", [ fullRefreshUrl ] );
					return fullRefreshUrl; // String
				},                
				_updateInternalState: function ( /*XMLDocument*/newState ) {
					// summary: Updates the internal state document and selection model if necessary.
					// newState: the new internal state
					ibm.portal.debug.entry( "NavigationalStateController._updateInternalState", [ dojox.data.dom.innerXML( newState ) ] );
					if ( this._hasSelectionChanged( newState ) ) {
						var newSelection = this._getPageSelection( newState );
						//This must be done in this order since the selection model could calculate a 
						//different selection than the one reflected in the newState document.
						this._mergeState(newState);
						this._updateInternalSelection( newSelection );		
					}
					else {
						// merge the new state into our current state
						this._mergeState(newState);
					}
					ibm.portal.debug.exit( "NavigationalStateController._updateInternalState" );
				},
				_mergeState: function(newState) {
					// summary: Merges the new state into the current state
					// description: Embeds both state documents into one "merge document" and applies
					// 		the "StateMerge.xsl" transformation. The result of the transformation
					//		is the state document that represents the new "current" state.
					if (this._stateDOM != null) {
						var serializedState = dojox.data.dom.innerXML(this._stateDOM);
						// get rid of the XML header
						var pos = serializedState.indexOf("?>");
						if (pos > 0) {
							serializedState = serializedState.substring(pos + 2, serializedState.length);
						}
						var serializedNewState = dojox.data.dom.innerXML(newState);
						pos = serializedNewState.indexOf("?>");
						if (pos > 0) {
							serializedNewState = serializedNewState.substring(pos + 2, serializedNewState.length);
						}
						var merge = this._mergeTemplate.replace(/--basestate--/g, serializedState).replace(/--newstate--/g, serializedNewState);
						var mergeDoc = dojox.data.dom.createDocument(merge);
						var results = this._mergeXslt.transformToDocument( mergeDoc, {}, true);
						// update the internal state with the merge result
						this._stateDOM = dojox.data.dom.createDocument(results);
					} else {
						// just assign the new state
						this._stateDOM = newState;
					}
				},
				_updateInternalSelection: function ( /*String*/newSelection ) {
					// summary: Updates the selection in the internal state.	
					if ( newSelection.indexOf("cm:oid:") != 0 )  {
						newSelection = "cm:oid:" + newSelection;
					}				
					this._selectionModel = new com.ibm.portal.services.NavigationSelectionModel( newSelection );
					
					//Set the selection.					
					var stateManager = this._getStateManager();
					var selectionAcc = stateManager.newSelectionAccessor();
                    if(this._selectionModel.getSelectedNode()){
    					selectionAcc.setPageSelection( this._selectionModel.getSelectedNode().URI.substring( 7 ) );	
                    }
				},
				_updatePortletState: function (/*PortletInfo*/portletInfo) {
					// summary: Sets the new portlet state.
					// description: Since we don't know the new portlet state until after we get the response
					//		from the portlet fragment feed, we subscribe to the fragment feed's response topic
					// 		and update the state "posthumously" instead of pre-emptively as we do for selection
					//		changes. It's rare that this function should ever be called directly.
					// portletInfo: the portlet info object passed by the fragment feed response topic
					ibm.portal.debug.entry( "NavigationalStateController.setPortletState", [ portletInfo ] );
					
					var portletWindowId = portletInfo.windowId;
					var stateManager = this._getStateManager();
					
					// register the state vary expressions
					this._updateStateVary( portletInfo.uri, portletInfo.stateVaryExpressions );

					var newState = portletInfo.updatedState;
					if (newState != null) {
						// check if we need to update some of the other portlets due to a navigational state change
						this._determineStateDependencies( portletInfo.otherPortlets, portletInfo.dependentPortlets, portletInfo.updatedState );
						// update the navigational state
						this._updateInternalState( newState );
						//this._updateInternalState( newState );
						ibm.portal.debug.text( "New state (after state merge) is: " + dojox.data.dom.innerXML( this._stateDOM ) );
						this._firePortletStateUpdated();
					}
					ibm.portal.debug.exit( "NavigationalStateController.setPortletState" );
				},
				_updateStateVary: function( /*String*/ id, /*Array*/ expr) {
					// summary: Updates the state vary expressions.
					if (expr != null) {
						this._varyMgr.setExpressions(id, expr);
					}
				},
				_determineStateDependencies: function(/*Array*/ otherPortlets, /*Array*/ depPortlets, /*XMLDocument*/ newState) {
					// summary: Detects state dependencies
					// description: Iterates over all portlets returned by portletInfo.otherPortlets and
					//		checks whether the state of these portlets has changed. State changes are
					//		detected by applying the XPath expressions the specific portlet returned
					//		in its last render response. The XPath expressions are stored by the
					//		StateVaryManager.
					if (otherPortlets != null) {
						for (var i=0; i<otherPortlets.length; i++) {
							var other = otherPortlets[i];
							var id = other.uri;
							var expressions = this._varyMgr.getExpressions(id);
							if (expressions != null) {
								for (var j=0; j<expressions.length; j++) {
									var expr = expressions[j];
									if (this._hasStateChanged(newState, expr)) {
										// add the portlet to the list of dependent portlets
										depPortlets.push({ "link": other.link, "portlet": other.portlet, "uri": other.uri });
										break;
									};
								}
							}
						}
					}
				},
				_hasStateChanged: function(/*dojo.dom.DOMDocument*/newState, /*String*/ xpath) {
					// alert("hasStateChanged: " + xpath);
					// alert(dojo.dom.innerXML(newState));
					// alert(dojo.dom.innerXML(this._stateDOM));

					// assume the state has not changed
					var result = false;

					// apply the xpath expression to both state documents
					var newNodes = com.ibm.portal.xpath.evaluateXPath(xpath, newState, this._namespaces );
					var prevNodes = com.ibm.portal.xpath.evaluateXPath(xpath, this._stateDOM, this._namespaces );
                    
					if (newNodes != null && prevNodes == null) {
						result = true;
					} else if (newNodes != null && prevNodes != null) {
						var nl = newNodes.length;
						if (nl > 0 && nl != prevNodes.length) {
							result = true;
						} else {
							// iterate over nodes
							for (var i=0; i<newNodes.length; i++) {
								var n1 = newNodes[i];
								var n2 = prevNodes[i];
								if (n1.attributes.length != n2.attributes.length) {
									result = true;
									break;
								} else if (n1.childNodes.length != n2.childNodes.length) {
									result = true;
									break;
								} else {
									// serialize
									var ser1 = dojox.data.dom.innerXML(n1);
									var ser2 = dojox.data.dom.innerXML(n2);
									if (ser1 != ser2) {                 
										result = true;
										break;
									}
								}
							}
						}
					}
					return result;
				},
				_hasThemeChanged: function ( /*NavigationSelectionModel*/selectionModel ) {
					var result = false;
					if ( this.currentTheme != null ) {
						result = ( this.currentTheme != selectionModel.getSelectedNodeTheme() );
					}
					else {
						this.currentTheme = selectionModel.getSelectedNodeTheme();
					}
					return result;
				},
				_hasThemePolicyChanged: function ( /*NavigationSelectionModel*/selectionModel ) {
					var result = false;
					if ( this.currentThemePolicy != null ) {
						result = ( this.currentThemePolicy != selectionModel.getSelectedNodeThemePolicy() );
					}
					else {
						this.currentThemePolicy = selectionModel.getSelectedNodeThemePolicy();
					}
					return result;	
				},
				_hasSelectionChanged: function ( /*XMLDocument*/newState ) {
					// summary: Determines if a selection change has occurred.
					// newState: a valid state document
					// returns: true if the selection has changed from the currently stored internal state
					ibm.portal.debug.entry( "NavigationalStateController._hasSelectionChanged", [newState] );
					
					//If there is no internal state, the selection MUST have changed.
					if ( !this._stateDOM ) {
						ibm.portal.debug.exit( "NavigationalStateController._hasSelectionChanged", [true] );
						return true;
					}
					//Get the current page selection.
					var currentSelection = this._getPageSelection();
					ibm.portal.debug.text( "Current selection is: " + currentSelection );
					//Get the page selection in the new state document.
					var newSelection = this._getPageSelection( newState );
					ibm.portal.debug.text( "New selection is: " + newSelection );
					//Determine the return value.
					var retVal = ( currentSelection != newSelection );
									
					ibm.portal.debug.exit( "NavigationalStateController._hasSelectionChanged", [retVal] );
					return retVal;
				},
				_getStateManager: function ( /*XMLDocument?*/ stateDom ) {
					// summary: Returns an instance of the state manager.
					// stateDom: the state document to operate on.
					if ( !this._stateManager ) {
						this._stateManager = new com.ibm.portal.state.StateManager( ibmPortalConfig["contentHandlerURI"] );
					}
					if ( !stateDom ) {
						stateDom = this._stateDOM;
					}	 
					this._stateManager.reset( stateDom );
					return this._stateManager;  
				},
				_getPageSelection: function ( /*XMLDocument?*/ stateDom ) {
					// summary: Returns the current selection.
					// stateDom: the state document to operate upon.					
					//Get the current page selection.
					var stateMgr = this._getStateManager( stateDom );
					var selectionAcc = stateMgr.newSelectionAccessor();
					return selectionAcc.getPageSelection();	
				},
				_fireNavigationalStateChanged: function ( backButton ) {
					// summary: Fires the navigational state changed event. Passes the NavigationalState and the
					//		backbutton indicator to the topic.
					//dojo.publish( "/portal/NavigationalStateController/stateChanged", { state: this.getState(), backButton: backButton } );
					com.ibm.portal.EVENT_BROKER.stateChanged.fire( { state: this.getState(), backButton: backButton } );
				},
				_firePortletStateUpdated: function ( backButton ) {
					// summary: Fires the navigational state changed event. Passes the NavigationalState and the
					//		backbutton indicator to the topic.
					//dojo.publish( "/portal/NavigationalStateController/portletStateUpdated", { state: this.getState(), backButton: backButton } );
					com.ibm.portal.EVENT_BROKER.fragmentUpdated.fire( { state: this.getState(), backButton: backButton } );
				},
				_fireExpansion: function ( /*String*/pageId ) {
					// summary: Fires the node expansion event. Passes the pageId to the topic.
					dojo.publish( "/portal/NavigationalStateController/expansion", [{ pageId: pageId }] );	
				},
				_fireCollapse: function ( /*String*/pageId ) {
					// summary: Fires the node collapse event. Passes the pageId to the topic.
					dojo.publish( "/portal/NavigationalStateController/collapse", [{ pageId: pageId }] );	
				}	
			  }
);

com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER = new com.ibm.portal.navigation.controller.NavigationalStateController();

}

if(!dojo._hasResource["com.ibm.portal.aggregation.error"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.error"] = true;
dojo.provide( "com.ibm.portal.aggregation.error" );



dojo.declare( "com.ibm.portal.aggregation.error.ErrorHandler",
              null,
              {
                constructor: function () {
                    // Take over the onerror handler directly and not as an event listener through dojo
                    // so that when reportError gets called, all 3 parameters have error information
                    window.onerror = this.reportError;
                    //dojo.connect( window, "onerror", dojo.hitch( this, "reportError" ) );
                    
                    //Take over the error handling here from dojo.
                    dojo.raise = this.reportDojoError;
                    
                },
                reportError: function ( /*String*/msg, /*String*/detailedDescription, /*String*/lineNo ) {
                    if ( !detailedDescription ) { 
                        detailedDescription = "";
                        if (lineNo) {
                            detailedDescription = lineNo;
                        }
                    }

                    var errorMessage = new com.ibm.portal.aggregation.StatusMessage( 'error', msg, detailedDescription );
                    dojo.publish("/portal/status",[{message: errorMessage}]);
                },
                reportWarning: function ( /*String*/msg, /*String*/detailedDescription ) {
                    if ( !detailedDescription ) { detailedDescription = ""; }
                    var warningMessage = new com.ibm.portal.aggregation.StatusMessage( 'warning', msg, detailedDescription );
                    dojo.publish("/portal/status",[{message: warningMessage}]); 
                },
                reportDojoError: function ( /*String*/message, /*String*/exception ) {
                    var details = "";
                    if(exception){
                        details = message + ": "+dojo.errorToString(exception);
                      }else{
                        details = dojo.errorToString(message);
                      }
                    var v3themeProperties = dojo.i18n.getLocalization("com.ibm.portal", "v3theme", csaUserLocale);
                    var errorMessage = new com.ibm.portal.aggregation.StatusMessage( 'error', v3themeProperties["statusError"], details );
                    dojo.publish("/portal/status",[{message: errorMessage}]);
                    window.console.log( message );
                    
                    return true;
                }
              }
);

com.ibm.portal.aggregation.error.ERROR_HANDLER = null;

//Need to trigger this after the page is loaded, otherwise the dojo.connect call in the constructor will fail.
dojo.addOnLoad( function () { com.ibm.portal.aggregation.error.ERROR_HANDLER = new com.ibm.portal.aggregation.error.ErrorHandler();  } );

}

if(!dojo._hasResource["com.ibm.portal.aggregation.storage"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.storage"] = true;
dojo.provide( "com.ibm.portal.aggregation.storage" );

dojo.require( "dojo.cookie" );
dojo.require( "com.ibm.portal.aggregation.error" );

dojo.declare( "com.ibm.portal.aggregation.storage.StorageUnit",
			  null,
			  {
			  	// summary: Generic client-side storage interface. 
			  	hasRoom: function ( /*String*/name, /*String*/ value ) {		
			  		// summary: Indicates whether the storage unit has room for
			  		//		the given value. 
			  		// returns: true if there is room, false if it cannot be stored
			  	},
			  	store: function ( /*String*/name, /*String*/ value ) {
			  		// summary: Stores the given value under the given name.
			  		// description: If there is not room for the value, the value will not be stored. The
			  		// 		hasRoom function should be called prior to storing a value to allow for proper
			  		// 		error handling. This function will return true if the storage was successful,
			  		// 		false if it was not.
			  		// returns: true if the value was stored, false if it was not
			  	},
			  	retrieve: function ( /*String*/name ) {
			  		// summary: Retrieves a value given the name it was stored under.
			  		// returns: the value or NULL if the value can't be found
			  	},
			  	remove: function ( /*String*/ name ) {
			  		// summary: Removes a value from the storage unit.
			  		// returns: the value or NULL if the value doesn't exist
			  	},
			  	getContents: function () {
			  		// summary: Get the contents of the storage unit.
			  	}
			  }
);

dojo.declare( "com.ibm.portal.aggregation.storage.CookieStorageUnit",
			  com.ibm.portal.aggregation.storage.StorageUnit,
			  {
			  	maxStringSize: 7000,
			  	constructor: function () {
			  		/*if ( !dojo.cookie.isSupported() ) {
			  			this._cookiesEnabled = false;
			  			com.ibm.portal.aggregation.error.ERROR_HANDLER.reportWarning( "Cookies are not supported!", "Some features may not work correctly in client-side aggregation. Please enable cookies on your browser to fix this issue." );
			  		}*/
			  		this._cookiesEnabled = true;
			  	},
			  	hasRoom: function ( /*String*/name, /*String*/ value ) {		
			  		// summary: Indicates whether the storage unit has room for
			  		//		the given value. 
			  		// returns: true if there is room, false if it cannot be stored
			  		//return ( (name.length + value.length) <= this.maxStringSize );
			  		ibm.portal.debug.entry( "CookieStorageUnit.hasRoom", [ name, value ] );
			  		var hasRoom = true;
			  		
			  		//Based on the theory that browsers silently fail when setting a cookie that exceeds the 
			  		//maximum size and simply don't change the value of the cookie to the attempted value. Attempt to set the cookie
			  		//value, retrieve the value of the cookie after attempting to set it, and compare that value
			  		//to the value that we attempted to set the cookie to. If the two values don't match, then we know
			  		//the cookie is too big (or failed for another reason, but either way it's not going to work). This way,
			  		//if a browser allows a user configurable cookie size or if the standard maximum cookie size increases, we can
			  		//automagically take advantage of the bigger cookie size. Leave the maximum string size in place to provide some upper
			  		//limit on the cookie size; simply make it larger than we normally would.
			  		var cv = dojo.cookie( name );
			  		if ( cv != null ) {
			  			dojo.cookie( name, value, {path: "/"} );
			  			var sizePermitted = false;
			  			if ( dojo.isIE ) {
				  			var length = 0;
				  			if(cv && cv.length > 0) length = name.length + cv.length;
				  			sizePermitted = (document.cookie.length - length + name.length + value.length <= this.maxStringSize);
			  			}
			  			else {
			  				sizePermitted = ( (name.length + value.length) <= this.maxStringSize );
			  			}	
			  			hasRoom = ( ( value == dojo.cookie( name ) ) && sizePermitted );
			  			//Make sure to replace the original value in case the caller decides not to go ahead
			  			//with the store() operation.
			  			dojo.cookie( name, cv, {path: "/"} );
			  		}	
			  		
			  		ibm.portal.debug.exit( "CookieStorageUnit.hasRoom", hasRoom ); 
			  		return hasRoom;
			  	},
			  	store: function ( /*String*/name, /*String*/ value ) {
			  		// summary: Stores the given value under the given name.
			  		// description: If there is not room for the value, the value will not be stored. The
			  		// 		hasRoom function should be called prior to storing a value to allow for proper
			  		// 		error handling. This function will return true if the storage was successful,
			  		// 		false if it was not.
			  		// returns: true if the value was stored, false if it was not
			  		ibm.portal.debug.entry( "CookieStorageUnit.store", [ name, value ] );
			  		var success = false;
			  		
			  		if ( this._cookiesEnabled ) {
			  			dojo.cookie( name, value, {path: "/"} );
			  			success = ( dojo.cookie( name ) == value );
			  		}
			  		
			  		ibm.portal.debug.exit( "CookieStorageUnit.store", [ success ] );
			  		return success;
			  	},
			  	retrieve: function ( /*String*/name ) {
			  		// summary: Retrieves a value given the name it was stored under.
			  		// returns: the value or NULL if the value can't be found
			  		var retVal = null;
			  		if ( this._cookiesEnabled ) {
			  			retVal = dojo.cookie( name );
			  		}
			  		
			  		return retVal;
			  	},
			  	remove: function ( /*String*/ name ) {
			  		// summary: Removes a value from the storage unit.
			  		// returns: the value or NULL if the value doesn't exist
			  		var retVal = this.retrieve( name );
			  		if ( this._cookiesEnabled ) {
			  			dojo.cookie(name, null, {expires: -1});
			  		}	
			  		return retVal;
			  	}
			  }		
			  
);
			  

}

if(!dojo._hasResource["com.ibm.portal.aggregation.backbutton"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.backbutton"] = true;
dojo.provide( "com.ibm.portal.aggregation.backbutton" );

dojo.require( "dojo.back" );
dojo.require( "dojox.data.dom" );
dojo.require( "dojo.cookie" );

dojo.require( "com.ibm.portal.EventBroker" );
dojo.require( "com.ibm.portal.aggregation.storage" );
dojo.require( "com.ibm.portal.aggregation.error" );

com.ibm.portal.aggregation.backbutton.CONSTANTS = new function () {
	// summary: Constants for use with the back button support.
	this.storageKeyForSavedState = "testKey";
}

dojo.declare( "com.ibm.portal.aggregation.backbutton.PortalBackButtonHandler",
			  null,
			  {
			  	// summary: Handles all back button events. 
				// description: Subscribes to the state changed topic. Any time the state changes, a new entry is entered into
				// 		the history with the current nav state (TODO: just deltas or full state?). On back button/forward button
				// 		press, the NAVIGATION_CONTROLLER is "initialized" with the new state and the aggregator is called.
				// 		All navigation elements and page elements are responsible for determining whether their state has
				// 		changed from the NavigationalState provided by the NAVIGATION_CONTROLLER and updating accordingly.
				
				constructor: function () {
				  	this._previousState = null;
					this._waitingForPageToRender = false;
					this._isBackButtonStateChange = false;
					this._initialized = false;
					
					this._storageUnit = new com.ibm.portal.aggregation.storage.CookieStorageUnit();
					
				  	//The state is not complete until the page has fully rendered since the portlet
					//state documents aren't known until the portlet fragment request has returned.
					//So we have to ignore state updates for portlets until the page rendering process
					//is complete because we don't want those to be in the back button history. Any 
					//portlet state updates that come in after the page is rendered completely should 
					//be added to the back button history.
					//dojo.subscribe( "/portal/PortalPage/start", this, function () { this._waitingForPageToRender = true; } );
					this._startPageHandle = com.ibm.portal.EVENT_BROKER.startPage.register( this, function () { this._waitingForPageToRender = true; } );
					/*dojo.subscribe( "/portal/NavigationalStateController/portletStateUpdated", 
												this,
												function () {
													if ( !this._waitingForPageToRender ) {
														var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
														this.addToHistory( state );
														this.saveState( state );
													}
												} 
					);*/
					this._fragmentUpdatedHandle = com.ibm.portal.EVENT_BROKER.fragmentUpdated.register( this, 
																		  function () {
																				if ( !this._waitingForPageToRender ) {
																					var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
																					this.addToHistory( state );
																					this.saveState( state );
																				}
																			}
					);
																				
					/*dojo.subscribe( "/portal/PortalPage/complete", 
												this, 
												function () {
													this._waitingForPageToRender = false;
													if ( !this._isBackButtonStateChange) {
														var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
														this.addToHistory( state );
														this.saveState( state );
													}
													else {
														this._fireBackButtonPressedComplete();
													}	
												} 
					);*/
					this._endPageHandle = com.ibm.portal.EVENT_BROKER.endPage.register( this, 
																  function () {
																	this._waitingForPageToRender = false;
																	if ( !this._isBackButtonStateChange) {
																		var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
																		this.addToHistory( state );
																		this.saveState( state );
																	}
																	else {
																		this._isBackButtonStateChange = false;
																	}	
																} 
					);
					this._redirectHandle = com.ibm.portal.EVENT_BROKER.redirect.register( this, function () { this.clearState(); } );
			    },
				setInitialState: function ( /*NavigationalState*/state ) {
					// summary: Sets the initial state for the back button.
					// description Called when the controller/init topic is published so that the initial state
					//		can be recorded.
					ibm.portal.debug.entry( "PortalBackButtonHandler.setInitialState", [ state ] );	
				
					var initialState = dojox.data.dom.innerXML( state.stateDOM );
					this._previousState = initialState;
					dojo.back.setInitialState({
	                	back: this._newBackButtonFunction( initialState ),
	                	changeUrl: false
	             	});	
				 	this._initialized = true;
				 	ibm.portal.debug.exit( "PortalBackButtonHandler.setInitialState" );
				},
				addToHistory: function ( /*NavigationalState*/state ) {
					// summary: Adds the given state to the back button history.
					// description: Subscribes to the state changed	topic and stores the changed state
					//		in the back button history.				
					ibm.portal.debug.entry( "PortalBackButtonHandler.addToHistory" );
					
					if ( !this._initialized ) {
						this.setInitialState( state );	
					}
					else {
						var currentState = dojox.data.dom.innerXML(state.stateDOM);
						
						ibm.portal.debug.text( "Current state: " + currentState  );
						
						if ( currentState != this._previousState ) {
							dojo.back.addToHistory({
								handle: this._newBackButtonFunction( currentState ),
				                changeUrl: false
							});
							this._previousState = currentState;
						}
					}	
					
					ibm.portal.debug.exit( "PortalBackButton.addToHistory" );	
				},
				saveState: function ( /*NavigationalState*/state ) {
					// summary: Stores the current state in the client-side storage so that we can handle a
					//		full-page refresh.
					// description: First attempt to simpy store the xml string as the cookie value. If that value
					//		is too large to be stored as a cookie, attempt to store the serialized url version of 
					//		that state document (not ideal since it requires an additional request to the state
					//		serialization service). If that value is too large to fit in a cookie (probably unlikely), then
					//		report a warning to the user that the navigational state is unable to be maintained on the client.
					ibm.portal.debug.entry( "PortalBackButtonHandler.saveState", [ state ] );
					
					var stateText = dojox.data.dom.innerXML( state.stateDOM );
					var stateKey = this._getStateKey();
					var browserURL = top.location.href;
					var browserURLKey = this._getUrlKey();
					
					if ( this._storageUnit.hasRoom( stateKey, stateText ) ) {
						this._storageUnit.store( stateKey, stateText );
					}
					else {
						ibm.portal.debug.text( "State is too big for the storage unit; serialize state into an url.", "PortalBackButtonHandler.saveState" );
						//Serialize the xml document into an URL and attempt to store that value on the client. 
						var stateMgr = new com.ibm.portal.state.StateManager( ibmPortalConfig["contentHandlerURI"] );
						stateMgr.reset( state.stateDOM );
						var serializationMgr = stateMgr.getSerializationManager();
						var me = this;
						serializationMgr.serialize( state.stateDOM, false, function (status, stateDoc, url ) { 
							var _method = "PortalBackButtonHandler.saveState.serializationCallback";
							ibm.portal.debug.entry( _method, [ status, stateDoc, url ] );
							if ( !me._storageUnit.store( stateKey, "s8d:" + url ) ) {
								//The navigational state is simply too large to be stored. Report the warning to the user and move on.
								if ( !me._warningLogged ) {
									com.ibm.portal.aggregation.error.ERROR_HANDLER.reportWarning( "Unable to save the current state!", "Hitting the browser's refresh button now will lose the current navigational state." );
									me._warningLogged = true;
								}
							}
							ibm.portal.debug.exit( _method );
						} );
					}
					
					if ( this._storageUnit.hasRoom( browserURLKey, browserURL ) ) {
						this._storageUnit.store( browserURLKey, browserURL );
					}
					ibm.portal.debug.exit( "PortalBackButtonHandler.saveState" );
				},
				loadState: function () {
					// summary: Loads the current state from the client-side storage if it exists.
					ibm.portal.debug.entry( "PortalBackButtonHandler.loadState" );
					
					var stateText = this._storageUnit.retrieve( this._getStateKey() );
					var stateDocument = null;
					if ( stateText && stateText != "" ) {
						//Check to see if we have a serialized state url as we need to deserialize it before
						//we get anything useful.
						if ( stateText.indexOf( "s8d:" ) == 0 ) {
							ibm.portal.debug.text( "Serialized state discovered! Deserializing..." );
							var stateMgr = new com.ibm.portal.state.StateManager( ibmPortalConfig["contentHandlerURI"] );
							var serializationMgr = stateMgr.getSerializationManager();
							var url = stateText.substring( 4 );
							var returnObj = serializationMgr.deserialize( url );
							stateDocument = returnObj["state"];
						}	
						else {
							stateDocument = dojox.data.dom.createDocument( stateText );
						}	
					}
					
					ibm.portal.debug.exit( "PortalBackButtonHandler.loadState", stateDocument );
					return stateDocument;
				},
				clearState: function () {
					this._storageUnit.remove( this._getStateKey() );
				},
				loadUrlFrag: function () {
					var url = this._storageUnit.retrieve( this._getUrlKey() );
					if(!url) url = "";
					return url;
				},
				_newBackButtonFunction: function ( /*String*/state ) { 
					var me = this;
					var handler = function(){
									ibm.portal.debug.text( "backbutton -> " + state );
									me._isBackButtonStateChange = true;
									com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.setStateDocument( dojox.data.dom.createDocument( state ) );
				                }
					return handler;					
				},
				_getStateKey: function () { 
					var sessionID = dojo.cookie( "JSESSIONID" );
					var token = dojo.cookie( "LtpaToken" );
					if(token && token.length > 20) token = token.substr(0, 20);
					var key = window.location.host + sessionID + token;
					//Encode this as if it will be in a url because certain characters are unsafe.
					return encodeURIComponent( key );
				},
				_getUrlKey: function () {
					return this._getStateKey() + "URL";
				}	
			  }  
);

com.ibm.portal.aggregation.backbutton.BACK_BUTTON_HANDLER = new com.ibm.portal.aggregation.backbutton.PortalBackButtonHandler();

}

if(!dojo._hasResource["com.ibm.portal.aggregation.event"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.event"] = true;
dojo.provide( "com.ibm.portal.aggregation.event" );

dojo.require( "com.ibm.portal.utilities" );

com.ibm.portal.aggregation.event.normalizeEvent = function ( /*Event?*/e ) {
    // summary: Retrieves and normalizes the event object, hiding the browser differences.
    // description: From the object (or lack thereof) passed into the event handler, gets the
    //      global event object (if IE) and adds the W3C methods and properties to it so that
    //      the same event object can be used in both FF and IE. Also, provides two new properties
    //      "evt.defaultPrevented" and "evt.propagationStopped" which are set when the corresponding
    //      methods are called so that the event handling code can respond accordingly (normally
    //      the browser simply handles these calls with no need for the client to interfere). It is safe
    //      to call this function mulitple times per event. In other words, the event object should 
    //      ALWAYS be passed into this function before using it in an event handler.
    // e: the event object, if none is passed in, this function will try and use the global
    //      IE window.event object.
    // returns: the normalized event object safe for use in IE or FF
    ibm.portal.debug.entry( "normalizeEvent", [e] );
    
    //Check to see if we should be using the global IE event object. If so, we need to
    //wrapper the returnValue and cancelBubble properties in the W3C equivalents, as well
    //as set the target property.
    if ( !e || (!e.preventDefault && !e.stopPropagation) ) {
        e = window.event;
        e.preventDefault = function () {
            this.returnValue = false;
        }
        e.stopPropagation = function () {
            this.cancelBubble = true;
        }
        e.target = window.event.srcElement;             
    }
    
    //Override the preventDefault and stopPropagation methods so that our two new properties
    //get set in addition to the calls to the original functions. The final boolean parameter
    //ensures that this function will only be overridden ONCE if this function is called more
    //than once per event.
    com.ibm.portal.utilities.overwriteProperty( e, "preventDefault", function () { 
        this.defaultPrevented = true;
        com.ibm.portal.utilities.callOverwrittenFunction( this, "preventDefault" );
    }, true );
    com.ibm.portal.utilities.overwriteProperty( e, "stopPropagation", function () { 
        this.propagationStopped = true;
        com.ibm.portal.utilities.callOverwrittenFunction( this, "stopPropagation" );
    }, true );
    
    ibm.portal.debug.exit( "normalizeEvent", [e] );
    return e; // Event
}

dojo.declare( "com.ibm.portal.aggregation.event.EventFilter",
              null,
              {
                // summary: Base interface for a filter. Each implementation must implement
                //      the doFilter function.
                // description: Passes the given event through to give the implementation of
                //      this filter a chance to alter or handle the event in some way. 
                
                doFilter: function (e, eventSource, elementID) {
                    // summary: Filter the given event.
                    // description: Allows this filter to handle or alter the event. If this function
                    //      returns true, processing is stopped and the event is considered "handled".
                    // e: the event object
                    // eventSource: the HTML source element for the event
                    // elementID: 
                    dojo.unimplemented( "com.ibm.portal.aggregation.event.EventFilter.doFilter" );
                }   
              } 
);

dojo.declare( "com.ibm.portal.aggregation.event.PortletEventFilter",
              com.ibm.portal.aggregation.event.EventFilter,
              {
                // summary: Extends the EventFilter interface to provide some portlet specific
                //      capabilities.
                
                triggerPortletUpdate: function ( /*String*/ url, /*String*/portletId ) {
                    // summary: Sends the request for updating a portlet to the given url.
                    // description: Uses the PortletFragmentService to request new portlet markup
                    //      based on the state encoded in the given url.
                    // url: the url to update the portlet with
                    
                    var serviceReq = new com.ibm.portal.services.PortletFragmentService();
                    serviceReq.getPortletInfo( portletId, url );
                },
                isPortalUrl: function ( /*String*/ url ) {
                    // PK72027
                    // summary: Tests the given url to see if it is a portal uri.
                    // description: The given url string is checked to see if it begins with one of
                    //      portal uri strings defined in ibmPortalConfig.
                    // url: the url to test
                    // returns: true if this url matches a portal uri, otherwise false
                    ibm.portal.debug.entry( "PortletEventFilter.isPortalUrl", [ url ] );

                    // if this url is just a query or something to append onto the top.location, then it's a Portal URL
                    // also, if it's of the form http://...., then it'll get picked up later as an external URL and handled there
                    var retVal = false;

                    if(url.length > 0 && url.charAt(0) != "/" && url.indexOf( location.host) == -1) {
                    	retVal = true;
                    }
                    // does the url match one of our portal urls
                    if ( !retVal && ( url.indexOf( ibmPortalConfig["portalPublicURI"] ) >= 0 ) ||
                         ( url.indexOf( ibmPortalConfig["portalProtectedURI"] ) >= 0 ) ) {
                        retVal = true; // Boolean
                    }

                    ibm.portal.debug.exit( "PortletEventFilter.isPortalUrl", retVal );
                    return retVal;

               },
                isFriendlyUrl: function ( /*String*/ url ) {
                	// summary: Tests the given url to see if it is a friendly uri.
                     // description: The given url string is checked to see if it begins with one of
                     //      portal uri strings defined in ibmPortalConfig.
                     // url: the url to test
                     // returns: true if this url matches a portal uri, otherwise false
                     ibm.portal.debug.entry( "PortletEventFilter.isFriendlyUrl", [ url ] );
                     var retVal = false;
                     
                     var regx = new RegExp("(?:" + ibmPortalConfig["portalPublicURI"] + (ibmPortalConfig["isUserLoggedIn"] ? "|" + ibmPortalConfig["portalProtectedURI"] : "") + ")(/[^/]+)(?:/.*)*");
                     var tUrl = top.location.href;
                     
					 var umatch = null;
                     var tmatch = tUrl.match(regx);
                     if(tmatch && tmatch[1]) {
                     	 umatch = url.match(regx);
                     	 if(umatch && umatch[1] && umatch[1] == tmatch[1]) retval = false;
                     	 else {
                     	 	retval = true;
                     	 }
                     }
                    
                     ibm.portal.debug.exit( "PortletEventFilter.isFriendlyUrl", retVal );
                     return retval;
                }
              }  
);

dojo.declare( "com.ibm.portal.aggregation.event.EventFilterChain",
              null,
              {
                // summary: Controls the registration and execution of the filters associated
                //      with this filter chain.
                constructor: function () {
                    this._filters = new Array();
                },
                addFilter: function ( filter ) {
                    // summary: Register a filter at the end of the filter chain.
                    // filter: the filter to append
                    if ( !this._filters ) { this._filters = new Array(); }
                    this._filters.push( filter );
                },
                applyFilters: function (e, eventSource, elementID ) {
                    // summary: Execute the filter chain.
                    // description: Calls doFilter on every filter registered in this filter chain
                    //      until all filters have been called or one of the filters returns true.
                    // e: the event object
                    // returns: true if the event was handled, false if it was not
                    var i = 0;
                    var returnValue = false;
                    while ( i < this._filters.length && !returnValue )  {
                    	try {
                        	returnValue = this._filters[i].doFilter(e, eventSource, elementID);
                    	}
                    	catch(err) {
                    		com.ibm.portal.aggregation.error.ERROR_HANDLER.reportError(err.message, err.description, err.lineNumber);
                    	}
                        i = i + 1;      
                    }
                    
                    return returnValue; //Boolean
                }   
              } 
);

dojo.declare( "com.ibm.portal.aggregation.event.PortletEventHandler",
              null,
              {
                // summary: Examines all clicks that originate inside a portlet body
                //      and handles if appropriate.
                // description: Intercepts all clicks that originate from inside a 
                //      portlet and passes the event through the filter chain. If none of 
                //      filters can handle the event, it is left alone. Otherwise, it is
                //      cancelled and handled.
                constructor: function () {
                    this.onclickFilterChain = new com.ibm.portal.aggregation.event.EventFilterChain();
                    this.targetFilterChain = new com.ibm.portal.aggregation.event.EventFilterChain();
                },
                bubble: function ( /*EventFiltrChain*/filterChain, /*Event*/event, /*HTMLElement*/startElement, /*String*/idOfElemToStopAt ) {
                    // summary: Bubbles an event, executing the FilterChain for each element in the bubble path.
                    // filterChain: the filter chain to execute.
                    // event: the NORMALIZED event object
                    // startElement: the element to start bubbling from
                    // returns: true if any of the filters handles the event, false if none do
                    ibm.portal.debug.entry( "PortletEventHandler.bubble", [ filterChain, event, startElement ] );
                    
                    var retVal = false;
                    var element = startElement;
                    var utilities = com.ibm.portal.utilities;
                    
                    while ( element && !utilities.string.contains( element.getAttribute( "id" ), idOfElemToStopAt ) && !event.propagationStopped ) {
                            ibm.portal.debug.text( "Executing filters for: " + element );
                            //In IE, we have to overwrite any onclick handlers when we insert the markup because IE's event model 
                            //does not support the capture phase of the W3C event model. So before we apply the filters for each
                            //element, we need to check to see if any handlers have been overwritten and then restore the overwritten
                            //value when we're done (for the next click).
                            var overwrittenOnclick = null;
                            //First, restore any overwritten onclick handlers.
                            if ( utilities.getOverwrittenProperty( element, "onclick" ) ) {
                                overwrittenOnclick = utilities.restoreProperty( element, "onclick" );
                            }//If the event gets handled by one of our filters (a filter returns true), we don't want to 
                            //override that value when the next element return false. So once we get a true response,
                            //we just ignore any subsequent return values.
                            if ( !retVal ) {
                                retVal = filterChain.applyFilters(event, element, idOfElemToStopAt);
                            }
                            else {
                                filterChain.applyFilters(event, element, idOfElemToStopAt);
                            }    
                            
                            //Ok Now put it back.
                            if ( overwrittenOnclick ) { utilities.overwriteProperty( element, "onclick", overwrittenOnclick ); }
                            
                            element = element.parentNode;
                            ibm.portal.debug.text( "Next element in bubbling chain is: " + element );
                    }
                    
                    ibm.portal.debug.exit( "PortletEventHandler.bubble", retVal );
                    return retVal;      //Boolean
                },
                handle: function ( e ) {
                    // summary: Handles an event if it originiates from within a portlet.
                    // e: the event object
                    ibm.portal.debug.entry( "PortletEventHandler.handle", [ e ] );
                    
                    // get the event source
                    e = com.ibm.portal.aggregation.event.normalizeEvent( e );
                    var id = com.ibm.portal.utilities.findPortletIdByElement( e.target );
                  
                    ibm.portal.debug.text( "element ID to update: " + id + "; length=" + id.length)
                      
                    //Only want to handle clicks inside a portlet or a portlet toolbar.
                    if ( id.length > 0 ) {
                        //Bubbling simulation...
                        var element = e.target;
                        var activeForm = null;
                        if(element != null) {
					        	activeForm = com.ibm.portal.utilities.findFormByElement(e.target);
					        	if(activeForm) {
									var activeTrigger = { name: e.target.getAttribute( "name" ), value: e.target.getAttribute( "value" ) };
									com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER.setActiveSubmitTrigger(activeForm.getAttribute("id"), activeTrigger);
					        	}
						}
                        var val = this.bubble( this.onclickFilterChain, e, element, id );
                        
                        ibm.portal.debug.text( "Default action prevented after bubbling phase? " + e.defaultPrevented );
                        
                        if ( element == null ) {
                            // The parent node of any element in the chain is null, which means the current target probably no
                            // longer exists in the DOM. So, stop propagation and cancel the default action.
                            e.stopPropagation();
                            e.preventDefault();    
                        }
                        else {
                        
                            if ( !e.defaultPrevented ) {
                                ibm.portal.debug.text( "Executing target filters for: " + e.target );
                                val = this.bubble( this.targetFilterChain, e, e.target, id );
                            }
                            
                            //If the event was handled, stop the real bubbling phase from executing. If this is IE, where there is
                            //only a bubbling phase, we've already essentially executed this phase and there's no need for the event
                            //to propagate. In Firefox/W3C compliant browsers, since this function is executed during the capture 
                            //phase of the event handling AND we've already executed any onclick handlers, we want to make sure the
                            //onclick handler is not executed again if propagation is not stopped (i.e. when the event hits the target
                            //element in between the capture/bubble phases). 
                            if ( val || window.event ) {
                                e.stopPropagation();
                            } else if ( !e.propagationStopped ) {
                                ibm.portal.debug.text( 'replacing onclick handler: ' + e.target + "; " + e.target.onclick );
                                com.ibm.portal.utilities.overwriteProperty( e.target, "onclick", function (e) { 
                                                                                                   ibm.portal.debug.text( 'stopping existing onclick' ); 
                                                                                                   e.stopPropagation(); 
                                                                                                   com.ibm.portal.utilities.restoreProperty( this, "onclick" ); 
                                                                                                 } );    
                            }
                        }     
                        
                        if(activeForm) com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER.setActiveSubmitTrigger(activeForm.getAttribute("id"), null);   
                    }
        
                    ibm.portal.debug.exit( "PortletEventHandler.handle" );
                }
              }  
);

dojo.declare( "com.ibm.portal.aggregation.event.filters.JSLocationHrefFilter",
              com.ibm.portal.aggregation.event.PortletEventFilter,
              {
                // summary: Filters out any calls to set the location of the window via javascript.
                // description: Filters out and handles events which result in a javascript call to set the window location.
                //      We cannot allow these to execute unmodified since they would completely destroy our client-side
                //      aggregation. This filter checks for an onclick event or an href on the eventSource, parses
                //      out the target URL and loads that URL into the portlet. This filter stops processing if it 
                //      succesfully parses out a target URL and loads it into the portlet.
                doFilter: function ( e, eventSource, elementID ) {
                    // summary: Checks the eventSource element for any calls to set window.location
                    // e: the event object
                    // eventSource: the event source in the DOM
                    // returns: true if we succesfully parse out a target URL and load it into the portlet                  
                    ibm.portal.debug.entry( "JSLocationHrefFilter.doFilter", [ e, eventSource, elementID ] );
                    var targetUrl = "";
                    var jsString = "";
                    
                    if ( eventSource.getAttribute( "onclick" ) ) {
                        //IE departs from W3C DOM spec yet again and returns the function object here
                        jsString = eventSource.getAttribute( "onclick" );
                        
                        ibm.portal.debug.text( "onclick handler is " + jsString );
                        ibm.portal.debug.text( "typeof jsString is " + typeof(jsString ) );
            
                        if ( typeof( jsString ) == "function" ) {
                            ibm.portal.debug.text( "Converting function object into a string..." );
                            jsString = com.ibm.portal.utilities.eventHandlerToString( jsString );
                        }
                    }
                    else {
                        if ( eventSource.href ) {
                            jsString = eventSource.getAttribute( "href" );
                        }
                    }
                    
                    
                    ibm.portal.debug.text( "jsString is " + jsString );
                    
                    if ( jsString != "" ) {
                        targetUrl = this._parseTargetUrl( jsString );
                        
                        if (targetUrl!="")
                        {
                            ibm.portal.debug.text( "target URL is " + targetUrl );

                            var rv = this.isPortalUrl(targetUrl);
                            if (rv) {
                                // stop the event from propagating
                                e.preventDefault();
                                //e.stopPropagation();

                                this.triggerPortletUpdate( targetUrl, elementID );
                            }
                            
                            ibm.portal.debug.exit( "JSLocationHrefFilter.doFilter", [ rv ] );
                            return rv; // Boolean
                        }
                    }
                    
                    ibm.portal.debug.exit( "JSLocationHrefFilter.doFilter", [ false ] );
                    
                    return false; // Boolean    
                },
                
                _parseTargetUrl: function ( jsString ) {
                    // summary: Parse out the url that is being set to one of the location properties of the window.
                    // jsString: the javascript string to parse
                    if (((jsString.indexOf("self.location") != -1) ||
                          (jsString.indexOf("self.location.href") != -1) ||
                          (jsString.indexOf("location.href") != -1) ||
                          (jsString.indexOf("window.location") != -1)) ||
                          (jsString.indexOf("window.location.href") != -1 ) ||
                          (jsString.indexOf("top.location") != -1) ||
                          (jsString.indexOf("top.location.href") != -1) )
                    {
                        // if there's a semi-colon, get rid of everything after it
                        // gets rid of multiple statements in the onclick
                        var semiColonIndex = jsString.indexOf(";");
                        if (semiColonIndex > -1)
                        {
                            jsString.substring(0, semiColonIndex);
                        }
                            
                        // find starting quote char
                        var dblQuoteCharIndex = jsString.indexOf('"');
                        var quoteCharIndex = jsString.indexOf("'");
                    
                        var targetUrl = "";
                                
                        if (dblQuoteCharIndex == -1 && quoteCharIndex == -1)
                        {
                            //couldn't find the quote - just return
                            return;
                        }
                        else if (dblQuoteCharIndex == -1)
                        {
                            //code used single quotes
                            var endIndex = jsString.indexOf("'", quoteCharIndex+1);
                            targetUrl = jsString.substring(quoteCharIndex+1, endIndex);
                        }
                        else
                        {
                            //code used double quotes
                            var endIndex = jsString.indexOf('"', dblQuoteCharIndex+1);
                            targetUrl = jsString.substring(dblQuoteCharIndex+1, endIndex);
                        }
                        
                        return targetUrl; // String
                    }
                    
                    return ""; // String
                }   
              } 
);


dojo.declare( "com.ibm.portal.aggregation.event.filters.OnclickFilter",
              com.ibm.portal.aggregation.event.PortletEventFilter,
              {
                // summary: Handles any onclick events which do not set the window location.
                // description: Executes the onclick function and, based on the result, stops processing 
                //      or continues. If the onclick function returns false, the event is stopped and processing
                //      ended. Otherwise, processing will continue to the next filter.
                
                doFilter: function (e, eventSource, elementID ) {
                    // summary: Check for and execute the onclick event of the event source element.
                    // e: the event object
                    // eventSource: the source element
                    // returns true if the onclick handler indicates the event should be cancelled.
                    ibm.portal.debug.entry( "OnclickFilter.doFilter", [ e, eventSource, elementID ] );
                    
                    ibm.portal.debug.text( "onclick is " + eventSource.onclick );
                    
                    if ( eventSource.onclick ) {
                        var retVal = null; 
                        // this chain is used to keep track of multiple handlers that were
                        // added to this node's onclick event through dojo.connect
                        // the first handler attached is always on eventSource.onclick
                        // the chain only includes every handler after the first one
                        // and only the return value of the first one is kept 
                        var chain = eventSource._onclickChain;
                        if ( window.event ) {
                            retVal = eventSource.onclick();
                            if(chain) {
                            	for(var i = 0; i < chain.length; i++) {
                            		chain[i]();
                            	}
                            }
                        }
                        else {
                            retVal = eventSource.onclick(e);
                            if(chain) {
                            	for(var i = 0; i < chain.length; i++) {
                            		chain[i](e);
                            	}
                            }
                        }
                        
                        ibm.portal.debug.text( "return value from onclick is: " + retVal, "OnclickFilter.doFilter" );
                        
                        //If false, the event is handled.
                        if ( retVal == false ) {  
                            e.preventDefault();
                            //e.stopPropagation();
                            
                            ibm.portal.debug.exit( "OnclickFilter.doFilter", [ true ] );
                            return true;
                        }
                    }
                    
                    //TODO: what to do if the retVal is true???? We've already executed the onclick
                    //but we don't want to stop the event, this will cause the onclick to get executed
                    //again when the event reaches the anchor element.
            
                    ibm.portal.debug.exit( "OnclickFilter.doFilter", [ false ] );
                    return false;
                }   
              }  
);

dojo.declare( "com.ibm.portal.aggregation.event.filters.DefaultActionFilter",
			  com.ibm.portal.aggregation.event.PortletEventFilter,
			  {
			  	// summary: Handles any anchor elements that are clicked on.
                // description: It loads the specified URL into the portlet or, if the href contains javascript, 
                //      executes the javascript.
              
                doFilter: function ( e, eventSource, elementID ) {
                	ibm.portal.debug.entry( "DefaultActionFilter.doFilter", [ e, eventSource, elementID ] );
                	//If the default action was not canceled at some point in handling the event, go
					//ahead and execute our new default action handler.
					var defaultAction = eventSource.defaultAction;
					var retVal = false;
					
					if ( defaultAction ) {
						ibm.portal.debug.text( "Found default action: " + defaultAction );
						if ( !e.defaultPrevented ) {
							defaultAction.apply( eventSource, [ e ] );	
						}	
						
						//Prevent the browser from executing the default action and stop this event
						//from bubbling. The event handlers have already had a chance to handle any
						//original events. The default action is a "browser" action and so shouldn't be
						//handled by anyone except us.
						e.preventDefault();
						e.stopPropagation();
						retVal = true;
					}	
					
					ibm.portal.debug.exit( "DefaultActionFilter.doFilter" );
                }	
			  }
);

dojo.declare( "com.ibm.portal.aggregation.event.filters.AnchorFilter",
              com.ibm.portal.aggregation.event.PortletEventFilter,
              {
                // summary: Handles any anchor elements that are clicked on.
                // description: It loads the specified URL into the portlet or, if the href contains javascript, 
                //      executes the javascript.
              
                doFilter: function ( e, eventSource, elementID ) {
                    // summary: If the eventSource is an anchor, load the href into the portlet.
                    // e: the event object
                    // eventSource: the event source element
                    // returns true if an href value is succesfully loaded into the portlet                 
                    ibm.portal.debug.entry( "AnchorFilter.doFilter", [ e, eventSource, elementID ] );
                    
                    var nodeName = eventSource.nodeName.toLowerCase();
                    if (nodeName == 'a' && e.button == 0) {
                        var url = eventSource.getAttribute('href');
                        
                        ibm.portal.debug.text( "anchor's href is " + url );
                        var target = eventSource.getAttribute('target');
                        
                        //allow the browser to handle this since we need it in a new window.
                        if ( target != null && ( target == "_blank" || target == "_parent" ) ) {
                            return false;
                        } 
                        
                        if (url != null) {
                            
                            if (url.toLowerCase().indexOf("javascript:")==0)
                            {
                                //Stop the event from propagating.
                                //ibm.portal.Utilities.stopEvent(e);
                                e.preventDefault();
                                //e.stopPropagation();
                                //IE url encodes all href values, including javascript href values. This can cause some bizarre results when evaluating 
                                //this js href. So we're going to "decode" any encoded values. WE may need to do more here. For now, this is the only case
                                //we've run into.
                                if ( dojo.isIE ) {
                                    url = url.replace( /%20/g, ' ' );
                                }
                                ibm.portal.debug.text( "Evaluating javascript href: " + url );
                                eval( url );
                                ibm.portal.debug.exit( "AnchorFilter.doFilter", true );
                                return true;
                            }
                            else
                            {
                                var rv = this.isPortalUrl(url);
                                if (rv) {
                                    // stop the event from propagating
                                    e.preventDefault();
                                    //e.stopPropagation();

                                    this.triggerPortletUpdate( url, elementID );
                                }
								ibm.portal.debug.exit( "AnchorFilter.doFilter", rv );
                                return rv;
                            }
                        }
                    }
                    
                    ibm.portal.debug.exit( "AnchorFilter.doFilter", false );
                    return false;   
                }             
              } 
);

com.ibm.portal.aggregation.event.PORTLET_EVENT_HANDLER = new com.ibm.portal.aggregation.event.PortletEventHandler();

//Regsiter all of our filters. Please note that order IS important! The JSLocationHrefFilter should 
//always be executed before the OnclickFilter and AnchorFilter, since allowing the location to be set 
//completely breaks client side aggregation.
com.ibm.portal.aggregation.event.PORTLET_EVENT_HANDLER.onclickFilterChain.addFilter( new com.ibm.portal.aggregation.event.filters.JSLocationHrefFilter() );
com.ibm.portal.aggregation.event.PORTLET_EVENT_HANDLER.onclickFilterChain.addFilter( new com.ibm.portal.aggregation.event.filters.OnclickFilter() );
com.ibm.portal.aggregation.event.PORTLET_EVENT_HANDLER.targetFilterChain.addFilter( new com.ibm.portal.aggregation.event.filters.AnchorFilter() );
com.ibm.portal.aggregation.event.PORTLET_EVENT_HANDLER.targetFilterChain.addFilter( new com.ibm.portal.aggregation.event.filters.DefaultActionFilter() );

//Register the onclick event handler.
if (document.addEventListener)
{
    document.addEventListener("click", dojo.hitch( com.ibm.portal.aggregation.event.PORTLET_EVENT_HANDLER, "handle"), true);
}
else if (document.attachEvent)
{
    document.attachEvent("onclick", dojo.hitch( com.ibm.portal.aggregation.event.PORTLET_EVENT_HANDLER, "handle") );
}

dojo.declare( "com.ibm.portal.aggregation.event.PortletEventDampener",
              null,
              {
                  // summary: Creates a handler that can be used to suppress certain events from
                  //        bubbling. Optionally, the default action for the event can also be 
                  //        cancelled, based on the cancelTest, if one is specified.
                  // description: Event handler that can be attached/added to any DOM element to 
                  //        ensure that the event does not bubble any higher than that DOM
                  //        element. If a cancelTest is specified in the constructor, the boolean
                  //        result of that function execution is used to determine if the default
                  //        action of the event is also cancelled. 
                  constructor: function(/*Function?*/cancelTest) {
                      // summary: Sets the cancelTest if one is specified. 
                      // cancelTest: function to determine wether to cancel the default action for
                      //        an event. a return value of true will cancel the default action. the
                      //        lone parameter will be the event object (regardless of browser).
                      this._cancelDefault = cancelTest;
                      ibm.portal.debug.text( "Cancel test is: " + this._cancelDefault );
                  },
                  dampen: function ( /*Event?*/e) {
                      // summary: Suppresses the event to which this handler is attached.
                      // e: the event object (only expected in Firefox).
                      ibm.portal.debug.entry( "PortletEventDampener.dampen", [ e ] );
                      
                      e = com.ibm.portal.aggregation.event.normalizeEvent(e);
                      ibm.portal.debug.text( "Dampening event: " + e );
                      e.stopPropagation();
                      if ( this._cancelDefault && this._cancelDefault(e) ) {
                          e.preventDefault();
                      }
                      //return false;
                      
                      ibm.portal.debug.exit( "PortletEventDampener.dampen" );
                  }
              }
);


}

if(!dojo._hasResource["com.ibm.portal.aggregation.event.markup"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.event.markup"] = true;
dojo.provide( "com.ibm.portal.aggregation.event.markup" );



dojo.declare( "com.ibm.portal.aggregation.event.markup.PortletMarkupRewriter",
			  null,
			  {
			  	needsRewrite: function ( /*HTMLElement*/elem ) {
					// summary: Indicates whether or not this rewriter needs to handle this node.
					// elem: the current markup element
					// returns: true if the Rewriter can handle the element, false if it cannot
					dojo.unimplemented( "com.ibm.portal.aggregation.event.PortletMarkupRewriter.needsRewrite" );
				},
				rewrite: function ( /*HTMLElement*/elem ) {
					// summary: Rewrites the HTMLElement.
					// elem: the current markup element.
					dojo.unimplemented( "com.ibm.portal.aggregation.event.PortletMarkupRewriter.rewrite" );
				},
				selectNodes: function ( /*HTMLElement*/rootElement ) {
					// summary: Selects the nodes that need to be rewritten.
					// rootElement: the root node to search from
					// returns: an array of the nodes that need to be rewritten
					dojo.unimplemented( "com.ibm.portal.aggregation.event.PortletMarkupRewriter.selectNodes" );
				}
			  }
);

dojo.declare( "com.ibm.portal.aggregation.event.markup.PortletMarkupEventHandler",
			  null,
			  {
			  	// summary: Traverses any new portlet markup and gives each registered rewriter a
				//		chance to alter each node in the new markup individually.
				// description: Starts at the base portlet element and traverses each node below the
				//		root element, passing each node into the rewriters registered with this handler.
				
				constructor: function () {
				  	this._rewriters = new Array ();
					this._portletMarkupUpdatedHandle = dojo.subscribe( "/portal/PortletMarkup/updated",
											 	this,
												function ( /*Object*/args ) {
													var element = args[["newMarkup"]];
													this.process( element );
												} 
					);
			    },
				addRewriter: function( /*PortletMarkupRewriter*/rewriter) {
					// summary: Register a new rewriter to this handler.
					// rewriter: the rewriter to register.
					this._rewriters.push( rewriter );
				},
				process: function ( rootElement ) {
					// summary: Processes the updated portlet markup.
					// rootElement: the root markup element of the portlet markup.
					ibm.portal.debug.entry( "PortletMarkupEventHandler.process", [ rootElement ] );
					
					ibm.portal.debug.text( "Root element's html: " + rootElement.innerHTML );
					
					dojo.forEach( this._rewriters, 
  							      function ( rewriter ) {
										var elems = rewriter.selectNodes( rootElement );
									   	for ( var i = 0; i < elems.length; i++ ) {
										   	if ( rewriter.needsRewrite( elems[i] ) ) {
												rewriter.rewrite( elems[i] );
											}
										}		
								  }  
					);
						
					ibm.portal.debug.exit( "PortletMarkupEventHandler.process");	
				}
			  }  
);

dojo.declare( "com.ibm.portal.aggregation.event.markup.OnclickRewriter",
			  com.ibm.portal.aggregation.event.markup.PortletMarkupRewriter,
			  {
			  	// summary: Rewrites onclick events to call our PortletEventHandler first.
				// description: Overwrites all onclick events to call our PortletEventHandler first
				//		so that we get a chance to intercept those clicks in IE. The IE event model
				//		only allows for the bubbling phase of the W3C event model. So, in order to
				//		"intercept" clicks inside a portlet, we have to take over the onclick event
				//		at the lowest level possible in the DOM.
				
				needsRewrite: function ( node ) {
					// summary: Indicates whether or not this rewriter needs to handle this node.
					// elem: the current markup element
					// returns: true if the Rewriter can handle the element, false if it cannot
					ibm.portal.debug.entry( "OnclickRewriter.needsRewrite", [ node ] );
                    var retVal = (( node.nodeName == "a" ) || ( node.onclick != null ) ); 
					ibm.portal.debug.exit( "OnclickRewriter.needsRewrite", [ retVal ] );
					return retVal;
				},
				rewrite: function ( element ) {
					// summary: Rewrites the onclick event to point to the PortletEventHandler.
					// element: the current markup element
					ibm.portal.debug.entry( "OnclickRewriter.rewrite", [ element ] );	
				
					com.ibm.portal.utilities.overwriteProperty( element, "onclick", function () { 
							var restoreValue = null;                            
                            //Clear out the overwritten value for the onclick property so that the click 
                            //handler correctly handles any onclick handlers. We need to put it back in 
                            //case this handler is called multiple times.
                            if ( com.ibm.portal.utilities.getOverwrittenProperty(this, "onclick" ) != null ) { 
								restoreValue = com.ibm.portal.utilities.restoreProperty( this, "onclick" );
							} else { 
								restoreValue = this.onclick;
                                this.onclick = null; 
							}
                             
                            com.ibm.portal.aggregation.event.PORTLET_EVENT_HANDLER.handle();
                            
                            if ( restoreValue != null ) {
                                com.ibm.portal.utilities.overwriteProperty( this, "onclick", restoreValue );
                            }
					}, true );
					
					ibm.portal.debug.exit( "OnclickRewriter.rewrite" );
				},
				selectNodes: function ( /*HTMLElement*/rootElement ) {
					// summary: Selects the nodes that need to be rewritten.
					// rootElement: the root node to search from
					// returns: an array of the nodes that need to be rewritten
					return dojo.query( "[onclick]", rootElement );
				}		
			  } 
);

dojo.declare( "com.ibm.portal.aggregation.event.markup.OnloadRewriter",
			  com.ibm.portal.aggregation.event.markup.PortletMarkupRewriter,
			  {
			  	// summary: Rewrites onload event handlers to run when script handling is finished.
				// description: Overwrites all onload handlers in the DOM so that we can defer
				// 		running them until the portlet markup is updated and scripts have been run.
				
				constructor: function() {
					this._onloadHandlers = new Array();
				},
				needsRewrite: function ( node ) {
					// summary: Indicates whether or not this rewriter needs to handle this node.
					// elem: the current markup element
					// returns: true if the Rewriter can handle the element, false if it cannot
                    ibm.portal.debug.entry( "OnloadRewriter.needsRewrite", [ node ] );
                    var retVal = false;
                    try {
                    	retVal = ( node.nodeName.toUpperCase() != "OBJECT" && node.onload && node.onload != "" );
                    }
                    catch (e) {
                    	ibm.portal.debug.text( "Unable to check for onload! Assuming none used for this tag.");
                    }	 
					ibm.portal.debug.exit( "OnloadRewriter.needsRewrite", retVal );
					return retVal;
				},
				rewrite: function ( element ) {
					// summary: Rewrites the onload event handler to run when the portlet is ready.
					// element: the current markup element
					ibm.portal.debug.entry( "OnloadRewriter.rewrite", [ element ] );
					
					var nodeId = element.id;
					if(!nodeId || nodeId.length == 0) {
						nodeId = dojo.dnd.getUniqueId();
						element.setAttribute("id", nodeId);
					}
					
					this._onloadHandlers.push( {handler: element.onload, id: nodeId } );
					
					element._oldOnload = element.onload;
					element.setAttribute("onload", null);
					element.onload = null;

					ibm.portal.debug.exit( "OnloadRewriter.rewrite" );
				},
				process: function ( rootElement ) {
					// summary: Processes the updated portlet markup.
					// rootElement: the root markup element of the portlet markup.
					ibm.portal.debug.entry( "OnloadRewriter.process", [ rootElement ]);
					
					ibm.portal.debug.text( "Root element is: " + rootElement.nodeName );
					
					//var elems = com.ibm.portal.utilities.html.getElementsByTagNames(rootElement, "body", "frame", "frameset", "img", "link", "script");
					var elems = dojo.query( "body[onload]", rootElement);
					elems = elems.concat( dojo.query( "frame[onload]", rootElement ) );
					elems = elems.concat( dojo.query( "frameset[onload]", rootElement ) );
					elems = elems.concat( dojo.query( "img[onload]", rootElement ) );
					elems = elems.concat( dojo.query( "link[onload]", rootElement ) );
					elems = elems.concat( dojo.query( "script[onload]", rootElement ) );
					for( var i = 0; i < elems.length; i++ ) {
						if(!elems[i]) continue;
						//if(this.needsRewrite(elems[i])) {
							this.rewrite( elems[i] );
						//}
					}
					// replaced the below commented code since we don't need to walk the entire DOM, just certain elements
					/*
					// PK70387 for ( child in rootElement.childNodes ) {
					for ( var child = 0; child < rootElement.childNodes.length; child++ ) {
						var elem = rootElement.childNodes[child];
						
						ibm.portal.debug.text( "Examining chlid element: " + elem.nodeName );
						
						if ( !elem ) {
							continue;	
						}
						
					   	if ( this.needsRewrite( elem ) ) {
					   		this.rewrite( elem );
					   	}	
						
						
						if ( elem.childNodes && elem.childNodes.length > 0) {
							this.process( elem );
						}
					}
					*/	
					
					ibm.portal.debug.exit( "OnloadRewriter.process" );
				},
				runHandlers: function () {
					ibm.portal.debug.entry( "OnloadRewriter.runHandlers");
					for(var idx = 0; idx < this._onloadHandlers.length; idx++) {
						var node = dojo.byId(this._onloadHandlers[idx].id);
						if(node) {
							try {
								ibm.portal.debug.text( "OnloadRewriter.runHandlers " + this._onloadHandlers[idx].handler);
								if ( this._onloadHandlers[idx].handler.apply ) {
									this._onloadHandlers[idx].handler.apply( node );
									node.onload = this._onloadHandlers[idx].handler;
									node.setAttribute("onload", this._onloadHandlers[idx].handler);
								}	
							}
							catch(err) {
								// handle gracefully as if this were in its own thread
								ibm.portal.debug.text( "An error occurred during this execution: " + err.message );
							}
						}
					}
					ibm.portal.debug.exit( "OnloadRewriter.runHandlers");
				}
			  } 
);

com.ibm.portal.aggregation.event.markup.PORTLET_MARKUP_EVENT_HANDLER = new com.ibm.portal.aggregation.event.markup.PortletMarkupEventHandler();

if ( dojo.isIE ) {
	com.ibm.portal.aggregation.event.markup.PORTLET_MARKUP_EVENT_HANDLER.addRewriter( new com.ibm.portal.aggregation.event.markup.OnclickRewriter() );
}	

}

if(!dojo._hasResource["com.ibm.portal.aggregation.event.SemanticTagging"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.event.SemanticTagging"] = true;
dojo.provide( "com.ibm.portal.aggregation.event.SemanticTagging" );

dojo.declare( "com.ibm.portal.aggregation.event.SemanticTagging",
			  null,
			  {
				  constructor: function () {
				  	// summary: Triggers the semantic tag service parsing process everytime the portlet
					//		markup is updated.
					
					this._portletMarkupUpdatedHandle = dojo.subscribe( "/portal/PortletMarkup/updated",
											 	this,
												function ( /*Object*/args ) {
													if ( typeof( SemTagSvc) != "undefined" && !ibmPortalConfig["disableSemanticTagging"] ) {
														ibm.portal.debug.text( "Calling semantic tag service..." );
														var element = args["newMarkup"];
														SemTagSvc.parseDom( null, element );
													}	
												} 
					);	
				  }
			  }   
);

com.ibm.portal.aggregation.event.SEMANTIC_TAG_LISTENER = new com.ibm.portal.aggregation.event.SemanticTagging();

}

if(!dojo._hasResource["com.ibm.portal.aggregation.event.DefaultActionManager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.event.DefaultActionManager"] = true;
dojo.provide( "com.ibm.portal.aggregation.event.DefaultActionManager" );

dojo.require( "com.ibm.portal.utilities" );
dojo.require( "com.ibm.portal.aggregation.event" );

dojo.declare( "com.ibm.portal.aggregation.event.DefaultActionManager",
			  null,
			  {
			  	// summary: Allows for "hijacking" an HTML element's default action.
				// description: Most HTML elements that can be interacted with by a user have
				//		default actions. For example, an ANCHOR element's default action is to
				//		follow the href associated with the element. In some cases, we want to 
				//		take complete control of an element (e.g. an INPUT of type SUBMIT). This
				//		manager allows us to override the given event and provide a new handler 
				//		for the default action. The default action is ALWAYS prevented. If the
				//		event handlers don't cancel the default action (i.e. return false), we
				//		call the new default action handler instead of allowing the actual default
				//		action to execute.
				
				setDefaultAction: function ( /*HTMLElement*/element, /*String*/eventName, /*Function*/handler ) {
					// summary: Sets a new default action for the given HTML element and given event name.
					// element: the HTML element we want to take over the default action for
					// eventName: the name of the event we want to control
					// handler: the new default action handler to call instead
					ibm.portal.debug.entry( "DefaultActionManager.setDefaultAction", [ element, eventName, handler ]);
					
					//Set the new default action handler as a property on the object and overwrite the
					//event handler with our new function.
					element.defaultAction = handler;
					//com.ibm.portal.utilities.overwriteProperty( element, eventName, this._newDefaultActionWrapper( eventName ) )
					
					ibm.portal.debug.exit( "DefaultActionManager.setDefaultAction" );
				},
				_newDefaultActionWrapper: function (/*String*/eventName ) {
					// summary: Creates the new handler which wrappers the new default action.
					// eventName: the event name we are overwriting
					// returns: a new Function object
					ibm.portal.debug.entry( "DefaultActionManager._newDefaultActionWrapper" );
					
					var wrapper = function ( /*Event*/evt ) {
						ibm.portal.debug.entry( "NewDefaultAction", [ evt ]);
						
						//Get the normalized event object.
						evt = com.ibm.portal.aggregation.event.normalizeEvent( evt );
						
						//If there was an existing event handler for the given event, we want to restore the original
						//value before passing the event to the event handler. This way the event handler will 
						//automatically handle the original event and call preventDefault in the appropriate
						//places.
						if ( com.ibm.portal.utilities.getOverwrittenProperty(this,eventName) != null ) {
							com.ibm.portal.utilities.restoreProperty( this, eventName );
							com.ibm.portal.aggregation.event.PORTLET_EVENT_HANDLER.handle( evt );
						}
						
						//If the default action was not canceled at some point in handling the event, go
						//ahead and execute our new default action handler.
						if ( !evt.defaultPrevented ) {
							this.defaultAction( evt );	
						}	
						
						//Prevent the browser from executing the default action and stop this event
						//from bubbling. The event handlers have already had a chance to handle any
						//original events. The default action is a "browser" action and so shouldn't be
						//handled by anyone except us.
						evt.preventDefault();
						evt.stopPropagation();
						
						ibm.portal.debug.exit( "NewDefaultAction", [ false ] );
						return false; // Boolean
					}
					
					ibm.portal.debug.exit( "DefaultActionManager._newDefaultActionWrapper", [ wrapper ] );
					return wrapper;	// Function
				}	
			  }
);

com.ibm.portal.aggregation.event.DEFAULT_ACTION_MANAGER = new com.ibm.portal.aggregation.event.DefaultActionManager();

}

if(!dojo._hasResource["com.ibm.portal.aggregation.event.NamedAnchorHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.event.NamedAnchorHandler"] = true;
dojo.provide( "com.ibm.portal.aggregation.event.NamedAnchorHandler");

dojo.require( "com.ibm.portal.utilities.html" );
dojo.require( "com.ibm.portal.aggregation.event" );

dojo.declare( "com.ibm.portal.aggregation.event.NamedAnchorHandler",
			  null,
			  {
			  	// summary: Handles the case where an anchor points to a place inside the current 
			  	//		document.
			  	// description: This handler is specifically designed for the JSP based portlet
			  	//		helps which open in a new window. In Firefox, clicking on one of the 
			  	//		named anchors actually causes a  full page refresh. So this handler 
			  	//		emulates the same behavior while preventing a full page refresh.
			  	
			  	handle: function ( /*Event?*/e ) {
			  		// summary: Handles a click event, looking for named anchors as the target.
			  		// e: the event object.
			  		ibm.portal.debug.entry( "NamedAnchorHandler.handle", [ e ] );
			  		
			  		e = com.ibm.portal.aggregation.event.normalizeEvent(e);
			  		
			  		//Check to see if an anchor link is the target of this event.
			  		var targetElement = e.target;
			  		if ( targetElement.nodeName.toLowerCase() == "a" ) {
			  			var hrefValue = targetElement.getAttribute( "href" );
			  			//Check to see if the href is pointing at an internal link.
			  			if ( hrefValue.indexOf("#") == 0  ) {
			  				//Chop off the hash mark.
			  				var namedValue = hrefValue.substring(1);
			  				//Find the targetted element, calculate it's position, and 
			  				//scroll to that position.
			  				var elem = targetElement.ownerDocument.getElementsByName( namedValue )[0];
			  				//Need to calculate the position in a window/document independent manner.
			  				//The dojo functions for doing this are very dependent on the document/window.
			  				var x = com.ibm.portal.utilities.html.getX( elem );
			  				var y = com.ibm.portal.utilities.html.getY( elem );
			  				//dojo.html.getElementWindow( elem ).scrollTo( x, y );
							// RES: could this use dijit.scrollIntoView(elem) ?
			  				dijit.getDocumentWindow( elem.ownerDocument ).scrollTo( x, y );

							//Cancel the event.
			  				e.stopPropagation();
			  				e.preventDefault(); 
			  			}
			  		}
			  		
			  		ibm.portal.debug.exit( "NamedAnchorHandler.handle" );
			  	}
			  }
);			  

}

if(!dojo._hasResource["com.ibm.portal.aggregation.event.common"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.event.common"] = true;
dojo.provide("com.ibm.portal.aggregation.event.common");

dojo.require( "com.ibm.portal.aggregation.event.markup" );
dojo.require( "com.ibm.portal.aggregation.event.SemanticTagging" );
dojo.require( "com.ibm.portal.aggregation.event.DefaultActionManager" );
dojo.require( "com.ibm.portal.aggregation.event.NamedAnchorHandler" );

}

if(!dojo._hasResource["com.ibm.portal.aggregation.forms"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.forms"] = true;
dojo.provide( "com.ibm.portal.aggregation.forms" );


dojo.require( "com.ibm.portal.aggregation.event" );
dojo.require( "com.ibm.portal.aggregation.event.common" );

dojo.declare( "com.ibm.portal.aggregation.forms.actions.FormRewriteAction",
			  null,
			  {
			  	// summary:	Base interface for an action. Each implementation must implement
 				//		the apply function.
				// description: Passes the given form element through to give the implementation of
				//		this action to modify the form element. 
			  	
				apply: function ( /*HTMLElement*/formElem ) {
					// summary: Modify the given form element
					// description: Allows this action to the form. 
					// formElem: the form element 
					dojo.unimplemented( "com.ibm.portal.aggregation.FormRewriteAction.apply" );
				}	
			  }	
);

dojo.declare( "com.ibm.portal.aggregation.forms.actions.SubmitInputOverrideAction",
			  com.ibm.portal.aggregation.forms.actions.FormRewriteAction,
			  {
			  	// summary: Base interface for rewriting any input of type "submit" (or button) in Firefox.
			  	
			  	newSubmitOnclickHandler: function ( /*HTMLElement*/ submitInput ) {
					// summary: Creates a new onclick handler for a "submit" type input.
					// description: The onclick handler must handle any existing handlers, as well
					//		as overriding the form submission by calling our portlet form handler instead
					//		of letting the browser handle the form submission.
					// submitInput: the input element of type submit (or button in firefox) 
					// returns: an onclick Function object
					ibm.portal.debug.entry( "FormRewriteAction.newSubmitOnclickHandler", [ submitInput ] );
					
					var name = submitInput.getAttribute( "name" );
					var formElem = submitInput.form;
					var onSubmitFn = formElem.onsubmit; 
					var onclickFn = function(evt) { 
						ibm.portal.debug.entry( name + ".onclick", [evt] );
						
						var retVal = (formElem.submitted != true );
						
						//If there is an onsubmit function, we need to execute it in the correct context since
						//it's return value can affect whether or not we actually allow the form to be submitted.
						if ( onSubmitFn && retVal ) {
							ibm.portal.debug.text( "Found onsubmit function: " + onSubmitFn );
							try { 
								retVal = onSubmitFn.apply( evt.target.form );
							}
							catch ( err ) {
								//In the browser, if a javascript error occurs in the onsubmit handler, the browser
								//appears to just ignore the onsubmit function altogether. So we will mirror that behavior
								//here, for lack of a better solution.								
								ibm.portal.debug.text( "Error encountered executing onsubmit handler... pretending we never saw it since this seems to match the browser behavior");
								retVal = true;
							}
							
							ibm.portal.debug.text( "Return value from onsubmit function: " + retVal );		
						}
						
						if ( retVal != false ) {
							var target = formElem.getAttribute("target");
							ibm.portal.debug.text( "Found form target: " + target );
							if(target && target != "" && target != "_self" ) {
								ibm.portal.debug.text( "Executing old submit function: " + formElem._oldsubmit ); 
								if ( formElem._oldsubmit ) {
									formElem._oldsubmit();
								} else {
									com.ibm.portal.utilities.callOverwrittenFunction( formElem, "submit" );
								}
								return false;
							}
							
							//Set the active submit trigger so that we can encode it into the form. This is part of the form submission
							//"magic" that the browser usually handles.
							com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER.setActiveSubmitTrigger( formElem.getAttribute( "id" ), { name: evt.target.getAttribute( "name" ), value: evt.target.getAttribute( "value" ) } );
                            //Have to use getAttributeNode here, otherwise IE returns the INPUT object instead of the form action when there's an INPUT named 'action'
							com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER.handle( formElem.getAttributeNode( 'id' ).value, formElem.getAttributeNode( 'action' ).value );
						}
							
						ibm.portal.debug.exit( name + ".onclick", [ false ] );
						//evt.preventDefault();
						return false;
					};
				
					ibm.portal.debug.exit( "FormRewriteAction.newSubmitOnclickHandler", [ onclickFn ] );
					return onclickFn; // Function
				}
			  }
);

dojo.declare( "com.ibm.portal.aggregation.forms.actions.SubmitOverrideAction",
			  com.ibm.portal.aggregation.forms.actions.FormRewriteAction,
			  {
			  	// summary: Overrides the submit function associated with a form to call our portlet form handler. 
				// description: A form's action cannot execute normally since it will return an 
				//		atom feed as a response so we change the submit function to pass the action url to
				//		the form handler object which is responsible for actually submitting the form. This handles
				//		the case where the form is submitted via javascript.
				
				apply: function ( /*HTMLElement*/formElem ) {
					// summary: Modify the given form element
					// description: Allows this action to the form. 
					// formElem: the form element 
					ibm.portal.debug.entry( "SubmitOverrideAction.apply", [ formElem ] );

					//Give the form an ID if it doesn't already have one.
					var formId = formElem.getAttribute("id");
					if ( !formId || formId.length == 0 ) {
						formId = dojo.dnd.getUniqueId();
						formElem.setAttribute("id", formId );
					}
					
					//Override the submit function to handle the case where the action is set via JS and then
					//submitted via JS.
					//formElem._oldsubmit = formElem.submit;
					try{
						//Have to use getAttributeNode here, otherwise IE returns the INPUT object instead of the form action when there's an INPUT named 'action'
						//formElem.submit = function () { com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER.handle( this.getAttributeNode( 'id').value, this.getAttributeNode( 'action' ).value ); }
						com.ibm.portal.utilities.overwriteProperty(formElem, 
																   "submit", 
																   function () { com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER.handle( this.getAttributeNode( 'id').value, this.getAttributeNode( 'action' ).value ); },
																   true
																   );
					}
					catch(err) {	// IE error if form.submit is actually a child element of the form
						var attr = formElem.getAttributeNode( 'submit' );
						if(attr != null) {
							formElem._oldsubmit = attr.value;
							var newAttr = document.createAttribute( 'submit' );
							newAttr.value = "function () { com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER.handle( this.getAttributeNode( 'id').value, this.getAttributeNode( 'action' ).value ); }";
							formElem.setAttributeNode(newAttr);
						}
						else {
							var tempForm = document.createElement("form");
							com.ibm.portal.utilities.setOverwrittenProperty( formElem, "submit", tempForm.submit );
						}
					}
					ibm.portal.debug.exit( "SubmitOverrideAction.apply" );	
				}	
			  }	  
);

dojo.declare( "com.ibm.portal.aggregation.forms.actions.AddContentTypeInput",
			  com.ibm.portal.aggregation.forms.actions.FormRewriteAction,
			  {
			  	// summary: Required only for IE. Ensures that the content type of
				//		the feed response is text/xml since that is the only content-type
				//		for which IE sets the responseXML property on the XHR.	
			  
			  	apply: function ( formElem ) {
					// summary: Modify the given form element
					// description: Allows this action to the form. 
					// formElem: the form element
					ibm.portal.debug.entry( "AddContentTypeInput.apply", [ formElem ] );
					ibm.portal.debug.text( "Adding content type input to form " + formElem.name );
						
					var input = document.createElement( "<input name='ibm.web2.contentType' value='text/xml' type='hidden' />" );
					formElem.appendChild( input );
					
					ibm.portal.debug.exit( "AddContentTypeInput.apply" );
				}
			  }  
);

dojo.declare( "com.ibm.portal.aggregation.forms.actions.TagSubmitInputs",
			  com.ibm.portal.aggregation.forms.actions.SubmitInputOverrideAction,
			  {
			  	// summary: Tag all form submit elements with an onclick handler so that we can
				//		identify the element that triggered the submit when a form has multiple
				//		submit inputs.
				
				apply: function ( formElem ) {
					// summary: Modify the given form element
					// description: Allows this action to the form. 
					// formElem: the form element
					ibm.portal.debug.entry( "Portlet._tagSubmitInputs", [ formElem ] );
					
                    var inputs = com.ibm.portal.utilities.html.getElementsByTagNames( formElem, "input", "button" );
                    					
					ibm.portal.debug.text( "found " + inputs.length + " inputs associated with this form." );
					
					for ( input in inputs ) {
						
						ibm.portal.debug.text( "Processing input: " + inputs[input] );
						
						if ( !inputs[input].type ) { continue; }

						if ( inputs[input].type.toLowerCase() == "submit" || inputs[input].type.toLowerCase() == "image" ) {
								ibm.portal.debug.text( "Setting onclick handler for submit or image: " + inputs[input].name );
								var onclickFn = this.newSubmitOnclickHandler( inputs[input] );
								//dojo.event.connect( inputs[input], "onclick", onclickFn );
								com.ibm.portal.aggregation.event.DEFAULT_ACTION_MANAGER.setDefaultAction( inputs[input], "onclick", onclickFn );
						}
					}
					
					ibm.portal.debug.exit( "Portlet._tagSubmitInputs" );
				}
			  } 
);

dojo.declare( "com.ibm.portal.aggregation.forms.actions.IFrameOnloadOverrideAction",
			  null,
			  {
			  	// summary: Action for rewriting any iframe onload handler.
			  	
			  	apply: function ( iframeElem ) {
					// summary: Modify the given iframe element
					// description: Non-destructively changes the onload handler, or adds one 
					// 		if non-existent to publish the portlet markup updated on the iframe 
					// 		load event
					// iframeElem: the iframe element
					ibm.portal.debug.entry( "IFrameOnloadOverrideAction.apply", [ iframeElem ] );
					ibm.portal.debug.text( "Overriding iframe onload handler " + iframeElem.name );
					
					// only do this once per iframe
					if(!iframeElem._oldonload) {
						if(!iframeElem.onload || iframeElem.onload == "" || (dojo.isIE && iframeElem.onload === document.body.onload)) {
							iframeElem._oldonload = function() {};
						}
						else {
							iframeElem._oldonload = iframeElem.onload;
						}
						iframeElem.onload = function() {
							// change the iframe's onload handler to publish a markup change,
							// but make sure we call the existing onload handler also
							this._oldonload.apply( this );
							try{
								var portlet = dojo.byId( com.ibm.portal.utilities.findPortletIdByElement(this) );
								if(portlet != null) {
									try{
										// Firefox will see this iframe as empty later unless we delete the iframe reference
										delete window[iframeElem.name];	
									}
									catch(err) {}
									dojo.publish( "/portal/PortletMarkup/updated" , 
										[{newMarkup: portlet}] );
								}
							}
							catch(err) {}
						};
					}
					
					ibm.portal.debug.exit( "IFrameOnloadOverrideAction.apply" );
				}
			  }
);

//dojo.declare( "com.ibm.portal.aggregation.forms.actions.TagButtonInputs",
//			  com.ibm.portal.aggregation.forms.actions.SubmitInputOverrideAction,
//			  null,
//			  {
//			  	// summary: Tag all form button elements with an onclick handler so that we can
//				//		identify the element that triggered the submit when a form has multiple
//				//		submit inputs.
//				
//				apply: function ( formElem ) {
//					// summary: Modify the given form element
//					// description: Allows this action to the form. 
//					// formElem: the form element
//					ibm.portal.debug.entry( "Portlet._tagButtonInputs", [ formElem ] );
//					
//					var buttons = formElem.getElementsByTagName( "button" );
//										
//					ibm.portal.debug.text( "found " + buttons.length + " buttons associated with this form." );
//					
//					for ( var i = 0; i < buttons.length; i++ ) {
//						
//						ibm.portal.debug.text( "Processing button: " + buttons[i] );
//						
//						ibm.portal.debug.text( "Setting onclick handler for button: " + buttons[i].name );
//						var onclickFn = this.newSubmitOnclickHandler( buttons[i] );
//						//dojo.event.connect( buttons[i], "onclick", onclickFn );
//						com.ibm.portal.aggregation.event.DEFAULT_ACTION_MANAGER.setDefaultAction( buttons[i], "onclick", onclickFn );
//					}
//					
//					ibm.portal.debug.exit( "Portlet._tagButtonInputs" );
//				}
//			  }
//);

dojo.declare( "com.ibm.portal.aggregation.forms.PortletFormRewriter",
			  com.ibm.portal.aggregation.event.PortletMarkupRewriter,
			  {
			  	// summary: Controls the registration and execution of the actions associated
				//		with this rewriter.
				
				constructor: function () {
			  		this._actions = new Array();
			 	},
				needsRewrite: function ( /*HTMLElement*/elem ) {
					// summary: Indicates whether or not this rewriter needs to handle this node.
					// elem: the current markup element
					//return (dojo.html.isTag( elem, "form" ) != ""); // Boolean
					return (elem && elem.tagName && elem.tagName.toLowerCase() == "form" ); // Boolean
				}, 
				addAction: function ( action ) {
					// summary: Register a filter at the end of the filter chain.
					// filter: the filter to append
					if ( !this._actions ) { this._actions = new Array(); }
					this._actions.push( action );
				},
				rewrite: function ( formElem ) {
					// summary: Execute the filter chain.
					// description: Calls doFilter on every filter registered in this filter chain
					//		until all filters have been called or one of the filters returns true.
					ibm.portal.debug.entry( "PortletFormRewriter.rewrite", [ formElem ] );
					
					var i = 0;
					while ( i < this._actions.length )	{
						this._actions[i].apply( formElem );
						i = i + 1;		
					}
					
					ibm.portal.debug.exit( "PortletFormRewriter.rewrite" );
				},
				selectNodes: function ( /*HTMLElement*/rootElement ) {
					return dojo.query( "form", rootElement );
				}	
			  } 
);

dojo.declare( "com.ibm.portal.aggregation.forms.PortletIFrameRewriter",
			  com.ibm.portal.aggregation.event.PortletMarkupRewriter,
			  {
			  	// summary: Controls the registration and execution of the actions associated
				//		with this rewriter.
				
				constructor: function () {
			  		this._actions = new Array();
			 	},
				needsRewrite: function ( /*HTMLElement*/elem ) {
					// summary: Indicates whether or not this rewriter needs to handle this node.
					// elem: the current markup element
					//return (dojo.html.isTag( elem, "iframe" ) != ""); // Boolean
					return (elem && elem.tagName && elem.tagName.toLowerCase() == "iframe");
				}, 
				addAction: function ( action ) {
					// summary: Register a filter at the end of the filter chain.
					// filter: the filter to append
					if ( !this._actions ) { this._actions = new Array(); }
					this._actions.push( action );
				},
				rewrite: function ( iframeElem ) {
					// summary: Execute the filter chain.
					// description: Calls doFilter on every filter registered in this filter chain
					//		until all filters have been called or one of the filters returns true.
					ibm.portal.debug.entry( "PortletIFrameRewriter.rewrite", [ iframeElem ] );
					
					var i = 0;
					while ( i < this._actions.length )	{
						this._actions[i].apply( iframeElem );
						i = i + 1;		
					}
					
					ibm.portal.debug.exit( "PortletIFrameRewriter.rewrite" );
				},
				selectNodes: function (/*HTMLElement*/rootElement) {
					return dojo.query( "iframe", rootElement );
				}	
			  } 
);

com.ibm.portal.aggregation.forms.PORTLET_FORM_REWRITER = new com.ibm.portal.aggregation.forms.PortletFormRewriter();
com.ibm.portal.aggregation.forms.PORTLET_FORM_REWRITER.addAction( new com.ibm.portal.aggregation.forms.actions.SubmitOverrideAction() );
com.ibm.portal.aggregation.forms.PORTLET_FORM_REWRITER.addAction( new com.ibm.portal.aggregation.forms.actions.TagSubmitInputs() );

com.ibm.portal.aggregation.forms.PORTLET_IFRAME_REWRITER = new com.ibm.portal.aggregation.forms.PortletIFrameRewriter();
com.ibm.portal.aggregation.forms.PORTLET_IFRAME_REWRITER.addAction( new com.ibm.portal.aggregation.forms.actions.IFrameOnloadOverrideAction() );

if ( dojo.isIE ) {
	com.ibm.portal.aggregation.forms.PORTLET_FORM_REWRITER.addAction( new com.ibm.portal.aggregation.forms.actions.AddContentTypeInput() );
}

//Add the rewriter to the markup event handler
com.ibm.portal.aggregation.event.markup.PORTLET_MARKUP_EVENT_HANDLER.addRewriter( com.ibm.portal.aggregation.forms.PORTLET_FORM_REWRITER );
com.ibm.portal.aggregation.event.markup.PORTLET_MARKUP_EVENT_HANDLER.addRewriter( com.ibm.portal.aggregation.forms.PORTLET_IFRAME_REWRITER );

dojo.declare( "com.ibm.portal.aggregation.forms.PortletForm",
			  com.ibm.portal.services.PortalRestServiceForm,
			  {
			  	// summary: This object represents a form in the state of not being submitted to the 
			  	//		server yet, but after the user has performed the action which should result
			  	//		in the form being submitted.
			  	
			  	constructor: function ( /*HTMLElement*/formElem, /*{name:, value:}*/submitInputNameAndValue ) {
			  		this.submitTrigger = submitInputNameAndValue;
			  	},
			  	submit: function () {
			  		// summary: Submits the form. This is necessary because the native submit function
			  		//		may have been overridden to ensure the form submission was intercepted.
			  		ibm.portal.debug.entry( "PortletForm.submit" );
			  		var utils = com.ibm.portal.utilities;
			  		var formElem = this.getDOMElement();
			  		ibm.portal.debug.text( "Submitting: " + formElem + "; old submit=" + utils.getOverwrittenProperty( this.getDOMElement(), "submit" ) );
			  		try {
			  			//com.ibm.portal.utilities.callOverwrittenFunction( this.getDOMElement(), "submit" );
			  			//IE native functions don't support apply to change the scope, so we must restore the property
			  			//temporarily to get the correct scope.
			  			var restoreValue = utils.restoreProperty( formElem, "submit" );
			  			formElem.submit();
			  			utils.overwriteProperty( formElem, "submit", restoreValue );
			  		}
			  		catch ( e ) {
			  			ibm.portal.debug.text( "Error calling overwritten function: " + e.message );
			  		}	
			  		ibm.portal.debug.exit( "PortletForm.submit" );
			  	},
			  	toQuery: function () {
			  		// summary: Translates the form into a query string. Only elements which make sense
			  		//		for a query string are translated (i.e. NOT inputs of type "file").
			  		// returns: a string of name/value pairs suitable for being included as part of a url's
			  		//		query string.
			  		ibm.portal.debug.entry( "PortletForm.toQuery" );
			  		var queryStr =  com.ibm.portal.utilities.html.convertFormToQuery( this.getDOMElement(), this.submitTrigger );
			  		ibm.portal.debug.exit( "PortletForm.toQuery", queryStr );
			  		return queryStr;
			  	}
			  }
);

dojo.declare( "com.ibm.portal.aggregation.forms.PortletFormHandler",
			  null,
			  {
			  	// summary: Examines all clicks that originate inside a portlet body
				//		and handles if appropriate.
				// description: Intercepts all clicks that originate from inside a 
				//		portlet and passes the event through the filter chain. If none of 
				//		filters can handle the event, it is left alone. Otherwise, it is
				//		cancelled and handled.
				
				constructor: function () {
			  		this._activeFormSubmitTriggers = {};
			  		this._callbackfns = {};
			  	},
				handle: function ( formId, actionURL ) {
					ibm.portal.debug.entry( "PortletFormHandler.handle", [ formId, actionURL ] );
					
					var portletForm = new com.ibm.portal.aggregation.forms.PortletForm( dojo.byId( formId ), this._activeFormSubmitTriggers[ formId ] ); 
					ibm.portal.debug.text( "Found form node: " + portletForm.getDOMElement() );
					ibm.portal.debug.text( "form node's submit fn: " + portletForm.getDOMElement().submit );
					
					//It's possible this form could be submitting to a servlet packaged with a portlet, in which case we want to let
					//the natural form submission process occur.
					if ( !com.ibm.portal.utilities.isPortalUrl( actionURL ) ) {
						ibm.portal.debug.exit( "PortletFormHandler.handle (form points to non-portal servlet)" );
						portletForm.submit();
						return;
					}
					//It's possible the form has a target pointing to a new window. In this case, we also are going to let the natural
					//form submission process occur.
					var target = portletForm.getDOMElement().getAttributeNode( "target" );
					ibm.portal.debug.text( "Target node is : " + target );
					if ( target != null && target.value && target.value != "" && !( target.value == "_self" || target.value == "_top" ) ) {
    					ibm.portal.debug.text( "Target node value is : " + target.value );
						ibm.portal.debug.exit( "PortletFormHandler.handle (form has a target attribute)" );
						portletForm.submit();
						return;
					}
					
					if ( portletForm.getDOMElement().submitted == true ) { 
						ibm.portal.debug.text( "Form has already been submitted!!" );
					}
					else {
	                    if ( portletForm.isMultipart && this._activeFormSubmitTriggers[ formId ] ) {
                            //Create the url and add the necessary parameters.
					        var urlObj = new com.ibm.portal.utilities.HttpUrl( actionURL );
                            var activeSubmitInputName = this._activeFormSubmitTriggers[formId].name;
					        urlObj.addParameter( activeSubmitInputName, this._activeFormSubmitTriggers[formId].value );
                            actionURL = urlObj.toString();
                            ibm.portal.debug.text( "New action url is: " + actionURL );        
                        }				
                        else {
						    ibm.portal.debug.text( "Encoded form should be: " + portletForm.toQuery() );
                        }    
						
						var portletId = com.ibm.portal.utilities.findPortletIdByElement( portletForm.getDOMElement() );
						
						var serviceReq = new com.ibm.portal.services.PortletFragmentService();
						serviceReq.getPortletInfo( portletId, actionURL, null, portletForm );
						//formNode might have been overwritten.
						portletForm.getDOMElement().submitted = true;
					}	
					
					ibm.portal.debug.exit( "PortletFormHandler.handle (normal)" );
				},
				handleMultiPartResult: function ( iframeId ) { 
					// summary: Global function for IE to use as a callback for dynamically inserted IFRAMEs.
					// description: IE seems to not recognize onload handlers set in the traditional sense on dynamically created
					//		IFRAMEs. We have to set the onload handler in text written into the DOM which prevents us from sending
					//		along a Function object. The callback function is stored as an instance variable on this class, sorted
					//		by IFRAME id. This callback function is called when this function is notified of an IFRAME load event.
					//		The only case where we use IFRAMEs is for multi-part forms so hopefully this will be a rare occurrence.
					// iframeId: the iframe ID 
					ibm.portal.debug.entry( "PortletFormHandler.handleMultiPartResult", [ iframeId ] );
					
					var callbackfn = this._callbackfns[iframeId].fn;
					var callbackargs = this._callbackfns[iframeId].args;
					this._callbackfns[iframeId] = null;
					var origXmlText = this.extractEncodedXml(self.frames[iframeId].document);
                    
					ibm.portal.debug.text("XML from the multi-part form: " + origXmlText);
					
                    // unescape the xml text sent by multi-part form
					var decodedXmlText = com.ibm.portal.utilities.decodeXML(origXmlText);
					var xmlDoc = dojox.data.dom.createDocument(decodedXmlText);
					callbackfn( "load", xmlDoc, null, callbackargs );
                    
					ibm.portal.debug.exit( "PortletFormHandler.handleMultiPartResult" );
				}, 
				extractEncodedXml: function ( xmlDoc ) {
					ibm.portal.debug.entry( "PortletFormHandler.extractEncodedXml", [ xmlDoc ] );
					
					var xmlNodeArray = xmlDoc.getElementsByTagName('PRE');
					
					return xmlNodeArray[0].innerHTML;
				}, 
				setActiveSubmitTrigger: function ( /*String*/formId, /*{name: "", value:""}*/inputName ) {
                    this._activeFormSubmitTriggers[ formId ] = inputName;	
				},
                isMultipartForm: function ( /*String or HTMLElement*/ form ) {
                    return (dojo.byId( form ).getAttribute( "enctype" ) == "multipart/form-data");    
                }
                
			  }  
);

com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER = new com.ibm.portal.aggregation.forms.PortletFormHandler();

}

if(!dojo._hasResource["com.ibm.portal.aggregation.javascript.parser"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.javascript.parser"] = true;
dojo.provide( "com.ibm.portal.aggregation.javascript.parser" );

dojo.declare( "com.ibm.portal.aggregation.javascript.parser.ASParserResult",
			  null,
			  {
			  	// summary: Summarizes the result of an AssignmentStatementParser parsing operation.
				// description: Contains the relevant information about a parsing result. Keeps the start and end
				//		indices, the variable name and the variable value. The value will include any quotation
				//		marks surrounding it (or if a variable name, it will be just the variable name string).
				// parsingSuccessful: indicates whether the parsing operation was successful or not
				// originalString: the string passed into be parsed
				// startIndex: the starting index of the originalString for parsing
				// endIndex: the ending index of the parsed string 
				// variableName: the parsed portion on the left side of the assignment
				// variableValue: the parsed portion on the right side of the assignment
				constructor: function ( /*Boolean*/successful, /*String*/javascript, /*int*/sIndex, /*int*/eIndex, /*String*/varName, /*String*/varValue ) {
				  	this.parsingSuccessful = successful; 
				  	this.originalString = javascript;
					this.startIndex = sIndex;
					this.endIndex = eIndex;
					this.variableName = varName;
					this.variableValue = varValue;
			  	}
			  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.parser.AssignmentStatementParser",
			  null,
			  {
				// summary: Parses simple javascript assignment statements given a javascript string and a 
				//		starting index. Please note that the parser must be reset between parsing calls.
				// description: This parser is intended for parsing very simple javascript assignment statements. 
				//		For example, "window.location.href='http://www.xyz.com';". It's primary purpose is for
				//		extracting the value of a window.location.href assignment so that this assignment can
				//		be re-written for client-side aggregation.
				
				//Parsing States
				
				//parsing the variable name				
				_PARSING_NAME: 0,
				//parsing the assignment operator and any whitespace surrounding it
				_PARSING_ASSIGNMENT: 1,
				//started parsing the value of the assignment
				_PARSING_VALUE: 2,
				
				//We need to know which type of quote (if any) is used to delimit the value string since the single
				//quote could appear in the URL value.
				
				//the value starts with a single quote, another single quote closes the value parsing
				_PARSING_VALUE_OPEN_SINGLE_QUOTE: 3,
				//the value starts with a double quote, another double quote closes the value parsing
				_PARSING_VALUE_OPEN_DOUBLE_QUOTE: 4,
				//the value is most likely a variable name (integer would be another option, either should be parsed correctly)
				_PARSING_VALUE_VARIABLE: 5,
				//the opening quotation mark is closed
				_PARSING_VALUE_CLOSE_EITHER_QUOTE: 6,
				//the parsing is complete
				_PARSING_DONE: 7,
				//the statement could not be parsed
				_PARSING_ERROR: 8,
				
				constructor: function() {
				  	this.name = "";
					this.value = "";
					this._javascriptString = "";	
			  	},				
				parse: function ( /*String*/javascript, /*int*/startIndex ) {
					// summary: Parses the next assignment statement from the given javascript string starting from 
					//		the given index
					// javascript: the javascript string
					// startIndex: the index to start parsing from
					// returns: a parsing result 
					ibm.portal.debug.entry( "AssignmentStatementParser.parse", [ javascript, startIndex ]);
					
					var state = this._PARSING_NAME;
					var i = startIndex;
					this._javascriptString = javascript;
					
					//Basically using a state machine here. Our state can change based on
					//the character read in.
					while ( ( state != this._PARSING_DONE && state != this._PARSING_ERROR )	 && i < javascript.length ) {
						state = this._processCharacter( this._javascriptString.charAt(i), i, state );		
						i = i+1;
					}
					
					var returnObj = new com.ibm.portal.aggregation.javascript.parser.ASParserResult( (state == this._PARSING_DONE), javascript, startIndex, i, this.name, this.value );

					ibm.portal.debug.exit( "AssignmentStatementParser.parse" , [ returnObj ] ); 
					return returnObj;	// com.ibm.portal.aggregation.javascript.parser.ASParserResult		 
				},
				reset: function () {
					// summary: Resets the parser's internal variables so the parser object may be reused.
					this.name = "";
					this.value = "";
					this._javascriptString = "";
				},
				_processCharacter: function( /*String*/character, /*int*/index, /*int*/state ) {
					// summary: Processes the given character and state to produce the next state for the parser.
					// character: the character to process
					// state: the current state of the parser
					// returns: the next state of the parser
					
					//TODO: comment handling!
					ibm.portal.debug.entry( "AssignmentStatementParser._processCharacter", [ character, state ] );
					
					var addToName = false;
					var addToValue = false;
					
					//Default to the current state.
					var nextState = state;
					
					//The initial state is PARSING_NAME. We remain in this state as long as the character given
					//is not an illegal character for a variable name. Basically, whenever we encounter a space
					//or a non A-Z, 0-9 character.
					if ( state == this._PARSING_NAME ) {
						var add = character.search( /[A-Za-z_\.][A-Za-z_0-9\.]*/ );
						if ( add >= 0 ) {
							addToName = true;
						}
						else if ( character == "=") {
							//There's no whitespace before the assignment operator, so we can skip right to 
							//parsing the value.
							if ( (this._javascriptString.length > ( index + 1 )) && this._javascriptString.charAt( index+1 ) != '=' ) {
								nextState = this._PARSING_VALUE;
							}	
							else {
								nextState = this._PARSING_ERROR;
							}
						}
						else if ( character == "!" || character == ")" || character == ";" ) {
							//Not an assignment statement.
							nextState = this._PARSING_ERROR;
						}
						else {
							//Have to check for whitespace before the assignment operator.
							nextState = this._PARSING_ASSIGNMENT;
						}
					}
					//Throw away any whitespace before the assignment operator.
					else if ( state == this._PARSING_ASSIGNMENT ) {
						if ( character == "=" ) {
							//Found the assignment operator, move on to the value.
							if ( (this._javascriptString.length > ( index + 1 )) && this._javascriptString.charAt( index+1 ) != '=' ) {
								nextState = this._PARSING_VALUE;
							}	
							else {
								nextState = this._PARSING_ERROR;
							}
						}
						else if ( character == "!" || character == ")" || character == ";" ) {
							nextState = this._PARSING_ERROR;
						}
					}
					//If we have a string value, we need to track the open quotation mark so we can match
					//it later on and close the string. Otherwise, we assume we have a variable or an 
					//integer as the value. We skip any leading spaces.
					else if ( state == this._PARSING_VALUE ) {
						if ( character == "'" ) {
							addToValue = true;
							nextState = this._PARSING_VALUE_OPEN_SINGLE_QUOTE;
						}
						else if ( character == '"' ) {
							addToValue = true;
							nextState = this._PARSING_VALUE_OPEN_DOUBLE_QUOTE;
						}
						else if ( character == ' ' ) {
							nextState = this._PARSING_VALUE;
						}
						else {
							var add = character.search( /[A-Za-z_\.][A-Za-z_0-9\.]*/ );
							if ( add >= 0 ) {
								addToValue = true;
								nextState = this._PARSING_VALUE_VARIABLE;
							}
						}		
					}
					
					//Keep going until we find a matching closing quotation mark.
					else if ( state == this._PARSING_VALUE_OPEN_SINGLE_QUOTE ) {
						addToValue = true;
						if ( character == "'" ) {
							nextState = this._PARSING_VALUE_CLOSE_EITHER_QUOTE;
						}
					}
					else if ( state == this._PARSING_VALUE_OPEN_DOUBLE_QUOTE ) {
						addToValue = true;
						if ( character == '"' ) {
							nextState = this._PARSING_VALUE_CLOSE_EITHER_QUOTE;
						} 
					}
					
					//Keep going until we find an invalid character in an identifier or a semi-colon.
					else if ( state == this._PARSING_VALUE_VARIABLE ) {
						var add = character.search( /[A-Za-z_\.\[\]][A-Za-z_0-9\.\[\]]*/ );
						if ( add >= 0 ) {
							addToValue = true;
						}
						else if ( character == ';' ) {
							nextState = this._PARSING_DONE;
						}
						else {
							nextState = this._PARSING_VALUE_CLOSE_EITHER_QUOTE;
						}
					}
					else if ( state == this._PARSING_VALUE_CLOSE_EITHER_QUOTE ) {
						//TODO: what about the case where there is no semi-colon?
						if ( character == ';' ) {
							nextState = this._PARSING_DONE;
						}
					}
					
					//Add the character to the appropriate instance variable, if necessary.
					if ( addToName ) {
						this.name += character;
					}
					if ( addToValue ) {
						this.value += character;
					}
					
					ibm.portal.debug.exit( "AssignmentStatementParser._processCharacter", [ nextState ] );
					return nextState; // int
				}			  	
			  }
); 

}

if(!dojo._hasResource["com.ibm.portal.aggregation.javascript"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.javascript"] = true;
dojo.provide( "com.ibm.portal.aggregation.javascript" );

dojo.require( "com.ibm.portal.aggregation.javascript.parser" );
dojo.require( "com.ibm.portal.EventBroker" );

//NOTE: This function is defined using the Function constructor because it needs a clean scope. The build process munges all of these js files
//	inside of an anonymous function which is immediately executed to control the scope. This is what enables the dojo scopesMap support. This 
//  means that the current scope from inside that gigantic anonymous function is used when the function is executed so certain things may be
//	defined that aren't really (i.e. the "dojo" namespace). 

// summary: Evaluates a string of javascript in the global scope.
// description: Calling eval.call( self, ... ); from within a JS object apparently makes "this" point to the object from which the
//      eval call was made even though the scope is set using the "call" function. Therefore we need to have this function
//      available in the global scope and not as a member variable on a JS object.

if ( !dojo.isIE ) {
	window.com_ibm_portal_aggregation_javascript_globalEvalNonIE = new Function( "javascript", "dojo.eval( javascript );" );
}	

function com_ibm_portal_aggregation_javascript_windowLocationHref( /*String*/ portlet, /*String*/newLocation ) {
    var service = new com.ibm.portal.services.PortletFragmentService();
    service.getPortletInfo( portlet, newLocation );
    
    //Anytime a window.location change is handled, it effectively stops the processing of the current event/page
    //since in a non-CSA environment, we would load an entirely new document at this point. However, some portlets
    //seem to depend on this behavior and don't properly cancel an event which causes a window.location change -- so
    //we have to do it for them.
    com.ibm.portal.EVENT_BROKER.stopEvent.fire();
}

document.com_ibm_portal_aggregation_javascript_windowLocationHref = com_ibm_portal_aggregation_javascript_windowLocationHref;
//Necessary for IE because it does not correctly add global functions to the window object.
window.com_ibm_portal_aggregation_javascript_windowLocationHref = com_ibm_portal_aggregation_javascript_windowLocationHref;

// this class is a wrapper class around the script element
// it is used to insert script elements into the DOM without actually executing the script
// this class has to be instantiated through the FakeScriptFactory
dojo.declare( "com.ibm.portal.aggregation.javascript.FakeScript",
                    null,
                    {
                        setFakeSrc: function(source)
                        {
                            this.fakeSrc = source;
                        },
                        getAttribute: function(attr)
                        {
                            if (attr == 'src') {
                                return this.fakeSrc;
                            }
                            // this call is calling the super class of this class
                            return this.super_getAttribute(attr);
                        }
                    }
);

// this is the factory call to be used to create the fake scripts element
dojo.declare( "com.ibm.portal.aggregation.javascript.FakeScriptFactory",
                    null,
                    {
                        createFakeScriptElement: function () {
                             return com.ibm.portal.aggregation.javascript.INHERITANCE.applyInherit(
                                 document.createElement("script"),
                                 new com.ibm.portal.aggregation.javascript.FakeScript());
                        }
                    }
);

// SINGLETON to access the factory
com.ibm.portal.aggregation.javascript.FAKE_SCRIPT_FACTORY = new com.ibm.portal.aggregation.javascript.FakeScriptFactory();

// allows to do class inheritance in javascript and at the same time makes existing functions names available as super_XYZ
dojo.declare( "com.ibm.portal.aggregation.javascript.Inheritance",
                    null,
                    {
                        applyInherit: function(original, inheritee)
                        {
                            // first we iterate over the function of "inheritee" which is the new class that wants to inherit from original
                            // and create new functions that point to the original ones
                            for (method in inheritee) {
                                if (typeof(original[method])!="undefined") {
                                    inheritee["super_"+method] = original[method];
                                }
                            }
                            for (method in inheritee) {
                                original[method] = inheritee[method];
                            }
                            return original;
                        }
                    }
);

com.ibm.portal.aggregation.javascript.INHERITANCE = new com.ibm.portal.aggregation.javascript.Inheritance();

dojo.declare( "com.ibm.portal.aggregation.javascript.Filter",
              null,
              {
                // summary: Base interface for a filter. Each implementation must implement
                //      the doFilter function.
                // description: Passes the given event through to give the implementation of
                //      this filter a chance to alter or handle the event in some way. 
                
                doFilter: function ( /*String or HTMLElement*/script, portletId ) {
                    // summary: Filter the given event.
                    // description: Allows this filter to handle or alter the event. If this function
                    //      returns true, processing is stopped and the event is considered "handled".
                    // script: the script block (string or HTMLElement) 
                    dojo.unimplemented( "com.ibm.portal.aggregation.JavascriptFilter.doFilter" );
                },
                evalGlobal: function ( /*String*/javascript, /*String*/portletId, /*boolean?*/defer, /*Function?*/onreadystatechange ) {
                    // summary: Evaluates the given javascript in the global context.
                    // javascript: a string of javascript to evaluate.
                    // portletId: the id of the portlet that the javascript belongs to
                    // defer: whether the script has the defer attribute (queues the eval operation up with
                    //      the onload handler)
                    // onreadystatechange: IE only. If the script element has an onreadystatechange handler,
                    //      this should be a wrapper function which takes the readyState value as a parameter.                          
                    ibm.portal.debug.entry( "Filter.evalGlobal", [ javascript, portletId, defer, onreadystatechange ] );
                    
                    javascript = this._replaceWindowLocationReplaces( javascript, portletId );
                    javascript = this._replaceWindowLocationHrefs( javascript, portletId );
                    
                    var me = this;
                    var evalFunction = function () {
                        com.ibm.portal.EVENT_BROKER.startScriptExecution.fire( { portletId: portletId } );
                        
                        try {
	                        if ( window.execScript ) {
	                            window.execScript( me._stripHTMLComments( javascript ), "JavaScript" );
	                        }
	                        else {
	                            //Using eval.call( window, javascript ) was apparently not really eval'ing
	                            //in the global scope since 'this' was still referring to the Filter object 
	                            //and not the window object as it should have (or I think it should have)...
	                            com_ibm_portal_aggregation_javascript_globalEvalNonIE( javascript );
	                        }
	                        
	                        if ( onreadystatechange ) {
	                            onreadystatechange( "complete" );
	                        }
                        } 
                        //Don't catch for two reasons: 1. we want to surface any error in the portlet code as soon as 
                        //possible. 2. we want to avoid the potential performance hit entering the catch block frequently
                        //would cause. This results in an odd message in IE6 about an "unsupported operation", as IE6 seems
                        //to report to the browser the actual javascript error while execScript is executing and then generate a new error
                        //with the "unsupported operation" message that actually gets thrown. IE6 also ignores finally statements with
                        //no catch, but we really want to let the browser handle any errors so instead of doing some browser
                        //sniff here, we'll just check the number of before and after snapshots before attempting to create any
                        //restore points.
                        finally {
                        	com.ibm.portal.EVENT_BROKER.endScriptExecution.fire( { portletId: portletId } );
                        }	
                    }
                    
                    //If the script is "deferred", we want to execute it "onload". 
                    if ( defer ) {
                        com.ibm.portal.aggregation.javascript.JS_ONLOAD_HANDLER._registerHandler( evalFunction );   
                    }
                    else {
                        evalFunction();                     
                    }
                    
                    ibm.portal.debug.exit( "Filter.evalGlobal" );   
                },
                _stripHTMLComments: function( str ) {
                    // summary: Strip all HTML comments from a script block.
                    // description: Although HTML comments are allowed in a script block by the browser
                    //      IE throws up whenever you try and eval a javascript string in the global context
                    //      that has HTML comments in the string.
                    ibm.portal.debug.entry( "stripHTMLComments", [ str ] );
                    
                    var resultStr = "";
                    var startIndex = str.indexOf( "<!--" );
                    
                    if ( startIndex != -1 ) {
                        resultStr = str.substring( 0, startIndex );
                    }
                    else {
                        resultStr = str;
                    }
                    
                    while ( startIndex >= 0 ) {
                        var endIndex = startIndex;
                        //Look at the preceding character to see if we are inside a string
                        if ( startIndex - 1 >= 0 ) {
                            var openQuote = str.charAt( startIndex - 1 );
                            var closeQuote = str.charAt( startIndex + 4 );
                            if ( openQuote == "\"" && closeQuote == "\"" ) {
                                startIndex = str.indexOf( "<!--", endIndex + 3 );
                                if ( startIndex >= 0 ) {
                                    //found another comment later, add everything in between
                                    resultStr += str.substring( endIndex, startIndex );
                                }
                                else {
                                    //found no more comments, add the rest of the script
                                    resultStr += str.substring( endIndex );
                                }   
                                
                                continue;
                            }
                        }
                        
                        var endIndex = str.indexOf( "-->", startIndex );
                        if ( endIndex < 0 ) {
                            //no closing tag found for the comment, something's wrong
                            throw new Error( "Unclosed HTML comment found!!" );
                        }
                        
                        var lastNewLineIndex = str.substring( startIndex, endIndex ).lastIndexOf( "\n", endIndex );
                        if ( lastNewLineIndex < 0 ) {
                            lastNewLineIndex = startIndex;
                        }
                        else {
                            lastNewLineIndex += startIndex;
                        }
                        var doubleSlashIndex = str.substring( lastNewLineIndex, endIndex ).indexOf( "//" );
                        if ( doubleSlashIndex > -1 ) {
                            // This comment block needs to be executed as javascript, but don't add the comment tags themselves
                            resultStr += str.substring( startIndex + 4, lastNewLineIndex + doubleSlashIndex );
                        }
                        
                        
                        startIndex = str.indexOf( "<!--", endIndex + 3 );
                        if ( startIndex >= 0 ) {
                            //found another comment later, add everything in between
                            resultStr += str.substring( endIndex + 3, startIndex );
                        }
                        else {
                            //found no more comments, add the rest of the script
                            resultStr += str.substring( endIndex + 3 );
                        }
                    }
                    
                    ibm.portal.debug.exit( "stripHTMLComments", [ resultStr ] );
                    
                    return resultStr;
                },
                _setNewWindowLocationHrefFunction: function ( /*String*/portletId ) {
                    // summary: Sets a new function with a unique name to handle a window.location.href call.
                    // portletId: the portlet id for the portlet to update
                    // returns: the function name of the created function
                    var funcName = "com_ibm_portal_aggregation_javascript_windowLocationHref" + dojo.dnd.getUniqueId();
                    window[ funcName ] = function ( url ) {
                        ibm.portal.debug.entry( funcName, [ url ] );
                        com_ibm_portal_aggregation_javascript_windowLocationHref( portletId, url );
                        ibm.portal.debug.exit( funcName );
                    }
                    
                    return funcName;
                },
                _replaceWindowLocationHrefs: function ( /*String*/javascript, /*String*/ portletId ) {
                    // summary: Replaces window.location.href references with a function call that CSA can handle.
                    // javascript: the javascript string to process
                    // portletId: the portlet the javascript belongs to
                    // returns: the processed javascript string with all window.location.href references replaced.
                    
                    ibm.portal.debug.entry( "_replaceWindowLocationRefs", [ javascript ] );
                    
                    var resultStr = javascript;
                    var searchArray = [ "self.location.href", "top.location.href", "window.location.href", "window.location", "top.location", "self.location", "location.href" ];
                    var indexOf = com.ibm.portal.utilities.string.findNext( resultStr, searchArray, null );
                    
                    var parser = new com.ibm.portal.aggregation.javascript.parser.AssignmentStatementParser();
                    while( indexOf >= 0 ) {
                        //Always have to reset the parser.
                        parser.reset();
                        var result = parser.parse( resultStr, indexOf );
                        
                        if ( result.parsingSuccessful ) {
                            //Reconstruct the javascript string by taking the portion around the window.location.href reference and
                            //combining that around a new function call which takes the place of the window.location.href reference.
                            var beforePortion = resultStr.substring( 0, indexOf );
                            //Create a new function object. This must be a global, unique function for each portlet.
                            var funcName = this._setNewWindowLocationHrefFunction( portletId );
                            ibm.portal.debug.text( "Replacing: " + resultStr.substring(result.startIndex, result.endIndex - 1) + " with: " + funcName + "( "+ result.variableValue + " )" ); 
                            //Re-construct the javascript string.
                            resultStr = beforePortion + funcName + "( "+ result.variableValue + " )" + resultStr.substring( result.endIndex - 1 );
                        }
                            
                        //Check for the next instance.
                        indexOf = com.ibm.portal.utilities.string.findNext( resultStr, searchArray, indexOf );  
                    }
                    
                    ibm.portal.debug.exit( "_replaceWindowLocationRefs", [ resultStr ] );
                    return resultStr; // String
                },
                _replaceWindowLocationReplaces: function ( /*String*/javascript, portletId ) {
                    // summary: Replaces window.location.replace() function calls
                    // javascript: the javascript string to process
                    // portletId: the portlet the javascript belongs to
                    // returns: the processed javascript string with all window.location.replace() function calls replaced
                    ibm.portal.debug.entry( "_replaceWindowLocationReplaces", [ javascript ] );
                    
                    var str = javascript.replace( /location.replace\(/g, "com_ibm_portal_aggregation_javascript_windowLocationHref('"+ portletId + "'," );
                    
                    ibm.portal.debug.exit( "_replaceWindowLocationReplaces", [str] );
                    return str; // String
                },  
                prepareDocumentWrite: function( buffer ) {
                    // summary: Prepares support for document.write() and document.writeln
                    // description: Overrides the original document.write() and document.writeln()
                    //       functions with functions writing to the buffer
                    ibm.portal.debug.entry( "prepareDocumentWrite" );
                    var me = this;
                    document.write = function() {
                        me._documentWrite( buffer, document.write.arguments );
                    };
                    document.writeln = function(str) {
                        me._documentWrite( buffer, document.writeln.arguments );
                    };
                    ibm.portal.debug.exit( "prepareDocumentWrite" );
                },
                _documentWrite: function( buffer, /*String[]*/args ) {
                    // summary: Internal document.write() function writing to the given buffer
                    // description: Just appends the given String arguments to our buffer
					var _method = "Filter._documentWrite";
                	ibm.portal.debug.entry( _method, [ buffer, args ] );
                	var content = ""+buffer.innerHTML;
					var str = "";
                    for ( var i = 0; i < args.length; i++ ) {
                        str += args[i];
                    }
                    var markupMap = {markup: str, objects: {}};
                    if(dojo.isIE) {
                    	markupMap = com.ibm.portal.utilities.html.extractObjectElementsFromString(str);
                    }
					buffer.innerHTML = content + markupMap.markup;
					if(dojo.isIE) {
						// this if block is just for good measure since the markupMap.objects would be an
						// empty map if we weren't in IE anyway
						com.ibm.portal.utilities.html.replaceObjectElementsInMarkup(markupMap.objects);
					}
					ibm.portal.debug.exit( _method );
                },
                applyDocumentWrite: function( /*HTMLElement*/script, buffer) {
                    // summary: Injects the markup in the given buffer into the DOM
                    // description: Looks up the script element in the original DOM
                    //       and injects the markup before that element
                    var _method = "Filter.applyDocumentWrite";
                	ibm.portal.debug.entry( _method, [ script, buffer.innerHTML ] );
                    var cont = buffer.innerHTML;
                    var id = script.getAttribute( "id" );
                    var realScript = document.getElementById( id );
                    if ( cont != null && cont.length > 0 ) {
                        var tempDIV = buffer; //com.ibm.portal.utilities.html.createTemporaryMarkupDiv( cont );
                    	//var markupMap = com.ibm.portal.utilities.html.createTemporaryMarkupDiv( cont );
                        //var tempDIV = markupMap.markup;
                        var children = tempDIV.childNodes;
                        if ( children != null && children.length > 0 ) {
                            // copy the nodes over into the DOM
                            var pred = realScript;
                            // <PK92843>
                            if (pred != null) { // </PK92843>
			        var parent = pred.parentNode; // need parent to insert relative to pred
			        // we don't need an i++ at this point because
			        // a) the insertBefore call of a non-cloned element removes that element from the children list and
			        // b) in the script case we create a copy which is added to the DOM and therefore not removed from the children list
			        //    therefore we increment i to get the next element
			        for ( var i = 0; i < children.length; ) {
				    var node = children[i];
				    if (node.tagName != "SCRIPT") {
				    	// we need to inject the markup before the script element
				    	// to properly handle table updates
				    	//dojo.dom.insertAfter( node, pred );
					pred == parent.lastChild ? parent.appendChild(node) : parent.insertBefore(node, pred.nextSibling);
				    	// removes the element from the original children list, therefore we do not need to do i++
				    	pred = node;
				    }
				    else {
				    	// execute the script
				    	var fakeNode = 
						com.ibm.portal.aggregation.javascript.FAKE_SCRIPT_FACTORY.createFakeScriptElement(node.src);
				    	// this section is very important because in later script evals we read out the
				    	// id and use it as a reference to insert new statements
				    	// also it needs to be unique also in a nested use case since we remove the id
				    	// at the end of this function for cleanup reasons again
				    	node.setAttribute( "id", id + "_" + i );
				    	fakeNode.setAttribute( "id", id + "_" + i );

				    	//dojo.dom.insertAfter( fakeNode, pred );
					// insert after
					pred == parent.lastChild ? parent.appendChild(node) : parent.insertBefore(node, pred.nextSibling);
				        fakeNode.setFakeSrc(node.src);
				        pred = fakeNode;
				        i++;

				        com.ibm.portal.aggregation.javascript.JAVASCRIPT_HANDLER.handle( node );
				    }
			        }
			    } // end of if (pred != null)
                        }

                        //dojo.dom.destroyNode( tempDIV );
						if(tempDIV.parentNode) tempDIV.parentNode.removeChild(tempDIV);
						
						delete tempDIV;
						delete buffer;
                    }
                    // remove id attribute if it is an internal id
                    if (id.indexOf("_scr#") == 0) {
                        realScript.setAttribute( "id", "empty" );
                    }
                    ibm.portal.debug.exit( _method );
                }
              } 
);

dojo.declare( "com.ibm.portal.aggregation.javascript.ExternalScriptFilter",
              com.ibm.portal.aggregation.javascript.Filter,
              {
                doFilter: function ( /*HTMLElement*/script, portletId, dfd ) {
                    // summary: Filter the given event.
                    // description: Allows this filter to handle or alter the event. If this function
                    //      returns true, processing is stopped and the script is considered "handled".
                    // script: the script block (HTMLElement) 
                    ibm.portal.debug.entry( "ExternalScriptFilter.doFilter", [ script, portletId, dfd ]); 
                    
                    var url = this._getScriptUrl( script );
                    //Normally prefer to check for capabilities instead of a straight isIE check, but in this
                    //case, it's simply just a browser behavior issue.
                    var deferred = ( script.defer && ( dojo.isIE > 0 ) );
                    ibm.portal.debug.text( "deferred: \"" + deferred + "\"; script.defer: \"" + script.defer + "\"");
                    var onreadystatechange = script.onreadystatechange;
                    var handled = false;
                    
                    if ( url ) {
                        // check if the AJAX proxy is enabled
                        var proxyURL = ibmPortalConfig["proxyURL"];
                        if (proxyURL != null) {
                            url = this._rewriteURL(url, proxyURL);
                        }
                        // prepare bufferDiv for document.write/ln
                        var buffer = com.ibm.portal.utilities.html.createTemporaryMarkupDiv( "" ).node;
						dojo.style(buffer, "display", "none");
						dojo.body().appendChild(buffer);
                        this.prepareDocumentWrite(buffer);
                        
                        //handle onreadystatechange values
                        var onreadyFn = null;
                        if ( onreadystatechange && dojo.isIE ) {
                            ibm.portal.debug.text( "wrapping onreadystatechange handler: " + onreadystatechange );
                            onreadyFn = function ( /*String*/readyState ) {
                                ibm.portal.debug.entry( "onreadystate", [ readyState ] );
                                try {
                                    ibm.portal.debug.text( "Executing: " + onreadystatechange );
                                    onreadystatechange.apply( { readyState: readyState } );
                                }
                                catch ( error ) {
                                    ibm.portal.debug.text( "Unable to call script's onreadystatechange handler!" );
                                    com.ibm.portal.aggregation.error.ERROR_HANDLER.reportError( "Unable to execute script tag's onreadystatechange handler!", error.message );
                                }
                                ibm.portal.debug.exit( "onreadystate" );
                            }
                        }
                        
                        var status = this._loadExternalScript( url, portletId, deferred, onreadyFn, dfd );
                        if ( status && status.addCallback ) {
                        	// apply document.write/ln to DOM
                        	var me = this;
                        	status.addCallback( function () { me.applyDocumentWrite(script, buffer); } );
                        }
                        else {
                        	if ( status instanceof Error ) {
                        		throw status;
                        	}
                        }	
                        handled = true;
                    }
                    
                    ibm.portal.debug.exit( "ExternalScriptFilter.doFilter", handled );
                    return handled;
                },
                _getScriptUrl: function ( /*HTMLElement*/script ) {
                    // summary: Retrieve the src attribute value of a script tag if it exists.
                    // returns: the url for the script tag or null if the script tag doesn't specify a URL
                    var url = null;
                    
                    if ( script.getAttribute ) {
                        url = script.getAttribute( "src" ); 
                    }
                    else {
                        var start = script.toLowerCase().indexOf( "<script" );
                        var end = script.toLowerCase().indexOf( ">" );
                        
                        var scriptTagSubstr = script.substring( start, end );
                        var srcIndex = scriptTagSubstr.toLowerCase().indexOf("src");
                        
                        
                        if (srcIndex != -1)
                        {
                            // figure out whether it's a src='blah' or src="blah"
                            var quoteIndex = scriptTagSubstr.indexOf("'", srcIndex);
                            var dblQuoteIndex = scriptTagSubstr.indexOf('"', srcIndex);
                            var markerChar = '"';
                            var markerIndex = dblQuoteIndex;
                            if (dblQuoteIndex == -1 || (quoteIndex != -1 && quoteIndex < dblQuoteIndex))
                            {
                                markerChar = "'";
                                markerIndex = quoteIndex;
                            }
                            var markerIndex2 = scriptTagSubstr.indexOf(markerChar, markerIndex + 1);
                            url = scriptTagSubstr.substring(markerIndex + 1, markerIndex2);
                        }
                    }
                    
                    return url;
                },
                _loadExternalScript: function ( /*String*/url, /*String*/portletId, /*boolean?*/deferred, /*Function?*/onreadystatechange, /*dojo.Deferred?*/dfd ) {
                    // summary: Loads an external script.
                    var _method = "ExternalScriptFilter._loadExternalScript";
                    ibm.portal.debug.entry( _method, [ url, portletId, deferred, onreadystatechange ] ); 
                    
                    var status = null;
                    var me = this;
                    try {
                        if ( url == "//:" ) {
                            //probably a hack to detect onload
                            ibm.portal.debug.text( "Empty protocol url detected. Probably a hack to detect onload!", _method );
                            /*if ( onreadystatechange ) {
                                try {
                                    ibm.portal.debug.text( "Executing onreadystatechange handler for state \"loaded\": " + onreadystatechange );
                                    onreadystatechange( "loaded" );
                                }
                                catch ( error ) {
                                    com.ibm.portal.aggregation.error.ERROR_HANDLER.reportError( "Unable to execute script tag's onreadystatechange handler!", error.message );
                                }
                            }*/
                            me.evalGlobal( "void(0);", portletId, deferred, onreadystatechange );
                            
                            if ( dfd ) { dfd.callback(); }
                        }
                        else {
						  var ioArgs = { 
						  	url: url, 
						  	method: "GET",
						  	handle: function ( response, ioArgs ) {
						  		var xhr = ioArgs.xhr;
						  		if ( xhr.status == 200 ) {
									var data = xhr.responseText;
									ibm.portal.debug.text( "Retrieved JS file: " + data );
								  	if ( onreadystatechange ) {
								    	onreadystatechange( "loaded" );
								    }
								    me.evalGlobal( data, portletId, deferred, onreadystatechange );
								    if ( dfd ) { dfd.callback(); }
								  }
								  else if ( xhr.status == 403 ) {
									var messages = dojo.i18n.getLocalization("com.ibm.portal", "v3theme", csaUserLocale);
									status = new Error( messages[ "javascriptProxyNotEnabled" ] );
									if ( dfd ) { dfd.errback( status ); }
								  }
								  else {
									//Should likely never happen.
									status = new Error( "Javascript error from: " + url + ". Error: " + xhr.statusText );
									if ( dfd ) { dfd.errback( status ); }
								  }
						  		} 
                        	}
                        	status = dojo.xhrGet( ioArgs );
                        }      
                    } catch ( error ) {
                        ibm.portal.debug.text( "Unable to load javascript file: " + url, _method );
                        com.ibm.portal.aggregation.error.ERROR_HANDLER.reportError( "Error while loading javascript file: " + url, error.message );
                        if ( dfd ) { dfd.errback( "Unable to load javascript file: " + url, _method ); }
                    } 
                    
                    ibm.portal.debug.exit( _method, status );
                    return status;      
                },
                _rewriteURL: function (/*String*/ targetUrl, /*String*/ proxyUrl) {
                    var newURL = proxyUrl;
                    
                    // check if this is a proxy request:
                    if ( !com.ibm.portal.utilities.isExternalUrl( targetUrl ) ) {
                        return targetUrl;
                    }

                    // rewrite
                    if (targetUrl.indexOf("https") == 0) 
                        newURL += "https/";
                    else
                        newURL += "http/";

                    newURL += targetUrl.substr(targetUrl.indexOf("://") + 3);
                    return newURL;
                }
              }   
);

dojo.declare( "com.ibm.portal.aggregation.javascript.InlineScriptFilter",
              com.ibm.portal.aggregation.javascript.Filter,
              {
                doFilter: function ( /*HTMLElement*/script, /*String*/portletId, /*dojo.Deferred?*/deferred ) {
                    // summary: Filter the given event.
                    // description: Allows this filter to handle or alter the script. If this function
                    //      returns true, processing is stopped and the script is considered "handled".
                    // script: the script block (HTMLElement) 
                    

                    var handled = false, tokenContents = "";
                    
                    if ( !dojo.isString( script ) ) {
                        //Use innerHTML because IE doesn't correctly provide the textContent for a script tag.
                        tokenContents = script.innerHTML;
                    }
                    else {
                        var firstIndex = scriptStr.indexOf(">");
                        var lastIndex = scriptStr.lastIndexOf("<");
                        var scriptStr = script;
                        tokenContents = scriptStr.substring(firstIndex+1, lastIndex);
                    }
                    
                    ibm.portal.debug.text( "Stripped HTML tags out: " + tokenContents, "processScriptArray" );
                    if ( tokenContents ) {
                        // prepare bufferDiv for document.write/ln
                        var buffer = com.ibm.portal.utilities.html.createTemporaryMarkupDiv( "" ).node;
						dojo.style(buffer, "display", "none");
						dojo.body().appendChild(buffer);
                        this.prepareDocumentWrite(buffer);
                        // Eval the contents
                        this.evalGlobal( tokenContents, portletId );
                        handled = true;
                        // apply document.write/ln to DOM
                        this.applyDocumentWrite(script, buffer);
                        if ( deferred ) { deferred.callback(); }
                    }
                    
                    return handled; 
                }
              }  
);

dojo.declare( "com.ibm.portal.aggregation.javascript.PlainScriptFilter",
              com.ibm.portal.aggregation.javascript.Filter,
              {
                doFilter: function ( /*String*/script, /*String*/portletId, /*dojo.Deferred*/deferred ) {
                    // summary: Filter the given event.
                    // description: Allows this filter to handle or alter the script. If this function
                    //      returns true, processing is stopped and the script is considered "handled".
                    // script: the script block (String) 
                    
                    if (!dojo.isString(script) || (script && script.toLowerCase() == "script") )
                        return false; // not handled

                    var tokenContents = script;
                    
                    ibm.portal.debug.text( "Executed script: " + tokenContents, "processScriptArray" );
                    if ( tokenContents ) {
                        // prepare buffer for document.write/ln
                        //var buffer = {content: ""};
                        //this.prepareDocumentWrite(buffer);
                        // Eval the contents
                        this.evalGlobal( tokenContents, portletId );
                        
                        // apply document.write/ln to DOM
                        //this.applyDocumentWrite(script, buffer);
                        if ( deferred ) { deferred.callback(); }
                    }

                    return true;    
                }
              }  
);

dojo.declare( "com.ibm.portal.aggregation.javascript.FilterChain",
              null,          
              {
                // summary: Controls the registration and execution of the filters associated
                //      with this filter chain.
                constructor: function () {
                    this._filters = new Array();
                },
                addFilter: function ( filter ) {
                    // summary: Register a filter at the end of the filter chain.
                    // filter: the filter to append
                    if ( !this._filters ) { this._filters = new Array(); }
                    this._filters.push( filter );
                },
                applyFilters: function ( script, portletId, deferred ) {
                    // summary: Execute the filter chain.
                    // description: Calls doFilter on every filter registered in this filter chain
                    //      until all filters have been called or one of the filters returns true.
                    // e: the event object
                    // returns: true if the event was handled, false if it was not
                    var i = 0;
                    var returnValue = false;
                    while ( i < this._filters.length && !returnValue )  {
                        returnValue = this._filters[i].doFilter(script, portletId, deferred);
                        i = i + 1;      
                    }
                    
                    return returnValue; //Boolean
                }   
              } 
);

dojo.declare( "com.ibm.portal.aggregation.javascript.PortletJavascriptHandler",
              null,
              {
                // summary: Examines all javascript included by a portlet, modifies it if 
                //      necessary and evaluates it in the global context.
                // description: All script elements must be loaded manually since the portlet
                //      markup is added asynchronously. This handler will evaluate a script
                //      element in the global context or load an external script file if necessary.
                constructor: function () {
                    this.filterChain = new com.ibm.portal.aggregation.javascript.FilterChain();
                },
                handle: function ( script, portletId ) {
                    // summary: Handles a script element added asynchronously.
                    // script: the script element
                    ibm.portal.debug.entry( "JavascriptHandler.handle", [ script, portletId ] );
                    //try {
                    var val = this.filterChain.applyFilters( script, portletId );
                    //}
                    //catch ( error ) {
                    //   com.ibm.portal.aggregation.error.ERROR_HANDLER.reportError( "Error encountered while processing script tags for portlet: " + portletId, error.message );
                    //}   
                    ibm.portal.debug.exit( "JavascriptHandler.handle" );
                }
              }  
);

dojo.declare( "com.ibm.portal.aggregation.javascript.AsyncJavascriptProcessor",
			  null,
			  {
			  	constructor: function () {
			  		this.filterChain = new com.ibm.portal.aggregation.javascript.FilterChain();	
			  	},
			  	handle: function ( script, portletId ) {
                    // summary: Handles a script element added asynchronously.
                    // script: the script element
                    ibm.portal.debug.entry( "AsyncJavascriptProcessor.handle", [ script, portletId, deferred ] );

                    var deferred = new dojo.Deferred();
                    this.filterChain.applyFilters( script, portletId, deferred );

                    ibm.portal.debug.exit( "AsyncJavascriptProcessor.handle" );
                    return deferred;
                }
			  }	
);

dojo.declare( "com.ibm.portal.aggregation.javascript.AsyncPortletScriptElementsHandler",
			  null,
			  {
			  	constructor: function ( /*Array*/scriptElements, /*String*/pId, /*AsyncJavascriptProcessor*/scriptProcessor ) {
			  		// summary: Constructs a new object for handling an array of script elements associated with a single portlet instance. Each script
			  		//		element is looped over and evaluated asynchronously (non-blocking) to allow other scripts to continue to execute.
			  		// description: A consumer/interested party interacts with the asynchronous script handler through the event handler object. Before
			  		//		"start"ing the script handling process, the consumer or interested party should register an event handler object using the 
			  		//		setEventHandler method. The event handler object should define a method for "error", "scriptDone", "completed", and "started".
			  		//		The implementation does not have to DO anything for every event, but each method MUST be defined, even if it is empty. The script
			  		//		handler will call each method as each event occurs - "started" when script handling begins; "scriptDone" when an individual script
			  		//		element is done; "error" when a script element causes an error; and "completed" when all script elements have been executed.
			  		// scriptElements: the array of script elements to process
			  		// pId: the portlet ID of the portlet instance to which the script elements belong
			  		// scriptProcessor: the object which will actually process the individual script elements
			  		var _method = "AsyncPortletSCriptElementsHandler.constructor";
			  		ibm.portal.debug.entry( _method, [ scriptElements, pId, scriptProcessor ] );
			  		
			  		this.index = 0;
			  		this.elements = scriptElements;
			  		this.eventHandler = null;
			  		this.portletId = pId;
			  		this.processor = scriptProcessor;
			  		this.stopOnError = false;
			  		
			  		ibm.portal.debug.exit( _method );
			  	},
			  	setEventHandler: function ( /*Object*/handler ) {
			  		// summary: Sets the event handler object. If a required method is not defined, the handler will be ignored.
			  		// description: The event handler object should define a method for "error", "scriptDone", "completed", and "started". The implementation 
			  		// 		does not have to DO anything for every event, but each method MUST be defined, even if it is empty.
			  		// handler: the event handler object
			  		var _method = "AsyncPortletScriptElementsHandler.setEventHandler";
			  		ibm.portal.debug.entry( _method, [ handler ] );
			  		
			  		if ( handler.error && handler.scriptDone && handler.completed && handler.started ) {
			  			this.eventHandler = handler;
			  		}
			  		
			  		ibm.portal.debug.exit( _method );
			  	},
			  	_fireEvent: function ( /*String*/eventName, /*Array?*/args ) {
			  		// summary: Fires the given event on the provided event handler.
			  		// eventName: the name of the event to fire
			  		var _method = "_fireEvent";
			  		ibm.portal.debug.entry( _method, [ eventName ] );
			  		
			  		if ( this.eventHandler && this.eventHandler[eventName] ) {
			  			var eventFunc = this.eventHandler[eventName];
			  			//IE doesn't like apply calls with an undefined array. Firefox seems to fail gracefully in this case.
			  			if ( args ) {
			  				eventFunc.apply( eventFunc, args );
			  			}
			  			else {
			  				eventFunc();
			  			}	
			  		}
			  		
			  		ibm.portal.debug.exit( _method );
			  	},
			  	start: function () {
			  		// summary: Starts processing the script elements provided in the constructor.
			  		var _method = "AsyncPortletScriptElementsHandler.start";
			  		ibm.portal.debug.entry( _method );
			  		
			  		this._fireEvent( "started" );
			  		this.nextScript();
			  		
			  		ibm.portal.debug.exit( _method );
			  	},
			  	scriptDone: function () {
			  		// summary: Indicates an individual script element has completed.
			  		var _method = "AsyncPortletScriptElementsHandler.scriptDone";
			  		ibm.portal.debug.entry( _method );
			  		
			  		this._fireEvent( "scriptDone" );
			  		
			  		this.index = this.index+1;
			  		this.nextScript();
			  		
			  		ibm.portal.debug.exit( _method );
			  	},
			  	scriptError: function ( error ) {
			  		// summary: Indicates an error has occurred during script processing, not execution.
			  		// description: Please note that this refers to script processing, not execution. Errors that occur
			  		//		during script execution will interrupt this process.
			  		// error: the error that was generated
			  		var _method = "AsyncPortletScriptElementsHandler.scriptError";
			  		ibm.portal.debug.entry( _method, [ error ] );
			  		
			  		this._fireEvent( "error", [ error ] );
			  		
			  		if ( this.stopOnError ) {
			  			this.completed();
			  		}
			  		else {
			  			this.index = this.index+1;
			  			this.nextScript();
			  		}	
			  		
			  		ibm.portal.debug.exit( _method );
			  	},
			  	completed: function () {
			  		// summary: Indicates that the all script elements have been processed.
			  		var _method = "AsyncPortletScriptElementsHandler.completed";
			  		ibm.portal.debug.entry( _method );
			  		
			  		this._fireEvent( "completed" );
			  		
			  		ibm.portal.debug.exit( _method );
			  	},
			  	nextScript: function () {
			  		// summary: Called to indicate that the next script can be processed.
			  		// description: The actual call to execute the script element is wrapped inside a timeout to allow
			  		//		all running javascript to complete.
			  		var _method = "AsyncPortletScriptElementsHandler.nextScript";
			  		ibm.portal.debug.entry( _method );
			  		
			  		var me = this;
			  		setTimeout( function () {
			  			if ( me.index < me.elements.length ) {
				  			var d = me.processor.handle( me.elements[me.index], me.portletId );
				  			d.addCallback( dojo.hitch( me, "scriptDone" ) );
				  			d.addErrback( dojo.hitch( me, "scriptError" ) );
				  		} else {
				  			me.completed();
				  		}
			  		}, 5);
			  		
			  		
			  		ibm.portal.debug.exit( _method );
			  	}
			  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.FunctionExecutor",
        null,
        {
         	// summary: Executes the appropriate event broker events before and after the execution
			// 		of the provided function.
			// description: The primary use case for this executor is when executing onload handlers. We need
			//		to be able to clean up any javascript variables or widgets created in an onload handler 
			//		(for example, the dojo parser is usually called in an onload handler). The executor fires
			//		the startScriptExecution/endScriptExecution events around executing the provided function.
			// func: the function to execute
			// portletId: the id of the portlet executing the function
			// deferred: a deferred object (OPTIONAL)
			// returns: returns the return value from the provided function
		   	run: function ( /*Function*/func, /*String*/portletId, /*dojo.Deferred*/deferred ) {
              // summary: Filter the given event.
              // description: Allows this filter to handle or alter the script. If this function
              //      returns true, processing is stopped and the script is considered "handled".
              // script: the script block (String) 
              var _method = this.declaredClass + ".run";
			  ibm.portal.debug.entry( _method, [ func, portletId, deferred ] );
              
              ibm.portal.debug.text( "Executing function: " + func.toString() );
              var retVal = undefined;
              if ( func ) {
                  com.ibm.portal.EVENT_BROKER.startScriptExecution.fire( { portletId: portletId } );
                  retVal = func();
                  com.ibm.portal.EVENT_BROKER.endScriptExecution.fire( { portletId: portletId } );
                  if ( deferred ) { deferred.callback(); }
              }

              ibm.portal.debug.exit( _method, retVal );
              return retVal;    
          }
        }  
);

dojo.declare( "com.ibm.portal.aggregation.javascript.OnloadHandler",
              null,
              {
                // summary: Handles any onload handlers a portlet might register.
                // description: Looks for any onload handlers set using the W3C methods or added by
                //        setting an attribute on the document body.
                
                constructor: function () {
                    this._onLoadMap = [];
                    this._activePortlet = null;
                
                    this._startScriptHandlingHandle = com.ibm.portal.EVENT_BROKER.startScriptHandling.register( this, "startListening" );
                    this._setContextHandle = com.ibm.portal.EVENT_BROKER.startScriptExecution.register( this, "setRegistrationContext" );
                    this._clearContextHandle = com.ibm.portal.EVENT_BROKER.endScriptExecution.register( this, "clearRegistrationContext" );
                    this._endScriptHandlingHandle = com.ibm.portal.EVENT_BROKER.endScriptHandling.register( this, "stopListening" );    
                    //In the case where a portlet with an inline-mode enabled skin has one of the inline modes open upon initial rendering,
                    //(portlet is put into Edit Shared Settings mode and then the user hits Refresh), there may be multiple listen begin notifications
                    //for the case where we render the view mode as well as the edit mode. We only want to override the native functions once so
                    //we'll track the number of listeners and only restore (or override) the native functions when the number of listeners is zero.
                    this.listening = 0;
                },                
                startListening: function ( args ) {
                    // summary: Starts listening for the registration of onload handlers.
                    // description: Overwrites the addEventListener/attachEvent functions on the window
                    //        object, as well as, the setAttribute function on the body object.
                    ibm.portal.debug.entry( "OnloadHandler.startListening", [ args ] );
                	
                	if ( this.listening == 0 ) {
	                	var me = this;
	                    //this._activePortlet = args["portletId"];
	                    if ( window.addEventListener ) {
	                        this.window_addEventListener = window.addEventListener;
	                        window.addEventListener = function (a,b,c) { me._addEventListener( a,b,c ); };
	                    }
	                    else {
	                        window.window_attachEvent = window.attachEvent;                
	                        window.attachEvent = function(a,b) { me._attachEvent(a,b); };
	                    }   
	    
	                    this.original_setAttribute = document.body.setAttribute;
	                    document.body.setAttribute = function ( a,b,c) { me._interceptSetAttribute(a,b,c ); };
                	}
                	
                	this.listening++;
                    
                	ibm.portal.debug.text( "Number of listening handlers: " + this.listening );
                	
                    ibm.portal.debug.exit( "OnloadHandler.startListening" );
                },
                stopListening: function ( args ) {
                    // summary: Stops listening for onload handlers.
                    // description: Restores all overwritten functions to the original state. startListening should
                    //        never be called without a corresponding stopListening call.
                    ibm.portal.debug.entry( "OnloadHandler.stopListening" );
                    
                	this.listening--;
                
                	ibm.portal.debug.text( "Number of listening handlers: " + this.listening );
                	
                    if ( this.listening == 0 ) {
	                	if ( window.addEventListener ) {
	                        window.addEventListener = this.window_addEventListener;
	                    }
	                    else {
	                        window.attachEvent = window.window_attachEvent;                
	                    }   
	    
	                    document.body.setAttribute = this.original_setAttribute;
                    }    
                    //com.ibm.portal.EVENT_BROKER.startScriptHandling.unregister( this, "startListening" );
                    //com.ibm.portal.EVENT_BROKER.endScriptHandling.unregister( this, "stopListening" );    
                    
                    this.executeHandlers( args ); 
                    this._activePortlet = null;
                    
                    ibm.portal.debug.exit( "OnloadHandler.stopListening" );
                },
                executeHandlers: function( args ) {
                    // summary: Executes the registered onload handlers for the given portletid.
                    // portletid: the portlet id to execute onload handlers for
                    ibm.portal.debug.entry( "OnloadHandler.executeHandlers" );
                    
                    var portletid = args["portletId"];
                    var handlers = this._onLoadMap[ portletid ];
                    if ( handlers ) { 
                    	var executor = new com.ibm.portal.aggregation.javascript.FunctionExecutor();
                    	
                    	executor.run( function () { 
	                    	for (var i=0; i<handlers.length; i++) {
	                             var obj = handlers[i];
	
	                             try {
	                                 var fkt = obj.fkt;
	                                 var pthis = obj.pthis;
	                                 fkt.call(pthis);
	                             }
	                             catch (error)
	                             {
	                                    com.ibm.portal.aggregation.error.ERROR_HANDLER.reportError( "Unable to execute onload function!", error.message );
	                             }
	                         }
	                         
	                         
                    	}, portletid);
                    	
                    	delete this._onLoadMap[portletid];
                    }
                    
                    ibm.portal.debug.exit( "OnloadHandlers.executeHandlers" );   
                },
                setRegistrationContext: function ( /*Object*/ args ) {
                    // summary: Sets the active portlet id.
                    this._activePortlet = args["portletId"];    
                },
                clearRegistrationContext: function ( /*Object*/ args ) {
                    // summary: Clears the active portlet id.
                    this._activePortlet = null;
                },
                _registerHandler: function ( /*Function*/ handler ) {
                    // summary: Registers an onload handler.
                    // handler: the onload handler function
                    ibm.portal.debug.entry( "OnloadHandler._registerHandler", [ handler ] );
                    
                    var map = this._onLoadMap[this._activePortlet];
                    
                    //Create the map for this portlet if necessary. 
                    if ( !map ) {
                        this._onLoadMap[this._activePortlet] = [];
                        map = this._onLoadMap[this._activePortlet];
                    }
                    //Create the function that will be called when the onload handlers
                    //are executed. We need to create an event object to simulate the
                    //actual onload call.
                    var _fkt = function () {
                        ibm.portal.debug.text( "handler function is: " + handler );
                        if ( document.createEvent ) {
                            var event = document.createEvent("Events");
                            event.initEvent( "load", true, true );
                            handler.apply( window, [ event ] );
                        }
                        else {
                            var funct = function () { return handler(); };
                            document.body.attachEvent( "onload", funct );
                            document.body.fireEvent( "onload" );
                            document.body.detachEvent( "onload", funct );
                        }
                    };
                    //Add to the map.
                    map.push( {text: "",
                               fkt: _fkt,
                               pthis: window });    
                               
                    ibm.portal.debug.exit( "OnloadHandler._registerHandler" );
                },
                _addEventListener: function(eventType, handler, useCapture) {
                    // summary: Overwritten function for adding an onload handler.
                    if (eventType=="load" && this._activePortlet ) {
                        this._registerHandler( handler );                   
                    }
                    else {
                        this.window_addEventListener.apply(window, [eventType, handler, useCapture]);
                    }   
                },
                _attachEvent: function(event, handler) {
                    // summary: Overwritten function for adding an onload handler in IE.
                    if (event=="onload" && this._activePortlet ) {
                        this._registerHandler( handler );
                    }
                    else {
                        window.window_attachEvent( event, handler );
                    }
                },
                _interceptSetAttribute: function(name, value, caseSensitiv) {
                    // summary: Overwritten function for setting an onload handler on the body.
                    if (name=="onload" && this._activePortlet ) {
                        var fkt = null;
                                        
                        if ( typeof(value) == "string" && value.indexOf("function" ) != 0 ) {
                            //JSF Hack
                            var text = "var event = document.createEvent(\"Events\"); event.initEvent(\"load\", true, true); " + value;
                            fkt = new Function( text );
                        }
                        else if ( typeof(value) == "function" ){
                            fkt = value;    
                        }
                        
                        this._registerHandler( fkt );
                    }
                    else {
                        this.original_setAttribute.apply( document.body, [ name, value, caseSensitiv ]);
                    }
                }   
              }
);

com.ibm.portal.aggregation.javascript.JS_ONLOAD_HANDLER = new com.ibm.portal.aggregation.javascript.OnloadHandler();
com.ibm.portal.aggregation.javascript.JAVASCRIPT_HANDLER = new com.ibm.portal.aggregation.javascript.PortletJavascriptHandler();
com.ibm.portal.aggregation.javascript.ASYNC_JAVASCRIPT_HANDLER = new com.ibm.portal.aggregation.javascript.AsyncJavascriptProcessor();

//Register the filters. The order registered is the order executed.
com.ibm.portal.aggregation.javascript.JAVASCRIPT_HANDLER.filterChain.addFilter( new com.ibm.portal.aggregation.javascript.PlainScriptFilter() );
com.ibm.portal.aggregation.javascript.JAVASCRIPT_HANDLER.filterChain.addFilter( new com.ibm.portal.aggregation.javascript.ExternalScriptFilter() );
com.ibm.portal.aggregation.javascript.JAVASCRIPT_HANDLER.filterChain.addFilter( new com.ibm.portal.aggregation.javascript.InlineScriptFilter() );

com.ibm.portal.aggregation.javascript.ASYNC_JAVASCRIPT_HANDLER.filterChain.addFilter( new com.ibm.portal.aggregation.javascript.PlainScriptFilter() );
com.ibm.portal.aggregation.javascript.ASYNC_JAVASCRIPT_HANDLER.filterChain.addFilter( new com.ibm.portal.aggregation.javascript.ExternalScriptFilter() );
com.ibm.portal.aggregation.javascript.ASYNC_JAVASCRIPT_HANDLER.filterChain.addFilter( new com.ibm.portal.aggregation.javascript.InlineScriptFilter() );

}

if(!dojo._hasResource["com.ibm.portal.aggregation.javascript.uneval"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.javascript.uneval"] = true;
dojo.provide( "com.ibm.portal.aggregation.javascript.uneval" );

dojo.require( "com.ibm.portal.EventBroker" );

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.Collector",
			  null,
			  {
				// summary: A collector is called before or after a piece of javascript is executed. The delta between the two
				//		collections determines what javascript variables/functions were declared in that piece of javascript. THis
				//		approach works for all browsers except IE because IE does not enumerate global javascript variables/functions
				//		as part of the global object/scope. In IE, any global javascript variables that need to be cleaned up should
				//		added to the com.ibm.portal.aggregation.javascript.uneval.collectors.COMMON_GLOBAL_VARIABLES collection.
				collect: function () {
					// summary: Collect the necessary information required to clean up a set of properties or objects.
					dojo.unimplemented( "com.ibm.portal.aggregation.javascript.uneval.Collector.collect" );
				},
				getValues: function () {
					// summary: Retrieves the values collected by the collect action.
					// return: implementation specific; a scrubber and a collector usually go hand in hand
					dojo.unimplemented( this.declaredClass + ".getValues" );
				}	
			  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.Scrubber",
			  null,
			  {
				// summary: A scrubber acts as a factory to create specific collector instances and will perform the process of 
				//		cleaning up or null'ing out the delta between the values collected by the collectors. 
				newCollector: function () {
					// summary: Creates a new collector appropriate for this scrubber.
					// returns: a new instance of a Collector
					dojo.unimplemented( "com.ibm.portal.aggregation.javascript.uneval.Scrubber.newCollector");
				},
				clean: function ( /*Collector*/collector1, /*Collector*/collector2 ) {
					// summary: Given two collectors, clean the difference between the two.
					// collector1: the initial state to return to
					// collector2: the final state
					dojo.unimplemented( "com.ibm.portal.aggregation.javascript.uneval.Scrubber.clean" );
				},
				getId: function () {
					// summary: Retrieve an ID describing this Scrubber. The ID is unique to the class, but not
					//		the instance.
					return this.declaredClass;
				}	
			  }
);



dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.Snapshot",
			  null,
			  {
			  	// summary: A Snapshot cycles through all the registered scrubbers and executes the collector for
				//		that scrubber. A Snapshot is taken before and after each piece of JavaScript is executed.
				constructor: function () { 
			  		ibm.portal.debug.entry( "Snapshot.constructor" );
			  		
			  		this._collectors = {};
			  		var scrubbers = com.ibm.portal.aggregation.javascript.uneval.RestorePoint.prototype.SCRUBBERS;
			  		var length = scrubbers.length;
			  		ibm.portal.debug.text( "Executing " + length + " scrubbers.");
			  		for ( var i = 0; i < length; i++ ) {
			  			var scrubber = scrubbers[i];
			  			ibm.portal.debug.text( "Executing scrubber: " + scrubber.declaredClass );
			  			var collector = scrubber.newCollector();
			  			collector.collect();
			  			this._collectors[ scrubber.getId() ] = collector;
			  		}	
			  		
			  		ibm.portal.debug.exit( "Snapshot.constructor" );
			  	},
				getCollectors: function () {
			  		return this._collectors;
			  	}	
			  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.RestorePoint",
			  null, 
			  {
			  	// summary: Represents the current globally available javascript functions and
				//		variables.
				// description: The "recording" is made by essentially seeing what properties the window object has
				//		before the portlet's javascript is executed and then again after the portlet's
				//		javascript has been loaded. The new properties are the ones that will be 
				//		deleted when the "restoration" is performed. 
				SCRUBBERS: [],
				constructor: function ( /*Snapshot*/state1, /*Snapshot*/state2 ) { 
			  		this._state1 = state1;
			  		this._state2 = state2;
			  	},
				restore: function ( /*Scrubber*/scrubber ) {
					// summary: Restores the window object to the given restore point. If no deltaRestorePoint
					//		is specified, the current window object's properties are used as a comparison point.
					
			  		var collectors1 = this._state1.getCollectors();
			  		var collectors2 = this._state2.getCollectors();
			  		
			  		var collector1 = collectors1[scrubber.getId()];
			  		var collector2 = collectors2[scrubber.getId()];
			  		
			  		if ( collector1 && collector2 ) {
			  			scrubber.clean( collector1, collector2 );
			  		}	
					
					com.ibm.portal.EVENT_BROKER.javascriptCleanup.fire();
				},
				restoreAll: function () {
					var scrubbers = this.SCRUBBERS;
			  		var collectors1 = this._state1.getCollectors();
			  		var collectors2 = this._state2.getCollectors();
			  		
			  		var length = scrubbers.length;
			  		for ( var i = 0; i < length; i++ ) {
			  			var scrubber = scrubbers[i];
			  			var collector1 = collectors1[scrubber.getId()];
			  			var collector2 = collectors2[scrubber.getId()];
			  			
			  			if ( collector1 && collector2 ) {
			  				scrubber.clean( collector1, collector2 );
			  			}	
			  		}	
					
					com.ibm.portal.EVENT_BROKER.javascriptCleanup.fire();
				},	
				getSupportedScrubbers: function() {
					return this._state1.SCRUBBERS;
				}
			  }		
);

}

if(!dojo._hasResource["com.ibm.portal.aggregation.javascript.uneval.collectors"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.javascript.uneval.collectors"] = true;
dojo.provide( "com.ibm.portal.aggregation.javascript.uneval.collectors" );

dojo.require( "com.ibm.portal.aggregation.javascript.uneval" );

com.ibm.portal.aggregation.javascript.uneval.collectors.COMMON_GLOBAL_VARIABLES = {
	// summary: Tracks any global variables that should be tested for when doing JavaScript
	//		clean up between portlet requests and page switches. This is really only applicable
	//		for IE since we cannot detect global variables in IE. 
	// description: IE does not properly enumerate global javascript variables as properties of the global scope (usually window).
	//		Therefore, we need to have an explicit list of global javascript variables that COULD be created by a portlet and need
	// 		to be cleaned up when the portlet is no longer on the page (or between requests). The primary use case for this would be for
	//		javascript libraries (i.e. JSF, dojo, etc).
	_variableNames: [],
	_variableNamesIndex: {},
	addVariable: function ( /*String*/ varName ) {
		// summary: Add a variable name to the "watched" list.
		// varName: the name of the variable to watch for
		if ( !this.contains( varName ) ) {
			this._variableNames.push( varName );
			this._variableNamesIndex[ varName ] = true;
		}
	},
	contains: function ( /*String*/ varName ) {
		// summary: Indicates whether or not the given variable name is on the "watched" list.
		// varName: the variable name to test for
		// returns: true if the variable name is watched; false if it is not
		return this._variableNamesIndex[ varName ];
	},
	getVariableNames: function () {
		// summary: Returns a list of all "watched" variable names.
		// returns: an array of Strings consisting of the "watched" variable names
		return this._variableNames;
	}
} 

if ( dojo.isIE ) {
	//IBM JSF v3 support
	com.ibm.portal.aggregation.javascript.uneval.collectors.COMMON_GLOBAL_VARIABLES.addVariable( "hX_5" );
}

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.collectors.WindowPropertyCollector",
		  com.ibm.portal.aggregation.javascript.uneval.Collector,
		  {
			// summary: Collects the enumerated properties in the global scope in most browsers. In IE, collects
			//		any variables in the COMMON_GLOBAL_VARIABLES list that are actually defined.
			constructor: function ( /*Array<String>*/ globalJSVariables ) {
				this._properties = [];
			},
			collect: function () {
				// summary: Collect the necessary information required to clean up a set of properties or objects.
				var _method = this.declaredClass + ".collect";
				ibm.portal.debug.entry( _method );
				
				for ( prop in window ) {
					this._properties[prop] = true;
				}
				var globalVars = com.ibm.portal.aggregation.javascript.uneval.collectors.COMMON_GLOBAL_VARIABLES.getVariableNames();
				var len = globalVars.length;
				for ( var i = 0; i < len; i++ ) {
					ibm.portal.debug.text( "Eval'ing global variable name: " + globalVars[i] );
					//Unfortunately, the only way I can think of to change the string variable name to the actual variable object is 
					//through eval. Since this code will only be executed in IE, we cannot simply test the window[variableName] property.
					//If an error occurs eval'ing the variable name, we will assume that it is undefined.
					try {
						var varType = eval (globalVars[i]);
					} catch ( err ) {
						varType = undefined;
					}
					ibm.portal.debug.text( "varType is: " + varType );
					if ( typeof varType != "undefined" ) {
						ibm.portal.debug.text( "Marking property \"" + globalVars[i] + "\" as defined!" );
						this._properties[globalVars[i]] = true;
					}
				}
				
				ibm.portal.debug.exit( _method );
			},
			getValues: function () {
				// summary: Retrieves the values collected by the collect action.
				// return: implementation specific; a scrubber and a collector usually go hand in hand
				var _method = this.declaredClass + ".getValues";
				ibm.portal.debug.entry( _method );
				ibm.portal.debug.exit( _method, this._properties );
				return this._properties;
			}
		  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.collectors.DijitCollector",
		  com.ibm.portal.aggregation.javascript.uneval.collectors.WindowPropertyCollector,
		  {
			// summary: Collects any dijits (Dojo widgets) that are currently defined.
			collect: function () {
				var _method = this.declaredClass + ".collect";
				ibm.portal.debug.entry( _method );
				// summary: Collect the necessary information required to clean up a set of properties or objects.
				if(typeof dijit != "undefined" && dijit.registry) {
					var array = this._properties;
					dijit.registry.forEach(function(widget){ ibm.portal.debug.text( "Adding widget: " + widget.id ); array[widget.id] = true; } );
				}
				ibm.portal.debug.exit( _method );
			}
		  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.collectors.DojoCollector",
		  com.ibm.portal.aggregation.javascript.uneval.collectors.WindowPropertyCollector,
		  {
			// summary: Collects any dojo modules that are currently defined. 
			collect: function () {
				// summary: Collect the necessary information required to clean up a set of properties or objects.
				var _method = this.declaredClass + ".collect";
				ibm.portal.debug.entry( _method );
				var array = this._properties;
				if ( dojo && dojo._modulePrefixes ) {
					for( moduleName in dojo._modulePrefixes ) {
						array[moduleName] = true;
					}
				}
				ibm.portal.debug.exit( _method );
			}
		  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.collectors.IntervalCollector",
		  com.ibm.portal.aggregation.javascript.uneval.collectors.WindowPropertyCollector,
		  {
			// summary: Collects any intervals that have been set. We need to clean these up when the portlet
			//		goes out of scope since the interval function would no longer be relevant.
			collect: function () {
				// summary: Collect the necessary information required to clean up a set of properties or objects.
				var _method = this.declaredClass + ".collect";
				ibm.portal.debug.entry( _method );
				var utils = com.ibm.portal.utilities;
					
				if ( utils.getOverwrittenProperty( window, "setInterval" ) ) {
					ibm.portal.debug.text( "Restoring interval property." );
					utils.restoreProperty( window, "setInterval" );
				}
				else {	
					ibm.portal.debug.text( "Overwriting interval property." );
					var me = this;
					utils.overwriteProperty( window, "setInterval", function () { return me._setInterval.apply( me, arguments ); } );
				}
				ibm.portal.debug.exit( _method );
			},
			_setInterval: function () {
				ibm.portal.debug.entry( "IntervalCollector._setInterval", arguments );
				
				ibm.portal.debug.text( "window.setInterval is: " + window.setInterval );
				ibm.portal.debug.text( "window[_overwritten_]: " + window["_overwritten_"]);
				ibm.portal.debug.text( "window[_overwritten_][setInterval]: " + window["_overwritten_"]["setInterval"]);
				
				var timeoutFn = com.ibm.portal.utilities.getOverwrittenProperty( window, "setInterval" );
				ibm.portal.debug.text( "Retrieved original overwritten property: " + timeoutFn );
				var pointer = timeoutFn( arguments[0], arguments[1] );
				this._properties.push( pointer );
				
				ibm.portal.debug.exit( "IntervalCollector._setInterval", pointer );
				return pointer;
			}
		  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.collectors.IBMJSFv3Collector",
		  com.ibm.portal.aggregation.javascript.uneval.collectors.WindowPropertyCollector,
		  {
			// summary: Collects any JSF components that are currently defined. NOTE: Only applicable to 
			//		IBM's JSF v3 implementation.
			collect: function () {
				// summary: Collect the necessary information required to clean up a set of properties or objects.
				var _method = this.declaredClass + ".collect";
				ibm.portal.debug.entry( _method );
	
				this._properties = [];
				
				if(typeof hX_5 != "undefined") {
					var components = hX_5.getComponentsInId( "mainContent" );
					ibm.portal.debug.text( "retrieved components: " + components );
					
					if ( components ) {
						var length = components.length;
						for ( var i = 0; i < length; i++ ) {
							this._properties[ components[i] ] = true;
						}
					}	
				}
				
				ibm.portal.debug.exit( _method );
			}
		  }
);

}

if(!dojo._hasResource["com.ibm.portal.aggregation.javascript.uneval.scrubbers"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.javascript.uneval.scrubbers"] = true;
dojo.provide( "com.ibm.portal.aggregation.javascript.uneval.scrubbers" );

dojo.require( "com.ibm.portal.aggregation.javascript.uneval" );
dojo.require( "com.ibm.portal.aggregation.javascript.uneval.collectors" );

//summary: Provides a list of methods and properties which should never be deleted during cleanup.
//description: In IE, even native methods cannot be enumerated using the global scope. This is a bug in IE or the JScript
//		engine. If we overwrite a native javascript function via window.<method_name> = , then the function becomes declared
//		in the global scope. Since it didn't "exist" in a retrievable form before we took the initial snapshot, the function 
//		appears new to our cleanup code and we attempt to delete it. This list is taken from the MSDN documentation for the window
//		object.	 
com.ibm.portal.aggregation.javascript.uneval.DO_NOT_DELETE = {   "alert": true,
																 "attachEvent": true,
																 "blur": true,
																 "clearInterval": true,
																 "clearTimeout": true,
																 "close": true,
																 "confirm": true,
																 "createPopup": true,
																 "detachEvent": true,
																 "execScript": true,
																 "focus": true,
																 "moveBy": true,
																 "moveTo": true,
																 "navigate": true,
																 "open": true,
																 "postMessage": true,
																 "print": true,
																 "prompt": true,
																 "resizeBy": true,
																 "resizeTo": true,
																 "scroll": true,
																 "scrollTo": true,
																 "setActive": true,
																 "setInterval": true,
																 "setTimeout": true,
																 "showHelp": true,
																 "showModalDialog": true,
																 "showModelessDialog": true,
																 "_overwritten_": true};

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.scrubbers.WindowPropertyScrubber",
			  com.ibm.portal.aggregation.javascript.uneval.Scrubber,
			  {
				// summary: Manages the WindowPropertyCollectors.
				newCollector: function () {
					// summary: Creates a new collector appropriate for this scrubber.
					// returns: a new instance of a Collector
					return new com.ibm.portal.aggregation.javascript.uneval.collectors.WindowPropertyCollector();
				},
				clean: function ( /*Collector*/collector1, /*Collector*/collector2 ) {
					// summary: Given two collectors, clean the difference between the two.
					// collector1: the initial state to return to
					// collector2: the final state
					var _method = this.declaredClass + ".clean";
					ibm.portal.debug.entry( _method, [ collector1, collector2 ] );
					
					var undeletables = com.ibm.portal.aggregation.javascript.uneval.DO_NOT_DELETE;
					var propsToDelete = this._getDelta( collector1.getValues(), collector2.getValues(), function ( prop ) { return undeletables[ prop ] != true; } );
					var commonGlobals = com.ibm.portal.aggregation.javascript.uneval.collectors.COMMON_GLOBAL_VARIABLES;
					
					//Delete global variables and functions that are new to this restore point.				
					for ( var i = 0; i < propsToDelete.length; i++ ) {
						var prop = propsToDelete[i];
						ibm.portal.debug.text( "Deleting: " + prop );
						
						//Ideally, we want to delete the property using the javascript delete keyword. If that
						//fails, at the very least, we can null out the property.
						try { 
							if ( commonGlobals.contains( prop ) ) {
								try {
									var varValue = eval( prop );
								} catch ( err ) {
									varValue = null;
								}
								ibm.portal.debug.text( "Eval'd variable value: " + varValue );
								varValue = null;
							} else {
								var success = delete window[prop];
								if(!success) window[prop] = null;
							}	
						}
						catch (err) {	
							ibm.portal.debug.text( "An error occurred while deleting property \"" + prop + "\": " + err.message )
							window[prop] = null;
						}						
					}	
					
					ibm.portal.debug.exit( _method );
				},
				_getDelta: function ( /*Array*/beforeArray, /*Array*/afterArray, /*Function?*/conditional ) {
			  		// summary: Creates an array of all the property names that satisfy the given conditional
			  		// beforeArray: the "before" array
			  		// afterArray: the "after" array
			  		// conditional: a function that can be optionally be used to test additional conditions for inclusion
			  		//		in the delta.
			  		// returns: an array of property names that were introduced by the current portlet executing javascript code
			  		var _method = this.declaredClass + "._getDelta";
					ibm.portal.debug.entry( _method, [ beforeArray, afterArray ] );
			  		
			  		var delta = [];
			  		//In case the optional conditional isn't specified...
			  		if ( !conditional ) { conditional = function () { return true; } }
			  		
			  		for ( var prop in afterArray ) {
			  			if ( beforeArray[prop] != true && conditional( prop ) ) {
			  				delta.push( prop );
			  			}
			  		}
			  		
			  		ibm.portal.debug.exit( _method, delta );
			  		return delta;
			  	}
			  }
);



dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.scrubbers.DijitScrubber",
		  com.ibm.portal.aggregation.javascript.uneval.scrubbers.WindowPropertyScrubber,
		  {
			// summary: Manages the DijitCollectors.
			newCollector: function () {
				// summary: Creates a new collector appropriate for this scrubber.
				// returns: a new instance of a Collector
				return new com.ibm.portal.aggregation.javascript.uneval.collectors.DijitCollector();
			},
			clean: function ( /*Collector*/collector1, /*Collector*/collector2 ) {
				// summary: Given two collectors, clean the difference between the two.
				// collector1: the initial state to return to
				// collector2: the final state
				var _method = this.declaredClass + ".clean";
				ibm.portal.debug.entry( _method, [ collector1, collector2 ] );
				
				var dijitsToDelete = this._getDelta( collector1.getValues(), collector2.getValues() );
				
				//Call destroy on any widgets created in this restore point.
				for ( var i = 0; i < dijitsToDelete.length; i++ ) {
					ibm.portal.debug.text( "Deleting: " + dijitsToDelete[i] );

					try { 
						var widget = dijit.byId(dijitsToDelete[i]);
						//destroy recursive, may also destroy ones we haven't gotten to yet
						//so we check to ensure the widget exists first in case it was already removed
						if(widget) widget.destroyRecursive();	
					}
					catch (err) {
						dijit.registry.remove(dijitsToDelete[i]);
					}						
				}
				
				ibm.portal.debug.exit( _method );
			}
		  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.scrubbers.DojoScrubber",
		  com.ibm.portal.aggregation.javascript.uneval.scrubbers.WindowPropertyScrubber,
		  {
			// summary: Manages the DojoCollectors.
			newCollector: function () {
				// summary: Creates a new collector appropriate for this scrubber.
				// returns: a new instance of a Collector
				return new com.ibm.portal.aggregation.javascript.uneval.collectors.DojoCollector();
			},
			clean: function ( /*Collector*/collector1, /*Collector*/collector2 ) {
				// summary: Given two collectors, clean the difference between the two.
				// collector1: the initial state to return to
				// collector2: the final state
				var _method = this.declaredClass + ".clean";
				ibm.portal.debug.entry( _method, [ collector1, collector2 ] );
				
				var dojoModulesToDelete = this._getDelta( collector1.getValues(), collector2.getValues() );
				
				//Delete any dojo modules loaded via dojo.require in this restore point.
				for ( var i = 0; i < dojoModulesToDelete.length; i++ ) {
					ibm.portal.debug.text( "Deleting Dojo Module: " + dojoModulesToDelete[i] );
					
					//Ideally, we want to delete the property using the javascript delete keyword. If that
					//fails, at the very least, we can null out the property.
					try { 
						//Basically have to undo everything here that dojo.require does...
						var moduleName = dojoModulesToDelete[i];
						var moduleUrl = dojo._getModuleSymbols(moduleName).join("/") + ".js";
						var success1 = delete dojo._modulePrefixes[moduleName];
						var success2 = delete dojo._loadedModules[moduleName];
						var success3 = delete dojo._loadedUrls[moduleUrl];
						
						var j = 0; 
						var success4 = false;
						while ( j < dojo._loadedUrls.length && !success4 ) {
							success4 = (dojo._loadedUrls[j] == moduleUrl );
							if (success4) { 
								delete dojo._loadedUrls[j]; 
							}
							j=j+1; 
						}
						
						if(!success1 && !success2 && !success3 && !success4) {
							dojo._modulePefixes[moduleName] = null;
							dojo._loadedModules[moduleName] = null;
							dojo._loadedUrls[moduleUrl] = null;
						}	
					}
					catch (err) {	
						dojo._modulePrefixes[dojoModulesToDelete[i]] = null;
						dojo._loadedModules[dojoModulesToDelete[i]] = null;
					}	
					
					ibm.portal.debug.exit( _method );
				}			
			}
		  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.scrubbers.IntervalScrubber",
		  com.ibm.portal.aggregation.javascript.uneval.scrubbers.WindowPropertyScrubber,
		  {
			// summary: Manages the IntervalCollectors.
			newCollector: function () {
				// summary: Creates a new collector appropriate for this scrubber.
				// returns: a new instance of a Collector
				return new com.ibm.portal.aggregation.javascript.uneval.collectors.IntervalCollector();
			},
			clean: function ( /*Collector*/collector1, /*Collector*/collector2 ) {
				// summary: Given two collectors, clean the difference between the two.
				// collector1: the initial state to return to
				// collector2: the final state
				var _method = this.declaredClass + ".clean";
				ibm.portal.debug.entry( _method, [ collector1, collector2 ] );
				
				var intervalsToDelete = collector1.getValues();
				
				//Clear any intervals set by this restore point.
				for ( i = 0; i < intervalsToDelete.length; i++ ) {
					ibm.portal.debug.text( "Clearing interval: " + intervalsToDelete[i] );
					clearInterval( intervalsToDelete[i] );
				}	
				
				ibm.portal.debug.exit( _method );
			}
		  }
);

dojo.declare( "com.ibm.portal.aggregation.javascript.uneval.scrubbers.IBMJSFv3Scrubber",
		  com.ibm.portal.aggregation.javascript.uneval.scrubbers.WindowPropertyScrubber,
		  {
			// summary: Manages the JSFv3Collectors.
			newCollector: function () {
				// summary: Creates a new collector appropriate for this scrubber.
				// returns: a new instance of a Collector
				return new com.ibm.portal.aggregation.javascript.uneval.collectors.IBMJSFv3Collector();
			},
			clean: function ( /*Collector*/collector1, /*Collector*/collector2 ) {
				// summary: Given two collectors, clean the difference between the two.
				// collector1: the initial state to return to
				// collector2: the final state
				var _method = this.declaredClass + ".clean";
				ibm.portal.debug.entry( _method, [ collector1, collector2 ] );
				
				var compsToDelete = this._getDelta( collector1.getValues(), collector2.getValues() );
				
				//Call destroy on any widgets created in this restore point.
				for ( var i = 0; i < compsToDelete.length; i++ ) {
					ibm.portal.debug.text( "Deleting: " + compsToDelete[i] );

					var comp = hX_5.getComponentById( compsToDelete[i] )
					hX_5.removeComponent( comp );
				}
				
				ibm.portal.debug.exit( _method );
			}
		  }
);

com.ibm.portal.aggregation.javascript.uneval.RestorePoint.prototype.SCRUBBERS.push( new com.ibm.portal.aggregation.javascript.uneval.scrubbers.IBMJSFv3Scrubber() );
com.ibm.portal.aggregation.javascript.uneval.RestorePoint.prototype.SCRUBBERS.push( new com.ibm.portal.aggregation.javascript.uneval.scrubbers.DijitScrubber() );
com.ibm.portal.aggregation.javascript.uneval.RestorePoint.prototype.SCRUBBERS.push( new com.ibm.portal.aggregation.javascript.uneval.scrubbers.DojoScrubber() );
com.ibm.portal.aggregation.javascript.uneval.RestorePoint.prototype.SCRUBBERS.push( new com.ibm.portal.aggregation.javascript.uneval.scrubbers.IntervalScrubber() );
com.ibm.portal.aggregation.javascript.uneval.RestorePoint.prototype.SCRUBBERS.push( new com.ibm.portal.aggregation.javascript.uneval.scrubbers.WindowPropertyScrubber() );

}

if(!dojo._hasResource["com.ibm.portal.aggregation.javascript.uneval.common"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.javascript.uneval.common"] = true;
dojo.provide( "com.ibm.portal.aggregation.javascript.uneval.common");

dojo.require( "com.ibm.portal.aggregation.javascript.uneval" );
dojo.require( "com.ibm.portal.aggregation.javascript.uneval.collectors" );
dojo.require( "com.ibm.portal.aggregation.javascript.uneval.scrubbers" );

}

if(!dojo._hasResource["com.ibm.portal.aggregation.skin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.skin"] = true;
dojo.provide( "com.ibm.portal.aggregation.skin" );

dojo.require( "dojox.data.dom" );
dojo.require( "dojo.fx" );


dojo.require( "com.ibm.portal.xslt" );


dojo.declare( "com.ibm.portal.aggregation.PortletLoadingDecoration",
              null,
              {
                  // summary: Abstract representation of a loading decoration to show when a portlet is loading or
                  //        refreshing to indicate progress to the user.
                  // description: All loading decorations should inherit from this class and override the _doShow
                  //        and _doHide functions to implement the actual decoration behavior.
                  constructor: function ( portletid ) {
                      this.id = portletid;
                      this._resourceBase = dojo.moduleUrl("com", "ibm/portal/aggregation/");
                      this._isIE = dojo.isIE;
                  },
                  show: function (/*Object*/args) {
                      // summary: Shows the loading decoration.
                      // args: the arguments passed from the startFragment event
                      ibm.portal.debug.entry( "PortletLoadingDecoration.show");
                      
                      if ( this.id.indexOf( args["id"] ) >= 0 ) {
                          //Call the implementation function.
                          this._doShow();
                          
                          //Hide any select boxes in IE. Otherwise they will appear on top
                          //of any overlaid div.
                          if ( this._isIE ) {
                                this._hideSelectBoxes();
                          }
                      }    

                      ibm.portal.debug.exit( "PortletLoadingDecoration.show" );   
                  },
                  _doShow: function () {
                      // summary: Performs the actual display of the loading decoration.
                      dojo.unimplemented( "PortletLoadingDecoration._doShow" );
                  },
                  hide: function ( /*Object*/args) {
                      // summary: Hides the loading decoration.
                      // args: the arguments passed from the endFragment event
                      ibm.portal.debug.entry( "PortletLoadingDecoration.hide");
                      if ( this.id.indexOf( args["id"] ) >= 0 ) {
                          //Call the implementation function.
                          this._doHide();
                          
                          //Restore the select boxes.                      
                          if ( this._isIE ) {
                              this._showSelectBoxes();
                          }
                      }    
                      
                      ibm.portal.debug.exit( "PortletLoadingDecoration.hide" );
                  },
                   _doHide: function () {
                      // summary: Performs the hiding of the loading decoration.
                      dojo.unimplemented( "PortletLoadingDecoration._doHide" );
                  },
                  _getWpsPortletElement: function () {
                      // summary: Get the skin element with the wpsPortlet class.
                      return dojo.query( ".wpsPortlet", this._getRootPortletElement() )[0]; // HTMLElement  
                  },
                   _getWpsPortletBodyElement: function () {
                      // summary: Get the skin element with the wpsPortletBody class.
                      return dojo.query( ".wpsPortletBody", this._getRootPortletElement() )[0]; // HTMLElement  
                  },
                  _getRootPortletElement: function () {
                      // summary: Get the root element for the skin.
                      return dojo.byId( "portlet_" + this.id );    
                  },
                  _hideSelectBoxes: function () {
                      // summary: Hides all select boxes in the portlet/skin markup since IE will
                      //        render those on top of any overlaid elements.
                      var elem = this._getRootPortletElement();
                      var selectBoxes = elem.getElementsByTagName( "SELECT" );
                      
                      for ( var i = 0; i < selectBoxes.length; i++ ) {
                          selectBoxes[i]._hiddenByCsa = true;
                          selectBoxes[i].style.visibility = "hidden";
                      }
                  },
                  _showSelectBoxes: function () {
                      // summary: Restore all the hidden select boxes.
                      var elem = this._getRootPortletElement();
                      var selectBoxes = elem.getElementsByTagName( "SELECT" );
                      
                      for ( var i = 0; i < selectBoxes.length; i++ ) {
                          if ( selectBoxes[i]._hiddenByCsa ) {
                              selectBoxes[i].style.visibility = "visible";
                              selectBoxes[i]._hiddenByCsa = null;
                          }    
                      }    
                  }  
              }
);

dojo.declare( "com.ibm.portal.aggregation.Skin",
    null,
    {
        constructor: function( /*String*/resourceRoot, /*String*/skinId, /*String*/ xslt ) {
            this._resourceRoot = resourceRoot;
            this._skinId = skinId;
            this._xslt = xslt;
            this._fullPortletId = null;
            this._parentControl = null;
            this._atomEntryXml = null;
            this._loadingDecorator = null;
            this._resourceBaseURL = dojo.moduleUrl( "com", "ibm/portal/aggregation/");
            this.isRendered = false;
            this.isInstance = false;
            this._topics = new Array();
            
            this._startFragmentHandle = null;
            this._endFragmentHandle = null;
        },
        renderSkin: function(){
            ibm.portal.debug.entry("Skin.renderSkin");
            var trf = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer(this._xslt);
            if (trf == null) {
            ibm.portal.debug.text("XSL transformer is null");
            }
            var xmlDoc = this._atomEntryXml;
            if ( dojo.isIE ) {
                // make sure we have a valid XML document
                var xml = dojox.data.dom.innerXML(this._atomEntryXml);
                xmlDoc = dojox.data.dom.createDocument( xml );
            }
            var xsltParams = { "loadingImagePath": this._resourceBaseURL + "progress-anim.gif"};
            var node = dojo.byId( "portlet_"+this._fullPortletId );
            trf.transformToRegion(xmlDoc, xsltParams, node, document);
            this.isRendered = true;
            ibm.portal.debug.exit( "Skin.renderSkin" );
        },
        updateSkin: function ( pInfo ) {
            ibm.portal.debug.entry( "Skin.updateSkin", [ pInfo ] );
            ibm.portal.debug.exit( "Skin.updateSkin" );
        },
        getActiveDiv: function () {
            // summary: Returns the div which portlet markup updates should go in (for inline mode changes).                    
            ibm.portal.debug.entry( "getActiveDiv" );
            var result = dojo.byId( this._fullPortletId );
            ibm.portal.debug.exit( "getActiveDiv", result );
            return result;
        },
        instance: function (fullPortletId, parentControl, atomEntryXml){
            //summary: factory function to create an instance of this skin template that is tied to a specific portlet on a page.
            //TODO: add an isInstance flag and throw an error if the skin template is used directly
            var newInstance = new com.ibm.portal.aggregation.Skin(this._resourceRoot, this._skinId, this._xslt);
            newInstance._fullPortletId = fullPortletId;
            newInstance._parentControl = parentControl;
            newInstance._atomEntryXml = atomEntryXml;
            newInstance._isInstance = true;
            newInstance._loadingDecorator = new com.ibm.portal.aggregation.InitialPortletLoadingDecoration( fullPortletId );
            newInstance._startFragmentHandle = com.ibm.portal.EVENT_BROKER.startFragment.register( newInstance._loadingDecorator, "show" );
            newInstance._endFragmentHandle = com.ibm.portal.EVENT_BROKER.endFragment.register( newInstance._loadingDecorator, "hide" );
            return newInstance;
        },
        destroy: function () {
            //Even though the loading decorator may not be used, we want to leave this at this level for safety's sake.
            //We ALWAYS need any registered events unregistered otherwise we'll have unwanted references in the event topic code.
            if ( this._loadingDecorator != null ) {
                com.ibm.portal.EVENT_BROKER.startFragment.unregister( this._startFragmentHandle );
                com.ibm.portal.EVENT_BROKER.endFragment.unregister( this._endFragmentHandle );
            } 
            
            for(var i = 0; i < this._topics.length; i++) {
                dojo.unsubscribe(this._topics[i]);
            }
        },
        validate: function() {
            //summary: checks to see that all required properties are non-null and non-empty.
            result = true;
            if (this._resourceRoot == null || this._resourceRoot == "") result = false;
            if (this._skinId == null || this._skinId == "") result = false;
            if (this._xslt == null) result = false;
            return result;
        },
        hideLoadingIfNecessary: function () {
            // summary: Called to ensure that the loading decoration is hidden before the 
            //        portlet's markup is changed.
            if ( this._loadingDecorator != null ) {
                this._loadingDecorator.hide({id: this._fullPortletId});
            }
        }

}
);

dojo.declare( "com.ibm.portal.aggregation.OpaquePortletRefreshingDecoration",
              com.ibm.portal.aggregation.PortletLoadingDecoration,
              {
                  // summary: Overlays a transparent div with an animated gif and the 
                  //        word "Refreshing..." on top of the portlet.
                  _thread: null,
                  _doShow: function () {
                      ibm.portal.debug.entry( "PortletLoadingDecoration._doShow");
                      var elem = dojo.byId( "loading_" + this.id );
                        if( !elem ) { 
                            elem = document.createElement( "DIV" );
                            elem.style.backgroundColor = "#3c3c3c";
                            elem.style.color = "white";
                            elem.style.fontFamily = "ARIAL";
                            elem.style.fontSize = "13";
                            elem.style.fontWeight = "bold";
                            dojo.style(elem, "opacity", 0.85);
                            elem.style.zIndex = 100;
                            elem.style.position = "absolute";
                            elem.id = "loading_" + this.id;
                            var text = document.createElement( "DIV" );
                            text.style.position = "relative";
                            text.style.top = "50%";
                            text.style.left = "50%";
                            text.innerHTML = "<img src='" + this._resourceBase + "/loading-dk.gif' />&nbsp;" + dojo.i18n.getLocalization("com.ibm.portal", "v3theme", csaUserLocale)["loading"];
                            elem.appendChild( text );
                            document.body.appendChild( elem );
                            
                        } 
                        else { 
                            elem.style.display="block";
                        }
                        
                      //Since the size of the portlet could change at any point in time
                      //(browser window resize, contents of other portlets reflowing, etc),
                      //we need to constantly check our overlay to see if it needs to be
                      //resized (only while it's showing of course).
                      this.resize();
                      
                      if ( this._thread == null ) {
                          var me = this;
                          this._thread = setInterval( function() { me.resize(); }, 10 );
                      }
                                                 
                      ibm.portal.debug.exit( "PortletLoadingDecoration._doShow" );   
                  },
                  _doHide: function () {
                      ibm.portal.debug.entry( "PortletLoadingDecoration._doHide");
                      var elem = dojo.byId( "loading_" + this.id );
                      if ( elem ) {
	                      elem.style.display = "none";
	                      //Clear the resizing interval
	                      clearInterval( this._thread );
	                      this._thread = null;    
	                      ibm.portal.debug.exit( "PortletLoadingDecoration._doHide" );
                      }    
                  },
                  resize: function () {
                      // summary: Resizes the overlaid div so that it is 1px less than
                      //        the entire portlet.
                      ibm.portal.debug.entry( "PortletLoadingDecoration.resize" );
                      
                      var elem = dojo.byId( "loading_" + this.id );
                      ibm.portal.debug.text( elem.style.display );
                      //Failsafe
                      if ( elem.style.display == 'none' ) {
                          ibm.portal.debug.text( 'clearing timeout: ' + this._thread );
                          clearTimeout( this._thread );
                      }
                      else {
                          var refNode = this._getWpsPortletElement();
                          //The dojo.html.getAbsoluteX and getAbsoluteY functions appear to work slightly differently
                          //in FF and IE. In FF, we don't want the scroll and in IE we do.
                          // PK70437 - evidently Firefox fixed above behavior in a subsequent fixpack so
                          // make everything work like IE
                          // elem.style.top = (dojo.coords(refNode, dojo.isIE).y + 1) + "px";
                          // elem.style.left = (dojo.coords(refNode, dojo.isIE).x + 1)  + "px";
                          elem.style.top = (dojo.coords(refNode, true).y + 1) + "px";
                          elem.style.left = (dojo.coords(refNode, true).x + 1) + "px"
                          
                          var size = dojo.contentBox( refNode );
                          elem.style.height = (size.h - 2) + "px";
                          elem.style.width = (size.w - 2) + "px";
                      }    
                      ibm.portal.debug.exit( "PortletLoadingDecoration.resize" );
                  }  
              }
);

dojo.declare( "com.ibm.portal.aggregation.InitialPortletLoadingDecoration",
              com.ibm.portal.aggregation.PortletLoadingDecoration,
              {
                  // summary: Since the page reflows so significantly during the initial page 
                  //        loading, the overlaid div looks horrible constantly resizing. Instead,
                  //        we will use the simple loading message described here which does not 
                  //        have to be absolutely positioned, meaning it will flow properly with the
                  //        page changes.
                  _doShow: function () {
                      var elem = this._getWpsPortletBodyElement();
                      //We can't simply blow the existing markup away in case we are in the middle of a form
                      //submission or something similar. The _doShow function gets called when the portlet
                      //starts to update (before the actual request has even been sent). So if we blow away
                      //the existing markup, it will cause the form to disappear (except for the reference to 
                      //the actual form node) and the inputs will no longer exist (at least in IE).
                      dojo.forEach( elem.childNodes, function(item) { if ( item.style ) { item.style.display = 'none'; } } );
                      //Interestingly, using innerHTML+= here in IE blows away the markup that we just set display to none in the
                      //statement above. Using the W3C methods works correctly. Go figure.
                      //elem.innerHTML += "<span style='font-family: arial; font-weight: bold;'><img src='" + this._resourceBase + "/loading.gif' />&nbsp;Loading...</span>";
                      //Create the span to set our font styles.
                      var span = document.createElement( "SPAN" );
                      span.style.fontFamily = "arial";
                      span.style.fontWeight = "bold";
                      //Create the loading image.
                      var img = document.createElement( "IMG" );
                      img.src = this._resourceBase + "/loading.gif";
                      //...And the actual text.
                      var text = document.createTextNode(" " + dojo.i18n.getLocalization("com.ibm.portal", "v3theme", csaUserLocale)["loading"]);
                      span.appendChild( img );
                      span.appendChild( text );
                      elem.appendChild( span );
                  },
                  _doHide: function () {
                      //do nothing...
                  }        
              }
);

dojo.declare( "com.ibm.portal.aggregation.IbmSkinUIState",
              null,
              {
                // summary: Maintains the state of the UI for the IBM Skin.
                // description: The IBM Skin contains several complex UI areas and the state of one
                //      can effect the behavior of another. This object maintains the state of each of
                //      these complex UI areas so that this state is separated from the actual display
                //      properties of the UI. This allows non-synchronous operations to be used in transition
                //      effects for individual areas (the most obvious example is using an animation to 
                //      open or close the actions shelf). The complex areas that we want to maintain state
                //      for are the following:
                //          Portlet Actions Shelf: the area in the skin which shows the portlet modes and other actions (e.g. Delete)
                //                                 it can be expanded or collapsed.
                //          Portlet Inline Mode Container: the area which shows a portlet mode other than View inline above view mode
                //                                  visibility can be toggled when the user clicks on the active mode (e.g. Personalize)
                // EXPANDED: constant that indicates the EXPANDED state
                // COLLAPSED: constant that indicates the COLLAPSED state
                // VISIBLE: constant that indicates the VISIBLE state
                // HIDDEN: constant that indicates the HIDDEN state
                // PORTLET_ACTIONS_SHELF_STATE: constant for the state key for the portlet actions shelf state
                // INLINE_MODE_CONTAINER_STATE: constant for the state key for the inline mode container state
                //
                // NOTE: This state does NOT make any changes to the actual display/CSS. It merely maintains what should
                //       be the state of various elements so that dependent elements can adjust accordingly without having
                //       to wait for any animations to finish, etc. The skin controller object has the responsibility of 
                //       actually updating the display/CSS properties based on the UI state.
                //_properties: {},
                EXPANDED: 1,
                COLLAPSED: 0,
                VISIBLE: 1,
                HIDDEN: 0,
                //Key for the state of the Portlet Actions shelf (i.e. the shelf area that displays the different portlet modes)
                PORTLET_ACTIONS_SHELF_STATE: "ibmSkin.actionsShelf",
                //Key for thes state of the inline mode container (the inline mode display state can be toggled by clicking on the
                //active mode name in the portlet actions shelf.
                INLINE_MODE_CONTAINER_STATE: "ibmSkin.inlineModeContainer",
                constructor: function () {
                    this._properties = {};
                },
                setState: function ( /*String*/property, /*Object*/state ) {
                    // summary: Sets the state value for the given property.
                    // property: the property name or key
                    // state: the state value for the given property
                    ibm.portal.debug.entry( "IbmSkinUIState.setState", [ property, state ] );
                    this._properties[ property ] = state;
                    ibm.portal.debug.exit( "IbmSkinUIState.setState" );
                },
                getState: function ( /*String*/ property ) {
                    // summary: Retrieves the state for the given property.
                    // returns: the state for the given property
                    ibm.portal.debug.entry( "IbmSkinUIState.getState", [ property ] );
                    var retVal = ( this._properties[ property ] ? this._properties[ property ] : false );
                    ibm.portal.debug.exit( "IbmSkinUIState.getState", retVal );
                    return retVal;  // Object
                },
                setPortletActionsShelfState: function ( /*EXPANDED or COLLAPSED*/state ) {
                    // summary: Sets the state for the portlet actions shelf.
                    // state: use the EXPANDED or COLLAPSED constants on this IbmSkinUIState object
                    ibm.portal.debug.entry( "IbmSkinUIState.setPortletActionsShelfState", [ state ] );
                    this.setState( this.PORTLET_ACTIONS_SHELF_STATE, state );
                    ibm.portal.debug.exit( "IbmSkinUIState.setPortletActionsShelfState" );
                },
                setPortletActionsShelfExpanded: function () {
                    // summary: Sets the portlet actions shelf state to EXPANDED
                    ibm.portal.debug.entry( "IbmSkinUIState.setPortletActionsShelfExpanded" );
                    this.setPortletActionsShelfState( this.EXPANDED );
                    ibm.portal.debug.exit( "IbmSkinUIState.setPortletActionsShelfExpanded" );
                },
                setPortletActionsShelfCollapsed: function () {
                    // summary: Sets the portlet actions shelf state to COLLAPSED
                    ibm.portal.debug.entry( "IbmSkinUIState.setPortletActionsShelfCollapsed" );
                    this.setPortletActionsShelfState( this.COLLAPSED );
                    ibm.portal.debug.exit( "IbmSkinUIState.setPortletActionsShelfCollapsed" );
                },
                isPortletActionsShelfExpanded: function () {
                    // summary: Indicates if the portlet actions shelf is in the EXPANDED state
                    // returns: true if the shelf is expanded, false if it is collapsed
                    ibm.portal.debug.entry( "IbmSkinUIState.isPortletActionsShelfExpanded" );
                    var retVal = ( this.getState( this.PORTLET_ACTIONS_SHELF_STATE ) == this.EXPANDED );
                    ibm.portal.debug.exit( "IbmSkinUIState.isPortletActionsShelfExpanded", retVal );
                    return retVal;  // boolean
                },
                setInlineModeContainerState: function ( /*VISIBLE or HIDDEN*/state ) {
                    // summary: Sets the inline mode container's state to VISIBLE or HIDDEN
                    // state: use the VISIBLE or HIDDEN constants on this IbmSkinUIState object
                    ibm.portal.debug.entry( "IbmSkinUIState.setInlineModeContainerState", [ state ] );
                    this.setState( this.INLINE_MODE_CONTAINER_STATE, state );
                    ibm.portal.debug.exit( "IbmSkinUIState.setInlineModeContainerState" );
                },
                setInlineModeContainerVisible: function () {
                    // summary: Sets the inline mode container's state to VISIBLE
                    ibm.portal.debug.entry( "IbmSkinUIState.setInlineModeContainerVisible" );
                    this.setInlineModeContainerState( this.VISIBLE );
                    ibm.portal.debug.exit( "IbmSkinUIState.setInlineModeContainerVisible" );
                },
                setInlineModeContainerHidden: function () {
                    // summary: Sets the inline mode container's state to HIDDEN
                    ibm.portal.debug.entry( "IbmSkinUIState.setInlineModeContainerHidden" );
                    this.setInlineModeContainerState( this.HIDDEN );
                    ibm.portal.debug.exit( "IbmSkinUIState.setInlineModeContainerHidden" );
                },
                isInlineModeContainerVisible: function () {
                    // summery: Indicates if the inline mode container is in the VISIBLE state
                    // returns: true if the container is visible, false if it is hidden
                    ibm.portal.debug.entry( "IbmSkinUIState.isInlineModeContainerVisible" );
                    var retVal = ( this.getState( this.INLINE_MODE_CONTAINER_STATE ) == this.VISIBLE );
                    ibm.portal.debug.exit( "IbmSkinUIState.isInlineModeContainerVisible", retVal );
                    return retVal;
                }
              }
);

dojo.declare( "com.ibm.portal.aggregation.IbmSkin",
    com.ibm.portal.aggregation.Skin,
    {
        constructor: function( /*String*/resourceRoot, /*String*/skinId, /*String*/xslt ) {
            //NOTE: this constructor should not be called directly (except for during skin registration)! Instances of the IbmSkin object
            //      should be obtained using the IbmSkin.instance function after registration at bootstrap time.
            ibm.portal.debug.entry( "IbmSkin.constructor", arguments );
            
            //Call the parent class' constructor to initialize common properties
            com.ibm.portal.aggregation.IbmSkin.superclass.constructor( resourceRoot, skinId, xslt );
            //Array of supported directions the portlet can be moved in
            this.moveDirectionsArray = new Array();
            
            //The active DIV element. This is supplied to the contained portlet so that alternate modes
            //than VIEW can be rendered in separate locations.
            this._activeDiv = null;
            
            //State information about the portlet's mode and window state.
            this._previousWindowState = null;
            this._currentWindowState = null;
            this._currentMode=null;

            //Indicates whether the actions have been requested yet. They are only requested when the user
            //mouses over the portlet.
            this.actionsLoaded = false;
            //Indicates if this is the initial rendering of the skin.
            this._firstRendering = true;
            
            //NLS bundles            
            this._v3themeProperties = dojo.i18n.getLocalization("com.ibm.portal", "v3theme", csaUserLocale);
            this._titlebarProperties = dojo.i18n.getLocalization("com.ibm.portal", "titlebar", csaUserLocale);
            this._engineProperties = dojo.i18n.getLocalization("com.ibm.portal", "engine", csaUserLocale);
            
            //IDs of layout nodes that could be targeted during a move operation.
            this._leftMoveTargetId = null;
            this._rightMoveTargetId = null;
            this._upMoveTargetId = null;
            this._downMoveTargetId = null;
            
            //Event handles to disconnect when destroyed. These are initialized in the factory instance function
            //which constructs the IbmSkin object.
            this._startFragmentHandle = null;
            this._endFragmentHandle = null;
            
            //Initialize the UI state object.
            this._UIState = new com.ibm.portal.aggregation.IbmSkinUIState();
            this._UIState.setInlineModeContainerHidden();
            this._UIState.setPortletActionsShelfCollapsed();
            
            ibm.portal.debug.exit( "IbmSkin.constructor" );
        },
        instance: function (fullPortletId, parentControl, atomEntryXml){
            // summary: Factory function for constructing new instances of the IbmSkin object.
            ibm.portal.debug.entry( "IbmSkin.instance", [ fullPortletId, parentControl, atomEntryXml ] );
            
            var newInstance = new com.ibm.portal.aggregation.IbmSkin(this._resourceRoot, this._skinId, this._xslt);
            newInstance._fullPortletId = fullPortletId;
            newInstance._portletWindowId = com.ibm.portal.utilities.getControlId( fullPortletId );
            newInstance._parentControl = parentControl;
            newInstance._atomEntryXml = atomEntryXml;
            //The loading decorator object handles the UI display for the skin while new portlet markup is being retrieved.
            //Initially, we use a very simple decoration (see the InitialPortletLoadingDecoration object comments for more info).
            //After the skin is rendered the first time, the decorator is switched to use a more appealing decoration.
            newInstance._loadingDecorator = new com.ibm.portal.aggregation.InitialPortletLoadingDecoration( fullPortletId );
            //These events tell the skin when a request to update the contained portlet's markup is sent and when it is done
            //updating. We need to listen to these events so that we can display the loading decoration to the user while the
            //request is pending.
            newInstance._startFragmentHandle = com.ibm.portal.EVENT_BROKER.startFragment.register( function(args) {
                if(!newInstance._isHidden(newInstance._getPortletContainer())) {
                    newInstance._loadingDecorator.show(args);
                } 
            });
            newInstance._endFragmentHandle = com.ibm.portal.EVENT_BROKER.endFragment.register( function(args) {
                if(!newInstance._isHidden(newInstance._getPortletContainer())) {
                    newInstance._loadingDecorator.hide(args);
                } 
            });

            //Need to unsubscribe to these in destroy()
            if(!newInstance._topics) newInstance._topics = new Array();
            newInstance._topics.push(dojo.subscribe( 
                "/portal/Skin/maximized/"+newInstance._fullPortletId, 
                newInstance, 
                "_doMaximize"));
            newInstance._topics.push(dojo.subscribe( 
                "/portal/Skin/minimized/"+newInstance._fullPortletId, 
                newInstance, 
                "_doMinimize"));
            newInstance._topics.push(dojo.subscribe( 
                "/portal/Skin/normal/"+newInstance._fullPortletId, 
                newInstance, 
                "_doRestore"));
            newInstance._topics.push(dojo.subscribe(
                "/portal/Skin/ClearMenu/"+newInstance._fullPortletId,
                newInstance, 
                "clearMenuContents"));
            newInstance._topics.push(dojo.subscribe(
                "/portal/Skin/MoveMenu/"+newInstance._fullPortletId,
                newInstance, 
                "movePortlet"));
            newInstance._topics.push(dojo.subscribe(
                "/portal/Skin/RefreshSkins/",
                newInstance, 
                "refreshSkins"));

            ibm.portal.debug.exit( "IbmSkin.newInstance", newInstance );
            return newInstance;     // IbmSkin 
        },
        getActiveDiv: function () {
            // summary: Returns the div which portlet markup updates should go in from the current response.
            // description: The portlet control calls this function to retrieve the container which should end
            //      up with the portlet markup from the current portlet response. This allows the markup to be
            //      distributed based on the portlet mode, for example.
            ibm.portal.debug.entry( "IbmSkin.getActiveDiv" );
            
            var result = null;
            if( this._activeDiv != null ) {
                if( this._activeDiv == "clientModeHandling" ) {
                    ibm.portal.debug.text( "getActiveDiv: Client mode handling being used without update." );
                    return result;
                }
                else {
                    result= dojo.byId( this._activeDiv );
                }
            } else {
                result = dojo.byId( this._fullPortletId );
            }
            
            ibm.portal.debug.exit( "IbmSkin.getActiveDiv", result );
            return result;
        },
        _getTitlebarActionsContainer: function () {
            return dojo.byId( "portletActions_" + this._fullPortletId );    
        },
        _getPortletActionsShelf: function () {
            return dojo.byId( "portletModes_" + this._fullPortletId );  
        },
        _getModesContainer: function () {
            return dojo.byId( this._fullPortletId + "_mode");   
        },
        _getPortletContainer: function () {
            return dojo.byId( "portlet_"+this._fullPortletId);  
        },
        _getPreviousWindowState: function () {
            return this._previousWindowState;
        },
        _getWindowStatesArray: function () {
            return this._windowStatesArray;
        },
        _isHidden: function(node) {
            return dojo.style(node, 'display') == 'none';
        },
        _hide: function(node) {
            ibm.portal.debug.entry( "IbmSkin._hide", arguments );
            dojo.fx.wipeOut({node: node, duration: 5}).play();
            ibm.portal.debug.exit( "IbmSkin._hide" );
        },
        _show: function(node) {
            ibm.portal.debug.entry( "IbmSkin._show", arguments );
            dojo.fx.wipeIn({node: node, duration: 5}).play();
            ibm.portal.debug.exit( "IbmSkin._show" );
        },
        togglePortletModesDisplay: function(){
            ibm.portal.debug.entry( "IbmSkin.togglePortletModesDisplay" );
            var portletModesDiv = this._getModesContainer();
            
            if( this._UIState.isInlineModeContainerVisible() ){
                this._UIState.setInlineModeContainerHidden();
                this._hide( portletModesDiv );
            } else {
                this._UIState.setInlineModeContainerVisible();
                this._show( portletModesDiv );
            }
            ibm.portal.debug.exit( "IbmSkin.togglePortletModesDisplay", false );
            return false;
        },
        togglePortletModesShelfDisplay: function(){
            ibm.portal.debug.entry( "IbmSkin.togglePortletModesShelfDisplay" );
            
            var portletModesId = "portletModes_"+this._fullPortletId;
            var portletModesLink = dojo.byId("portletModesLink_"+this._fullPortletId);
            var portletModesDiv = this._getModesContainer();
            
            ibm.portal.debug.text( "UIState.isPortletActionsShelfExpanded? " + this._UIState.isPortletActionsShelfExpanded() );
            
            if( this._UIState.isPortletActionsShelfExpanded() ){
                portletModesLink.innerHTML = this._v3themeProperties["actions"];
                this._hide(portletModesId);
                this._UIState.setPortletActionsShelfCollapsed();
            } else {
                portletModesLink.innerHTML = this._v3themeProperties["closeActions"];
                this._show(portletModesId);
                this._UIState.setPortletActionsShelfExpanded();
            }
            
            ibm.portal.debug.exit( "IbmSkin.togglePortletModesShelfDisplay", false );
            return false;
        },
        
        // ************************** RENDERING FUNCTIONS *************************************
        updateSkin: function ( pInfo ) {
            // summary: Updates the skin based on the provided PortletInfo object (see PortletFragmentService.js).
            // description: This is the starting point for rendering the IbmSkin. This function is called by the 
            //      PortletControl object to update the skin according to the retrieved portlet navigational state.
            // pInfo: the PortletInfo object
            ibm.portal.debug.entry( "IbmSkin.updateSkin", [ pInfo ] );
            
            //Actions aren't loaded until the user mouses over the portlet.
            this.actionsLoaded = false;
            //Keep the current window state and current portlet mode around for comparisons.
            this._currentWindowState = pInfo.currentWindowState;
            this._currentMode = pInfo.currentMode;
            //Update the skin based on the portlet window state first (e.g. Maximize, Minimize, etc.)
            this.setSkinToCurrentWindowState(pInfo);
            //Select the active div based on the portlet mode AND give any client-side mode handling functions
            //a chance to grab control.
            this.selectActiveDiv(pInfo);
            //Create and attach the event handlers which hide and show the skin links when the user mouses over the skin.
            //Also create the keyboard event handlers which will allow the user to see the links when TABbing through the page.
            this.attachEventHandlers();
            //If this is the first rendering, we want to switch loading decorators and use one that is a little more attractive than
            //plain text + image. We don't want to use the more attractive decorator during the initial page rendering because the 
            //page contents re-flow so quickly, the resize function can't keep up.
            if ( this._firstRendering ) {
                //Clean up the trash from the initial loading decorator.
                com.ibm.portal.EVENT_BROKER.startFragment.unregister( this._startFragmentHandle );
                com.ibm.portal.EVENT_BROKER.endFragment.unregister( this._endFragmentHandle );
                this._loadingDecorator = new com.ibm.portal.aggregation.OpaquePortletRefreshingDecoration( this._fullPortletId );
                //Register the new loading decorator.
                var me = this;
                this._startFragmentHandle = com.ibm.portal.EVENT_BROKER.startFragment.register( function(args) {
                    if(!me._isHidden(me._getPortletContainer())) {
                        me._loadingDecorator.show(args);
                    } 
                });
                this._endFragmentHandle = com.ibm.portal.EVENT_BROKER.endFragment.register( function(args) {
                    if(!me._isHidden(me._getPortletContainer())) {
                        me._loadingDecorator.hide(args);
                    } 
                });
                //Make sure this doesn't get executed again while this skin is not destroyed.
                this._firstRendering = false;
            }
                        
            ibm.portal.debug.exit( "IbmSkin.updateSkin" );
        },
        setSkinToCurrentWindowState: function(/*PortletInfo*/pInfo){
            // summary: When rendering the portlet, if the current window state is not normal,
            //          then we call the appropriate function to display the portlet in the 
            //          correct state
            ibm.portal.debug.entry( "IbmSkin.setSkinToCurrentWindowState", [ pInfo ] );
            ibm.portal.debug.text("IbmSkin.setSkinToCurrentWindowState: currentWindowState= "+pInfo.currentWindowState);
            
            //The client side programming model allows a portlet to define a specific function which can control the 
            //transition from window state to window state. We must first find out if a portlet has defined such a function. 
            //If so, whether or not our skin specific functions are called are dependent on the return value of the portlet's
            //function.
            var isWindowStateOnClient = false;
            var update = true;
            //var clientWindowStateFn = "PC_"+this._portletWindowId+"_doWindowState";
            var clientWindowStateFn = "PC_"+pInfo.windowId+"_doWindowState";
            //Check to see if the client-side function is defined.
            if( typeof window[clientWindowStateFn] == "function" ){
                isWindowStateOnClient = true;
            }   
            //If the portlet's new window state is minimized, we want to check if the portlet is already minimized. If so, we 
            //just leave it alone. If the previous state was null, then this is likely the first time rendering this portlet. 
            if( pInfo.currentWindowState == "minimized" && (this._previousWindowState == null || dojo.byId(this._fullPortletId+"_minimized") == null) ){
                if ( isWindowStateOnClient ) {
                    update = window[clientWindowStateFn]( this._currentWindowState , this._getPortletContainer() );
                }
                //The client window state function has to return TRUE for our skin-specific function to be called.      
                if ( update ) {
                    this._doMinimize(null);
                }    
            }
            //If the portlet's new window state is maximized, we want to check if the portlet is already maximized. Again, if the 
            //previous state was null, then this is likely the first time rendering this portlet and it cannot be already maximized.
            else if( pInfo.currentWindowState == "maximized" && (this._previousWindowState == null || dojo.byId("maximized_portlet_placeholder"+this._fullPortletId) == null) ){ 
                if( isWindowStateOnClient ) {
                    update = window[clientWindowStateFn]( pInfo.currentWindowState , this._getPortletContainer() );
                }       
                //The client window state function has to return TRUE for our skin-specific function to be called.
                if( update ) {
                    this._doMaximize(null);
                }    
            }
            
            //In the following two cases, we are returning the portlet from maximized or minimized window state to NORMAL window state. 
            if(dojo.byId("maximized_portlet_placeholder"+this._fullPortletId) != null && this._previousWindowState == "maximized" && pInfo.currentWindowState == "normal" ){
                if( isWindowStateOnClient ) {
                    update = window[clientWindowStateFn]( pInfo.currentWindowState , this._getPortletContainer() );
                }       
                //The client window state function has to return TRUE for our skin-specific function to be called.
                if( update ) {
                    this.unMaximize();
                }               
            }
            if(dojo.byId(this._fullPortletId+"_minimized") != null && this._previousWindowState == "minimized" &&  pInfo.currentWindowState == "normal" ){
                if( isWindowStateOnClient ) {
                    update = window[clientWindowStateFn]( pInfo.currentWindowState , this._getPortletContainer() );
                }       
                if( update ) {
                    //The client window state function has to return TRUE for our skin-specific function to be called.
                    this.unMinimize();
                }               
            }
            
            this._previousWindowState = pInfo.currentWindowState;
            ibm.portal.debug.exit( "IbmSkin.setSkinToCurrentWindowState" );
        },
        selectActiveDiv: function( pInfo ){
            // summary: Selects the container which should receive the portlet markup from this portlet request.
            // pInfo: the object containing the results of the portlet request (see PortletFragmentService.js)
            ibm.portal.debug.entry( "IbmSkin.selectActiveDiv", [ pInfo ]);
            
            //The IbmSkin has two containers for portlet markup. One container is reserved for the VIEW mode markup.
            //The other container is reserved for any other mode's markup. 
            if( pInfo.currentMode == "view" ){
                ibm.portal.debug.text( "IbmSkin.selectActiveDiv: mode is view");                        
                //Hide the modes container because we are in view mode.
                if( this._UIState.isPortletActionsShelfExpanded() ){
                    this._UIState.setInlineModeContainerHidden();
                    this._hide( this._getModesContainer() );
                    this._UIState.setPortletActionsShelfCollapsed();
                    this._hide( this._getPortletActionsShelf() );
                }
                //Set the active div to the view container.
                this._activeDiv = dojo.byId( this._fullPortletId);
            } else {
                var update = true;
            
                //The client side programming model allows a portlet to define a specific function which can control the 
                //transition from portlet mode to portlet mode. We must first find out if a portlet has defined such a function. 
                //If so, whether or not our skin specific functions are called are dependent on the return value of the portlet's
                //function.
                if(typeof window["PC_"+pInfo.windowId+"_doPortletMode"] == "function" ){
                    update = window["PC_"+pInfo.windowId+"_doPortletMode"]( pInfo.currentMode , this._getModesContainer() );
                }   
                
                if(update){
                    //The portlet is NOT in VIEW mode, so if the portlet is also not minimized, then we need to show the portlet
                    //actions shelf and the portlet modes container so that the current mode's markup is visible.                       
                    if(pInfo.currentWindowState != "minimized"){
                        this._show( this._getPortletActionsShelf() );
                        this._UIState.setPortletActionsShelfExpanded();
                        this._show( this._getModesContainer() );
                        this._UIState.setInlineModeContainerVisible();
                    }
                    
                    //Set active div to the mode container.
                    this._activeDiv = dojo.byId( this._getModesContainer() );
                }else{
                    //This will require getActiveDiv to return null. Since mode handling is taken over by the client, we do not want
                    //to update the portlet modes container.
                    this._activeDiv = "clientModeHandling";
                }
                
                //The view mode is shown at the same time as any other mode so that updates made in Personalize or Configure mode, for
                //example, are shown immediately to the user.
                if ( dojo.byId( this._fullPortletId ).getAttribute( "loaded" ) != "true" ) {
                    this._parentControl.renderViewMode( pInfo, dojo.byId(this._fullPortletId) );
                }
                
                //Since the mode shelf will be visible, we can't wait for the user to mouseover to load the portlet actions shelf 
                //contents.
                this.loadPortletActions();                
            }
            
            ibm.portal.debug.exit( "IbmSkin.selectActiveDiv" );
        },
        attachEventHandlers: function () {
            // summary: Attach any required event handlers for the IbmSkin.
            ibm.portal.debug.entry( "IbmSkin.attachEventHandlers" );
            
            this._createActionsMouseOver();
            this._createActionsOnFocus();
            
            ibm.portal.debug.exit( "IbmSkin.attachEventHandlers" );
        },
        _createActionsOnFocus: function(){
            // summary: Creates a markup element just before the hidden titlebar actions so that a user tabbing through
            //      the HTML document will have a markup element that can be used to show the titlebar actions since they
            //      are hidden by default (and usually only shown via mouse events). This is required to make the show/hide
            //      behavior accessible.
            ibm.portal.debug.entry( "IbmSkin._createActionsOnFocus" );
            
            var me = this;
            var titleContainer = dojo.byId("portletTitle_"+me._fullPortletId);

            var skipPortletAnchor = com.ibm.portal.utilities.html.createAnchor( titleContainer.childNodes[0], 
                                                                                "#skipportlet"+me._fullPortletId );
                                                                                
            dojo.connect(skipPortletAnchor, "onfocus", function () { me.showPortletTitlebarActions(); });
            
            var skipPortletImage = com.ibm.portal.utilities.html.createImage( skipPortletAnchor, 
                                                                              ibm.portal.ThemeImages.clearPixelPath, 
                                                                              null, 
                                                                              this._engineProperties["link.skiptonextportlet"]);
            skipPortletImage.border = "0px none";
            
            ibm.portal.debug.exit( "IbmSkin._createActionsOnFocus" );                                               
        },          
        _createActionsMouseOver: function(){
            // summary: Create the onmouseover event for the portlet container that shows and hides the titlebar
            //      actions.
            ibm.portal.debug.entry( "IbmSkin._createActionsMouseOver" );
            
            var portlet = this._getPortletContainer();
            var me = this;
            
            if (dojo.isIE){
                dojo.connect(portlet, "onmouseenter", function () { me.showPortletTitlebarActions(); });
                dojo.connect(portlet, "onmouseleave", this, "hidePortletTitlebarActions");
            }else{
                dojo.connect(portlet, "onmouseover", function () { me.showPortletTitlebarActions(); });
                dojo.connect(portlet, "onmouseout", this, "hidePortletTitlebarActions");
            }
            
            ibm.portal.debug.exit( "IbmSkin._createActionsMouseOver" );
        },
        showPortletTitlebarActions: function(){
            // summary: Makes the portlet's titlebar actions visible and loads the allowed portlet modes and window states 
            //      from the operations REST service, if they haven't been previously loaded. The request to load the actions
            //      is asynchronous, so there may be some delay from the first mouseover event and the actions actually being
            //      displayed.
            ibm.portal.debug.entry( "IbmSkin.showPortletTitlebarActions" );
            if(this.actionsLoaded != true){
                this.loadPortletActions();
            }
            dojo.style("portletActions_"+this._fullPortletId, "visibility", "visible");
            ibm.portal.debug.exit( "IbmSkin.showPortletTitlebarActions" );
        },
        hidePortletTitlebarActions: function( ){
            // summary: Makes the portlet's titlebar actions invisible to the user.
            ibm.portal.debug.entry( "IbmSkin.hidePortletTitlebarActions" );
            dojo.style("portletActions_"+this._fullPortletId, "visibility", "hidden");
            ibm.portal.debug.exit( "IbmSkin.hidePortletTitlebarActions" );
        },
        loadPortletActions: function(){
            // summary: Retrieve the operations feed link from the ATOM document and request it's contents for this portlet.
            // description: The operations feed allows us to know which modes and window state the user is allowed to access.
            ibm.portal.debug.entry( "IbmSkin.loadPortletActions" );
            
            var portletTd = this._getPortletContainer();
            var uri = portletTd.getAttribute("menuLink");
            var url = new com.ibm.portal.services.ContentHandlerURL( /*String*/uri, /*int*/null, /*String*/null, /*String?*/null );
            var serviceReq = new com.ibm.portal.services.PortalRestServiceRequest( url );
            var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
            //Make sure we don't try to load these twice.
            this.actionsLoaded = true;
            
            //Send the request.
            serviceReq.read( dojo.hitch( this, "applyXslt" ) );
            
            ibm.portal.debug.exit( "IbmSkin.loadPortletActions" );
        },
        applyXslt: function ( /*String*/type, /*dojox.data.dom.Document*/xmlData, /*XMLHttpRequest*/xhr, /*Array*/callbackargs ) {
            // summary: Apply the XSLT to gather the portlet actions from the menu feed, 
            //          then populate the action array objects
            // type: the type of response
            // xmlData: the xml of the Feed
            // xhr: the XMLHttpRequest used to retrieve the feed
            // callbackargs: any arguments passed through to this function
            ibm.portal.debug.entry( "IbmSkin.applyXslt", [xmlData,xhr,callbackargs] );
            
            if ( xmlData instanceof Error ) {
                //Clear out all skin actions.
                var container = this._getTitlebarActionsContainer();
                dojox.data.dom.removeChildren( container );
            } else {
                var trf = this._xslt = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer( ibmPortalConfig["contextMenuXslt"] );
                if (trf == null) {
                    ibm.portal.debug.text("XSL transformer is null");
                }
                
                //The results of this XSLT will be javascript string. Eval turns it into an
                //object for us.
                var results = trf.transformToDocument(xmlData, {}, true);
                eval( results );
                //menuItems is defined by the XSL file
                this._renderActions( menuItems );
            }    
            
            ibm.portal.debug.exit( "IbmSkin.applyXslt" );    
        },
        _renderActions: function ( /*Array*/items ) {
            // summary: Renders the items in the menu items array.
            // description: Unfortunately, there's not a generic way to render these actions. A large switch statement
            //      provides the necessary logic currently. Adding an action to the portlet skin should be done in this
            //      function or one of the functions it calls.
            // items: the array of items to render
            ibm.portal.debug.entry( "IbmSkin._renderActions" );
            
            var me = this;
            
            var titlebarContainer = this._getTitlebarActionsContainer();
            dojox.data.dom.removeChildren( titlebarContainer );
            
            var shelfContainer = this._getPortletActionsShelf();
            dojox.data.dom.removeChildren( shelfContainer );
            
            //Index the menu items by label.
            var indexedItems = {}; 
            for ( var i = 0; i < items.length; i++ ) {
                ibm.portal.debug.text( "Indexing label \"" + items[i].label + "\"." );
                indexedItems[ items[i].label ] = items[i];
            }
            
            //Render the window states menu.
            this._renderWindowStatesMenuButton( titlebarContainer, [ indexedItems[ "maximized" ], indexedItems["minimized"], indexedItems["normal"] ] );
            
            //Render the move menu only if the portlet is NOT maximized.
            if(dojo.byId("maximized_portlet_placeholder"+this._fullPortletId) == null){
                this._renderMoveMenuButton( titlebarContainer );
            }
            
            //Render refresh icon. This will refresh the portlet using it's last known navigational state.
            this._renderRefreshIcon( titlebarContainer );       
            
            //Render the help icon.
            this._renderHelpIcon( titlebarContainer, indexedItems[ "help" ] );
            
            //Render Actions Link.
            var showActionsLink = dojo.some(items, function(item, index, array) {
                if(item.label != "view"
                    && item.label != "normal" 
                    && item.label != "minimized"
                    && item.label != "maximized"
                    && item.label != "help") {
                    return true;
                }
                return false;
            });
            if(showActionsLink) this._renderActionsToggleLink( titlebarContainer );
            
            //Render portlet actions shelf (includes the portlet modes).
            var deletePermission = ( typeof ( indexedItems[ "delete_portlet" ] ) != "undefined" );
            this._renderPortletActionsShelf( shelfContainer, [ indexedItems[ "config" ], indexedItems[ "edit" ], indexedItems[ "edit_defaults" ] ], deletePermission ); 
            
            ibm.portal.debug.exit( "IbmSkin._renderActions" );
        },
        _renderRefreshIcon: function ( /*HTMLElement*/container ) {
            // summary: Renders the refresh icon and attaches it's event handlers.
            // container: the element which should contain the newly created refresh icon
            ibm.portal.debug.entry( "IbmSkin._renderRefreshIcon", [ container ] );
            
            var refreshAnchor = com.ibm.portal.utilities.html.createAnchor( container, "javascript:void(0);" );
            var refreshImage = com.ibm.portal.utilities.html.createImage( 
                refreshAnchor,  
                ibm.portal.ThemeImages.skinRefreshIconPath,
                "refresh_" + this._fullPortletId,
                this._titlebarProperties["refresh"],
                "wpsPortletIcons" ); 
                
            dojo.connect( refreshAnchor, "onclick", dojo.hitch(this,"refreshPortlet") );
            //The icon colors in when the mouse is over top of it.
            var me = this;
            dojo.connect( refreshImage, "onmouseover", function () { dojo.byId( "refresh_" + me._fullPortletId ).src=ibm.portal.ThemeImages.skinRefreshIconHoverPath; } );
            dojo.connect( refreshImage, "onmouseout", function () { dojo.byId( "refresh_" + me._fullPortletId ).src=ibm.portal.ThemeImages.skinRefreshIconPath; } );
            
            ibm.portal.debug.exit( "IbmSkin._renderRefreshIcon" );
        },
        _renderHelpIcon: function ( /*HTMLElement*/container, /*Object*/helpInfo ) {
            // summary: Renders the help icon and attaches it's event handlers.
            // container: the element which should contain the newly created help icon
            // helpInfo: the menu item object containing the info about the help mode, may be NULL to indicate
            //      there is no help mode for this portlet
            ibm.portal.debug.entry( "IbmSkin._renderHelpIcon", [ container, helpInfo ] );
            
            if( helpInfo != null ){
                var me = this;
                var helpAnchor = com.ibm.portal.utilities.html.createImageAnchor( 
                    container,  
                    ibm.portal.ThemeImages.skinHelpIconPath,
                    "help_" + this._fullPortletId,
                    this._titlebarProperties["help"],
                    "wpsPortletIcons" );
                    
                dojo.connect( helpAnchor, "onclick", function (){ me["_createModeFunction"](helpInfo.label, "Help"); }); //me.helpObject.oncommand,
                //The icon colors in when the mouse is over top of it.
                dojo.connect( helpAnchor, "onmouseover", function () { dojo.byId( "help_" + me._fullPortletId ).src=ibm.portal.ThemeImages.skinHelpIconHoverPath; } );
                dojo.connect( helpAnchor, "onmouseout", function () { dojo.byId( "help_" + me._fullPortletId ).src=ibm.portal.ThemeImages.skinHelpIconPath; } );
            }
            
            ibm.portal.debug.exit( "IbmSkin._renderHelpIcon" );
        },
        _renderActionsToggleLink: function ( /*HTMLElement*/container ) {
            // summary: Renders the Actions/Close Actions toggle link and attaches it's event handlers.
            // container: the element which should contain the newly create link
            ibm.portal.debug.entry( "IbmSkin._renderActionsToggleLink", [ container ] );
            
            var actionsText = null;
            
            //The text of the anchor depends upon whether the portlet actions shelf is showing or not.
            if( this._UIState.isPortletActionsShelfExpanded() ){
                actionsText = this._v3themeProperties["closeActions"];
            } else {
                actionsText = this._v3themeProperties["actions"];
            }
            
            ibm.portal.debug.text( "Actions text is: " + actionsText );
             
            var actionsAnchor = com.ibm.portal.utilities.html.createAnchor( 
                container, 
                "javascript:void(0);",
                "portletModesLink_"+this._fullPortletId,
                actionsText,
                "wpsPortletActions");
            //The background of the anchor link changes when the mouse is over top of it.
            var me = this;
            dojo.connect( actionsAnchor, "onmouseover", function () { dojo.byId( "portletModesLink_"+me._fullPortletId ).className = 'wpsPortletActionsHover' });
            dojo.connect( actionsAnchor, "onmouseout", function () { dojo.byId( "portletModesLink_"+me._fullPortletId ).className = 'wpsPortletActions' });
            
            var isMinimized = ( this._currentWindowState == 'minimized' );
            
            //If the current portlet mode is VIEW or the portlet is minimized, the Actions/Close Actions link acts
            //simply as a toggle. Otherwise, it acts as the "Back To View" link since the user is essentially
            //returning to VIEW mode by closing the inline mode element.
            if ( ( this._currentMode == "view" ) || isMinimized ) {            
                dojo.connect( actionsAnchor, "onclick", dojo.hitch( this, "togglePortletModesShelfDisplay" ) ); 
            }                                                                                   
            else {
                dojo.connect( actionsAnchor, "onclick", function () { me["_createModeFunction"]("view", "Back"); }) 
            }
            
            ibm.portal.debug.exit( "IbmSkin._renderActionsToggleLink" );
        },
        _renderPortletActionsShelf: function ( /*HTMLElement*/container, /*Array*/pModes, /*boolean*/isDeletable ) {
            // summary: Renders the portlet actions shelf.
            // description: This is the area that displays the links for the different portlet modes, as well as
            //      the Delete portlet link, if the user has permission to delete the portlet.
            // container: the element which will contain each of these links
            // pModes: the portlet modes this portlet supports
            // isDeletable: indicates if the portlet is deletable by the current user
            ibm.portal.debug.entry( "IbmSkin._renderPortletActionsShelf", [ container, pModes, isDeletable ] );
            
            //If this is a Right-to-Left language (Hebrew, Arabic), we want to render the Delete link first.
            /*
            if( bidiSupport.isRTL && isDeletable ){
                this._createDeletePortletLink( container );
            }
            */
            
            var isMinimized = ( this._currentWindowState == 'minimized' );

            //If the portlet is minimized, we don't render any of the mode links.                   
            if( !isMinimized ){                         
            
                //Cycle through the possible modes and create anchor links for each one 
                //for bidi go from pModes.length to 0   
                //  for ( var i = pModes.length-1; i >= 0; i-- ) 
                //for non bidi go from 0 to pModes.length               
                //  for ( var i = 0; i < pModes.length; i++ ) 
                //instead of using two for loops, 
                //this logic was contained in one while loop
                var modesEnd = false;
                var i = 0;
                /*
                if( bidiSupport.isRTL ) {
                    i = pModes.length - 1;
                }
                */
                            
                while(pModes != null && !modesEnd){
                    if ( pModes[i] != null ) {
                        var me = this;
                        if ( this._currentMode == pModes[i].label ) {
                            //Render the active mode link differently than the other modes. 
                            //Since it is our active mode, this will not be in our array.                        
                            this._createActiveModeAnchor(container, this._currentMode, pModes[i].title);
                        }
                        else {
                            ibm.portal.debug.text( "Creating mode link for: " + pModes[i].title );
                            
                            var spanElement = document.createElement('SPAN');
                            spanElement.className= "wpsPortletModesSpan" ;
        
                            var anchor = com.ibm.portal.utilities.html.createAnchor( spanElement, 
                                                                                        "javascript:void(0);",
                                                                                        null,
                                                                                        pModes[i].title,
                                                                                        "wpsPortletModes" );
                            //Check for client mode handling.
                            var isModeOnClient = false;
                            var clientModeFn = "PC_"+this._portletWindowId+"_doPortletMode";
                            if( typeof window[clientModeFn] == "function" ){
                                isModeOnClient = true;
                            }   
                                 
                            //To maintain the variable that contains the mode url throughout the loop, 
                            //we need to use unique variables for each mode                 
                            if(pModes[i]["label"] == "config"){             
                                var configMode = pModes[i]["label"];
                                if(isModeOnClient){
                                    dojo.connect( anchor, "onclick",function () { me["_createClientModeFunction"](  configMode, clientModeFn ) }  );    
                                }else{                                          
                                    dojo.connect( anchor, "onclick", function () { me["_createModeFunction"](  configMode , "Mode" ) } );   
                                }
                            }
                                            
                            if(pModes[i]["label"] == "edit"){               
                                var editMode = pModes[i]["label"];  
                                if(isModeOnClient){
                                    dojo.connect( anchor, "onclick", function () { me["_createClientModeFunction"](  editMode, clientModeFn ) } );                              
                                }else{                  
                                    dojo.connect( anchor, "onclick", function () { me["_createModeFunction"](  editMode , "Mode" ) } ); 
                                }
                            }
                                            
                            if(pModes[i]["label"] == "edit_defaults"){              
                                var editDefaultsMode = pModes[i]["label"];  
                                if(isModeOnClient){
                                    dojo.connect( anchor, "onclick", function () { me["_createClientModeFunction"](  editDefaultsMode, clientModeFn ) } );                              
                                }else{              
                                    dojo.connect( anchor, "onclick", function () { me["_createModeFunction"](  editDefaultsMode , "Mode" ) } ); 
                                }
                            }
                            container.appendChild(spanElement); 
                        }                                                               
                    }
                                
                    modesEnd = true;

                    /*
                    if(bidiSupport.isRTL){
                        i--;
                        if(i>=0){
                            modesEnd = false;
                        }
                    }else*/{
                        i++;
                        if(i<pModes.length){
                            modesEnd = false;
                        }
                    }
                }
            }
            
            //If this is a Left-to-Right language (English, etc.), we want the Delete link last.
            //Note the Delete link is the only actions shelf link rendered when the portlet is minimized.
            if( /*!bidiSupport.isRTL &&*/ isDeletable ) {
                this._createDeletePortletLink( container );
            }
        },
        _renderWindowStatesMenuButton: function ( /*HTMLElement*/parent, /*Array*/wStates ) {
            // summary: will create the window state context menu icon and initialize it's contents
            // parent: the container where the window states menu is rendered
            // wStates: the window states supported by the portlet
            ibm.portal.debug.entry( "IbmSkin._renderWindowStatesMenuButton", [ parent ] );
            
            //Create the button element. We use a button so that the context menu will appear
            //directly below the image (limitation of the WCL context menu code).
            var wStatesButton = com.ibm.portal.utilities.html.createButton( 
                parent,  
                "javascript:void(0);",
                "menu_windowStateMenu" + this._fullPortletId,
                null,
                "wptheme-contextMenuIcon" );
            
            //Create the image to go on the button.
            var wStatesIcon = com.ibm.portal.utilities.html.createImage( 
                wStatesButton, 
                ibm.portal.ThemeImages.skinWindowStateIconPath, 
                "menu_windowStateMenu"+this._fullPortletId+"_img", 
                this._v3themeProperties["windowState"],
                "wpsPortletIcons" );
            
            //Connect all events.
            var me = this;
            dojo.connect( wStatesButton, "onclick", function(){ me["displayPortletMenu"]("windowStateMenu"); });
            dojo.connect( wStatesButton, "onmouseover", function () { document.getElementById("menu_windowStateMenu"+me._fullPortletId+"_img").src=ibm.portal.ThemeImages.skinWindowStateIconHoverPath; });
            dojo.connect( wStatesButton, "onmouseout", function () { document.getElementById("menu_windowStateMenu"+me._fullPortletId+"_img").src=ibm.portal.ThemeImages.skinWindowStateIconPath; });
            
            //Clear the menu.
            this.clearMenuContents("windowStateMenu");
            //Load the menu items.
            this._createWindowStatesMenu( wStates );
            
            ibm.portal.debug.exit( "IbmSkin._renderWindowStatesMenuButton");
        },
        _createWindowStatesMenu: function( /*Array*/contents ){
            ibm.portal.debug.entry( "IbmSkin._createWindowStatesMenu", [ contents ] );
            
            var menuId = "contextMenu_"+"windowStateMenu"+this._fullPortletId;
            /* PK73199 */
            var menu = createContextMenu(menuId, !bidiSupport.isRTL, ""/*menuBorderStyle*/, "wptheme-skinContextMenuTable"/*menuTableStyle*/, this._engineProperties["info.emptymenu"] );
            var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                        
            // Cycle through the menu item array.
            // The structure of the array object should be {oncommand: portletWindowId, label: modeName].
            /* PK73199 */
            for ( var i=0; i < contents.length; i++ ) {
                if ( contents[i] && contents[i].length != 0 && ( contents[i].label != this._currentWindowState ) ) {
                    menu.add( new UilMenuItem( contents[i].title, true, '',  'dojo.publish("/portal/Skin/'+contents[i].label+'/'+this._fullPortletId+'",[{url: "'+state.newPortletRenderURL(this._portletWindowId, null, contents[i].label.toString())+'"}]);', null, /*no icon*/ null, null, 'wptheme-skinContextMenuItem'/*menuItemStyle*/, 'wptheme-skinContextMenuItemHover'/*menuItemSelectedStyle*/) );
                }
            }
            
            ibm.portal.debug.exit( "IbmSkin._createWindowStatesMenu" ); 
        },  
        _renderMoveMenuButton: function ( /*HTMLElement*/parent ) {
            // summary: will create the move context menu icon
            // parent: the container where the move menu is rendered
            ibm.portal.debug.entry( "IbmSkin._renderMoveMenuButton", [ parent ] );
            
            var moveDirectionsArray = this._getValidMoves();
            if ( moveDirectionsArray.length != 0 ) {
                //Create the button element. We use a button so that the context menu will appear
                //directly below the image (limitation of the WCL context menu code).
                var moveButton = com.ibm.portal.utilities.html.createButton( 
                    parent,  
                    "javascript:void(0);",
                    "menu_moveMenu" + this._fullPortletId,
                    null,
                    "wptheme-contextMenuIcon" );
                
                //Create the image to go on the button.
                var moveIcon = com.ibm.portal.utilities.html.createImage( 
                    moveButton, 
                    ibm.portal.ThemeImages.skinMoveIconPath, 
                    "menu_moveMenu"+this._fullPortletId+"_img", 
                    this._v3themeProperties["movePortlet"],
                    "wpsPortletIcons" );
                
                //Connect all events.
                var me = this;
                dojo.connect( moveButton, "onclick", function(){ me["displayPortletMenu"]("moveMenu"); });                  
                dojo.connect( moveButton, "onmouseover", function () { document.getElementById("menu_moveMenu"+me._fullPortletId+"_img").src=ibm.portal.ThemeImages.skinMoveIconHoverPath; });
                dojo.connect( moveButton, "onmouseout", function () { document.getElementById("menu_moveMenu"+me._fullPortletId+"_img").src=ibm.portal.ThemeImages.skinMoveIconPath; });
                
                //Clear the menu.
                this.clearMenuContents("moveMenu");
                //Load the menu items.
                this._createPortletMoveMenu( moveDirectionsArray );
            }
            
            ibm.portal.debug.exit( "IbmSkin._renderMoveMenuButton");
        },
        _getValidMoves: function (){
            // summary: Determines the valid move locations for the current portlet using the
            //      portlet move controller.
            // returns: an array of objects describing the valid moves for this portlet            
            ibm.portal.debug.entry( "IbmSkin._getValidMoves" );
            
            var moveDirectionsArray = new Array();
            var i = 0;               
            if(com.ibm.portal.aggregation.PORTAL_AGGREGATOR.page.portletMoveController._isValidLeftMove("dndSrc"+this._fullPortletId) != null){
                this._leftMoveTargetId = com.ibm.portal.aggregation.PORTAL_AGGREGATOR.page.portletMoveController._isValidLeftMove("dndSrc"+this._fullPortletId);
                moveDirectionsArray[i] = new Object();
                moveDirectionsArray[i].title = this._titlebarProperties["move.left"];
                moveDirectionsArray[i].direction = "left";
                i++
                ibm.portal.debug.text( "this._leftMoveTargetId :"+this._leftMoveTargetId );
            }
            
            if(com.ibm.portal.aggregation.PORTAL_AGGREGATOR.page.portletMoveController._isValidRightMove("dndSrc"+this._fullPortletId) != null){
                this._rightMoveTargetId = com.ibm.portal.aggregation.PORTAL_AGGREGATOR.page.portletMoveController._isValidRightMove("dndSrc"+this._fullPortletId);
                moveDirectionsArray[i] = new Object();
                moveDirectionsArray[i].title = this._titlebarProperties["move.right"];
                moveDirectionsArray[i].direction = "right";
                i++;
                ibm.portal.debug.text( "this._rightMoveTargetId :"+this._rightMoveTargetId );
            }
            
            if(com.ibm.portal.aggregation.PORTAL_AGGREGATOR.page.portletMoveController._isValidUpMove("dndSrc"+this._fullPortletId) != null){
                this._upMoveTargetId = com.ibm.portal.aggregation.PORTAL_AGGREGATOR.page.portletMoveController._isValidUpMove("dndSrc"+this._fullPortletId);
                moveDirectionsArray[i] = new Object();
                moveDirectionsArray[i].title = this._titlebarProperties["move.up"];
                moveDirectionsArray[i].direction = "up";
                i++;
                ibm.portal.debug.text( "this._upMoveTargetId :"+this._upMoveTargetId );
            }
            
            if(com.ibm.portal.aggregation.PORTAL_AGGREGATOR.page.portletMoveController._isValidDownMove("dndSrc"+this._fullPortletId) != null){
                this._downMoveTargetId = com.ibm.portal.aggregation.PORTAL_AGGREGATOR.page.portletMoveController._isValidDownMove("dndSrc"+this._fullPortletId);
                moveDirectionsArray[i] = new Object();
                moveDirectionsArray[i].title = this._titlebarProperties["move.down"];
                moveDirectionsArray[i].direction = "down";
                ibm.portal.debug.text( "this._downMoveTargetId :"+this._downMoveTargetId );
            }
            
            ibm.portal.debug.exit( "IbmSkin._getValidMoves", moveDirectionsArray );
            return moveDirectionsArray;     // Array
        },
        _createPortletMoveMenu: function( /*Array*/ moveMenuItemsArray ) {
            // summary: Creates and populatese the portlet move menu.
            // moveMenuItemsArray: the array of menu items
            ibm.portal.debug.entry( "IbmSkin._createPortletMoveMenu" );
            
            var menuId = "contextMenu_"+"moveMenu"+this._fullPortletId;
            menu = createContextMenu(menuId, !bidiSupport.isRTL, ""/*menuBorderStyle*/, "wptheme-skinContextMenuTable"/*menuTableStyle*/, this._engineProperties["info.emptymenu"] );
            // Cycle through the move menu item array.
            for ( i=0; i < moveMenuItemsArray.length; i+=1 ) {
                if ( moveMenuItemsArray[i] && moveMenuItemsArray[i].length != 0 ){
                    menu.add( new UilMenuItem( moveMenuItemsArray[i].title, true, '',  'dojo.publish("/portal/Skin/MoveMenu/'+this._fullPortletId+'",[{direction: "'+moveMenuItemsArray[i].direction+'"}]);', null, null/*icon*/, null, 'wptheme-skinContextMenuItem'/*menuItemStyle*/, 'wptheme-skinContextMenuItemHover'/*menuItemSelectedStyle*/) );
                }
            }
            
            ibm.portal.debug.exit( "IbmSkin._createPortletMoveMenu" ); 
        },
        _createDeletePortletLink: function( container ){
            // summary: Creates the delete portlet link and attaches it's event handlers.
            // container: the element which will contain the portlet link
            ibm.portal.debug.entry( "IbmSkin._createDeletePortletLink", [ container ] );
            
            //Render the delete portlet link. This is the lone link rendered on the portletModesElem if the portlet is minimized.
            var spanDeleteElement = document.createElement('SPAN');
            spanDeleteElement.className= "wpsPortletModesSpan";
            var titlebarProperties = dojo.i18n.getLocalization("com.ibm.portal", "titlebar", csaUserLocale);
            var deleteAnchor = com.ibm.portal.utilities.html.createAnchor( spanDeleteElement,
                                                                          "javascript:void(0);",
                                                                          "portletModesLink_" + this._id,
                                                                          titlebarProperties["delete"],
                                                                          "wpsPortletModes" );
                              
            dojo.connect( deleteAnchor, "onclick", dojo.hitch(this,"deletePortlet") );
            container.appendChild(spanDeleteElement);
           
           ibm.portal.debug.exit( "IbmSkin._createDeletePortletLink" );
        },
        _createActiveModeAnchor: function( container, mode, title ) {
            // summary: Creates the currently active mode anchor and attaches it's event handlers.
            // description: The currently active mode anchor is different from the other modes in that
            //      toggles the display of the active mode (for modes other than VIEW only).
            // container: the element which should contain the link
            // mode: the active mode name
            // title: the localized string for the active mode
            ibm.portal.debug.entry( "IbmSkin._createActiveModeAnchor", [ container, mode, title ] );
            
            var me = this;
            var spanActiveElement = document.createElement('SPAN');
            spanActiveElement.className= "wpsPortletModesSpan" ;

            var activeModeAnchor = com.ibm.portal.utilities.html.createAnchor( 
                spanActiveElement, 
                "javascript:void(0);",
                "activeModeLink_" + me._fullPortletId, 
                title,
                "wpsActiveModeLink");
                   
            dojo.connect( activeModeAnchor , "onmouseover", function () { dojo.byId( "activeModeLink_" + me._fullPortletId ).className='wpsActiveModeLinkHover'; } );
            dojo.connect( activeModeAnchor , "onmouseout", function () { dojo.byId( "activeModeLink_" + me._fullPortletId ).className='wpsActiveModeLink'; } );
            dojo.connect( activeModeAnchor , "onclick", dojo.hitch(me,"togglePortletModesDisplay") );
            container.appendChild(spanActiveElement);
            
            ibm.portal.debug.exit( "IbmSkin._createActiveModeAnchor" );
        },
        
        // ************************** PORTLET MODE SWITCHING FUNCTIONS *************************************
        _createModeFunction: function ( label, type) {
            //summary: create and return a function to load a portlet mode
            // label: actual mode name to be created, values must match mode constants as specified by state apis
            // type: skin method call suffix. i.e. "Help" will cause function _doHelp to be returned.
            ibm.portal.debug.entry("IbmSkin._createModeFunction",[label, type]);
            
            var portletWindowState = null;
            
            if ( label == "help" && this._currentWindowState == ibm.portal.portlet.WindowState.MINIMIZED ) {
                portletWindowState = ibm.portal.portlet.WindowState.NORMAL;
            }
            
            var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
            var url = state.newPortletRenderURL(/*portletWindowId*/this._portletWindowId, /*portletMode*/label, /*windowstate*/portletWindowState );
            
            ibm.portal.debug.exit( "IbmSkin._createModeFunction" );
            return this["_do"+type]( url );
        },
        _createClientModeFunction: function ( label, clientModeFn ) {
            //summary: Use mode function on client and check for markup update
            // label: actual mode name to be created, values must match mode constants as specified by state apis
            // clientModeFn: the function on the client to be called
            ibm.portal.debug.entry("IbmSkin._createClientModeFunction",[label, clientModeFn]);
            
            var updateMarkup = window[clientModeFn](label, this._getModesContainer() );
            
            if(updateMarkup){
                var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                var url = state.newPortletRenderURL(this._portletWindowId, label);
                this._doMode(url);
            }
            
            ibm.portal.debug.exit( "IbmSkin._createClientModeFunction" );
        },
        _doMode: function ( url ) {
            // summary: Change to a particular mode.
            // url: the portlet url for switching to another portlet mode (Any except VIEW and HELP)
            ibm.portal.debug.entry( "IbmSkin._doMode", [ url ] );
            
            this.actionsLoaded = false;
            this._activeDiv = this._getModesContainer();
            
            var serviceReq = new com.ibm.portal.services.PortletFragmentService();
            serviceReq.getPortletInfo( this._fullPortletId, url );
            
            this._show(this._getModesContainer());
            this._show(this._getPortletActionsShelf().id);
            
            ibm.portal.debug.exit( "IbmSkin._doMode" );
        },
        _doBack: function ( url ) {
            // summary: Return to VIEW mode.
            // url: the url to return to VIEW mode
            ibm.portal.debug.entry( "IbmSkin._doBack", [ url ] );
            
            this._activeDiv = this._getModesContainer();
            this.actionsLoaded = false;
            
            var serviceReq = new com.ibm.portal.services.PortletFragmentService();
            serviceReq.getPortletInfo( this._fullPortletId, url );
            
            this._UIState.setInlineModeContainerHidden();
            this._hide(this._getModesContainer());
            this._UIState.setPortletActionsShelfCollapsed();
            this._hide(this._getPortletActionsShelf());
            
            dojox.data.dom.removeChildren( this._getModesContainer() );
            ibm.portal.debug.exit( "IbmSkin._doBack" );
        },
        _doHelp: function ( url ) {
            // summary: Changes to help mode.
            // description: Help mode is unique in this skin because it renders the help in a new browser
            //      window. To accomplish that, we have to intercept the response (make a stateless portlet request
            //      - see PortletFragmentService.js) and simply write the markup to the new window.
            // url: the url for help mode
            ibm.portal.debug.entry( "IbmSkin._doHelp", [ url ] );
            
            var stateManager = new com.ibm.portal.state.StateManager( ibmPortalConfig.contentHandlerURI );
            stateManager.reset( dojox.data.dom.createDocument() );
            
            var selectionAcc = stateManager.newSelectionAccessor();
            var selectionId = com.ibm.portal.utilities.getSelectionNodeId( this._fullPortletId );
            selectionAcc.setPageSelection( selectionId );
            selectionAcc.getParameters().setValue( "renderMode", "force_ssa" );
            
            var portletAcc = stateManager.newPortletAccessor( this._portletWindowId );
            portletAcc.setPortletMode( ibm.portal.portlet.PortletMode.HELP );
            
            var templateAcc = stateManager.newThemeTemplateAccessor();
            templateAcc.setThemeTemplate( "Plain" );
            
            var s11nManager = stateManager.getSerializationManager();
            s11nManager.serialize( stateManager.getState(), false, function ( status, state, serializedURL ) { 
            	window.open(serializedURL, 'helpWindow','resizable=yes,scrollbars=yes,menubar=no,toolbar=yes,status=no,width=800px,height=600px,screenX=10px,screenY=10px,top=10px,left=10px');
            });
            
            //var service = new com.ibm.portal.services.PortletFragmentService();
            //var me = this;
            //var win = window.open(url + "&renderMode=force_ssa", 'helpWindow','resizable=yes,scrollbars=yes,menubar=no,toolbar=yes,status=no,width=800px,height=600px,screenX=10px,screenY=10px,top=10px,left=10px');                              
                                                                        
            /*service.getPortletInfo( 
                me._fullPortletId, 
                url,
                function ( pInfo ) { 
                    win.document.write('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">');
                    win.document.write('<html><head><title>IBM WebSphere Portal - Help</title><style type="text/css">');
                    
                    //Copy over stylesheet information so everything displays correctly.
                    if ( dojo.isIE ){
                        for(var i = 0; i < document.styleSheets.length; i++ ){
                            win.document.write(document.styleSheets[i].cssText);
                        }
                    } else {
                        for(var i = 0; i < document.styleSheets.length; i++ ){
                            var ii = 0;
                            do{
                                var cssRule = document.styleSheets[i].cssRules[ii];                                                     
                                if(cssRule){
                                    win.document.write(cssRule.cssText);
                                }                                                           
                                ii++;
                            }while(cssRule)
                        }
                    }
                    win.document.write('</style></head><body>');
                    
                    //Write the help markup into the body element.
                    win.document.write(pInfo.markup);
                    
                    win.document.write('</body></html>');
                    win.document.close();
                    //We need special handling for FF.
                    if ( window.addEventListener ) {
                        var handler = new com.ibm.portal.aggregation.event.NamedAnchorHandler();
                        win.document.addEventListener( "click", function (e) { handler.handle(e); }, true );
                    }   
                },
                null,
                //This parameter makes the request stateless. This means our normal processing of a response will not occur and
                //it's entirely up to the callback function passed into the getPortletInfo function to do something interesting
                //with the response. The navigational state on the client for this portlet will also NOT be updated by this request.
                true
            );*/
            
            ibm.portal.debug.exit( "IbmSkin._doHelp" );
        },
        // ************************** PORTLET WINDOW STATE SWITCHING FUNCTIONS *************************************
        _doMaximize: function( /*Object*/ args ) {           
            // summary: Maximize the window object. 
            // description: This function can be called two ways. One is the dojo subscription topic which is called
            //      when the user selects the "Maximize" option on the window states menu. The second way is when the portlet
            //      is first rendered, if it's initial window state is "Maximized", this function will be called to perform
            //      the necessary DOM manipulations to make the window maximized with the existing markup (as the initial
            //      request will already contain the "maximized" markup). In this second case, the "args" parameter is expected
            //      to be NULL. 
            // args: Expects a single property "url" with the maximize portlet url. 
            ibm.portal.debug.entry( "IbmSkin._doMaximize", [ args ] );
            
            var update = true;
            
            if( typeof window["PC_"+this._portletWindowId+"_doWindowState"] == "function" ){
                update = window["PC_"+this._portletWindowId+"_doWindowState"]("maximized",this._getPortletContainer());
            }   
            
            if(update){
                var url = null;
                if(args != null){
                    this.actionsLoaded = false;
                    url = args["url"];
                }
    
                if(dojo.byId(this._fullPortletId+"_minimized") != null){
                    this.unMinimize();
                }
                
                var main = dojo.byId("mainContent");
                var portlet = this._getPortletContainer();
                var portletParent = dojo.byId("portlet_"+this._fullPortletId).parentNode;
                
                var placeHolderDiv = document.createElement("div");
                placeHolderDiv.id = "maximized_portlet_placeholder"+this._fullPortletId;
                portletParent.appendChild(placeHolderDiv);
                
                main.appendChild(portlet);
                
                this._hide(main.childNodes[0].id);
                
                //Markup can be different depending on the portlet window state so give the portlet a chance to adjust to it's
                //new window state.
                if(url != null){                    
                    dojo.publish("/portal/Skin/ClearMenu/"+this._fullPortletId,[{menuId: "contextMenu_windowStateMenu"+this._fullPortletId}]);  
                    var serviceReq = new com.ibm.portal.services.PortletFragmentService();
                    serviceReq.getPortletInfo( this._fullPortletId, url );
                }
            }
            
            ibm.portal.debug.exit( "IbmSkin._doMaximize" );
        },
        _doMinimize: function ( /*Object*/ args ) {
            // summary: Minimize the window object. 
            // description: This function can be called two ways. One is the dojo subscription topic which is called
            //      when the user selects the "Minimize" option on the window states menu. The second way is when the portlet
            //      is first rendered, if it's initial window state is "Minimized", this function will be called to perform
            //      the necessary DOM manipulations to make the window minimized with the existing markup (as the initial
            //      request will already contain the "minimized" markup). In this second case, the "args" parameter is expected
            //      to be NULL. 
            // args: Expects a single property "url" with the maximize portlet url.
            ibm.portal.debug.entry( "IbmSkin._doMinimize", [ args ] );
            
            var update = true;
            
            if( typeof window["PC_"+this._portletWindowId+"_doWindowState"] == "function" ){
                update = window["PC_"+this._portletWindowId+"_doWindowState"]("minimized",this._getPortletContainer());
            }  
            
            if(update){
                var url = null;
                if(args != null){
                    this.actionsLoaded = false;
                    url = args["url"];
                }
                if(dojo.byId("maximized_portlet_placeholder"+this._fullPortletId) != null){
                    this.unMaximize();
                }
                
				// create a div  that will contain the inline mode and the portlet body div 
                var hiddenDiv = document.createElement("DIV");
                hiddenDiv.id = this._fullPortletId+"_minimized";
                if(!this._isHidden(this._getPortletActionsShelf())){
                    this._hide(this._getPortletActionsShelf());
                }
                
                var portlet = dojo.byId(this._fullPortletId);
                var modesContainer = dojo.byId(this._getModesContainer());
                var portletParent = portlet.parentNode;
                
                portletParent.insertBefore(hiddenDiv, modesContainer);
                hiddenDiv.appendChild(modesContainer);
                portletParent.removeChild(portlet);
                var tempNode = portlet;
				// this code must be here for a reason?
                if(dojo.isOpera) {
                    tempNode = document.createElement("div");
                    tempNode.innerHTML = portlet.innerHTML;
                    tempNode.setAttribute('id', portlet.getAttribute('id'));
                    tempNode.setAttribute('class', portlet.getAttribute('class'));
                    tempNode.setAttribute('loaded', portlet.getAttribute('loaded'));
                    delete portlet;
                }
                hiddenDiv.appendChild(tempNode);
                
				// hide the new containing div so both the inline mode and portletbody will be hidden
                this._hide(this._fullPortletId+"_minimized");
                
                //Markup can be different depending on the portlet window state so give the portlet a chance to adjust to it's
                //new window state.
                if(url != null){
                    dojo.publish("/portal/Skin/ClearMenu/"+this._fullPortletId,[{menuId: "contextMenu_windowStateMenu"+this._fullPortletId}]);
                    var serviceReq = new com.ibm.portal.services.PortletFragmentService();
                    serviceReq.getPortletInfo( this._fullPortletId, url );
                }
            }
            
            ibm.portal.debug.exit( "IbmSkin._doMinimize" );
        },
        _doRestore: function ( /*Object*/ args ) {
            // summary: Returns the portlet window state to NORMAL.
            // args: Expects a single property "url" with the maximize portlet url.
            ibm.portal.debug.entry( "IbmSkin._doRestore", [ args ]);
            
            var update = true;
            
            if( typeof window["PC_"+this._portletWindowId+"_doWindowState"] == "function" ){
                update = window["PC_"+this._portletWindowId+"_doWindowState"]("normal",this._getPortletContainer());
            }  
            
            if(update){
                var url = null;
                if(args != null){
                    this.actionsLoaded = false;
                    url = args["url"];
                }
                
                if(url != null){
                    dojo.publish("/portal/Skin/ClearMenu/"+this._fullPortletId,[{menuId: "contextMenu_windowStateMenu"+this._fullPortletId}]);
                    var serviceReq = new com.ibm.portal.services.PortletFragmentService();
                    serviceReq.getPortletInfo( this._fullPortletId, url );
                }else{
                    if(dojo.byId("maximized_portlet_placeholder"+this._fullPortletId) != null){
                        this.unMaximize();
                    }else{
                        this.unMinimize();
                    }
                }
            }
            
            ibm.portal.debug.exit( "IbmSkin._doRestore" );
        },
        unMaximize: function () {
            // summary: Performs the necessary DOM manipulations to return the window state to NORMAL from MAXIMIZED.
            //      NO portlet markup is updated as a result of a call to this function.
            ibm.portal.debug.entry( "IbmSkin.unMaximize");
            this.actionsLoaded = false;
            
            var main = dojo.byId("mainContent");
            
            maximizedPortlet = this._getPortletContainer();
            
            var origPortletDiv = dojo.byId("maximized_portlet_placeholder"+this._fullPortletId);
            var origPortletParent = origPortletDiv.parentNode;
            
            origPortletParent.replaceChild(maximizedPortlet, origPortletDiv);
            
            var origLayoutNode = main.childNodes[0];
            this._show(origLayoutNode.id);
            
            ibm.portal.debug.exit( "IbmSkin.unMaximize" );
        },
        unMinimize: function () {
            // summary: Performs the necessary DOM manipulations to return the window state to NORMAL from MINIMIZED.
            //      NO portlet markup is updated as a result of a call to this function.
            ibm.portal.debug.entry( "IbmSkin.unMinimize");
            this.actionsLoaded = false;
            
			// find the hidden div that contains the portlet markup
            var hiddenDiv = dojo.byId(this._fullPortletId+"_minimized");
            var modesContainer = hiddenDiv.childNodes[0];
            var portlet = hiddenDiv.childNodes[1];
            var portletParent = hiddenDiv.parentNode;
            // move the  contents back out of the hidden div 
            portletParent.appendChild(modesContainer);
            portletParent.appendChild(portlet);
            
            // cleanup by removing the hiddendiv 
            portletParent.removeChild(hiddenDiv);
            ibm.portal.debug.exit( "IbmSkin.unMinimize" );
        },
        
        // ************************** MISCELLANEOUS SKIN ACTION FUNCTIONS *************************************
        refreshPortlet: function () {
            // summary: Refreshes this portlet using the navigational state currently stored on the client.
            ibm.portal.debug.entry( "IbmSkin.refreshPortlet" );
            
            this.actionsLoaded = false;
            
            var portletElem = this._getPortletContainer();
            
            if (document.all ){     
                portletElem.style.cssText = "filter: alpha(opacity=25); background-color: #c1c1c1;";
            }else{
                portletElem.setAttribute('style', "-moz-opacity: .25; background-color: #c1c1c1;");
            }
                        
            var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
            var url = state.newPortletRenderURL( this._portletWindowId );
            var serviceReq = new com.ibm.portal.services.PortletFragmentService();
            var me = this;
            
            
            serviceReq.getPortletInfo( 
                this._fullPortletId,
                url,
                function () {       
                    portletElem.removeAttribute("style");   
                }
            );
            
            ibm.portal.debug.exit( "IbmSkin.refreshPortlet" );
        },
        deletePortlet: function(){
            // summary: Confirms that the user wishes to delete the portlet, performs the DOM manipulations
            //      and sends the DELETE request to the Portal REST services.
            ibm.portal.debug.entry( "IbmSkin.deletePortlet" );
            
            //Are you sure?????
            var check = confirm(this._titlebarProperties["delete.confirm"]);
            
            if( check ) {
                //If the portlet is maximized, we have to undo our MAXIMIZE DOM manipulations so that the
                //other portlets on the page are visible once this one is deleted. Also there are no drop 
                //zones to clean up in the maximized case.
                if( dojo.byId("maximized_portlet_placeholder"+this._fullPortletId) != null ) {
                    var main = dojo.byId("mainContent");                   
                    var maxPortlet = main.childNodes[0];
                    var origLayoutNode = main.childNodes[1];
                    
                    this._show(origLayoutNode.id);
                    main.removeChild(maxPortlet);
                    
                    // var portlet = dojo.byId("dndSrc"+this._fullPortletId);
                    var portlet = dojo.byId("portlet_"+this._fullPortletId);
                    var portletParent = portlet.parentNode;
                    
                    portletParent.removeChild(portlet);
                } else {
                    //Clean up the drop zones and containers.
                    var portlet = dojo.byId("dndSrc"+this._fullPortletId);
                    var portletDZ = dojo.byId("tgt_dndSrc"+this._fullPortletId);
                    var container = CSADND_getPortletsContainer(portlet);
                    var containerType = container.getAttribute("CSAContainerType");
                    
                    if(containerType == "UCV"){
                        var parent = portlet.parentNode;
                        var parentDZ = portletDZ.parentNode.parentNode;
                        
                        container.childNodes[0].removeChild(parentDZ);
                        container.childNodes[0].removeChild(parent);
                    }else{
                        var parentDZ = portletDZ.parentNode;
                        
                        container.childNodes[0].childNodes[0].removeChild(parentDZ);
                        container.childNodes[0].childNodes[0].removeChild(portlet);
                    }
                }
                //Make sure to call destroy on the fragment so that all the events are properly unhooked.
                var fragment = com.ibm.portal.aggregation.PORTAL_AGGREGATOR.page.getFragment( this._fullPortletId );
                if ( fragment ) {
                    fragment.destroy();
                }
                com.ibm.portal.aggregation.PORTAL_AGGREGATOR.page.layoutManager.deletePortletInLayoutManager( this._fullPortletId );
                //Finally, send the request to the Portal REST services to persist the change to the layout model.  
                this._makeDeleteRequest( this._fullPortletId );
            } else {
                //The user apparently didn't really want to the delete the portlet.
                ibm.portal.debug.exit( "IbmSkin.deletePortlet", false );
                return false;
            }
            
            ibm.portal.debug.exit( "IbmSkin.deletePortlet" );
        },
        _makeDeleteRequest: function( controlId ) {
            // summary: Sends the DELETE request to the Portal REST Services to remove this portlet from
            //      the layout model.
            // controlId: the object id of the Control(PortletWindow)
            ibm.portal.debug.entry( "IbmSkin._makeDeleteRequest", arguments );
            
            //validate input params
            if (!controlId){
                ibm.portal.debug.text("_makeDeleteRequest: a required parameter is missing or null.");
                return false;
            }

            var deleteURL = ibmPortalConfig["contentHandlerURI"]+ '?uri='+controlId;
            ibm.portal.debug.text("_makeDeleteRequest: Delete URL is " + deleteURL);

            //get resource bundle for status messages
            var v3themeProperties = dojo.i18n.getLocalization("com.ibm.portal", "v3theme", csaUserLocale);
                            
            var request = new com.ibm.portal.services.PortalRestServiceRequest( { url: deleteURL } );
            request.remove( function(type, data, xhr, callbackargs){
                    var resTxt = data;
                    if (type == "load") {
                        ibm.portal.debug.text("_makeDeleteRequest: success: "+resTxt);
                        var statusMessage = new com.ibm.portal.aggregation.StatusMessage('info', v3themeProperties["portletDeleteOk"], "");
                        dojo.publish("/portal/status",[{message: statusMessage}]);
                        dojo.publish("/portal/Skin/RefreshSkins/");
                    } else {
                        ibm.portal.debug.text("_makeDeleteRequest: failure: "+resTxt);
                        var statusMessage = new com.ibm.portal.aggregation.StatusMessage('error', v3themeProperties["portletDeleteFailed"], resTxt);
                        dojo.publish("/portal/status",[{message: statusMessage}]);
                        dojo.publish("/portal/PortalPage/reload");
                    }
            } );
            
            ibm.portal.debug.exit( "IbmSkin._makeDeleteRequest" );
        },
        movePortlet: function( /*Object*/ args ) {
            // summary: Moves the portlet in the specified direction.
            // args: Expects a single property called "direction" with a value of "left","right","up","down"
            ibm.portal.debug.entry( "IbmSkin.movePortlet", [ args ] );
            
            //Make sure we have a valid direction.
            var direction = null;
            if(args != null){
                direction = args["direction"];
            }
            //Part of the complication of moving portlets around is the drop zones. The DND javascript
            //already knows how to do this, so let it handle the heavy lifting for us.
            var sourceContainer = CSADND_getPortletsContainer(dojo.byId("dndSrc"+this._fullPortletId));
            ibm.portal.debug.text("movePortlet: _leftMoveTargetId is " + this._leftMoveTargetId);
            ibm.portal.debug.text("movePortlet: _rightMoveTargetId is " + this._rightMoveTargetId);
            ibm.portal.debug.text("movePortlet: _upMoveTargetId is " + this._upMoveTargetId);
            ibm.portal.debug.text("movePortlet: _downMoveTargetId is " + this._downMoveTargetId);
            ibm.portal.debug.text("movePortlet: direction is " + direction);
            
            var targetId = null;
            if(direction != null){
                if(direction == "left"){
                    targetId = this._leftMoveTargetId;
                }
                else if(direction == "right"){
                    targetId = this._rightMoveTargetId;
                }
                else if(direction == "up"){
                    targetId = this._upMoveTargetId;
                }
                else if(direction == "down"){
                    targetId = this._downMoveTargetId;
                }
            }
            
            if(targetId != null) {
                com.ibm.portal.aggregation.PORTAL_AGGREGATOR.page.portletMoveController._findAndMovePortlet(sourceContainer, "dndSrc"+this._fullPortletId, targetId, direction);
            }
            
            if(direction != null){
                dojo.publish("/portal/Skin/MoveMenu/"+this._fullPortletId,[{menuId: "contextMenu_moveMenu"+this._fullPortletId}]);
                dojo.publish("/portal/Skin/RefreshSkins/", [{menuId: "contextMenu_moveMenu"+this._fullPortletId}]);
            }
            
            ibm.portal.debug.exit( "IbmSkin.movePortlet" );
        },
        refreshSkins: function() {
            // summary: Called to update the move menus after a portlet is moved on the client.
            ibm.portal.debug.entry( "IbmSkin.refreshSkins" );            
            
            //Clear the menu.
            this.clearMenuContents( "moveMenu" );
            var moveDirectionsArray = this._getValidMoves();
            //Load the menu items.
            this._createPortletMoveMenu( moveDirectionsArray );
            
            ibm.portal.debug.exit( "IbmSkin.refreshSkins" );
        },
        displayPortletMenu: function ( menuType ){
            // summary: Displays a portlet menu.
            // menuType: the menu type - "moveMenu" or "windowStatesMenu"
            ibm.portal.debug.entry( "IbmSkin.displayPortletMenu", [ menuType ] );
            
            var menuId = "contextMenu_"+menuType+this._fullPortletId;
            var menu = getContextMenu(menuId);
            var targetImg = document.getElementById( "menu_"+menuType+this._fullPortletId );                
            
            ibm.portal.debug.exit( "IbmSkin.displayPortletMenu" );
            return showContextMenu( menuId , targetImg ); 
        },
        clearMenuContents: function( menuType ){                    
            // summary: Clears the contents of a portlet menu.
            // menuType: the menu type - "moveMenu" or "windowStatesMenu"
            ibm.portal.debug.entry( "IbmSkin.clearMenuContents" );
            
            var menuId = "contextMenu_"+menuType+this._fullPortletId;
            var menu = getContextMenu(menuId);
            if ( menu ) {
                uncacheContextMenu(menu);
            }
            
            ibm.portal.debug.exit( "IbmSkin.clearMenuContents" );   
        }
    }
);


}

if(!dojo._hasResource["com.ibm.portal.aggregation.status"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.status"] = true;
dojo.provide( "com.ibm.portal.aggregation.status" );

dojo.require( "dojox.data.dom" );

//dojo.require( "com.ibm.portal.aggregation" );


dojo.declare( "com.ibm.portal.aggregation.StatusType", 
    null,
    {
	    constructor: function(/*int*/id,  /*String*/styleClass, /*String*/ iconPath, /*String*/ colorStr) {
	        this._id = id;
	        this._styleClass = "wptheme-"+styleClass;
	        this._iconPath = iconPath;
	        this._color = colorStr;  // status color fader start color for this class
	    }
    }
);

dojo.declare( "com.ibm.portal.aggregation.StatusMessage", 
    null,
    {
    	constructor: function(/*int*/type, /*String*/message, /*String*/details) {
	        this._type = type;
	        this._message = message;
	        this._details = details;
    	}	
    }
);


dojo.declare( "com.ibm.portal.aggregation.StatusColorFader", 
    null,
    {
    	constructor: function(/*String*/containerId, /*String*/endColorStr) {
	        this._containerId = containerId;
	        this._beginColorStr = "";
	        this._endColorStr = "";
	
	        this._redCtr = 0x00;
	        this._greenCtr = 0x00;
	        this._blueCtr = 0x00;
	
	        this._endRed = 0x00;
	        this._endGreen = 0x00;
	        this._endBlue = 0x00;
	
	        this.setEndColor(endColorStr);
	
	        this._fadeCallbackId = null;
	
	        this._enable = true;
	    },
        fade: function() {
             ibm.portal.debug.entry( "StatusColorFader.fade");

             // adjust r, g, b counters and see if we have reached end color
             var shouldStop = true;

             // looping here rather than incrementing/decrementing by 10
             // this makes sure we approach the end value without going past it
             // and we can still support any arbitrary start and end color values
             for (var ctr = 0; ctr < 10; ctr++) {
                 if (this._redCtr < this._endRed) {
                     this._redCtr++;
                     shouldStop = false;
                 }
                 else if (this._redCtr > this._endRed) {
                     this._redCtr--;
                     shouldStop = false;
                 }
    
                 if (this._greenCtr < this._endGreen) {
                     this._greenCtr++;
                     shouldStop = false;
                 }
                 else if (this._greenCtr > this._endGreen) {
                     this._greenCtr--;
                     shouldStop = false;
                 }
    
                 if (this._blueCtr < this._endBlue) {
                     this._blueCtr++;
                     shouldStop = false;
                 }
                 else if (this._blueCtr > this._endBlue) {
                     this._blueCtr--;
                     shouldStop = false;
                 }
             }

             if (shouldStop) {
                 this.stop();
             }
             else {
                 var updateColorStr = "#" + this.getHexString(this._redCtr) + this.getHexString(this._greenCtr) + this.getHexString(this._blueCtr);
                 var header = document.getElementById(this._containerId);
                 if (header != null) {
                     header.style.backgroundColor = updateColorStr;
                     ibm.portal.debug.text( "StatusColorFader.fade - updateColorStr = " + updateColorStr);  //remove this
                 }
                 else {
                     //alert("problem - header was null");
                     ibm.portal.debug.text( "StatusColorFader.fade - error, header was null");
                     this.stop();
                 }
             }

             ibm.portal.debug.exit( "StatusColorFader.fade");
        },
        start: function() {
             ibm.portal.debug.entry( "StatusColorFader.start");

             if (this._enable) {
                 var header = document.getElementById(this._containerId);
                 header.style.backgroundColor = this._beginColorStr;
                 var me = this;
                 this._fadeCallbackId = self.setInterval(function () { me.fade() }, 20);
             }

             ibm.portal.debug.exit( "StatusColorFader.start");
        },
        stop: function() {
             ibm.portal.debug.entry( "StatusColorFader.stop");

             if (this._fadeCallbackId != null) {
                 clearInterval(this._fadeCallbackId);
             }

             ibm.portal.debug.exit( "StatusColorFader.stop");
        },
        setBeginColor: function(/*string*/ hexString) {
             ibm.portal.debug.entry( "StatusColorFader.setBeginColor");

             this._beginColorStr = hexString;

             this._redCtr = this.getColorValue(hexString, "r");
             this._greenCtr = this.getColorValue(hexString, "g");
             this._blueCtr = this.getColorValue(hexString, "b");

             ibm.portal.debug.exit( "StatusColorFader.setBeginColor");
        },
        setEndColor: function(/*string*/ hexString) {
             ibm.portal.debug.entry( "StatusColorFader.setEndColor");

             this._endColorStr = hexString;

             this._endRed = this.getColorValue(hexString, "r");
             this._endGreen = this.getColorValue(hexString, "g");
             this._endBlue = this.getColorValue(hexString, "b");

             ibm.portal.debug.exit( "StatusColorFader.setEndColor");
        },
        getColorValue: function(/*string*/ hexString, /*string*/ rgb) {
             ibm.portal.debug.entry( "StatusColorFader.getColorValue");

             var subStr = "";
             if (rgb == "r") {
                 subStr = hexString.slice(1, 3);
             }
             else if (rgb == "g") {
                 subStr = hexString.slice(3, 5);
             }
             else if (rgb == "b") {
                 subStr = hexString.slice(5);
             }
             var rv = parseInt(subStr, 16);

             ibm.portal.debug.exit( "StatusColorFader.getColorValue", rv);
             return rv;
        },
        getHexString: function(/*int*/ num) {
             ibm.portal.debug.entry( "StatusColorFader.getHexString");

             var rv = 0;
             if (num >= 0x10) {
                 rv = num.toString(16);
             }
             else {
                 rv = "0" + num.toString(16);
             }
             ibm.portal.debug.exit( "StatusColorFader.getHexString");
             return rv;
        },
        setEnable: function(/*boolean*/ value) {
             ibm.portal.debug.entry( "StatusColorFader.setEnable");

             this._enable = value;

             ibm.portal.debug.exit( "StatusColorFader.setEnable");
        }
    }
);



dojo.declare( "com.ibm.portal.aggregation.StatusBar", 
    null,
    // end constructor

    {
    	constructor: function() {  
	        this._messages = new Array();
	        this._id = "wptheme-statusBar";
	        this._numMessages = 0;
	        this._showDetails = false;
	        this._baseURL = window.location.protocol+"//"+window.location.host;
	        
	        this._statusTypesMap = { 
	            "error" : new com.ibm.portal.aggregation.StatusType(0, "error",this._baseURL+ibm.portal.ThemeImages.errorGifPath,"#FF0000"),
	            "warning" : new com.ibm.portal.aggregation.StatusType(1, "warning",this._baseURL+ibm.portal.ThemeImages.warningGifPath,"FFFF00"),
	            "info" : new com.ibm.portal.aggregation.StatusType(2, "info",this._baseURL+ibm.portal.ThemeImages.infoGifPath,"#0077FF"),
	            "multiple" : new com.ibm.portal.aggregation.StatusType(3, "multiple",this._baseURL+ibm.portal.ThemeImages.multipleMessagesGifPath,"#E9CAF9")
	        };
	        //Subscribe to status topics.
	        this._statusHandler = dojo.subscribe("/portal/status",this, this.addMessage);
	
	        // create new message status bar color fader
	        this._summaryHeaderId = "wptheme-statusBarHeader";
	        var endBackgroundColorStr = this._statusTypesMap["multiple"]._color;
	        this.colorFader = new com.ibm.portal.aggregation.StatusColorFader( this._summaryHeaderId, endBackgroundColorStr);
	
	    }, 
    	
        _getStatusContainer: function () {
            return dojo.byId( this._id );   
        },
        setShowDetails: function (/*boolean*/showDetails) {
            this._showDetails = showDetails;    
        },
        toggleShowDetails: function() {
            ibm.portal.debug.entry( "StatusBar.toggleShowDetails");
            this.setShowDetails(!this._showDetails);
            //var statusContainer = this._getStatusContainer();
            //dojox.data.dom.removeChildren( statusContainer );
            this.colorFader.setEnable(false);  // disable the fader when toggling details
            this.render();
            this.colorFader.setEnable(true);
            ibm.portal.debug.exit( "StatusBar.toggleShowDetails" ); 
        },
        clear: function () {
            //summary: clears all contents of the statusBar
            var statusContainer = this._getStatusContainer();
            dojox.data.dom.removeChildren( statusContainer );
            this._messages.length = 0;
        },
        print: function() {
            var v3themeProperties = dojo.i18n.getLocalization("com.ibm.portal", "v3theme", csaUserLocale);
            
            //create new window
            var printWindow=window.open( '', 'status', 'resizable=yes,scrollbars=yes,menubar=no,toolbar=yes,status=no,width=800px,height=600px,screenX=10px,screenY=10px,top=10px,left=10px');
            var printDoc = printWindow.document;
            printDoc.title="Status";
            // set base url so relative image paths will resolve correctly
            var baseElem = printWindow.document.createElement("base");
            baseElem.setAttribute("href",window.location.protocol+"//"+window.location.host);
            printDoc.documentElement.childNodes[0].appendChild(baseElem);
            
            //copy in portal styles - This way doesn't work
            //var portalStylesLink = document.getElementById("portalStyles");
            //printDoc.documentElement.childNodes[0].appendChild(portalStylesLink.cloneNode(true));
            
            // this way doesn't work either
            //printWindow.document.head.appendChild(portalStylesLink.cloneNode());
            //if(document.createStyleSheet) { 
            //    document.createStyleSheet("javascript:'"+styles+"'"); 
            //} else { 
            //    newSS=document.createElement('link'); 
            //    newSS.rel='stylesheet'; 
            //    newSS.href='data:text/css,'+escape(styles); 
            //    document.documentElement.childNodes[0].appendChild(newSS); 
            //}  
            
            
            //neither does this one at the moment          
            /*
            var portalStyles = printDoc.createElement("style");
            for(var i = 0; i < document.styleSheets.length; i++ ){
                var ii = 0;
                do{
                    var cssRule = document.styleSheets[i].cssRules[ii];
                    
                    if(cssRule){
                        portalStyles.appendChild(printDoc.createTextNode(cssRule.cssText));
                    }
                    
                    ii++;
                }while(cssRule)  
            }
            printDoc.body.appendChild(portalStyles);         
             */

            // for(var i = 0; i < document.styleSheets.length; i++ ){
                //printWindow.stylesheets.appendChild(document.stylesheets[1].cloneNode());
                //var ii = 0;
                //do{
                //  var cssRule = document.styleSheets[i].cssRules[ii];
                //  
                //  if(cssRule){
                //      win.document.write(cssRule.cssText);
                //  }
                //  
                //  ii++;
                //}while(cssRule)  
            //}         
            // write in status
            //var statusHTML = this._getStatusContainer().innerHTML;
            //printWindow.document.body.innerHTML = "<div id='printStatusBar'>"+statusHTML+"</div>";
            
            //add container div
            printDoc.body.innerHTML = "<div id='wptheme-statusBar'></div>";
//          this.render(printWindow.document.getElementById("wptheme-statusBar"));
            
            for (x in this._messages){
                var statusType = this._statusTypesMap[this._messages[x]._type];
            
            // status entry base container
                var entry = printDoc.createElement( "div" );
                entry.className = statusType._styleClass;
            
            // summary header - the colored area of each status message
                var header = printDoc.createElement("h3");
            // Lefthandside (LHS) contains icon, message, and show/hide details
                var header_LHS = printDoc.createElement("div");
                header_LHS.className = "wptheme-statusHeaderLHS";
                var icon = printDoc.createElement("img");
                icon.src = statusType._iconPath;
                icon.alt = v3themeProperties[this._messages[x]._type];
                icon.setAttribute("title",v3themeProperties[this._messages[x]._type]);
                header_LHS.appendChild(icon);
                header_LHS.appendChild(printDoc.createTextNode(this._messages[x]._message));
                header.appendChild(header_LHS);
                entry.appendChild(header);
                
                var detailsDiv = printDoc.createElement("div");
                detailsDiv.className = "wptheme-details";
                detailsDiv.appendChild(printDoc.createTextNode(this._messages[x]._details));
                entry.appendChild(detailsDiv);
                printWindow.document.getElementById("wptheme-statusBar").appendChild(entry);
            }

            printDoc.close();
            printWindow.focus();
        },
        addMessage: function(/*Object*/args) {
            //summary: adds a message to the message array  
            ibm.portal.debug.entry( "StatusBar.addMessage", [args] );
            var message = args.message;
            ibm.portal.debug.text( "Adding message: " + message._message );
            this._messages.push(message);
            // temp limit to handle infinite loops if there is an error raised in this code
            if (this._messages.length < 100){
               this.render(this._getStatusContainer());
            }
            ibm.portal.debug.exit( "StatusBar.addMessage" );    
        },
        render: function(/*Node*/ targetContainer) {
            //summary: render the message(s) in the statusBar  
            ibm.portal.debug.entry( "StatusBar.render" );
            if (targetContainer == null){
            targetContainer = this._getStatusContainer();
            }
            dojox.data.dom.removeChildren( targetContainer );
            // only render multiple header if there is more than one message.
            // in renderMessage, if their is only one message, it will render as the header
            if (this._messages.length > 1) {
                this.renderHeader(targetContainer);
            }
            else if (this._messages.length == 1) {
                this.renderMessage(targetContainer,this._messages[0], this._showDetails,0);
            }
            
            // if showing details, render all the messages
            if(this._showDetails && this._messages.length > 1){
                for (x in this._messages){
                    this.renderMessage(targetContainer, this._messages[x], this._showDetails, x);
                }
            }
            // if not showing details, only show the last one
            //else {
            //    this.renderMessage(this._messages[this._messages.length-1], this._showDetails, this._messages.length-1 );
            //}


            // if we have more than one message and we are not showing details,
            // then start a color fade based on the last message received
            if(!this._showDetails && this._messages.length > 1){
                // in case its already running for a previous message
                this.colorFader.stop();

                // get the last message type that came in to get its color
                var end = this._messages.length - 1;
                var startBackgroundColorStr = this._statusTypesMap[this._messages[end]._type]._color;
                this.colorFader.setBeginColor(startBackgroundColorStr);
                this.colorFader.start();
            }
        
            ibm.portal.debug.exit( "StatusBar.render" );    
        },
        renderHeader: function(/*Node*/ targetContainer) {
            var v3themeProperties = dojo.i18n.getLocalization("com.ibm.portal", "v3theme", csaUserLocale);
            
            //summary: render the main statusBar 
            //description: this is the main statusBar, which includes the main controls, and either the single status message,
            // or an indicator of the number of messages contained within the details view. 
            ibm.portal.debug.entry( "StatusBar.renderHeader");
            //
            if(this._messages.length > 1) {
                var headerMessage = new com.ibm.portal.aggregation.StatusMessage( "multiple", v3themeProperties["itemsAvailable"], "" );
                this.renderMessage(targetContainer, headerMessage, this._showDetails, -1);
            }
            // if only one message, render it as the header
            else if(this._messages.length == 1){
               // this.renderMessage(targetContainer, this._messages[0], this._showDetails, 0);
            }
            ibm.portal.debug.exit( "StatusBar.renderHeader" );  
        },
        renderMessage: function (/*Node*/ targetContainer, /*com.ibm.portal.aggregation.StatusMessage*/message, /*boolean*/ showDetails, /*int*/index ) {
            //summary: renders a single message in the statusBar
            //description: renders the following markup:
            //<div class="error">
            //<h3><img src="./error.gif"/> An Error Occurred <a href="#"> 3 more...</a></h3>
            //<div class="details">Its a really really bad one</div>
            //</div>
            ibm.portal.debug.entry( "StatusBar.renderMessage", [message, this._numMessages] );
            //var statusContainer = this._getStatusContainer();
            var statusType = this._statusTypesMap[message._type];
            //dojox.data.dom.removeChildren( statusContainer );
            
            // status entry base container
            var entry = document.createElement( "div" );
            entry.className = statusType._styleClass;
            
            // summary header - the colored area of each status message
            var header = document.createElement("h3");
            header.id = this._summaryHeaderId;   // set id for color fader
            // Lefthandside (LHS) contains icon, message, and show/hide details
            var header_LHS = document.createElement("div");
            if(!bidiSupport.isRTL){
                header_LHS.className = "wptheme-statusHeaderLHS";
            }
            else{
                header_LHS.className = "wptheme-statusHeaderRHS";
            }               

            var v3themeProperties = dojo.i18n.getLocalization("com.ibm.portal", "v3theme", csaUserLocale);

            var icon = document.createElement("img");
            icon.src = statusType._iconPath;
            icon.alt = v3themeProperties[message._type];
            icon.setAttribute("title", v3themeProperties[message._type]);
            header_LHS.appendChild(icon);
            
            var messageText = null;
          
            // is this the top header status summary and we have more than one message
            if(this._messages.length > 1 && index < 0){
//                header_LHS.appendChild(document.createTextNode(''+this._messages.length+' messages are available for review.'));
                // then format the message
                if(!bidiSupport.isRTL)
                    messageText = ''+this._messages.length+' '+v3themeProperties["messagesAvailable"];
                else
                    messageText = v3themeProperties["messagesAvailable"]+' '+this._messages.length+'';
            }
            // do we have only one message
            else if(this._messages.length == 1){
                messageText = message._message;
                // adjust the left margin for a single message which will have a style class that is normally indented
                entry.style.marginLeft="0px";
            }
            else{
                header_LHS.appendChild(document.createTextNode(message._message));
            }

            header.appendChild(header_LHS);

            // render top level multiple messages status bar header
            if((this._messages.length > 1 && index < 0)||(this._messages.length == 1)){
                var me = this;
                var detailsAnchor = document.createElement("a");
                detailsAnchor.href="javascript:void(0)";
                detailsAnchor.onclick=function (e) { me.toggleShowDetails(); return false; };

                var header_RHS = document.createElement("div");

                var clearAnchor = document.createElement("a");
                clearAnchor.href="javascript:void(0);";
                clearAnchor.onclick=function (e) { me.clear(); return false; };
                clearAnchor.appendChild(document.createTextNode(v3themeProperties["clear"]));

                var printAnchor = document.createElement("a");
                printAnchor.href="javascript:void(0);";
                printAnchor.onclick=function(e){ me.print(); };
                printAnchor.appendChild(document.createTextNode(v3themeProperties["print"]));

                var showDetailsText = v3themeProperties["showDetails"];
                var hideDetailsText = v3themeProperties["hideDetails"];
                
                if(bidiSupport.isRTL){
                	header_RHS.className = "wptheme-statusHeaderLHS";
                }
                else {
                	header_RHS.className = "wptheme-statusHeaderRHS";
                }

                header_LHS.appendChild(document.createTextNode(messageText));
                if (showDetails){
                    detailsAnchor.appendChild(document.createTextNode(hideDetailsText));
                }
                else {
                    detailsAnchor.appendChild(document.createTextNode(showDetailsText));
                }
                header_LHS.appendChild(detailsAnchor);
                // clear link
                header_RHS.appendChild(clearAnchor);
                // print link
                header_RHS.appendChild(printAnchor);
                header.appendChild(header_RHS);
            }

            var clearingDiv = document.createElement("div");
            clearingDiv.className="clearing";
            header.appendChild(clearingDiv);

            entry.appendChild(header);
    
            // details
            if (showDetails && index >=0){
                var detailsDiv = document.createElement("div");
                detailsDiv.className = "wptheme-details";
                detailsDiv.appendChild(document.createTextNode(message._details));
                entry.appendChild(detailsDiv);
            }
            targetContainer.appendChild(entry);
            ibm.portal.debug.exit( "StatusBar.renderMessage" );     
        }
    }
); // end declare StatusBar

}

if(!dojo._hasResource["com.ibm.portal.aggregation.common"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.common"] = true;
dojo.provide("com.ibm.portal.aggregation.common");













}

if(!dojo._hasResource["com.ibm.portal.aggregation"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation"] = true;
dojo.provide( "com.ibm.portal.aggregation" );

dojo.require( "com.ibm.aggregation" );
dojo.require( "com.ibm.portal.aggregation.common" );

dojo.require( "com.ibm.portal.xslt" );
dojo.require( "com.ibm.portal.xpath" );
dojo.require( "com.ibm.portal.EventBroker" );
dojo.require( "com.ibm.portal.utilities.html" );


dojo.declare( "com.ibm.portal.aggregation.PortalAggregator", 
              com.ibm.aggregation.Aggregator,
                          {
                // summary: Implementation of the portal aggregation.
                // description: The aggreagtion consists of two elements, the page itself and the
                //      navigation for navigating between pages. Subscribes to the state change topic
                //      so that any navigational state change triggers the rendering process. Each element
                //      involved in the rendering process is responsible for determining if it has changed
                //      based on the new state.
                // navigation: the navigation implementation
                // page: the page implementation.
                
                constructor: function () {
                    //Initialize instance variables.
                    this.navigation = new com.ibm.portal.aggregation.theme.PortalNavigation();
                    this.page = new com.ibm.portal.aggregation.PortalPage();
                    this.statusBar = new com.ibm.portal.aggregation.StatusBar();
                    
                    //Subscribe to state changes. Any time the navigational state changes, we want
                    //to start the aggregation again.
                    //dojo.event.topic.subscribe( "/portal/NavigationalStateController/stateChanged", this, this.start );
                    this._stateChangedStartHandle = com.ibm.portal.EVENT_BROKER.stateChanged.register( this, this.start );
                },
                                
                start: function () {
                    // summary: Starts the aggregation rendering process.
                    // description: Pulls the aggregation information and starts
                    //      the rendering process. The information is based on the
                    //      current navigational state provided by the Navigation 
                    //      object.
                    ibm.portal.debug.entry( "PortalAggregator.start" );
                    //Get the current state.
                    var currentState = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                    ibm.portal.debug.text( "current navigational state is: " + dojox.data.dom.innerXML( currentState.stateDOM ) );
                    //Start the rendering process given the current state.
                    this.navigation.render( currentState );
                    this.page.render( currentState );
                    
                    
                    ibm.portal.debug.exit( "PortalAggregator.start" );
                }
              }
);

dojo.declare( "com.ibm.portal.aggregation.PortalPage", 
              com.ibm.aggregation.Page,
              {
                // summary: Implementation of a portal page.
                // description: Responsible for rendering the layout and any fragments (portlets)
                //      on the selected page. 
                layoutManager: null,
                portletMoveController: null,
                selectedTheme: null,
                _skins: [],
                _defaultSkinId: null,
                _fragments: [],
                _fragmentIdIndex: [],
                _previousSelection: null,
                _renderedFragments: 0,
                _resourceBaseURL: dojo.moduleUrl("com.ibm.portal.aggregation", ""),
                _ssaPagesCookieName: "com.ibm.wps.web2.ssaPages",
                
                constructor: function () {
                    //Subscribe to portlet render events so that we know when the page is completely rendered.
                    this._portletCompleteHandler = dojo.subscribe( "/portal/Portlet/complete", this, function () { this._renderedFragments++; } );  
                    this._portalPageReloadHandler = dojo.subscribe("/portal/PortalPage/reload", this, "reload");
                    this.layoutManager = new com.ibm.portal.aggregation.LayoutManager();
                    this.portletMoveController = new com.ibm.portal.aggregation.PortletMoveController();

                },
                render: function ( /*NavigationalState*/ state ) {
                    // summary: Renders the page.
                    // description: Renders the page and all its fragments in its specified layout.
                    // state: com.ibm.aggregation.NavigationState - the current navigation state
                    ibm.portal.debug.entry( "PortalPage.render", [state] );
                    this._firePageRenderPhaseStart();
                    
                    var selectedPage = state.selectionModel.getSelectedNode();
                    ibm.portal.debug.text( "Selected page is: " + selectedPage.URI );
                    if ( this._hasPageChanged( state.getPageSelection() ) ) {
                        this._clearHTMLElement();
                        this.setLoading();
                        this._destroyFragments();
                        if ( selectedPage.isLabel ){
                            
                            var problemProperties = dojo.i18n.getLocalization("com.ibm.portal", "problem", csaUserLocale);
                            var newTextNode = document.createTextNode(problemProperties["content.not.available"] + "  " + problemProperties["content.not.available.answer"]);
                            
                            var newDiv = document.createElement("DIV");
                            newDiv.style.cssText = "text-align: center; padding: 20px;";
                            newDiv.className = "generalErrorText";
                            newDiv.appendChild(newTextNode);
                            
                            var mainContent = dojo.byId("mainContent");
                            mainContent.innerHTML = "";
                            mainContent.appendChild(newDiv);
                            
                        }else{
                    
                            selectedPage.getLayoutModel( dojo.hitch( this, "applyXslt" ), { state: state } );
                        
                        }
                        
                    }
                    else {
                        //Page hasn't changed but maybe one of the fragments has
                        dojo.forEach( this._fragments, function ( frag ) { frag.render( state ); } );
                    }   
                    
                    ibm.portal.debug.exit( "PortalPage.render" );
                },
                reload: function(){
                    com.ibm.portal.aggregation.PORTAL_AGGREGATOR.page._previousSelection = null;
                    portalAggregator.page.render( com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState() );
                },
                setLoading: function () {
                    this._getHTMLElement().innerHTML = "<div id='mainContentLoading' ><img src='" + this._resourceBaseURL + "/progress-anim.gif' />&nbsp;Loading...</div>";
                },
                clearLoading: function () {
                    //dojo.dom.destroyNode( dojo.dom.removeNode( dojo.byId( 'mainContentLoading' ) ) );
                    var nodeToDelete = dojo.byId( 'mainContentLoading' );
                    if ( nodeToDelete ) {
                    	nodeToDelete.parentNode.removeChild( nodeToDelete );
                    	delete nodeToDelete;
                    }
                },
                applyXslt: function ( /*String*/type, /*dojo.dom.Document*/xmlData, /*XMLHttpRequest*/xhr, /*Array*/callbackargs ) {
                    // summary: Apply the XSLT to create the page's layout.
                    // type: the type of response
                    // xmlData: the xml of the Feed
                    // xhr: the XMLHttpRequest used to retrieve the feed
                    // callbackargs: any arguments passed through to this function
                    ibm.portal.debug.entry( "PortalPage.applyXslt", [xmlData,xhr,callbackargs] );
                    
                    var state = callbackargs["state"];
                    
                    var selection = state.getPageSelection();
                    var forceSSA = state.selectionModel.getSelectedNode().isStaticPage;
                    
                    ibm.portal.debug.text( "selection is: " + selection + "; isStaticPage? " + state.selectionModel.getSelectedNode().isStaticPage );
                                                            
                    //First, check to see if we have any portlets that require server-side aggregation.
                    if ( ibmPortalConfig["enforceSSAOnPortletLevel"] && !forceSSA) {
                         forceSSA = (this._ssaPagesCookieContainsSelection( selection ) || this._pageContainsSSAPortlets( xmlData ));
                    }
                    
                    if ( forceSSA ) {
                        ibm.portal.debug.text( "A condition exists which requires us to switch to server side aggregation mode for page: " + selection );
                        this._updateSSAPagesCookieIfNecessary( selection );
                        com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.fullPageRefresh( selection );
                    }
                    else {
                    
                        this.clearLoading();
                        // render the page layout
                        var trf = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer(ibmPortalConfig["mainContentXslt"]);
                        var xsltParams = { "loadingImagePath": this._resourceBaseURL + "progress-anim.gif"};
                        trf.transformToRegion( xmlData, xsltParams, this._getHTMLElement(), document );
                        // initialize the layoutManager with the new page's layoutmodel
                        this.layoutManager.setLayoutModel(xmlData);
                        ibm.portal.debug.text( "PortalPage.applyXslt after creating LayOutManager is "+this.layoutManager );
                        // find portlets that need to be rendered within the page layout
                        this._extractAndStoreFragments( xmlData );
                        
                        // render the portlets
                        this._renderedFragments = 0;    // setting here before we start rendering fragments
                        for ( var i = 0; i < this._fragments.length; i++ ) {
                            this._fragments[i].render( state ); 
                        }
                        
                        CSADND_ScanForNewSources();
                        this._firePageLayoutComplete();
                        this._watchForFragmentRenderComplete();
                    }   
                    
                    ibm.portal.debug.exit( "PortalPage.applyXslt" );
                },
                getFragment: function ( /*String*/ fragId ) {
                    // summary: Retrieves the Fragment on this page that matches the given id.
                    // description: Retrieves the Fragment with the given id and returns the
                    //      Fragment object. Returns null if no fragment with the given id can
                    //      be found.
                    // fragId: String - the fragment id
                    // returns: com.ibm.aggregation.Fragment or null
                    ibm.portal.debug.entry( "Page.getFragment", [ fragId ] );
                    
                    var retVal = null;
                    //dojo.debugShallow( this._fragmentIdIndex );
                    ibm.portal.debug.text( "Found index: " + this._fragmentIdIndex[ fragId ] );
                    retVal = this._fragments[ this._fragmentIdIndex[fragId] ];                  
                    
                    ibm.portal.debug.exit( "Page.getFragment", [ retVal ] );
                    return retVal; 
                },
                addSkin: function ( /*Skin*/ skinTemplate, /*Boolean*/isDefault ) {
                    // summary: Registers a new skin  for use in rendering the page.
                    // description: Adds the skinTemplate to the Page object for use in rendering.
                    //      The Page object stores an associative array of skin templates, and a reference to the 
                    //      default skin to use if one is not specified for the portlet. There is one tempate object
                    //      for each 'skin', but a factory method: skinTemplate.instance(fullPortletId) is used to
                    //      create an actual skin for a specific portlet.
                    // skinTemplate: com.ibm.portal.aggregation.Skin - the skin template object.
                    // isDefault: Boolean - whether the skin should be registered as the default for the theme.
                    ibm.portal.debug.entry( "Page.addSkin", [ skinTemplate, isDefault ] );
                    //validate before adding. most likely scenario for failure is the xsltUrl is empty if the findInTheme failed in bootstrap
                    if(skinTemplate != null && skinTemplate.validate()){
                        this._skins[skinTemplate._skinId] = skinTemplate;
                        if(isDefault) {
                            this._defaultSkinId = skinTemplate._skinId;
                        }
                    }
                    else {
                        //report error
                        var statusMessage = new com.ibm.portal.aggregation.StatusMessage(
                            "warning", 
                            "A skin defined for this theme could not be added. The default skin will be used in its place.", 
                            "This is likely due to the xslt not being found in the theme.\nThe skins attributes:\n resourceRoot:"+skinTemplate._resourceRoot+"\n xsltUrl: "+skinTemplate._xsltUrl);
                        dojo.publish("/portal/status",[{message: statusMessage}]);

                    }
                    ibm.portal.debug.exit( "Page.addSkin" );
                },
                getSkin: function ( /*String*/ skinId ) {
                    // summary: Retrieves the skin definition that matches the given id.
                    // description: Retrieves the skin with the given id and returns the
                    //      skin object. Returns null if no skin with the given id can
                    //      be found.
                    // skinId: String - the skin id
                    // returns: com.ibm.portal.aggregation.SkinDefinition or null
                    ibm.portal.debug.entry( "Page.getSkin", [ skinId ] );
                    var retVal = this._skins[skinId];
                    if(retVal == null){
                        retVal = this._skins[this._defaultSkinId];
                    }
                    ibm.portal.debug.exit( "Page.getSkin: return"+retVal._resourceRoot, [ retVal ] );
                    return retVal; 
                },
                setTheme: function(/*String*/ theme){
                    // summary: Set the page's selected theme resource root
                    this.selectedTheme = theme;
                },
                getTheme: function(){
                    // returns: String - page's selected theme resource root
                    return this.selectedTheme;
                },
                
                _getHTMLElement: function () {
                    // summary: Retrieve the main content area in which to render the page.
                    // returns: HTMLElement - the main content area
                    return dojo.byId( "mainContent" ); //HTMLElement
                },
                _clearHTMLElement: function () {
                    // summary: Clears the main content area where the page is rendered.
                    CSADND_RemoveDropZones();
                    var node = this._getHTMLElement();
                    while(node && node.childNodes.length > 0) {
                    	var elem = node.childNodes[0];
                    	elem.parentNode.removeChild( elem ); 
                    	delete elem;
                    }
                },
                _destroyFragments: function () {
                    dojo.forEach( this._fragments, function ( frag ) { frag.destroy(); } );
                    this._fragments = new Array();
                },
				addFragment: function (/*PortletControl*/ portletControl) {
					this._fragments.push( portletControl);
                    this._fragmentIdIndex[ portletControl.id ] = (this._fragments.length - 1);

				},
                _extractAndStoreFragments: function ( /*dojo.dom.Document*/ xmlData ) {
                    // summary: Determines the fragments that need to be rendered on this page.
                    // description: Uses xpath expressions to pull out all the portlets on this
                    //      page and stores them in an array.
                    ibm.portal.debug.entry( "PortalPage._extractAndStoreFragments", [xmlData] );
                    var xpathExpr = "/atom:feed/atom:entry[atom:content[model:layout-control[model:templateName='Control']]]";
                    var nodes = com.ibm.portal.xpath.evaluateXPath( xpathExpr, xmlData, com.ibm.portal.services.LAYOUT_MODEL.namespaces );
                    
                    ibm.portal.debug.text( "found " + nodes.length + " portlets on this page." )
                    
                    if (nodes != null && nodes.length > 0) {
                       var nodeCount = nodes.length;
                        for (var i = 0; i < nodeCount; i++) {
                            var atomIdStr = "id";
                            if ( dojo.isIE || dojo.isFF >= 3) {
                                atomIdStr = "atom:id";
                            }
                            var atomId = nodes[i].getElementsByTagName( atomIdStr )[0];
                            var skinId = this.layoutManager.getAssignedSkinId(atomId.childNodes[0].nodeValue);
                            var skin = this.getSkin(skinId);
                            ibm.portal.debug.text("getAssignedSkin ResourceRoot:"+skin._resourceRoot); 
							this.addFragment(new com.ibm.portal.aggregation.PortletControl( atomId.childNodes[0].nodeValue, nodes[i], skin ) );
                        }
                    }
                                    
                    ibm.portal.debug.exit( "PortalPage._extractAndStoreFragments" );
                },
                _hasPageChanged: function ( /*String*/ newSelection ) { 
                    // summary: Determines if the element has changed.
                    // description: Indicates if the element has changed since the last time
                    //      it was rendered.
                    // newState: the state that is being rendered
                    ibm.portal.debug.entry( "PortalPage.hasPageChanged", [ newSelection ] );
                    
                    var retVal = true;
                    
                    if ( this._previousSelection != null ) {
                        retVal = ( this._previousSelection != newSelection );   
                    }
                    
                    if ( retVal ) {
                        this._previousSelection = newSelection;
                    }
                    
                    ibm.portal.debug.exit ( "PortalPage.hasPageChanged", [ retVal ]);
                    return retVal;
                },
                _pageContainsSSAPortlets: function ( /*XMLDocument*/xmlData ) {
                    // summary: Determines if the page has portlets which require server-side aggregation.
                    // xmlData: the layout model xml feed
                    // returns: true if the page contains any portlets which require SSA mode
                    ibm.portal.debug.entry( "PortalPage._pageContainsSSAPortlets", [xmlData] );
                    
                    //Extract all the portlet definition links from the ATOM feed.
                    var xpathExpr = "/atom:feed/atom:entry[atom:content[model:layout-control[model:templateName='Control']]]/atom:link[@portal:rel='portlet-definition']";
                    var nodes = com.ibm.portal.xpath.evaluateXPath( xpathExpr, xmlData, com.ibm.portal.services.LAYOUT_MODEL.namespaces );
                    
                    ibm.portal.debug.text( "found " + nodes.length + " portlet definitions on this page." )
                    
                    //We want to cycle through the portlet definitions until we find any one that is marked
                    //as requiring SSA. 
                    var retVal = false;
                    
                    if (nodes != null && nodes.length > 0) {
                       var nodeCount = nodes.length;
                       
                       for (var i = 0; (i < nodeCount && !retVal); i++) {
                            //Get the href for retrieving the portlet definition.
                            var portletDefHref = nodes[i].getAttribute( "href" );
                            //Send the request for the portlet definition.
                            var url = new com.ibm.portal.services.ContentHandlerURL( portletDefHref );
                            var serviceReq = new com.ibm.portal.services.PortalRestServiceRequest( url, null, null, true );
                            serviceReq.read( function ( type, xmlData, xhr ) {
                                //See if our special config parameter is set for this portlet.
                                var prefExpr = "/atom:feed/atom:entry/atom:content/model:portlet-definition/model:portletpreferences[@name='com.ibm.wps.web2.renderMode']/base:value";
                                var prefNodes = com.ibm.portal.xpath.evaluateXPath( prefExpr, xmlData, com.ibm.portal.services.LAYOUT_MODEL.namespaces );
                                if ( prefNodes && prefNodes.length > 0 ) {
                                    var value = prefNodes[0].getAttribute( "value" );
                                    if ( value == com.ibm.portal.navigation.controller.CONSTANTS.forceSsaRenderMode ) {
                                        retVal = true;  
                                    }
                                }
                                else {
                                    //Now check our blacklist, if it exists.
                                    if ( portletsForServerSideOnly ) {
                                        prefExpr = "/atom:feed/atom:entry/atom:id[@portal:guid]";
                                        prefNodes = com.ibm.portal.xpath.evaluateXPath( prefExpr, xmlData, com.ibm.portal.services.LAYOUT_MODEL.namespaces );
                                        if ( prefNodes && prefNodes.length > 0 ) {
                                            var value = "";
                                            if(prefNodes[0].getAttributeNS) value = prefNodes[0].getAttributeNS( com.ibm.portal.services.LAYOUT_MODEL.namespaces.portal, "guid" );
                                            else value = prefNodes[0].getAttribute( "portal:guid" );
                                            value = com.ibm.portal.utilities.string.strip( value, ".webmod" );
                                            if ( portletsForServerSideOnly[value] == true ) {
                                                retVal = true;
                                            }
                                        }
                                    }
                                }
                            } );
                        }
                    }
                    
                    return retVal; // Boolean
                },
                _ssaPagesCookieContainsSelection: function ( /*String*/selection ) {
                    // summary: Determines if the current list of ssa pages contains the selected page.
                    // selection: the current selection
                    // returns: true if the page is in the selection
                    var list = dojo.cookie( this._ssaPagesCookieName );
                    return ( list && list.indexOf( selection ) >= 0 );
                },
                _updateSSAPagesCookieIfNecessary: function ( /*String*/selectionToAdd ) {
                    // summary: Adds the selection to the ssa pages list stored in the cookie if
                    //      it isn't already in the list.
                    // selectionToAdd: the page to add to the list
                    var list = dojo.cookie( this._ssaPagesCookieName );
                    if ( list == null ) {
                        list = "";
                    }
                    else {
                        list += ",";
                    }
                    
                    if ( list.indexOf( selectionToAdd ) == -1 ) {
                        list += selectionToAdd;
                    }   
                    dojo.cookie( this._ssaPagesCookieName, list, {path: "/"} ); 
                },
                _watchForFragmentRenderComplete: function () {
                    // summary: Waits for all fragments to render before firing the page render
                    //      complete event.
                    var me = this;
                    com.ibm.portal.utilities.waitFor( function () {
                        return me._renderedFragments == me._fragments.length;   
                    },
                    500,
                    me._firePageRenderPhaseComplete  
                    );
                },
                _firePageLayoutComplete: function () {
                    dojo.publish( "/portal/PortalPage/layoutComplete" );
                },
                _firePageRenderPhaseStart: function () {
                    //dojo.event.topic.publish( "/portal/PortalPage/start" );
                    com.ibm.portal.EVENT_BROKER.startPage.fire();
                },
                _firePageRenderPhaseComplete: function () {
                    //dojo.event.topic.publish( "/portal/PortalPage/complete" );
                    com.ibm.portal.EVENT_BROKER.endPage.fire();
                }
              }
);

//dojo.require( "com.ibm.portal.aggregation.javascript.uneval" );

dojo.declare( "com.ibm.portal.aggregation.PortletControl",
              com.ibm.aggregation.Fragment, {
              constructor: function ( fullPortletId, atomEntryXml, /*com.ibm.portal.aggregation.Skin*/skin ) {
                ibm.portal.debug.entry( "PortletControl.constructor", [ fullPortletId ] );            
                
                //Initialize instance variables.
                this.id = fullPortletId;
                this.portletWindowId = com.ibm.portal.utilities.getControlId( fullPortletId );
                this._atomEntryXml = atomEntryXml;
                this.portlet = new com.ibm.portal.aggregation.Portlet(fullPortletId);
                this.skin = skin.instance(fullPortletId, this, atomEntryXml);
                this._resourceBaseURL = dojo.moduleUrl("com.ibm.portal.aggregation", "");
                
                ibm.portal.debug.text( "Subscribing portletControl to: /portal/PortletFragmentService/portletInfoRetrieved/" + this.id.substring(3) );
                //Subscribe to event notifications for this portlet.
                /*dojo.subscribe( "/portal/PortletFragmentService/portletInfoRetrieved/" + this.id.substring(3), 
                                            this, 
                                            "handlePortletInfoRetrieved"                             
                );*/
                //com.ibm.portal.EVENT_BROKER.startFragment.register( this, "setLoading" );
                this._endFragmentHandle = com.ibm.portal.EVENT_BROKER.endFragment.register( this, "handlePortletInfoRetrieved" );
                
                ibm.portal.debug.exit( "PortletControl.constructor" );
              },
              
                // summary: The portlet implementation of a fragment.
                // description: Responsible for rendering the portlet markup. Any updates to the portlet
                //      markup are received by subscribing to the PortletFragmentService's topic for this
                //      particular portlet.
                handlePortletInfoRetrieved: function ( /*Object*/args ) {
                    // summary: Called whenever this portlet's info is retrieved. Pulls out the relevant
                    //      arguments and calls the update functions.
                    var pInfo = args["portletInfo"];
                    if ( !(pInfo instanceof Error) ) {
                        if ( pInfo.windowId == this.portletWindowId ) {
                            this.clearLoading();
                            this._update( pInfo );
                        }
                    }
                    else {
                        if ( this.id == args["id"] ) {
                            //renaming the variable here just to make it clear that we are working with
                            //an Error object and not the typicaly PortletInfo object.
                            var error = pInfo;
                            this.clearLoading( args );
                            var v3themeProperties = dojo.i18n.getLocalization("com.ibm.portal", "v3theme", csaUserLocale);
                            dojo.byId( this.id ).innerHTML = v3themeProperties["portletRequestFailed"];
                            
                            var errorMessage = new com.ibm.portal.aggregation.StatusMessage( 'error', v3themeProperties["portletRequestFailed"], this.id + " - " + error.message );
                    		dojo.publish("/portal/status",[{message: errorMessage}]);
                            //Fire the render complete event.
                    		this._firePortletRenderComplete();
                        }   
                    }       
                },
                destroy: function () {
                    /*dojo.unsubscribe( "/portal/PortletFragmentService/portletInfoRetrieved/" + this.id.substring(3), 
                                            this, 
                                            "handlePortletInfoRetrieved"                             
                    );*/
                    //com.ibm.portal.EVENT_BROKER.startFragment.unregister( this, "setLoading" );
                    com.ibm.portal.EVENT_BROKER.endFragment.unregister( this._endFragmentHandle );
                    this.portlet.destroy();
                    this.skin.destroy();    
                },
                render: function ( /*NavigationalState*/ state) {
                    // summary: Renders this HTML fragment.
                    // description: Renders this HTML fragment. Processes any included javascript using 
                    //      the specified javascript filter. 
                    // state: the current navigational state
                    ibm.portal.debug.entry( "PortletControl.render", [ state ] );
                    
                    var solo = state.getSoloPortlet();
                    if(!solo || solo == this.portletWindowId) {
                    	// if we're in solo, remember it
                    	if(solo == this.portletWindowId) this._inSolo = true;
                    	else this._inSolo = false;
						if (!this.skin.isRendered){
	                        this.skin.renderSkin();
	                    }
	                    if ( this.portlet._needsRefresh( state.getPortletState( this.portletWindowId ), this.portlet._currentState ) ) {
	                        //We want to ignore dependent portlets when this is the initial rendering of 
	                        //the portlet because all the portlets might not have rendered yet. The only 
	                        //case we run into this is when 2 portlets on the page use a shared render 
	                        //parmaeter (JSR286 support).
	                        var portletService = null;
	                        if ( this.portlet._currentState == null ) {
	                            portletService = new com.ibm.portal.services.IndependentPortletFragmentService();
	                        }
	                        else {
	                            portletService = new com.ibm.portal.services.PortletFragmentService();
	                        }
	                        
	                        portletService.getPortletInfo( this.id, state.newPortletRenderURL( this.portletWindowId ) );
	                    }
                    }
                    ibm.portal.debug.exit( "PortletControl.render" );
                },
                /* This function will render the view mode without a portlet state update */
                renderViewMode: function( pInfo, elem){
                    // summary: renders the portlets view mode, without a state update.
                    // description: if a skin supports inline edit modes, this method can
                    //     be used to update the view mode markup without changing the portlets
                    //     state to view mode. This is not called from normal page aggregation, but is 
                    //     initiated from the Skin object itself
                    ibm.portal.debug.entry( "PortletControl.renderViewMode", [pInfo, elem] );
                    var state = com.ibm.portal.navigation.controller.NAVIGATION_CONTROLLER.getState();
                    var url = state.newPortletRenderURL( this.portletWindowId, "view" );
                    var service = new com.ibm.portal.services.PortletFragmentService();
                    var me = this;
                    ibm.portal.debug.text("PortletControl.renderViewMode: calling PortletFragmentService.getPortletInfo(id:"+this.id+",url:"+url+",elem:"+elem+")")
                    service.getPortletInfo( this.id,
                                            url,
                                            function ( pInfo ) { 
                                                //me.portlet._markup.update( elem , pInfo.markup );
                    							var args = me.portlet.portletMarkup.prepareUpdate( elem , pInfo.markup, false, me.id );
                    							me.portlet.portletMarkup.update( args ); 
                                            },
                                            null,
                                            true
                                        );
                },
                
                setLoading: function ( args ) {
                },
                clearLoading: function ( args ) {
                    if ( !args ) {
                        this.skin.hideLoadingIfNecessary();
                    }
                    else if ( args.id.indexOf( this.portletWindowId ) != -1 ) {
                        this.skin.hideLoadingIfNecessary();
                    }    
                },
                _update: function( /*PortletInfo*/portletInfo, /*HTMLElement?*/elemToUpdate ) {
                    // summary: Updates the page with the actual markup and skin information.
                    ibm.portal.debug.entry( "PortletControl._update", [portletInfo] );
                    //Insert the markup.
                    //skin.render, just updates the skins state(activediv,max,min, etc), 
                    // and in case its not in view mode, requests a view mode update as well                    
                    this.skin.updateSkin( portletInfo );
                    if(this._inSolo) {
                    	// we are in solo mode, so tell the skin to maximize itself... won't change the state
                    	// or send off any requests.
                    	dojo.publish("/portal/Skin/maximized/"+this.id);
                    }
                    // this actually updates the active div with the markup in portletInfo
                    if(this.skin.getActiveDiv() != null){
                        this.portlet.update(portletInfo,this.skin.getActiveDiv());
                    }
                    
                    //IN IE, we need to run this every time we do an update.
                    if ( !this._dampened || window.attachEvent ) {
                        var dampener = new com.ibm.portal.aggregation.event.PortletEventDampener(function (e) { 
                                                                                                    ibm.portal.debug.entry ( "_cancelDefault", [ e ] );
                                                                                                    var form = e.target;
                                                                                                    if ( form && ! form.tagName.toLowerCase()=="form") {
                                                                                                        form = e.target.form;    
                                                                                                    }                                                                                                    
                                                                                                    ibm.portal.debug.text( "e.target = " + form );
                                                                                                    ibm.portal.debug.text( "form encoding type = " + form.enctype );
                                                                                                    return ( form.enctype != "multipart/form-data"); 
                                                                                                 });
                        var elem = dojo.byId( this.id );
                        //In Firefox (and other W3C compliant browsers), the onsubmit event bubbles. This allows us to 
                        //simply attach to the highest point in the browser DOM and catch the event there. Unfortunately,
                        //in IE, the onsubmit event does not bubble, so we have to attach the dampener to every form
                        //element in the portlet DOM.  
                        if ( elem.addEventListener ) {
                             elem.addEventListener( "submit", 
                                                               dojo.hitch( dampener, "dampen" ), 
                                                               false );
                        }
                        else if ( elem.attachEvent ) {
                            var forms = elem.getElementsByTagName( "FORM" );
                            dojo.forEach( forms, function ( formElem ) { 
                                                          formElem.attachEvent( "onsubmit", dojo.hitch( dampener, "dampen" ) );    
                                                      } );    
                        }
                                                               
                        this._dampened = true;
                    }    
                    //Update dependent portlets, if any.
                    this._processDependentPortlets( portletInfo.dependentPortlets );
                    //Fire the render complete event.
                    this._firePortletRenderComplete();
                    
                    ibm.portal.debug.exit( "PortletControl._update" );
                },
                _processDependentPortlets: function ( /*Array*/dependents ) {
                    // summary: Processes any dependent portlets, sending off requests to update
                    //      those portlets. 
                    // dependents: the array of dependent portlets
                    ibm.portal.debug.entry( "PortletControl._processDependentPortlets", [dependents] );
                    
                    //We want to ignore any dependent portlets that come back as a result of a dependent
                    //portlet update (got all that??). The reason is that with shared render parameters,
                    //if one portlet using a shared render param is updated, the other portlets that use 
                    //that shared param will always come back as dependent. Then when the dependent portlet
                    //is updated it will claim that the original portlet that was updated is dependent. Thus,
                    //we end up in a nasty circular dependency.
                    var portletService = new com.ibm.portal.services.IndependentPortletFragmentService();
                    
                    for ( var i = 0; i < dependents.length; i++ ) {
                        var portletid = dependents[i].uri;
                        portletid = portletid.replace( /fragment:pm:/g, "lm:" );
                        com.ibm.portal.aggregation.PORTAL_AGGREGATOR.page.getFragment( portletid ).setLoading();
                        portletService.getPortletInfo( portletid, dependents[i].link ); 
                    }   
                    
                    ibm.portal.debug.exit( "PortletControl._processDependentPortlets" );
                },
                _firePortletRenderComplete: function () {
                    dojo.publish( "/portal/Portlet/complete" );
                }       
              }
);

dojo.declare( "com.ibm.portal.aggregation.Portlet",
              com.ibm.aggregation.Fragment,
              {
                // summary: The portlet implementation of a fragment.
                // description: Responsible for rendering the portlet markup. Any updates to the portlet
                //      markup are received by subscribing to the PortletFragmentService's topic for this
                //      particular portlet.

                constructor: function ( fullPortletId ) {
                    var _method = "Portlet.constructor"
                    ibm.portal.debug.entry( _method, [ fullPortletId ] );           
                    
                    //Initialize instance variables.
                    this.id = fullPortletId;
                    this.portletMarkup = new com.ibm.portal.aggregation.PortletMarkup();
                    this._currentState = null;
                    this._restorePoints = [];
                    this.cleanUp = true;
                                    
                    ibm.portal.debug.exit( _method );
                },
                _cleanupJavascript: function ( /*String*/mode ) {
                	var _method = "Portlet._cleanupJavascript"
                    ibm.portal.debug.entry( _method, [ mode ] );
                	
                	var restorePoints = this._restorePoints[mode];
                    if ( restorePoints ) {
                        var scrubbers = com.ibm.portal.aggregation.javascript.uneval.RestorePoint.prototype.SCRUBBERS;
                    	var length = scrubbers.length;
                        for ( var i = 0; i < length; i++ ) {
                        	var scrubber = scrubbers[i];
                        	for ( var j = 0; j < restorePoints.length; j++ ) {
                        		restorePoints[j].restore( scrubber );
                        	}
                        }	
                    }
                    
                    ibm.portal.debug.exit( _method );
                },
                _executeInlineCSAScript: function(/*HTMLElement*/scriptElem) {
                	var script = scriptElem.innerHTML;
                	var f = new Function(script);
                	var me = this;
                	f.call(me);
                },
                registerUpdateHandler: function(/*Function*/ func) {
            		this._customUpdateHandler = func;
                },
                unregisterUpdateHandler: function() {
                	delete this._customUpdateHandler;
                },
                update: function( /*PortletInfo*/portletInfo, /*HTMLElement?*/elemToUpdate ) {
                    // summary: Updates the page with the actual markup and information.
                    var _method = "Portlet.update";
                    ibm.portal.debug.entry( _method, [portletInfo] );
                    if(this._timer) {
                    	try {
		                	clearTimeout(this._timer);
                    	}
                    	catch(e) {
                    	}
                    	this._timer = null;
                    }
                    					
                    //Store all the before and after snapshots; later we will create the restore points from
                    //these. These are unique to the portlet mode.
                    var me = this;
                    var currentMode = portletInfo.currentMode;
                    var beforePoints = [];
                    var afterPoints = [];
                    //Since all javascript code is executed asynchronously now, we need to use an event-based approach to take our
                    //snapshots. The startScriptExecution event fires directly before the script code is evaluated and the 
                    //endScriptExecution event fires directly after the script code is evaluated, with no relinquishing of control.
                    //This should keep our snapshots honest. 
                    var startHandle = com.ibm.portal.EVENT_BROKER.startScriptExecution.register( function ( args ) { 
                    	if ( args.portletId == me.id ) {
                    		beforePoints.push( new com.ibm.portal.aggregation.javascript.uneval.Snapshot() );
                    	}	
                    } );
                    var endHandle = com.ibm.portal.EVENT_BROKER.endScriptExecution.register( function ( args ) { 
                    	if ( args.portletId == me.id ) {
                    		afterPoints.push( new com.ibm.portal.aggregation.javascript.uneval.Snapshot() );
                    	}	
                    } );
                    var completeHandle = com.ibm.portal.EVENT_BROKER.endScriptHandling.register( function ( args ) {
                    	if ( args.portletId == me.id ) {
                    		//Clear out any existing restore points.
                    		me._restorePoints[currentMode] = [];
                    		ibm.portal.debug.text( "There are " + beforePoints.length + " before points and " + afterPoints.length + " after points.", _method );
                    		if ( beforePoints.length == afterPoints.length ) {
	                    		//Create each restore point from a before and after snapshot pair.
	                    		for ( var i = 0; i < beforePoints.length; i++ ) {
		                    		me._restorePoints[currentMode].push( new com.ibm.portal.aggregation.javascript.uneval.RestorePoint( beforePoints[i], afterPoints[i] ) ); 
	    	                	}
                    		}
                    		else {
                    			//Something bad happened - the number of before and after snapshots are not the same. We can not operate reliably here so just don't try and
                    			//clean anything up.
                    			beforePoints = null;
                    			afterPoints = null;
                    		}	
        	            	//Clean up after ourselves now that we don't care anymore.
            	        	var eb = com.ibm.portal.EVENT_BROKER;
                	    	eb.startScriptExecution.unregister( startHandle );
                    		eb.endScriptExecution.unregister( endHandle );
                    		eb.endScriptHandling.unregister( completeHandle );
                    	}	
                    });
                    
                    var args = this.portletMarkup.prepareUpdate( elemToUpdate, portletInfo.markup, false, this.id );
                    args.currentMode = portletInfo.currentMode;
                    for(var i = 0; i < args._CSAScripts.length; i++) {
                    	this._executeInlineCSAScript(args._CSAScripts[i]);
                    }
                    delete args._CSAScripts;
                    
                    //this._markup.update( elemToUpdate, portletInfo.markup, false, this.id );
                    this.preUpdate(portletInfo, elemToUpdate);
                    var me = this;
                    args.onScriptCompleted = function () {
                    	me.postUpdate(portletInfo, elemToUpdate);
                    }	
                    
                    if(this._customUpdateHandler) {
                    	this._customUpdateHandler.call(this.portletMarkup, args );
                    }
                    else {
                    	this.portletMarkup.update( args );
                    }
                    this._currentState = portletInfo.updatedState;
                    
                    //this.postUpdate( portletInfo, elemToUpdate );
                    
                    ibm.portal.debug.exit( _method );
                },
                preUpdate: function(/*PortletInfo*/portletInfo, /*HTMLElement?*/elemToUpdate) {
                	if(this.cleanUp) {
                        this._cleanupJavascript( portletInfo.currentMode );
                	}
                },
                postUpdate: function(/*PortletInfo*/portletInfo, /*HTMLElement?*/elemToUpdate) {

					// look for meta tags that indicate a timed refresh should occur, and refresh this portlet
					// with the URL specified if one exists
					try {
						var metas = dojo.query("meta", elemToUpdate);
						for(var i = 0; i < metas.length; i++) {
							var httpEquiv = dojo.isIE ? metas[i].httpEquiv : metas[i].getAttribute("http-equiv");
			            	if(httpEquiv != "Refresh") continue;
			            	var meta = metas[0];
							var content = meta.getAttribute("content");
							var match = content.match(/^\s*(\d+)\s*(?:;\s*url=\s*(.+)\s*)?\s*$/im);
							if(match && match[1]) {
								var time = match[1] * 1000;
								var url = match[2];
								if(!url) {
									var state = new com.ibm.portal.navigation.controller.NavigationalState(this._currentState);
									url = state.newPortletRenderURL( com.ibm.portal.utilities.getControlId( this.id ) );
								} 
								url = dojo.string.trim(url);
								
								var serviceReq = new com.ibm.portal.services.PortletFragmentService();
								ibm.portal.debug.text( "Setting auto-refresh timer on portlet " + this.id + " for " + time + " ms");
								this._timer = setTimeout(dojo.hitch(this, function(){serviceReq.getPortletInfo(this.id, url)}), time);
								break;
							}
			            }
					}
					catch(e){
						// not mission critical, so don't expose any errors, just don't break everything else
					}		
					
					this.unregisterUpdateHandler();
					ibm.portal.debug.exit( "Portlet._update" );
                },
                _needsRefresh: function( /*XMLDocument*/newState, /*XMLDocument*/currentState ) {
                    ibm.portal.debug.entry( "Portlet._needsRefresh", [ newState, currentState ] );
                    var retVal = true;
                    
                    if ( currentState != null && newState != null ) {
                        retVal = ( dojox.data.dom.innerXML( currentState ) != dojox.data.dom.innerXML( newState ) );
                    }
                    ibm.portal.debug.exit( "Portlet._needsRefresh", [ retVal ] );
                    return retVal;
                },
                destroy: function () { 
                    var _method = "destroy";
                    ibm.portal.debug.entry( _method );
                	
                	for ( mode in this._restorePoints ) {
                    	if ( this._restorePoints.hasOwnProperty( mode ) ) {
                    		this._cleanupJavascript( mode );
                    	}	
                    }
                	
                	ibm.portal.debug.exit( _method );
                }
              }
);

dojo.declare( "com.ibm.portal.aggregation.PortletMarkup",
              com.ibm.aggregation.FragmentMarkup,
              {
                // summary: The portlet markup must be handled specially when used in client-side
                //      aggregation. This implementation is responsible for making all necessary 
                //      changes.
                // description: Handles inserting the portlet markup into the current page. The 
                //      javascript handler is responsible for handling any <script> elements in 
                //      the portlet markup. When the markup is complete, the the portlet markup event
                //      is fired to allow for any further modifications that must occur.
                //          /portal/navigation/portlet/markup - portlet markup update is complete    
                constructor: function () {
                    this._onloadHandler = new com.ibm.portal.aggregation.event.markup.OnloadRewriter();
                },
                //_jsHandler: com.ibm.portal.aggregation.javascript.JAVASCRIPT_HANDLER,
                _jsHandler: com.ibm.portal.aggregation.javascript.ASYNC_JAVASCRIPT_HANDLER,
                prepareUpdate: function ( /*HTMLElement*/elementToUpdate, /*String*/markup, /*Boolean*/append, /*String*/portletId ) {
                	// summary: Update the specified element with the given markup.
                    // elementToUpdate: HTMLElement - the element to update
                    // markup: the markup string.
                    var _method = "PortletMarkup.prepareUpdate";
                	ibm.portal.debug.entry( _method, [ elementToUpdate, markup, append, portletId ] );
                    // clean up any forms that were submitted before in case we have a custom update handler that doesn't
                    // update the entire portlet but appends markup or something; if we don't do this, those forms won't
                    // work anymore
                    dojo.query("form", elementToUpdate).forEach(
                    	function(formElem) {
                    		if(formElem.submitted) formElem.submitted = false;
                    	}
                    );
                    var markupMap = this.prepareMarkup( markup );
                    
                    var args = {portletId: portletId, append: append, markupMap: markupMap, elementToUpdate: elementToUpdate};
                    args = this.prepareScripts(args);
                   
                    ibm.portal.debug.exit( _method );
                    return args;
                },
                update: function ( args ) {
                    // summary: Default update handler to update the specified element with the given markup.
                    // args: Object - Map specifying the following properties.
                    //		
                    //	-	portletId: String - portletId
                    // 	- 	elementToUpdate: HTMLElement - the element to update
                    //  -	append: Boolean - True if the markup should be appended to the existing markup, false to replace.
                    // 	-	markupMap: Object - contains these properties:
                    //		-		node: A HTMLElement containing the filtered markup to insert
                    // 		-		objects: OBJECT elements stripped from the markup to be added by the 
                    // 		-			insertMarkup function
                    var _method = "PortletMarkup.update";
                    ibm.portal.debug.entry( _method, [ args ] );
                	
                	if ( !args.append ) {
                        args.elementToUpdate.innerHTML = "";
                    }
                    this.prepareOnloadHandlers( args );
                    this.insertMarkup( args );
                    this.runScripts( args );
                    
                    ibm.portal.debug.exit( _method );
                },
                prepareScripts: function( args ) {
                	var _method = "PortletMarkup.prepareScripts";
                	ibm.portal.debug.entry( _method, [ args ] );
                	
                	if(args.scriptElems) return;
                    args._CSAScripts = new Array();
                    // find the script elements
                    args.scriptElems = args.markupMap.node.getElementsByTagName( "script" );
                    if ( args.scriptElems != null ) {
                        // iterate over script elements and attach an id attribute
                        for ( var i = 0; i < args.scriptElems.length; i++ ) {
                            var scriptElem = args.scriptElems[i];
                            // include an id if there is none so far
                            // (needed to lookup script in original DOM later on)
                            var id = scriptElem.getAttribute( "id" );
                            if (id == null || id == "" ) {
                                scriptElem.setAttribute( "id", "_scr#" + i );
                            }
                            
                            if(scriptElem.getAttribute("type") == "ibmCSA/portlet") {
                            	args._CSAScripts.push(scriptElem);
                            	scriptElem.parentNode.removeChild(scriptElem);
                            	i--;
                            }
                        }
                    }
                    
                    ibm.portal.debug.exit( _method );
                    return args;
                },
                prepareMarkup: function( markup ) {
                    var _method = "PortletMarkup.prepareMarkup";
                    ibm.portal.debug.entry( _method, [ markup ] );
                	
                	// parse the markup into' a temporary div element
                    var retVal = com.ibm.portal.utilities.html.createTemporaryMarkupDiv( markup );
                    
                    ibm.portal.debug.exit( _method, retVal );
                    return retVal;
                },
                prepareOnloadHandlers: function( args ) {
                	var _method = "PortletMarkup.prepareOnloadHandlers";
                	ibm.portal.debug.entry( _method, [ args ] );
                	// find and rewrite all onload handlers to run them later
                    this._onloadHandler.process( args.markupMap.node );
                    ibm.portal.debug.exit( _method );
                },
                insertMarkup: function( args ) {
                	var _method = "PortletMarkup.insertMarkup";
                	ibm.portal.debug.entry( _method, [ args ] );
                	
                	// update the DOM before executing the scripts
                    // (the scripts might rely on elements in the DOM)
                    args.elementToUpdate.innerHTML = args.markupMap.node.innerHTML;
                    if(dojo.isIE) {
                    	com.ibm.portal.utilities.html.replaceObjectElementsInMarkup(args.markupMap.objects);
                    }
                    
                    ibm.portal.debug.exit( _method );
                },
                runScripts: function( args ) {
                	var _method = "PortletMarkup.runScripts";
                	ibm.portal.debug.entry( _method, [ args ] );
                	
                	com.ibm.portal.EVENT_BROKER.startScriptHandling.fire( { portletId: args.portletId } );
                    
                	var scriptElems = args.scriptElems;
                	var portletId = args.portletId;
                	var elementToUpdate = args.elementToUpdate;
                	var markupMap = args.markupMap;

                	// execute the scripts
                    if ( scriptElems != null ) {
                    	var me = this;
                        ibm.portal.debug.text( "There are " + scriptElems.length + " script elements in this markup.", _method );
                        var scriptHandler = new com.ibm.portal.aggregation.javascript.AsyncPortletScriptElementsHandler( scriptElems, portletId, this._jsHandler );
                        scriptHandler.stopOnError = true;
                        scriptHandler.setEventHandler( {
                        	error: function ( error ) { 
                        		if ( !this._firstErrorLogged ) {
                        			elementToUpdate.innerHTML = error.message;
                        			this._firstErrorLogged = true;
                        		}
                        		else {
                        			elementToUpdate.innerHTML += error.message;
                        		}	
                        		//elementToUpdate.innerHTML = "<div style=\"color: red; font-weight: bold;\">" + error.message + "</div>" + elementToUpdate.innerHTML;
                        	},
                        	scriptDone: function () { },
                        	completed: function () { 
                        		ibm.portal.debug.entry( "ScriptEventHandler.completed" );
                        		// finally destroy the temporary div element
			                    if ( markupMap ) {
			                    	delete tempDiv;
			                    	delete markupMap.node;
			                    	delete markupMap.objects;
			                    }
			
			                    // now run the onload handlers that were detected before
			                    me.runOnloadHandlers( args );
			                    
			                    com.ibm.portal.EVENT_BROKER.endScriptHandling.fire( { portletId: portletId } );
			                    
			                    // flag the element as loaded.
			                    me.finalizeMarkup( args );
			                    
			                    if ( args && args.onScriptCompleted ) {
			                    	args.onScriptCompleted();
			                    }	
                        		
                        		ibm.portal.debug.exit( "ScriptEventHandler.completed" );
                        	},
                        	started: function () { }
                        } );
                        scriptHandler.start();
                    }
                    
                    ibm.portal.debug.exit( _method );
                },
                runOnloadHandlers: function( args ) {
                    var _method = "PortletMarkup.runOnloadHandlers";
                    ibm.portal.debug.entry( _method, [ args ] );
                	// now run the onload handlers that were detected before
                    this._onloadHandler.runHandlers();
                    ibm.portal.debug.exit( _method );
                },
                finalizeMarkup: function( args ) {
                	var _method = "PortleMarkup.finalizeMarkup";
                	ibm.portal.debug.entry( _method, [ args ] );
                	
                	if(args.node) delete args.node;
                	
                    // flag the element as loaded.
                    args.elementToUpdate.setAttribute( "loaded", "true" );
                    this._firePortletMarkupUpdated( args.elementToUpdate );
                    ibm.portal.debug.exit( _method );
                },
                _firePortletMarkupUpdated: function ( markupRootElement ) {
                    dojo.publish( "/portal/PortletMarkup/updated", [ {newMarkup: markupRootElement} ] );
                }
            }   
);




/**
 * Layout Manager reads the atom feed and creates the layout tree in the form of a hidden div.
 * The PortletMove Controller uses this hidden div as a reference layout tree to show the move menu.
 */
dojo.declare( "com.ibm.portal.aggregation.LayoutManager", 
    null,
    {
        constructor: function() {  
            ibm.portal.debug.entry("LayoutManager.constructor",[]);
            this._id = "layoutmanager";
            this.moveWithinContainer = null;
            this.targetPortletId = null;
            this._lmgrPrefix = "lmgr";
            this._hiddenDivId = "wptheme-lmgr-root";
            this._hiddenDiv = null; //needs to be created after initial html page has loaded, so we create it lazily
            this._xsltURL = dojo.moduleUrl( "com", "ibm/portal/");
            this._xslt = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer(this._xsltURL + "LayoutManager.xsl");
            this._xsltParams = { "lmgrPrefix": this._lmgrPrefix };
            ibm.portal.debug.exit("LayoutManager.constructor",[this]);
        },
        setLayoutModel: function(xmlData){
            if (this._hiddenDiv == null){ this._createHiddenDiv();} //lazy creation of div
            this._xslt.transformToRegion( xmlData, this._xsltParams, this._hiddenDiv, document );
			com.ibm.portal.utilities.html.removeNodesOnCondition(this._hiddenDiv, function(node) {
				if(node && node.nodeType != 1) return true;
				else return false;
			});
        }, 
        getAssignedSkinId: function(/*String*/ fullPortletId) {
            ibm.portal.debug.entry("LayoutManager.getAssignedSkin",[fullPortletId]);
            // modify the portletId to match layoutmanager DOM prefixing        
            var prefixedFullPortletId = this._lmgrPrefix+"dndSrc"+fullPortletId;
            // find the control node for this portlet
            var control = dojo.byId(prefixedFullPortletId);
            // see if a skin is assigned (if not, then it has the default skin)  
            var result = null;
            if (control != null) {
                ibm.portal.debug.text("LayoutManager.getAssignedSkin: found control: "+dojox.data.dom.innerXML(control));
                result = control.getAttribute("skin");
            }
            ibm.portal.debug.exit("LayoutManager.getAssignedSkin",[result]);
            return result;
    
        },

        _createHiddenDiv: function() {
            //create the container div for all of model information       
            var div = document.createElement( "DIV" );
            div.style.display = "none";
            div.setAttribute( "id", "wptheme-lmgr-root" ); 
            dojo.body().appendChild( div );
            this._hiddenDiv = div;
            return div;
        },

        _getRootContainerDiv: function() {
            var rootDiv = document.getElementById(this._hiddenDivId).childNodes[0];
            return rootDiv;
        },      

        _getHiddenDivId: function() {
            window.console.log("_getHiddenDivId: this._hiddenDivId: " + this._hiddenDivId);
            return this._hiddenDivId;
        },      

        _getHiddenDiv: function() {
            window.console.log("_getHiddenDiv: this._hiddenDiv: " + this._hiddenDiv);
            return this._hiddenDiv;
        },      

        _hasChildcontainers: function(container) {
            if(container.hasChildNodes){
                if(container.lastChild.getAttribute("containertype") == "C"){
                    return false;
                }
                else{
                    return true;
                }
            }
            else{
                return false;
            }
        },

//Update the LayoutManager div after deleting a portlet on a page.      
        deletePortletInLayoutManager: function(selectedSourceId) {
            var lmgrPrefix = "lmgrdndSrc";
            
            var parent = document.getElementById(lmgrPrefix+selectedSourceId).parentNode;
            parent.removeChild(document.getElementById(lmgrPrefix+selectedSourceId));
        },

//Update the LayoutManager div after moving a portlet on a page.        
        movePortletInLayoutManager: function(selectedSourceId, tgtPortletId, containerId) {
            var lmgrPrefix = "lmgr";
            
            if(tgtPortletId != "emptyDZ"){
                var parent = document.getElementById(lmgrPrefix+tgtPortletId).parentNode;   
                parent.insertBefore(document.getElementById(lmgrPrefix+selectedSourceId), document.getElementById(lmgrPrefix+tgtPortletId));
            }
            else{
                var parent = document.getElementById(lmgrPrefix+containerId);   
                parent.appendChild(document.getElementById(lmgrPrefix+selectedSourceId));               
            }
        },
        

//Update the LayoutManager div after dragging a portlet from a palette to a page.       
        addControl: function(nextControlId, id, title, containerId, pageId, skinId) {
            var lmgrPrefix = "lmgr";
            if(nextControlId == ""){
                var newPortletDivElement = document.createElement("div");
                newPortletDivElement.setAttribute("id", lmgrPrefix+'dndSrc'+id);
                newPortletDivElement.setAttribute("containertype", "C");
                newPortletDivElement.setAttribute("skin", skinId);
                newPortletDivElement.setAttribute("portlettitle", title);

                var parent = document.getElementById(lmgrPrefix+'lm:oid:'+containerId+'@oid:'+pageId);  
                parent.appendChild(newPortletDivElement);               
            }
            else{
                var newPortletDivElement = document.createElement("div");
                newPortletDivElement.setAttribute("id", lmgrPrefix+'dndSrc'+id);
                newPortletDivElement.setAttribute("containertype", "C");
                newPortletDivElement.setAttribute("skin", skinId);
                newPortletDivElement.setAttribute("portlettitle", title);

                var parent = document.getElementById(lmgrPrefix+'dndSrclm:oid:'+nextControlId+'@oid:'+pageId).parentNode;
                parent.insertBefore(newPortletDivElement, document.getElementById(lmgrPrefix+'dndSrclm:oid:'+nextControlId+'@oid:'+pageId));
            }

        }
        
    }
);      

/**
 * PortletMoveController controls the move menu for the portlet in the skin.
 * If a move target id is found, the appropriate move direction is a loaded into the skin along with the target id.
 * If not, the direction is not shown.
 */

dojo.declare( "com.ibm.portal.aggregation.PortletMoveController", 
    null, 
    {
        constructor: function() {  
            this._id = "portletmovecontroller";
            this._lmgrPrefix = "lmgr";
        },
        _getRootContainerDiv: function() {
            var rootDiv = document.getElementById(com.ibm.portal.aggregation.PORTAL_AGGREGATOR.page.layoutManager._getHiddenDivId()).firstChild;
            return rootDiv;
        },      

        _hasChildContainers: function(container) {
            ibm.portal.debug.entry ("_hasChildContainers");
            if(container.hasChildNodes){
                if(container.firstChild != null){
                    if(container.firstChild.getAttribute("containertype") != null && container.firstChild.getAttribute("containertype") == "C"){
                        return false;
                    }
                    else{
                        return true;
                    }
                }
                else{
                    return false;
                }
            }
            else{
                return false;
            }
        },
            
        _getPreviousSiblingContainerMoveId: function(container) {
            ibm.portal.debug.entry( "_getPreviousSiblingContainerMoveId" );
            var firstUnlockedContainerId = null;
            var sibling = container.previousSibling;

//          for(var i = 0; i < container.parentNode.childNodes.length; i++){
            while(sibling != null){             
                if(sibling != null && sibling.getAttribute && sibling.getAttribute("modifiableFlag") == "true"){
                    if(this._hasChildContainers(sibling)){
                        firstUnlockedContainerId = this._getContainerMoveId(sibling);
                        if(firstUnlockedContainerId == null){
                            sibling = sibling.previousSibling;
                        }
                    }
                    else{
                        firstUnlockedContainerId = sibling.getAttribute("id");
                    }
                }
                else{
                    sibling = sibling.previousSibling;
                }
                if(firstUnlockedContainerId != null){break}
            }
            
            return firstUnlockedContainerId;
        },      

        _getNextSiblingContainerMoveId: function(container) {
            ibm.portal.debug.entry( "_getNextSiblingContainerMoveId" );
            var firstUnlockedContainerId = null;
            var sibling = container.nextSibling;

//          for(var i = 0; i < container.parentNode.childNodes.length; i++){
            while(sibling != null){             
                if(sibling != null && sibling.getAttribute && sibling.getAttribute("modifiableFlag") == "true"){
                    if(this._hasChildContainers(sibling)){
                        firstUnlockedContainerId = this._getContainerMoveId(sibling);
                        if(firstUnlockedContainerId == null){
                            sibling = sibling.nextSibling;
                        }
                    }
                    else{
                        firstUnlockedContainerId = sibling.getAttribute("id");
                    }
                }
                else{
                    sibling = sibling.nextSibling;
                }

                if(firstUnlockedContainerId != null){break;}
            }
            
            return firstUnlockedContainerId;
        },      


        _getContainerMoveId: function(container) {
            ibm.portal.debug.entry ("_getContainerMoveId");
            if(container.hasChildNodes && container.firstChild.getAttribute("containertype") != "C"){
                var firstUnlockedContainerId = null;
                var child = null;
                var i = 0;
                for(child = container.firstChild; i < container.childNodes.length; i++){
                    if(child.getAttribute("modifiableFlag") == "true"){
                        if(this._hasChildContainers(child)){
                            firstUnlockedContainerId = this._getContainerMoveId(child);
                        }
                        else{
                            firstUnlockedContainerId = child.getAttribute("id");
                        }
                        if(firstUnlockedContainerId == null){
                            child = child.nextSibling;
                        }
                    }
                    else{
                        child = child.nextSibling;
                    }
                    if(firstUnlockedContainerId != null){break}
                }
                return firstUnlockedContainerId;
            }
            else{
                return container.getAttribute("id");
            }
        },


        _getLeftUCHorUpUCVneighbour: function(id){
            ibm.portal.debug.entry ("_getLeftUCHorUpUCVneighbour");

            var portletDiv = document.getElementById(this._lmgrPrefix+id);
            var parentContainer1 = portletDiv.parentNode;
            var parentContainer2 = parentContainer1.parentNode;
            var parentContainer3 = parentContainer2.parentNode;
            var parentContainer4 = parentContainer3.parentNode;

            if(parentContainer1.getAttribute("modifiableFlag") == "false"){
                return null;
            }
            else{
                if(parentContainer1.firstChild != portletDiv){
                    return portletDiv.previousSibling.getAttribute("id");
                }
                else{
                    if(parentContainer2 != null && parentContainer2 == this._getRootContainerDiv()){
                        return null;
                    }
                    else{
//                      var targetId = null;

                        while(parentContainer2 != null && parentContainer3 != null)
                        {
                            if(parentContainer3 == this._getRootContainerDiv() || (parentContainer4 != null && parentContainer4 == this._getRootContainerDiv())){
                                if(parentContainer3.firstChild != parentContainer2){
                                    if(parentContainer2.previousSibling != null ){
                                        return this._getPreviousSiblingContainerMoveId(parentContainer2);
//                                      targetId = this._getPreviousSiblingContainerMoveId(parentContainer2);
                                    }
                                    else{
                                        return null;
//                                      targetId = null;
                                    }
                                }
                                else{
                                    return null;
//                                  targetId = null;
                                }
                            }
                            else{
                                parentContainer2 = parentContainer2.parentNode;
                                if(parentContainer2 != null) parentContainer2 = parentContainer2.parentNode;
                                if(parentContainer2 != null) parentContainer3 = parentContainer2.parentNode;
                                if(parentContainer3 != null) parentContainer4 = parentContainer3.parentNode;
                                
                            }
                        }
                    }
                }
            }
            
        },


//same as _getUpNeighbourForUCH
//Left UCV

        _getLeftUCVorUpUCHNeighbour: function(id){
            ibm.portal.debug.entry ("_getLeftUCVorUpUCHNeighbour");

            var portletDiv = document.getElementById(this._lmgrPrefix+id);
            var parentContainer1 = portletDiv.parentNode;
            var parentContainer2 = parentContainer1.parentNode;
            var parentContainer3 = parentContainer2.parentNode;
            var parentContainer4 = parentContainer3.parentNode;
            var parentContainer5 = parentContainer4.parentNode;

            if(parentContainer1.getAttribute("modifiableFlag") == "false"){
                return null;
            }
            else{
                if((parentContainer2 != null && parentContainer2 == this._getRootContainerDiv()) || (parentContainer3 != null && parentContainer3 == this._getRootContainerDiv())){
                    if(parentContainer2.firstChild == parentContainer1){
                        return null;
                    }
                    else{
//TODO look for previous sibling unlocked container
                        if(parentContainer1.previousSibling!= null){
                            return this._getPreviousSiblingContainerMoveId(parentContainer1);
                        }
                        else{
                            return null;
                        }
                    }
                }
                else{
                    while(parentContainer4 != null){
                    if(parentContainer4 == this._getRootContainerDiv() || (parentContainer5 != null && parentContainer5 == this._getRootContainerDiv())){
                        if(parentContainer2.firstChild == parentContainer1){
                            if(parentContainer4.firstChild == parentContainer3){
                                return null;
                            }
//TODO look for previous sibling unlocked container
                            else if(parentContainer3.previousSibling!= null){
                                return this._getPreviousSiblingContainerMoveId(parentContainer3);
                            }
                        }
                        else{
                            if(parentContainer1.previousSibling!= null ){
                                var id = this._getPreviousSiblingContainerMoveId(parentContainer1);
                                if(id != null){
                                    return id;
                                } 
                                else{
//TODO look for previous sibling unlocked container
                                    if(parentContainer3.previousSibling!= null){                                    
                                        return this._getPreviousSiblingContainerMoveId(parentContainer3);
                                    }
                                }
                            }
                            else{
                                return null;
                            }
                        }
                    }
                    else{
                        parentContainer4 = parentContainer4.parentNode;
                        if(parentContainer4 != null) parentContainer4 = parentContainer4.parentNode;
                        if(parentContainer4 != null) parentContainer5 = parentContainer5.parentNode;
                    }
                }
                }
            }
        },

        _isValidLeftMove: function(id) {
            ibm.portal.debug.entry ("_isValidLeftMove: dojo.byId(this._lmgrPrefix+id): " + document.getElementById(this._lmgrPrefix+id));
            var portletDiv = document.getElementById(this._lmgrPrefix+id);
            var parentContainer1 = portletDiv.parentNode;

            if(parentContainer1.getAttribute("containertype") == "UCH"){
//              return this._getPreviousNeighbourForUCH(id);
                return this._getLeftUCHorUpUCVneighbour(id);
            }
            else if(parentContainer1.getAttribute("containertype") == "UCV"){
//              return this._getPreviousNeighbourForUCV(id);
                return this._getLeftUCVorUpUCHNeighbour(id);

            }
        },

        _isValidUpMove: function(id) {
            ibm.portal.debug.entry ("_isValidUpMove: dojo.byId(this._lmgrPrefix+id): " + document.getElementById(this._lmgrPrefix+id));
            var portletDiv = document.getElementById(this._lmgrPrefix+id);
            var parentContainer1 = portletDiv.parentNode;

//Previous UCV similar to up UCH            
            if(parentContainer1.getAttribute("containertype") == "UCH"){
//              return this._getPreviousNeighbourForUCV(id);
                return this._getLeftUCVorUpUCHNeighbour(id);
            }
//Previous UCH similar to up UCV            
            else if(parentContainer1.getAttribute("containertype") == "UCV"){
//              return this._getPreviousNeighbourForUCH(id);
                return this._getLeftUCHorUpUCVneighbour(id);
            }
        },


        _getRightUCHorDownUCV: function(id){
            ibm.portal.debug.entry ("_getRightUCHorDownUCV");

            var portletDiv = document.getElementById(this._lmgrPrefix+id);
            var parentContainer1 = portletDiv.parentNode;
            var parentContainer2 = parentContainer1.parentNode;
            var parentContainer3 = parentContainer2.parentNode;
            var parentContainer4 = parentContainer3.parentNode;

            if(parentContainer1.getAttribute("modifiableFlag") == "false"){
                return null;
            }
            else{
                if(parentContainer1.lastChild != portletDiv){
                    return portletDiv.nextSibling.getAttribute("id");
                }
                else{
                    if(parentContainer2 != null && parentContainer2 == this._getRootContainerDiv()){
                        return null;
                    }
                    else{
                        while(parentContainer2 != null && parentContainer3 != null)
                        {
                            if(parentContainer3 == this._getRootContainerDiv() || (parentContainer4 != null && parentContainer4 == this._getRootContainerDiv())){
                                if(parentContainer3.lastChild != parentContainer2){
                                    if(parentContainer2.nextSibling != null){
                                        return this._getNextSiblingContainerMoveId(parentContainer2);
                                    }
                                    else{
                                        return null;
                                    }
                                }
                                else{
                                    return null;
                                }
                            }
                            else{
                                parentContainer2 = parentContainer2.parentNode;
                                
                                if(parentContainer2 != null) parentContainer2 = parentContainer2.parentNode;
                                if(parentContainer2 != null) parentContainer3 = parentContainer2.parentNode;
                                if(parentContainer3 != null) parentContainer4 = parentContainer3.parentNode;
                            }
                        }
                    }
                }
            }
            
        },



        _getRightUCVorDownUCH: function(id){
            ibm.portal.debug.entry ("_getRightUCVorDownUCH");
            
            var portletDiv = document.getElementById(this._lmgrPrefix+id);
            var parentContainer1 = portletDiv.parentNode;
            var parentContainer2 = parentContainer1.parentNode;
            var parentContainer3 = parentContainer2.parentNode;
            var parentContainer4 = parentContainer3.parentNode;
            var parentContainer5 = parentContainer4.parentNode;

            if(parentContainer1.getAttribute("modifiableFlag") == "false"){
                return null;
            }
            else{
                if((parentContainer2 != null && parentContainer2 == this._getRootContainerDiv()) || (parentContainer3 != null && parentContainer3 == this._getRootContainerDiv())){
                    if(parentContainer2.lastChild == parentContainer1){
                        return null;
                    }
                    else{
//TODO look for next sibling unlocked container
                        if(parentContainer1.nextSibling!= null){
                            return this._getNextSiblingContainerMoveId(parentContainer1);
                        }
                        else{
                            return null;
                        }
                    }
                }
                else{
                    while(parentContainer4 != null)
                    {
                    if(parentContainer4 == this._getRootContainerDiv() || (parentContainer5 != null && parentContainer5 == this._getRootContainerDiv())){
                        if(parentContainer2.lastChild == parentContainer1){
                            if(parentContainer4.lastChild == parentContainer3){
                                return null;
                            }
//TODO look for next sibling unlocked container
                            else if(parentContainer3.nextSibling!= null){
                                var id = this._getNextSiblingContainerMoveId(parentContainer3);
                                if(id != null){
                                    return id;
                                } 
//fallback
                                else{
                                    return this._getNextSiblingContainerMoveId(parentContainer3.nextSibling);
                                }
                            }
                        }
                        else{
                            if(parentContainer1.nextSibling!= null ){
                                var id = this._getNextSiblingContainerMoveId(parentContainer1);
                                if(id != null){
                                    return id;
                                } 
                                else{
//TODO look for next sibling unlocked container
                                    if(parentContainer3.nextSibling!= null){                                    
                                        return this._getNextSiblingContainerMoveId(parentContainer3);
                                    }
                                }
                            }
                            else{
                                return null;
                            }
                        }
                    }
                    else{
                        parentContainer4 = parentContainer4.parentNode;

                        if(parentContainer4 != null) parentContainer4 = parentContainer4.parentNode;
                        if(parentContainer4 != null) parentContainer5 = parentContainer5.parentNode;
                    }
                    }
                }
            }
        },

        _isValidRightMove: function(id) {
            ibm.portal.debug.entry ("_isValidRightMove: dojo.byId(this._lmgrPrefix+id): " + document.getElementById(this._lmgrPrefix+id));
            var portletDiv = document.getElementById(this._lmgrPrefix+id);
            var parentContainer1 = portletDiv.parentNode;

            if(parentContainer1.getAttribute("containertype") == "UCH"){
//              return this._getNextNeighbourForUCH(id);
                return this._getRightUCHorDownUCV(id);
            }
            else if(parentContainer1.getAttribute("containertype") == "UCV"){
//              return this._getNextNeighbourForUCV(id);
                return this._getRightUCVorDownUCH(id);
            }
        },


        _isValidDownMove: function(id) {
            ibm.portal.debug.entry ("_isValidDownMove: dojo.byId(this._lmgrPrefix+id): " + document.getElementById(this._lmgrPrefix+id));
            var portletDiv = document.getElementById(this._lmgrPrefix+id);
            var parentContainer1 = portletDiv.parentNode;

//Next UCH similar to down UCV          
            if(parentContainer1.getAttribute("containertype") == "UCV"){
//              return this._getNextNeighbourForUCH(id);
                return this._getRightUCHorDownUCV(id);
            }
//Next UCV similar to down UCH          
            else if(parentContainer1.getAttribute("containertype") == "UCH"){
//              return this._getNextNeighbourForUCV(id);
                return this._getRightUCVorDownUCH(id);
            }
        },


        _findAndMovePortlet: function(sourceContainer, movePortletId, targetId, direction) {
            ibm.portal.debug.entry( "_findAndMovePortlet" );

            ibm.portal.debug.text ("sourceContainer:" + sourceContainer);
            ibm.portal.debug.text ("movePortletId:" + movePortletId);
            ibm.portal.debug.text ("direction:" + direction);
            ibm.portal.debug.text ("targetId:" + targetId);

//Move Left
            if(direction == "left"){
//if move within Container
                if(document.getElementById(targetId).getAttribute("containertype") == "C"){
                    window.console.log("_findAndMovePortlet: targetId.substring(4) " +targetId.substring(4));
                    CSADND_alternateSubmitFunction(sourceContainer.id, targetId.substring(4), movePortletId);
                    
                }
                else{
                    CSADND_alternateSubmitFunction(targetId.substring(4), "emptyDZ", movePortletId);
                }
            }
            else if(direction == "right"){              
                if(document.getElementById(targetId).getAttribute("containertype") == "C"){
                    //loop through the portlets(ignoring dropzones) and find the movePortlet
                    //if the move portlet is last but one, send emptyDZ for targetPortletID
                    var i = sourceContainer.childNodes[0].childNodes[0].childNodes.length;
                    for (var j = 1; j < i; j=j+2) {
                        if(sourceContainer.childNodes[0].childNodes[0].childNodes[j].id == movePortletId){
                            //check to see if if it is the one before last, 
                            //then need to send emptyDZ for targetPortletID
                            if(sourceContainer.childNodes[0].childNodes[0].childNodes[i-4].id == movePortletId){
                                CSADND_alternateSubmitFunction(sourceContainer.id, "emptyDZ", movePortletId);
                                break;
                            }
                            else{
                                CSADND_alternateSubmitFunction(sourceContainer.id, sourceContainer.childNodes[0].childNodes[0].childNodes[j+4].id, movePortletId);
                                break;                                  
                            }
                        }                   
                    }

                }
                else{
                    CSADND_alternateSubmitFunction(targetId.substring(4),"emptyDZ" , movePortletId);
                }
            }
            else if(direction == "up"){
//if move within Container
                if(document.getElementById(targetId).getAttribute("containertype") == "C"){
                    ibm.portal.debug.text("_findAndMovePortlet: targetId.substring(4) " +targetId.substring(4));
                    CSADND_alternateSubmitFunction(sourceContainer.id, targetId.substring(4), movePortletId);
                }
                else{
                    CSADND_alternateSubmitFunction(targetId.substring(4), "emptyDZ", movePortletId);
                }
            }
            else if(direction == "down"){               
                if(document.getElementById(targetId).getAttribute("containertype") == "C"){
                    var i = sourceContainer.childNodes[0].childNodes.length;
                        //loop through the portlets(ignoring dropzones) and find the movePortlet
                        //if the move portlet is last but one, send emptyDZ for targetPortletID
                        for (var j = 1; j < i; j=j+2) {
                            if(sourceContainer.childNodes[0].childNodes[j].childNodes[0].id == movePortletId){
                                if(sourceContainer.childNodes[0].childNodes[i-4].childNodes[0].id == movePortletId){
                                    CSADND_alternateSubmitFunction(sourceContainer.id, "emptyDZ", movePortletId);
                                    break;                                  
                                }
                                else{
                                    CSADND_alternateSubmitFunction(sourceContainer.id, sourceContainer.childNodes[0].childNodes[j+4].childNodes[0].id, movePortletId);
                                    break;                                  
                                }
                            }                   
                        }
                }
                else{
                    CSADND_alternateSubmitFunction(targetId.substring(4),"emptyDZ" , movePortletId);
                }
            }
        }
    }

);

com.ibm.portal.aggregation.PORTAL_AGGREGATOR = new com.ibm.portal.aggregation.PortalAggregator();

}

if(!dojo._hasResource["com.ibm.portal.debug.NewWindow"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.debug.NewWindow"] = true;
dojo.provide("com.ibm.portal.debug.NewWindow");

dojo.require( "dojo.dnd.common" );
dojo.require( "com.ibm.portal.debug" );

/* create the new window for debug statements */
if ( djConfig["openDebugWindowOnDemand"] == true ) {
	com.ibm.portal.debug._newWindowObj = null;	
}
else {
	com.ibm.portal.debug._newWindowObj = self.open( "about:blank", "_blank", "toolbar=no,location=no,menubar=yes,titlebar=no,scrollbars=yes" );
}

/* track the number of statements we've logged so we can alternate the background colors, making the debug statements easier to read */
com.ibm.portal.debug._statementCount = 0;

com.ibm.portal.debug.NewWindow.startLogging = function () {
	/*
	 * summary: Opens the new browser window which will serve as the debug console, if necessary.
	 * 
	 * description: If the new browser window to be used as the debug console has not been opened
	 * yet, this function will create and open this window.
	 */
	if ( com.ibm.portal.debug._newWindowObj == null ) {
		com.ibm.portal.debug._newWindowObj = self.open( "about:blank", "_blank", "toolbar=no,location=no,menubar=yes,titlebar=no,scrollbars=yes" );
	}	
}

com.ibm.portal.debug.NewWindow.endLogging = function () {
	/*
	 * summary: Closes the new browser window which contains the debugging statements.
	 */
	if ( com.ibm.portal.debug._newWindowObj != null ) {
		com.ibm.portal.debug._newWindowObj.close();
	}
}

com.ibm.portal.debug.NewWindow.isOpen = function () {
	/*
	 * summary: Indicates whether the debug window is currently open.
	 */
	return ( com.ibm.portal.debug._newWindowObj != null );
}

com.ibm.portal.debug.NewWindow.println = function ( /*String*/line ) {
	/*
	 * summary: Prints a string to a new window.
	 *
	 * description: Writes a given string to the new window's document. This allows for debugging statements to be 
	 * tracked in a separate browser window from the browser window in which the code is being executed. This seems 
	 * to be cleaner when having to copy and paste code or save "logs" using the debug console widget or the firebug 
	 * console. To use the new window debugging, simply add the following code to your html file:
	 *
	 *						dojo.requires( "com.ibm.portal.debug.NewWindow" );
	 *
	 * line: String				The text to write to the new window's document.
	 *
	 */
	 
	if ( com.ibm.portal.debug._newWindowObj == null ) return;
	
	try {
		var doc = com.ibm.portal.debug._newWindowObj.document;
		
		//Setting the title initially doesn't seem to work in Firefox. So do it here. 
		//Note: This isn't necessary, it really just makes it easier to find the debug window from many open 
		//	    browser windows
		if ( doc.title != "Debug Console" ) {
			doc.title = "Debug Console";
		}
		
		var console = doc.getElementsByTagName("body")[0] || doc.body;
		
		var unique = dojo.dnd.getUniqueId();
		var unique2 = dojo.dnd.getUniqueId();
			
		var div = doc.createElement("div");
		var div2 = doc.createElement( "div" );
		div.innerHTML = "<a id=\"" + unique + "\" href=\"javascript:void(0);\" onclick=\"document.getElementById('"+ unique2 + "').focus()\" >End</a>";
		div2.appendChild(doc.createTextNode(line))
		div.appendChild( div2 );
		div.innerHTML += "<a id=\"" + unique2 + "\" href=\"javascript:void(0);\" onclick=\"document.getElementById('"+ unique + "').focus()\">Top</a>";		
		
		//Every other statement that is logged, change the background color to make it easier to see where each 
		//statement begins and ends. This is useful when large amounts of information are being logged in a single 
		//statement.
		if ( com.ibm.portal.debug._statementCount % 2 == 0 ) {
			div.style.backgroundColor = "#CCCCCC";
		}
		
		console.appendChild(div);
		
		var hr = doc.createElement( "hr" );
		console.appendChild( hr );
	} catch (e) {
		try{
			// safari needs the output wrapped in an element for some reason
			doc.write("<div>" + line + "</div>");
		}catch(e2){
			window.status = line;
		}
	}
	
	com.ibm.portal.debug._statementCount++;
}

// overwriting the console doesn't always seem to work, so just overwrite the
// ibm.portal.debug.text function instead

// window.console.log = com.ibm.portal.debug.NewWindow.println;
ibm.portal.debug.text = function( str, functionName ) {
	com.ibm.portal.debug.NewWindow.println(str);
}

dojo.addOnUnload( function () { 
	if ( com.ibm.portal.debug._newWindowObj == null || ( com.ibm.portal.debug._newWindowObj && com.ibm.portal.debug._newWindowObj.closed ) ) return; 
	var val = confirm( "Do you want to close the debug window?"); 
	if ( val ) { 
		com.ibm.portal.debug._newWindowObj.close() 
	} 
} ); 

}

if(!dojo._hasResource["dijit._base.focus"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.focus"] = true;
dojo.provide("dijit._base.focus");

// summary:
//		These functions are used to query or set the focus and selection.
//
//		Also, they trace when widgets become actived/deactivated,
//		so that the widget can fire _onFocus/_onBlur events.
//		"Active" here means something similar to "focused", but
//		"focus" isn't quite the right word because we keep track of
//		a whole stack of "active" widgets.  Example:  Combobutton --> Menu -->
//		MenuItem.   The onBlur event for Combobutton doesn't fire due to focusing
//		on the Menu or a MenuItem, since they are considered part of the
//		Combobutton widget.  It only happens when focus is shifted
//		somewhere completely different.

dojo.mixin(dijit,
{
	// _curFocus: DomNode
	//		Currently focused item on screen
	_curFocus: null,

	// _prevFocus: DomNode
	//		Previously focused item on screen
	_prevFocus: null,

	isCollapsed: function(){
		// summary:
		//		Returns true if there is no text selected
		var _document = dojo.doc;
		if(_document.selection && !dojo.isOpera){ // IE
			var s=_document.selection;
			if(s.type=='Text'){
				return !s.createRange().htmlText.length; // Boolean
			}else{ //Control range
				return !s.createRange().length; // Boolean
			}
		}else{
			var _window = dojo.global;
			var selection = _window.getSelection();
			
			if(dojo.isString(selection)){ // Safari
				// TODO: this is dead code; safari is taking the else branch.  remove after 1.3.
				return !selection; // Boolean
			}else{ // Mozilla/W3
				return !selection || selection.isCollapsed || !selection.toString(); // Boolean
			}
		}
	},

	getBookmark: function(){
		// summary:
		//		Retrieves a bookmark that can be used with moveToBookmark to return to the same range
		var bookmark, selection = dojo.doc.selection;
		if(selection && !dojo.isOpera){ // IE
			var range = selection.createRange();
			if(selection.type.toUpperCase()=='CONTROL'){
				if(range.length){
					bookmark=[];
					var i=0,len=range.length;
					while(i<len){
						bookmark.push(range.item(i++));
					}
				}else{
					bookmark=null;
				}
			}else{
				bookmark = range.getBookmark();
			}
		}else{
			if(window.getSelection){
				selection = dojo.global.getSelection();
				if(selection){
					range = selection.getRangeAt(0);
					bookmark = range.cloneRange();
				}
			}else{
				console.warn("No idea how to store the current selection for this browser!");
			}
		}
		return bookmark; // Array
	},

	moveToBookmark: function(/*Object*/bookmark){
		// summary:
		//		Moves current selection to a bookmark
		// bookmark:
		//		This should be a returned object from dojo.html.selection.getBookmark()
		var _document = dojo.doc;
		if(_document.selection && !dojo.isOpera){ // IE
			var range;
			if(dojo.isArray(bookmark)){
				range = _document.body.createControlRange();
				//range.addElement does not have call/apply method, so can not call it directly
				//range is not available in "range.addElement(item)", so can't use that either
				dojo.forEach(bookmark, function(n){
					range.addElement(n);
				});
			}else{
				range = _document.selection.createRange();
				range.moveToBookmark(bookmark);
			}
			range.select();
		}else{ //Moz/W3C
			var selection = dojo.global.getSelection && dojo.global.getSelection();
			if(selection && selection.removeAllRanges){
				selection.removeAllRanges();
				selection.addRange(bookmark);
			}else{
				console.warn("No idea how to restore selection for this browser!");
			}
		}
	},

	getFocus: function(/*Widget?*/menu, /*Window?*/openedForWindow){
		// summary:
		//		Returns the current focus and selection.
		//		Called when a popup appears (either a top level menu or a dialog),
		//		or when a toolbar/menubar receives focus
		//
		// menu:
		//		The menu that's being opened
		//
		// openedForWindow:
		//		iframe in which menu was opened
		//
		// returns:
		//		A handle to restore focus/selection

		return {
			// Node to return focus to
			node: menu && dojo.isDescendant(dijit._curFocus, menu.domNode) ? dijit._prevFocus : dijit._curFocus,

			// Previously selected text
			bookmark:
				!dojo.withGlobal(openedForWindow||dojo.global, dijit.isCollapsed) ?
				dojo.withGlobal(openedForWindow||dojo.global, dijit.getBookmark) :
				null,

			openedForWindow: openedForWindow
		}; // Object
	},

	focus: function(/*Object || DomNode */ handle){
		// summary:
		//		Sets the focused node and the selection according to argument.
		//		To set focus to an iframe's content, pass in the iframe itself.
		// handle:
		//		object returned by get(), or a DomNode

		if(!handle){ return; }

		var node = "node" in handle ? handle.node : handle,		// because handle is either DomNode or a composite object
			bookmark = handle.bookmark,
			openedForWindow = handle.openedForWindow;

		// Set the focus
		// Note that for iframe's we need to use the <iframe> to follow the parentNode chain,
		// but we need to set focus to iframe.contentWindow
		if(node){
			var focusNode = (node.tagName.toLowerCase()=="iframe") ? node.contentWindow : node;
			if(focusNode && focusNode.focus){
				try{
					// Gecko throws sometimes if setting focus is impossible,
					// node not displayed or something like that
					focusNode.focus();
				}catch(e){/*quiet*/}
			}			
			dijit._onFocusNode(node);
		}

		// set the selection
		// do not need to restore if current selection is not empty
		// (use keyboard to select a menu item)
		if(bookmark && dojo.withGlobal(openedForWindow||dojo.global, dijit.isCollapsed)){
			if(openedForWindow){
				openedForWindow.focus();
			}
			try{
				dojo.withGlobal(openedForWindow||dojo.global, dijit.moveToBookmark, null, [bookmark]);
			}catch(e){
				/*squelch IE internal error, see http://trac.dojotoolkit.org/ticket/1984 */
			}
		}
	},

	// _activeStack: Array
	//		List of currently active widgets (focused widget and it's ancestors)
	_activeStack: [],

	registerIframe: function(/*DomNode*/ iframe){
		// summary:
		//		Registers listeners on the specified iframe so that any click
		//		or focus event on that iframe (or anything in it) is reported
		//		as a focus/click event on the <iframe> itself.
		// description:
		//		Currently only used by editor.
		dijit.registerWin(iframe.contentWindow, iframe);
	},
		

	registerWin: function(/*Window?*/targetWindow, /*DomNode?*/ effectiveNode){
		// summary:
		//		Registers listeners on the specified window (either the main
		//		window or an iframe's window) to detect when the user has clicked somewhere
		//		or focused somewhere.
		// description:
		//		Users should call registerIframe() instead of this method.
		// targetWindow:
		//		If specified this is the window associated with the iframe,
		//		i.e. iframe.contentWindow.
		// effectiveNode:
		//		If specified, report any focus events inside targetWindow as
		//		an event on effectiveNode, rather than on evt.target.

		// TODO: make this function private in 2.0; Editor/users should call registerIframe(),
		// or if Editor stops using <iframe> altogether than we can probably just drop
		// the whole public API.

		dojo.connect(targetWindow.document, "onmousedown", function(evt){
			dijit._justMouseDowned = true;
			setTimeout(function(){ dijit._justMouseDowned = false; }, 0);
			dijit._onTouchNode(effectiveNode||evt.target||evt.srcElement);
		});
		//dojo.connect(targetWindow, "onscroll", ???);

		// Listen for blur and focus events on targetWindow's document.
		// IIRC, I'm using attachEvent() rather than dojo.connect() because focus/blur events don't bubble
		// through dojo.connect(), and also maybe to catch the focus events early, before onfocus handlers
		// fire.
		var doc = targetWindow.document;
		if(doc){
			if(dojo.isIE){
				doc.attachEvent('onactivate', function(evt){
					if(evt.srcElement.tagName.toLowerCase() != "#document"){
						dijit._onFocusNode(effectiveNode||evt.srcElement);
					}
				});
				doc.attachEvent('ondeactivate', function(evt){
					dijit._onBlurNode(effectiveNode||evt.srcElement);
				});
			}else{
				doc.addEventListener('focus', function(evt){
					dijit._onFocusNode(effectiveNode||evt.target);
				}, true);
				doc.addEventListener('blur', function(evt){
					dijit._onBlurNode(effectiveNode||evt.target);
				}, true);
			}
		}
		doc = null;	// prevent memory leak (apparent circular reference via closure)
	},

	_onBlurNode: function(/*DomNode*/ node){
		// summary:
		// 		Called when focus leaves a node.
		//		Usually ignored, _unless_ it *isn't* follwed by touching another node,
		//		which indicates that we tabbed off the last field on the page,
		//		in which case every widget is marked inactive
		dijit._prevFocus = dijit._curFocus;
		dijit._curFocus = null;

		if(dijit._justMouseDowned){
			// the mouse down caused a new widget to be marked as active; this blur event
			// is coming late, so ignore it.
			return;
		}

		// if the blur event isn't followed by a focus event then mark all widgets as inactive.
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
		}
		dijit._clearActiveWidgetsTimer = setTimeout(function(){
			delete dijit._clearActiveWidgetsTimer;
			dijit._setStack([]);
			dijit._prevFocus = null;
		}, 100);
	},

	_onTouchNode: function(/*DomNode*/ node){
		// summary:
		//		Callback when node is focused or mouse-downed

		// ignore the recent blurNode event
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
			delete dijit._clearActiveWidgetsTimer;
		}

		// compute stack of active widgets (ex: ComboButton --> Menu --> MenuItem)
		var newStack=[];
		try{
			while(node){
				if(node.dijitPopupParent){
					node=dijit.byId(node.dijitPopupParent).domNode;
				}else if(node.tagName && node.tagName.toLowerCase()=="body"){
					// is this the root of the document or just the root of an iframe?
					if(node===dojo.body()){
						// node is the root of the main document
						break;
					}
					// otherwise, find the iframe this node refers to (can't access it via parentNode,
					// need to do this trick instead). window.frameElement is supported in IE/FF/Webkit
					node=dijit.getDocumentWindow(node.ownerDocument).frameElement;
				}else{
					var id = node.getAttribute && node.getAttribute("widgetId");
					if(id){
						newStack.unshift(id);
					}
					node=node.parentNode;
				}
			}
		}catch(e){ /* squelch */ }

		dijit._setStack(newStack);
	},

	_onFocusNode: function(/*DomNode*/ node){
		// summary:
		//		Callback when node is focused

		if(!node){
			return;
		}

		if(node.nodeType == 9){
			// Ignore focus events on the document itself.  This is here so that
			// (for example) clicking the up/down arrows of a spinner
			// (which don't get focus) won't cause that widget to blur. (FF issue)
			return;
		}

		dijit._onTouchNode(node);

		if(node==dijit._curFocus){ return; }
		if(dijit._curFocus){
			dijit._prevFocus = dijit._curFocus;
		}
		dijit._curFocus = node;
		dojo.publish("focusNode", [node]);
	},

	_setStack: function(newStack){
		// summary:
		//		The stack of active widgets has changed.  Send out appropriate events and records new stack.

		var oldStack = dijit._activeStack;
		dijit._activeStack = newStack;

		// compare old stack to new stack to see how many elements they have in common
		for(var nCommon=0; nCommon<Math.min(oldStack.length, newStack.length); nCommon++){
			if(oldStack[nCommon] != newStack[nCommon]){
				break;
			}
		}

		// for all elements that have gone out of focus, send blur event
		for(var i=oldStack.length-1; i>=nCommon; i--){
			var widget = dijit.byId(oldStack[i]);
			if(widget){
				widget._focused = false;
				widget._hasBeenBlurred = true;
				if(widget._onBlur){
					widget._onBlur();
				}
				if (widget._setStateClass){
					widget._setStateClass();
				}
				dojo.publish("widgetBlur", [widget]);
			}
		}

		// for all element that have come into focus, send focus event
		for(i=nCommon; i<newStack.length; i++){
			widget = dijit.byId(newStack[i]);
			if(widget){
				widget._focused = true;
				if(widget._onFocus){
					widget._onFocus();
				}
				if (widget._setStateClass){
					widget._setStateClass();
				}
				dojo.publish("widgetFocus", [widget]);
			}
		}
	}
});

// register top window and all the iframes it contains
dojo.addOnLoad(function(){dijit.registerWin(window); });

}

if(!dojo._hasResource["dijit._base.manager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.manager"] = true;
dojo.provide("dijit._base.manager");

dojo.declare("dijit.WidgetSet", null, {
	// summary:
	//		A set of widgets indexed by id. A default instance of this class is 
	//		available as `dijit.registry`
	//
	// example:
	//		Create a small list of widgets:
	//		|	var ws = new dijit.WidgetSet();
	//		|	ws.add(dijit.byId("one"));
	//		| 	ws.add(dijit.byId("two"));
	//		|	// destroy both:
	//		|	ws.forEach(function(w){ w.destroy(); });
	//
	// example:
	//		Using dijit.registry:
	//		|	dijit.registry.forEach(function(w){ /* do something */ });
	
	constructor: function(){
		this._hash = {};
	},

	add: function(/*Widget*/ widget){
		// summary:
		//		Add a widget to this list. If a duplicate ID is detected, a error is thrown.
		//
		// widget: dijit._Widget
		//		Any dijit._Widget subclass.
		if(this._hash[widget.id]){
			throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
		}
		this._hash[widget.id]=widget;
	},

	remove: function(/*String*/ id){
		// summary:
		//		Remove a widget from this WidgetSet. Does not destroy the widget; simply
		//		removes the reference.
		delete this._hash[id];
	},

	forEach: function(/*Function*/ func){
		// summary:
		//		Call specified function for each widget in this set.
		//
		// func:
		//		A callback function to run for each item. Is passed a the widget.
		//
		// example:
		//		Using the default `dijit.registry` instance:
		//		|	dijit.registry.forEach(function(widget){
		//		|		console.log(widget.declaredClass);	
		//		|	});
		for(var id in this._hash){
			func(this._hash[id]);
		}
	},

	filter: function(/*Function*/ filter){
		// summary:
		//		Filter down this WidgetSet to a smaller new WidgetSet
		//		Works the same as `dojo.filter` and `dojo.NodeList.filter`
		//		
		// filter:
		//		Callback function to test truthiness.
		//
		// example:
		//		Arbitrary: select the odd widgets in this list
		//		|	var i = 0;
		//		|	dijit.registry.filter(function(w){
		//		|		return ++i % 2 == 0;
		//		|	}).forEach(function(w){ /* odd ones */ });

		var res = new dijit.WidgetSet();
		this.forEach(function(widget){
			if(filter(widget)){ res.add(widget); }
		});
		return res; // dijit.WidgetSet
	},

	byId: function(/*String*/ id){
		// summary:
		//		Find a widget in this list by it's id. 
		// example:
		//		Test if an id is in a particular WidgetSet
		//		| var ws = new dijit.WidgetSet();
		//		| ws.add(dijit.byId("bar"));
		//		| var t = ws.byId("bar") // returns a widget
		//		| var x = ws.byId("foo"); // returns undefined
		
		return this._hash[id];	// dijit._Widget
	},

	byClass: function(/*String*/ cls){
		// summary:
		//		Reduce this widgetset to a new WidgetSet of a particular declaredClass
		// 
		// example:
		//		Find all titlePane's in a page:
		//		|	dijit.registry.byClass("dijit.TitlePane").forEach(function(tp){ tp.close(); });
		
		return this.filter(function(widget){ return widget.declaredClass==cls; });	// dijit.WidgetSet
	}
	
});

/*=====
dijit.registry = {
	// summary: A list of widgets on a page.
	// description: Is an instance of `dijit.WidgetSet`
};
=====*/
dijit.registry = new dijit.WidgetSet();

dijit._widgetTypeCtr = {};

dijit.getUniqueId = function(/*String*/widgetType){
	// summary: Generates a unique id for a given widgetType

	var id;
	do{
		id = widgetType + "_" +
			(widgetType in dijit._widgetTypeCtr ?
				++dijit._widgetTypeCtr[widgetType] : dijit._widgetTypeCtr[widgetType] = 0);
	}while(dijit.byId(id));
	return id; // String
};

dijit.findWidgets = function(/*DomNode*/ root){
	// summary:
	//		Search subtree under root, putting found widgets in outAry.
	//		Doesn't search for nested widgets (ie, widgets inside other widgets)
	
	var outAry = [];

	function getChildrenHelper(root){
		var list = dojo.isIE ? root.children : root.childNodes, i = 0, node;
		while(node = list[i++]){
			if(node.nodeType != 1){ continue; }
			var widgetId = node.getAttribute("widgetId");
			if(widgetId){
				var widget = dijit.byId(widgetId);
				outAry.push(widget);
			}else{
				getChildrenHelper(node);
			}
		}
	}

	getChildrenHelper(root);
	return outAry;
};

if(dojo.isIE){
	// Only run this for IE because we think it's only necessary in that case,
	// and because it causes problems on FF.  See bug #3531 for details.
	dojo.addOnWindowUnload(function(){
		dojo.forEach(dijit.findWidgets(dojo.body()), function(widget){
			if(widget.destroyRecursive){
				widget.destroyRecursive();
			}else if(widget.destroy){
				widget.destroy();
			}
		});
	});
}

dijit.byId = function(/*String|Widget*/id){
	// summary:
	//		Returns a widget by it's id, or if passed a widget, no-op (like dojo.byId())
	return (dojo.isString(id)) ? dijit.registry.byId(id) : id; // Widget
};

dijit.byNode = function(/* DOMNode */ node){
	// summary:
	//		Returns the widget corresponding to the given DOMNode
	return dijit.registry.byId(node.getAttribute("widgetId")); // Widget
};

dijit.getEnclosingWidget = function(/* DOMNode */ node){
	// summary:
	//		Returns the widget whose DOM tree contains the specified DOMNode, or null if
	//		the node is not contained within the DOM tree of any widget
	while(node){
		if(node.getAttribute && node.getAttribute("widgetId")){
			return dijit.registry.byId(node.getAttribute("widgetId"));
		}
		node = node.parentNode;
	}
	return null;
};

// elements that are tab-navigable if they have no tabindex value set
// (except for "a", which must have an href attribute)
dijit._tabElements = {
	area: true,
	button: true,
	input: true,
	object: true,
	select: true,
	textarea: true
};

dijit._isElementShown = function(/*Element*/elem){
	var style = dojo.style(elem);
	return (style.visibility != "hidden")
		&& (style.visibility != "collapsed")
		&& (style.display != "none")
		&& (dojo.attr(elem, "type") != "hidden");
}

dijit.isTabNavigable = function(/*Element*/elem){
	// summary:
	//		Tests if an element is tab-navigable
	if(dojo.hasAttr(elem, "disabled")){ return false; }
	var hasTabindex = dojo.hasAttr(elem, "tabindex");
	var tabindex = dojo.attr(elem, "tabindex");
	if(hasTabindex && tabindex >= 0) {
		return true; // boolean
	}
	var name = elem.nodeName.toLowerCase();
	if(((name == "a" && dojo.hasAttr(elem, "href"))
			|| dijit._tabElements[name])
		&& (!hasTabindex || tabindex >= 0)){
		return true; // boolean
	}
	return false; // boolean
};

dijit._getTabNavigable = function(/*DOMNode*/root){
	// summary:
	//		Finds descendants of the specified root node.
	//
	// description:
	//		Finds the following descendants of the specified root node:
	//		* the first tab-navigable element in document order
	//		  without a tabindex or with tabindex="0"
	//		* the last tab-navigable element in document order
	//		  without a tabindex or with tabindex="0"
	//		* the first element in document order with the lowest
	//		  positive tabindex value
	//		* the last element in document order with the highest
	//		  positive tabindex value
	var first, last, lowest, lowestTabindex, highest, highestTabindex;
	var walkTree = function(/*DOMNode*/parent){
		dojo.query("> *", parent).forEach(function(child){
			var isShown = dijit._isElementShown(child);
			if(isShown && dijit.isTabNavigable(child)){
				var tabindex = dojo.attr(child, "tabindex");
				if(!dojo.hasAttr(child, "tabindex") || tabindex == 0){
					if(!first){ first = child; }
					last = child;
				}else if(tabindex > 0){
					if(!lowest || tabindex < lowestTabindex){
						lowestTabindex = tabindex;
						lowest = child;
					}
					if(!highest || tabindex >= highestTabindex){
						highestTabindex = tabindex;
						highest = child;
					}
				}
			}
			if(isShown && child.nodeName.toUpperCase() != 'SELECT'){ walkTree(child) }
		});
	};
	if(dijit._isElementShown(root)){ walkTree(root) }
	return { first: first, last: last, lowest: lowest, highest: highest };
}
dijit.getFirstInTabbingOrder = function(/*String|DOMNode*/root){
	// summary:
	//		Finds the descendant of the specified root node
	//		that is first in the tabbing order
	var elems = dijit._getTabNavigable(dojo.byId(root));
	return elems.lowest ? elems.lowest : elems.first; // DomNode
};

dijit.getLastInTabbingOrder = function(/*String|DOMNode*/root){
	// summary:
	//		Finds the descendant of the specified root node
	//		that is last in the tabbing order
	var elems = dijit._getTabNavigable(dojo.byId(root));
	return elems.last ? elems.last : elems.highest; // DomNode
};

/*=====
dojo.mixin(dijit, {
	// defaultDuration: Integer
	//		The default animation speed (in ms) to use for all Dijit
	//		transitional animations, unless otherwise specified 
	//		on a per-instance basis. Defaults to 200, overrided by 
	//		`djConfig.defaultDuration`
	defaultDuration: 300
});
=====*/

dijit.defaultDuration = dojo.config["defaultDuration"] || 200;

}

if(!dojo._hasResource["dojo.AdapterRegistry"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.AdapterRegistry"] = true;
dojo.provide("dojo.AdapterRegistry");

dojo.AdapterRegistry = function(/*Boolean?*/ returnWrappers){
	//	summary:
	//		A registry to make contextual calling/searching easier.
	//	description:
	//		Objects of this class keep list of arrays in the form [name, check,
	//		wrap, directReturn] that are used to determine what the contextual
	//		result of a set of checked arguments is. All check/wrap functions
	//		in this registry should be of the same arity.
	//	example:
	//	|	// create a new registry
	//	|	var reg = new dojo.AdapterRegistry();
	//	|	reg.register("handleString",
	//	|		dojo.isString,
	//	|		function(str){
	//	|			// do something with the string here
	//	|		}
	//	|	);
	//	|	reg.register("handleArr",
	//	|		dojo.isArray,
	//	|		function(arr){
	//	|			// do something with the array here
	//	|		}
	//	|	);
	//	|
	//	|	// now we can pass reg.match() *either* an array or a string and
	//	|	// the value we pass will get handled by the right function
	//	|	reg.match("someValue"); // will call the first function
	//	|	reg.match(["someValue"]); // will call the second

	this.pairs = [];
	this.returnWrappers = returnWrappers || false; // Boolean
}

dojo.extend(dojo.AdapterRegistry, {
	register: function(/*String*/ name, /*Function*/ check, /*Function*/ wrap, /*Boolean?*/ directReturn, /*Boolean?*/ override){
		//	summary: 
		//		register a check function to determine if the wrap function or
		//		object gets selected
		//	name:
		//		a way to identify this matcher.
		//	check:
		//		a function that arguments are passed to from the adapter's
		//		match() function.  The check function should return true if the
		//		given arguments are appropriate for the wrap function.
		//	directReturn:
		//		If directReturn is true, the value passed in for wrap will be
		//		returned instead of being called. Alternately, the
		//		AdapterRegistry can be set globally to "return not call" using
		//		the returnWrappers property. Either way, this behavior allows
		//		the registry to act as a "search" function instead of a
		//		function interception library.
		//	override:
		//		If override is given and true, the check function will be given
		//		highest priority. Otherwise, it will be the lowest priority
		//		adapter.
		this.pairs[((override) ? "unshift" : "push")]([name, check, wrap, directReturn]);
	},

	match: function(/* ... */){
		// summary:
		//		Find an adapter for the given arguments. If no suitable adapter
		//		is found, throws an exception. match() accepts any number of
		//		arguments, all of which are passed to all matching functions
		//		from the registered pairs.
		for(var i = 0; i < this.pairs.length; i++){
			var pair = this.pairs[i];
			if(pair[1].apply(this, arguments)){
				if((pair[3])||(this.returnWrappers)){
					return pair[2];
				}else{
					return pair[2].apply(this, arguments);
				}
			}
		}
		throw new Error("No match found");
	},

	unregister: function(name){
		// summary: Remove a named adapter from the registry

		// FIXME: this is kind of a dumb way to handle this. On a large
		// registry this will be slow-ish and we can use the name as a lookup
		// should we choose to trade memory for speed.
		for(var i = 0; i < this.pairs.length; i++){
			var pair = this.pairs[i];
			if(pair[0] == name){
				this.pairs.splice(i, 1);
				return true;
			}
		}
		return false;
	}
});

}

if(!dojo._hasResource["dijit._base.place"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.place"] = true;
dojo.provide("dijit._base.place");



// ported from dojo.html.util

dijit.getViewport = function(){
	// summary:
	//		Returns the dimensions and scroll position of the viewable area of a browser window

	var scrollRoot = (dojo.doc.compatMode == 'BackCompat')? dojo.body() : dojo.doc.documentElement;

	// get scroll position
	var scroll = dojo._docScroll(); // scrollRoot.scrollTop/Left should work
	return { w: scrollRoot.clientWidth, h: scrollRoot.clientHeight, l: scroll.x, t: scroll.y };
};

/*=====
dijit.__Position = function(){
	// x: Integer
	//		horizontal coordinate in pixels, relative to document body
	// y: Integer
	//		vertical coordinate in pixels, relative to document body

	thix.x = x;
	this.y = y;
}
=====*/


dijit.placeOnScreen = function(
	/* DomNode */			node,
	/* dijit.__Position */	pos,
	/* String[] */			corners,
	/* dijit.__Position? */	padding){
	//	summary:
	//		Positions one of the node's corners at specified position
	//		such that node is fully visible in viewport.
	//	description:
	//		NOTE: node is assumed to be absolutely or relatively positioned.
	//	pos:
	//		Object like {x: 10, y: 20}
	//	corners:
	//		Array of Strings representing order to try corners in, like ["TR", "BL"].
	//		Possible values are:
	//			* "BL" - bottom left
	//			* "BR" - bottom right
	//			* "TL" - top left
	//			* "TR" - top right
	//	padding:
	//		set padding to put some buffer around the element you want to position.
	//	example:	
	//		Try to place node's top right corner at (10,20).
	//		If that makes node go (partially) off screen, then try placing
	//		bottom left corner at (10,20).
	//	|	placeOnScreen(node, {x: 10, y: 20}, ["TR", "BL"])

	var choices = dojo.map(corners, function(corner){
		var c = { corner: corner, pos: {x:pos.x,y:pos.y} };
		if(padding){
			c.pos.x += corner.charAt(1) == 'L' ? padding.x : -padding.x;
			c.pos.y += corner.charAt(0) == 'T' ? padding.y : -padding.y;
		}
		return c; 
	});

	return dijit._place(node, choices);
}

dijit._place = function(/*DomNode*/ node, /* Array */ choices, /* Function */ layoutNode){
	// summary:
	//		Given a list of spots to put node, put it at the first spot where it fits,
	//		of if it doesn't fit anywhere then the place with the least overflow
	// choices: Array
	//		Array of elements like: {corner: 'TL', pos: {x: 10, y: 20} }
	//		Above example says to put the top-left corner of the node at (10,20)
	// layoutNode: Function(node, aroundNodeCorner, nodeCorner)
	//		for things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.

	// get {x: 10, y: 10, w: 100, h:100} type obj representing position of
	// viewport over document
	var view = dijit.getViewport();

	// This won't work if the node is inside a <div style="position: relative">,
	// so reattach it to dojo.doc.body.   (Otherwise, the positioning will be wrong
	// and also it might get cutoff)
	if(!node.parentNode || String(node.parentNode.tagName).toLowerCase() != "body"){
		dojo.body().appendChild(node);
	}

	var best = null;
	dojo.some(choices, function(choice){
		var corner = choice.corner;
		var pos = choice.pos;

		// configure node to be displayed in given position relative to button
		// (need to do this in order to get an accurate size for the node, because
		// a tooltips size changes based on position, due to triangle)
		if(layoutNode){
			layoutNode(node, choice.aroundCorner, corner);
		}

		// get node's size
		var style = node.style;
		var oldDisplay = style.display;
		var oldVis = style.visibility;
		style.visibility = "hidden";
		style.display = "";
		var mb = dojo.marginBox(node);
		style.display = oldDisplay;
		style.visibility = oldVis;

		// coordinates and size of node with specified corner placed at pos,
		// and clipped by viewport
		var startX = (corner.charAt(1) == 'L' ? pos.x : Math.max(view.l, pos.x - mb.w)),
			startY = (corner.charAt(0) == 'T' ? pos.y : Math.max(view.t, pos.y -  mb.h)),
			endX = (corner.charAt(1) == 'L' ? Math.min(view.l + view.w, startX + mb.w) : pos.x),
			endY = (corner.charAt(0) == 'T' ? Math.min(view.t + view.h, startY + mb.h) : pos.y),
			width = endX - startX,
			height = endY - startY,
			overflow = (mb.w - width) + (mb.h - height);

		if(best == null || overflow < best.overflow){
			best = {
				corner: corner,
				aroundCorner: choice.aroundCorner,
				x: startX,
				y: startY,
				w: width,
				h: height,
				overflow: overflow
			};
		}
		return !overflow;
	});

	node.style.left = best.x + "px";
	node.style.top = best.y + "px";
	if(best.overflow && layoutNode){
		layoutNode(node, best.aroundCorner, best.corner);
	}
	return best;
}

dijit.placeOnScreenAroundNode = function(
	/* DomNode */		node,
	/* DomNode */		aroundNode,
	/* Object */		aroundCorners,
	/* Function? */		layoutNode){

	// summary:
	//		Position node adjacent or kitty-corner to aroundNode
	//		such that it's fully visible in viewport.
	//
	// description:
	//		Place node such that corner of node touches a corner of
	//		aroundNode, and that node is fully visible.
	//
	// aroundCorners:
	//		Ordered list of pairs of corners to try matching up.
	//		Each pair of corners is represented as a key/value in the hash,
	//		where the key corresponds to the aroundNode's corner, and
	//		the value corresponds to the node's corner:
	//
	//	|	{ aroundNodeCorner1: nodeCorner1, aroundNodeCorner2: nodeCorner2,  ...}
	//
	//		The following strings are used to represent the four corners:
	//			* "BL" - bottom left
	//			* "BR" - bottom right
	//			* "TL" - top left
	//			* "TR" - top right
	//
	// layoutNode: Function(node, aroundNodeCorner, nodeCorner)
	//		For things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.
	//
	// example:
	//	|	dijit.placeOnScreenAroundNode(node, aroundNode, {'BL':'TL', 'TR':'BR'}); 
	//		This will try to position node such that node's top-left corner is at the same position
	//		as the bottom left corner of the aroundNode (ie, put node below
	//		aroundNode, with left edges aligned).  If that fails it will try to put
	// 		the bottom-right corner of node where the top right corner of aroundNode is
	//		(ie, put node above aroundNode, with right edges aligned)
	//

	// get coordinates of aroundNode
	aroundNode = dojo.byId(aroundNode);
	var oldDisplay = aroundNode.style.display;
	aroundNode.style.display="";
	// #3172: use the slightly tighter border box instead of marginBox
	var aroundNodeW = aroundNode.offsetWidth; //mb.w; 
	var aroundNodeH = aroundNode.offsetHeight; //mb.h;
	var aroundNodePos = dojo.coords(aroundNode, true);
	aroundNode.style.display=oldDisplay;

	// place the node around the calculated rectangle
	return dijit._placeOnScreenAroundRect(node, 
		aroundNodePos.x, aroundNodePos.y, aroundNodeW, aroundNodeH,	// rectangle
		aroundCorners, layoutNode);
};

/*=====
dijit.__Rectangle = function(){
	// x: Integer
	//		horizontal offset in pixels, relative to document body
	// y: Integer
	//		vertical offset in pixels, relative to document body
	// width: Integer
	//		width in pixels
	// height: Integer
	//		height in pixels

	thix.x = x;
	this.y = y;
	thix.width = width;
	this.height = height;
}
=====*/


dijit.placeOnScreenAroundRectangle = function(
	/* DomNode */			node,
	/* dijit.__Rectangle */	aroundRect,
	/* Object */			aroundCorners,
	/* Function */			layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except that the "around"
	//		parameter is an arbitrary rectangle on the screen (x, y, width, height)
	//		instead of a dom node.

	return dijit._placeOnScreenAroundRect(node, 
		aroundRect.x, aroundRect.y, aroundRect.width, aroundRect.height,	// rectangle
		aroundCorners, layoutNode);
};

dijit._placeOnScreenAroundRect = function(
	/* DomNode */		node,
	/* Number */		x,
	/* Number */		y,
	/* Number */		width,
	/* Number */		height,
	/* Object */		aroundCorners,
	/* Function */		layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except it accepts coordinates
	//		of a rectangle to place node adjacent to.

	// TODO: combine with placeOnScreenAroundRectangle()

	// Generate list of possible positions for node
	var choices = [];
	for(var nodeCorner in aroundCorners){
		choices.push( {
			aroundCorner: nodeCorner,
			corner: aroundCorners[nodeCorner],
			pos: {
				x: x + (nodeCorner.charAt(1) == 'L' ? 0 : width),
				y: y + (nodeCorner.charAt(0) == 'T' ? 0 : height)
			}
		});
	}

	return dijit._place(node, choices, layoutNode);
};

dijit.placementRegistry = new dojo.AdapterRegistry();
dijit.placementRegistry.register("node",
	function(n, x){
		return typeof x == "object" &&
			typeof x.offsetWidth != "undefined" && typeof x.offsetHeight != "undefined";
	},
	dijit.placeOnScreenAroundNode);
dijit.placementRegistry.register("rect",
	function(n, x){
		return typeof x == "object" &&
			"x" in x && "y" in x && "width" in x && "height" in x;
	},
	dijit.placeOnScreenAroundRectangle);

dijit.placeOnScreenAroundElement = function(
	/* DomNode */		node,
	/* Object */		aroundElement,
	/* Object */		aroundCorners,
	/* Function */		layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except it accepts an arbitrary object
	//		for the "around" argument and finds a proper processor to place a node.

	return dijit.placementRegistry.match.apply(dijit.placementRegistry, arguments);
};

}

if(!dojo._hasResource["dijit._base.window"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.window"] = true;
dojo.provide("dijit._base.window");

// TODO: remove this in 2.0, it's not used anymore, or at least not internally

dijit.getDocumentWindow = function(doc){
	// summary:
	// 		Get window object associated with document doc

	// In some IE versions (at least 6.0), document.parentWindow does not return a
	// reference to the real window object (maybe a copy), so we must fix it as well
	// We use IE specific execScript to attach the real window reference to
	// document._parentWindow for later use
	if(dojo.isIE && window !== document.parentWindow && !doc._parentWindow){
		/*
		In IE 6, only the variable "window" can be used to connect events (others
		may be only copies).
		*/
		doc.parentWindow.execScript("document._parentWindow = window;", "Javascript");
		//to prevent memory leak, unset it after use
		//another possibility is to add an onUnload handler which seems overkill to me (liucougar)
		var win = doc._parentWindow;
		doc._parentWindow = null;
		return win;	//	Window
	}

	return doc._parentWindow || doc.parentWindow || doc.defaultView;	//	Window
}

}

if(!dojo._hasResource["dijit._base.popup"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.popup"] = true;
dojo.provide("dijit._base.popup");





dijit.popup = new function(){
	// summary:
	//		This class is used to show/hide widgets as popups.

	var stack = [],
		beginZIndex=1000,
		idGen = 1;

	this.prepare = function(/*DomNode*/ node){
		// summary:
		//		Prepares a node to be used as a popup
		//
		// description:
		//		Attaches node to dojo.doc.body, and
		//		positions it off screen, but not display:none, so that
		//		the widget doesn't appear in the page flow and/or cause a blank
		//		area at the bottom of the viewport (making scrollbar longer), but
		//		initialization of contained widgets works correctly

		var s = node.style;
		s.visibility = "hidden";	// so TAB key doesn't navigate to hidden popup
		s.position = "absolute";
		s.top = "-9999px";
		if(s.display == "none"){
			s.display="";
		}
		dojo.body().appendChild(node);
	};

/*=====
dijit.popup.__OpenArgs = function(){
	// popup: Widget
	//		widget to display
	// parent: Widget
	//		the button etc. that is displaying this popup
	// around: DomNode
	//		DOM node (typically a button); place popup relative to this node.  (Specify this *or* "x" and "y" parameters.)
	// x: Integer
	//		Absolute horizontal position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
	// y: Integer
	//		Absolute vertical position (in pixels) to place node at.  (Specity this *or* "around" parameter.)
	// orient: Object || String
	//		When the around parameter is specified, orient should be an 
	//		ordered list of tuples of the form (around-node-corner, popup-node-corner).
	//		dijit.popup.open() tries to position the popup according to each tuple in the list, in order,
	//		until the popup appears fully within the viewport.
	//
	//		The default value is {BL:'TL', TL:'BL'}, which represents a list of two tuples:
	//			1. (BL, TL)
	//			2. (TL, BL)
	//		where BL means "bottom left" and "TL" means "top left".
	//		So by default, it first tries putting the popup below the around node, left-aligning them,
	//		and then tries to put it above the around node, still left-aligning them.   Note that the
	//		default is horizontally reversed when in RTL mode.
	//
	//		When an (x,y) position is specified rather than an around node, orient is either
	//		"R" or "L".  R (for right) means that it tries to put the popup to the right of the mouse,
	//		specifically positioning the popup's top-right corner at the mouse position, and if that doesn't
	//		fit in the viewport, then it tries, in order, the bottom-right corner, the top left corner,
	//		and the top-right corner.
	// onCancel: Function
	//		callback when user has canceled the popup by
	//			1. hitting ESC or
	//			2. by using the popup widget's proprietary cancel mechanism (like a cancel button in a dialog);
	//			   i.e. whenever popupWidget.onCancel() is called, args.onCancel is called
	// onClose: Function
	//		callback whenever this popup is closed
	// onExecute: Function
	//		callback when user "executed" on the popup/sub-popup by selecting a menu choice, etc. (top menu only)
	// padding: dijit.__Position
	//		adding a buffer around the opening position. This is only useful when around is not set.
	this.popup = popup;
	this.parent = parent;
	this.around = around;
	this.x = x;
	this.y = y;
	this.orient = orient;
	this.onCancel = onCancel;
	this.onClose = onClose;
	this.onExecute = onExecute;
	this.padding = padding;
}
=====*/
	this.open = function(/*dijit.popup.__OpenArgs*/ args){
		// summary:
		//		Popup the widget at the specified position
		//
		// example:
		//		opening at the mouse position
		//		|		dijit.popup.open({popup: menuWidget, x: evt.pageX, y: evt.pageY});
		//
		// example:
		//		opening the widget as a dropdown
		//		|		dijit.popup.open({parent: this, popup: menuWidget, around: this.domNode, onClose: function(){...}  });
		//
		//		Note that whatever widget called dijit.popup.open() should also listen to its own _onBlur callback
		//		(fired from _base/focus.js) to know that focus has moved somewhere else and thus the popup should be closed.

		var widget = args.popup,
			orient = args.orient || {'BL':'TL', 'TL':'BL'},
			around = args.around,
			id = (args.around && args.around.id) ? (args.around.id+"_dropdown") : ("popup_"+idGen++);

		// make wrapper div to hold widget and possibly hold iframe behind it.
		// we can't attach the iframe as a child of the widget.domNode because
		// widget.domNode might be a <table>, <ul>, etc.
		var wrapper = dojo.create("div",{
			id: id, 
			"class":"dijitPopup",
			style:{
				zIndex: beginZIndex + stack.length,
				visibility:"hidden"
			}
		}, dojo.body());
		dijit.setWaiRole(wrapper, "presentation");
		
		// prevent transient scrollbar causing misalign (#5776)
		wrapper.style.left = wrapper.style.top = "0px";		

		if(args.parent){
			wrapper.dijitPopupParent=args.parent.id;
		}

		var s = widget.domNode.style;
		s.display = "";
		s.visibility = "";
		s.position = "";
		s.top = "0px";
		wrapper.appendChild(widget.domNode);

		var iframe = new dijit.BackgroundIframe(wrapper);

		// position the wrapper node
		var best = around ?
			dijit.placeOnScreenAroundElement(wrapper, around, orient, widget.orient ? dojo.hitch(widget, "orient") : null) :
			dijit.placeOnScreen(wrapper, args, orient == 'R' ? ['TR','BR','TL','BL'] : ['TL','BL','TR','BR'], args.padding);

		wrapper.style.visibility = "visible";
		// TODO: use effects to fade in wrapper

		var handlers = [];

		// Compute the closest ancestor popup that's *not* a child of another popup.
		// Ex: For a TooltipDialog with a button that spawns a tree of menus, find the popup of the button.
		var getTopPopup = function(){
			for(var pi=stack.length-1; pi > 0 && stack[pi].parent === stack[pi-1].widget; pi--){
				/* do nothing, just trying to get right value for pi */
			}
			return stack[pi];
		}

		// provide default escape and tab key handling
		// (this will work for any widget, not just menu)
		handlers.push(dojo.connect(wrapper, "onkeypress", this, function(evt){
			if(evt.charOrCode == dojo.keys.ESCAPE && args.onCancel){
				dojo.stopEvent(evt);
				args.onCancel();
			}else if(evt.charOrCode === dojo.keys.TAB){
				dojo.stopEvent(evt);
				var topPopup = getTopPopup();
				if(topPopup && topPopup.onCancel){
					topPopup.onCancel();
				}
			}
		}));

		// watch for cancel/execute events on the popup and notify the caller
		// (for a menu, "execute" means clicking an item)
		if(widget.onCancel){
			handlers.push(dojo.connect(widget, "onCancel", null, args.onCancel));
		}

		handlers.push(dojo.connect(widget, widget.onExecute ? "onExecute" : "onChange", null, function(){
			var topPopup = getTopPopup();
			if(topPopup && topPopup.onExecute){
				topPopup.onExecute();
			}
		}));

		stack.push({
			wrapper: wrapper,
			iframe: iframe,
			widget: widget,
			parent: args.parent,
			onExecute: args.onExecute,
			onCancel: args.onCancel,
 			onClose: args.onClose,
			handlers: handlers
		});

		if(widget.onOpen){
			widget.onOpen(best);
		}

		return best;
	};

	this.close = function(/*Widget*/ popup){
		// summary:
		//		Close specified popup and any popups that it parented
		while(dojo.some(stack, function(elem){return elem.widget == popup;})){
			var top = stack.pop(),
				wrapper = top.wrapper,
				iframe = top.iframe,
				widget = top.widget,
				onClose = top.onClose;
	
			if(widget.onClose){
				widget.onClose();
			}
			dojo.forEach(top.handlers, dojo.disconnect);
	
			// #2685: check if the widget still has a domNode so ContentPane can change its URL without getting an error
			if(!widget||!widget.domNode){ return; }
			
			this.prepare(widget.domNode);

			iframe.destroy();
			dojo.destroy(wrapper);
	
			if(onClose){
				onClose();
			}
		}
	};
}();

dijit._frames = new function(){
	// summary: cache of iframes
	var queue = [];

	this.pop = function(){
		var iframe;
		if(queue.length){
			iframe = queue.pop();
			iframe.style.display="";
		}else{
			if(dojo.isIE){
				var burl = dojo.config["dojoBlankHtmlUrl"] || (dojo.moduleUrl("dojo", "resources/blank.html")+"") || "javascript:\"\"";
				var html="<iframe src='" + burl + "'"
					+ " style='position: absolute; left: 0px; top: 0px;"
					+ "z-index: -1; filter:Alpha(Opacity=\"0\");'>";
				iframe = dojo.doc.createElement(html);
			}else{
			 	iframe = dojo.create("iframe");
				iframe.src = 'javascript:""';
				iframe.className = "dijitBackgroundIframe";
			}
			iframe.tabIndex = -1; // Magic to prevent iframe from getting focus on tab keypress - as style didnt work.
			dojo.body().appendChild(iframe);
		}
		return iframe;
	};

	this.push = function(iframe){
		iframe.style.display="none";
		if(dojo.isIE){
			iframe.style.removeExpression("width");
			iframe.style.removeExpression("height");
		}
		queue.push(iframe);
	}
}();


dijit.BackgroundIframe = function(/* DomNode */node){
	// summary:
	//		For IE z-index schenanigans. id attribute is required.
	//
	// description:
	//		new dijit.BackgroundIframe(node)
	//			Makes a background iframe as a child of node, that fills
	//			area (and position) of node

	if(!node.id){ throw new Error("no id"); }
	if(dojo.isIE < 7 || (dojo.isFF < 3 && dojo.hasClass(dojo.body(), "dijit_a11y"))){
		var iframe = dijit._frames.pop();
		node.appendChild(iframe);
		if(dojo.isIE){
			iframe.style.setExpression("width", dojo._scopeName + ".doc.getElementById('" + node.id + "').offsetWidth");
			iframe.style.setExpression("height", dojo._scopeName + ".doc.getElementById('" + node.id + "').offsetHeight");
		}
		this.iframe = iframe;
	}
};

dojo.extend(dijit.BackgroundIframe, {
	destroy: function(){
		//	summary: destroy the iframe
		if(this.iframe){
			dijit._frames.push(this.iframe);
			delete this.iframe;
		}
	}
});

}

if(!dojo._hasResource["dijit._base.scroll"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.scroll"] = true;
dojo.provide("dijit._base.scroll");

dijit.scrollIntoView = function(/* DomNode */node){
	// summary:
	//		Scroll the passed node into view, if it is not.

	// don't rely on that node.scrollIntoView works just because the function is there
	// it doesnt work in Konqueror or Opera even though the function is there and probably
	//	not safari either
	// native scrollIntoView() causes FF3's whole window to scroll if there is no scroll bar 
	//	on the immediate parent
	// dont like browser sniffs implementations but sometimes you have to use it
	// It's not enough just to scroll the menu node into view if
	// node.scrollIntoView hides part of the parent's scrollbar,
	// so just manage the parent scrollbar ourselves

	//var testdir="H"; //debug
	try{ // catch unexpected/unrecreatable errors (#7808) since we can recover using a semi-acceptable native method
	node = dojo.byId(node);
	var doc = node.ownerDocument || dojo.doc;
	var body = doc.body || dojo.body();
	var html = doc.documentElement || body.parentNode;
	// if FF2 (which is perfect) or an untested browser, then use the native method

	if((!(dojo.isFF >= 3 || dojo.isIE || dojo.isWebKit) || node == body || node == html) && (typeof node.scrollIntoView == "function")){ // FF2 is perfect, too bad FF3 is not
		node.scrollIntoView(false); // short-circuit to native if possible
		return;
	}
	var ltr = dojo._isBodyLtr();
	var isIE8strict = dojo.isIE >= 8 && !compatMode;
	var rtl = !ltr && !isIE8strict; // IE8 flips scrolling so pretend it's ltr
	// body and html elements are all messed up due to browser bugs and inconsistencies related to doctype
	// normalize the values before proceeding (FF2 is not listed since its native behavior is perfect)
	// for computation simplification, client and offset width and height are the same for body and html
	// strict:       html:       |      body:       | compatMode:
	//           width   height  |  width   height  |------------
	//    ie*:  clientW  clientH | scrollW  clientH | CSS1Compat
	//    ff3:  clientW  clientH |HscrollW  clientH | CSS1Compat
	//    sf3:  clientW  clientH | clientW HclientH | CSS1Compat
	//    op9:  clientW  clientH |HscrollW  clientH | CSS1Compat
	// ---------------------------------------------|-----------
	//   none:        html:      |      body:       |
	//           width    height |  width   height  |
	//    ie*: BclientW BclientH | clientW  clientH | BackCompat
	//    ff3: BclientW BclientH | clientW  clientH | BackCompat
	//    sf3:  clientW  clientH | clientW HclientH | CSS1Compat
	//    op9: BclientW BclientH | clientW  clientH | BackCompat
	// ---------------------------------------------|-----------
	//  loose:        html:      |      body:       |
	//           width    height |  width   height  |
	//    ie*:  clientW  clientH | scrollW  clientH | CSS1Compat
	//    ff3: BclientW BclientH | clientW  clientH | BackCompat
	//    sf3:  clientW  clientH | clientW HclientH | CSS1Compat
	//    op9:  clientW  clientH |HscrollW  clientH | CSS1Compat
	var scrollRoot = body;
	var compatMode = doc.compatMode == 'BackCompat';
	if(compatMode){ // BODY is scrollable, HTML has same client size
		// body client values already OK
		html._offsetWidth = html._clientWidth = body._offsetWidth = body.clientWidth;
		html._offsetHeight = html._clientHeight = body._offsetHeight = body.clientHeight;
	}else{
		if(dojo.isWebKit){
			body._offsetWidth = body._clientWidth  = html.clientWidth;
			body._offsetHeight = body._clientHeight = html.clientHeight;
		}else{
			scrollRoot = html;
		}
		html._offsetHeight = html.clientHeight;
		html._offsetWidth  = html.clientWidth;
	}

	function isFixedPosition(element){
		var ie = dojo.isIE;
		return ((ie <= 6 || (ie >= 7 && compatMode))? false : (dojo.style(element, 'position').toLowerCase() == "fixed"));
	}

	function addPseudoAttrs(element){
		var parent = element.parentNode;
		var offsetParent = element.offsetParent;
		if(offsetParent == null || isFixedPosition(element)){ // position:fixed has no real offsetParent
			offsetParent = html; // prevents exeptions
			parent = (element == body)? html : null;
		}
		// all the V/H object members below are to reuse code for both directions
		element._offsetParent = offsetParent;
		element._parent = parent;
		//console.debug('parent = ' + (element._parentTag = element._parent?element._parent.tagName:'NULL'));
		//console.debug('offsetParent = ' + (element._offsetParentTag = element._offsetParent.tagName));
		var bp = dojo._getBorderExtents(element);
		element._borderStart = { H:(isIE8strict && !ltr)? (bp.w-bp.l):bp.l, V:bp.t };
		element._borderSize = { H:bp.w, V:bp.h };
		element._scrolledAmount = { H:element.scrollLeft, V:element.scrollTop };
		element._offsetSize = { H: element._offsetWidth||element.offsetWidth, V: element._offsetHeight||element.offsetHeight };
		//console.debug('element = ' + element.tagName + ', '+testdir+' size = ' + element[testdir=='H'?"offsetWidth":"offsetHeight"] + ', parent = ' + element._parentTag);
		// IE8 flips everything in rtl mode except offsetLeft and borderLeft - so manually change offsetLeft to offsetRight here 
		element._offsetStart = { H:(isIE8strict && !ltr)? offsetParent.clientWidth-element.offsetLeft-element._offsetSize.H:element.offsetLeft, V:element.offsetTop };
		//console.debug('element = ' + element.tagName + ', initial _relativeOffset = ' + element._offsetStart[testdir]);
		element._clientSize = { H:element._clientWidth||element.clientWidth, V:element._clientHeight||element.clientHeight };
		if(element != body && element != html && element != node){
			for(var dir in element._offsetSize){ // for both x and y directions
				var scrollBarSize = element._offsetSize[dir] - element._clientSize[dir] - element._borderSize[dir];
				//if(dir==testdir)console.log('element = ' + element.tagName + ', scrollBarSize = ' + scrollBarSize + ', clientSize = ' + element._clientSize[dir] + ', offsetSize = ' + element._offsetSize[dir] + ', border size = ' + element._borderSize[dir]);
				var hasScrollBar = element._clientSize[dir] > 0 && scrollBarSize > 0; // can't check for a specific scrollbar size since it changes dramatically as you zoom
				//if(dir==testdir)console.log('element = ' + element.tagName + ', hasScrollBar = ' + hasScrollBar);
				if(hasScrollBar){
					element._offsetSize[dir] -= scrollBarSize;
					if(dojo.isIE && rtl && dir=="H"){ element._offsetStart[dir] += scrollBarSize; }
				}
			}
		}
	}

	var element = node;
	while(element != null){
		if(isFixedPosition(element)){ node.scrollIntoView(false); return; } //TODO: handle without native call
		addPseudoAttrs(element);
		element = element._parent;
	}
	if(dojo.isIE && node._parent){ // if no parent, then offsetParent._borderStart may not tbe set
		var offsetParent = node._offsetParent;
		//console.debug('adding offsetParent borderStart = ' + offsetParent._borderStart.H + ' to node offsetStart');
		node._offsetStart.H += offsetParent._borderStart.H;
		node._offsetStart.V += offsetParent._borderStart.V;
	}
	if(dojo.isIE >= 7 && scrollRoot == html && rtl && body._offsetStart && body._offsetStart.H == 0){ // IE7 bug
		var scroll = html.scrollWidth - html._offsetSize.H;
		if(scroll > 0){
			//console.debug('adjusting html scroll by ' + -scroll + ', scrollWidth = ' + html.scrollWidth + ', offsetSize = ' + html._offsetSize.H);
			body._offsetStart.H = -scroll;
		}
	}
	if(dojo.isIE <= 6 && !compatMode){
		html._offsetSize.H += html._borderSize.H;
		html._offsetSize.V += html._borderSize.V;
	}
	// eliminate offsetLeft/Top oddities by tweaking scroll for ease of computation
	if(rtl && body._offsetStart && scrollRoot == html && html._scrolledAmount){
		var ofs = body._offsetStart.H;
		if(ofs < 0){
			html._scrolledAmount.H += ofs;
			body._offsetStart.H = 0;
		}
	}
	element = node;
	while(element){
		var parent = element._parent;
		if(!parent){ break; }
			//console.debug('element = ' + element.tagName + ', parent = ' + parent.tagName + ', parent offsetSize = ' + parent._offsetSize[testdir] + ' clientSize = ' + parent._clientSize[testdir]);
			if(parent.tagName == "TD"){
				var table = parent._parent._parent._parent; // point to TABLE
				if(parent != element._offsetParent && parent._offsetParent != element._offsetParent){
					parent = table; // child of TD has the same offsetParent as TABLE, so skip TD, TR, and TBODY (ie. verticalslider)
				}
			}
			// check if this node and its parent share the same offsetParent
			var relative = element._offsetParent == parent;
			//console.debug('element = ' + element.tagName + ', offsetParent = ' + element._offsetParent.tagName + ', parent = ' + parent.tagName + ', relative = ' + relative);
			for(var dir in element._offsetStart){ // for both x and y directions
				var otherDir = dir=="H"? "V" : "H";
				if(rtl && dir=="H" && (parent != html) && (parent != body) && (dojo.isIE || dojo.isWebKit) && parent._clientSize.H > 0 && parent.scrollWidth > parent._clientSize.H){ // scroll starts on the right
					var delta = parent.scrollWidth - parent._clientSize.H;
					//console.debug('rtl scroll delta = ' + delta + ', changing ' + parent.tagName + ' scroll from ' + parent._scrolledAmount.H + ' to ' + (parent._scrolledAmount.H - delta)  + ', parent.scrollWidth = ' + parent.scrollWidth + ', parent._clientSize.H = ' + parent._clientSize.H);
					if(delta > 0){
						parent._scrolledAmount.H -= delta;
					} // match FF3 which has cool negative scrollLeft values
				}
				if(parent._offsetParent.tagName == "TABLE"){ // make it consistent
					if(dojo.isIE){ // make it consistent with Safari and FF3 and exclude the starting TABLE border of TABLE children
						parent._offsetStart[dir] -= parent._offsetParent._borderStart[dir];
						parent._borderStart[dir] = parent._borderSize[dir] = 0;
					}
					else{
						parent._offsetStart[dir] += parent._offsetParent._borderStart[dir];
					}
				}
				//if(dir==testdir)console.debug('border start = ' + parent._borderStart[dir] + ',  border size = ' + parent._borderSize[dir]);
				if(dojo.isIE){
					//if(dir==testdir)console.debug('changing parent offsetStart from ' + parent._offsetStart[dir] + ' by adding offsetParent ' + parent._offsetParent.tagName + ' border start = ' + parent._offsetParent._borderStart[dir]);
					parent._offsetStart[dir] += parent._offsetParent._borderStart[dir];
				}
				//if(dir==testdir)console.debug('subtracting border start = ' + parent._borderStart[dir]);
				// underflow = visible gap between parent and this node taking scrolling into account
				// if negative, part of the node is obscured by the parent's beginning and should be scrolled to become visible
				var underflow = element._offsetStart[dir] - parent._scrolledAmount[dir] - (relative? 0 : parent._offsetStart[dir]) - parent._borderStart[dir];
				// if overflow is positive, number of pixels obscured by the parent's end
				var overflow = underflow + element._offsetSize[dir] - parent._offsetSize[dir] + parent._borderSize[dir];
				//if(dir==testdir)console.debug('element = ' + element.tagName + ', offsetStart = ' + element._offsetStart[dir] + ', relative = ' + relative + ', parent offsetStart = ' + parent._offsetStart[dir] + ', scroll = ' + parent._scrolledAmount[dir] + ', parent border start = ' + parent._borderStart[dir] + ', parent border size = ' + parent._borderSize[dir] + ', underflow = ' + underflow + ', overflow = ' + overflow + ', element offsetSize = ' + element._offsetSize[dir] + ', parent offsetSize = ' + parent._offsetSize[dir]);
				var scrollAttr = (dir=="H")? "scrollLeft" : "scrollTop";
				// see if we should scroll forward or backward
				var reverse = dir=="H" && rtl; // flip everything
				var underflowScroll = reverse? -overflow : underflow;
				var overflowScroll = reverse? -underflow : overflow;
				// don't scroll if the over/underflow signs are opposite since that means that
				// the node extends beyond parent's boundary in both/neither directions
				var scrollAmount = (underflowScroll*overflowScroll <= 0)? 0 : Math[(underflowScroll < 0)? "max" : "min"](underflowScroll, overflowScroll);
				//if(dir==testdir)console.debug('element = ' + element.tagName + ' dir = ' + dir + ', scrollAmount = ' + scrollAmount);
				if(scrollAmount != 0){
					var oldScroll = parent[scrollAttr];
					parent[scrollAttr] += (reverse)? -scrollAmount : scrollAmount; // actually perform the scroll
					var scrolledAmount = parent[scrollAttr] - oldScroll; // in case the scroll failed
					//if(dir==testdir)console.debug('scrolledAmount = ' + scrolledAmount);
				}
				if(relative){
					element._offsetStart[dir] += parent._offsetStart[dir];
				}
				element._offsetStart[dir] -= parent[scrollAttr];
			}
			element._parent = parent._parent;
			element._offsetParent = parent._offsetParent;
	}
	parent = node;
	var next;
	while(parent && parent.removeAttribute){
		next = parent.parentNode;
		parent.removeAttribute('_offsetParent');
		parent.removeAttribute('_parent');
		parent = next;
	}
	}catch(error){
		console.error('scrollIntoView: ' + error);
		node.scrollIntoView(false);
	}
};

}

if(!dojo._hasResource["dijit._base.sniff"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.sniff"] = true;
// summary:
//		Applies pre-set CSS classes to the top-level HTML node, based on:
// 			- browser (ex: dj_ie)
//			- browser version (ex: dj_ie6)
//			- box model (ex: dj_contentBox)
//			- text direction (ex: dijitRtl)
//
//		In addition, browser, browser version, and box model are
//		combined with an RTL flag when browser text is RTL.  ex: dj_ie-rtl.
//
//		Simply doing a require on this module will
//		establish this CSS.  Modified version of Morris' CSS hack.

dojo.provide("dijit._base.sniff");

(function(){
	
	var d = dojo,
		html = d.doc.documentElement,
		ie = d.isIE,
		opera = d.isOpera,
		maj = Math.floor,
		ff = d.isFF,
		boxModel = d.boxModel.replace(/-/,''),
		classes = {
			dj_ie: ie,
//			dj_ie55: ie == 5.5,
			dj_ie6: maj(ie) == 6,
			dj_ie7: maj(ie) == 7,
			dj_iequirks: ie && d.isQuirks,
			// NOTE: Opera not supported by dijit
			dj_opera: opera,
			dj_opera8: maj(opera) == 8,
			dj_opera9: maj(opera) == 9,
			dj_khtml: d.isKhtml,
			dj_webkit: d.isWebKit,
			dj_safari: d.isSafari,
			dj_gecko: d.isMozilla,
			dj_ff2: maj(ff) == 2,
			dj_ff3: maj(ff) == 3
		}; // no dojo unsupported browsers
		
	classes["dj_" + boxModel] = true;
	
	// apply browser, browser version, and box model class names
	for(var p in classes){
		if(classes[p]){
			if(html.className){
				html.className += " " + p;
			}else{
				html.className = p;
			}
		}
	}

	// If RTL mode then add dijitRtl flag plus repeat existing classes
	// with -rtl extension
	// (unshift is to make this code run after <body> node is loaded but before parser runs)
	dojo._loaders.unshift(function(){
		if(!dojo._isBodyLtr()){
			html.className += " dijitRtl";
			for(var p in classes){
				if(classes[p]){
					html.className += " " + p + "-rtl";
				}
			}
		}
	});
	
})();

}

if(!dojo._hasResource["dijit._base.typematic"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.typematic"] = true;
dojo.provide("dijit._base.typematic");

dijit.typematic = {
	// summary:
	//		These functions are used to repetitively call a user specified callback
	//		method when a specific key or mouse click over a specific DOM node is
	//		held down for a specific amount of time.
	//		Only 1 such event is allowed to occur on the browser page at 1 time.

	_fireEventAndReload: function(){
		this._timer = null;
		this._callback(++this._count, this._node, this._evt);
		this._currentTimeout = (this._currentTimeout < 0) ? this._initialDelay : ((this._subsequentDelay > 1) ? this._subsequentDelay : Math.round(this._currentTimeout * this._subsequentDelay));
		this._timer = setTimeout(dojo.hitch(this, "_fireEventAndReload"), this._currentTimeout);
	},

	trigger: function(/*Event*/ evt, /* Object */ _this, /*DOMNode*/ node, /* Function */ callback, /* Object */ obj, /* Number */ subsequentDelay, /* Number */ initialDelay){
		// summary:
		//	    Start a timed, repeating callback sequence.
		//	    If already started, the function call is ignored.
		//	    This method is not normally called by the user but can be
		//	    when the normal listener code is insufficient.
		// evt:
		//		key or mouse event object to pass to the user callback
		// _this:
		//		pointer to the user's widget space.
		// node:
		//		the DOM node object to pass the the callback function
		// callback:
		//		function to call until the sequence is stopped called with 3 parameters:
		// count:
		//		integer representing number of repeated calls (0..n) with -1 indicating the iteration has stopped
		// node:
		//		the DOM node object passed in
		// evt:
		//		key or mouse event object
		// obj:
		//		user space object used to uniquely identify each typematic sequence
		// subsequentDelay:
		//		if > 1, the number of milliseconds until the 3->n events occur
		//		or else the fractional time multiplier for the next event's delay, default=0.9
		// initialDelay:
		//		the number of milliseconds until the 2nd event occurs, default=500ms
		if(obj != this._obj){
			this.stop();
			this._initialDelay = initialDelay || 500;
			this._subsequentDelay = subsequentDelay || 0.90;
			this._obj = obj;
			this._evt = evt;
			this._node = node;
			this._currentTimeout = -1;
			this._count = -1;
			this._callback = dojo.hitch(_this, callback);
			this._fireEventAndReload();
		}
	},

	stop: function(){
		// summary:
		//	  Stop an ongoing timed, repeating callback sequence.
		if(this._timer){
			clearTimeout(this._timer);
			this._timer = null;
		}
		if(this._obj){
			this._callback(-1, this._node, this._evt);
			this._obj = null;
		}
	},

	addKeyListener: function(/*DOMNode*/ node, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay){
		// summary:
		//		Start listening for a specific typematic key.
		//		See also the trigger method for other parameters.
		// keyObject:
		//		an object defining the key to listen for.
		// charOrCode:
		//		the printable character (string) or keyCode (number) to listen for.
		// keyCode:
		//		(deprecated - use charOrCode) the keyCode (number) to listen for (implies charCode = 0).
		// charCode:
		//		(deprecated - use charOrCode) the charCode (number) to listen for.
		// ctrlKey:
		//		desired ctrl key state to initiate the calback sequence:
		//			- pressed (true)
		//			- released (false)
		//			- either (unspecified)
		// altKey:
		//		same as ctrlKey but for the alt key
		// shiftKey:
		//		same as ctrlKey but for the shift key
		// returns:
		//		an array of dojo.connect handles
		if(keyObject.keyCode){
			keyObject.charOrCode = keyObject.keyCode;
			dojo.deprecated("keyCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.", "", "2.0");
		}else if(keyObject.charCode){
			keyObject.charOrCode = String.fromCharCode(keyObject.charCode);
			dojo.deprecated("charCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.", "", "2.0");
		}
		return [
			dojo.connect(node, "onkeypress", this, function(evt){
				if(evt.charOrCode == keyObject.charOrCode &&
				(keyObject.ctrlKey === undefined || keyObject.ctrlKey == evt.ctrlKey) &&
				(keyObject.altKey === undefined || keyObject.altKey == evt.ctrlKey) &&
				(keyObject.shiftKey === undefined || keyObject.shiftKey == evt.ctrlKey)){
					dojo.stopEvent(evt);
					dijit.typematic.trigger(keyObject, _this, node, callback, keyObject, subsequentDelay, initialDelay);
				}else if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			}),
			dojo.connect(node, "onkeyup", this, function(evt){
				if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			})
		];
	},

	addMouseListener: function(/*DOMNode*/ node, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay){
		// summary:
		//		Start listening for a typematic mouse click.
		//		See the trigger method for other parameters.
		// returns:
		//		an array of dojo.connect handles
		var dc = dojo.connect;
		return [
			dc(node, "mousedown", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay);
			}),
			dc(node, "mouseup", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mouseout", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mousemove", this, function(evt){
				dojo.stopEvent(evt);
			}),
			dc(node, "dblclick", this, function(evt){
				dojo.stopEvent(evt);
				if(dojo.isIE){
					dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay);
					setTimeout(dojo.hitch(this, dijit.typematic.stop), 50);
				}
			})
		];
	},

	addListener: function(/*Node*/ mouseNode, /*Node*/ keyNode, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay){
		// summary:
		//		Start listening for a specific typematic key and mouseclick.
		//		This is a thin wrapper to addKeyListener and addMouseListener.
		//		See the addMouseListener and addKeyListener methods for other parameters.
		// mouseNode:
		//		the DOM node object to listen on for mouse events.
		// keyNode:
		//		the DOM node object to listen on for key events.
		// returns:
		//		an array of dojo.connect handles
		return this.addKeyListener(keyNode, keyObject, _this, callback, subsequentDelay, initialDelay).concat(
			this.addMouseListener(mouseNode, _this, callback, subsequentDelay, initialDelay));
	}
};

}

if(!dojo._hasResource["dijit._base.wai"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.wai"] = true;
dojo.provide("dijit._base.wai");

dijit.wai = {
	onload: function(){
		// summary:
		//		Detects if we are in high-contrast mode or not

		// This must be a named function and not an anonymous
		// function, so that the widget parsing code can make sure it
		// registers its onload function after this function.
		// DO NOT USE "this" within this function.

		// create div for testing if high contrast mode is on or images are turned off
		var div = dojo.create("div",{
			id: "a11yTestNode",
			style:{
				cssText:'border: 1px solid;'
					+ 'border-color:red green;'
					+ 'position: absolute;'
					+ 'height: 5px;'
					+ 'top: -999px;'
					+ 'background-image: url("' + (dojo.config.blankGif || dojo.moduleUrl("dojo", "resources/blank.gif")) + '");'
			}
		}, dojo.body());

		// test it
		var cs = dojo.getComputedStyle(div);
		if(cs){
			var bkImg = cs.backgroundImage;
			var needsA11y = (cs.borderTopColor==cs.borderRightColor) || (bkImg != null && (bkImg == "none" || bkImg == "url(invalid-url:)" ));
			dojo[needsA11y ? "addClass" : "removeClass"](dojo.body(), "dijit_a11y");
			if(dojo.isIE){
				div.outerHTML = "";		// prevent mixed-content warning, see http://support.microsoft.com/kb/925014
			}else{
				dojo.body().removeChild(div);
			}
		}
	}
};

// Test if computer is in high contrast mode.
// Make sure the a11y test runs first, before widgets are instantiated.
if(dojo.isIE || dojo.isMoz){	// NOTE: checking in Safari messes things up
	dojo._loaders.unshift(dijit.wai.onload);
}

dojo.mixin(dijit,
{
	_XhtmlRoles: /banner|contentinfo|definition|main|navigation|search|note|secondary|seealso/,

	hasWaiRole: function(/*Element*/ elem, /*String*/ role){
		// summary:
		//		Determines if an element has a particular non-XHTML role.
		// returns:
		//		True if elem has the specific non-XHTML role attribute and false if not.
		// 		For backwards compatibility if role parameter not provided, 
		// 		returns true if has non XHTML role 
		var waiRole = this.getWaiRole(elem);		
		return role ? (waiRole.indexOf(role) > -1) : (waiRole.length > 0);
	},

	getWaiRole: function(/*Element*/ elem){
		// summary:
		//		Gets the non-XHTML role for an element (which should be a wai role).
		// returns:
		//		The non-XHTML role of elem or an empty string if elem
		//		does not have a role.
		 return dojo.trim((dojo.attr(elem, "role") || "").replace(this._XhtmlRoles,"").replace("wairole:",""));
	},

	setWaiRole: function(/*Element*/ elem, /*String*/ role){
		// summary:
		//		Sets the role on an element.
		// description:
		//		In other than FF2 replace existing role attribute with new role.
		//		FF3 supports XHTML and ARIA roles so    
		//		if elem already has an XHTML role, append this role to XHTML role 
		//		and remove other ARIA roles.
		//		On Firefox 2 and below, "wairole:" is
		//		prepended to the provided role value.

		var curRole = dojo.attr(elem, "role") || "";
		if(dojo.isFF < 3 || !this._XhtmlRoles.test(curRole)){
			dojo.attr(elem, "role", dojo.isFF < 3 ? "wairole:" + role : role);
		}else{
			if((" "+ curRole +" ").indexOf(" " + role + " ") < 0){
				var clearXhtml = dojo.trim(curRole.replace(this._XhtmlRoles, ""));
				var cleanRole = dojo.trim(curRole.replace(clearXhtml, ""));	 
         		dojo.attr(elem, "role", cleanRole + (cleanRole ? ' ' : '') + role);
			}
		}
	},

	removeWaiRole: function(/*Element*/ elem, /*String*/ role){
		// summary:
		//		Removes the specified non-XHTML role from an element.
		// 		Removes role attribute if no specific role provided (for backwards compat.)

		var roleValue = dojo.attr(elem, "role"); 
		if(!roleValue){ return; }
		if(role){
			var searchRole = dojo.isFF < 3 ? "wairole:" + role : role;
			var t = dojo.trim((" " + roleValue + " ").replace(" " + searchRole + " ", " "));
			dojo.attr(elem, "role", t);
		}else{
			elem.removeAttribute("role");	
		}
	},

	hasWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Determines if an element has a given state.
		// description:
		//		On Firefox 2 and below, we check for an attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we check for an attribute
		//		called "aria-"+state.
		// returns:
		//		true if elem has a value for the given state and
		//		false if it does not.
		if(dojo.isFF < 3){
			return elem.hasAttributeNS("http://www.w3.org/2005/07/aaa", state);
		}
		return elem.hasAttribute ? elem.hasAttribute("aria-"+state) : !!elem.getAttribute("aria-"+state);
	},

	getWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Gets the value of a state on an element.
		// description:
		//		On Firefox 2 and below, we check for an attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we check for an attribute called
		//		"aria-"+state.
		// returns:
		//		The value of the requested state on elem
		//		or an empty string if elem has no value for state.
		if(dojo.isFF < 3){
			return elem.getAttributeNS("http://www.w3.org/2005/07/aaa", state);
		}
		return elem.getAttribute("aria-"+state) || "";
	},

	setWaiState: function(/*Element*/ elem, /*String*/ state, /*String*/ value){
		// summary:
		//		Sets a state on an element.
		// description:
		//		On Firefox 2 and below, we set an attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we set an attribute called
		//		"aria-"+state.
		if(dojo.isFF < 3){
			elem.setAttributeNS("http://www.w3.org/2005/07/aaa",
				"aaa:"+state, value);
		}else{
			elem.setAttribute("aria-"+state, value);
		}
	},

	removeWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Removes a state from an element.
		// description:
		//		On Firefox 2 and below, we remove the attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we remove the attribute called
		//		"aria-"+state.
		if(dojo.isFF < 3){
			elem.removeAttributeNS("http://www.w3.org/2005/07/aaa", state);
		}else{
			elem.removeAttribute("aria-"+state);
		}
	}
});

}

if(!dojo._hasResource["dijit._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base"] = true;
dojo.provide("dijit._base");











}

if(!dojo._hasResource["dijit._Widget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Widget"] = true;
dojo.provide("dijit._Widget");

dojo.require( "dijit._base" );

// This code is to assist deferring dojo.connect() calls in widgets (connecting to events on the widgets'
// DOM nodes) until someone actually needs to monitor that event.
dojo.connect(dojo, "connect", 
	function(/*Widget*/ widget, /*String*/ event){
		if(widget && dojo.isFunction(widget._onConnect)){
			widget._onConnect(event);
		}
	});

dijit._connectOnUseEventHandler = function(/*Event*/ event){};

//Keep track of where the last keydown event was, to help avoid generating
//spurious ondijitclick events when:
//1. focus is on a <button> or <a>
//2. user presses then releases the ENTER key
//3. onclick handler fires and shifts focus to another node, with an ondijitclick handler
//4. onkeyup event fires, causing the ondijitclick handler to fire
dijit._lastKeyDownNode = null;
if(dojo.isIE){
	dojo.doc.attachEvent('onkeydown', function(evt){
		//console.log("keypress on IE");
		dijit._lastKeyDownNode = evt.srcElement;
	});
}else{
	dojo.doc.addEventListener('keydown', function(evt){
		//console.log("keydown,  non-IE", evt.target);
		dijit._lastKeyDownNode = evt.target;
	}, true);
}

(function(){

var _attrReg = {};
var getAttrReg = function(dc){
	if(!_attrReg[dc]){
		var r = [];
		var attrs;
		var proto = dojo.getObject(dc).prototype;
		for(var fxName in proto){
			if(dojo.isFunction(proto[fxName]) && (attrs = fxName.match(/^_set([a-zA-Z]*)Attr$/)) && attrs[1]){
				r.push(attrs[1].charAt(0).toLowerCase() + attrs[1].substr(1));
			}
		}
		_attrReg[dc] = r;
	}
	return _attrReg[dc]||[];
}

dojo.declare("dijit._Widget", null, {
	// summary:
	//		Base class for all dijit widgets. 	

	// id: [const] String
	//		A unique, opaque ID string that can be assigned by users or by the
	//		system. If the developer passes an ID which is known not to be
	//		unique, the specified ID is ignored and the system-generated ID is
	//		used instead.
	id: "",

	// lang: [const] String
	//		Rarely used.  Overrides the default Dojo locale used to render this widget,
	//		as defined by the [HTML LANG](http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang) attribute.
	//		Value must be among the list of locales specified during by the Dojo bootstrap,
	//		formatted according to [RFC 3066](http://www.ietf.org/rfc/rfc3066.txt) (like en-us).
	lang: "",

	// dir: [const] String
	//		Unsupported by Dijit, but here for completeness.  Dijit only supports setting text direction on the
	//		entire document.
	//		Bi-directional support, as defined by the [HTML DIR](http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir)
	//		attribute. Either left-to-right "ltr" or right-to-left "rtl".
	dir: "",

	// class: String
	//		HTML class attribute
	"class": "",

	// style: String||Object
	//		HTML style attributes as cssText string or name/value hash
	style: "",

	// title: String
	//		HTML title attribute, used to specify the title of tabs, accordion panes, etc.
	title: "",

	// srcNodeRef: [readonly] DomNode
	//		pointer to original dom node
	srcNodeRef: null,

	// domNode: [readonly] DomNode
	//		This is our visible representation of the widget! Other DOM
	//		Nodes may by assigned to other properties, usually through the
	//		template system's dojoAttachPoint syntax, but the domNode
	//		property is the canonical "top level" node in widget UI.
	domNode: null,

	// containerNode: [readonly] DomNode
	//		Designates where children of the source dom node will be placed.
	//		"Children" in this case refers to both dom nodes and widgets.
	//		For example, for myWidget:
	//
	//		|	<div dojoType=myWidget>
	//		|		<b> here's a plain dom node
	//		|		<span dojoType=subWidget>and a widget</span>
	//		|		<i> and another plain dom node </i>
	//		|	</div>
	//
	//		containerNode would point to:
	//
	//		|		<b> here's a plain dom node
	//		|		<span dojoType=subWidget>and a widget</span>
	//		|		<i> and another plain dom node </i>
	//
	//		In templated widgets, "containerNode" is set via a
	//		dojoAttachPoint assignment.
	//
	//		containerNode must be defined for any widget that accepts innerHTML
	//		(like ContentPane or BorderContainer or even Button), and conversely
	//		is null for widgets that don't, like TextBox.
	containerNode: null,

	// attributeMap: [protected] Object
	//		attributeMap sets up a "binding" between attributes (aka properties)
	//		of the widget and the widget's DOM.
	//		Changes to widget attributes listed in attributeMap will be 
	//		reflected into the DOM.
	//
	//		For example, calling attr('title', 'hello')
	//		on a TitlePane will automatically cause the TitlePane's DOM to update
	//		with the new title.
	//
	//		attributeMap is a hash where the key is an attribute of the widget,
	//		and the value reflects a binding to a:
	//
	//		- DOM node attribute
	// |		focus: {node: "focusNode", type: "attribute"}
	// 		Maps this.focus to this.focusNode.focus
	//
	//		- DOM node innerHTML
	//	|		title: { node: "titleNode", type: "innerHTML" }
	//		Maps this.title to this.titleNode.innerHTML
	//
	//		- DOM node CSS class
	// |		myClass: { node: "domNode", type: "class" }
	//		Maps this.myClass to this.domNode.className
	//
	//		If the value is an array, then each element in the array matches one of the
	//		formats of the above list.
	//
	//		There are also some shorthands for backwards compatibility:
	//		- string --> { node: string, type: "attribute" }, for example:
	//	|	"focusNode" ---> { node: "focusNode", type: "attribute" }
	//		- "" --> { node: "domNode", type: "attribute" }
	attributeMap: {id:"", dir:"", lang:"", "class":"", style:"", title:""},

	// _deferredConnects: [protected] Object
	//		attributeMap addendum for event handlers that should be connected only on first use
	_deferredConnects: {
		onClick: "",
		onDblClick: "",
		onKeyDown: "",
		onKeyPress: "",
		onKeyUp: "",
		onMouseMove: "",
		onMouseDown: "",
		onMouseOut: "",
		onMouseOver: "",
		onMouseLeave: "",
		onMouseEnter: "",
		onMouseUp: ""},

	onClick: dijit._connectOnUseEventHandler,
	/*=====
	onClick: function(event){
		// summary: 
		//		Connect to this function to receive notifications of mouse click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onDblClick: dijit._connectOnUseEventHandler,
	/*=====
	onDblClick: function(event){
		// summary: 
		//		Connect to this function to receive notifications of mouse double click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onKeyDown: dijit._connectOnUseEventHandler,
	/*=====
	onKeyDown: function(event){
		// summary: 
		//		Connect to this function to receive notifications of keys being pressed down.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyPress: dijit._connectOnUseEventHandler,
	/*=====
	onKeyPress: function(event){
		// summary: 
		//		Connect to this function to receive notifications of printable keys being typed.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyUp: dijit._connectOnUseEventHandler,
	/*=====
	onKeyUp: function(event){
		// summary: 
		//		Connect to this function to receive notifications of keys being released.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onMouseDown: dijit._connectOnUseEventHandler,
	/*=====
	onMouseDown: function(event){
		// summary: 
		//		Connect to this function to receive notifications of when the mouse button is pressed down.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseMove: dijit._connectOnUseEventHandler,
	/*=====
	onMouseMove: function(event){
		// summary: 
		//		Connect to this function to receive notifications of when the mouse moves over nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOut: dijit._connectOnUseEventHandler,
	/*=====
	onMouseOut: function(event){
		// summary: 
		//		Connect to this function to receive notifications of when the mouse moves off of nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOver: dijit._connectOnUseEventHandler,
	/*=====
	onMouseOver: function(event){
		// summary: 
		//		Connect to this function to receive notifications of when the mouse moves onto nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseLeave: dijit._connectOnUseEventHandler,
	/*=====
	onMouseLeave: function(event){
		// summary: 
		//		Connect to this function to receive notifications of when the mouse moves off of this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseEnter: dijit._connectOnUseEventHandler,
	/*=====
	onMouseEnter: function(event){
		// summary: 
		//		Connect to this function to receive notifications of when the mouse moves onto this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseUp: dijit._connectOnUseEventHandler,
	/*=====
	onMouseUp: function(event){
		// summary: 
		//		Connect to this function to receive notifications of when the mouse button is released.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/

	// Constants used in templates
	
	// _blankGif: [protected] URL
	//		Used by <img> nodes in templates that really get there image via CSS background-image
	_blankGif: (dojo.config.blankGif || dojo.moduleUrl("dojo", "resources/blank.gif")),

	//////////// INITIALIZATION METHODS ///////////////////////////////////////

	postscript: function(/*Object?*/params, /*DomNode|String*/srcNodeRef){
		// summary:
		//		Kicks off widget instantiation.  See create() for details.
		// tags:
		//		private
		this.create(params, srcNodeRef);
	},

	create: function(/*Object?*/params, /*DomNode|String?*/srcNodeRef){
		// summary:
		//		Kick off the life-cycle of a widget
		// params:
		//		Hash of initialization parameters for widget, including
		//		scalar values (like title, duration etc.) and functions,
		//		typically callbacks like onClick.
		// srcNodeRef:
		//		If a srcNodeRef (dom node) is specified:
		//			- use srcNodeRef.innerHTML as my contents
		//			- if this is a behavioral widget then apply behavior
		//			  to that srcNodeRef 
		//			- otherwise, replace srcNodeRef with my generated DOM
		//			  tree
		// description:
		//		To understand the process by which widgets are instantiated, it
		//		is critical to understand what other methods create calls and
		//		which of them you'll want to override. Of course, adventurous
		//		developers could override create entirely, but this should
		//		only be done as a last resort.
		//
		//		Below is a list of the methods that are called, in the order
		//		they are fired, along with notes about what they do and if/when
		//		you should over-ride them in your widget:
		//
		// * postMixInProperties:
		//	|	* a stub function that you can over-ride to modify
		//		variables that may have been naively assigned by
		//		mixInProperties
		// * widget is added to manager object here
		// * buildRendering:
		//	|	* Subclasses use this method to handle all UI initialization
		//		Sets this.domNode.  Templated widgets do this automatically
		//		and otherwise it just uses the source dom node.
		// * postCreate:
		//	|	* a stub function that you can over-ride to modify take
		//		actions once the widget has been placed in the UI
		// tags:
		//		private

		// store pointer to original dom tree
		this.srcNodeRef = dojo.byId(srcNodeRef);

		// For garbage collection.  An array of handles returned by Widget.connect()
		// Each handle returned from Widget.connect() is an array of handles from dojo.connect()
		this._connects = [];

		// To avoid double-connects, remove entries from _deferredConnects
		// that have been setup manually by a subclass (ex, by dojoAttachEvent).
		// If a subclass has redefined a callback (ex: onClick) then assume it's being
		// connected to manually.
		this._deferredConnects = dojo.clone(this._deferredConnects);
		for(var attr in this.attributeMap){
			delete this._deferredConnects[attr]; // can't be in both attributeMap and _deferredConnects
		}
		for(attr in this._deferredConnects){
			if(this[attr] !== dijit._connectOnUseEventHandler){
				delete this._deferredConnects[attr];	// redefined, probably dojoAttachEvent exists
			}
		}

		//mixin our passed parameters
		if(this.srcNodeRef && (typeof this.srcNodeRef.id == "string")){ this.id = this.srcNodeRef.id; }
		if(params){
			this.params = params;
			dojo.mixin(this,params);
		}
		this.postMixInProperties();

		// generate an id for the widget if one wasn't specified
		// (be sure to do this before buildRendering() because that function might
		// expect the id to be there.)
		if(!this.id){
			this.id = dijit.getUniqueId(this.declaredClass.replace(/\./g,"_"));
		}
		dijit.registry.add(this);

		this.buildRendering();

		if(this.domNode){
			// Copy attributes listed in attributeMap into the [newly created] DOM for the widget.
			this._applyAttributes();

			var source = this.srcNodeRef;
			if(source && source.parentNode){
				source.parentNode.replaceChild(this.domNode, source);
			}

			// If the developer has specified a handler as a widget parameter
			// (ex: new Button({onClick: ...})
			// then naturally need to connect from dom node to that handler immediately, 
			for(attr in this.params){
				this._onConnect(attr);
			}
		}
		
		if(this.domNode){
			this.domNode.setAttribute("widgetId", this.id);
		}
		this.postCreate();

		// If srcNodeRef has been processed and removed from the DOM (e.g. TemplatedWidget) then delete it to allow GC.
		if(this.srcNodeRef && !this.srcNodeRef.parentNode){
			delete this.srcNodeRef;
		}	

		this._created = true;
	},

	_applyAttributes: function(){
		// summary:
		//		Step during widget creation to copy all widget attributes to the
		//		DOM as per attributeMap and _setXXXAttr functions.
		// description:
		//		Skips over blank/false attribute values, unless they were explicitly specified
		//		as parameters to the widget, since those are the default anyway,
		//		and setting tabIndex="" is different than not setting tabIndex at all.
		//
		//		It processes the attributes in the attribute map first, and then
		//		it goes through and processes the attributes for the _setXXXAttr
		//		functions that have been specified
		// tags:
		//		private
		var condAttrApply = function(attr, scope){
			if( (scope.params && attr in scope.params) || scope[attr]){
				scope.attr(attr, scope[attr]);
			}
		};
		for(var attr in this.attributeMap){
			condAttrApply(attr, this);
		}
		dojo.forEach(getAttrReg(this.declaredClass), function(a){
			if(!(a in this.attributeMap)){
				condAttrApply(a, this);
			}
		}, this);
	},

	postMixInProperties: function(){
		// summary:
		//		Called after the parameters to the widget have been read-in,
		//		but before the widget template is instantiated. Especially
		//		useful to set properties that are referenced in the widget
		//		template.
		// tags:
		//		protected
	},

	buildRendering: function(){
		// summary:
		//		Construct the UI for this widget, setting this.domNode.  Most
		//		widgets will mixin `dijit._Templated`, which implements this
		//		method.
		// tags:
		//		protected
		this.domNode = this.srcNodeRef || dojo.create('div');
	},

	postCreate: function(){
		// summary:
		//		Called after a widget's dom has been setup
		// tags:
		//		protected
	},

	startup: function(){
		// summary:
		//		Called after a widget's children, and other widgets on the page, have been created.
		//		Provides an opportunity to manipulate any children before they are displayed.
		//		This is useful for composite widgets that need to control or layout sub-widgets.
		//		Many layout widgets can use this as a wiring phase.
		this._started = true;
	},

	//////////// DESTROY FUNCTIONS ////////////////////////////////

	destroyRecursive: function(/*Boolean?*/ preserveDom){
		// summary:
		// 		Destroy this widget and it's descendants. This is the generic
		// 		"destructor" function that all widget users should call to
		// 		cleanly discard with a widget. Once a widget is destroyed, it's
		// 		removed from the manager object.
		// preserveDom:
		//		If true, this method will leave the original Dom structure
		//		alone of descendant Widgets. Note: This will NOT work with
		//		dijit._Templated widgets.

		this.destroyDescendants(preserveDom);
		this.destroy(preserveDom);
	},

	destroy: function(/*Boolean*/ preserveDom){
		// summary:
		// 		Destroy this widget, but not its descendants.
		//		Will, however, destroy internal widgets such as those used within a template.
		// preserveDom: Boolean
		//		If true, this method will leave the original Dom structure alone.
		//		Note: This will not yet work with _Templated widgets

		this.uninitialize();
		dojo.forEach(this._connects, function(array){
			dojo.forEach(array, dojo.disconnect);
		});

		// destroy widgets created as part of template, etc.
		dojo.forEach(this._supportingWidgets||[], function(w){ 
			if(w.destroy){
				w.destroy();
			}
		});
		
		this.destroyRendering(preserveDom);
		dijit.registry.remove(this.id);
	},

	destroyRendering: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Destroys the DOM nodes associated with this widget
		// preserveDom:
		//		If true, this method will leave the original Dom structure alone
		//		during tear-down. Note: this will not work with _Templated
		//		widgets yet. 
		// tags:
		//		protected

		if(this.bgIframe){
			this.bgIframe.destroy(preserveDom);
			delete this.bgIframe;
		}

		if(this.domNode){
			if(preserveDom){
				dojo.removeAttr(this.domNode, "widgetId");
			}else{
				dojo.destroy(this.domNode);
			}
			delete this.domNode;
		}

		if(this.srcNodeRef){
			if(!preserveDom){
				dojo.destroy(this.srcNodeRef);
			}
			delete this.srcNodeRef;
		}
	},

	destroyDescendants: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Recursively destroy the children of this widget and their
		//		descendants.
		// preserveDom:
		//		If true, the preserveDom attribute is passed to all descendant
		//		widget's .destroy() method. Not for use with _Templated
		//		widgets.

		// get all direct descendants and destroy them recursively
		dojo.forEach(this.getChildren(), function(widget){ 
			if(widget.destroyRecursive){
				widget.destroyRecursive(preserveDom);
			}
		});
	},


	uninitialize: function(){
		// summary:
		//		Stub function. Override to implement custom widget tear-down
		//		behavior.
		// tags:
		//		protected
		return false;
	},

	////////////////// MISCELLANEOUS METHODS ///////////////////

	onFocus: function(){
		// summary:
		//		Called when the widget becomes "active" because
		//		it or a widget inside of it either has focus, or has recently
		//		been clicked.
		// tags:
		//		callback
	},

	onBlur: function(){
		// summary:
		//		Called when the widget stops being "active" because
		//		focus moved to something outside of it, or the user
		//		clicked somewhere outside of it, or the widget was
		//		hidden.
		// tags:
		//		callback
	},

	_onFocus: function(e){
		// summary:
		//		This is where widgets do processing for when they are active,
		//		such as changing CSS classes.  See onFocus() for more details.
		// tags:
		//		protected
		this.onFocus();
	},

	_onBlur: function(){
		// summary:
		//		This is where widgets do processing for when they stop being active,
		//		such as changing CSS classes.  See onBlur() for more details.
		// tags:
		//		protected
		this.onBlur();
	},

	_onConnect: function(/*String*/ event){
		// summary:
		//		Called when someone connects to one of my handlers.
		//		"Turn on" that handler if it isn't active yet.
		//
		//		This is also called for every single initialization parameter
		//		so need to do nothing for parameters like "id".
		// tags:
		//		private
		if(event in this._deferredConnects){
			var mapNode = this[this._deferredConnects[event]||'domNode'];
			this.connect(mapNode, event.toLowerCase(), event);
			delete this._deferredConnects[event];
		}
	},

	_setClassAttr: function(/*String*/ value){
		// summary:
		//		Custom setter for the CSS "class" attribute
		// tags:
		//		protected
		var mapNode = this[this.attributeMap["class"]||'domNode'];
		dojo.removeClass(mapNode, this["class"])
		this["class"] = value;
		dojo.addClass(mapNode, value);
	},

	_setStyleAttr: function(/*String||Object*/ value){
		// summary:
		//		Sets the style attribut of the widget according to value,
		//		which is either a hash like {height: "5px", width: "3px"}
		//		or a plain string
		// description:
		//		Determines which node to set the style on based on style setting
		//		in attributeMap.
		// tags:
		//		protected

		var mapNode = this[this.attributeMap["style"]||'domNode'];
		
		// Note: technically we should revert any style setting made in a previous call
		// to his method, but that's difficult to keep track of.

		if(dojo.isObject(value)){
			dojo.style(mapNode, value);
		}else{
			if(mapNode.style.cssText){
				mapNode.style.cssText += "; " + value;
			}else{
				mapNode.style.cssText = value;
			}
		}

		this["style"] = value;
	},

	setAttribute: function(/*String*/ attr, /*anything*/ value){
		// summary:
		//		Deprecated.  Use attr() instead.
		// tags:
		//		deprecated
		dojo.deprecated(this.declaredClass+"::setAttribute() is deprecated. Use attr() instead.", "", "2.0");
		this.attr(attr, value);
	},
	
	_attrToDom: function(/*String*/ attr, /*String*/ value){
		// summary:
		//		Reflect a widget attribute (title, tabIndex, duration etc.) to
		//		the widget DOM, as specified in attributeMap.
		//
		// description:
		//		Also sets this["attr"] to the new value.
		//		Note some attributes like "type"
		//		cannot be processed this way as they are not mutable.
		//
		// tags:
		//		private

		var commands = this.attributeMap[attr];
		dojo.forEach( dojo.isArray(commands) ? commands : [commands], function(command){

			// Get target node and what we are doing to that node
			var mapNode = this[command.node || command || "domNode"];	// DOM node
			var type = command.type || "attribute";	// class, innerHTML, or attribute
	
			switch(type){
				case "attribute":
					if(dojo.isFunction(value)){ // functions execute in the context of the widget
						value = dojo.hitch(this, value);
					}
					if(/^on[A-Z][a-zA-Z]*$/.test(attr)){ // eg. onSubmit needs to be onsubmit
						attr = attr.toLowerCase();
					}
					dojo.attr(mapNode, attr, value);
					break;
				case "innerHTML":
					mapNode.innerHTML = value;
					break;
				case "class":
					dojo.removeClass(mapNode, this[attr]);
					dojo.addClass(mapNode, value);
					break;
			}
		}, this);
		this[attr] = value;
	},

	attr: function(/*String|Object*/name, /*Object?*/value){
		//	summary:
		//		Set or get properties on a widget instance.
		//	name:
		//		The property to get or set. If an object is passed here and not
		//		a string, its keys are used as names of attributes to be set
		//		and the value of the object as values to set in the widget.
		//	value:
		//		Optional. If provided, attr() operates as a setter. If omitted,
		//		the current value of the named property is returned.
		//	description:
		//		Get or set named properties on a widget. If no value is
		//		provided, the current value of the attribute is returned,
		//		potentially via a getter method. If a value is provided, then
		//		the method acts as a setter, assigning the value to the name,
		//		potentially calling any explicitly provided setters to handle
		//		the operation. For instance, if the widget has properties "foo"
		//		and "bar" and a method named "_setFooAttr", calling:
		//	|	myWidget.attr("foo", "Howdy!");
		//		would be equivalent to calling:
		//	|	widget._setFooAttr("Howdy!");
		//		while calling:
		//	|	myWidget.attr("bar", "Howdy!");
		//		would be the same as writing:
		//	|	widget.bar = "Howdy!";
		//		It also tries to copy the changes to the widget's DOM according
		//		to settings in attributeMap (see description of `dijit._Widget.attributeMap`
		//		for details)
		//		For example, calling:
		//	|	myTitlePane.attr("title", "Howdy!");
		//		will do
		//	|	myTitlePane.title = "Howdy!";
		//	|	myTitlePane.title.innerHTML = "Howdy!";
		//		It works for dom node attributes too.  Calling
		//	|	widget.attr("disabled", true)
		//		will set the disabled attribute on the widget's focusNode,
		//		among other housekeeping for a change in disabled state.

		//	open questions:
		//		- how to handle build shortcut for attributes which want to map
		//		into DOM attributes?
		//		- what relationship should setAttribute()/attr() have to
		//		layout() calls?
		var args = arguments.length;
		if(args == 1 && !dojo.isString(name)){
			for(var x in name){ this.attr(x, name[x]); }
			return this;
		}
		var names = this._getAttrNames(name);
		if(args == 2){ // setter
			if(this[names.s]){
				// use the explicit setter
				return this[names.s](value) || this;
			}else{
				// if param is specified as DOM node attribute, copy it
				if(name in this.attributeMap){
					this._attrToDom(name, value);
				}

				// FIXME: what about function assignments? Any way to connect() here?
				this[name] = value;
			}
			return this;
		}else{ // getter
			if(this[names.g]){
				return this[names.g]();
			}else{
				return this[name];
			}
		}
	},

	_attrPairNames: {},		// shared between all widgets
	_getAttrNames: function(name){
		// summary:
		//		Helper function for Widget.attr().
		//		Caches attribute name values so we don't do the string ops every time.
		// tags:
		//		private

		var apn = this._attrPairNames;
		if(apn[name]){ return apn[name]; }
		var uc = name.charAt(0).toUpperCase() + name.substr(1);
		return apn[name] = {
			n: name+"Node",
			s: "_set"+uc+"Attr",
			g: "_get"+uc+"Attr"
		};
	},

	toString: function(){
		// summary:
		//		Returns a string that represents the widget. When a widget is
		//		cast to a string, this method will be used to generate the
		//		output. Currently, it does not implement any sort of reversable
		//		serialization.
		return '[Widget ' + this.declaredClass + ', ' + (this.id || 'NO ID') + ']'; // String
	},

	getDescendants: function(){
		// summary:
		//		Returns all the widgets that contained by this, i.e., all widgets underneath this.containerNode.
		//		This method should generally be avoided as it returns widgets declared in templates, which are
		//		supposed to be internal/hidden, but it's left here for back-compat reasons.

		if(this.containerNode){
			var list = dojo.query('[widgetId]', this.containerNode);
			return list.map(dijit.byNode);		// Array
		}else{
			return [];
		}
	},

	getChildren: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		Does not return nested widgets, nor widgets that are part of this widget's template.
		if(this.containerNode){
			return dijit.findWidgets(this.containerNode);
		}else{
			return [];
		}
	},

	// nodesWithKeyClick: [private] String[]
	//		List of nodes that correctly handle click events via native browser support,
	//		and don't need dijit's help
	nodesWithKeyClick: ["input", "button"],

	connect: function(
			/*Object|null*/ obj,
			/*String|Function*/ event,
			/*String|Function*/ method){
		// summary:
		//		Connects specified obj/event to specified method of this object
		//		and registers for disconnect() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.connect, except with the
		//		implicit use of this widget as the target object.
		//		This version of connect also provides a special "ondijitclick"
		//		event which triggers on a click or space or enter keyup
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when foo.bar() is called, call the listener we're going to
		//	|	// provide in the scope of btn
		//	|	btn.connect(foo, "bar", function(){ 
		//	|		console.debug(this.toString());
		//	|	});
		// tags:
		//		protected

		var d = dojo;
		var dc = dojo.connect;
		var handles =[];
		if(event == "ondijitclick"){
			// add key based click activation for unsupported nodes.
			// do all processing onkey up to prevent spurious clicks
			// for details see comments at top of this file where _lastKeyDownNode is defined
			if(!this.nodesWithKeyClick[obj.tagName.toLowerCase()]){
				var m = d.hitch(this, method);
				handles.push(
					dc(obj, "onkeydown", this, function(e){
						//console.log(this.id + ": onkeydown, e.target = ", e.target, ", lastKeyDownNode was ", dijit._lastKeyDownNode, ", equality is ", (e.target === dijit._lastKeyDownNode));
						if((e.keyCode == d.keys.ENTER || e.keyCode == d.keys.SPACE) &&
							!e.ctrlKey && !e.shiftKey && !e.altKey && !e.metaKey){
							// needed on IE for when focus changes between keydown and keyup - otherwise dropdown menus do not work
							dijit._lastKeyDownNode = e.target;
							d.stopEvent(e);		// stop event to prevent scrolling on space key in IE
						}
			 		}),
					dc(obj, "onkeyup", this, function(e){
						//console.log(this.id + ": onkeyup, e.target = ", e.target, ", lastKeyDownNode was ", dijit._lastKeyDownNode, ", equality is ", (e.target === dijit._lastKeyDownNode));
						if( (e.keyCode == d.keys.ENTER || e.keyCode == d.keys.SPACE) &&
							e.target === dijit._lastKeyDownNode &&
							!e.ctrlKey && !e.shiftKey && !e.altKey && !e.metaKey){
								//need reset here or have problems in FF when focus returns to trigger element after closing popup/alert
								dijit._lastKeyDownNode = null;  
								return m(e);
						}
					})
				);
			}
			event = "onclick";
		}
		handles.push(dc(obj, event, this, method));

		// return handles for FormElement and ComboBox
		this._connects.push(handles);
		return handles;
	},

	disconnect: function(/*Object*/ handles){
		// summary:
		//		Disconnects handle created by this.connect.
		//		Also removes handle from this widget's list of connects
		// tags:
		//		protected
		for(var i=0; i<this._connects.length; i++){
			if(this._connects[i]==handles){
				dojo.forEach(handles, dojo.disconnect);
				this._connects.splice(i, 1);
				return;
			}
		}
	},

	isLeftToRight: function(){
		// summary:
		//		Checks the page for text direction
		// tags:
		//		protected
		return dojo._isBodyLtr(); //Boolean
	},

	isFocusable: function(){
		// summary:
		//		Return true if this widget can currently be focused
		//		and false if not
		return this.focus && (dojo.style(this.domNode, "display") != "none");
	},
	
	placeAt: function(/* String|DomNode|_Widget */reference, /* String?|Int? */position){
		// summary:
		//		Place this widget's domNode reference somewhere in the DOM based
		//		on standard dojo.place conventions, or passing a Widget reference that
		//		contains and addChild member.
		//
		// description:
		//		A convenience function provided in all _Widgets, providing a simple
		//		shorthand mechanism to put an existing (or newly created) Widget
		//		somewhere in the dom, and allow chaining.
		//
		// reference: 
		//		The String id of a domNode, a domNode reference, or a reference to a Widget posessing 
		//		an addChild method.
		//
		// position: 
		//		If passed a string or domNode reference, the position argument
		//		accepts a string just as dojo.place does, one of: "first", "last", 
		//		"before", or "after". 
		//
		//		If passed a _Widget reference, and that widget reference has an ".addChild" method, 
		//		it will be called passing this widget instance into that method, supplying the optional
		//		position index passed.
		//
		// returns: dijit._Widget
		//		Provides a useful return of the newly created dijit._Widget instance so you 
		//		can "chain" this function by instantiating, placing, then saving the return value
		//		to a variable. 
		//
		// example:
		// | 	// create a Button with no srcNodeRef, and place it in the body:
		// | 	var button = new dijit.form.Button({ label:"click" }).placeAt(dojo.body());
		// | 	// now, 'button' is still the widget reference to the newly created button
		// | 	dojo.connect(button, "onClick", function(e){ console.log('click'); });
		//
		// example:
		// |	// create a button out of a node with id="src" and append it to id="wrapper":
		// | 	var button = new dijit.form.Button({},"src").placeAt("wrapper");
		//
		// example:
		// |	// place a new button as the first element of some div
		// |	var button = new dijit.form.Button({ label:"click" }).placeAt("wrapper","first");
		//
		// example: 
		// |	// create a contentpane and add it to a TabContainer
		// |	var tc = dijit.byId("myTabs");
		// |	new dijit.layout.ContentPane({ href:"foo.html", title:"Wow!" }).placeAt(tc)

		if(reference["declaredClass"] && reference["addChild"]){
			reference.addChild(this, position);
		}else{
			dojo.place(this.domNode, reference, position);
		}
		return this;
	}

});

})();

}

if(!dojo._hasResource["com.ibm.portal.aggregation.PortletDojoSupport"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.portal.aggregation.PortletDojoSupport"] = true;
dojo.provide( "com.ibm.portal.aggregation.PortletDojoSupport" );

dojo.require( "dijit._Widget" );

com.ibm.portal.aggregation.PortletDojoSupport = {
	globalObj: window,
	init: function () {
		// we don't need to handle ondijitclick events because underneath the covers
		// it used an onclick handler (which was already catch below) and then adds
		// some other handlers for onmouseup, onkeyup, etc which we don't need to care
		// about since we don't catch those in a special way
		//dojo.connect( dijit._Widget.prototype, "connect", com.ibm.portal.aggregation.PortletDojoSupport.dijitConnect );
		
		//dojo.connect
		this._oldconnect = dojo.connect;
		// dojo.connect in IE works fine in CSA because of the way dojo adds the event handlers
		if(!dojo.isIE) dojo.connect = dojo.hitch( this, this.dojoConnect );
		//dojo.disconnect
		this._olddisconnect = dojo.disconnect;
		if(!dojo.isIE) dojo.disconnect = dojo.hitch( this, this.dojoDisconnect );
		//dojo.addOnLoad
		this._oldaddonload = dojo.addOnLoad;
		dojo.addOnLoad = dojo.hitch( this, this.dojoAddOnLoad );
	},
	dijitConnect: function ( element, event, method ) {
		if ( event == "onclick" || event == "ondijitclick" ) {
			if ( event == "ondijitclick") { event = "onclick"; }
			if ( typeof method == "string") {
				element[event] = dojo.hitch( this, this[method] );
			}
			else {
				element[event] = dojo.hitch(this, method);
			}
		}	
	},
	dojoConnect: function ( element, event, context, method ) {
		// we need the connection handle itself, so we need to
		// actually connect the event handler, but we may be disconnecting
		// it later if it's an onclick handler and we handle it ourselves
		var connection = this._oldconnect( element, event, context, method);
		if ( event == "onclick" ) {
			// dojo.connect creates different types of connection handles, and it's hard to get that perfectly
			// right for every case, so we'll just let dojo do it for us, then immediately disconnect it if it's
			// for an onclick handler (so we won't have the onclick executed twice)
			// we need to copy it because dojo.disconnect actually modifies the passed in array, but we want the
			// caller to get the original so that it doesn't break anything later
			var copy = new Array();
			for(var i = 0; i < connection.length; i++) {
				copy[i] = connection[i];
			}
			this._olddisconnect(copy);
			element = element || dojo.global;
			if ( context == null ) { context = this.globalObj; }
			var handler = dojo.hitch( context, method );
			var wrappedHandler = function (e) {
				if ( !e ) {
					e = window.event;
				}
				e = dojo.fixEvent(e, element);
				return handler(e);
			}
			if(element[event]) {
				// if there is already a handler here, create a chain of additional handlers
				if(!element._onclickChain) {
					element._onclickChain = new Array();
				}
				element._onclickChain.push(wrappedHandler);
			}
			else {
				element[event] = wrappedHandler;
			}
			connection._csaHandler = wrappedHandler;
		}
		return connection;
	},
	dojoDisconnect: function ( handle ) {
		var element = null;
		var event = "";
		if ( handle && handle.length >= 2 ) {
			element = handle[0];
			event = handle[1];
		}	
		if ( event == "onclick" ) {
			// do not use dojo.disconnect on onclick handlers because we already disconnected them beforehand in
			// the dojo.connect overwritten function - so just clear the event handler from the node itself
			if(element[event] != handle._csaHandler && element._onclickChain) {
				// if the first handler isn't this one, iterate through the onclick chain to remove the right one
				for(var i = 0; i < element._onclickChain.length; i++) {
					if(element._onclickChain[i] == handle._csaHandler) {
						element._onclickChain.splice(i, 1);
						return;	// return now since we found the right one and don't want to go through the rest
					}
				}
			}
			else {
				if(element._onclickChain && element._onclickChain.length > 0) {
					// since we removed the first handler, check if there are others after it
					// if so, reset the base first handler to the next handler (which is the first on the onclick chain)
					element[event] = element._onclickChain.splice(0,1)[0];
				}
				else {
					element[event] = null;
				}
			}
		}
		else {
			this._olddisconnect( handle );
		}
	},
	dojoAddOnLoad: function ( obj, func ) {
		var handler = null;
		if(!func) handler = obj;
		else {
			handler = dojo.hitch(obj, func);
		}

		
		if ( window.addEventListener ) {
			window.addEventListener( "load", handler, false );
		}
		else if ( window.attachEvent ) {
			window.attachEvent( "onload", handler );
		}
		else {
			this._oldaddonload( handler );
		}
	}
}

com.ibm.portal.aggregation.PortletDojoSupport.init();

//dojo.subscribe( "/portal/PortletMarkup/updated", null, function ( args ) { dojo.parser.parse( args["newMarkup"] ); } );

	
	
	

}

